function lsl_starter(Exp)

if isstruct(Exp)
  PAR = Exp;
else
  PAR = create_PAR(Exp);
  lsl_file = [PAR.organism.name '_lsl'];
  PAR = feval(lsl_file,PAR);
end

fprintf(1,'preparing training blocks\n')
model = feval(PAR.method.model_fct,PAR);

fn_workspace = ['~/tmp/' PAR.organism.name 'ws_tmp.mat']; 
if fexist(fn_workspace)
  warning('load workspace from file; %s ', fn_workspace)
  load(fn_workspace);
else
  fprintf(1,'load genes from file \n')
  load(PAR.FN.output.fn_genes_merged,'genes') ;
  
  genes = update_splicegraph(genes,PAR.tasks.run_locally);

  fprintf(1,'load training blocks from file: \n    %s\n',PAR.FN.output.fn_training_blocks)
  load(PAR.FN.output.fn_training_blocks, 'blocks')
  
  fprintf(1,'check genes, generate segmentation and do some filtering... \n')
  [blocks, genes, split] = gen_lsl_blocks(genes, blocks, PAR.FN.genome.fn_genome_config ,model, PAR.FN.output.fn_training_blocks,PAR.tasks.run_locally);

  save(fn_workspace);
end

% load tiling array data
%-------------------------------------------------------------
if isfield(model, 'tiling')&& ~isfield(blocks,'intensities')
   blocks=add_intensities2blocks(blocks,PAR.FN.input_lsl.fn_tiling_data,80);
end

% filter blocks
%-------------------------------------------------------------
rm_idx =feval(PAR.method.block_filter_fct,PAR,blocks,genes);
fprintf('remove %i (%i) blocks according to filter function: %s \n',length(rm_idx), length(blocks),PAR.method.block_filter_fct)
blocks(rm_idx) = [];

gene_statistics(genes([blocks.gene_ids]));

% reduce number of blocks
%-------------------------------------------------------------
%nob = round(length(blocks)*3/4); nob_eval = length(blocks)-nob;
%fprintf('reduce number of blocks to %i\n',nob+nob_eval)
%blocks = blocks(perm(1:nob));
clear  perm alt_idx


% add signals to blocks and generate feature matrix
%------------------------------------------------------------- 
blocks = make_prediction_blocks(PAR,blocks, model);

fn_ws2 = ['~/tmp/' PAR.organism.name '_ws2_tmp'];
save_struct([fn_ws2 '_blocks.mat'],blocks,'blocks')
save(fn_ws2)
%blocks = load_struct([fn_ws2 '_blocks.mat'],'blocks')
%load ~/tmp/A_thaliana_ws2_tmp.mat

% move tss and cleave signal positions in true segmentation to nearest 
% position with tss or cleave prediction
%-------------------------------------------------------------
if ~isfield(PAR.method, 'model_name')||~strcmp(PAR.method.model_name, 'onlyorf')
  fprintf(1,'find nearest predicted tss and cleave position for the annotated ones and move them\n')
  for b = 1:length(blocks)
    for t=1:length(blocks(b).truth)
      [blocks(b).truth(t)]=correct_cleave_and_tss_pos(PAR,blocks(b).truth(t),blocks(b), model);
    end
  end
end

% create paths from segmentation
%-------------------------------------------------------------
fprintf(1,'create paths from segmentations\n')
for b = 1:length(blocks)
  for t=1:length(blocks(b).truth)
    [blocks(b).truth(t).path,blocks(b).truth(t).pos_idx,blocks(b).truth(t).pos]=...
         segmentation2path(blocks(b).truth(t),blocks(b),model);
    % check that the inverse function produces again the original segmentation
    segments = path2segmentation(blocks(b).truth(t).path,blocks(b).truth(t).pos_idx,blocks(b), model);
    assert(isequal(segments,blocks(b).truth(t).segments(:,1:3)))
    clear segments
  end
end

% check consistency of 'true' path with model
%-------------------------------------------------------------
fprintf(1,'check block model consistency\n')
ok_idx = check_block_model_consistency(blocks,model);
if length(ok_idx)<length(blocks)*99/100
  warning('%i out of %i blocks have invalid paths; type "return" to remove them', length(blocks)-length(ok_idx),length(blocks));
  keyboard
end
blocks = blocks(ok_idx);

%% get boundaries and contents for model
%-------------------------------------------------------------
PAR.model = model;
PAR.model = prepare_model(PAR,blocks) ;
clear model

% map trained contents to svms for best_path_trans
%-------------------------------------------------------------
if strcmp(PAR.organism.name,'A_thaliana')
  PAR.FN.output_cont.utr_exon = PAR.FN.output_cont.utr3exon;
  PAR.FN.output_cont.cds_frame0 = PAR.FN.output_cont.frame0;
  PAR.FN.output_cont.cds_frame1 = PAR.FN.output_cont.frame0;
  PAR.FN.output_cont.cds_frame2 = PAR.FN.output_cont.frame0;
elseif strcmp(PAR.organism.name,'F_graminearum')
  PAR.FN.output_cont.utr_exon.fn_SVMs   = '/fml/ag-raetsch/share/projects/genefinding/F_graminearum/exp_j/exp/contents/utr_exon/output_SVM/SVM'
  PAR.FN.output_cont.cds_frame0.fn_SVMs = '/fml/ag-raetsch/share/projects/genefinding/F_graminearum/exp_j/exp/contents/utr_exon/output_SVM/SVM'
  PAR.FN.output_cont.cds_frame1.fn_SVMs = '/fml/ag-raetsch/share/projects/genefinding/F_graminearum/exp_j/exp/contents/utr_exon/output_SVM/SVM'
  PAR.FN.output_cont.cds_frame2.fn_SVMs = '/fml/ag-raetsch/share/projects/genefinding/F_graminearum/exp_j/exp/contents/utr_exon/output_SVM/SVM'
end

% compute weights for the true paths
%-------------------------------------------------------------
blocks = initialize_blocks(blocks, PAR.model, PAR.method.max_neg, PAR.FN);

% make split and save
%-------------------------------------------------------------
rand('seed',1234)
perm = randperm(length(blocks));

nob = round(length(blocks)*3/4); 
fprintf(1,'save training blocks to %s \n',PAR.FN.input_lsl.fn_training_blocks_all);
save_struct(PAR.FN.input_lsl.fn_training_blocks_all,blocks(perm(1:nob)),'blocks')
save(PAR.FN.input_lsl.fn_training_blocks_all,'-append', 'PAR')

fprintf(1,'save validation blocks to %s \n',PAR.FN.input_lsl.fn_val_blocks_all);
save_struct(PAR.FN.input_lsl.fn_val_blocks_all,blocks(perm(nob+1:end)),'blocks')
save(PAR.FN.input_lsl.fn_val_blocks_all,'-append', 'PAR')

%train_path(PAR) ;


