function nGASP_starter(Exp, status, sensors, contents, verb, next_step)
% nGASP_starter(Exp, status,sensors,contents,verb)

if nargin<2
  status ='layer1';
end    
if nargin<3 
  sensors={'acc','don','cdsStop','tis','tss','cleave','polya','transacc'};
end
if nargin<4 
  contents={'intergenic','utr5exon','cds_exon','frame0','utr3exon','intron'};
end
if nargin<5
  verb=0;
end    
if nargin<6,
  next_step=0;
end 
if ~iscell(sensors)
  sensors={sensors};
end

check_files =1 ;
paths
addpath ~/svn/projects/genefinding/experiments/nGASP/
addpath ~/svn/projects/genefinding/lsl_paper/
rmpath ~/svn/projects/genefinding/lsl_new/
rmpath ~/svn/projects/genefinding/lsl/
addpath( '~/svn/projects/genefinding/shogun');

PAR = create_PAR(Exp);
create_directories(PAR) 


%----------------PREPARE REGIONS AND BLOCKS----------------%
if isequal(status,'prep')  
  if ~isequal(Exp,'exp_C_elegans_nGASP')
    Nematodes_prep(PAR)
  else
    C_elegans_nGASP_prep(PAR)
  end
  PAR = masterscript_data(PAR);
  if next_step, next_step=next_step-1 ; status = 'layer1'; end ;
end

if isequal(status,'prep2')  
  % keyboard
  PAR = masterscript_data(PAR,2);
  if next_step, next_step=next_step-1 ; status = 'layer1'; end ;
  %sensors = {'transacc'}
end

%--------------------PREDICT ALL SIGNALS-------------------%
PAR.tasks.run_locally = 1;
PAR.RPROC.options.express = 0;
% PAR.RPROC.numjunks = 10;
% PAR.RPROC.divide4pred = 1 ; 

all_predictions_done = 0 ; 
if isequal(status,'layer1')    
  while ~all_predictions_done
    JOBINFO = [];ALL_DONE=[];
    fprintf('\nstart layer 1 predictions\n')
    for i=1:length(sensors)
      sensor = sensors{i};
      PAR.Signal_name = sensor;
      [PAR, jobinfo.(sensor),all_done.(sensor)] = masterscript_sensors(PAR,check_files);
      if ~isempty(jobinfo.(sensor))
        jobinfo.(sensor) = rproc_resubmit(jobinfo.(sensor));
        JOBINFO = [JOBINFO jobinfo.(sensor)];
      end
      ALL_DONE = [ALL_DONE all_done.(sensor)];
    end      
    rproc_wait(JOBINFO, 30);  
    all_predictions_done = all(ALL_DONE);
    if all(all_predictions_done)
      fprintf('\nSIGNALS COMPLETE n')
    end
  end
  if next_step, next_step=next_step-1 ; status = 'prep_blocks'; end ;
end

if ~isfield(PAR,'keep_signals')
  PAR.keep_signals = 0;
end

PAR.model.use.pre_comp=0 ;

% prepare the training blocks
if isequal(status,'prep_blocks')  & ~fexist(PAR.FN.input_lsl.fn_training_blocks)
  PAR.fn_blocks = PAR.FN.input_lsl.fn_training_blocks;
  PAR.fn_split = PAR.FN.input_lsl.fn_training_split;
  PAR.fn_blocks_old = PAR.FN.input_lsl.fn_old_training_blocks;
  prepare_nGASP_blocks(PAR,'train')

  if next_step, next_step=next_step-1 ; status = 'layer2'; end ;
end

% train the second layer of mGene
if  isequal(status,'layer2') 
  PAR = C_elegans_new_training_config(PAR);
  PAR.model.penalties=PAR.model.transition_pointers ;

  PAR.fn_blocks = PAR.FN.input_lsl.fn_training_blocks;
  PAR.fn_splits = PAR.FN.input_lsl.fn_training_split;
  %PAR.fn_blocks_old = PAR.FN.input_lsl.fn_old_training_blocks;

  assert(fexist(PAR.fn_blocks) | fexist([PAR.fn_blocks '.mat'])) ;

  PAR.ignore_init_file = 0 ;
  PAR.skip_no_violation = 0 ;
  
  PAR.initial_subset_iteration = 50 ;
  train_path(PAR) ;
  
  if next_step, next_step=next_step-1 ; status = 'tune_tis_on_validation' ; end ;
end

% compute the validation error for several parameters of the tis_offset
if isequal(status, 'tune_tis_on_validation'),
  keyboard
  PAR = C_elegans_new_training_config(PAR);
  PAR.model.penalties=PAR.model.transition_pointers ;
  PAR.fn_blocks = PAR.FN.input_lsl.fn_training_blocks;
  PAR.fn_split = PAR.FN.input_lsl.fn_training_split;
  PAR.RPROC.MEMREQ=1000 ;
  
  jobinfo = rproc_empty(0) ;
  options = [] ;
  options.addpaths=separate(path, ':') ;
  pred={} ;
  offsets=[3:0.5:5.5]
  for offset=offsets,
    PAR.cleave_offset = offset ;
    %pred{end+1}=pred_path_cleave(PAR) ;
    jobinfo(end+1) = rproc('pred_path_cleave', PAR, 8000, options, 30) ;
  end ;
  jobinfo = rproc_wait(jobinfo, 10, 1, 0) ;
  for i=1:length(jobinfo),
    pred{i} = rproc_result(jobinfo(i)) ;
  end ;
  rproc_cleanup(jobinfo) ;
  
  genes_anno=[] ; res_cat1={} ; genes_pred={} ; operons_pred={} ;
  for j=1:length(pred),
    blocks=pred{j} ;
    genome_info = init_genome('/fml/ag-raetsch/share/projects/genefinding//C_elegans/WS160/orig_data/genome.config');
    L=load('/fml/ag-raetsch/share/projects/nGASP/training_regions.mat');
    regions=L.training_regions; clear L
    
    [genes_pred{end+1},operons_pred{end+1}] = blocks2genes(blocks, PAR.model, 'prediction', genome_info, 'nematode');
    
    if isempty(genes_anno),
      for i=1:length(blocks),
        blocks(i).my_truth = blocks(i).truth(1) ; 
      end ;
      [genes_anno,operons_anno] = blocks2genes(blocks, PAR.model, 'my_truth', genome_info, 'nematode');
    end ;
    offsets(j)
    res_cat1{end+1} = evaluate_predictions(regions,genes_anno,genes_pred{end},genome_info,0);
  end 
  res_cat1=[res_cat1{:}] ;
  
  R=[res_cat1.cds_nucleotides]; Rn=[R.mean_SNSP] ;
  R=[res_cat1.cds_exons];       Re=[R.mean_SNSP] ;
  R=[res_cat1.cds_transcripts]; Rt=[R.mean_SNSP] ;
  R=(Rn+Re+Rt)/3 ;
  [tmp,idx]=max(R) ;
  best_offset = offsets(idx) 

  save tune_tis_on_validation res_cat1 genes_pred operons_pred pred best_offset R Rn Re Rt

  if next_step, next_step=next_step-1 ; status = 'prep_blocks_test' ; end 
end ;

if isequal(status, 'prep_blocks_test')  && ~fexist(PAR.FN.input_lsl.fn_test_blocks)
  PAR = C_elegans_new_training_config(PAR);
  PAR.model.penalties=PAR.model.transition_pointers ;
  PAR.fn_blocks = PAR.FN.input_lsl.fn_test_blocks;
  PAR.fn_blocks_old = PAR.FN.input_lsl.fn_old_test_blocks;
  prepare_nGASP_blocks(PAR,'test')

  if next_step, next_step=next_step-1 ; status = 'predict_blocks_test' ; end ;
end

if isequal(status, 'predict_blocks_test'),
  PAR = C_elegans_new_training_config(PAR);
  PAR.model.penalties=PAR.model.transition_pointers ;
  PAR.fn_blocks = PAR.FN.input_lsl.fn_test_blocks;
  PAR.fn_pred = strrep(PAR.fn_pred,'pred_split=5', 'pred_test') ;

  if isfield(PAR,'fn_splits')
    PAR=rmfield(PAR, 'fn_splits') ;
  end
  PAR.RPROC.MEMREQ=4000 ;
  L=load('tune_tis_on_validation', 'best_offset') ;
  PAR.cleave_offset=best_offset ;
  blocks = pred_path_cleave(PAR) ;

  genome_info = init_genome('/fml/ag-raetsch/share/projects/genefinding//C_elegans/WS160/orig_data/genome.config');
  L=load('/fml/ag-raetsch/share/projects/nGASP/test_regions.mat');
  regions=L.test_regions; clear L
  
  [genes_pred,operons_pred] = blocks2genes(blocks, PAR.model, 'prediction', genome_info, 'nematode');
  
  L=load('/fml/ag-raetsch/share/projects/nGASP/benchmark_gene_sets/phase1_confirmed_isoforms_CNEW_complete.mat')
  genes_anno = L.genes ;

  res_cat1 = evaluate_predictions(regions, genes_anno, genes_pred, genome_info, 0);
  save predict_blocks_test res_cat1 genes_pred operons_pred blocks PAR;

  if next_step, next_step=next_step-1 ; status='prepare_cat3_blocks' ; end ;
end

if isequal(status, 'prepare_cat3_blocks'),
  PAR = C_elegans_new_training_config(PAR);
  PAR.model.penalties=PAR.model.transition_pointers ;

  prepare_cat3_blocks

  if next_step, next_step=next_step-1 ; status='predict_blocks_test_cat3' ; end ;
end ;

if isequal(status, 'predict_blocks_test_cat3'),
  PAR = C_elegans_new_training_config(PAR);
  PAR.model.penalties=PAR.model.transition_pointers ;

  PAR.fn_blocks = [PAR.FN.input_lsl.fn_test_blocks '-with-signalmasks_withends.mat'] ;
  PAR.fn_pred = [PAR.fn_pred '-with-signalmasks_withends'] ;
  PAR.fn_pred = strrep(PAR.fn_pred,'pred_split=5', 'pred_test') ;

  if isfield(PAR,'fn_splits')
    PAR=rmfield(PAR, 'fn_splits') ;
  end
  PAR.RPROC.MEMREQ=4000 ;
  % L=load('tune_tis_on_validation', 'best_offset') ;
  % PAR.cleave_offset=best_offset ;
  offsets=[4.5:0.5:6.0]
  for offset=offsets
    PAR.cleave_offset=offset ;
    blocks=pred_path_cleave(PAR) ;

    % genome_info = init_genome('/fml/ag-raetsch/share/projects/genefinding//C_elegans/WS160/orig_data/genome.config');
    % L = load('/fml/ag-raetsch/share/projects/nGASP/test_regions.mat');
    % regions = L.test_regions; clear L
    
    % [genes_pred, operons_pred] = blocks2genes(blocks, PAR.model, 'prediction', genome_info, 'nematode');
    
    % L=load('/fml/ag-raetsch/share/projects/nGASP/benchmark_gene_sets/phase1_confirmed_isoforms_CNEW_complete.mat')
    % genes_anno = L.genes ;
    
    % res_cat3 = evaluate_predictions(regions, genes_anno, genes_pred, genome_info, 0);
    
    % save predict_blocks_test_cat3 res_cat3 genes_pred operons_pred blocks PAR;
  end ;
end