function prepare_nGASP_blocks(PAR, mode)

assert(isequal(mode,'train')|isequal(mode,'test'))  
  
%load old blocks
blocks = load_struct(PAR.fn_blocks_old, 'blocks') ;

if ~isfield(PAR,'keep_signals')
  PAR.keep_signals = 0;
end ;

if PAR.keep_signals
  warning('old signals used')
end

%-----------------------
% add signals
%-----------------------
if ~PAR.keep_signals,
  blocks_tmp = blocks;
  cont2chrom;
  genome_info = init_genome(PAR.FN.input.fn_genome_config);
  
  %-----------------------
  % temporarly transform to nGASP coordinates
  %-----------------------
  
  for i=1:length(blocks)  
    if blocks(i).strand=='+'
      contig = find(strcmp({cont.chr},blocks(i).chr)&[cont.start]<=blocks(i).block_start&[cont.stop]>=blocks(i).block_end);
      assert(length(contig)==1)
      blocks(i).config = PAR.FN.input.fn_genome_config;
      
      blocks(i).chr = cont(contig).name;
      blocks(i).chr_num = contig;
      blocks(i).start = blocks(i).block_start-cont(contig).start+1;
      blocks(i).stop = blocks(i).block_end-cont(contig).start+1;
    else
      contig = find(strcmp({cont.chr},blocks(i).chr)&[cont.start]<=blocks(i).block_end&[cont.stop]>=blocks(i).block_start);
      assert(length(contig)==1)
      blocks(i).config = PAR.FN.input.fn_genome_config;    
      blocks(i).chr = cont(contig).name;
      blocks(i).chr_num = contig;
      blocks(i).start = blocks(i).block_end-cont(contig).start+1;
      blocks(i).stop = blocks(i).block_start-cont(contig).start+1;
    end
  end
  
  sig_names = fieldnames(PAR.model.signals);
  idx = find(strcmp(sig_names,'trans'));
  sig_names(idx) =[];
  % set offset =100;
  blocks = add_signals2blocks(blocks,PAR,sig_names);

  if 0,
    % this is a quick-fix to handle a problem with the nGASPcheat
    % position problems for don and cdsStop
    % problem that remained unresolved so far
    warning('quick-fixing donor and cdsStop signals') ;
    for b = 1:length(blocks) 
      blocks(b).Signals.don.pos = blocks(b).Signals.don.pos+1 ;
      blocks(b).Signals.cdsStop.pos = blocks(b).Signals.cdsStop.pos-2 ;
    end ;
  end ;
 
  for b = 1:length(blocks) 
    % trans = score of transc_vs_acc times score of acc_vs_nonacc
    blocks(b).Signals.trans = blocks(b).Signals.transacc;
    blocks(b).Signals.trans.Conf_cum = blocks(b).Signals.transacc.Conf_cum.* blocks(b).Signals.acc.Conf_cum ;
    % transacc = score of acc_vs_nonacc 
    blocks(b).Signals.transacc = blocks(b).Signals.acc;  
    blocks(b).Signals.stop = blocks(b).Signals.cdsStop; 
  end   

  %-----------------------
  % checks
  %-----------------------
  
  idx = find(strcmp(sig_names,'cdsStop'));
  sig_names{idx} = 'stop';
  
  if isfield(blocks, 'details')
    for i=1:length(sig_names)
      sig_names{i}
      for b = 1:length(blocks) 
        if isequal(sig_names{i},'acc')||isequal(sig_names{i},'don')||...
              isequal(sig_names{i},'tis')||isequal(sig_names{i},'cdsStop')
          assert(isempty(setdiff(blocks(b).Signals.(sig_names{i}).pos,blocks(b).details.pos.(sig_names{i}))));
        end
        % blocks(b).Signals.(sig_names{i}) = rmfield(blocks(b).Signals.(sig_names{i}),contig_pos);
        %if mod(b,20)==0
        %  [tmp,idx1,idx2] = intersect(blocks(b).Signals.(sig_names{i}).pos,blocks(b).details.pos.(sig_names{i}));
        %  plot(blocks(b).Signals.(sig_names{i}).Conf_cum(idx1), ...
        %       blocks(b).details.score.(sig_names{i})(idx2),'*');drawnow
        %end
      end
    end   
  end ;
  blocks = load_sequence(blocks);
  if isfield(blocks, 'genestr'),
    for i=1:length(blocks)
      assert(isequal(blocks(i).genestr,blocks(i).seq))
    end
  else
    for i=1:length(blocks),
      blocks_tmp(i).genestr = blocks(i).seq ; 
    end ;
  end ;
  % check_signals(blocks,PAR)
  
  %-----------------------
  % convert to old type
  %-----------------------
  
  if isfield(blocks, 'details'),
    blocks = rmfield(blocks,'details');
  end ;
  blocks = convertblocks2oldtype(blocks);
  for i=1:length(blocks)
    blocks_tmp(i).details = blocks(i).details;
  end
  blocks = blocks_tmp;
  clear blocks_tmp ;
  
else
  for i=1:length(blocks),
    blocks(i).details.pos.cdsStop = blocks(i).details.pos.stop ;
    blocks(i).details.score.cdsStop = blocks(i).details.score.stop ;
  end ;
  if isempty(PAR.model.states(PAR.model.state_ids.polya).consensus{:})
    warning('polya consensus empty; loading old consensus')
    O=load(PAR.fn_blocks_old)
    PAR.model.states(PAR.model.state_ids.polya).consensus = ...
        O.PAR.model.states(O.PAR.model.state_ids.polya).consensus ;
    PAR.model.states(PAR.model.state_ids.polya).consensus_pos = ... 
        O.PAR.model.states(O.PAR.model.state_ids.polya).consensus_pos ;    
    PAR.model.states(PAR.model.state_ids.polya)
    clear O
  end
end ;

if isfield(blocks, 'seq'),
  blocks = rmfield(blocks,'seq');
end ;
if isfield(blocks, 'pos'),
  blocks = rmfield(blocks,'pos');
end ;
save_struct(PAR.fn_blocks, blocks, 'blocks')

%-----------------------
% regenerate seq and pos
%-----------------------

for i=1:length(blocks)
  fprintf('%i\r',i)
  [blocks(i).seq, blocks(i).pos] = gen_seq(blocks(i), PAR.model);
end
save_struct(PAR.fn_blocks,blocks,'blocks')
 
% blocks = load_struct('/fml/ag-raetsch/share/projects/genefinding/C_elegans/nGASP/betterOperonModel/lsl/cat-1//data/old_blocks_new_signals','blocks')

if isequal(mode,'train')
  %-----------------------
  % regenerate truth
  %-----------------------

  blocks = rmfield(blocks,'truth');
  which gen_truth
  blocks = gen_truth(blocks,PAR.model) ;
  save_struct(PAR.fn_blocks, blocks, 'blocks')
  
  %-----------------------
  % check consistency
  %-----------------------
  
  ok_idx1 = check_blocks_segmentation(blocks,PAR.model)
  %addpath ~/svn/projects/genefinding/lsl_new/

  for i=1:length(blocks)
    blocks(i).all_pos =  blocks(i).pos;
    blocks(i).features =  blocks(i).seq;
    blocks(i).seq =  blocks(i).genestr;
  end
  ok_idx2 = check_block_model_consistency(blocks, PAR.model)
  ok_idx = intersect(ok_idx1, ok_idx2) ;

  %-----------------------
  % make split
  %-----------------------
  L=load(PAR.FN.input_lsl.fn_old_split)
  blocks_split{1}=intersect(L.blocks_split{1},ok_idx);
  blocks_split{5}=intersect(L.blocks_split{5},ok_idx);
  save(PAR.fn_split,'blocks_split')
end
return

%%%correcting

blocks = load_struct(PAR.FN.input_lsl.fn_old_training_blocks, 'blocks') ;
blocks_n = load_struct(PAR.FN.input_lsl.fn_training_blocks, 'blocks') ;
O=load(PAR.FN.input_lsl.fn_old_training_blocks)
PAR.model.states(PAR.model.state_ids.polya).consensus = ...
    O.PAR.model.states(O.PAR.model.state_ids.polya).consensus ;
PAR.model.states(PAR.model.state_ids.polya).consensus_pos = ... 
    O.PAR.model.states(O.PAR.model.state_ids.polya).consensus_pos ;    
PAR.model.states(PAR.model.state_ids.polya)
clear O

for i=1:length(blocks)

  blocks(i).pos=blocks_n(i).pos;
  blocks(i).seq=blocks_n(i).seq;
  
  blocks(i).details = blocks_n(i).details;
  blocks(i).truth = blocks_n(i).truth;
  blocks(i).has_valid_transcript = blocks_n(i).has_valid_transcript;
  blocks(i).overlaps_with_prev = blocks_n(i).overlaps_with_prev;
  blocks(i).gene_complete = blocks_n(i).gene_complete;
end ;
save_struct(PAR.FN.input_lsl.fn_training_blocks, blocks, 'blocks')
 
ok_idx1 = check_blocks_segmentation(blocks,PAR.model)
for i=1:length(blocks)
  blocks(i).all_pos =  blocks(i).pos;
  blocks(i).features =  blocks(i).seq;
  blocks(i).seq =  blocks(i).genestr;
end ;
ok_idx2 = check_block_model_consistency(blocks, PAR.model)
ok_idx = intersect(ok_idx1, ok_idx2) ;

%-----------------------
% make split
%-----------------------
L=load(PAR.FN.input_lsl.fn_old_split)
blocks_split{1}=intersect(L.blocks_split{1},ok_idx);
blocks_split{5}=intersect(L.blocks_split{5},ok_idx);
save(PAR.fn_split,'blocks_split')
