function annotate_frames(PAR)
% function annotate_frames(PAR)

% load blocks used for LSL
tmp = load(PAR.LSL_block_file);
segment_types = tmp.PAR.model.segments;
blocks = tmp.blocks;
clear tmp

% prepare subset indices for blocks
num_blocks_total = length(blocks);
num_blocks_per_subset = ceil(num_blocks_total/PAR.SETs.num_splits)
subset_ids = [];
for i=1:PAR.SETs.num_splits,
  subset_ids = [subset_ids i*ones(1,num_blocks_per_subset)];
end;
% randomize subset ids
subset_ids = subset_ids(randperm(length(subset_ids)));
% assert that we created enough subset ids
assert(num_blocks_total<=length(subset_ids));

% get rid of uninteresting fields
blocks = rmfield(blocks, 'is_alt_spliced');
blocks = rmfield(blocks, 'trans');
blocks = rmfield(blocks, 'anno_tss');
blocks = rmfield(blocks, 'tss');
blocks = rmfield(blocks, 'polya');
blocks = rmfield(blocks, 'anno_cleave');
blocks = rmfield(blocks, 'cleave');
blocks = rmfield(blocks, 'splicegraph');
blocks = rmfield(blocks, 'details');
blocks = rmfield(blocks, 'content');
blocks = rmfield(blocks, 'gene_complete');

% process LSL blocks
for b=1:length(blocks),
  block = blocks(b);
  skip = ~block.has_valid_transcript;
  if PAR.ignore_alt_genes,
    skip = skip | block.is_alt;
  end;
  blocks(b).frame.exons = cell(1,length(skip));
  blocks(b).frame.subset_id = subset_ids(b);
  
  for g=1:length(block.truth(1).genes),
    if skip(g),
      continue;
    end;
    gene = block.truth(1).genes{g};
    if isempty(gene)
      skip(g) = 1;
      continue;
    end;
    
    cnt = 1;
    frame = 0;
    blocks(b).frame.exons{g} = ...
        zeros(sum(gene(:,3)==segment_types.cds_exon),3);
    for s=1:size(gene,1),
      if gene(s,3)==segment_types.cds_exon,
        % interval coords in truth(1).genes are half-open...
        blocks(b).frame.exons{g}(cnt,:) = [gene(s,1) gene(s,2)-1 frame];
        exon_len = gene(s,2)-gene(s,1);
        frame = mod(frame+exon_len,3);
        cnt = cnt+1;
      end;
    end;
    % to correct for akward coords in truth(1).genes
    blocks(b).frame.exons{g}(end,2) = ...
        blocks(b).frame.exons{g}(end,2)+3;
    exon_lens = blocks(b).frame.exons{g}(:,2)...
        - blocks(b).frame.exons{g}(:,1) + 1;
    assert(mod(sum(exon_lens),3)==0);
    assert(frame==0);

    start_codon = blocks(b).frame.exons{g}(1,1);
    start_codon = block.genestr(start_codon:start_codon+2);
    
    assert(upper(start_codon) == 'ATG');
    
    stop_codon  = blocks(b).frame.exons{g}(end,2)-2;
    stop_codon  = block.genestr(stop_codon:stop_codon+2);    
    assert(upper(stop_codon) == 'TAA' ...
           | upper(stop_codon) == 'TAG' ...
           | upper(stop_codon) == 'TGA');
  end;
  blocks(b).frame.skip = skip;  
end;

% get rid of uninteresting fields
blocks = rmfield(blocks, 'is_alt');
blocks = rmfield(blocks, 'is_valid');
blocks = rmfield(blocks, 'tis');
blocks = rmfield(blocks, 'tis_is_valid');
blocks = rmfield(blocks, 'stop');
blocks = rmfield(blocks, 'stop_is_valid');
blocks = rmfield(blocks, 'has_invalid_don');
blocks = rmfield(blocks, 'has_invalid_acc');
blocks = rmfield(blocks, 'truth');


save(PAR.frame_block_file, 'blocks');
fprintf('\nsaved blocks to %s\n', PAR.frame_block_file);  

