function [pos_DATA neg_DATA pos_DATA_cons neg_DATA_cons] = balance_length(type, DATA, PAR)

VERBOSE = 0;
assert(type<=length(DATA));

pos_DATA = {DATA(type).segments{:}};
if PAR.use_conservation,
  pos_DATA_cons = {DATA(type).conservation{:}};
end;
neg_idx = 1:length(DATA);
neg_idx(neg_idx==type) = [];

% chop large intergenic regions into smaller pieces
if type == PAR.track_info.intergenic,
  pos_DATA = {};
  if PAR.use_conservation,
    pos_DATA_cons = {};
  end;
  t1 = PAR.track_info.CDS_exon;
  t2 = PAR.track_info.intron;
  other_lens = zeros(1,length(DATA(t1).segments)+length(DATA(t2).segments));
  for i=1:length(DATA(t1).segments),
    other_lens(i) = length(DATA(t1).segments{i});    
  end;
  for i=1:length(DATA(t2).segments),
    other_lens(i+length(DATA(t1).segments)) = length(DATA(t2).segments{i});
  end;
  mean_len = mean(other_lens);
  cnt = 1;
  ig_lens = [];
  num_picks = 2;
  t = PAR.track_info.intergenic;
  for i=1:length(DATA(t).segments),
    % chop intergenic regions into pieces
    num_chops = round(length(DATA(t).segments{i})/mean_len)-1;
    chops = [0 ceil(rand(1,num_chops)*(length(DATA(t).segments{i}))-1) ...
             length(DATA(t).segments{i})];
    chops = sort(chops);
    for j=1:num_picks,
      k = ceil(rand(1)*(length(chops)-1));
      if chops(k+1)-chops(k) >= PAR.Contents.min_segment_len,
        pos_DATA{cnt} = DATA(t).segments{i}(chops(k)+1:chops(k+1));
        if PAR.use_conservation,
          pos_DATA_cons{cnt} = DATA(t).conservation{i}(chops(k)+1:chops(k+1));
        end;       
        ig_lens(cnt) = length(pos_DATA{cnt});
        cnt = cnt + 1;
      end;
    end;
  end;
end;

neg_DATA{length(pos_DATA)} = [];
if PAR.use_conservation,
  neg_DATA_cons{length(pos_DATA_cons)} = [];
end;
pos_lens = zeros(1,length(pos_DATA));
neg_lens = zeros(1,length(neg_DATA));
for i=1:length(pos_DATA),
  r_type = max([1 ceil(length(neg_idx)*rand(1))]);
  r_idx = neg_idx(r_type);
  
  if PAR.use_conservation,
    [neg_DATA{i} rm_idx neg_DATA_cons{i}] = ...
        equalize_length_cons(length(pos_DATA{i}), DATA(r_idx).segments, DATA(r_idx).conservation);
  else
    [neg_DATA{i} rm_idx] = equalize_length(length(pos_DATA{i}), DATA(r_idx).segments);
  end;
  % remove negative example after it has been sampled
  DATA(r_idx).segments(rm_idx) = [];
  if PAR.use_conservation,
    DATA(r_idx).conservation(rm_idx) = [];
  end;
  % remove whole segment class if depleted
  if (length(DATA(r_idx).segments)==0),
    neg_idx(r_type) = [];
  end;
  
  pos_lens(i) = length(pos_DATA{i});
  neg_lens(i) = length(neg_DATA{i});
  if pos_lens(i) > neg_lens(i) & PAR.Contents.strict_len_balance,
    pos_start = ceil(rand*(pos_lens(i)-neg_lens(i)));
    pos_DATA{i} = pos_DATA{i}(pos_start:pos_start+neg_lens(i)-1);
    if PAR.use_conservation,
      pos_DATA_cons{i} = pos_DATA_cons{i}(pos_start:pos_start+neg_lens(i)-1);
      assert(length(pos_DATA{i})==length(pos_DATA_cons{i}));
    end;
    assert(length(pos_DATA{i})==length(neg_DATA{i}));
    pos_lens(i) = length(pos_DATA{i});
  end;
end;

if VERBOSE,
  figure; hold on;
  [n1,x1] = hist(pos_lens,50);
  [n2,x2] = hist(neg_lens,x1);
  plot(x1,n1);
  plot(x2,n2,'r');
  title('Length histogram of content sensor examples');
end;

for i=1:length(pos_DATA),
  pos_DATA{i} = upper(pos_DATA{i});
  if PAR.use_conservation,
    pos_DATA_cons{i} = upper(pos_DATA_cons{i});
  end;
end;
for i=1:length(neg_DATA),
  neg_DATA{i} = upper(neg_DATA{i});
  if PAR.use_conservation,
    neg_DATA_cons{i} = upper(neg_DATA_cons{i});
  end;
end;
