function DATA = collect_content_examples(blocks, PAR)
% DATA = collect_content_examples(blocks, PAR)
% DATA struct of length num_types (intergenic, intron, exon...)
% DATA.segments  cell each entry containing one example sequence
% (corresponds to XT in Signals)
% DATA.type  
% DATA.type_descr 
  
  types = [PAR.track_info.intergenic, ...
         PAR.track_info.UTR_5prime, ...
         PAR.track_info.CDS_exon, ...
         PAR.track_info.intron, ...
         PAR.track_info.UTR_3prime, ...
        ];


for t=1:PAR.num_types,
  DATA(t).segments = {};
  if PAR.use_conservation,
    DATA(t).conservation = {};
  end;
end;

for t=1:length(types),
  for b=1:length(blocks)
    assert(all(blocks(b).unique_type~=0));
    starts = find([0 blocks(b).unique_type(1:end-1)]~=t ...
                  & blocks(b).unique_type==t);
    stops  = find(blocks(b).unique_type==t ...
                  & [blocks(b).unique_type(2:end) 0]~=t);
    if t==PAR.track_info.UTR_5prime & PAR.Contents.combine_UTRs,
      u = PAR.track_info.UTR_3prime;
      starts = [starts find([0 blocks(b).unique_type(1:end-1)]~=u ...
                            & blocks(b).unique_type==u)];
      stops  = [stops find(blocks(b).unique_type==u ...
                            & [blocks(b).unique_type(2:end) 0]~=u)];
    end;
    assert(all(stops>=starts));
    
    % process intergenic segments in a different way than other types
    if t==PAR.track_info.intergenic
      % check whether there are UTRs annotated around intergenic region
      % if not, cut out sequence further away from genes
      has_UTR_3prime = [0 blocks(b).unique_type(starts(2:end) - 1) == ...
                        PAR.track_info.UTR_3prime];
      has_UTR_5prime = [blocks(b).unique_type(stops(1:end-1) + 1) == ...
                        PAR.track_info.UTR_5prime 0];
      intercistronic = find(stops-starts+1 <= PAR.Contents.max_intercistronic_len);
           
      if ~isempty(intercistronic),
        % first or last intergenic segment cannot be intercistronic
        if intercistronic(1)==1,
          intercistronic = intercistronic(2:end);
        end;
        if ~isempty(intercistronic) & intercistronic(end)==length(starts),
          intercistronic = intercistronic(1:end-1);
        end;
        
        for i=1:length(intercistronic),
          idx = intercistronic(i);
          DATA(PAR.num_types).segments{end+1} = ...
              blocks(b).genestr(starts(idx):stops(idx));
          
          if PAR.use_conservation,
            DATA(PAR.num_types).conservation{end+1} = ...
                blocks(b).conservation(starts(idx):stops(idx));
          end;
        end;
        % remove intercistronic segments from intergenic segments
        starts(intercistronic)         = [];
        stops(intercistronic)          = [];
        has_UTR_3prime(intercistronic) = [];
        has_UTR_5prime(intercistronic) = [];
      end;
      
      trunc_left_idx = find(~has_UTR_3prime);
      % never truncate first intergenic segment in block
      trunc_left_idx = trunc_left_idx(2:end); 
      starts(trunc_left_idx) = starts(trunc_left_idx) + PAR.Contents.truncate_intergenic;
      
      trunc_right_idx = find(~has_UTR_5prime);
      % never truncate last intergenic segment in block
      trunc_right_idx = trunc_right_idx(1:end-1); 
      stops(trunc_right_idx) = stops(trunc_right_idx) - PAR.Contents.truncate_intergenic;
      
      too_small = find(stops-starts+1 < PAR.Contents.min_segment_len);
      starts(too_small) = [];
      stops(too_small)  = [];
      has_UTR_3prime(too_small) = [];
      has_UTR_5prime(too_small) = [];
      
      utr_3p_idx = find(has_UTR_3prime);
      utr_5p_idx = find(has_UTR_5prime);
      % collect intergenic sequences downstream of genes
      for i=1:length(utr_3p_idx),
        idx = utr_3p_idx(i);
        DATA(t).segments{end+1} = ...
            blocks(b).genestr(starts(idx):min(stops(idx), starts(idx)+999));
        if PAR.use_conservation,
          DATA(t).conservation{end+1} = ...
              blocks(b).conservation(starts(idx):min(stops(idx), starts(idx)+999));
        end;
      end;
      % collect intergenic sequences upstream of genes
      for i=1:length(utr_5p_idx),
        idx = utr_5p_idx(i);
        DATA(t).segments{end+1} = ...
            blocks(b).genestr(max(starts(idx), stops(idx)-999):stops(idx));
        if PAR.use_conservation,
          DATA(t).conservation{end+1} = ...
              blocks(b).conservation(max(starts(idx), stops(idx)-999):stops(idx));
        end;
      end;
    else % for all segment types other than intergenic
      too_small = find(stops-starts+1 < PAR.Contents.min_segment_len);
      starts(too_small) = [];
      stops(too_small)  = [];
      
      if ~isempty(starts),
        % collect sequences
        for i=1:length(starts),
          DATA(t).segments{end+1} = blocks(b).genestr(starts(i):stops(i));
          if PAR.use_conservation,
            DATA(t).conservation{end+1} = ...
                blocks(b).conservation(starts(i):stops(i));
          end;
        end;
      end;
      clear starts stops
    end;
  end;
  
  r_idx = randperm(length(DATA(t).segments));
  DATA(t).segments = {DATA(t).segments{r_idx}};
  if PAR.use_conservation,
    DATA(t).conservation = {DATA(t).conservation{r_idx}};
  end;
  DATA(t).type = t;
  DATA(t).type_descr = PAR.type_descrs{t};
end;
