function evaluate_content()

addpath ~/svn/tools/shogun/

results_dir = '/fml/ag-raetsch/share/projects/nGASP/content/';
%%%%% SET THIS:
results_dir = [results_dir 'RESULTS_2006-12-22_17h27/'];

% load parameters
param_file = [results_dir 'global_parameters.mat'];
load(param_file, 'PAR');

% load blocks
load(PAR.content_block_file, 'content_blocks');

num_track_types = PAR.num_types;
if PAR.Contents.max_intercistronic_len>0
  num_track_types = num_track_types-1;
end;
  
% collect segment
segments = [];
segment_seqs = {};
cnt_seg = 0;
cnt_blc = 0;
for b=1:length(content_blocks),
  block = content_blocks(b);
  % find unambiguous segments in unique_type
  for t=1:num_track_types,
    if PAR.Contents.combine_UTRs & t==2,
      starts1 = find([0 block.unique_type(1:end-1)]~=t & block.unique_type==t);
      stops1  = find(block.unique_type==t & [block.unique_type(2:end) 0]~=t);
      
      u = PAR.track_info.UTR_3prime;
      starts2 = find([0 block.unique_type(1:end-1)]~=u & block.unique_type==u);
      stops2  = find(block.unique_type==u & [block.unique_type(2:end) 0]~=u);
      starts = [starts1 starts2];
      stops  = [stops1  stops2];
    else
      starts = find([0 block.unique_type(1:end-1)]~=t & block.unique_type==t);
      stops  = find(block.unique_type==t & [block.unique_type(2:end) 0]~=t);
    end;
    if isempty(starts),
      assert(isempty(stops));
    else
      assert(all(stops>=starts));
      % "allocate memory"
      segments(:,cnt_seg+length(starts)) = zeros(3,1);
      
      % store segment boundaries and true label
      segments(:,cnt_seg+1:cnt_seg+length(starts)) = [starts; stops; t*ones(size(starts))];
      % differentiate between intergenic and intercistronic simply
      % by length
      if t==PAR.track_info.intergenic,
        ics_idx = find(stops-starts+1 <= PAR.Contents.max_intercistronic_len);
        segments(3,cnt_seg+ics_idx) = PAR.num_types;
      end;
      
      for i=1:length(starts),
        segment_seqs{cnt_seg+i} = block.genestr(starts(i):stops(i));
        segment_test_set_ids(cnt_seg+i) = block.content.subset_id;
      end;
      assert(size(segments,2)==length(segment_seqs));
      assert(size(segments,2)==size(segment_test_set_ids,2));
      cnt_seg = size(segments,2);
    end;
    clear starts stops
    assert(all(sum(segments==0)==0));
  end;
  b_idx = cnt_blc+1;
  e_idx = size(segments,2);
  added_segments = segments(:,b_idx:e_idx);
  added_seqs = segment_seqs(b_idx:e_idx);
  added_test_set_ids = segment_test_set_ids(b_idx:e_idx);
  
  [dummy perm_idx] = sort(added_segments(1,:));
  segments(:,b_idx:e_idx) = added_segments(:,perm_idx);
  segment_seqs(b_idx:e_idx) = added_seqs(perm_idx);
  segment_test_set_ids(b_idx:e_idx) = added_test_set_ids(perm_idx);
  
  cnt_blc = size(segments,2);
  fprintf('Parsed segments of %i blocks      \r', b);
end;
fprintf('\n\n');

segment_kmers   = cell(1,length(PAR.orders));
segment_pred    = cell(1,length(PAR.orders));
segment_max_idx = cell(1,length(PAR.orders));

for o=PAR.orders,
  % for this evaluation we need the SVMs (alphs and bs) as we have no
  % PLIFs that could correct for biases (b)
  for t=1:PAR.num_types,
    for i=1:PAR.SETs.num_splits,
      test_subset = 1+mod(i+PAR.SETs.num_splits-2,PAR.SETs.num_splits);
      % load content SVMs for intergenic segments
      file_name = [PAR.svm_file_prefix '_' PAR.type_abbrs{t} ...
                   '_split' num2str(i) '_order' num2str(o)];
      load(file_name);
      fprintf('Loaded SVM from %s\n', file_name);
      assert(SVM.PARAM.test_set_id == test_subset);      
      assert(SVM.PARAM.order == o);      
      
      SVMs(t,i) = SVM;
    end;
  end;
  clear SVM
    

  % predict on segments
  C          = 1 ;      
  cache_size = 10 ;
  
  segment_kmers{o} = zeros(4^o,size(segments,2));
  for i=1:length(segment_seqs),
    segment_kmers{o}(:,i) = compute_mers(segment_seqs{i}, o);
  end;
  
  segment_pred{o} = zeros(PAR.num_types,size(segment_kmers{o},2));
  for t=1:PAR.num_types,
    for i=1:PAR.SETs.num_splits,
      SVM = SVMs(t,i);
      subset_idx = find(segment_test_set_ids==SVM.PARAM.test_set_id);
      XTE = segment_kmers{o}(:,subset_idx);
      type_labels = segments(3,subset_idx);
      labels = -ones(size(type_labels));
      labels(type_labels==t) = 1;
      
      sg('send_command', 'use_linadd 1');
      sg('set_features', 'TRAIN', SVM.hist_XTR) ;
      sg('set_labels', 'TRAIN', SVM.LTR) ;

      sg('send_command', sprintf('set_kernel LINEAR REAL %i 1.0', cache_size));
      sg('send_command', 'new_svm LIGHT') ;
      sg('send_command', 'init_kernel TRAIN');
      sg('send_command', sprintf( 'c %1.2e', C));
      
      sg('set_svm', SVM.b, SVM.alphas_sg) ;  

      
      % determine test error
      sg('send_command', 'init_kernel_optimization') ;
      sg('set_features', 'TEST', SVM.hist_XTE) ;
      sg('send_command', 'init_kernel TEST');
      test_out = sg('svm_classify');
      ROC = calcrocscore(test_out, SVM.LTE);
      fprintf('\n\nRecomputed ROC score for SVMs test set: %2.2f (%s)\n', ...
              ROC, PAR.type_abbrs{t});
      fprintf(' should equal original score            %2.2f (%s)\n\n', ...
              SVM.ROC_score, PAR.type_abbrs{t});
      
      clear ROC
      % predict real fragments
      sg('set_features', 'TEST', XTE) ;
      sg('send_command', 'init_kernel TEST');
      
      out = sg('svm_classify');
      ROC = calcrocscore(out, labels);
      if isnan(ROC), keyboard; end;
      fprintf('\n\nROC score for "real" fragments:         %2.2f (%s)\n\n', ...
              ROC, PAR.type_abbrs{t});
      segment_pred{o}(t,subset_idx) = out;
    end;
  end;
  fprintf('\n\n');

  [dummy segment_max_idx{o}] = max(segment_pred{o});
end;

for o=PAR.orders,
  figure;
  part_beg = 2000; part_end = 2500;
  part_max_idx = segment_max_idx{o}(part_beg:part_end);
  part_labels  = segments(3,part_beg:part_end);
  part_lengths = segments(2,part_beg:part_end) - segments(1,part_beg:part_end) + 1;
  part_kmers   = segment_kmers{o}(:,part_beg:part_end);
  subplot('position',[0.05 0.20 0.9 0.75]);
  imagesc(sqrt(part_kmers));
  title(sprintf('Content sensor performance for order %i',o));
  subplot('position',[0.05 0.11 0.9 0.06]);
  imagesc([part_labels; part_max_idx]);
  hold on; plot([0, part_end-part_beg+2],[1.5 1.5], 'k')
  subplot('position',[0.05 0.05 0.9 0.03]);
  imagesc(log2(part_lengths)); 
end;
