function evaluate_frame()
  
results_dir = '/fml/ag-raetsch/share/projects/nGASP/content/';
%%%%% SET THIS:
results_dir = [results_dir 'RESULTS_2006-12-22_16h34_conserv/'];

% load parameters
param_file = [results_dir 'global_parameters.mat'];
load(param_file, 'PAR');

PAR

binary_hist = 1;
cons_binary_hist = 0;

for i=1:PAR.SETs.num_splits,
  test_subset = 1+mod(i+PAR.SETs.num_splits-2,PAR.SETs.num_splits);
  
  fprintf('\n');
  for o=PAR.orders, % orders of spectrum kernels
    order_idx = find(PAR.orders==o);
    
    % load frame SVMs
    file_name = [PAR.svm_file_prefix '_split' num2str(i) '_order' num2str(o) ...
                 '_binHist' num2str(binary_hist)];
    load(file_name);
    fprintf('Loaded SVM from %s\n', file_name);
    assert(SVM.PARAM.test_set_id == test_subset);      
    
    frame_SVMs{order_idx}.order = o;
    frame_SVMs{order_idx}.subset(i) = test_subset;
    frame_SVMs{order_idx}.w(i,:) = SVM.w;
    
    fprintf('ROC score for frame SVM (order %i, split %i): %.4f\n', ...
            o, i, SVM.ROC_score);
    fprintf('Prec. Recall score  (order %i, split %i):     %.4f\n', ...
            o, i, SVM.PrR_score);
    
    if PAR.use_conservation,
      % load frame SVMs
      file_name = [PAR.svm_file_prefix '_conservation_only_split' ...
                   num2str(i) '_order' num2str(o) '_binHist' num2str(cons_binary_hist)];
      tmp = load(file_name);
      SVM2 = tmp.SVM;
      clear tmp
      fprintf('Loaded SVM from %s\n', file_name);
      assert(SVM2.PARAM.test_set_id == test_subset);      

      cons_frame_SVMs{order_idx}.order = o;
      cons_frame_SVMs{order_idx}.subset(i) = test_subset;
      cons_frame_SVMs{order_idx}.w(i,:) = SVM2.w;
      
      fprintf('ROC score for frame SVM (conservation only) (order %i, split %i): %.4f\n', ...
              o, i, SVM2.ROC_score);
      fprintf('Prec. Recall score (conservation only) (order %i, split %i):      %.4f\n', ...
              o, i, SVM2.PrR_score);
      
      % combine both classifiers
      weights = 0.5:0.05:1.0;
      for j=1:length(weights),
        comb_out = SVM.out_XTE.*weights(j)+SVM2.out_XTE.*(1-weights(j));
        ROC(j) = calcrocscore(comb_out, SVM.LTE);
        PrR(j) = calcrfcscore(comb_out, SVM.LTE);
      end;
      figure; hold on;
      plot(weights, ROC, 'or'); 
      plot(weights, PrR, 'ob'); 
      title(sprintf('ROC (red) / PrR (blue) scores combined (test set %i, order %i)',...
                     test_subset, o));
    end;
  end;
end;

% save results
load(PAR.frame_block_file)
save(PAR.frame_block_file, 'frame_blocks', 'frame_SVMs');
fprintf('\nsaved frame blocks and SVMs to %s\n', PAR.frame_block_file);  

% save results for conservation frame SVMs
if PAR.use_conservation,
  fname = PAR.frame_block_file;
  if fname(end-3:end)=='.mat',
    fname = fname(1:end-4);
  end
  fname = [fname '_conserv.mat'];
  save(fname, 'frame_blocks', 'cons_frame_SVMs');
  fprintf('\nsaved frame blocks and conservation SVMs to %s\n', fname);  
end;

