function gen_annotation_tracks(fn_genes_contents, fn_genome_info, fn_training_blocks, fn_out, type_info)
%gen_annotation_tracks(fn_genes_contents, fn_genome_info, fn_training_blocks, fn_out, type_info)

VERBOSE = 1;

% load gene structures
load(fn_genes_contents, 'genes');

genome_info = init_genome(fn_genome_info);
% load training regions
load(fn_training_blocks, 'blocks');

num_types = length(fieldnames(type_info));
% initialize tracks 
tracks  = blocks;
for fn = fieldnames(tracks)'
  if ~isequal(fn{1},'start')&~isequal(fn{1},'stop')&~isequal(fn{1},'chr')
    tracks = rmfield(tracks, fn{1});
  end
end

for r=1:length(blocks),
  track = tracks(r);
  fprintf(1, 'block %i/%i\r',r,length(blocks));
  track.length = blocks(r).stop-blocks(r).start+1;
  track.type   = {zeros(num_types, track.length) ...
                      zeros(num_types, track.length)};
  track.unique_type_fw = zeros(1, track.length);
  track.unique_type_rv = zeros(1, track.length);
  track.unique_type = zeros(1, track.length);
  track_offset = track.start-1;
 
  %find genes that lie within this track
  gidx1 = strmatch(track.chr, upper({genes.chr}));
  gidx2 = find([genes.start]>=track.start);
  gidx3 = find([genes.stop]<=track.stop);
  gidx = intersect(intersect(gidx1,gidx2),gidx3);
  
  track_genes = genes(gidx);
  clear gidx gidx3 gidx2 gidx1
  
  for gene_idx = 1:length(track_genes)
    gene = track_genes(gene_idx);
    left_pad    = inf;
    right_pad   = inf;
    min_g_start = inf;
    max_g_stop  = -inf;
    for i=1:length(gene.transcripts),
      %left_pad  = min(gene.exons{i}(1,1)-1,                      left_pad); 
      left_pad  = min(gene.exons{i}(1,1)-1-gene.start+1,left_pad); 
      %right_pad = min((gene.stop-gene.start+1)-gene.exons{i}(end,2), right_pad);   
      right_pad = min(gene.stop-gene.exons{i}(end,2), right_pad);   
      %min_g_start = min(gene.original_exons{i}(1,1),   min_g_start);   
      min_g_start = min(gene.exons{i}(1,1),   min_g_start);   
      %max_g_stop  = max(gene.original_exons{i}(end,2), max_g_stop);   
      max_g_stop  = max(gene.exons{i}(end,2), max_g_stop);   
    end;

    %--------------------------------------------
    %%% traverse gene structure isoform by isoform
    %---------------------------------------------
    for i=1:length(gene.transcripts),
      % use a local track for each transcript
      local_track = zeros(num_types, gene.stop-gene.start+1);
      %start_idx = gene.start - track_offset - left_pad;
      start_idx = gene.start - track_offset;
      %end_idx   = gene.stop  - track_offset + right_pad;
      end_idx   = gene.stop  - track_offset;
  
      %assert(end_idx-start_idx == gene.stop-gene.start+left_pad+right_pad);
      assert(end_idx-start_idx == gene.stop-gene.start);
    
      % do not fill in intergenic blocks outside of the gene boundaries
      % this still allows to recover alternative transcript ends
      %local_track(type_info.intergenic,left_pad+1:end-right_pad) = 1;
      local_track(type_info.intergenic,1:end) = 1;
      assert(length(local_track)==gene.stop-gene.start+1)
      % clear intergenic track within transcript boundaries
      local_track(type_info.intergenic, gene.exons{i}(1,1)-gene.start+1:gene.exons{i}(end,2)-gene.start+1-1) = 0;
      assert(length(local_track)==gene.stop-gene.start+1)
  
      local_track(type_info.UTR_5prime, gene.exons{i}(1,1)-gene.start+1:gene.tis(i)-gene.start+1-1)    = 1;
      assert(length(local_track)==gene.stop-gene.start+1)
      local_track(type_info.UTR_3prime, gene.cdsStop(i)+1-gene.start+1:gene.exons{i}(end,2)-gene.start+1-1) = 1;
      assert(length(local_track)==gene.stop-gene.start+1)
      for e=1:size(gene.exons{i},1)-1,
        local_track(type_info.intron, gene.exons{i}(e,2)-gene.start+1+1:gene.exons{i}(e+1,1)-gene.start+1-1) = 1;
        assert(length(local_track)==gene.stop-gene.start+1)
        % clear introns in UTRs
        local_track(type_info.UTR_5prime, gene.exons{i}(e,2)-gene.start+1+1:gene.exons{i}(e+1,1)-gene.start+1-1) = 0;
        assert(length(local_track)==gene.stop-gene.start+1)
        local_track(type_info.UTR_3prime, gene.exons{i}(e,2)-gene.start+1+1:gene.exons{i}(e+1,1)-gene.start+1-1) = 0;        
        assert(length(local_track)==gene.stop-gene.start+1)
      end;
      for e=1:size(gene.cds_exons{i},1),
        local_track(type_info.CDS_exon, gene.cds_exons{i}(e,1)-gene.start+1:gene.cds_exons{i}(e,2)-gene.start+1) = 1;
        assert(length(local_track)==gene.stop-gene.start+1)
      end;      
      
      assert(all(sum(local_track)<=1));
      assert(all(sum(local_track(:,gene.exons{i}(1,1)-gene.start+1:gene.exons{i}(end,2)-gene.start+1-1))==1));
      assert(all(sum(local_track(type_info.intergenic+1:end, 1:gene.exons{i}(1,1)-gene.start+1-1))==0));
      assert(all(sum(local_track(type_info.intergenic+1:end, gene.exons{i}(end,2)-gene.start+1+1:end))==0));

      % some genestr extend betond region boundaries...
      assert(start_idx >= 1)
%      if start_idx < 1
%        % truncate left, till local_track fits into the region track
%        trunc = -1*start_idx + 1;
%        if gene.original_strands(i)=='+',
%          local_track(:,1:trunc) = [];
%        else
%          local_track(:,end-trunc+1:end) = [];
%        end;
%        start_idx = 1;      
%        fprintf('truncated gene [%i   %i] on the left by %i, strand %s.\n', ...
%                gene.start, gene.stop, trunc, gene.original_strands(i));   
%      end;
      assert(end_idx <= track.length)
%      if end_idx > track.length, 
%        % truncate right, till local_track fits into the region track
%        trunc = end_idx - track.length;
%        if gene.original_strands(i)=='+',
%          local_track(:,end-trunc+1:end) = [];
%        else
%          local_track(:,1:trunc) = [];
%        end;
%        end_idx = end_idx - trunc;    
%        fprintf('truncated gene [%i   %i] on the right by %i, strand %s.\n', ...
%              gene.start, gene.stop, trunc, gene.original_strands(i));   
%      end;
    
      % insert local_track into track    
      if gene.strand=='+',
        track.type{1}(:,start_idx:end_idx) = track.type{1}(:,start_idx:end_idx) + local_track;
      else % reverse track order on - strand
        track.type{2}(:,start_idx:end_idx) = track.type{2}(:,start_idx:end_idx) + local_track(:,end:-1:1);
      end;
      fprintf('Parsed gene %i, isoform %i     \r', gene_idx, i);
    end;%loop over transcripts

    %%%%% both strands combined  
    % combine tracks before distal intergenic blocks are filled in
    combined_track = track.type{1} + track.type{2};
    % positions that are of the same type on both strands will not be treated as ambiguous
    combined_track(combined_track > 1) = 1;
    % ambiguous positions
    ambiguous = find(sum(combined_track) > 1);
    % through away information about ambiguous intergenic sites
    tmp_intergenic = combined_track(type_info.intergenic,:);
    tmp_intergenic(ambiguous) = 0;
    % distal positions (intergenic regions outside of any gene on plus or minus strand)
    distal = find(sum(combined_track) == 0);

    % collaps tracks into a single one
    track.unique_type = track.unique_type + type_info.intergenic * tmp_intergenic;
    track.unique_type = track.unique_type + type_info.UTR_5prime * combined_track(type_info.UTR_5prime,:);
    track.unique_type = track.unique_type + type_info.UTR_3prime * combined_track(type_info.UTR_3prime,:);
    track.unique_type = track.unique_type + type_info.CDS_exon   * combined_track(type_info.CDS_exon,  :);
    track.unique_type = track.unique_type + type_info.intron     * combined_track(type_info.intron,    :);

    % flag ambiguous positions
    track.unique_type(ambiguous) = -1; 
    % fill in distal intergenic positions
    assert(all(sum(track.unique_type(:,distal))==0));
    track.unique_type(type_info.intergenic,distal) = type_info.intergenic; 

    % fill in "strand-specific" intergenic regions between genes
    distal_plus = find(sum(track.type{1}) == 0);  
    track.type{1}(type_info.intergenic,distal_plus) = 1;
    distal_minus = find(sum(track.type{2}) == 0);  
    track.type{2}(type_info.intergenic,distal_minus) = 1;
    
    %%%%% forward and reverse strand
    for s=1:2,
      % positions that are (annotated several times) of the same type will not be treated as ambiguous
      for t=1:num_types,
        idx = track.type{s}(t,:) > 1;
        track.type{s}(t,idx) = 1;
      end;    
      % ambiguous positions 
      ambiguous = find(sum(track.type{s}) > 1);
      % distal positions (intergenic regions outside of any gene on plus or minus strand)
      distal = find(sum(track.type{s}) == 0);
      if s==1,
        % through away information about ambiguous intergenic sites
        tmp_intergenic = track.type{s}(type_info.intergenic,:);
        tmp_intergenic(ambiguous) = 0;
        % collaps tracks into a single one
        track.unique_type_fw = track.unique_type_fw ...
              + type_info.intergenic * tmp_intergenic;
        track.unique_type_fw = track.unique_type_fw ...
              + type_info.UTR_5prime * track.type{s}(type_info.UTR_5prime,:);
        track.unique_type_fw = track.unique_type_fw ...
              + type_info.UTR_3prime * track.type{s}(type_info.UTR_3prime,:);
        track.unique_type_fw = track.unique_type_fw ...
              + type_info.CDS_exon   * track.type{s}(type_info.CDS_exon,  :);
        track.unique_type_fw = track.unique_type_fw ...
              + type_info.intron     * track.type{s}(type_info.intron,    :);

        % recompute ambiguous positions (this time not considering intergenic ambiguities)
        ambiguous = find(sum([tmp_intergenic; track.type{s}(type_info.intergenic+1:end,:)]) > 1);
        % flag ambiguous positions
        track.unique_type_fw(ambiguous) = -1; 
        % fill in distal intergenic positions
        assert(all(sum(track.unique_type_fw(:,distal))==0));
        track.unique_type_fw(type_info.intergenic,distal) = type_info.intergenic; 
      else
        assert(s==2);
        % through away information about ambiguous intergenic sites
        tmp_intergenic = track.type{s}(type_info.intergenic,:);
        tmp_intergenic(ambiguous) = 0;
            
        % collaps tracks into a single one
        track.unique_type_rv = track.unique_type_rv ...
            + type_info.intergenic * tmp_intergenic;
        track.unique_type_rv = track.unique_type_rv ...
            + type_info.UTR_5prime * track.type{s}(type_info.UTR_5prime,:);
        track.unique_type_rv = track.unique_type_rv ...
            + type_info.UTR_3prime * track.type{s}(type_info.UTR_3prime,:);
        track.unique_type_rv = track.unique_type_rv ...
            + type_info.CDS_exon   * track.type{s}(type_info.CDS_exon,  :);
        track.unique_type_rv = track.unique_type_rv ...
            + type_info.intron     * track.type{s}(type_info.intron,    :);

        % recompute ambiguous positions (this time not considering intergenic ambiguities)
        ambiguous = find(sum([tmp_intergenic; track.type{s}(type_info.intergenic+1:end,:)]) > 1);
        % flag ambiguous positions
        track.unique_type_rv(ambiguous) = -1; 
        % fill in distal intergenic positions
        assert(all(sum(track.unique_type_rv(:,distal))==0));
        track.unique_type_rv(type_info.intergenic,distal) = type_info.intergenic; 

      end;
    end;%loop over strands
    if VERBOSE,
      len = min(length(track.unique_type_fw),300000);
      subplot(4,1,1),
      imagesc(track.unique_type_fw(:,1:len));
      subplot(4,1,2),
      imagesc(track.unique_type_rv(:,1:len));
      subplot(4,1,3),
      imagesc(track.type{1}(:,1:len));
      subplot(4,1,4),
      imagesc(track.type{2}(:,1:len));
      pause
    end 
    track.type = [];
  end%loop over track_genes
  for field=fieldnames(track)'
    field = field{1};
    if ~isfield(tracks, field)
      tracks(1).(field)=[];
    end
  end
  tracks(r)=track;
end%loop over blocks
clear blocks

  
  
fprintf('\n\n');



training_tracks = tracks;
% save results
save(fn_out, 'training_tracks', 'type_info');
fprintf('\nsaved training tracks to %s\n', fn_out);  

