function [pos_examples neg_examples pos_block_ids neg_block_ids  pos_segments_all neg_segments_all blocks]=load_blocks_and_gen_segments(fn_blocks,fn_genome_config, which_contents, which_splits, order, use_frame)
% examples will overlay the rearmost segment boundary by at most 'order' positions


% load blocks
%assert(all(ismember({'blocks', 'split'},who('-file',fn_blocks))))
l = load(fn_blocks);
if ~isfield(l,'split')
  l.split = [l.blocks.split];
end
%blocks = load_struct(fn_blocks,'blocks');
[l.blocks.config] = deal(fn_genome_config);
genome_info = init_genome(fn_genome_config);

%select blocks of training splits
%----------------------------------------------------------------
train_block_ids = [];
if strcmp(class(l.split),'struct')&&length(fieldnames(l.split))==5 
  for sp=which_splits
    train_block_ids = [train_block_ids, l.split.(['split_' num2str(sp)])];
  end
else
  for sp=1:length(which_splits)
    train_block_ids = [train_block_ids,find(l.split==which_splits(sp))];
  end
end
blocks = l.blocks(train_block_ids);

for j=1:length(blocks)
  rm_(j)=isempty(blocks(j).truth);
end
if sum(rm_)>0
  warning('remove %i blocks because they have no truth field',sum(rm_))
end
blocks(find(rm_))=[];


pos_examples = cell(1,1e6);
neg_examples = cell(1,1e6);
num_pos = 1;
num_neg = 1;

pos_block_ids = ones(1,1e6);
neg_block_ids = ones(1,1e6);

pos_segments_all = ones(1e6,size(blocks(1).truth(1).segments,2));
neg_segments_all = ones(1e6,size(blocks(1).truth(1).segments,2));


for block_no =1:length(blocks)
  
  block = blocks(block_no);

  % prepare block for load_sequence
  %----------------------------------------------------------------
  if ~isfield(block, 'seq')
    block = load_sequence(block);
  end
  % find common/consens segments from different truths
  %----------------------------------------------------------------
  cons_segments = block.truth(1).segments;
  if ~use_frame
    for t = 2:length(block.truth)
      cons_segments = intersect(cons_segments,block.truth(t).segments,'rows');
    end
  end


  % extract lines from cons_segments 
  %----------------------------------------------------------------
  pos_segments = cons_segments(find(ismember(cons_segments(:,3),which_contents{1})),:);
  neg_segments = cons_segments(find(ismember(cons_segments(:,3),which_contents{2})),:);

 
 
  % cut out sequences corresponding to the segments;
  % segment coordinates have to be local block coordinates
  %----------------------------------------------------------------

  if ~use_frame
    pos_cell = cell(1,size(pos_segments,1));
    neg_cell = cell(1,size(neg_segments,1));
    for seg = 1:size(pos_segments,1)
      if pos_segments(seg,2)-1+order-1<=length(block.seq)
        pos_cell{seg} = block.seq(pos_segments(seg,1):pos_segments(seg,2)-1+order-1); 
      else
        pos_cell{seg} = block.seq(pos_segments(seg,1):pos_segments(seg,2)-1); % -1 because of half open intervals
      end
      %if block.strand=='-'
      %  pos_cell{seg} = pos_cell{seg}(end:-1:1); % train only on positive examples in correct orientation
      %end
    end
    for seg = 1:size(neg_segments,1)
      if neg_segments(seg,2)-1+order-1<=length(block.seq)
        neg_cell{seg} = block.seq(neg_segments(seg,1):neg_segments(seg,2)-1+order-1);
      else
        neg_cell{seg} = block.seq(neg_segments(seg,1):neg_segments(seg,2)-1); % -1 because of half open intervals 
      end
    end
  else % make examples for frame training
    neg_cell = cell(1);
    if block.strand=='+'
      pos_cell{1} = load_genomic_2(block.chr, block.strand, pos_segments(:,1)+block.start-1,pos_segments(:,2)+block.start-1-1,genome_info);
    else
      pos_cell{1} = load_genomic_2(block.chr, block.strand, -pos_segments(end:-1:1,2)+block.stop+1+1,-pos_segments(end:-1:1,1)+block.stop+1,genome_info);
    end
    pos_cell{1} = upper(pos_cell{1});
    if ~strcmp(upper(pos_cell{1}(1:3)),'ATG');
      keyboard
    end
  end
  % concat cell_arrays from different blocks 
  %----------------------------------------------------------------
  [pos_examples{num_pos:num_pos+length(pos_cell)-1}] = pos_cell{:};
  [neg_examples{num_neg:num_neg+length(neg_cell)-1}] = neg_cell{:};
  
  if nargout>2
    pos_block_ids(num_pos:num_pos+length(pos_cell)-1) = block_no*ones(1,length(pos_cell));
    neg_block_ids(num_neg:num_neg+length(neg_cell)-1) = block_no*ones(1,length(neg_cell));
    pos_segments_all(num_pos:num_pos+length(pos_cell)-1,:)  = pos_segments;
    neg_segments_all(num_neg:num_neg+length(neg_cell)-1,:)  = neg_segments;
  end
  num_pos = num_pos+length(pos_cell);
  num_neg = num_neg+length(neg_cell);


end

assert(num_pos>0)
assert(num_neg>0)

pos_examples(num_pos:end)=[];
neg_examples(num_neg:end)=[];
pos_block_ids(num_pos:end)=[];
neg_block_ids(num_neg:end)=[];
pos_segments_all(num_pos:end,:)=[];
neg_segments_all(num_neg:end,:)=[];


