function load_content_and_train(PAR)
%
% PAR is a struct with fields: 
%		SETs.which_train -- array with numbers b (1<=b<=num_partitions) defining the splits for training
%		SETs.which_contents -- cell of length two containing arrays with segment ids: 
%					array 1: positive examples, array 2: negative examples
%		method.par_ms.C -- C parameter for the SVM
%		Content_name 
%		FN.output.fn_training_blocks: blocks for training
%		FN.input.fn_genome_config: filename of the genome.config-file
%		fn_SVM: filename to save the trained SVM to
%		method.kernels{1}.order: order of kmers for feature generation


fprintf(1,'start training\n');

paths

%PAR.SETs.which_train

% PAR.which_contents: 2D cell array: 
%                                -first dimension: content names for positive contents
%                                -second dimension: content names for negative contents


% PAR.FN.....blocks
%fn_blocks = '/fml/ag-raetsch/share/projects/genefinding/C_elegans/lsl/cat-1/data/blocks_new_signals_new_labels_trans_corrected_better.mat'
fn_blocks = PAR.FN.output.fn_training_blocks;
if strcmp(PAR.Content_name,'frame0')
  use_frame=1;
else
  use_frame=0;
end

% load blocks and generate cell arrays of sequences corresponding to the segments
%----------------------------------------------------------------
fprintf(1,'start loading blocks and generating training examples\n');
[pos_examples neg_examples]=load_blocks_and_gen_segments(fn_blocks,PAR.FN.input.fn_genome_config, PAR.SETs.which_contents, PAR.SETs.which_train,PAR.method.kernels{1}.order,use_frame);


% balance length distribution of negative examples
%---------------------------------------------------------------
fprintf(1,'start fit_length_hist\n');
bins = 100;
reduce_pos=0;% throw away positive examples if necessary to achieve similar histograms
if ~use_frame
  neg_examples = fit_length_hist(pos_examples, neg_examples,bins, reduce_pos);
end

%new_neg = cell(1,length(pos_examples));
%for j=1:length(pos_examples)
%  [new_neg{j} neg_idx] = equalize_length(length(pos_examples{j}),neg_examples);
%  neg_examples(neg_idx)=[];
%  if isempty(neg_examples)
%    if j+1<=length(pos_examples)
%      new_neg(j+1:end)=[];
%    end
%    break;
%  end
%end
%neg_examples = new_neg; 

% cut off the leading 'ATG' 
%----------------------------------------------------------------
if use_frame
  for j=1:length(pos_examples)
    has_atg(j) = strcmp(pos_examples{j}(1:3),'ATG');
    len = length(pos_examples{j});
    %pos_examples{j} = pos_examples{j}(4:end-mod(len,3));
    pos_examples{j} = pos_examples{j}(4:end);
    assert(mod(length(pos_examples{j}),3)==0);
  end
  fprintf('%i of %i do not start with AGT (they will be removed from training set)\n',sum(has_atg==0),length(pos_examples))
  pos_examples(find(has_atg==0))=[];
  neg_examples = make_frame_neg_examples(pos_examples);
end
% concat examples and generate labels
%----------------------------------------------------------------
XT_cell = {pos_examples{:}, neg_examples{:}};
LT = [ones(1,length(pos_examples)), -ones(1,length(neg_examples))];
clear pos_examples neg_examples

perm = randperm(length(LT));
XT_cell = XT_cell(perm);
LT = LT(perm);

clear perm 

% generate feature vectors
%----------------------------------------------------------------
XT = zeros(4^PAR.method.kernels{1}.order,length(XT_cell));
if use_frame
  stepping = 3;
  offset = mod(3-mod(PAR.method.kernels{1}.order,3),3);
else
  stepping = 1;
  offset = 0;
end
for j=1:length(XT_cell)
  [mask hist] = compute_mers(XT_cell{j}, PAR.method.kernels{1}.order, stepping, offset);
  XT(:,j) = hist';
end
clear XT_cell
% train
%----------------------------------------------------------------
train_content(XT,LT,PAR.method.par_ms.C, PAR.fn_SVMs);



% simple test to check performance
%----------------------------------------------------------------

%l = floor(length(LT)/4*3);
%train_content(XT(:,1:l),LT(1:l),PAR.method.par_ms.C, PAR.fn_SVMs);
%outfile = '/tmp/schrott2';

%predict_content(XT(:,l+1:end),PAR.fn_SVMs,outfile);
%load(outfile, 'output')
%roc = calcrocscore(output,LT(l+1:end))
%rfc = calcrfcscore(output,LT(l+1:end))
%keyboard

