
addpath ../utils/

% construct for all parameters
PAR = [];

%%%% SET THIS
PAR.use_conservation = 0;

% number of subsets for crossvalidation
PAR.SETs.num_splits = 5;

PAR.LSL_block_file = ['/fml/ag-raetsch/share/projects/nGASP/LSL/data/' ...
                      'blocks_conserv_gz_2006-12-19.mat'];

% optionally fix date
dt = datestr(now,'yyyy-mm-dd_HHhMM');
%dt = '2006-12-22_10h05';

PAR.results_dir = ['/fml/ag-raetsch/share/projects/nGASP/content/RESULTS_' dt '/'];
if PAR.use_conservation,
  PAR.results_dir = [PAR.results_dir(1:end-1) '_conserv/'];
end;
if ~fexist(PAR.results_dir),
  mkdir(PAR.results_dir);
end;

PAR.data_split_file_prefix = [PAR.results_dir 'frame_data_split'];
PAR.svm_file_prefix = [PAR.results_dir 'frame_SVM'];
PAR.frame_block_file = [PAR.results_dir 'frame_blocks.mat'];

PAR.ignore_alt_genes    = 1;
PAR.frame_sensitive     = 1;

PAR

if ~fexist(PAR.frame_block_file),
  annotate_frames(PAR);
else
  load(PAR.frame_block_file);  
end;

JOB_INFO = [];
job_cnt = 1;
for i=1:PAR.SETs.num_splits,
  % prepare train, test, and crossvalidation data
  file_name = [PAR.data_split_file_prefix num2str(i) '.mat'];
  
  if fexist(file_name),
    load(file_name);
    fprintf('Loaded data from %s\n', file_name);
  else
    [TRAIN, TEST, test_set_id] = prepare_frame_data(PAR, i);
    save(file_name, 'TRAIN', 'TEST', 'test_set_id');
  end;

  SVM_PARAM = PAR;
  % subset_id of blocks in the test set
  SVM_PARAM.test_set_id = test_set_id;
      
  for order=PAR.orders,
    SVM_PARAM.order = order;
    
    SVM.PARAM = SVM_PARAM;
    SVM.PARAM.binary_hist = 1;
    SVM.PARAM.svm_file = [PAR.svm_file_prefix  '_split' num2str(i) ...
                        '_order' num2str(order) '_binHist' num2str(SVM.PARAM.binary_hist)];
    
    SVM.XTR = TRAIN.cds;
    SVM.LTR = -1*ones(1,length(TRAIN.cds));
    SVM.LTR(TRAIN.frame==0) = 1;
    
    SVM.XTE = TEST.cds;
    SVM.LTE = -1*ones(1,length(TEST.cds));
    SVM.LTE(TEST.frame==0) = 1;
    
    % combined parameters for script run in rproc
    RPROC_PAR.SVM             = SVM;
    % rproc parameters and options
    RPROC_MEMREQ              = 3700;
    RPROC_OPT.express         = 0;
    RPROC_OPT.immediately_bg  = 0;
    RPROC_OPT.immediately     = 0;
    RPROC_OPT.arch            = 64; % take only 64 bit nodes others
                                    % seem to have problems with shogun
    RPROC_OPT.identifier      = ['GZ_train_frame_split' num2str(i) ...
                                 '_order' num2str(order) '_'] ;
    RPROC_TIME                = 10; % mins
    
%%%%% for debugging...
%    SVM = train_content_svm_RPROC(RPROC_PAR);
%    fprintf('\n\nPerformance evaluation of frame SVM order %i, subset %i:\n', ...
%            order, i);
%    fprintf('ROC score on test set: %2.2f.\n\n', SVM.ROC_score);
%%%%%
    
    JOB_INFO{job_cnt} = rproc('train_content_svm_RPROC', ...
                              RPROC_PAR, RPROC_MEMREQ, RPROC_OPT, RPROC_TIME);
    fprintf('\nSubmitted job %i\n\n', job_cnt);
    job_cnt = job_cnt + 1;

    RPROC_PAR.SVM = [];
    clear SVM

    if PAR.use_conservation,
      SVM2.PARAM = SVM_PARAM;
      SVM2.PARAM.binary_hist = 0;
      SVM2.PARAM.svm_file = [PAR.svm_file_prefix  '_conservation_only_split' ...
                          num2str(i) '_order' num2str(order) '_binHist' num2str(SVM2.PARAM.binary_hist)];
      for j=1:length(TRAIN.conservation),
        SVM2.XTR{j} = conservation2DNA_alphabet(TRAIN.conservation{j});
      end;
      for j=1:length(TEST.conservation),
        SVM2.XTE{j} = conservation2DNA_alphabet(TEST.conservation{j});
      end;
      SVM2.LTR = -1*ones(1,length(TRAIN.conservation));
      SVM2.LTR(TRAIN.frame==0) = 1;
  
      SVM2.LTE = -1*ones(1,length(TEST.conservation));
      SVM2.LTE(TEST.frame==0) = 1;
      RPROC_PAR.SVM = SVM2;
      
%%%%% for debugging...
%      SVM2 = train_content_svm_RPROC(RPROC_PAR);
%      fprintf('\n\nPerformance evaluation of frame SVM (using only conservation) order %i, subset %i:\n', ...
%              order, i);
%      fprintf('ROC score on test set: %2.2f.\n\n', SVM2.ROC_score);
%%%%%

      JOB_INFO{job_cnt} = rproc('train_content_svm_RPROC', ...
                                RPROC_PAR, RPROC_MEMREQ, RPROC_OPT, RPROC_TIME);
      fprintf('\nSubmitted job %i\n\n', job_cnt);
      job_cnt = job_cnt + 1;

      RPROC_PAR.SVM = [];
      clear SVM2
    end;
  end;
end;

param_file = [PAR.results_dir 'global_parameters.mat'];
save(param_file, 'PAR');


%%%%% FOR THIS TO RUN THROUGH ALL JOBS HAVE TO BE FINISHED FIRST...
% attach w vector of content SVMs to blocks for label sequence learning and evaluate performance
% evaluate_frame();
