function partition_annotation_tracks(fn_blocks, fn_training_blocks, fn_tracks, fn_out, num_splits)
%partition_annotation_tracks(fn_blocks, fn_training_blocks, fn_tracks, fn_out, num_splits)

% load blocks generated for label sequence learning
load(fn_blocks, 'blocks');

% load annotation tracks
load(fn_track,'training_tracks','type_info');
% load region data
load(fn_training_blocks,'training_regions');
num_regions = length(training_regions);
assert(size(training_tracks) == [1,num_regions]);

% remove fields from blocks that are not needed for content sensors
blocks = rmfield(blocks, 'is_alt_spliced');
blocks = rmfield(blocks, 'is_alt');
blocks = rmfield(blocks, 'is_valid');
blocks = rmfield(blocks, 'anno_tss');
blocks = rmfield(blocks, 'tss');
blocks = rmfield(blocks, 'trans');
blocks = rmfield(blocks, 'tis');
blocks = rmfield(blocks, 'tis_is_valid');
blocks = rmfield(blocks, 'stop');
blocks = rmfield(blocks, 'stop_is_valid');
blocks = rmfield(blocks, 'polya');
blocks = rmfield(blocks, 'anno_cleave');
blocks = rmfield(blocks, 'cleave');
blocks = rmfield(blocks, 'splicegraph');
blocks = rmfield(blocks, 'details');
blocks = rmfield(blocks, 'content');
blocks = rmfield(blocks, 'has_invalid_don');
blocks = rmfield(blocks, 'has_invalid_acc');
blocks = rmfield(blocks, 'gene_complete');
blocks = rmfield(blocks, 'truth');

% prepare subset indices for blocks
num_blocks_total = length(blocks);
num_blocks_per_subset = ceil(num_blocks_total/num_splits)
subset_ids = [];
for i=1:num_splits,
  subset_ids = [subset_ids i*ones(1,num_blocks_per_subset)];
end;
% randomize subset ids
subset_ids = subset_ids(randperm(length(subset_ids)));
% assert that we created enough subset ids
assert(num_blocks_total<=length(subset_ids));

% process LSL blocks
first_block = 1;
for b=1:length(blocks),
  block = blocks(b);
  r = block.reg;
  region = training_regions(r);
  if block.strand=='+',
    offset = region.start - 1;
    blocks(b).unique_type = ...
        training_tracks(r).unique_type_fw(block.block_start-offset:block.block_end-offset);
    assert(length(blocks(b).unique_type) == length(block.genestr));
    
    if ~first_block,
      assert(block.block_start-1 == blocks(b-1).block_end);
    end;
    if block.block_end==region.stop,
      first_block = 1;
    else
      first_block = 0;
    end;
  else
    assert(block.strand=='-');
    assert(block.block_end<block.block_start);
    offset = region.start - 1;
    blocks(b).unique_type = ...
        training_tracks(r).unique_type_rv(block.block_end-offset:block.block_start-offset);
    assert(length(blocks(b).unique_type) == length(block.genestr));
    % reverse track order to have it match genestr
    blocks(b).unique_type = blocks(b).unique_type(end:-1:1);

    if ~first_block,
      assert(block.block_start+1 == blocks(b-1).block_end);
    end;
    if block.block_end==region.start,
      first_block = 1;
    else
      first_block = 0;
    end;
  end;
  
  % check if track is in agreement with genes
  genic_idx = zeros(1,length(blocks(b).unique_type));
  for g=1:size(block.gene_pos,1),
    genic_idx(block.gene_pos(g,1):block.gene_pos(g,2)) = 1;
  end;
%%%%% TODO these checks only work if the same annotation is used!!!
%  if ~(all(blocks(b).unique_type(find(genic_idx)) ~= PAR.track_info.intergenic)),
%    keyboard; 
%  end;
%  if ~(all(blocks(b).unique_type(find(~genic_idx)) == PAR.track_info.intergenic)),
%    keyboard; 
%  end;

  blocks(b).content.subset_id = subset_ids(b);
end;

save(fn_out, 'blocks');
fprintf('\nsaved blocks to %s\n', fn_out);  

