function SVM = train_content_svm_RPROC(RPROC_PAR)

addpath ~/svn/tools/shogun/

SVM = RPROC_PAR.SVM;
order      = SVM.PARAM.order ;    % length of strings
C          = 1 ;                  % regularization strength
cache_size = 10 ;
stepping   = 1;
if isfield(SVM.PARAM, 'frame_sensitive') && SVM.PARAM.frame_sensitive,
  stepping = 3;
end;
binary_hist = 0;
if isfield(SVM.PARAM, 'binary_hist') && SVM.PARAM.binary_hist,
  binary_hist = 1;
end;


% filter training and test set for examples that are long enough
good_idx = zeros(1,length(SVM.XTR));
for i=1:length(SVM.XTR),
  if SVM.XTR{i}>order,
    good_idx(i) = 1;
  end
end
SVM.XTR = {SVM.XTR{good_idx==1}};
SVM.LTR = SVM.LTR(good_idx==1);

good_idx = zeros(1,length(SVM.XTE));
for i=1:length(SVM.XTE),
  if SVM.XTE{i}>order,
    good_idx(i) = 1;
  end
end
SVM.XTE={SVM.XTE{good_idx==1}};
SVM.LTE=SVM.LTE(good_idx==1);


SVM.mask_XTR = zeros(4^order, length(SVM.XTR));
SVM.hist_XTR = zeros(4^order, length(SVM.XTR));
for i=1:length(SVM.XTR),
  [SVM.mask_XTR(:,i) SVM.hist_XTR(:,i)] = ...
      compute_mers(SVM.XTR{i}, order, stepping);
end

SVM.mask_XTE = zeros(4^order,length(SVM.XTE));
SVM.hist_XTE = zeros(4^order,length(SVM.XTE));
for i=1:length(SVM.XTE),
  [SVM.mask_XTE(:,i) SVM.hist_XTE(:,i)] = ...
      compute_mers(SVM.XTE{i}, order, stepping);
end


fprintf('Training SVM on %i pos. and %i neg. examples...\n', ...
        sum(SVM.LTR>0), sum(SVM.LTR<0));

sg('send_command', 'use_linadd 1');
if binary_hist,
  sg('set_features', 'TRAIN', SVM.mask_XTR) ;
else
  sg('set_features', 'TRAIN', SVM.hist_XTR) ;
end;
sg('set_labels', 'TRAIN', SVM.LTR) ;

sg('send_command', sprintf('set_kernel LINEAR REAL %i 1.0', cache_size));
sg('send_command', 'new_svm LIGHT') ;
sg('send_command', 'init_kernel TRAIN');
sg('send_command', sprintf( 'c %1.2e', C));

sg('send_command', 'svm_train' );


% determine test error
sg('send_command', 'init_kernel_optimization') ;
if binary_hist,
  sg('set_features', 'TEST', SVM.mask_XTE) ;
else
  sg('set_features', 'TEST', SVM.hist_XTE) ;
end;
sg('send_command', 'init_kernel TEST');
SVM.out_XTE = sg('svm_classify');
ROC = calcrocscore(SVM.out_XTE, SVM.LTE);
PrR = calcrfcscore(SVM.out_XTE, SVM.LTE);

% save SVM parameters
[b, alpha_tmp] = sg('get_svm');

alphas = zeros(1, length(SVM.LTE));
alphas(alpha_tmp(:,2)+1) = alpha_tmp(:,1);
idx = find(alphas);
w = 0;
for i=1:length(idx),
  [v_mask v_hist] = compute_mers(SVM.XTR{idx(i)}, order, stepping);
  if binary_hist,
    w = w + alphas(idx(i))*v_mask;
  else
    w = w + alphas(idx(i))*v_hist;
  end;
end
SVM.b           = b;
SVM.alphas_sg   = alpha_tmp; % alphas in shogun format
SVM.alphas_conv = alphas; % alphas in converted format
SVM.w           = w;

SVM.ROC_score = ROC;
SVM.PrR_score = PrR;

      
fprintf('\n\nROC score on test set: %2.2f.\n\n', SVM.ROC_score);
save(SVM.PARAM.svm_file, 'SVM');
