function blocks=add_truth2blocks(blocks,genes,model)
  
% blocks=add_truth2blocks(blocks,genes,model)
  
blocks(1).truth = struct;
blocks(1).is_ok = 1;
for b_idx=1:length(blocks),
  block = blocks(b_idx);
  isoforms = cell(length(block.gene_ids),1);
  for g_idx=1:length(block.gene_ids)
    gene_ids(g_idx) = block.gene_ids(g_idx);
    gene = genes(block.gene_ids(g_idx));
    
    fragment_idx = find([block.start]<gene.start&[block.stop]>gene.start&...
                        [block.start]<gene.stop&[block.stop]>gene.stop);
    assert(length(fragment_idx)==1);
    
    for i_idx = 1:length(gene.truth)
      iso = gene.truth{i_idx};
      if gene.strand=='+'
        iso(:,1:2) = iso(:,1:2) - block.offset(fragment_idx);
      else
        iso(:,1:2) = iso(:,2:-1:1);
        iso = iso(end:-1:1,:);
        iso(:,1:2) = -iso(:,1:2) + block.offset(fragment_idx);
      end
      isoforms{g_idx}{i_idx} = iso;
    end
  end ; %%% loop over genes in block
  
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % generate segmentation for all valid transcripts of a block %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  %%% determine all combinations of valid isoforms in current block
  iso_idx = [1:size(isoforms{1},2)]';
  num_segmentations = size(isoforms{1},2);

  for g_idx=2:size(isoforms,1),
    
    tmp = repmat(iso_idx,size(isoforms{g_idx},2),1);
    last_col = [];
    for j=1:size(isoforms{g_idx},2),
      last_col = [last_col; j*ones(size(iso_idx,1),1)];
    end
    iso_idx = [tmp last_col];
    num_segmentations = num_segmentations * size(isoforms{g_idx},2);
  end
  assert(size(iso_idx,1)==num_segmentations);
  assert(size(iso_idx,2)==size(isoforms,1));
  
  %%% build truth, one entry per valid block segmentation 
  truth =struct ;
  for j = 1:num_segmentations, 
    upstream_gene_end = 1;
  
    truth(j).segments = [];
    truth(j).genes = {};
    for g_idx = 1:size(isoforms,1),
      truth(j).genes{g_idx} = isoforms{g_idx}{iso_idx(j,g_idx)};
      has_trans = 0;
      if isfield(model.segments,'transexon')
        has_trans = truth(j).genes{g_idx}(1,3) == model.segments.transexon;
      end
      % decide whether to annotate spacer between genes as intergenic or intercistronic
      gene_start = truth(j).genes{g_idx}(1,1);
      spc_dist = gene_start - upstream_gene_end;
      %if spc_dist>max_intercistronic_length ...
      %      | ~has_trans | upstream_gene_end==1,
      if ~has_trans | upstream_gene_end==1 |~isfield(model.segments,'intercistronic'),
        truth(j).segments = [truth(j).segments; ...
                            [upstream_gene_end gene_start model.segments.intergenic]; ...
                            truth(j).genes{g_idx}];
      else
        truth(j).segments = [truth(j).segments; ...
                            [upstream_gene_end gene_start model.segments.intercistronic]; ...
                            truth(j).genes{g_idx}];
      end
      upstream_gene_end = truth(j).genes{g_idx}(end,2);
    end
    % append last intergenic segment
    block_len = abs(block.stop(fragment_idx)-block.offset(fragment_idx));
    truth(j).segments = [truth(j).segments; [upstream_gene_end block_len model.segments.intergenic]];
    % assert tight segmentation
    assert(all(truth(j).segments(2:end,1) == truth(j).segments(1:end-1,2)));
    if 1 
      [truth(j).path,truth(j).pos_idx,truth(j).pos,ok] = segmentation2path(truth(j), block, model);
    else
      ok=0;
    end
  end %%% loop over segmentations
  block.is_ok = ok;
  block.truth = truth;
  blocks(b_idx) = block;
  
  %%%%% simple printing
  if 0,
    num_segments = size(block.truth(j).segments,1);
    patch([block.truth(j).segments(:,1)';block.truth(j).segments(:,1)'; ...
           block.truth(j).segments(:,2)';block.truth(j).segments(:,2)'], ...
          [ones(1,num_segments);zeros(1,num_segments); ...
           zeros(1,num_segments);ones(1,num_segments)],...
          block.truth(j).segments(:,3)');
    axis([1 block_len, -0.2, 1.2]);
    % keyboard
  end
  %%%%%

% keyboard
  fprintf('Processed block %i\r', b_idx);
end %%% loop over blocks
fprintf('\n\n');
