function regions = build_regions_from_file(fn,fn_config,organism)

% regions = build_regions_from_file(fn,fn_config,organism)  
% 
% function reads gff file fn and returns matlab struct regions.
%   
% INPUT  - fn         gff file 
%        - fn_config  (optional)
%        - organism   (optional) organism name 
% 
% SEE ALSO: init_regions; write_regions2gff3
  
  
  
if nargin<2
  fn_config = [];
end
if nargin<3
  organism = [];
end
[tmp,num_lines] = unix(sprintf('wc -l %s',fn));  
num_lines = separate(num_lines,' ' );
num_lines = str2num(num_lines{1});

regions(num_lines) = init_regions;
if ~isempty(fn_config)
  genome_info= init_genome(fn_config);
end
fd = fopen(fn,'r') ;
cnt = 0;
while ~feof(fd),
  cnt=cnt+1;
  Line = fgetl(fd);
  elems = separate(Line);
  regions(cnt).chr = elems{1};
  if ~isempty(fn_config)
    regions(cnt).chr_num = strmatch(elems{1},genome_info.contig_names,'exact');
  end
  regions(cnt).strand = elems{7};
  regions(cnt).start = str2num(elems{4});
  regions(cnt).stop = str2num(elems{5});
  regions(cnt).id = cnt;
  regions(cnt).organism = organism;
  regions(cnt).config = fn_config;
  if isempty(elems{7})|isequal(elems{7},'.')
    regions(cnt).strand = '+';
    cnt = cnt+1;
    regions(cnt) = regions(cnt-1);
    regions(cnt).strand = '-';
  end
end

for i=1:length(regions)
  switch regions(i).strand
   case '+',
    regions(i).offset = regions(i).start - 1;
   case '-',
    regions(i).offset = regions(i).stop + 1;
  end
end

fclose(fd);