function [SEG]=correct_cleave_and_tss_pos(PAR, SEG, block, model)
% 
% if there are no predictions for the annotated cleave and tss positions then 
% set cleave and tss to the nearest predicted  position 

col4 = SEG.segments(:,4);
num_genes = length(unique(col4))-1;
assert(all(unique(col4)'==0:num_genes))


for i=1:num_genes
  gene_lines = find(col4==i);
  segments = SEG.segments(gene_lines,1:3);

  upos = unique(segments(:,1:2));
  compos = intersect(block.all_pos,upos);
  if 1
    diffpos = setdiff(upos,compos);

    % find cleave position
    %------------------------------------------------------------
    cleave_idx = find(segments(:,3)== model.segments.polya_tail,1,'last');
    if isempty(cleave_idx)
      cleave_idx = find(segments(:,3)== model.segments.utr3exon,1,'last');
    end
    cleave_pos = segments(cleave_idx,2);

    res = PAR.Signals.cleave.export_settings.resolution;
    lower_bound = segments(cleave_idx,1);
    upper_bound = SEG.segments(min(gene_lines)-1+cleave_idx+1,2);

    new_pos = find_nearest_pos(cleave_pos, block.Signals.cleave.pos,res, lower_bound,upper_bound);
    SEG.segments(min(gene_lines)-1+cleave_idx,2) = new_pos;
    SEG.segments(min(gene_lines)-1+cleave_idx+1,1) = new_pos;

    % find tss position
    %------------------------------------------------------------
    tss_idx = find(segments(:,3)== model.segments.utr5exon,1,'first');
    tss_pos = segments(tss_idx,1);

    res = PAR.Signals.tss.export_settings.resolution;
    lower_bound = SEG.segments(min(gene_lines)-1+tss_idx-1,1);
    upper_bound = segments(tss_idx,2);

    new_pos = find_nearest_pos(tss_pos, block.Signals.tss.pos,res, lower_bound,upper_bound);
    SEG.segments(min(gene_lines)-1+tss_idx,1) = new_pos;
    SEG.segments(min(gene_lines)-1+tss_idx-1,2) = new_pos;

    assert(all(ismember(diffpos,cleave_pos)|ismember(diffpos, tss_pos)));
    upos = unique(SEG.segments(gene_lines,1:2));
    assert(length(intersect(block.all_pos,upos))==length(upos));
  end
end

return

function pos=find_nearest_pos(old_pos,all_pos, resolution,lb,ub)

  idx = find(all_pos<old_pos,1,'last');
  if old_pos-all_pos(idx)<all_pos(idx+1)-old_pos && all_pos(idx)>lb && all_pos(idx)<ub||((all_pos(idx+1)<=lb || all_pos(idx+1)>=ub) &&all_pos(idx)>lb && all_pos(idx)<ub)
    pos = all_pos(idx);
    assert(old_pos-pos<2*resolution);
  elseif all_pos(idx+1)>lb && all_pos(idx+1)<ub
    pos = all_pos(idx+1);
    assert(pos-old_pos<2*resolution);
  else
    error('cannot find correct position for tss/cleave');
  end
return
