function idx = filter_high_expressed(PAR,blocks,genes)
% filter out blocks for creation of training sets
% blocks must have fields 'probe_local_pos' 
% with the probe positions in block coordinates and 'intensities'
% containing the tiling array intensities
% 
% first filter step: 
% keep only blocks with all genes annotated and EST-evidence
% for all genes (supporting the annotation exactly) 
%
% second filter step:
% find blocks with exon-intensity mean among top 20 percent 
% according to tiling array data in PAR.FN.input_lsl.fn_tiling_data

assert(isfield(blocks,'probe_local_pos'))
assert(isfield(blocks,'intensities'))

% step 1: 
idx = filter_high_conf([],blocks,genes);

% step 2:
percentile_ids = expression_bins_for_blocks(blocks, genes,10);

idx = unique([idx percentile_ids{1:8}]);
