function [blocks genes split] = gen_lsl_blocks(genes, blocks, fn_genome_config, model, fn_split,run_locally)


% find genes that are invalid or not complete
%------------------------------------------------------------
fprintf(1,'check genes for completeness\n')
which_checks.exons_sorted = 1;
which_checks.intron_length = 1;
which_checks.exon_length = 1;
which_checks.splicesites = 1;
which_checks.orf = 1;
which_checks.gene_length = 1;
which_checks.graph = 1;
which_checks.transacc = 0;
which_checks.complete = 1;

genome_info = init_genome(fn_genome_config);

if ~run_locally
  chunks = 20;
  parts = ceil((length(genes)/chunks)*(0:chunks));
  for j=2:length(parts)
    outfiles{j-1}= ['~' tempname];
    Data.outfile = outfiles{j-1};
    Data.genome_info = genome_info;
    Data.which_checks = which_checks;
    Data.genes = genes(parts(j-1)+1:parts(j));
    jobinfo(j-1)=rproc('check_genes_rproc_wrapper',Data,1000,10*60);
  end
  jobinfo = rproc_wait(jobinfo,30,1,0);
  checked_genes = [];
  for j=1:length(jobinfo)
    l = load(outfiles{j},'genes');
    checked_genes=[checked_genes l.genes];
  end  
  assert(length(genes)==length(checked_genes))
  genes = checked_genes;
else
  genes = check_genes(genes,genome_info, which_checks);
end

valid_idx = find([genes.is_complete]&[genes.is_valid]);
fprintf('\n\n %i of %i genes are valid and have at least one complete transcript \n', length(valid_idx),length(genes))

%assert(length(valid_idx)>0)

clear checked_genes valid_idx j chunks parts

% remove blocks that contain no or incomplete genes
%--------------------------------------------------------------
block_rm_idx = [];
for j=1:length(blocks)
  gene_ids = blocks(j).gene_ids;
  if isempty(gene_ids)
    block_rm_idx = [block_rm_idx j];
    continue;
  end
  for id=gene_ids
    if ~genes(id).is_complete
      block_rm_idx = [block_rm_idx j];
      break;
    end
  end
end

fprintf('remove %i (%i) blocks containing incomplete genes\n',length(block_rm_idx),length(blocks))
blocks(block_rm_idx)=[];
clear curr_genes curr_blocks s_blocks_idx s_genes_idx c_blocks_idx c_genes_idx s j k g block_rm_idx gene_idxs

% filter out blocks that contain overlapping genes
%------------------------------------------------------------
fprintf('filter out blocks that contain overlapping genes\n')
rm_idx = [];
for j=1:length(blocks)
  if length(blocks(j).gene_ids) <= 1, continue; end
  starts = [genes(blocks(j).gene_ids).start];
  stops = [genes(blocks(j).gene_ids).stop];
  [starts sort_idx] = sort(starts);
  stops = stops(sort_idx);
  for k=2:length(starts)
    if ~(starts(k)>stops(k-1))
      rm_idx = [rm_idx j];
      break; 
    end
  end
end
fprintf('remove %i (%i) blocks containing overlapping genes\n',length(rm_idx),length(blocks))
blocks(rm_idx) = [];
clear rm_idx sort_idx starts stops

% add segmentation to genes
%------------------------------------------------------------
genes = gen_gene_segmentation(genes,model);


% add segmentation to blocks
%-------------------------------------------------------------
[blocks.config] = deal(fn_genome_config);
blocks = load_sequence(blocks);
blocks = gen_block_segmentation(genes,blocks,model);

% get split from training blocks
%-------------------------------------------------------------
split = [];
if ~isfield(blocks,'split')||isempty(blocks(1).split)
  l = load(fn_split,'split');
  split = -1*ones(1,length(blocks));
  for j=1:length(blocks), 
    for k=1:length(fieldnames(l.split)), 
      idx = find(l.split.(['split_' num2str(k)])==blocks(j).id); 
      if ~isempty(idx), 
        split(j)=k;
        blocks(j).split=k; 
      end, 
    end, 
  end
  assert(isequal(unique(split), 1:5))
  clear l j k idx
end
% remove blocks with empty truth 
%-------------------------------------------------------------
fprintf(1,'filter out blocks with empty truth\n')
rm_idx = [];
val_idx = [];
for b = 1:length(blocks)
  if isempty(blocks(b).truth)
    rm_idx = [rm_idx b];
    continue;
  end
end
blocks(rm_idx) = [];
fprintf('remove %i (%i) blocks where generation of truth has failed\n',length(rm_idx), length(blocks))


