function genes = gen_truth(genes, model)

% genes = gen_truth(genes, model)
% generates all possible true segmentations for the genes.
%   
% INPUT genes
%       model
%      
% 
% calls functions
%       -get_isoforms
%       -append_utr5p_segment
%       -append_cds_exon_segment  
%       -append_utr3p_segment
%       -find_open_frames
%         

  

genes(1).truth = struct;

for g_idx = 1:length(genes)
  gene = genes(g_idx);
  exons = gene.exons;
  % if ~gene.is_complete|~gene.is_valid
  %   warning(sprintf('incomplete gene'));
  %   continue
  % end
  cds_exons = gene.cds_exons;
  utr5_exons = gene.utr5_exons;
  utr3_exons = gene.utr3_exons;
  assert(length(cds_exons)==length(utr5_exons))
  assert(length(cds_exons)==length(utr3_exons))
  for j=1:length(cds_exons) %%% loop over isoforms
    clear cds_segments utr5_segments utr3_segments
    cds_exons{j}(:,3) = model.segments.cds_exon;
    cds_segments(1,:) = cds_exons{j}(1,:);
    for k=2:size(cds_exons{j},1)
      cds_segments(2*(k-1),:) =  [cds_exons{j}(k-1,2) cds_exons{j}(k,1) model.segments.intron];
      cds_segments(2*(k-1)+1,:) = cds_exons{j}(k,:);
    end
    if isfield(model.segments,'utr5exon')
      utr5_exons{j}(:,3) = model.segments.utr5exon;
      if isfield(model.signals,'trans')
        utr5_segments{j}(1,:) = utr5_exons{j}(1,:);
        for k=2:size(utr5_exons{j},1)
          utr5_segments(2*(k-1),:) =  [utr5_exons{j}(k-1,2) utr5_exons{j}(k,1) model.segments.intron];
          utr5_segments(2*(k-1)+1,:) = utr5_exons{j}(k,:);
        end
      else
        %%% TRANS STILL MISSING
        %keyboard
      end 
    end
    if isfield(model.segments,'utr3exon')
      utr3_exons{j}(:,3) = model.segments.utr3exon;
      if isfield(model.signals,'polya')
        utr3_segments(1,:) = utr3_exons{j}(1,:);
        for k=2:size(utr3_exons{j},1)
          utr3_segments(2*(k-1),:) =  [utr3_exons{j}(k-1,2) utr3_exons{j}(k,1) model.segments.intron];
          utr3_segments(2*(k-1)+1,:) = utr3_exons{j}(k,:);
        end
      else
        %%% POLY A STILL MISSING
        keyboard
      end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % generate segmentation for all valid transcripts of a gene %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if exist('utr5_segments')&exist('utr3_segments')
      isoform = [utr5_segments, cds_segments, utr3_segments];
    else
      isoform = [cds_segments];
    end
    % assert a tight segmentation
    assert(all(isoform(2:end, 1) == isoform(1:end-1, 2)));
    
    % assert that there is an ORF
    if isfield(model.segments,'utr5exon')&isfield(model.segments,'utr3exon')
      assert(sum(isoform(:,3)==model.segments.cds_exon ...
                 | isoform(:,3)==model.segments.utr5exon ...
                 | isoform(:,3)==model.segments.utr3exon)<=size(exons{j},1)+2);
    else
      assert(sum(isoform(:,3)==model.segments.cds_exon)<=size(exons{j},1));
    end
    % cds_idx = find(isoform(:,3)==model.segments.cds_exon);
    % assert(mod(sum(isoform(cds_idx,2)-isoform(cds_idx)),3)==0)
    % str = [];
    % for k=1:length(cds_idx),
    %   str = [str block.genestr(isoform(cds_idx(k),1):isoform(cds_idx(k),2)-1)];
    % end
    % f = find_open_frames(lower(str),1); 
    % assert(f(1));
    % if ~f(1)
    %   warning('no orf found')
    % end  
    isoforms{j} = isoform;
  end %%% loop over isoforms 
  gene.truth = isoforms;
  genes(g_idx)=gene;
end %%% loop over genes

  
