function [good_isoforms_,tis,stop] = get_isoforms(gene,model) ;
% isoforms = get_isoforms(gene,model) ;
%
% Generates all possible isoforms from the splicegraph,
% and locate all the start and stop codons.





% find all the leftmost and rightmost exons (transcription starts and stops)
tstart = [];
vertices = gene.splicegraph{1};
edges = gene.splicegraph{2};

if gene.strand=='-'
  edges= edges(end:-1:1,end:-1:1) ; 
  vertices = vertices(end:-1:1,end:-1:1) ;
end ;

for ixv = 1:size(vertices,2)
  if sum(edges(ixv,1:ixv-1)) == 0
    tstart(end+1) = ixv;
  end
end

% find all transcription ends
tstop = [];
for ixv = 1:size(vertices,2)
  if sum(edges(ixv+1:end,ixv)) == 0
    tstop(end+1) = ixv;
  end
end

% do a depth first search through the graph, generating a list of isoforms
% - for each transcription start, do depth first search.
global isoforms
isoforms = {};

for ixt = tstart
  dfs_visit(triu(edges),ixt,[]);
end

% for each path, find all the start and stop codons.
% and get the longest in frame coding sequence

tis=[]; stop=[]; len=[] ; utr_len=[] ; good_isoforms=[] ;
for ixf = 1:length(isoforms)
  for i = 1:length(gene.tis)
    for j = 1:length(gene.stop)
      exons = vertices(:,isoforms{ixf})';
      
      if gene.strand=='+'
        tis_temp = gene.tis(i);
        stop_temp = gene.stop(i);
      else
        tis_temp = gene.stop(j);
        stop_temp = gene.tis(j);
      end
      idx = find(exons(:,1)<=tis_temp&exons(:,2)>= tis_temp);
      if isempty(idx), continue ; end ;
      utr_len_ = sum(exons(1:idx-1,2)-exons(1:idx-1,1)) + tis_temp - exons(idx,1)  ;
      exons(1:idx-1,:) = [];
      exons(1,1) = tis_temp;
      idx = find(exons(:,1)<=stop_temp & exons(:,2)>=stop_temp);
      if isempty(idx), continue ; end ;
      utr_len_ = utr_len_ + sum(exons(idx+1:end,2)-exons(idx+1:end,1)) + exons(idx,2) - stop_temp   ;
      exons(idx+1:end,:) = [];
      exons(end,2) = stop_temp+3;
      if ~mod(sum(exons(:,2)-exons(:,1)),3)
        str = [];
        for k=1:size(exons,1)
          str = [str gene.genestr(exons(k,1):exons(k,2)-1)];
        end
        f=find_open_frames(lower(str),1) ;
        if f(1)==1,
          tis(end+1) = gene.tis(i) ;
          stop(end+1) = gene.stop(j) ;
          len(end+1) = length(str) ;
          utr_len(end+1) = utr_len_ ;
          good_isoforms{end+1}=isoforms{ixf} ;
        end ;
      end
    end
  end
end 

[tmp idx] = sort(len+1e-5*utr_len,'descend');
good_isoforms_={} ;
for i=1:length(idx)
  good_isoforms_{i} = vertices(:,good_isoforms{idx(i)})';
end ;
tis=tis(idx);
stop =stop(idx);

