USE_TEST_BLOCKS = 0

if USE_TEST_BLOCKS,
  fn_in = '/fml/ag-raetsch/share/projects/nGASP/LSL/cat-1/test_data/blocks_2006-12-30-final.mat'
else
%  fn_in = '/fml/ag-raetsch/share/projects/nGASP/LSL/cat-1/data/blocks_2006-12-30-final.mat'
  fn_in = '/fml/ag-raetsch/share/projects/genefinding/C_elegans/lsl/cat-1/data/blocks_old_pre.mat'
%  fn_in = '/fml/ag-raetsch/share/projects/genefinding/C_elegans/lsl/cat-1/data/blocks_old.mat'
end
load(fn_in);
nGASP_PAR = PAR
if ~exist('blocks', 'var') & exist('DNA_blocks', 'var'),
  blocks = DNA_blocks;
  clear DNA_blocks
end
clear PAR

idx = find(fn_in=='/');
if idx(end)==length(fn_in),
  idx = idx(end-1);
else
  idx = idx(end);  
end
if USE_TEST_BLOCKS,
  fn_out = ['test_blocks/' fn_in(idx+1:end)]
else
%  fn_out = ['train_blocks/' fn_in(idx+1:end)]
  idx = strfind(fn_in, '_old');
  fn_out = [fn_in(1:idx-1) fn_in(idx+4:end)];
  fn_out = fn_out(1:strfind(fn_out, '.mat')-1);
  fn_out = [fn_out '_' datestr(now,'yyyy-mm-dd') '.mat'];
end

addpath ~/svn/projects/genefinding/sensors
paths 
%addpath ~/svn/projects/genefinding/sensors
%addpath ~/svn/projects/genefinding/sensors/configs
%addpath ~/svn/projects/genefinding/sensors/template_files
%addpath ~/svn/projects/genefinding/utils/
addpath /fml/ag-raetsch/share/software/matlab_tools/genomes/

regions_fn = '/fml/ag-raetsch/share/projects/genefinding/C_elegans/genome_data/trivial_regions.mat';
load(regions_fn);
fn_genome_config = PAR.FN.input.fn_genome_config
genome_info = init_genome(fn_genome_config);
clear PAR

% paramter objects, one for each signal
PARAMS = {};
% field names of signals in nGASP data structures
nGASP_names = {};
% hand-coded offsets to shift signals in place for nGASP
signal_offsets = zeros(2,0);

PARAMS{end+1} = C_elegans_acc_SVM_WDS; % should be C_elegans_acc_SVM_WDS
nGASP_names{end+1} = 'acc';
signal_offsets(:,end+1) = [0; 0];
PARAMS{end+1} = C_elegans_don_SVM_WDS; % should be C_elegans_don_SVM_WDS
nGASP_names{end+1} = 'don';
signal_offsets(:,end+1) = [1; 1];
PARAMS{end+1} = C_elegans_tss_SVM_SPWDSP; % should be C_elegans_tss_SVM_COMBINED
nGASP_names{end+1} = 'tss';
signal_offsets(:,end+1) = [0; 0];
PARAMS{end+1} = C_elegans_tis_SVM_COMBINED;
nGASP_names{end+1} = 'tis';
signal_offsets(:,end+1) = [0; 0];
PARAMS{end+1} = C_elegans_cdsStop_SVM_COMBINED;
nGASP_names{end+1} = 'stop';
signal_offsets(:,end+1) = [-2; -2];
PARAMS{end+1} = C_elegans_polya_SVM_COMBINED;
nGASP_names{end+1} = 'polya';
signal_offsets(:,end+1) = [0; -5];
PARAMS{end+1} = C_elegans_cleave_SVM_SPWDSP; % C_elegans_cleave_SVM_COMBINED
nGASP_names{end+1} = 'cleave';
signal_offsets(:,end+1) = [0; 0];
PARAMS{end+1} = C_elegans_transacc_SVM_WDS; 
nGASP_names{end+1} = 'trans';
signal_offsets(:,end+1) = [0; 0];

assert(length(PARAMS) == length(nGASP_names));
assert(length(PARAMS) == size(signal_offsets,2));

% assert that later acc will be processed BEFORE transacc
assert(strmatch('acc',nGASP_names,'exact') < strmatch('trans',nGASP_names,'exact'));

signal_names = {};
base_dirs    = {};
for i=1:length(PARAMS),
  signal_names{i}  = PARAMS{i}.Signal_name;
  tmp = getfield(PARAMS{i}.FN.output_sig, signal_names{i});
  base_dirs{i} = tmp.fn_pred;
  
  assert(isequal(fn_genome_config, ...
                 PARAMS{i}.FN.input.fn_genome_config));
end
assert(length(signal_names) == length(nGASP_names));
clear trivial_regions PARAMS SAVE_BINARY tmp i 

fields = {'output', 'Conf', 'Conf_cum'};

rm_block_idx = [];
tic
for b=1:length(blocks),
  fprintf('Processing block %i (of %i)...\n', b, length(blocks));
  block = blocks(b);

  tmp_region.chr_num = strmatch(block.chr, genome_info.contig_names, 'exact');
  tmp_region.strand  = block.strand;
  switch block.strand,
   case '+',
    tmp_region.start = block.block_start;
    tmp_region.stop  = block.block_end;
   case '-',
    tmp_region.start = block.block_end;
    tmp_region.stop  = block.block_start;
  end
  tmp_region.config = fn_genome_config;
  tmp_region = load_sequence(tmp_region);
  
  if ~isequal(block.genestr, tmp_region.seq),
    % this can happen due to different genome versions used for
    % nGASP and whole-genome signal prediction
%    warning('  sequence differences due to different genome versions');

    fprintf('  Attempting sequence shifts for block %i (r %i, chr %s, %s, [%i, %i])...\n', ...
            b, block.reg, block.chr, block.strand, block.block_start, block.block_end);
    num_X = 10;
    old_seq = [repmat('X', 1, num_X) tmp_region.seq repmat('X', 1, num_X)];
    len = length(block.genestr);
    success = 0;
    for shift=-5:1:5,
      new_seq = old_seq(num_X+1-shift:len+num_X-shift);
      assert(length(new_seq) == len);
      idx = max(1,1+shift):len+min(0,shift);
      if isequal(new_seq(idx), block.genestr(idx)),
        fprintf('  shifting tmp_region.seq by %i\n', shift);
        success = 1;
        break
      end
    end
    if ~success,
      fprintf('  NO APPROPRIATE SHIFT FOUND!\n removing block...\n');
      rm_block_idx = [rm_block_idx b];
      
      continue
    end
  else 
    shift = 0;
  end
  
  for s=1:length(signal_names),
    fprintf('  Processing signal %s...\n', signal_names{s});
    tmp_region = retrieve_signals(tmp_region, base_dirs{s}, signal_names{s}, fields);
  
    tmp = getfield(tmp_region.Signals, signal_names{s});
    switch block.strand,
     case '+',
      pos_region = tmp.pos + signal_offsets(1,s) + shift;
     case '-',
      pos_region = tmp.pos + signal_offsets(2,s) + shift;
    end
    % map signal name to in nGASP name 
    % (essentially needed for cds_stop/stop)
    pos_block = getfield(blocks(b).details.pos, nGASP_names{s});
 
    map_idx = find(ismember(pos_region, pos_block));
    tmp = getfield(tmp_region.Signals, signal_names{s});
    
    switch signal_names{s},
     case {'tss', 'cleave'},
      % transformation to sparse signals: take max around nGASP positions
      splits = round((pos_block(1:end-1) + pos_block(2:end)) / 2);
      splits = [max(1, pos_block(1)-35); splits];
      len = length(tmp.Conf_cum);
      splits = [splits; min(len, pos_block(end)+35)]; 
      
      % always use Conf_cum as score for label sequence learning
      new_score = nan(length(splits)-1,1);
      min_score = min(tmp.Conf_cum);
      for i=1:length(splits)-1,
        idx = find(tmp.pos>splits(i) & tmp.pos<=splits(i+1));
        % if no whole-genome predictions are found, fill in most neg. score
        if isempty(idx),
          new_score(i) = min_score;
        else
          new_score(i) = max(tmp.Conf_cum(idx));
        end
        if mod(i,100)==0,
          fprintf('    %2.1f%%\r',100*i/length(splits));
        end
      end
      fprintf('    %2.1f%%\n',100*i/length(splits));
      assert(~any(isnan(new_score)));

%      figure
%      hold on
%      plot(map_idx(1:100), new_score(1:100), '-db');
%      plot(tmp.Conf_cum(1:splits(101)), 'r');
%      plot(splits(1:101), repmat(0.85,101,1), 'vk');
%      keyboard
     case 'polya',
      % insert additional positions (for sixmers not originally
      % considered for nGASP)

%      figure
%      plot(blocks(b).details.score.polya, tmp.Conf_cum(map_idx), '.k');
%      title(signal_names{s});

      map_idx = map_idx(1):map_idx(end);
      new_pos = pos_region(map_idx);
      idx2 = ismember(new_pos, blocks(b).details.pos.polya);
      new_label = -ones(size(new_pos));
      new_label(idx2) = blocks(b).details.label.polya;
      blocks(b).details.pos.polya = new_pos;
      blocks(b).details.label.polya = new_label;
      assert(all(size(blocks(b).details.label.polya) ...
                 == size(blocks(b).details.pos.polya)));
      
      % always use Conf_cum as score for label sequence learning
      new_score = tmp.Conf_cum(map_idx); 
      assert(all(size(new_score) == size(blocks(b).details.pos.polya)));
     
      % assert that there is a new score for every pos in nGASP block
      assert(isempty(setdiff(pos_block, pos_region(map_idx))));
     otherwise,
      % assert that only positions at the very start and end of the
      % region are different and everything matches up in the middle
      assert(map_idx(end)-map_idx(1)+1 == length(map_idx));

      % always use Conf_cum as score for label sequence learning
      new_score = tmp.Conf_cum(map_idx); 
    
      % assert that there is a new score for every pos in nGASP block
      assert(isempty(setdiff(pos_block, pos_region)));
    end
    
    switch signal_names{s}, 
     case 'acc',
      % transacc scores are set to acc scores, nGASP field trans
      % has the specific trans-splice site scores
      assert(isequal(blocks(b).details.score.acc, ...
                     blocks(b).details.score.transacc));
      blocks(b).details.score.transacc = new_score;
      % also provide raw SVM outputs
      new_output = tmp.output(map_idx); 
      blocks(b).details.output = setfield(blocks(b).details.output, ...
                                          nGASP_names{s}, new_output);
     case 'don',
      % also provide raw SVM outputs
      new_output = tmp.output(map_idx); 
      blocks(b).details.output = setfield(blocks(b).details.output, ...
                                       nGASP_names{s}, new_output);
     % needed or not ???
     case 'transacc',
      new_score = new_score.*blocks(b).details.score.acc;
    end

%    tr_idx = strmatch('trans', {nGASP_PAR.model.states.name});
    if 0%isequal(signal_names{s}, 'polya') %any(blocks(b).truth(1).path==tr_idx),
      figure
      hold on
      plot_idx = 1:length(pos_block);%1:(min(2000, length(pos_block)));
      plot(pos_block(plot_idx), new_score(plot_idx), '.-b');
      tmp = getfield(blocks(b).details.score, nGASP_names{s});
      plot(pos_block(plot_idx), tmp(plot_idx), '.-r');
      if isfield(blocks(b), 'truth'),
        for j=1:length(blocks(b).truth),
          plot(blocks(b).truth(j).pos, ...
               blocks(b).truth(j).path./25, '.-g');
        end
        for j=1:length(nGASP_PAR.model.states),
          state = nGASP_PAR.model.states(j);
          text(max(blocks(b).truth(1).pos)+1000, j./25, state.name);
        end
      end
      title(signal_names{s});
      grid on
      if ~isequal(signal_names{s}, 'polya'),
        figure
        plot(tmp, new_score, '.k');
        title(signal_names{s});
      end
      pause(1)
    end
    
    blocks(b).details.score = setfield(blocks(b).details.score, ...
                                       nGASP_names{s}, new_score);
    
    clear tmp     
  end
%  keyboard
  clear tmp_region
  
  fprintf('    %i (%.2f sec)\r', b, toc);
end
fprintf('\nRemoved %i blocks.\n', length(rm_block_idx));
blocks(rm_block_idx) = [];
PAR = nGASP_PAR;
save(fn_out, 'blocks', 'PAR');
fprintf('saved new blocks to %s\n', fn_out);

