function signal_masks=gen_signal_masks(genome_info, tol)
% signal_masks=gen_signal_masks(genome_info)

%genome_info = init_genome('genome.config') ;
  
fn = sprintf('%s/signal_masks_withends.mat', genome_info.basedir)  
if fexist(fn) 
  fprintf('signal mask alrady exists\n') 
  signal_masks= load_struct(sprintf('%s/signal_masks_withends.mat', genome_info.basedir),  'signal_masks') ;
  return
end
genes=load_struct(sprintf('%s/confirmed_sequences.mat', genome_info.basedir), 'genes') ;

signal_masks=[] ;
fnames = {'no_ss', 'likely_acc', 'likely_don', 'sure_acc', 'sure_don', 'no_tis', 'no_stop', 'likely_tis', 'likely_stop','no_tss','no_cleave','likely_tss', 'likely_cleave','good_tss', 'good_cleave'} ;
for i=1:length(genome_info.flat_fnames)
  d=dir(genome_info.flat_fnames{i}) ;
  for j=1:2,
    for k=1:length(fnames)
      signal_masks(j,i).(fnames{k}) = logical(zeros(1,d(1).bytes)) ;
    end ;
  end ;
end ;


if nargin<3
  tol.no_ss = 5 ; %20
  tol.likely_ss = 5 ;
  
  tol.no_tis = 20 ;
  tol.likely_tis = 5; % 5 
  tol.good_tis = 5 ; % 3

  tol.no_stop = 20 ;
  tol.likely_stop = 5 ; 
  tol.good_stop = 5 ;%3

  tol.no_tss = 20 ;
  tol.likely_tss = 1000; 
  tol.good_tss = 5 ;

  tol.no_cleave = 20 ;
  tol.likely_cleave = 1000 ; 
  tol.good_cleave = 5 ;

end ;


num_ss=0 ;
for i=1:length(genes)
  fprintf('%i (%i)\r',i,length(genes))
  donors = [] ;
  acceptors = [] ;

  chr_num=find(ismember(genome_info.contig_names, genes(i).chr)) ; assert(length(chr_num)==1) ;
  signal_mask = signal_masks(1+(genes(i).strands(1)=='-'), chr_num) ;
  %if chr_num~=1 || genes(i).strands(1)~='+', continue ; end ;

  for k=1:length(genes(i).transcripts)
    
    % no tss and cleave within alignment
    signal_mask.no_tss(genes(i).exons{k}(1,1)+tol.no_tss:genes(i).exons{k}(end,2)-tol.no_tss) = 1 ;
    signal_mask.no_cleave(genes(i).exons{k}(1,1)+tol.no_cleave:genes(i).exons{k}(end,2)-tol.no_cleave) = 1 ;

    
    
    %% ---- START WITH PROTEIN ALIGNMENTS
    
    if isequal(genes(i).transcripts{k}(1:3), 'PR-')
      % no splice site in exons
      for q=1:size(genes(i).exons{k},1)
        signal_mask.no_ss(genes(i).exons{k}(q,1)+tol.no_ss:genes(i).exons{k}(q,2)-tol.no_ss) = 1 ;
      end ;
      for q=1:size(genes(i).exons{k},1)-1
        num_ss=num_ss+1 ;
        % no splice site, tis, stop,tss, cleave in introns
        signal_mask.no_ss(genes(i).exons{k}(q,2)+tol.no_ss:genes(i).exons{k}(q+1,1)-tol.no_ss) = 1 ;
        signal_mask.no_tis(genes(i).exons{k}(q,2):genes(i).exons{k}(q+1,1)) = 1 ;
        signal_mask.no_stop(genes(i).exons{k}(q,2):genes(i).exons{k}(q+1,1)) = 1 ;
        
        % likely splice sites near the boundaries
        
        if genes(i).strands(1)=='+',
          signal_mask.likely_don(genes(i).exons{k}(q,2)-tol.likely_ss:genes(i).exons{k}(q,2)+tol.likely_ss) = 1 ;
          signal_mask.likely_acc(genes(i).exons{k}(q+1,1)-tol.likely_ss:genes(i).exons{k}(q+1,1)+tol.likely_ss) = 1 ;
        else
          signal_mask.likely_acc(genes(i).exons{k}(q,2)-tol.likely_ss:genes(i).exons{k}(q,2)+tol.likely_ss) = 1 ;
          signal_mask.likely_don(genes(i).exons{k}(q+1,1)-tol.likely_ss:genes(i).exons{k}(q+1,1)+tol.likely_ss) = 1 ;
        end ;
      end ;
      % try to identify good protein sequence match boundaries
      if genes(i).terminal_info(1,k)>=0 && genes(i).terminal_info(1,k)<=tol.good_tis, % then it is a good end
        if genes(i).strands(1)=='+',
          signal_mask.likely_tis(genes(i).exons{k}(1,1)-tol.likely_tis:genes(i).exons{k}(1,1)+tol.likely_tis) = 1 ;
          signal_mask.no_tis(genes(i).exons{k}(1,1)+tol.no_tis:genes(i).exons{k}(end,2)) = 1 ;
          signal_mask.likely_tss(max(1,genes(i).exons{k}(1,1)-tol.likely_tss):genes(i).exons{k}(1,1)) = 1 ;
        else
          signal_mask.likely_stop(max(1,genes(i).exons{k}(1,1)-tol.likely_stop):genes(i).exons{k}(1,1)+tol.likely_stop) = 1 ;
          signal_mask.no_stop(genes(i).exons{k}(1,1)+tol.no_stop:genes(i).exons{k}(end,2)) = 1 ;
          signal_mask.likely_cleave(max(1,genes(i).exons{k}(1,1)-tol.likely_tss):genes(i).exons{k}(1,1)) = 1 ;
        end 
      end ;
      if genes(i).terminal_info(2,k)>=0 && genes(i).terminal_info(2,k)<=tol.good_stop, % then it is a good end
        if genes(i).strands(1)=='+',
          signal_mask.likely_stop(genes(i).exons{k}(end,2)-tol.likely_stop:genes(i).exons{k}(end,2)+tol.likely_stop) = 1 ;
          signal_mask.no_stop(genes(i).exons{k}(1,1):genes(i).exons{k}(end,2)-tol.no_stop) = 1 ;
          signal_mask.likely_cleave(genes(i).exons{k}(end,2):min(length(signal_mask.likely_cleave),genes(i).exons{k}(end,2)+tol.likely_cleave)) = 1 ;
        else
          signal_mask.likely_tis(genes(i).exons{k}(end,2)-tol.likely_tis:genes(i).exons{k}(end,2)+tol.likely_tis) = 1 ;
          signal_mask.no_tis(genes(i).exons{k}(1,1):genes(i).exons{k}(end,2)-tol.no_tis) = 1 ;
          signal_mask.likely_tss(genes(i).exons{k}(end,2):min(length(signal_mask.likely_tss),genes(i).exons{k}(end,2)+tol.likely_tss)) = 1 ;
        end 
      end ;
    end ;
  end ;
  signal_mask.likely_tss = signal_mask.likely_tss&~signal_mask.no_tss;
  signal_mask.likely_cleave = signal_mask.likely_cleave&~signal_mask.no_cleave;
  
  %% ---- START WITH EST / cDNA ALIGNMENTS
  
  for k=1:length(genes(i).transcripts)
    if isequal(genes(i).transcripts{k}(1:4), 'EST-') || isequal(genes(i).transcripts{k}(1:3), 'GR-')
      
      %%good TSS AND CLEAVE if tis found downstream/ stop found upstream 
      tmp=size(genes(i).exons{k},1);
      if genes(i).strands(1)=='+' 
        signal_mask.good_tss(genes(i).exons{k}(1,1)-tol.good_tss:genes(i).exons{k}(1,1)+tol.good_tss)= ...
            signal_mask.likely_tss(genes(i).exons{k}(1,1)-tol.good_tss:genes(i).exons{k}(1,1)+tol.good_tss);
        signal_mask.good_cleave(genes(i).exons{k}(tmp,2)-tol.good_cleave:genes(i).exons{k}(tmp,2)+tol.good_cleave)= ...
            signal_mask.likely_cleave(genes(i).exons{k}(tmp,2)-tol.good_cleave:genes(i).exons{k}(tmp,2)+tol.good_cleave);
       
      else
        signal_mask.good_cleave(genes(i).exons{k}(1,1)-tol.good_cleave:genes(i).exons{k}(1,1)+tol.good_cleave)= ...
            signal_mask.likely_cleave(genes(i).exons{k}(1,1)-tol.good_cleave:genes(i).exons{k}(1,1)+tol.good_cleave);
        signal_mask.good_tss(genes(i).exons{k}(tmp,2)-tol.good_tss:genes(i).exons{k}(tmp,2)+tol.good_tss)= ...
            signal_mask.likely_tss(genes(i).exons{k}(tmp,2)-tol.good_tss:genes(i).exons{k}(tmp,2)+tol.good_tss);
      end    
      
      if genes(i).strands(1)=='+',
        signal_mask.sure_don(genes(i).exons{k}(1:end-1,2))=1 ;
        signal_mask.sure_acc(genes(i).exons{k}(2:end,1))=1 ;
      else
        signal_mask.sure_don(genes(i).exons{k}(2:end,1))=1 ;
        signal_mask.sure_acc(genes(i).exons{k}(1:end-1,2))=1 ;
      end ;

      % treat first and last exons slightly different
      signal_mask.no_ss(genes(i).exons{k}(1,1)+tol.no_ss:genes(i).exons{k}(1,2)-1) = 1 ;
      
      

      
      % no splice sites in exons
      for q=2:size(genes(i).exons{k},1)-1
        signal_mask.no_ss(genes(i).exons{k}(q,1)+1:genes(i).exons{k}(q,2)-1) = 1 ;
      end ;
      q=size(genes(i).exons{k},1) ;
      % no splice sites in last exon
      signal_mask.no_ss(genes(i).exons{k}(q,1)+1:genes(i).exons{k}(q,2)-tol.no_ss) = 1 ;
      
      
      % all introns are treated equal -> no splice sites
      for q=1:size(genes(i).exons{k},1)-1
        signal_mask.no_ss(genes(i).exons{k}(q,2)+1:genes(i).exons{k}(q+1,1)-1) = 1 ;
        signal_mask.no_tis(genes(i).exons{k}(q,2)+1:genes(i).exons{k}(q+1,1)-1) = 1 ;
        signal_mask.no_stop(genes(i).exons{k}(q,2)+1:genes(i).exons{k}(q+1,1)-1) = 1 ;
      end ;
    end ;
  end ;

  if 0,
    % if there is any disagreement, then forget about all this...
    regions = detectaltregions(genes(i))  ;
    if ~isempty(regions), 
      for k=1:size(regions,2)
        signal_mask.no_ss(regions(1,k)-tol.no_ss:regions(2,k)+tol.no_ss) = 0 ;
        signal_mask.likely_don(regions(1,k)-tol.no_ss:regions(2,k)+tol.no_ss) = 0 ;
        signal_mask.likely_acc(regions(1,k)-tol.no_ss:regions(2,k)+tol.no_ss) = 0 ;
        signal_mask.sure_don(regions(1,k)-tol.no_ss:regions(2,k)+tol.no_ss) = 0 ;
        signal_mask.sure_acc(regions(1,k)-tol.no_ss:regions(2,k)+tol.no_ss) = 0 ;
        signal_mask.likely_tis(regions(1,k)-tol.no_tis:regions(2,k)+tol.no_tis) = 0 ;
        signal_mask.likely_stop(regions(1,k)-tol.no_stop:regions(2,k)+tol.no_stop) = 0 ;
        signal_mask.no_tis(regions(1,k)-tol.no_tis:regions(2,k)+tol.no_tis) = 0 ;
        signal_mask.no_stop(regions(1,k)-tol.no_stop:regions(2,k)+tol.no_stop) = 0 ;
      end ;
    end ;
  end ;

  signal_masks(1+(genes(i).strands(1)=='-'), chr_num) = signal_mask ;
end ;


save_struct(sprintf('%s/signal_masks_withends.mat', genome_info.basedir), signal_masks, 'signal_masks') ;

return

for i=1:length(genome_info.flat_fnames)
i
  for j=1:2,
    signal_masks(j,i).acc = zeros(1,length(signal_masks(j,i).no_ss)) ;
    signal_masks(j,i).acc(signal_masks(j,i).no_ss) = -1 ;
    signal_masks(j,i).acc(signal_masks(j,i).likely_acc) = 1 ;
    signal_masks(j,i).acc(signal_masks(j,i).sure_acc) = 2 ;

    signal_masks(j,i).don = zeros(1,length(signal_masks(j,i).no_ss)) ;
    signal_masks(j,i).don(signal_masks(j,i).no_ss) = -1 ;
    signal_masks(j,i).don(signal_masks(j,i).likely_don) = 1 ;
    signal_masks(j,i).don(signal_masks(j,i).sure_don) = 2 ;

    signal_masks(j,i).tis = zeros(1,length(signal_masks(j,i).no_ss)) ;
    signal_masks(j,i).tis(signal_masks(j,i).no_tis) = -1 ;
    signal_masks(j,i).tis(signal_masks(j,i).likely_tis) = 1 ;

    signal_masks(j,i).stop = zeros(1,length(signal_masks(j,i).no_ss)) ;
    signal_masks(j,i).stop(signal_masks(j,i).no_stop) = -1 ;
    signal_masks(j,i).stop(signal_masks(j,i).likely_stop) = 1 ;


    signal_masks(j,i).tss = zeros(1,length(signal_masks(j,i).no_ss)) ;
    signal_masks(j,i).tss(signal_masks(j,i).no_tss) = -1 ;
    signal_masks(j,i).tss(signal_masks(j,i).likely_tss) = 1 ;

    signal_masks(j,i).cleave = zeros(1,length(signal_masks(j,i).no_ss)) ;
    signal_masks(j,i).cleave(signal_masks(j,i).no_cleave) = -1 ;
    signal_masks(j,i).cleave(signal_masks(j,i).likely_cleave) = 1 ;
    
    fnames={'acc', 'don', 'tis', 'stop'} ;
    for k=1:length(fnames)
      signal_masks(j,i).(fnames{k})=sparse(signal_masks(j,i).(fnames{k})) ;
    end ;
  end ;
end ;

fnames = {'no_ss', 'likely_acc', 'likely_don', 'sure_acc', 'sure_don', 'no_tis', 'no_stop', 'likely_tis', 'likely_stop'} ;
for k=1:length(fnames)
  signal_masks=rmfield(signal_masks, fnames{k}) ;
end ;
