
function [ genes ] = add_cand_cds( PAR, genes );


% === init
patterns_start = PAR.Signals.tis.consensus;
patterns_stop  = PAR.Signals.stop.consensus;
max_nof_paths  = PAR.label_source_par.cds.max_nof_paths;
nof_best_kept  = PAR.label_source_par.cds.nof_best_kept;
SPLICEGRAPH = PAR.info_genes.cds.splicegraph;
FULLLENGTH  = PAR.info_genes.cds.fulllength;
assert_all( PAR.info_genes.transcript.fulllength == FULLLENGTH );

% === go thru all genes
cds_list = zeros( 0, 6 );
for( i = 1:length(genes) )
  gene = genes(i);
  fprintf( '%i\r', i);
  % fprintf( 'Gene %5d: %s\n', i, gene.name );
  strand = gene.strand;
  assert( strand=='+' | strand=='-' );
  ss = flip_strand_a( [gene.start,gene.stop], strand );
  start = ss(1);
  stop = ss(2);
  gene.seq = load_genomic_a( gene.chr, start, stop );

  % === start from existsing CDSes
  if( isempty(gene.cds) )
    cds_i = zeros( 0, 7 );
    assert( isempty(gene.cds_info) );
    assert( isempty(gene.cds_conf) );
  else
    n_i = size( gene.cds, 1 );
    assert_all( size(gene.cds) == [n_i,2] );
    assert_all( size(gene.cds_conf) == [n_i,4] );
    assert_all( size(gene.cds_info) == [n_i,1] );
    if( strand == '+' )
      cds = gene.cds;
    else
      cds = -gene.cds( :, [2,1] );
    end;
    cds_i = [ cds, gene.cds_conf, gene.cds_info ];
  end;
  assert( size(cds_i,2) == 7 );
  
  % === DEBUG
  if( 0 )
    gene.strands = repmat( gene.strand, size(gene.info_type) );
    viewsplicegraph( gene );
    print( '-dpsc', sprintf('splicegraph,gene=%d.ps',i) );
  end;

  % === guess CDSes from transcripts
  js = find( gene.transcript_info == FULLLENGTH );
  js = 1:length( gene.exons );
  for( j = js )
    transcript = flip_strand_a( gene.exons{j}, strand );
    cds_ij = find_ORFs_in_transcript( gene.seq, start, transcript, patterns_start, patterns_stop );
    cds_i = storeCds( cds_i, cds_ij, FULLLENGTH, nof_best_kept );
  end;

  % === guess CDSes from splice graph
  splicegraph = gene.splicegraph;
  nof_paths = nof_paths_in_splicegraph( splicegraph );
  if( nof_paths <= max_nof_paths )
    splicegraph{1} = flip_strand_a( splicegraph{1}, strand );
    splicegraph{2} = abs( flip_strand_a( splicegraph{2}, strand ) );
    [ transcripts, paths ] = enum_transcripts_from_splicegraph( splicegraph, nof_paths );
    for( j = 1:nof_paths )
      transcript = transcripts{j};
      cds_ij = find_ORFs_in_transcript( gene.seq, start, transcript, patterns_start, patterns_stop );
      cds_i = storeCds( cds_i, cds_ij, SPLICEGRAPH, nof_best_kept );
    end;
  end;
  
  % === eliminate duplicates (but keep if from different source)
  [ orfs ] = unique( cds_i(:,[1:2,7]), 'rows' );
  nofClean = size( orfs, 1 );
  cds_clean = repmat( nan, nofClean, size(cds_i,2) );
  for( j = 1:nofClean )
    idx = find( cds_i(:,1)==orfs(j,1) & cds_i(:,2)==orfs(j,2) & cds_i(:,7)==orfs(j,3) );
    cds_clean(j,:) = max( cds_i(idx,:), [], 1 );
  end;
  
  % === store candidates
  %%% gene.cds      = cds_clean( :, 1:2 );
  %%% gene.cds_conf = cds_clean( :, 3:6 );
  %%% gene.cds_info = cds_clean( :, 7 );
  %%% genes(i) = gene;
  if( strand == '+' )
    genes(i).cds = cds_clean( :, 1:2 );
  else
    genes(i).cds = -cds_clean( :, [2,1] );
  end;
  genes(i).cds_conf = cds_clean( :, 3:6 );
  genes(i).cds_info = cds_clean( :, 7 );
  
end;



function [ cds_i ] = storeCds( cds_i, cds_ij, info, nof_best_kept );

[ dummy, perm ] = sort( cds_ij(:,3), 'descend' );
t = min( length(perm), nof_best_kept );
cds_ij = cds_ij( perm(1:t), : );
cds_ij = [ cds_ij , repmat( info, size(cds_ij,1), 1 ) ];
assert( size(cds_ij,2) == size(cds_i,2) );
cds_i = [ cds_i ; cds_ij ];


