function genes = addlabels2genes(genes,PAR,old_signals)
%  genes = addlabels2genes(genes,PAR)
  
if isempty(old_signals)
  
end
%% transcript_ends
if ~PAR.add_signals.tss %~isequal(PAR.Signal_name,'tss')
  fprintf(1,'ignoring tss labels \n')
elseif ~isempty(old_signals)& old_signals.tss
  fprintf(1,'tss labels have been added to genes before;ignoring tss \n')
else
  source = PAR.label_source.tss
  if any(source.est_clusters|source.fulllength|source.db)
   fprintf(1,'adding tss labels \n')
   if source.est_clusters
      fprintf(1,'getting TSS labels from est_clusters\n')
      load(PAR.FN.input_sig.tss.fn_est_cluster,'TSS')
      genes = addlabel2genes(genes,TSS,'tss',PAR.info_genes.tss.est_clusters);
    end
    if source.fulllength
      fprintf(1,'getting TSS labels from full-length cDNAs\n')
      load(PAR.FN.input_sig.tss.fn_fulllength,'TSS')
      genes = addlabel2genes(genes,TSS,'tss',PAR.info_genes.tss.fulllength);
    end
    if source.db
      fprintf(1,'getting TSS labels from database\n')
      load(PAR.FN.input_sig.tss.fn_db,'TSS')
      genes = addlabel2genes(genes,TSS,'tss',PAR.info_genes.tss.db);
    end
  end
end

if  ~PAR.add_signals.cleave
  fprintf(1,'ignoring cleave labels \n')
elseif ~isempty(old_signals)&old_signals.cleave
  fprintf(1,'cleave labels have been added to genes before;ignoring cleave \n')
else
  source = PAR.label_source.cleave
  if any(source.est_clusters|source.fulllength|source.db)
    fprintf(1,'adding cleave labels \n')  
    if source.est_clusters
      fprintf(1,'getting CLEAVE labels from est_clusters\n')
      load(PAR.FN.input_sig.cleave.fn_est_cluster,'CLEAVE')
      genes = addlabel2genes(genes,CLEAVE,'cleave',PAR.info_genes.cleave.est_clusters);
    end
    if source.fulllength
      fprintf(1,'getting CLEAVE labels from full-length cDNAs\n')
      load(PAR.FN.input_sig.cleave.fn_fulllength,'CLEAVE')
      genes = addlabel2genes(genes,CLEAVE,'cleave',PAR.info_genes.cleave.fulllength);
    end
    if source.db
      fprintf(1,'getting CLEAVE labels from database\n')
      load(PAR.FN.input_sig.cleave.fn_db,'CLEAVE')
      genes = addlabel2genes(genes,CLEAVE,'cleave',PAR.info_genes.cleave.db);
    end
  end
end

%%TRANSACC
if ~PAR.add_signals.transacc
  fprintf(1,'ignoring transacc labels \n')
elseif ~isempty(old_signals)&old_signals.transacc
  fprintf(1,'transacc labels have been added to genes before;ignoring transacc \n')
else
  source = PAR.label_source.transacc
  if source.anno_SL1
    fprintf(1,'adding transacc labels \n')
    fprintf(1,'getting TRANSACC labels from annotation SL1\n')
    load(PAR.FN.input_sig.transacc.fn_anno_SL1,'TRANSACC')
    genes = addlabel2genes(genes,TRANSACC,'transacc',...
                                      PAR.info_genes.transacc.anno_SL1,...
                                      PAR.label_source_par.transacc.win_anno_SL1);
    % save(PAR.FN.input_sig.transacc.fn_anno_SL1,'TRANSACC');
  end
  if source.anno_SL2
    fprintf(1,'adding transacc labels \n')
    fprintf(1,'getting TRANSACC labels from annotation SL2\n')
    load(PAR.FN.input_sig.transacc.fn_anno_SL2,'TRANSACC')
    genes = addlabel2genes(genes,TRANSACC,'transacc',...
                                      PAR.info_genes.transacc.anno_SL2,...
                                      PAR.label_source_par.transacc.win_anno_SL2 );
    % save(PAR.FN.input_sig.transacc.fn_anno_SL2,'TRANSACC');
  end
end

%% polya
if  ~PAR.add_signals.polya
 fprintf(1,'ignoring polya labels \n')
elseif ~isempty(old_signals)&old_signals.polya
  fprintf(1,'polya labels have been added to genes before;ignoring polya \n')
else
  fprintf(1,'adding polya labels \n')
  source = PAR.label_source.polya
  if source.from_cleave
    fprintf(1,'getting POLYA labels from cleave\n')
    load(PAR.FN.input_sig.polya.fn_polya,'polya')
    genes = addlabel2genes(genes,polya,'polya',PAR.info_genes.polya.from_cleave);
    fprintf(1,'addlabels2genes.m line: 56')
  end
end

%cds
if ~PAR.add_signals.tis
  fprintf(1,'ignoring tis labels \n')
elseif ~isempty(old_signals)&old_signals.tis
  fprintf(1,'tis labels have been added to genes before;ignoring tis \n')
else
  source = PAR.label_source.tis;
  if source.anno & ~PAR.label_source.use_anno
    fprintf(1,'getting tis labels from annotation \n')
    load(PAR.FN.input_sig.tis.fn_anno,'TIS')
    genes = addlabel2genes(genes,TIS,'tis',PAR.info_genes.tis.anno);
  end
end

if ~PAR.add_signals.cdsStop
  fprintf(1,'ignoring cdsStop labels \n')
elseif ~isempty(old_signals)& old_signals.cdsStop
  fprintf(1,'cdsStop labels have been added to genes before;ignoring cdsStop \n')
else
  source = PAR.label_source.cdsStop
  if source.anno& ~PAR.label_source.use_anno
    fprintf(1,'getting cdsStop labels\n')
    load(PAR.FN.input_sig.cdsStop.fn_anno,'STOP')
    genes = addlabel2genes(genes,STOP,'cdsStop',PAR.info_genes.cdsStop.anno);  
  end
end

return



function [genes,labels] = addlabel2genes(genes,labels,field,info,dist_threshold)

fprintf(1,'number of new labels %i\n',length(labels)) 

if isequal(field,'transacc')
  warning('conf Level will be overridden with distance to nearest gene')
end

CHR = [genes.chr_num]; 
STR = [genes.strand]; 

label_covered = [];
cnt_rm=0;
for l_idx = 1:length(labels)
 
  fprintf('%i\r',l_idx);
  label = labels(l_idx);
  geneidx = find(CHR==label.chr_num & STR ==label.strand);
  clear ends ;
  if isempty(geneidx)
    continue
  end
  
  ends(1,:) = [genes(geneidx).start];
  ends(2,:) = [genes(geneidx).stop];
  if (isequal(field,'tss')&label.strand=='+')|...
     (isequal(field,'tis')&label.strand=='+')|...
        (isequal(field,'transacc')&label.strand=='+')|...
        (isequal(field,'cleave')&label.strand=='-')|...
        (isequal(field,'polya')&label.strand=='-')|...
        (isequal(field,'cdsStop')&label.strand=='-'),
    [dist,idx] = min(abs(min(ends)-label.pos));
  elseif (isequal(field,'tss')&label.strand=='-')|...
         (isequal(field,'tis')&label.strand=='-')|...
        (isequal(field,'transacc')&label.strand=='-')|...
        (isequal(field,'cleave')&label.strand=='+')|...
        (isequal(field,'polya')&label.strand=='+')|...
        (isequal(field,'cdsStop')&label.strand=='+')
    [dist,idx] = min(abs(max(ends)-label.pos));
  end
  
  if isfield(label,'confirmed_genes') 
    %assert(any(ismember(label.confirmed_genes,genes(geneidx(idx)).confgenes_id)))
    if ~any(ismember(label.confirmed_genes,genes(geneidx(idx)).confgenes_id));
      warning('label.confirmed_genes different from next gene!')
      idx_conf = find4structarrays( genes(geneidx), label.confirmed_genes, 'confgenes_id' );
      % assert(logical(genes(geneidx(idx_conf)).is_alt));
      if ~(genes(geneidx(idx_conf)).start<=label.pos&genes(geneidx(idx_conf)).stop>=label.pos)|...
            (genes(geneidx(idx_conf)).stop<=label.pos& ...
             genes(geneidx(idx_conf)).start>=label.pos)
        idx=[];
      end
      idx = idx_conf;
      % sum(triu(genes(geneidx(idx_conf)).splicegraph{2}))
      % warning('something wrong')
      l_idx;
      % figure;viewsplicegraph(genes(geneidx(idx)))
      % figure;viewsplicegraph(C.genes(label.confirmed_genes))
      % hold on;plot([label.pos label.pos],[1  10],'b'); 
      % keyboard
    end
  end
  
  if isfield(label,'anno_genes') 
    if ~any(ismember(label.anno_genes,genes(geneidx(idx)).anno_id));
      warning('label.anno_genes different from next gene!')
      idx_anno = find4structarrays( genes(geneidx), label.anno_genes, 'anno_id' );
      % assert(logical(genes(geneidx(idx_anno)).is_alt));
      if ~(genes(geneidx(idx_anno)).start<=label.pos&genes(geneidx(idx_anno)).stop>=label.pos)|...
            (genes(geneidx(idx_anno)).stop<=label.pos& ...
             genes(geneidx(idx_anno)).start>=label.pos)
        idx=[];
      end
      idx = idx_anno;
      % sum(triu(genes(geneidx(idx_anno)).splicegraph{2}))
      % warning('something wrong')
      l_idx;
      % figure;viewsplicegraph(genes(geneidx(idx)))
      % figure;viewsplicegraph(C.genes(label.annoirmed_genes))
      % hold on;plot([label.pos label.pos],[1  10],'b'); 
      % keyboard
    end
  end
  
  if isempty(idx)
    continue
  end
  old_pos = genes(geneidx(idx)).(field);
  old_info = genes(geneidx(idx)).([field '_info']);
  old_conf = genes(geneidx(idx)).([field '_conf']);
  
  % make old_pos an column vector
  if size(old_pos,1)<size(old_pos,2)
    old_pos = old_pos';
  end
  % make old_info an column vector
  if size(old_info,1)<size(old_info,2)
    old_info = old_info';
  end
  % make old_conf an column vector
  if size(old_conf,1)<size(old_conf,2)
    old_conf = old_conf';
  end

  if isequal(field,'transacc')
    if dist>dist_threshold
      cnt_rm=cnt_rm+1;
      continue
    end
    % label.confLevel==dist;
  end
  
  if ~isempty(old_pos)&&~isequal(old_pos,label.pos)
    new_pos = [old_pos; label.pos'];
    new_info = [old_info; info'];
      new_conf = [old_conf; label.confLevel'];
  else
    new_pos = label.pos;
    new_info = info;
    new_conf = label.confLevel;
  end
  
  if ~(length(new_pos)==length(new_info)), warning('check this!!!!'), end;
  if ~(length(new_pos)==length(new_conf)), warning('check this!!!!'), end;
  genes(geneidx(idx)).(field) = new_pos;
  genes(geneidx(idx)).([field '_info']) = new_info;
  genes(geneidx(idx)).([field '_conf']) = new_conf;
  if isequal(field,'transacc')
    labels(l_idx) = label;
  end
end
if isequal(field,'transacc')
  fprintf(1,'number of labels removed (distance to next gene > %i): %i %i\n',dist_threshold,cnt_rm) 
end
