function [train,test] = assign_subsets(PAR)
%  [train,test] = assign_subsets(PAR)

load(PAR.FN.output.fn_trivial_regions,'trivial_regions');
  
if ~isempty(PAR.FN.input.fn_trainings_regions)&fexist(PAR.FN.input.fn_trainings_regions)
  assert(ismember('training_regions',who('-file',PAR.FN.input.fn_trainings_regions)))
  load(PAR.FN.input.fn_trainings_regions,'training_regions');
else
  training_regions = trivial_regions;
  fprintf('no training region specified, training on whole genome except test region\n')
end

if ~isempty(PAR.FN.input.fn_test_regions)&fexist(PAR.FN.input.fn_test_regions)
  assert(ismember('test_regions',who('-file',PAR.FN.input.fn_test_regions)))
  load(PAR.FN.input.fn_test_regions,'test_regions');
else
  test_regions = trivial_regions(1);
  test_regions.stop = 0;
  test_regions.start = 0;
  fprintf('no test region specified, training on whole genome\n')
end
fprintf('make shure there is no overlap between training and test regions\n')  
if ~(PAR.tasks.pred_on_genome & PAR.tasks.train_on)
  training_regions = assert_noRegionOverlap(test_regions,training_regions);
else
  warning('region overlap not checked!! Training and Test region could overlap!\n')
end
rand('seed',1234);
idx = [];
PAR.regions.offset = PAR.regions.offset_train;
train.blocks = split_regions(training_regions,PAR,0);
PAR.regions.offset = PAR.regions.offset_test;
test.blocks = split_regions(test_regions,PAR,0);


if ~isempty(test.blocks)
  test.blocks = rmfield(test.blocks,'left_dist');
  test.blocks = rmfield(test.blocks,'right_dist');
  % test.blocks = sort_regions(test.blocks) ;
  % for j=1:length(test.blocks)
  %   test.blocks(j).id = j;
  % end
  [test.blocks, test.split] = make_split(test.blocks,PAR)
else
  test.split = [];
end

% merge training blocks and generate training split
if ~isempty(train.blocks)
  % train.blocks = sort_regions(train.blocks) ; 
  % for j=1:length(train.blocks)
  %   train.blocks(j).id = j;
  % end
  if PAR.regions.merge_confirmed
    train.blocks = construct_merged_blocks(train.blocks,PAR);
  else
    train.blocks = rmfield(train.blocks,'left_dist');
    train.blocks = rmfield(train.blocks,'right_dist');
  end
  if PAR.regions.cluster_paralogs
    split_idx = splits_by_paralogs(train.blocks,PAR.SETs.num_splits);
    for s = 1:PAR.SETs.num_splits
      idx = setfield(idx,sprintf('split_%i',s),split_idx{s});
      error('something wron here; check implementation')
      train.blocks(idx.(sprintf('split_%i',s))) = s;
    end
  else
    [train.blocks, train.split] = make_split(blocks,PAR)
  end
else
  train.split = [];
end



function [blocks, split] = make_split(blocks,PAR)

blocks_per_split = ceil(length(blocks)/PAR.SETs.num_splits);
block_idx  = randperm(length(blocks));
for s = 1:PAR.SETs.num_splits
  IDX = (s-1)*blocks_per_split+[1:blocks_per_split];
  IDX(IDX>length(blocks)) = [];
  IDX = block_idx(IDX);
  split = setfield(split,sprintf('split_%i',s),IDX);
  [blocks(IDX).split] = deal(s);
end

y = [];
for s = 1:PAR.SETs.num_splits
  y = [y getfield(split,sprintf('split_%i',s))];
end
assert(length(y)==length(unique(y)))