function clearedCLEAVE = cluster_EST_matches_cleave_chrwise(PAR)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% cluster_EST_matches_cleave_chrwise.m
% May 2007

% Filters those ESTs from the data set est_matches.mat that contain a putative cleavage sites (chromosome-wise).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load est_matches
M = load(PAR.fn_in_ests,'genes');
genome_info =  init_genome(PAR.FN.input.fn_genome_config) ;

for i=1:length(M.genes)
  M.genes(i).strand = M.genes(i).strands(1);
  M.genes(i).chr_num = strmatch(M.genes(i).chr,genome_info.contig_names,'exact') ;
end

% genes are confirmed_genes, est are est_matches
est = M.genes;
clear M.genes;

% load confirmed_genes
load(PAR.FN.input.fn_genes_conf_orig ,'genes');
for i=1:length(genes)
  genes(i).strand = genes(i).strands(1);
  genes(i).chr_num = strmatch(genes(i).chr,genome_info.contig_names,'exact') ;
end


debug = PAR.debug;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A. Finding putative 5 prime ends
%%%%%
% We find putative 5 prime ends in respect to the following conditions:

% 1. Indicates whether other EST starts lie inside a [-5,5] window around the 3 prime end of the EST.
% 2. Indicates whether another EST of the cluster covers [5,11] downstream of current_stop.
% 3a. Indicates that the next matching EST is at least 1000 bp downstream.
% 3b. Indicates that the next matching EST is at least 500 bp downstream.
% 4. Indicates whether the EST is included in confirmed_sequences and lies around the gene end.


% counters
discarded_clusters_not_conf = 0;
discarded_clusters_not_has_cDNA = 0;
discarded_clusters_too_short = 0;
discard_downstream_overlap = 0;
discarded_dist500 = 0;
discarded_not_near_conf = 0;

% window around the gene end in which the site should be
win = 20;

if (debug) fprintf(1,'Number of EST clusters: %d\n', length(est)); end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% check first and second condition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

counter = 1;
% for each EST cluster
for i = 1:length(est)
								 
  if (debug) if (mod(i,100)==0) fprintf(1,'%i\r',i); end; end;
  % discard current EST cluster if confirmed genes is empty 
  if isempty(est(i).confirmed_genes)
    discarded_clusters_not_conf = discarded_clusters_not_conf + 1;
    continue;
  end;

  % discard current EST cluster if it is not confirmed by cDNA 
  if PAR.label_source_par.cleave.est_clusters_useonlycDNA & (genes(est(i).confirmed_genes(1)).has_cDNA == 0)
    discarded_clusters_not_has_cDNA = discarded_clusters_not_has_cDNA + 1;
    continue;
  end;

  % discard current EST cluster if it is not long enough - here 100
  lengthEst = abs(est(i).stop - est(i).start);
  if lengthEst < 100
    discarded_clusters_too_short = discarded_clusters_too_short + 1;
    continue;
  end;

  numEsts = length(est(i).exons);

  % for all ESTs in the current EST cluster
  for j = 1:numEsts
    cur_start = est(i).exons{j}(1,1);
    cur_stop = est(i).exons{j}(1,2);
    cur_confirmed_genes = [];

    % check condition 4
    conf_overlap = 0;
    if est(i).strand == '+'
      for conf_idx=est(i).confirmed_genes
        fidx = find(sum(triu(genes(conf_idx).splicegraph{2})')==0);
        if (any(abs(genes(conf_idx).splicegraph{1}(2,fidx)-cur_stop)<=win))
           conf_overlap = 1;
           cur_confirmed_genes = [cur_confirmed_genes conf_idx];
        end;
      end;
    else
      for conf_idx=est(i).confirmed_genes
        fidx = find(sum(triu(genes(conf_idx).splicegraph{2}))==0);
        if (any(abs(genes(conf_idx).splicegraph{1}(1,fidx)-cur_start)<=win))
           conf_overlap = 1;
           cur_confirmed_genes = [cur_confirmed_genes conf_idx];
        end;
      end;
    end;


    if ~conf_overlap
      discarded_not_near_conf = discarded_not_near_conf + 1;
      continue;
    end;

    % count number of matches at the 3 prime position with [-5,5] (condition 1)
    matches = 0;    

    % check whether another EST of the cluster covers [5,11] downstream of cur_stop (condition 2)
    downstream_overlap = 0;

    for k = 1:numEsts
      idx_start = est(i).exons{k}(1,1);
      idx_stop = est(i).exons{k}(1,2);

      if est(i).strand == '+'
        % check condition 1
        if k ~= j && abs(cur_stop - idx_stop) <= 5   
          matches = matches + 1;
        end;
        % check condition 2
        if (idx_start <= cur_stop + 5) && (idx_stop >= cur_stop + 11)
	  downstream_overlap = 1;
          discard_downstream_overlap = discard_downstream_overlap + 1;
          break;
        end;
      else % negative strand
        % check condition 1
        if k ~= j && abs(cur_start - idx_start) <= 5   
          matches = matches + 1;
        end;
        % check condition 2
        if (idx_stop >= cur_start - 5) && (idx_start <= cur_start - 11)
	  downstream_overlap = 1;
          discard_downstream_overlap = discard_downstream_overlap + 1;
          break;
        end;
      end; % if pos/neg strand

    end; % loop k 

    % discard if downstream overlap
    if ~downstream_overlap     
      clevel = 1;
      if (matches >= 1) clevel = 2; end;  
      if (matches >= 3) clevel = 3; end;     
      CLEAVE(counter) = struct('name',est(i).transcripts{j},'start',cur_start,'stop', cur_stop,'confLevel',clevel,'strand',est(i).strand,'chr_num',est(i).chr_num, 'confirmed_genes', cur_confirmed_genes, 'ti', est(i).terminal_info(2,j));
      counter = counter + 1;
    end;

  end; % loop j
end; % loop i

    if (debug) fprintf(1,'Number of discarded clusters: \n not in confirmed: %d\n no cDNA: %d\n too short: %d\nNumber of discarded ESTs\n because not included as EST in confirmed: %d\n because of downstream overlap: %d\nNumber of detected ESTs: %d\n', discarded_clusters_not_conf, discarded_clusters_not_has_cDNA, discarded_clusters_too_short, discarded_not_near_conf, discard_downstream_overlap, length(CLEAVE)); end;



%%%%%%%%%%%%%%%%%%%%%%%
% process putative ESTs
%%%%%%%%%%%%%%%%%%%%%%%

if (debug) fprintf(1, '\nProcessing putative cleavage sites...\n'); end;

% separate ESTs from the positive strand and negative strand
CLEAVEPidx = find([CLEAVE.strand]=='+');
CLEAVEsizeP = length(CLEAVEPidx);
CLEAVENidx = find([CLEAVE.strand]=='-');
CLEAVEsizeN = length(CLEAVENidx);

to_erase = [];

% process ESTs on positive strand
if CLEAVEsizeP ~= 0
  m_idx = 0; % counts number of cleavage sites on positive strand
  for m = CLEAVEPidx
    m_idx = m_idx + 1;
    if (debug) if (mod(m_idx,100)==0) fprintf(1,'%i\r',m_idx); end; end;
    cur_start = CLEAVE(m).start;
    cur_stop = CLEAVE(m).stop;
    tmp_idx = CLEAVEPidx;
    tmp_idx(m_idx) = []; % do not consider current cleavage site

    downstream_match_500 = 0;
    downstream_match_1000 = 0;

    diff = [CLEAVE(tmp_idx).start] - cur_stop;
    if ~isempty(diff(diff>0 & diff<=500))
      downstream_match_500 = 1;
    end;
    if ~isempty(diff(diff>0 & diff<=1000))
      downstream_match_1000 = 1;
    end;

    if downstream_match_500 == 1
      discarded_dist500 = discarded_dist500 + 1;
      to_erase = [to_erase m];
      continue;
    end;

    % set confidence 3 to 2 if distance <= 1000 bp
    if downstream_match_1000 == 1
      CLEAVE(m).confLevel = min(CLEAVE(m).confLevel, 2);
    end;

  end; % loop m
end;

% process ESTs on negative strand
if CLEAVEsizeN ~= 0
  m_idx = 0; % counts number of cleavage sites on negative strand
  for m = CLEAVENidx
    m_idx = m_idx + 1;
    if (debug) if (mod(m_idx,100)==0) fprintf(1,'%i\r',m_idx); end; end;
    cur_start = CLEAVE(m).start;
    cur_stop = CLEAVE(m).stop;
    tmp_idx = CLEAVENidx;
    tmp_idx(m_idx) = []; % do not consider current CLEAVE

    downstream_match_500 = 0;
    downstream_match_1000 = 0;

    diff = cur_start - [CLEAVE(tmp_idx).stop];
    if ~isempty(diff(diff>0 & diff<=500))
      downstream_match_500 = 1;
    end;
    if ~isempty(diff(diff>0 & diff<=1000))
      downstream_match_1000 = 1;
    end;

    if downstream_match_500 == 1
      discarded_dist500 = discarded_dist500 + 1;
      to_erase = [to_erase m];
      continue;
    end;

    % set confidence 3 to 2 if distance <= 1000 bp
    if downstream_match_1000 == 1
      CLEAVE(m).confLevel = min(CLEAVE(m).confLevel, 2);
    end; 

  end; % loop m
end;


CLEAVE(to_erase) = [];
if (debug) fprintf(1,'\nDone!\n'); end;


if (debug) fprintf(1,'\nNumber of discarded cleavage sites because of 500-downstream match: %d\n',discarded_dist500); end;
if (debug) fprintf(1,'Total number of putative cleavage sites : %d, p %d, n %d\n', length(CLEAVE), sum(ismember([CLEAVE.strand],'+')), sum(ismember([CLEAVE.strand],'-'))); end;
if (debug) fprintf(1,' c3: %d, p %d, n %d\n', sum(ismember([CLEAVE.confLevel],3)), sum(ismember([CLEAVE.confLevel],3) & ismember([CLEAVE.strand],'+')), sum(ismember([CLEAVE.confLevel],3) & ismember([CLEAVE.strand],'-'))); end;
if (debug) fprintf(1,' c2: %d, p %d, n %d\n', sum(ismember([CLEAVE.confLevel],2)), sum(ismember([CLEAVE.confLevel],2) & ismember([CLEAVE.strand],'+')), sum(ismember([CLEAVE.confLevel],2) & ismember([CLEAVE.strand],'-'))); end;
if (debug) fprintf(1,' c1: %d, p %d, n %d\n', sum(ismember([CLEAVE.confLevel],1)), sum(ismember([CLEAVE.confLevel],1) & ismember([CLEAVE.strand],'+')), sum(ismember([CLEAVE.confLevel],1) & ismember([CLEAVE.strand],'-'))); end;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% B.reducing clusters to a single position by picking the most likely position
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% separate ESTs from the positive strand and negative strand
RESULTP = CLEAVE([CLEAVE.strand] == '+' );
RESULTN = CLEAVE([CLEAVE.strand] == '-' );
clearedCLEAVE = [];

% positive strand
if ~isempty(RESULTP)
  counter = 1;
  % take all the ends of the positive putative ESTs
  clear testVector sortVector idxV uniqueSortVector midx nidx histVector;
  testVector  = [RESULTP(:).stop];
  [sortVector,idxV] = sort(testVector);
  [uniqueSortVector, midx, nidx] = unique(sortVector);
  % sort, find the unique ones, count their occurences
  histVector = hist(sortVector, unique(sortVector));

  l = length(uniqueSortVector);
  i = 1;
  while i <= l,
    nexti = i + 1;
    % this is the reference site
    refvalue = uniqueSortVector(i);
    refocc = histVector(i);
    % the index of the representing site 
    refi = i;
    TMP = RESULTP([RESULTP(:).stop] == refvalue);
    if sum([TMP(:).ti] == 1) >= 1
      tiflag = 1;
    else
      tiflag = 0;
    end;
    % the index of the larger site with same number of occurrences
    refi2 = 0;

    for j=max(1,i-10):min(l,i+10)
      if j~= i,
       % compare with an alternative site 
       altvalue = uniqueSortVector(j);
       % compare all which are within a 21bp window centered at refvalue
       if abs(refvalue - altvalue) <= 10
         % does it have ti  == 1?
         TMP = RESULTP([RESULTP(:).stop] == altvalue);
         tiflag = tiflag || (sum([TMP(:).ti] == 1) >= 1);
         nexti = max(nexti,j+1);
         if histVector(j) > refocc,
	   % this will be the representative
	   refocc = histVector(j);
	   refi = j;
	   refi2 = 0;
         elseif histVector(j) == refocc,
           refi2 = j;
         end;
       end;
      end;
    end % loop j
  
    if ~tiflag
      i = nexti;
      continue;
    end; 
   
    if refi2 == 0,
       mostPV = RESULTP([RESULTP(:).stop] == uniqueSortVector(refi));
       mostP = mostPV(1);
       mm = mostP.stop;
    else
      % take the median of values between the first and last peak
      r1 = min(refi, refi2);
      r2 = max(refi,refi2);
      mostPV = [RESULTP([RESULTP(:).stop] == uniqueSortVector(r1))  RESULTP([RESULTP(:).stop] == uniqueSortVector(r2))]; 
      mostP = mostPV(1);
      sV = sortVector(sortVector >= uniqueSortVector(r1) & sortVector <= uniqueSortVector(r2));
      mm = floor(median(sV));
    end;

    if counter > 1 && mm == REPRP(counter-1).pos
      i = nexti;
      continue;
    end;
  
    REPRP(counter) = struct('name',mostP.name, 'pos', mm, 'confLevel', mostP.confLevel, 'strand', mostP.strand, 'chr_num', mostP.chr_num,'confirmed_genes', mostP.confirmed_genes, 'start', mostP.start, 'stop', mostP.stop);

    counter = counter + 1;
    i = nexti;
  end % loop i
  clearedCLEAVE = REPRP;
end;

% negative strand
if ~isempty(RESULTN)
  counter = 1;
  clear testVector sortVector idxV uniqueSortVector midx nidx histVector;
  testVector  = [RESULTN(:).start];
  [sortVector,idxV] = sort(testVector);
  [uniqueSortVector, midx, nidx] = unique(sortVector);
  histVector = hist(sortVector, unique(sortVector));

  l = length(uniqueSortVector);
  i = 1;
  while i <= l,
    nexti = i + 1;
    refvalue = uniqueSortVector(i);
    refocc = histVector(i);
    refi = i;
    TMP = RESULTN([RESULTN(:).start] == refvalue);
    if sum([TMP(:).ti] == 1) >= 1
      tiflag = 1;
    else
      tiflag = 0;
    end;
    refi2 = 0;

    for j=max(1,i-10):min(l,i+10)
      if j~= i,
       altvalue = uniqueSortVector(j);
       if abs(refvalue - altvalue) <= 10,
         % does it have ti  == 1?
         TMP = RESULTN([RESULTN(:).start] == altvalue);
         tiflag = tiflag || (sum([TMP(:).ti] == 1) >= 1);
         nexti = max(nexti, j+1);
         if histVector(j) > refocc,
    	   refocc = histVector(j);
	   refi = j;
 	   refi2 = 0;
         elseif histVector(j) == refocc,
           refi2 = j;
         end;
       end;
      end;
    end % loop j
  
    if ~tiflag
      i = nexti;
      continue;
    end;
   
    if refi2 == 0,
      mostNV = RESULTN([RESULTN(:).start] == uniqueSortVector(refi));
      mostN = mostNV(1);
      mm = mostN.start;
    else
      % take the median of values between the first and last peak
      r1 = min(refi, refi2);
      r2 = max(refi,refi2);
      mostNV = [RESULTN([RESULTN(:).start] == uniqueSortVector(r1)) RESULTN([RESULTN(:).start] == uniqueSortVector(r2))]; 
      mostN = mostNV(1);
      sV = sortVector(sortVector >= uniqueSortVector(r1) & sortVector <= uniqueSortVector(r2));
      mm = floor(median(sV));
    end;

     if counter > 1 && mm == REPRN(counter-1).pos
      i = nexti;
      continue;
    end;

    REPRN(counter) = struct('name',mostN.name, 'pos', mm, 'confLevel', mostN.confLevel, 'strand', mostN.strand, 'chr_num', mostN.chr_num,'confirmed_genes', mostN.confirmed_genes, 'start', mostN.start, 'stop', mostN.stop);  
     counter = counter + 1;
     i = nexti;
  end % loop i
  clearedCLEAVE = [clearedCLEAVE REPRN];
end;


if (debug) fprintf(1,'Total number of putative cleavage sites : %d, p %d, n %d\n', length(clearedCLEAVE), sum(ismember([clearedCLEAVE.strand],'+')), sum(ismember([clearedCLEAVE.strand],'-'))); end;
if (debug) fprintf(1,' c3: %d, p %d, n %d\n', sum(ismember([clearedCLEAVE.confLevel],3)), sum(ismember([clearedCLEAVE.confLevel],3) & ismember([clearedCLEAVE.strand],'+')), sum(ismember([clearedCLEAVE.confLevel],3) & ismember([clearedCLEAVE.strand],'-'))); end;
if (debug) fprintf(1,' c2: %d, p %d, n %d\n', sum(ismember([clearedCLEAVE.confLevel],2)), sum(ismember([clearedCLEAVE.confLevel],2) & ismember([clearedCLEAVE.strand],'+')), sum(ismember([clearedCLEAVE.confLevel],2) & ismember([clearedCLEAVE.strand],'-'))); end;
if (debug) fprintf(1,' c1: %d, p %d, n %d\n', sum(ismember([clearedCLEAVE.confLevel],1)), sum(ismember([clearedCLEAVE.confLevel],1) & ismember([clearedCLEAVE.strand],'+')), sum(ismember([clearedCLEAVE.confLevel],1) & ismember([clearedCLEAVE.strand],'-'))); end;


CLEAVE = clearedCLEAVE;
save(PAR.fn_out,'CLEAVE') 
