function clearedTSS  = cluster_EST_matches_tss_chrwise(PAR)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% cluster_EST_matches_tss_chrwise.m
% May 2007

% Filters those ESTs from the data set est_matches.mat that contain a putative TSS (chromosome-wise).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% load est_matches
M = load(PAR.fn_in_ests,'genes');
genome_info =  init_genome(PAR.FN.input.fn_genome_config) ;

for i=1:length(M.genes)
  M.genes(i).strand = M.genes(i).strands(1);
  M.genes(i).chr_num = strmatch(M.genes(i).chr,genome_info.contig_names,'exact') ;
end

% genes are confirmed_genes, est are est_matches
est = M.genes;
clear M.genes;

% load confirmed_genes
load(PAR.FN.input.fn_genes_conf_orig ,'genes');
for i=1:length(genes)
  genes(i).strand = genes(i).strands(1);
  genes(i).chr_num = strmatch(genes(i).chr,genome_info.contig_names,'exact') ;
end


% set transsplice tag
transsplice = 0;
if 0% strcmp(PAR.organism.name,'C_elegans')
  transsplice = 1;
  transacc = [];
  if PAR.label_source_par.tss.filter_anno_SL1
    fprintf(1,'loading transacceptor files: %s\n',PAR.FN.input_sig.transacc.fn_anno_SL1)
    SL1 = load(PAR.FN.input_sig.transacc.fn_anno_SL1,'TRANSACC');
    idx = find([SL1.TRANSACC.dist]>PAR.label_source_par.tss.win_anno_SL1);
    transacc = [transacc SL1.TRANSACC(idx)];
    fprintf(1,'number of transacc used: %i (total:%i; window:+-%i)\n',...
            length(idx),length(SL1.TRANSACC),PAR.label_source_par.tss.win_anno_SL1);
  end
  if PAR.label_source_par.tss.filter_anno_SL2
    fprintf(1,'loading transacceptor files: %s\n',PAR.FN.input_sig.transacc.fn_anno_SL2)
    SL2 = load(PAR.FN.input_sig.transacc.fn_anno_SL2,'TRANSACC');
    idx = find([SL2.TRANSACC.dist]>PAR.label_source_par.tss.win_anno_SL2);
    transacc = [transacc SL2.TRANSACC(idx)];
    fprintf(1,'number of transacc used: %i (total:%i; window:+-%i)\n',...
            length(idx),length(SL2.TRANSACC),PAR.label_source_par.tss.win_anno_SL2);
  end
  if PAR.label_source_par.tss.filter_pred 
    fprintf(1,'loading transacceptor files: %s\n',PAR.FN.input_sig.transacc.fn_pred)
    SLpred = load(PAR.FN.input_sig.transacc.fn_pred,'TRANSACC');
    idx = find([SLpred.TRANSACC.confLevel]>PAR.label_source_par.tss.threshold_pred);
    transacc = [transacc SLpred.TRANSACC(idx)];
    fprintf(1,'number of transacc used: %i (total:%i; threshold:%i)\n',...
            length(idx),length(SLpred.TRANSACC),PAR.label_source_par.tss.threshold_pred);
  end;
  fprintf(1,'total number of transacc: %i',length(transacc)) 
  clear 
end

debug = PAR.debug;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A. Finding putative 5 prime ends
%%%%%
% We find putative 5 prime ends in respect to the following conditions:

% 1. Indicates whether other EST starts lie inside a [-5,5] window around the 5 prime end of the EST.
% 2. Indicates whether another EST of the cluster covers [5,11] upstream of current_start.
% 3a. Indicates that the next matching EST is at least 1000 bp upstream.
% 3b. Indicates that the next matching EST is at least 500 bp upstream.
% 4. Indicates whether the EST is included in confirmed_sequences and lies around the gene start.

% counters
discarded_clusters_not_conf = 0;
discarded_clusters_not_has_cDNA = 0;
discarded_clusters_too_short = 0;
discard_upstream_overlap = 0;
discarded_dist500 = 0;
discarded_transacc = 0;
discarded_not_near_conf = 0;

% window around the gene start in which the site should be
win = 20;

if (debug) fprintf(1,'Number of EST clusters: %d\n', length(est)); end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% check first and second condition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

counter = 1;
% for each EST cluster
for i = 1:length(est)
								 
  if (debug) if (mod(i,100)==0) fprintf(1,'%i\r',i); end; end;
  % discard current EST cluster if confirmed genes is empty 
  if isempty(est(i).confirmed_genes)
    discarded_clusters_not_conf = discarded_clusters_not_conf + 1;
    continue;
  end;

  % discard current EST cluster if it is not confirmed by cDNA 
  if PAR.label_source_par.tss.est_clusters_useonlycDNA & (genes(est(i).confirmed_genes(1)).has_cDNA == 0)
    discarded_clusters_not_has_cDNA = discarded_clusters_not_has_cDNA + 1;
    continue;
  end;

  % discard current EST cluster if it is not long enough - here 100
  lengthEst = abs(est(i).stop - est(i).start);
  if lengthEst < 100
    discarded_clusters_too_short = discarded_clusters_too_short + 1;
    continue;
  end;

  numEsts = length(est(i).exons);

  % for all ESTs in the current EST cluster
  for j = 1:numEsts
    cur_start = est(i).exons{j}(1,1);
    cur_stop = est(i).exons{j}(1,2);
    cur_confirmed_genes = [];

    % check condition 4
    conf_overlap = 0;
    if est(i).strand == '+'
      for conf_idx=est(i).confirmed_genes
        fidx = find(sum(triu(genes(conf_idx).splicegraph{2}))==0);
        if (any(abs(genes(conf_idx).splicegraph{1}(1,fidx)-cur_start)<=win))
           conf_overlap = 1;
           cur_confirmed_genes = [cur_confirmed_genes conf_idx];
        end;
      end;
    else
      for conf_idx=est(i).confirmed_genes
        fidx = find(sum(triu(genes(conf_idx).splicegraph{2})')==0);
        if (any(abs(genes(conf_idx).splicegraph{1}(2,fidx)-cur_stop)<=win))
           conf_overlap = 1;
           cur_confirmed_genes = [cur_confirmed_genes conf_idx];
        end;
      end;
    end;

    if ~conf_overlap
      discarded_not_near_conf = discarded_not_near_conf + 1;
      continue;
    end;

    % count number of matches at the 5 prime position with [-5,5] (condition 1)
    matches = 0;    

    % check whether another EST of the cluster covers [5,11] upstream of cur_start (condition 2)
    upstream_overlap = 0;

    for k = 1:numEsts
      idx_start = est(i).exons{k}(1,1);
      idx_stop = est(i).exons{k}(1,2);

      if est(i).strand == '+'
        % check condition 1
        if k ~= j && abs(cur_start - idx_start) <= 5   
          matches = matches + 1;
        end;
        % check condition 2
        if (idx_stop >= cur_start - 5) && (idx_start <= cur_start - 11)
	  upstream_overlap = 1;
          discard_upstream_overlap = discard_upstream_overlap + 1;
          break;
        end;
      else % negative strand
        % check condition 1
        if k ~= j && abs(cur_stop - idx_stop) <= 5   
          matches = matches + 1;
        end;
        % check condition 2
        if (idx_start <= cur_stop + 5) && (idx_stop >= cur_stop + 11)
	  upstream_overlap = 1;
          discard_upstream_overlap = discard_upstream_overlap + 1;
          break;
        end;
      end; % if pos/neg strand

    end; % loop k 

    % discard if upstream overlap
    if ~upstream_overlap     
      clevel = 1;
      if (matches >= 1) clevel = 2; end;  
      if (matches >= 3) clevel = 3; end;     
      TSS(counter) = struct('name',est(i).transcripts{j},'start',cur_start, 'stop', cur_stop,'confLevel',clevel,'strand',est(i).strand,'chr_num', est(i).chr_num, 'confirmed_genes', cur_confirmed_genes, 'ti', est(i).terminal_info(1,j));
      counter = counter + 1;
    end;

  end; % loop j
end; % loop i

if (debug) 
  fprintf(1,'Number of discarded clusters: \n')
  fprintf(1,'not in confirmed: %d\n', discarded_clusters_not_conf);
  fprintf(1,'no cDNA: %d\n', discarded_clusters_not_has_cDNA);
  fprintf(1,'too short: %d\n', discarded_clusters_too_short);
  fprintf(1,'Number of discarded ESTs\n because not included as EST in confirmed: %d\n  ', discarded_not_near_conf);
  fprintf(1,'because of upstream overlap: %d\n',discard_upstream_overlap);
  fprintf(1,'Number of detected ESTs: %d\n',length(TSS)); 
end;
  


%%%%%%%%%%%%%%%%%%%%%%%
% process putative ESTs
%%%%%%%%%%%%%%%%%%%%%%%

if (debug) fprintf(1, '\nProcessing putative TSSs...\n'); end;

% separate ESTs from the positive strand and negative strand
TSSPidx = find([TSS.strand]=='+');
TSSsizeP = length(TSSPidx);
TSSNidx = find([TSS.strand]=='-');
TSSsizeN = length(TSSNidx);

to_erase = [];

% process ESTs on positive strand
if TSSsizeP ~= 0
  m_idx = 0; % counts number of TSSs on positive strand
  for m = TSSPidx
    m_idx = m_idx + 1;
    if (debug) if (mod(m_idx,100)==0) fprintf(1,'%i\r',m_idx); end; end;
    cur_start = TSS(m).start;
    cur_stop = TSS(m).stop;
    tmp_idx = TSSPidx;
    tmp_idx(m_idx) = []; % do not consider current TSS

    upstream_match_500 = 0;
    upstream_match_1000 = 0;

    diff = cur_start - [TSS(tmp_idx).stop];
    if ~isempty(diff(diff>0 & diff<=500))
      upstream_match_500 = 1;
    end;
    if ~isempty(diff(diff>0 & diff<=1000))
      upstream_match_1000 = 1;
    end;

    if upstream_match_500 == 1
      discarded_dist500 = discarded_dist500 + 1;
      to_erase = [to_erase m];
      continue;
    end;

    % set confidence 3 to 2 if distance <= 1000 bp
    if upstream_match_1000 == 1
      TSS(m).confLevel = min(TSS(m).confLevel, 2);
    end;

  end; % loop m
end;

% process ESTs on negative strand
if TSSsizeN ~= 0
  m_idx = 0; % counts number of TSSs on negative strand
  for m = TSSNidx
    m_idx = m_idx + 1;
    if (debug) if (mod(m_idx,100)==0) fprintf(1,'%i\r',m_idx); end; end;
    cur_start = TSS(m).start;
    cur_stop = TSS(m).stop;
    tmp_idx = TSSNidx;
    tmp_idx(m_idx) = []; % do not consider current TSS

    upstream_match_500 = 0;
    upstream_match_1000 = 0;

    diff = [TSS(tmp_idx).start] - cur_stop;
    if ~isempty(diff(diff>0 & diff<=500))
      upstream_match_500 = 1;
    end;
    if ~isempty(diff(diff>0 & diff<=1000))
      upstream_match_1000 = 1;
    end;

    if upstream_match_500 == 1
      discarded_dist500 = discarded_dist500 + 1;
      to_erase = [to_erase m];
      continue;
    end;

    % set confidence 3 to 2 if distance <= 1000 bp
    if upstream_match_1000 == 1
      TSS(m).confLevel = min(TSS(m).confLevel, 2);
    end; 

  end; % loop m
end;


TSS(to_erase) = [];
if (debug) fprintf(1,'\nDone!\n'); end;


if (debug) fprintf(1,'\nNumber of discarded TSSs because of 500-upstream match: %d\n',discarded_dist500); end;
if (debug) fprintf(1,'Total number of putative TSSs : %d, p %d, n %d\n', length(TSS), sum(ismember([TSS.strand],'+')), sum(ismember([TSS.strand],'-'))); end;
if (debug) fprintf(1,' c3: %d, p %d, n %d\n', sum(ismember([TSS.confLevel],3)), sum(ismember([TSS.confLevel],3) & ismember([TSS.strand],'+')), sum(ismember([TSS.confLevel],3) & ismember([TSS.strand],'-'))); end;
if (debug) fprintf(1,' c2: %d, p %d, n %d\n', sum(ismember([TSS.confLevel],2)), sum(ismember([TSS.confLevel],2) & ismember([TSS.strand],'+')), sum(ismember([TSS.confLevel],2) & ismember([TSS.strand],'-'))); end;
if (debug) fprintf(1,' c1: %d, p %d, n %d\n', sum(ismember([TSS.confLevel],1)), sum(ismember([TSS.confLevel],1) & ismember([TSS.strand],'+')), sum(ismember([TSS.confLevel],1) & ismember([TSS.strand],'-'))); end;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% B.reducing clusters to a single position by picking the most likely position
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% separate ESTs from the positive strand and negative strand
RESULTP = TSS([TSS.strand] == '+' );
RESULTN = TSS([TSS.strand] == '-' );
clearedTSS = [];

% positive strand
if ~isempty(RESULTP)
  counter = 1;
  % take all the ends of the positive putative ESTs
  clear testVector sortVector idxV uniqueSortVector midx nidx histVector;
  testVector  = [RESULTP(:).start];
  [sortVector,idxV] = sort(testVector);
  [uniqueSortVector, midx, nidx] = unique(sortVector);
  % sort, find the unique ones, count their occurences
  histVector = hist(sortVector, unique(sortVector));

  l = length(uniqueSortVector);
  i = 1;
  while i <= l,
    nexti = i + 1;
    % this is the reference site
    refvalue = uniqueSortVector(i);
    refocc = histVector(i);
    % the index of the representing site 
    refi = i;
    TMP = RESULTP([RESULTP(:).start] == refvalue);
    if sum([TMP(:).ti] == 1) >= 1
      tiflag = 1;
    else
      tiflag = 0;
    end;
    % the index of the larger site with same number of occurrences
    refi2 = 0;

    for j=max(1,i-10):min(l,i+10)
      if j~= i,
       % compare with an alternative site 
       altvalue = uniqueSortVector(j);
       % compare all which are within a 21bp window centered at refvalue
       if abs(refvalue - altvalue) <= 10
         % does it have ti  == 1?
         TMP = RESULTP([RESULTP(:).start] == altvalue);
         tiflag = tiflag || (sum([TMP(:).ti] == 1) >= 1);
         nexti = max(nexti,j+1);
         if histVector(j) > refocc,
      	   % this will be the representative
	   refocc = histVector(j);
	   refi = j;
	   refi2 = 0;
         elseif histVector(j) == refocc,
           refi2 = j;
         end;
       end;
      end;
    end % loop j
  
    if ~tiflag
      i = nexti;
      continue;
    end; 
   
    if refi2 == 0,
       mostPV = RESULTP([RESULTP(:).start] == uniqueSortVector(refi));
       mostP = mostPV(1);
       mm = mostP.start;
    else
      % take the median of values between the first and last peak
      r1 = min(refi, refi2);
      r2 = max(refi,refi2);
      mostPV = [RESULTP([RESULTP(:).start] == uniqueSortVector(r1))  RESULTP([RESULTP(:).start] == uniqueSortVector(r2))]; 
      mostP = mostPV(1);
      sV = sortVector(sortVector >= uniqueSortVector(r1) & sortVector <= uniqueSortVector(r2));
      mm = floor(median(sV));
    end;

    if counter > 1 && mm == REPRP(counter-1).pos
      i = nexti;
      continue;
    end;
  
    REPRP(counter) = struct('name',mostP.name, 'pos', mm, 'confLevel', mostP.confLevel, ...
                            'strand', mostP.strand, 'chr_num', mostP.chr_num,'confirmed_genes', ...
                            mostP.confirmed_genes, 'start', mostP.start, 'stop', mostP.stop);

    counter = counter + 1;
    i = nexti;   
  end % loop i
  clearedTSS = REPRP;
end;

% negative strand
if ~isempty(RESULTN)
  counter = 1;
  clear testVector sortVector idxV uniqueSortVector midx nidx histVector;
  testVector  = [RESULTN(:).stop];
  [sortVector,idxV] = sort(testVector);
  [uniqueSortVector, midx, nidx] = unique(sortVector);
  histVector = hist(sortVector, unique(sortVector));

  l = length(uniqueSortVector);
  i = 1;
  while i <= l,
    nexti = i + 1;
    refvalue = uniqueSortVector(i);
    refocc = histVector(i);
    refi = i;
    TMP = RESULTN([RESULTN(:).stop] == refvalue);
    if sum([TMP(:).ti] == 1) >= 1
      tiflag = 1;
    else
      tiflag = 0;
    end;
    refi2 = 0;

    for j=max(1,i-10):min(l,i+10)
      if j~= i,
       altvalue = uniqueSortVector(j);
       if abs(refvalue - altvalue) <= 10,
         % does it have ti  == 1?
         TMP = RESULTN([RESULTN(:).stop] == altvalue);
         tiflag = tiflag || (sum([TMP(:).ti] == 1) >= 1);
         nexti = max(nexti, j+1);
         if histVector(j) > refocc,
    	   refocc = histVector(j);
  	   refi = j;
	   refi2 = 0;
         elseif histVector(j) == refocc,
           refi2 = j;
         end;
       end;
      end;
    end % loop j
  
    if ~tiflag
      i = nexti;
      continue;
    end;
   
    if refi2 == 0,
      mostNV = RESULTN([RESULTN(:).stop] == uniqueSortVector(refi));
      mostN = mostNV(1);
      mm = mostN.stop;
    else
      % take the median of values between the first and last peak
      r1 = min(refi, refi2);
      r2 = max(refi,refi2);
      mostNV = [RESULTN([RESULTN(:).stop] == uniqueSortVector(r1)) RESULTN([RESULTN(:).stop] == uniqueSortVector(r2))]; 
      mostN = mostNV(1);
      sV = sortVector(sortVector >= uniqueSortVector(r1) & sortVector <= uniqueSortVector(r2));
      mm = floor(median(sV));
    end;

    if counter > 1 && mm == REPRN(counter-1).pos
      i = nexti;
      continue;
    end;

    REPRN(counter) = struct('name',mostN.name, 'pos', mm, 'confLevel', mostN.confLevel, 'strand', mostN.strand, 'chr_num', mostN.chr_num,'confirmed_genes', mostN.confirmed_genes, 'start', mostN.start, 'stop', mostN.stop);  
     counter = counter + 1;
     i = nexti;
  end % loop i
  clearedTSS = [clearedTSS REPRN];
end;


if transsplice
  to_erase = [];
  for i=1:length(clearedTSS)
    if (sum(ismember(clearedTSS(i).pos-10:clearedTSS(i).pos+10,[transacc.pos])) > 0)
      discarded_transacc = discarded_transacc + 1;
      to_erase = [to_erase i];
    end;
  end;
  if (debug) fprintf(1,'\nNumber of discarded TSSs because of trans-splicing site: %d\n',discarded_transacc); end;
  clearedTSS(to_erase) =[];
end;


if (debug) fprintf(1,'\nTotal number of TSSs : %d, p %d, n %d\n', length(clearedTSS), sum(ismember([clearedTSS.strand],'+')), sum(ismember([clearedTSS.strand],'-'))); end;
if (debug) fprintf(1,' c3: %d, p %d, n %d\n', sum(ismember([clearedTSS.confLevel],3)), sum(ismember([clearedTSS.confLevel],3) & ismember([clearedTSS.strand],'+')), sum(ismember([clearedTSS.confLevel],3) & ismember([clearedTSS.strand],'-'))); end;
if (debug) fprintf(1,' c2: %d, p %d, n %d\n', sum(ismember([clearedTSS.confLevel],2)), sum(ismember([clearedTSS.confLevel],2) & ismember([clearedTSS.strand],'+')), sum(ismember([clearedTSS.confLevel],2) & ismember([clearedTSS.strand],'-'))); end;
if (debug) fprintf(1,' c1: %d, p %d, n %d\n\n', sum(ismember([clearedTSS.confLevel],1)), sum(ismember([clearedTSS.confLevel],1) & ismember([clearedTSS.strand],'+')), sum(ismember([clearedTSS.confLevel],1) & ismember([clearedTSS.strand],'-'))); end;

TSS =clearedTSS;
save(PAR.fn_out,'TSS') 
