function blocks_new = construct_merged_blocks(blocks,PAR)
%
% blocks_new = construct_merged_regions(blocks,PAR)
% 
% merging blocks such that there are PAR.regions.min_genes_per_block genes
% per block. Input blocks should contain only one gene per block. 
% blocks with unconfirmed or uncomplete genes are ignored. Only blocks
% with confirmed and complete genes are merged, such that the length of
% the intergenic region after upstream gene is similar to the length of
% the intergenic region before downstream gene
%
  
assert(length([blocks.gene_ids])==length(blocks));  
  
idx1 = find([blocks.gene_status]==PAR.info_genes.transcript_status.confirmed&...
           [blocks.gene_complete]==1);
idx2 = setdiff([1:length(blocks)],idx1)

fprintf('number of blocks with confirmed and complete genes: %i (%i)\n',length(idx1),length(blocks));
fprintf('merging blocks, such that %i confirmed and complete genes per region.\n',PAR.regions.min_genes_per_region)


unconf_blocks = blocks(idx2);
conf_blocks = blocks(idx1);

left = blocks(idx1).left_dist;
right = blocks(idx1).right_dist;

blocks_remain = 1:length(conf_blocks);

num_new_blocks=0;
while length(blocks_remain)>0
  for i=1:length(conf_blocks)
    assert(length(conf_blocks(i).gene_ids)==1)
    assert(length(conf_blocks(i).gene_status)==1)
    assert(length(conf_blocks(i).gene_complete)==1)
    
    blocks_remain = setdiff(blocks_remain,i);
    num_new_blocks = num_new_blocks+1;
    num=0;
    while num<PAR.blocks.min_genes_per_blocks&length(blocks_remain)>0
      num=num+1;
      idx = min(abs(left(blocks_remain)-right(i)));
      conf_blocks_new(num_new_blocks) = conf_blocks(i);
      conf_blocks_new(num_new_blocks).gene_ids = [conf_blocks_new(num_new_blocks).gene_ids conf_blocks(blocks_remain(idx)).gene_ids];
      conf_blocks_new(num_new_blocks).gene_status = [conf_blocks_new(num_new_blocks).gene_status conf_blocks(blocks_remain(idx)).gene_status];
      conf_blocks_new(num_new_blocks).gene_complete = [conf_blocks_new(num_new_blocks).gene_complete conf_blocks(blocks_remain(idx)).gene_complete];
      conf_blocks_new(num_new_blocks).start = [conf_blocks_new(num_new_blocks).starts conf_blocks(blocks_remain(idx)).start];
      switch strands(s)
       case '+',
        conf_blocks_new(num_new_blocks).offset = [conf_blocks_new(num_new_blocks).offset conf_blocks(blocks_remain(idx)).start - 1];
       case '-',
        conf_blocks_new(num_new_blocks).offset = [conf_blocks_new(num_new_blocks).offset conf_blocks(blocks_remain(idx)).stop + 1];
      end
      conf_blocks_new(num_new_blocks).stop = [conf_blocks_new(num_new_blocks).start conf_blocks(blocks_remain(idx)).stop];
      conf_blocks_new(num_new_blocks).chr = {conf_blocks_new(num_new_blocks).chr{:} conf_blocks(blocks_remain(idx)).chr};
      conf_blocks_new(num_new_blocks).chr_num = [conf_blocks_new(num_new_blocks).chr_num conf_blocks(blocks_remain(idx)).chr_num];
      conf_blocks_new(num_new_blocks).strand = [conf_blocks_new(num_new_blocks).strand conf_blocks(blocks_remain(idx)).strand];
      conf_blocks_new(num_new_blocks).paralogs = [conf_blocks_new(num_new_blocks).paralogs conf_blocks(blocks_remain(idx)).paralogs];
      blocks_remain = setdiff(blocks_remain,blocks_remain(idx));
    end
  end
end

blocks_new = [conf_blocks_new unconf_blocks];
for i=1:length(blocks_new)
  blocks_new(i).id=i;
  blocks_new(i).num_fragments=length(blocks_new(i).start);
end
blocks_new = rmfield(blocks_new,'left_dist');
blocks_new = rmfield(blocks_new,'right_dist');
  
fprintf('%i regions remaining with confirmed and complete genes: %i (%i)\n',length(conf_reg_new),length(reg_new));