
function [ cds_cands, map_transcr_to_genome, raw_cds_cands ] = find_ORFs_in_transcript( seq, start, transcript, patternsStart, patternsStop );


% cds_cands(:,1)    global ATG position
% cds_cands(:,2)    global stop postition
% cds_cands(:,3)    ORF length
% cds_cands(:,4)    length difference to longest alternative ORF
% cds_cands(:,5)    is ORF complete?
% cds_cands(:,6)    does ORF have upstream inframe ATG?
%
% raw_cds_cands(:,1)   ATG (local position of ATG)
% raw_cds_cands(:,2)   stop (local position of first inframe downstream stop codon)
% raw_cds_cands(:,3)   ORF start (local position of first inframe upstream stop codon)
% raw_cds_cands(:,4)   preTIS (local position of first upstream inframe start codon)


% --- splice transcript
stop = start + length(seq) - 1;
seq1 = upper( seq );
seq1 = mask_seq( seq1, start, start, transcript(1,1)-1 );
seq1 = mask_seq( seq1, start, transcript(end,2)+1, stop );
for( k = 2:size(transcript,1) )
  seq1 = mask_seq( seq1, start, transcript(k-1,2)+1, transcript(k,1)-1 );
end;
seq2 = seq1( seq1 ~= '-' );
map_transcr_to_genome = start - 1 + find( seq1 ~= '-' )';

% --- find all possible TIS and stop codons
hitsStart = find_patterns( seq2, patternsStart );
hitsStop = find_patterns( seq2, patternsStop );
nofStarts = size( hitsStart, 1 );
cds_cands = zeros( 0, 6 );
if( nofStarts == 0 )
  raw_cds_cands = zeros( 0, 4 );
  return;
end;

% --- find all candiate CDSes
raw_cds_cands = [ hitsStart , repmat(nan,nofStarts,3) ];
for( k = 1:nofStarts )
  p = hitsStart(k);
  q = find( hitsStop>p & mod(hitsStop-p,3)==0 );
  if( ~isempty(q) )
    raw_cds_cands(k,2) = min( hitsStop(q) );
    %%%keyboard;
  end;
  r = find( hitsStop<p & mod(hitsStop-p,3)==0 );
  if( ~isempty(r) )
    raw_cds_cands(k,3) = max( hitsStop(r) );
  end;
  s = find( hitsStart<p & mod(hitsStart-p,3)==0 );
  if( ~isempty(s) )
    raw_cds_cands(k,4) = max( hitsStart(s) );
  end;
end;

% --- sort candiate CDSes by length
[ lens, perm ] = sort( raw_cds_cands(:,2)-raw_cds_cands(:,1), 'descend' );
perm = perm( ~isnan( lens ) );
lens = lens( ~isnan( lens ) );
raw_cds_cands = raw_cds_cands( perm, : );
if( size(raw_cds_cands,1) == 0 )
  return;
end;

% --- process candiate CDSes
feat1 = map_transcr_to_genome(raw_cds_cands(:,1));    % ATG position
feat2 = map_transcr_to_genome(raw_cds_cands(:,2));    % stop postition
feat3 = raw_cds_cands(:,2) - raw_cds_cands(:,1);      % ORF length
if( length(feat3) > 1 )
  feat4 = feat3 - feat3(1);                           % diff. to alternative ORF length
  feat4(1) = feat3(1) - feat3(2);
else
  feat4 = feat3(1);
end;
feat5 = ~isnan(raw_cds_cands(:,3));                   % is ORF complete?
feat6 = ~isnan(raw_cds_cands(:,4));                   % has ORF upstream ATG?
cds_cands = [ feat1 feat2 feat3 feat4 feat5 feat6 ];


