function [train_blocks,test_blocks,blocks] = intersect_regions(blocks,PAR,test,train)
%  train_idx = intersect_regions(blocks,train,test)
  
offset = PAR.regions.offset;
  
for i=1:length(blocks)
  assert(blocks(i).num_fragments==1)
  blocks(i).old_id = blocks(i).id;
end


% ------------
% collect training and test blocks
% ------------

test_ids = [];
if exist('train')
  fprintf('intersecting %i blocks with %i training regions\n',length(blocks),length(train));
  %   keyboard
  [blocks,train_ids] = int_reg(blocks,train); 
  train_blocks = blocks(train_ids) ;
  test_ids = setdiff([1:length(blocks)],train_ids);
  if isempty(test)
    fprintf('no test regions specified, using all other blocks for testing\n');
    test_blocks = blocks(test_ids);
  else
    fprintf('intersecting %i remaining blocks with %i test regions\n',length(blocks),length(test));
    blocks = blocks(test_ids);
    [blocks,test_ids] = int_reg(blocks,test);
    test_blocks = blocks(test_ids);
    blocks = [train_blocks blocks];
  end
elseif ~isempty(test)
  fprintf(['no training regions specified, using all blocks except ' ...
          'test regions for training \n']);
  [blocks,test_ids] = int_reg(blocks,test);
  test_blocks = blocks(test_ids);
  train_ids = setdiff([1:length(blocks)],test_ids);
  train_blocks = blocks(train_ids) ;
else
  fprintf('no training or test regions specified, using everything for training \n');
  train_blocks = blocks;
  test_blocks = [];
end

rm_blocks =[];
% for j=1:length(train_blocks)
%   if train_blocks(j).stop-train_blocks(j).start<500;
%     rm_blocks =[rm_blocks j];
%   end
% end
fprintf('removing %i training blocks which are smaller than offset\n',length(rm_blocks));
train_blocks(rm_blocks)=[];

train_blocks = sort_regions(train_blocks) ;

rm_blocks =[];
% for j=1:length(test_blocks)
%   if test_blocks(j).stop-test_blocks(j).start<500;
%     rm_blocks =[rm_blocks j];
%   end
% end
% fprintf('removing %i test blocks which are smaller than offset\n',length(rm_blocks));

if ~isempty(rm_blocks)
  keyboard
end
test_blocks(rm_blocks)=[];
% fprintf('removing %i test blocks which are smaller than offset\n',length(rm_blocks));

test_blocks = sort_regions(test_blocks) ;


fprintf('number of blocks used for training: %i (%i)\n',length(train_blocks),length(blocks));
fprintf('number of blocks used for testing: %i (%i)\n',length(test_blocks),length(blocks));
%-------------------------------


function [blocks,block_ids] = int_reg(blocks,regions)
  
missed=0;
cut_blocks=0;

last_block = length(blocks);
blocks(max(length(blocks)*3,length(regions))).start = []; 
block_ids = zeros(1,length(blocks));
last_id = 0;
for idx_reg = 1:length(regions)
  fprintf(' %i\r',idx_reg)
  strand = regions(idx_reg).strand;
  chrom = regions(idx_reg).chr_num;
  start = regions(idx_reg).start;
  stop = regions(idx_reg).stop;
  % find blocks that overlap region
  idx_temp = find([blocks.strand]==strand & [blocks.chr_num]==chrom & ...
                  [blocks.start]<=stop & [blocks.stop]>=start);
 
  if ~isempty(idx_temp)
    assert(all(~ismember(idx_temp,block_ids)));
    block_ids(last_id +[1:length(idx_temp)]) = idx_temp;
    last_id = last_id + length(idx_temp);
    left_overlap = [blocks(idx_temp).start]<start ;
    right_overlap = [blocks(idx_temp).stop]>stop ; 
    for j=1:length(idx_temp)
      block = blocks(idx_temp(j));
      if left_overlap(j) 
        last_block = last_block+1;
        blocks(last_block) = block;
        blocks(last_block).stop = start -1;
        blocks(last_block).right_dist = - 1;
        block.left_dist = block.left_dist - (block.start - start);
        block.start = start;
        cut_blocks = cut_blocks+1;
      end
      if right_overlap(j) 
        last_block = last_block+1;
        blocks(last_block) = block;
        blocks(last_block).start = stop +1;
        blocks(last_block).left_dist = - 1;
        block.right_dist = block.right_dist -(stop - block.stop);
        block.stop = stop;
        cut_blocks = cut_blocks+1;
      end
      blocks(idx_temp(j)) = block;
    end
  elseif regions(idx_reg).start==0&regions(idx_reg).stop==0
    warning('region empty')
  else
    missed = missed+1;
  end
end %%loop over regions
blocks = blocks(1:last_block);
block_ids = block_ids(1:last_id);
for i=1:length(blocks)
   switch blocks(i).strand
    case '+',
     blocks(i).offset = blocks(i).start - 1;
    case '-',
     blocks(i).offset = blocks(i).stop + 1;
   end
end
if cut_blocks>0|missed>0
  % keyboard
end
  
fprintf('number of blocks cut by regions: %i\n',cut_blocks);
fprintf('number of regions without block overlap (should be zero): %i\n',missed);


