function genes = find_maxorf_cds_boundaries(genes, PAR, allpaths)

if nargin<3
  allpaths = 0;
end

if ~isfield(genes,'seq')|| isempty(genes.seq)
  genes(1).config = PAR.FN.genome.fn_genome_config;
  genes = load_sequence(genes);
end

if ~allpaths
  for j=1:length(genes)
    for k=1:length(genes(j).transcripts)
      %[len tis cdsStop] = find_tis_and_stop(genes(j),paths{k},genes(j).seq);
      path = 1:size(genes(j).exons{k},1);
      [len tis cdsStop] = find_tis_and_stop(genes(j),path,genes(j).seq,genes(j).exons{k}');
      if len==-1,  % no orf found
        genes(j).tis(k)=[0];
        genes(j).cdsStop(k)=[0];
        continue;
      end
      assert(ismember(genes(j).seq(tis+(0:2)),PAR.Signals.tis.consensus))
      assert(ismember(genes(j).seq(cdsStop+(0:2)),PAR.Signals.cdsStop.consensus))
      genes(j).tis(k) = global_co(genes(j),tis);
      genes(j).cdsStop(k) = global_co(genes(j),cdsStop); 
    end
  end
else
  for j=1:length(genes)
    %find all paths through the splicegraph
    paths=find_all_paths(genes(j).splicegraph{2});
    maxlen = 0;
    for k=1:length(paths)
      [len tis cdsStop] = find_tis_and_stop(genes(j),paths{k},genes(j).seq, genes(j).splicegraph{1});
      if len==-1, continue; end % no orf found
      assert(ismember(genes(j).seq(tis+(0:2)),PAR.Signals.tis.consensus))
      assert(ismember(genes(j).seq(cdsStop+(0:2)),PAR.Signals.cdsStop.consensus))
      if len>maxlen
        maxlen = len;
        genes(j).maxorf_tis = global_co(genes(j),tis);
        genes(j).maxorf_cdsStop = global_co(genes(j),cdsStop);
      end
    end
  end
  if maxlen==0;
    genes(j).maxorf_tis=[];
    genes(j).maxorf_cdsStop=[];
  end
end
return


function pos = global_co(gene,pos)
  if gene.strand=='+'
    pos = pos+gene.start-1;
  else
    pos = gene.stop-pos+1;
  end
return
