function genes = merge_transcripts_by_colocation(genes,genome_info)

% genes = merge_transcripts_by_colocation(genes,genome_info)
% reconsider colocation

if length(genes)==1
  genes(1).gene_merged=[] ;
  return
end

for i=1:length(genes) ;
  genes(i).gene_merged =0 ;
  if isempty(genes(i).cds_exons)
    genes(i).cds_exons={[]};
  end 
  if isempty(genes(i).utr5_exons)
    genes(i).utr5_exons={[]};
  end 
  if isempty(genes(i).utr3_exons)
    genes(i).utr3_exons={[]};
  end 
end

cnt=0;
while (1)
  CHR = [genes.chr_num]; 
  STR = [genes.strand]; 
  chr = unique(CHR);
  str = unique(STR);
  
  chr_maps = {} ;
  for c=chr
    d = dir(genome_info.flat_fnames{c}) ;
    chr_maps{1,c} = uint32(zeros(1,d(1).bytes)) ;
    chr_maps{2,c} = uint32(zeros(1,d(1).bytes)) ;
  end ;
  
  mistakes = [] ; take_map = zeros(1,length(genes)) ;
  
  for i=1:length(genes) ;
    start=inf; stop=0 ;
    for j=1:length(genes(i).exons)
      if start>min(genes(i).exons{j}(:,1:2)), start=min(min(genes(i).exons{j}(:,1:2))) ; end ;
      if stop<max(genes(i).exons{j}(:,1:2)), stop=max(max(genes(i).exons{j}(:,1:2))) ; end ;
    end ;
    assert(~isinf(start) & stop~=0) ;
    genes(i).start=start ;
    genes(i).stop=stop ;
  end
  
  for c = chr
    for s = 1:length(str)
      fprintf(1,'chromosome %i%s\r',c,str(s))
      gene_idx = find(CHR==c & STR==str(s)) ;
      for i = gene_idx  ;
        gene_start = genes(i).start ;
        gene_end = genes(i).stop ;
        if gene_end>length(chr_maps{s,c}), 
          chr_maps{s,c}(gene_end)=0 ;
        end 
        if ~all(chr_maps{s,c}(gene_start:gene_end)==0)
          a = unique(chr_maps{s,c}(gene_start:gene_end)) ; a(a==0)=[] ;
          mistakes = [mistakes; i a(end)] ;
          chr_maps{s,c}(gene_start:gene_end) = a(end) ;
        
          genes(a(end)).transcripts = {genes(a(end)).transcripts{:} genes(i).transcripts{:}} ;
          genes(a(end)).transcript_info = [genes(a(end)).transcript_info genes(i).transcript_info] ;
          genes(a(end)).transcript_status = [genes(a(end)).transcript_status genes(i).transcript_status] ;
          genes(a(end)).transcript_valid = [genes(a(end)).transcript_valid genes(i).transcript_valid] ;
          % genes(a(end)).terminal_info = [genes(a(end)).terminal_info genes(i).terminal_info] ;
          genes(a(end)).exons = {genes(a(end)).exons{:} genes(i).exons{:}} ;
          genes(a(end)).exons_confirmed = {genes(a(end)).exons_confirmed{:} genes(i).exons_confirmed{:}} ;
          
          genes(a(end)).cds_exons = {genes(a(end)).cds_exons{:} genes(i).cds_exons{:}} ;
          genes(a(end)).utr5_exons = {genes(a(end)).utr5_exons{:} genes(i).utr5_exons{:}} ;
          genes(a(end)).utr3_exons = {genes(a(end)).utr3_exons{:} genes(i).utr3_exons{:}} ;
          
          genes(a(end)).tis = [genes(a(end)).tis genes(i).tis] ;
          genes(a(end)).tis_info = [genes(a(end)).tis_info genes(i).tis_info] ;
          genes(a(end)).tis_conf = [genes(a(end)).tis_conf genes(i).tis_conf] ;
          genes(a(end)).cdsStop = [genes(a(end)).cdsStop genes(i).cdsStop] ;
          genes(a(end)).cdsStop_info = [genes(a(end)).cdsStop_info genes(i).cdsStop_info] ;
          genes(a(end)).cdsStop_conf = [genes(a(end)).cdsStop_conf genes(i).cdsStop_conf] ;
          genes(a(end)).tss = [genes(a(end)).tss genes(i).tss] ;
          genes(a(end)).tss_info = [genes(a(end)).tss_info genes(i).tss_info] ;
          genes(a(end)).tss_conf = [genes(a(end)).tss_conf genes(i).tss_conf] ;
          genes(a(end)).cleave = [genes(a(end)).cleave genes(i).cleave] ;
          genes(a(end)).cleave_info = [genes(a(end)).cleave_info genes(i).cleave_info] ;
          genes(a(end)).cleave_conf = [genes(a(end)).cleave_info genes(i).cleave_conf] ;
          genes(a(end)).polya = [genes(a(end)).polya genes(i).polya] ;
          genes(a(end)).polya_info = [genes(a(end)).polya_info genes(i).polya_info] ;
          genes(a(end)).polya_conf = [genes(a(end)).polya_conf genes(i).polya_conf] ;
          genes(a(end)).anno_id = [genes(a(end)).anno_id genes(i).anno_id] ;
          genes(a(end)).confgenes_id = [genes(a(end)).confgenes_id genes(i).confgenes_id] ;
          genes(a(end)).paralogs = [genes(a(end)).paralogs genes(i).paralogs] ;
          if ~isequal(genes(a(end)).name,genes(i).name) 
            genes(a(end)).name2{end+1} = genes(i).name;
            cnt=cnt+1;
          end
          genes(a(end)).gene_merged = 1;
        else
          take_map(i) = 1 ;
          chr_maps{s,c}(gene_start:gene_end) = i ;
        end 
      end
    end
  end
  genes = genes(find(take_map)) ;
  fprintf(1,'reduced by %d transcripts, %d genes remaining\n',length(mistakes),length(genes));
  if (length(mistakes)==0), break ; end ;
end
 
for i=1:length(genes)
  assert(all(size(genes(i).transcripts)==size(genes(i).transcript_info)))
  assert(all(size(genes(i).transcripts)==size(genes(i).transcript_valid)))
  assert(all(size(genes(i).transcripts)==size(genes(i).exons)))
  assert(all(size(genes(i).transcripts)==size(genes(i).exons_confirmed)))
end
