function [TSS,CLEAVE,genes]=parse_fulllength_cDNA(PAR)
% [TSS,CLEAVE,genes]=parse_fulllength_cDNA(PAR)
  

genome_info =  init_genome(PAR.FN.input.fn_genome_config) ;

full_length = genome_info.flcdna_fnames ;

if isempty(full_length)
  error('no full length cDNA available')
end

cDNA_names = cell(30000,1);cnt=0;
for f = full_length 
  fd = fopen(f{1}, 'r') ;
  while ~feof(fd),
    line = fgetl(fd) ;
    if isempty(line) |line(1)~='>'
      continue
    end
    cnt = cnt+1;
    elems = separate(line,' ' ) ;
    cDNA_names{cnt} = elems{1}(2:end);
  end
  fclose(fd);
end 
cDNA_names = cDNA_names(1:cnt);
fprintf('number of cDNAs %i\n',length(cDNA_names) )


% load confirmed_genes
genes = load_struct(PAR.FN.output.fn_genes_conf ,'genes');
for i=1:length(genes)
  genes(i).strand = genes(i).strands(1);
  genes(i).chr_num = strmatch(genes(i).chr,genome_info.contig_names,'exact') ;
end


TSS(length(cDNA_names)) = struct('name',[],'strand',[],'chr_num',[],'pos',[],'confLevel',[],'confirmed_genes',[])
CLEAVE = TSS;
cnt = 0; cnt_tss = 0;
cnt_cleave = 0;
cleave_terminal_infos =[];
tss_terminal_infos =[];
for g = 1:length(genes)
  fprintf(1,'%i\r',g)
  idx_cdna = find(genes(g).is_cDNA == PAR.info_genes.transcript.cDNA);
  for t=idx_cdna 
    name_end = max(strfind(genes(g).transcripts{t},'|'));
    full_length = strfind(cDNA_names, genes(g).transcripts{t}(4:name_end));
    full_length = find(~cellfun('isempty',full_length ));
    
    if length(full_length)==1
      % keyboard
      genes(g).is_cDNA(t) = PAR.info_genes.transcript.fulllength;
      start = min(genes(g).exons{t}(:));
      stop = max(genes(g).exons{t}(:));
      tss_temp = []; cleave_temp = [];
      if genes(g).terminal_info(1,t)~=-1 %==0
        tss_temp.name =  cDNA_names(full_length);
        tss_temp.strand =  genes(g).strand;
        tss_temp.chr_num =  genes(g).chr_num;
        tss_temp.pos = start;
        tss_temp.confLevel = genes(g).terminal_info(1,t);
        tss_temp.confirmed_genes = g;
      elseif genes(g).terminal_info(1,t)==-1
         continue
      else
        keyboard
      end
      if genes(g).terminal_info(2,t)~=-1 %==0
        cleave_temp.name =  cDNA_names(full_length) ;
        cleave_temp.strand =  genes(g).strand;
        cleave_temp.chr_num = genes(g).chr_num ;
        cleave_temp.pos = stop;
        cleave_temp.confLevel = genes(g).terminal_info(2,t);
        cleave_temp.confirmed_genes = g;
      elseif genes(g).terminal_info(2,t)==-1
        continue
      else
        keyboard
      end
      if genes(g).strand=='+'
        if ~isempty(tss_temp)
          cnt_tss = cnt_tss+1;
          TSS(cnt_tss) = tss_temp;
          tss_terminal_infos =[tss_terminal_infos tss_temp.confLevel];
        end
        if ~isempty(cleave_temp)
          cnt_cleave = cnt_cleave +1;
          CLEAVE(cnt_cleave) = cleave_temp;
          cleave_terminal_infos = [cleave_terminal_infos cleave_temp.confLevel ];
        end
      else
        if ~isempty(cleave_temp)
          cnt_tss = cnt_tss+1;
          TSS(cnt_tss) = cleave_temp;
          tss_terminal_infos =[tss_terminal_infos cleave_temp.confLevel];
        end
        if ~isempty(tss_temp)
          cnt_cleave = cnt_cleave+1;
          CLEAVE(cnt_cleave) = tss_temp ;
          cleave_terminal_infos =[tss_terminal_infos tss_temp.confLevel];
        end
      end
      cnt=cnt+1;
    elseif length(full_length)>1
      keyboard
    end
  end
end
CLEAVE = CLEAVE(1:cnt_cleave);
TSS = TSS(1:cnt_tss);
fprintf('number of cDNAs mapped to genes: %i (of %i)\n',cnt,length(cDNA_names) )

x = find(PAR.FN.input_sig.tss.fn_fulllength=='/');
dir_ = PAR.FN.input_sig.tss.fn_fulllength(1:x(end));
if ~isdir(dir_)
  nice_mkdir(dir_)
end
x = find(PAR.FN.input_sig.cleave.fn_fulllength=='/');
dir_ = PAR.FN.input_sig.cleave.fn_fulllength(1:x(end));
if ~isdir(dir_)
  nice_mkdir(dir_)
end

save(PAR.FN.input_sig.tss.fn_fulllength,'TSS','PAR');
save(PAR.FN.input_sig.cleave.fn_fulllength,'CLEAVE','PAR');
