function POLYALIST = create_polya(POLYAPAR)
  addpath polya_utils/ ;
  addpath ~/svn/tools/genomes/ ;
  
  %%%%%%%%%%%
  logfile = sprintf('%screatepolya.log',POLYAPAR.logdir);
  
  fidlog = fopen(logfile,'wt');

  genome_config = init_genome(POLYAPAR.fn_genome_config);
  
  %%%%%%%%%%%%
  wl = POLYAPAR.wl;
  POSconstr = POLYAPAR.POSconstr;  
  cleave = POLYAPAR.cleavedata;
  POLYACONSENSUS = POLYAPAR.consensus;

  %%%%%%%%%%%%%%


  coverageVector=  zeros(1,length(POLYACONSENSUS));
  firstcoverageVector=  zeros(1,length(POLYACONSENSUS));
  
  % counts occurence of sixmers around which we will center 
  fprintf(fidlog, '\nNumber of consensus sixmers : %d\n', length(POLYACONSENSUS));

  counter = 1;
  seccounter = 1;
  polyacounter = 1;
  notfoundcounter = 0;

  for i=1:length(cleave) 
    cleaveposition = cleave(i).pos;
    if strcmp(cleave(i).strand, '+')
      strand = 1;
    else 
      strand = 0;
    end;
    
    if isfield(cleave(i),'confirmed_genes')&&isempty(cleave(i).confirmed_genes)
      continue;
    end;
    if isfield(cleave(i),'anno_genes')&&isempty(cleave(i).anno_genes)
      continue;
    end;
    
    % cut out the genes sequence and find the "best matching" polyA
    % sequence on it
    bs = cleaveposition-50*wl;
    es = cleaveposition+50*wl;
    
    clear sequcompl;
    sequcompl = load_genomic(genome_config.contig_names{cleave(i).chr_num}, cleave(i).strand, bs, es, genome_config);
    if es < 5,
      continue;
    end; 
    
    sixmerrelVector = [];
    notfound = 1;
    numberfound = 0;

    % if POSconstr position is contrained to 45 downstream of cleave, 20 upstream of cleave
    if POSconstr
      L1idx = 50*wl-45;
      L2idx = 50*wl+20;
    else
      L1idx = 1;
      L2idx = 50*2*wl-4;
    end;
    
    position_index_string = [];
    for ii=1:length(POLYACONSENSUS)
      % get consensus position (index in sequcompl, for right statistics for position) in the chosen constrained window
      position_index_string = [position_index_string findstr(sequcompl(L1idx:L2idx+5), POLYACONSENSUS{ii})+L1idx-1];
    end;
    if ~isempty(position_index_string)
      notfound = 0;
    else
      notfoundcounter = notfoundcounter +1;
      continue;
    end;
    numberfound =  length(position_index_string);
    for pi = 1:numberfound
      sixmerrelVector(pi) = mynormpdf(position_index_string(pi),POLYAPAR.distribution.posmutotal, ...
                                      POLYAPAR.distribution.possigmatotal);
    end;
    
    
    [ms,idxs] = sort(sixmerrelVector, 'descend');
    
    for k=1:numberfound
      idx = position_index_string(idxs(k));
      sequm = sequcompl(idx:idx+5);
    
      
      iiidx = 0;
      for ii=1:length(POLYACONSENSUS)
        % consider this sequence 
        if strcmp(sequm,POLYACONSENSUS{ii})
          iiidx = ii;
        end;
      end;
      assert(iiidx ~= 0);
  
      if strand == 1
        beginsx = bs+idx-1;
        beginloadgen = bs+idx-1;
      else
        beginsx = es -idx+1; % for Gabi, convention is beginning of signal on negative strand
        beginloadgen = es -idx+1-5; % for load_genomic, convention is beginning of signal on positive strand
      end;
				
      POLYALIST(polyacounter).name = cleave(i).name;
      POLYALIST(polyacounter).pos = beginsx;
      POLYALIST(polyacounter).strand = cleave(i).strand;
      POLYALIST(polyacounter).chr_num = cleave(i).chr_num;
      if isfield(cleave(i),'confirmed_genes')
        POLYALIST(polyacounter).confirmed_genes = cleave(i).confirmed_genes;
      end
      if isfield(cleave(i),'anno_genes')
        POLYALIST(polyacounter).anno_genes = cleave(i).anno_genes;
      end
      POLYALIST(polyacounter).consensus = load_genomic(genome_config.contig_names{cleave(i).chr_num}, ...
                                                    cleave(i).strand,  beginloadgen, beginloadgen+5, genome_config);
      iiidx = 0;
      for ii=1:length(POLYACONSENSUS)
        if strcmp(POLYALIST(polyacounter).consensus,POLYACONSENSUS{ii})
          iiidx = ii;
        end;
      end;
      assert(iiidx ~= 0);
        
      POLYALIST(polyacounter).cleaveConfLevel = cleave(i).confLevel;
    
      if k==1
        POLYALIST(polyacounter).confLevel = (cleave(i).confLevel+1)*100 + 1;
        firstcoverageVector(iiidx) = firstcoverageVector(iiidx) +1;
      else
        POLYALIST(polyacounter).confLevel = (cleave(i).confLevel+1)*100 + 0;
        seccounter = seccounter + 1;
      end;
    
      polyacounter = polyacounter+1;
      coverageVector(iiidx) = coverageVector(iiidx) +1;
  
    end; % for k 
  
    counter = counter + 1;
   
  end; % for i

  if counter ~= 0
    counter = counter -1;
    seccounter = seccounter - 1;
    polyacounter = polyacounter - 1;

    fprintf(fidlog, '\n Test coverageVector: sum(coverageVector) = %d, counter = %d\n', sum(coverageVector),polyacounter);
    fprintf(fidlog, '\n Test firstcoverageVector: sum(firstcoverageVector) = %d, counter = %d\n', sum(firstcoverageVector),counter);
    
    coverageVector = coverageVector *100/ polyacounter;   
    firstcoverageVector = firstcoverageVector *100/ counter;   
    
    % coverage for all putative polyas
    [coverageVectorSort, idxV] = sort(coverageVector);
    cs = cumsum(coverageVectorSort(length(coverageVector):-1:1));
    cs = cs(length(coverageVector):-1:1);
    fprintf(fidlog,'\n Coverage of sixmers around which we centered\n');
    for i=1:length(coverageVector)
      fprintf(fidlog, 'Sixmer %s, coverage %2.f, cumulative coverage %2.f \n', POLYACONSENSUS{idxV(length(coverageVector)+1-i)}, coverageVector(idxV(length(coverageVector)+1-i)),  cs(length(coverageVector)+1-i));
    end;
    
    % coverage only for the best fitting PAS (confLevel = 1)
    [firstcoverageVectorSort, idxV] = sort(firstcoverageVector);
    cs = cumsum(firstcoverageVectorSort(length(firstcoverageVector):-1:1));
    cs = cs(length(firstcoverageVector):-1:1);
    fprintf(fidlog,'\n Coverage of best fitting sixmers around which we centered\n');
    for i=1:length(firstcoverageVector)
      fprintf(fidlog, 'Sixmer %s, coverage %2.f, cumulative coverage %2.f \n', POLYACONSENSUS{idxV(length(firstcoverageVector)+1-i)}, firstcoverageVector(idxV(length(firstcoverageVector)+1-i)),  cs(length(firstcoverageVector)+1-i));
    end;
    

  end; % if counter == 0

  return;
  
