function [xmer, absobserved, absexpected, evalue] ...
      = polya_evalue(BGXMER, FGXMER, numXmer, bpdiff);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This script goes through the array of 6mers sequences BGXMER  and
% counts the histogramms, computes  a
% background model for these sixmers, and then the evalues for the 
% xmers in FGXMER relative to this background model.
% It returns the numXmer most significant 6mer which is a variant of AATAAA
% (with up to bpdiff differences)
% together with useful info on it.
% Petra Philips, June 2006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

numxmers = 4^6;

SIGNIFCDF = -ones(1,numxmers);
BACKGMODEL = zeros(1,numxmers);

  % build histogramm of xmers for the background  and foreground
  % model 
  
 % count occurrences in both models 
 BMabs = countXmers(BGXMER,6);
 FMabs = countXmers(FGXMER,6);
 
 BACKGMODEL = BMabs / numel(BGXMER);
 for i=2:numxmers
   % compute significance only for variants of AATAAA which were observed
   if (sum(int2acgt(i-1,6) == 'aataaa') >= (6-bpdiff)) && BMabs(i) > 0 
       SIGNIFCDF(i) = myevalue(FMabs(i), numel(FGXMER), ...
			    BACKGMODEL(i));
   end;
 end;

[sx,idx] = sort(SIGNIFCDF);
idxstart = sum(sx == -1);
% return numXmer most significant
for j=1:min(numXmer,numxmers-idxstart)
  xmer(j) = idx(idxstart+j)-1;
  evalue(j) = SIGNIFCDF(xmer(j)+1);
  absobserved(j) = sum(sum(FGXMER == xmer(j)));
  absexpected(j) = numel(FGXMER) * BACKGMODEL(xmer(j)+1);
end;

return;
  
