function genes=prepare_anno(genes_anno,PAR)
  
fprintf('\n\nPREPARE GENES FROM ANNOTATION\n\n')

genome_info=init_genome(PAR.FN.genome.fn_genome_config);

%%renmove genes from contigs that are not in genome_config (e.g.mitochondrial and pastide sequences)
del_idx =[];
for i=1:length(genes_anno)
  idx=find(ismember(upper(genome_info.contig_names),upper(genes_anno(i).chr))) ;
  if isempty(idx)
    del_idx = [del_idx i];
  end
end
if length(del_idx)==length(genes_anno)
  warning('different contig names in annotated genes vs genome_info')
  del_idx =[];
  genes_anno = replace_contignames(genes_anno,PAR.FN.genome.fn_name_to_contig_mapping);
  for i=1:length(genes_anno)
    idx=find(strcmp(upper(genome_info.contig_names),upper(genes_anno(i).chr)));
    if isempty(idx)
      del_idx = [del_idx i];
    end
  end
end
assert( length(del_idx)<length(genes_anno))
fprintf('\n removing %i genes,from contigs that are not in genome_info,(e.g. mitochondrial and pastide sequence',length(del_idx))
genes_anno(del_idx) = [];
fprintf('\n %i genes remaining\n',length(genes_anno));

for g_idx=1:length(genes_anno)
  genes_anno(g_idx).id = g_idx;
  
  genes_anno(g_idx).anno_id = genes_anno(g_idx).id ;
  if ~isfield(genes_anno(g_idx),'chr_num')|isempty(genes_anno(g_idx).chr_num)
    genes_anno(g_idx).chr_num =  strmatch(genes_anno(g_idx).chr,genome_info.contig_names,'exact') ;
  end
  %% add dummy exon if exons empty.
  if isempty(genes_anno(g_idx).exons)
     keyboard
  %  genes_anno(g_idx).exons{1}=[genes_anno(g_idx).start genes_anno(g_idx).stop 3];
  end
  
  if min(min(genes_anno(g_idx).exons{1}))<1
    warning('found position < 1')
    genes_anno(g_idx).exons{1}(genes_anno(g_idx).exons{1}<1)=1;
  end
  
  %% change cdsStop position to start of consensus
  %%is already done in process_gff3_genes
  if 0
    if ~isempty(genes_anno(g_idx).cdsStop)&genes_anno(g_idx).strand=='+'
      genes_anno(g_idx).cdsStop = genes_anno(g_idx).cdsStop-3;
    elseif genes_anno(g_idx).strand=='-'
      genes_anno(g_idx).cdsStop = genes_anno(g_idx).cdsStop+3;
    end
  end
  genes_anno(g_idx).cdsStop_info =  ones(size(genes_anno(g_idx).cdsStop))*PAR.info_genes.cdsStop.anno;
  genes_anno(g_idx).tis_info =  ones(size(genes_anno(g_idx).tis))*PAR.info_genes.tis.anno;
  genes_anno(g_idx).cleave_info =  ones(size(genes_anno(g_idx).cleave))*PAR.info_genes.cleave.anno;
  genes_anno(g_idx).tss_info =  ones(size(genes_anno(g_idx).tss))*PAR.info_genes.tss.anno;

  for j=1:length(genes_anno(g_idx).transcripts)
    %% exons_confirmed(:,1) tells you the evidence for each exon and
    %% exons_confirmed(:,2)  evidence for intron (in case of confirmed
    %sequences this can be >1 as several ESTs might be merged; for
    %annotation it is always 1; exons_confirmed(end,2) must always be 0
    %as there is no intron after last exon.)
    % keyboard
    genes_anno(g_idx).exons_confirmed{j} = ones(size(genes_anno(g_idx).exons{j})).*genes_anno(g_idx).transcript_status(j);
    genes_anno(g_idx).exons_confirmed{j}(end,2) = 0;
    genes_anno(g_idx).transcript_info(j) = PAR.info_genes.transcript.anno;
    
    %% if transcript_status is not given in the annotation I assume
    %lowest status; (predicted)
    
    if isempty(genes_anno(g_idx).transcript_status)|length(genes_anno(g_idx).transcript_status)<j
      genes_anno(g_idx).transcript_status(j) = PAR.info_genes.transcript_status.predicted;
    end 
    if genes_anno(g_idx).strand=='+'
      assert(genes_anno(g_idx).start-min(min(genes_anno(g_idx).exons{j}(:,1:2)))<=0)
      if genes_anno(g_idx).stop-max(max(genes_anno(g_idx).exons{j}(:,1:2)))<0
        genes_anno(g_idx).stop = genes_anno(g_idx).stop+1;
      end
      assert(genes_anno(g_idx).stop-max(max(genes_anno(g_idx).exons{j}(:,1:2)))>=0)
      genes_anno(g_idx).cleave(j) = max(max(genes_anno(g_idx).exons{j}(:,1:2)));
      genes_anno(g_idx).tss(j) = min(min(genes_anno(g_idx).exons{j}(:,1:2)));
    else
      assert(genes_anno(g_idx).stop-max(max(genes_anno(g_idx).exons{j}(:,1:2)))>=0)
      if genes_anno(g_idx).start-min(min(genes_anno(g_idx).exons{j}(:,1:2)))>0
        genes_anno(g_idx).start = genes_anno(g_idx).start-1;
      end
      assert(genes_anno(g_idx).start-max(max(genes_anno(g_idx).exons{j}(:,1:2)))<=0)
      genes_anno(g_idx).cleave(j) =  min(min(genes_anno(g_idx).exons{j}(:,1:2)));
      genes_anno(g_idx).tss(j) =  max(max(genes_anno(g_idx).exons{j}(:,1:2)));
    end
    transcript_status = genes_anno(g_idx).transcript_status(j);
    genes_anno(g_idx).cdsStop_conf(j) = transcript_status;
    genes_anno(g_idx).tis_conf(j)  =  transcript_status ;
    genes_anno(g_idx).cleave_conf(j)  = transcript_status; 
    genes_anno(g_idx).tss_conf(j)  =  transcript_status ;
  end
end

genes = init_genes(1,PAR); 
genes  = convert_struct(genes,genes_anno) ;


which_checks.exons_sorted = 1;
which_checks.intron_length = 1;
which_checks.splicesites = 1;
which_checks.orf = 1;
which_checks.gene_length = 1;
which_checks.graph = 1;
which_checks.complete = 1;

% keyboard
genes = check_genes(genes, genome_info,which_checks) ;