function genes = prepare_confirmed(confirmed_sequences,PAR)
% genes = prepare_confirmed(confirmed_sequences,PAR)
%
% PAR.FN.input.fn_genome_config
% PAR.info_genes
% PAR.organism
  

fprintf('\n\nPREPARE CONFIRMED SEQUENCES\n\n')

genome_info = init_genome(PAR.FN.input.fn_genome_config);
for g_idx = 1:length(confirmed_sequences)
  fprintf(1,'%i\r',g_idx);
  
  gene = confirmed_sequences(g_idx);
  
  %%% fix paralogs
  if isfield(gene,'homologs')& (~isfield(gene,'paralogs')|isempty(gene.paralogs))
    gene.paralogs = gene.homologs;
  elseif ~isfield(gene,'paralogs')
    gene.paralogs = [];
  end
  %assert that paralogs are consistent
  for k=1:length(gene.paralogs)
    gidxhom = gene.paralogs(k);
    confirmed_sequences(gidxhom).paralogs = gene.paralogs;
  end
  
  if ~isfield(gene,'id')
    gene.id = g_idx;
  end
  gene.confgenes_id = gene.id; 
  
  if ~isfield(gene,'chr')|isempty(gene.chr)
    gene.chr = genome_info.contig_names{gene.chr_num};
  end
   
  if ~isfield(gene,'chr_num')|isempty(gene.chr_num)
    gene.chr_num =  strmatch(gene.chr,genome_info.contig_names,'exact') ;
  end
  
  if ~isfield(gene,'strand')|isempty(gene.strand)
    gene.strand = gene.strands(1) ;
  end

  %% adjust exon and gene boundaries to meet the convention of half-open intervals 
  %% [start,stop) 
  if gene.strands(1)=='+'
    gene.stop = gene.stop+1;
  else
    gene.start = gene.start-1;
  end
  for k=1:length(gene.exons)
    if gene.strands(k)=='+'
      gene.exons{k}(:,2)=gene.exons{k}(:,2)+1;
    else
      gene.exons{k}(:,1)=gene.exons{k}(:,1)-1;
    end
  end
  
 
  %%%complete transcript fields
  assert(length(gene.is_cDNA)==length(gene.transcripts));
  for j=1:length(gene.transcripts)
    if gene.is_cDNA(j)
      gene.transcript_info(j) = PAR.info_genes.transcript.cDNA;
    else
      gene.transcript_info(j) = PAR.info_genes.transcript.est;
    end
    gene.transcript_status(j) = PAR.info_genes.transcript_status.confirmed;
  end

  assert(all(size(gene.transcripts)==size(gene.transcript_info)))
  assert(all(size(gene.transcripts)==size(gene.transcript_valid)))
  assert(all(size(gene.transcripts)==size(gene.transcript_status)))
  assert(all(size(gene.transcripts)==size(gene.exons)))
  assert(all(size(gene.transcripts)==size(gene.exons_confirmed)))
  
  
  genes(g_idx) = gene; 
  
end

G = init_genes(1,PAR); 
genes  = convert_struct(G,genes) ;


% rebuilt splicegraph

genes = build_splice_graph_caller(genes);
genes = infer_splice_graph_caller(genes);
genes = alt_const(genes);




which_checks.exons_sorted = 1;
which_checks.intron_length = 1;
which_checks.splicesites = 1;
which_checks.orf = 1;
which_checks.gene_length = 1;
which_checks.graph = 1;

genes = check_genes(genes, genome_info,which_checks) ;