function [genes_new,info_genes] = transform_confirmed_sequences(genes,fn_genome_config,info_genes)
% [genes,info_genes] = transform_confirmed_sequences(genes,fn_genome_config,PAR) 

if nargin<3
  PAR = init_PAR;
  info_genes = PAR.info_genes; 
end

genome_info = init_genome(fn_genome_config);

%%change consensus position

for j=1:length(genes)
  genes(j).strand = genes(j).strands(1);
  genes(j).chr_num = find(strcmp(genome_info.contig_names,genes(j).chr));
  genes(j).transcript_info = PAR.info_genes.transcript.est*ones(size(genes(j).is_cDNA));
  genes(j).transcript_info(genes(j).is_cDNA==1) = PAR.info_genes.transcript.cDNA;
  genes(j).transcript_status = PAR.info_genes.transcript_status.confirmed*ones(size(genes(j).is_cDNA));
  for t=1:length(genes(j).transcripts)
    if genes(j).strand =='+'
      genes(j).exons{t}(1:end-1,2) = genes(j).exons{t}(1:end-1,2)+1;
    else
      genes(j).exons{t}(2:end,1) = genes(j).exons{t}(2:end,1)-1;
    end
  end
  if isfield(genes(j),'homologs')
    genes(j).paralogs =  genes(j).homologs;
  end
end
genes = sort_regions(genes);
for j=1:length(genes)
  genes(j).id = j;
  genes(j).confgenes_id = j;
end

genes = rmfield(genes,'splicegraph') 
build_splice_graph;
genes = infer_splice_graph_caller(genes);

genes_new = init_genes;
genes_new  = map_two_genes_structures(genes_new, genes,genome_info) ;

which_checks.exons_sorted = 1;
which_checks.intron_length = 1;
which_checks.splicesites = 1;
which_checks.orf = 1;
which_checks.gene_length = 1;

genes_new = check_genes(genes_new,genome_info,which_checks);