addpath ~/svn/projects/genefinding/data_processing_sensors/
addpath ~/svn/tools/genomes/
addpath ~/svn/projects/splicing/utils
addpath ~/svn/projects/splicing/splicegraphs/
addpath ~/svn/projects/splicing/splicegraphs/detect_altsplice/


cleave = 6

base_dir = ['/fml/ag-raetsch/share/projects/genefinding/C_elegans/lsl/cat-1//test_predictions/'...
            'new_signals_new_labels_better_all-17-11-2007/'];
for i=1:12,
  fn_in{i} = sprintf('%spred_contig_corrected%i_cleave%1.2f.mat', base_dir,i, cleave);

  fn_out1{i} = sprintf('%sconcat_contig%i_cleave%1.2f.mat',base_dir,i, cleave);
  fn_out1g{i} = sprintf('%sgenes_contig%i_cleave%1.2f.mat',base_dir,i, cleave); 

  fn_out2{i} = sprintf('%sshiftedWS180_contig%i_cleave%1.2f.mat',base_dir,i, cleave);
  fn_out2g{i} = sprintf('%sgenes_shiftedWS180_contig%i_cleave%1.2f.mat',base_dir,i, cleave);
end

fn_out_target_genome_genes = sprintf('%sgenome_genes_shiftedWS180_cleave%1.2f.mat',base_dir, cleave);
fn_out_orig_genome_genes = sprintf('%sgenome_genes_cleave%1.2f.mat',base_dir, cleave);

% target_config = '/fml/ag-raetsch/share/databases/genomes/elegans_WS180/genome.config' ;
target_config = '/fml/ag-raetsch/share/databases/genomes/C_elegans/elegans_WS180/genebuild/genome.config';
orig_config = '/fml/ag-raetsch/share/projects/genefinding/C_elegans/orig_data/genome.config'

target_genome_info = init_genome(target_config);
orig_genome_info = init_genome(orig_config);

% change names: assumes that chromosomes are in the same order
target_genome_info.contig_names=orig_genome_info.contig_names ;

load('/fml/ag-raetsch/share/projects/genefinding/C_elegans/lsl/cat-1/train/new_signals_new_labels_better_all-17-11-2007/train.mat', 'PAR') ;
model = PAR.model; clear PAR                     

%%%

PAR=[] ;
PAR.FN.input.fn_genome_config = orig_config ;
PAR.organism.name = 'elegansWS170' ;
trivial_regions=init_regions(PAR) ;

% collect blocks
reconcatinate_blocks(fn_in,fn_out1,model,orig_genome_info);

for j=1:length(fn_out1)
  load(fn_out1{j},'blocks');
  genes = blocks2genes(blocks,model,'prediction',orig_genome_info);
  save(fn_out1g{j},'genes');
end
for j=1:length(trivial_regions)
  L=load(fn_out1g{j},'genes');
  if j==1, 
    orig_genes = L.genes ;
  else
    orig_genes = [orig_genes L.genes] ;
  end ;
end ;
for j=1:length(orig_genes)
  orig_genes(j).id=j;
end

for i=1:length(orig_genes),
  orig_genes(i).strands = orig_genes(i).strand ;
end ;

save_struct(fn_out_orig_genome_genes, orig_genes, 'genes') ;


% map to WS180
map_predictionsToDifferentAnno(fn_out1,fn_out2,target_genome_info,orig_genome_info);


PAR=[] ;
PAR.FN.input.fn_genome_config = target_config ;
PAR.organism.name = 'elegansWS180' ;
trivial_regions=init_regions(PAR) ;


% write transformed WS180 genes
for j=1:length(fn_out2)
  load(fn_out2{j},'blocks');
  genes = blocks2genes(blocks,model,'prediction', target_genome_info);
  save(fn_out2g{j},'genes')
end
for j=1:length(trivial_regions)
  L=load(fn_out2g{j},'genes');
  if j==1, 
    target_genes = L.genes ;
  else
    target_genes = [target_genes L.genes] ;
  end ;
end ;
for j=1:length(target_genes)
  target_genes(j).id=j;
end
save_struct(fn_out_target_genome_genes, target_genes, 'genes') ;


colmap = compute_colocation_map(orig_genes, target_genes, target_genome_info) ;
for i=1:length(colmap),
  assert(any(ismember(colmap{i},i))) ;
end ;

%% COMPARE WITH WS180 Annotation


% L=load('/fml/ag-raetsch/share/databases/genomes/elegans_WS180/anno_WS180_final.mat')
L=load('/fml/ag-raetsch/share/databases/genomes/C_elegans/elegans_WS180/annotation/anno_WS180_final.mat')
anno_genes = L.genes ;
for j=1:length(anno_genes)
  anno_genes(j).id=j;
end

for j=1:length(trivial_regions)
  fprintf('%s %c\n', trivial_regions(j).chr, trivial_regions(j).strand) ;
  pred_genes_j = target_genes([target_genes.chr_num]==trivial_regions(j).chr_num & [target_genes.strand]==trivial_regions(j).strand) ;
  anno_genes_j = anno_genes([anno_genes.chr_num]==trivial_regions(j).chr_num & [anno_genes.strand]==trivial_regions(j).strand) ;
  res_genome(j) = evaluate_predictions(trivial_regions(j),anno_genes_j, pred_genes_j, target_genome_info, 0);
end

new_genes_idx = [] ;
for i=1:length(res_genome)
  new_genes_idx = [new_genes_idx res_genome(i).genes.wrong_ids] ;
end ;

missing_anno_genes_idx = [] ;
missing_anno_genes_num= [] ;
for i=1:length(res_genome)
  missing_anno_genes_idx = [missing_anno_genes_idx res_genome(i).genes.missing_ids] ;
  missing_anno_genes_num = [missing_anno_genes_num; res_genome(i).genes.missing_conf res_genome(i).genes.missing_partconf res_genome(i).genes.missing_unconf] ;
end ;

missing_anno_genes_unconf_idx = [] ;
for i=missing_anno_genes_idx,
  if all(anno_genes(i).transcript_status==-1),
    missing_anno_genes_unconf_idx(end+1) = i ;
  end 
end ;

save(fn_out_target_genome_genes,'-append', 'res_genome', 'new_genes_idx', 'missing_anno_genes_idx', 'missing_anno_genes_num','missing_anno_genes_unconf_idx')     
save(fn_out_orig_genome_genes,'-append', 'res_genome', 'new_genes_idx', 'missing_anno_genes_idx', 'missing_anno_genes_num','missing_anno_genes_unconf_idx')     



% look for pseudo genes

L = load('/fml/ag-raetsch/share/databases/genomes/elegans_WS180/pseudogenes.mat') ;
pseudo_genes = L.genes ;
for j=1:length(pseudo_genes)
  pseudo_genes(j).id=j;
end

colmap = compute_colocation_map(pseudo_genes, target_genes, target_genome_info) ;

map = zeros(1,length(colmap)) ;
for i=1:length(colmap),
  map(i) = ~isempty(colmap{i}) ;
end 
idx_pseudo = find(map) ;

save(fn_out_target_genome_genes,'-append', 'idx_pseudo')     
save(fn_out_orig_genome_genes,'-append', 'idx_pseudo')     


% noncoding

L=load('/fml/ag-raetsch/share/databases/genomes/elegans_WS180/ncRNA.mat') ;
nc_genes = L.genes ;

colmap = compute_colocation_map(nc_genes, target_genes, target_genome_info) ;
map = zeros(1,length(colmap)) ;
for i=1:length(colmap),
  map(i) = ~isempty(colmap{i}) ;
end 
idx_nc = find(map) ;

save(fn_out_target_genome_genes,'-append', 'idx_nc')     
save(fn_out_orig_genome_genes,'-append', 'idx_nc')     


fprintf('%i total number of predicted genes\n', length(target_genes)) ;
fprintf('%i total number of annotated genes\n', length(anno_genes)) ;
fprintf('%i total number of ncRNA genes\n', length(nc_genes)) ;
fprintf('%i total number of pseudo genes\n', length(pseudo_genes)) ;

% new genes which overlap with annotated pseudo genes 
experiment1_idx = intersect(new_genes_idx,idx_pseudo) ; 
fprintf('%i pseudo_new_genes\n', length(experiment1_idx)) ;

% new genes which overlap with annotated noncoding genes
experiment2_idx = intersect(new_genes_idx,idx_nc) ; % "nc_new_genes"
fprintf('%i nc_new_genes\n', length(experiment2_idx)) ;

% new genes with no overlap
experiment3_idx = setdiff(setdiff(new_genes_idx,idx_pseudo), idx_nc) ; % "new_genes"
fprintf('%i new_genes\n', length(experiment3_idx)) ;

fprintf('%i missing unconfirmed genes\n', length(missing_anno_genes_unconf_idx)) ;

save(fn_out_target_genome_genes,'-append', 'experiment1_idx', 'experiment2_idx', 'experiment3_idx')     
save(fn_out_orig_genome_genes,'-append', 'experiment1_idx', 'experiment2_idx', 'experiment3_idx')     

save(fn_out_target_genome_genes,'-append', 'experiment1_idx', 'experiment2_idx', 'experiment3_idx', 'target_config')     
save(fn_out_orig_genome_genes,'-append', 'experiment1_idx', 'experiment2_idx', 'experiment3_idx', 'orig_config')     


rand('seed', 34598742388) ;
rand_idx = randperm(length(experiment1_idx)) ;
rand_idx = rand_idx(1:10) ;
experiment1_cands = genes2primers_py(orig_genes(experiment1_idx(rand_idx)), orig_config, '~/svn/tex/GR-mGene/results/mGPG_primers_.txt', 'pseudo_gene', 'AA_1') ;


experiment2_cands = genes2primers_py(orig_genes(experiment2_idx), orig_config, '~/svn/tex/GR-mGene/results/mGNC_primers.txt', 'noncoding_gene', 'AA_2') ;


rand('seed', 34598742388) ;
rand_idx = randperm(length(experiment3_idx)) ;
rand_idx = rand_idx(1:100) ;

experiment3_cands = genes2primers_py(orig_genes(experiment3_idx(rand_idx)), orig_config, '~/svn/tex/GR-mGene/results/mGNG_primers.txt', 'new_gene', 'AA_3') ;


rand('seed', 34598742388) ;
rand_idx = randperm(length(missing_anno_genes_unconf_idx)) ;
rand_idx = rand_idx(1:100) ;

experiment4_cands = genes2primers_py(anno_genes(missing_anno_genes_unconf_idx(rand_idx)), orig_config, '~/svn/tex/GR-mGene/results/mGMG_primers.txt', 'missing_gene', 'AA_4') ;

save ~/svn/tex/GR-mGene/results/experiment1_cands.mat experiment1_cands
save ~/svn/tex/GR-mGene/results/experiment2_cands.mat experiment2_cands
save ~/svn/tex/GR-mGene/results/experiment3_cands.mat experiment3_cands
save ~/svn/tex/GR-mGene/results/experiment4_cands.mat experiment4_cands
 
