
clear;
%paths;
evaluate_operons_params;
strands = '+-';



% === load data
fprintf( '\n' );
fn_allData = [ tmp_dir , '/genesAndOperons.mat' ];
fprintf( '=== loading %s ===\n', fn_allData );
load( fn_allData, 'allData', 'counts', 'chrs', 'chrLens', 'contigNames' );
nofContigs = 2 * length(chrs);



% === summary figures

% --- all counts (genes and operons)
figure;
bar( counts(:,[1,4,3,6]) );
grid on;
title( [annot,': Summary Statistics'] );
ylabel( 'count' );
legend( { 'annot genes', 'mGene genes', 'annot operons', 'mGene operons' }, 2 );
set( gca, 'XTickLabel', contigNames );
print( '-dps2c', ['eval_lsl/pics/',annot,',counts.ps'] );

% --- gene counts
figure;
bar( counts(:,[1,4]) );
grid on;
title( [annot,': Gene Counts'] );
ylabel( 'count' );
legend( { 'annotation', 'mGene' }, 2 );
set( gca, 'XTickLabel', contigNames );
print( '-dps2c', ['eval_lsl/pics/',annot,',genes,counts.ps'] );

% --- operon counts
figure;
bar( counts(:,[3,6]) );
grid on;
title( [annot,': Operon Counts'] );
ylabel( 'count' );
legend( { 'annotation', 'mGene' }, 2 );
set( gca, 'XTickLabel', contigNames );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,counts.ps'] );
nofAnnoOpers = sum( counts(1:nofContigs,3) );
nofPredOpers = sum( counts(1:nofContigs,6) );
fprintf( 'operons: %d annotated, %d predicted\n', nofAnnoOpers, nofPredOpers );

% --- operon density [nucleotide level]
figure;
bar( counts(1:nofContigs,[3,6]) ./ kron(ones(2,2),chrLens(:,1)) * 1e6 );
grid on;
title( [annot,': Operon Density'] );
ylabel( 'operons per magebase' );
legend( { 'annotation', 'mGene' }, 2 );
set( gca, 'XTickLabel', contigNames );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,density0.ps'] );

% --- operon density [gene level]
figure;
bar( counts(:,[3,6]) ./ counts(:,[1,1]) );
grid on;
title( [annot,': Operon Density'] );
ylabel( 'operons per gene' );
legend( { 'annotation', 'mGene' }, 2 );
set( gca, 'XTickLabel', contigNames );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,density1.ps'] );

% --- operon sizes
figure;
annotSizes = counts(:,2) ./ counts(:,3);
mgeneSizes = counts(:,5) ./ counts(:,6);
bar( [annotSizes,mgeneSizes] );
grid on;
title( [annot,': Average Operon Sizes'] );
ylabel( 'average number of genes per operon' );
legend( { 'annotation', 'mGene' }, 3 );
set( gca, 'XTickLabel', contigNames );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,sizes.ps'] );



% === go thru (chr,strand) pairs
fprintf( '\n' );
geneFracs0 = repmat( nan, nofContigs, 3 );
operFracs0 = repmat( nan, nofContigs, 3 );
operFracs1 = repmat( nan, nofContigs, 3 );
operFracs2 = zeros( nofContigs, 4 );
operCovers0 = repmat( nan, nofContigs, 4 );
k = 0;
for( iChr = 1:length(chrs) )
  chr = chrs{ iChr };
  for( strand = strands )
    fprintf( '=== chromosome/strand: %s%s ===\n', chr, strand );
    k = k + 1;
    
    % --- find items on (chr,strand) pair; find operons
    idx = find_items( allData, chr, strand, [], [] );
    data = allData( idx );
    [ adjMat, edges ] = find_overlaps( data );
    annoGenes = find_items( data, [], [], annot, 'gene' );
    predGenes = find_items( data, [], [], mGene, 'gene' );
    annoOpers = find_items( data, [], [], annot, 'operon' );
    predOpers = find_items( data, [], [], mGene, 'operon' );
    
    % --- quantify dependence: genes
    geneInd = zeros( chrLens(iChr,(strand~=strands(1))+1), 2 );
    for( i = annoGenes' )
      range = data(i).start : data(i).stop;
      %assert_all( geneInd(range,1) == 0 );
      geneInd(range,1) = geneInd(range,1) + 1;
    end;
    for( i = predGenes' )
      range = data(i).start : data(i).stop;
      assert_all( geneInd(range,2) == 0 );
      geneInd( range, 2 ) = 1;
    end;
    if( sum(geneInd(:,1)>1) > 0 )
      fprintf( 'WARNING: %d nucleotides in more than one annotated gene.\n', sum(geneInd(:,1)>1) );
    end;
    M = hist( (geneInd(:,1)>0) + 2*(geneInd(:,2)>0), -1:4 );
    assert( M(1)==0 & M(end)==0 );
    M = reshape( M(2:(end-1)), 2, 2 );
    geneFracAnno = sum( M(2,:) ) / sum( M(:) );
    geneFracPred = sum( M(:,2) ) / sum( M(:) );
    geneFracObsrv = M(2,2) / sum( M(:) );
    geneFracs0( k, : ) = [ geneFracAnno, geneFracPred, geneFracObsrv ];
    
    % --- quantify dependence: operons
    operInd = zeros( chrLens(iChr,(strand~=strands(1))+1), 2 );
    for( i = annoOpers' )
      range = data(i).start : data(i).stop;
      %assert_all( operInd(range,1) == 0 );
      operInd(range,1) = operInd(range,1) + 1;
    end;
    for( i = predOpers' )
      range = data(i).start : data(i).stop;
      assert_all( operInd(range,2) == 0 );
      operInd( range, 2 ) = 1;
    end;
    if( sum(operInd(:,1)>1) > 0 )
      fprintf( 'WARNING: %d nucleotides in more than one annotated operon.\n', sum(operInd(:,1)>1) );
    end;
    M = hist( (operInd(:,1)>0) + 2*(operInd(:,2)>0), -1:4 );
    assert( M(1)==0 & M(end)==0 );
    M = reshape( M(2:(end-1)), 2, 2 );
    operFracAnno = sum( M(2,:) ) / sum( M(:) );
    operFracPred = sum( M(:,2) ) / sum( M(:) );
    operFracObsrv = M(2,2) / sum( M(:) );
    operFracs0( k, : ) = [ operFracAnno, operFracPred, operFracObsrv ];
    operCovers0( k, : ) = [ sum(M(2,:)), sum(M(:,2)), M(2,2), sum(M(:)) ];
    
    % --- go thru annotated operons
    nofAnnoGenesOverlap = 0;
    for( i = annoOpers' )
      annoOverlap = find( full( adjMat(i,:) ) );
      % - overlap with predicted operons
      predOpersOverlap = annoOverlap( find_items( data(annoOverlap), [], [], mGene, 'operon' ) );
      n = length( predOpersOverlap );
      if( n > 3 )
	fprintf( 'WARNING: ignoring annotated operon %s that overlaps %d predicted operons.\n', data(i).name, n );
      else
	operFracs2(k,n+1) = operFracs2(k,n+1) + 1;
      end;
      if( n == 1 )
	j = predOpersOverlap( 1 );
	predOverlap = find( full( adjMat(j,:) ) );
      end;
      % - overlap with annotated genes
      annoGenesOverlap = annoOverlap( find_items( data(annoOverlap), [], [], annot, 'gene' ) );
      nofAnnoGenesOverlap = nofAnnoGenesOverlap + length( annoGenesOverlap );
    end;
    operFracs1(k,1) = nofAnnoGenesOverlap / length(annoOpers);
    
    % --- go thru predicted operons
    nofAnnoGenesOverlap = 0;
    nofPredGenesOverlap = 0;
    for( i = predOpers' )
      predOverlap = find( full( adjMat(i,:) ) );
      % - overlap with annotated genes
      annoGenesOverlap = predOverlap( find_items( data(predOverlap), [], [], annot, 'gene' ) );
      nofAnnoGenesOverlap = nofAnnoGenesOverlap + length( annoGenesOverlap );
      % - overlap with predicted genes
      predGenesOverlap = predOverlap( find_items( data(predOverlap), [], [], mGene, 'gene' ) );
      nofPredGenesOverlap = nofPredGenesOverlap + length( predGenesOverlap );
    end;
    operFracs1(k,2) = nofAnnoGenesOverlap / length(predOpers);
    operFracs1(k,3) = nofPredGenesOverlap / length(predOpers);
    
    fprintf( '\n' );
  end;
end;
assert_all( operCovers0(:,4) == reshape(chrLens',nofContigs,1) );



% === plot

% --- operon covering
figure;
bar( 100 * operCovers0(:,[1,2]) ./ operCovers0(:,[4,4]) );
grid on;
title( [annot,': Operon Covering'] );
ylabel( 'fraction of nucleotides in operon [%]' );
legend( { 'annotation', 'mGene' }, 1 );
set( gca, 'XTickLabel', contigNames );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,covering0.ps'] );

% --- gene length distribs
annoGenes = allData( find_items( allData, [], [], annot, 'gene' ) );
predGenes = allData( find_items( allData, [], [], mGene, 'gene' ) );
annoGeneLens = [annoGenes.stop] - [annoGenes.start];
predGeneLens = [predGenes.stop] - [predGenes.start];
[ dummy, geneLenBins ] = hist( log2([annoGeneLens,predGeneLens]), 50 );
[ annoGeneLenCounts ] = hist( log2(annoGeneLens), geneLenBins );
[ predGeneLenCounts ] = hist( log2(predGeneLens), geneLenBins );
figure;
hold on;
plot( 2.^geneLenBins, annoGeneLenCounts, 'Color', 'b', 'LineWidth', 3, 'Marker', 'o' );
plot( 2.^geneLenBins, predGeneLenCounts, 'Color', 'r', 'LineWidth', 3, 'Marker', 'o' );
set( gca, 'XScale', 'log' );
ax = axis;
axis( [ 2.^geneLenBins([1,end]) ax(3:4) ] );
grid on;
title( [annot,': Gene Length Distributions'] );
xlabel( 'gene length [nucleotides]' );
ylabel( 'frequency' );
legend( { 'annotation', 'mGene' } );
relGeneLenChange = 1 - ( mean(annoGeneLens) / mean(predGeneLens) );
fprintf( 'relative change in gene lengths: %4.1f%%\n', 100*relGeneLenChange );
print( '-dps2c', ['eval_lsl/pics/',annot,',genes,lenDistrib.ps'] );

% --- operon length distribs
annoOpers = allData( find_items( allData, [], [], annot, 'operon' ) );
predOpers = allData( find_items( allData, [], [], mGene, 'operon' ) );
annoOperLens = [annoOpers.stop] - [annoOpers.start];
predOperLens = [predOpers.stop] - [predOpers.start];
[ dummy, operLenBins ] = hist( log2([annoOperLens,predOperLens]), 25 );
[ annoOperLenCounts ] = hist( log2(annoOperLens), operLenBins );
[ predOperLenCounts ] = hist( log2(predOperLens), operLenBins );
figure;
hold on;
plot( 2.^operLenBins, annoOperLenCounts, 'Color', 'b', 'LineWidth', 3, 'Marker', 'o' );
plot( 2.^operLenBins, predOperLenCounts, 'Color', 'r', 'LineWidth', 3, 'Marker', 'o' );
set( gca, 'XScale', 'log' );
ax = axis;
axis( [ 2.^operLenBins([1,end]) ax(3:4) ] );
grid on;
title( [annot,': Operon Length Distributions'] );
xlabel( 'oper length [nucleotides]' );
ylabel( 'frequency' );
legend( { 'annotation', 'mGene' } );
relOperLenChange = 1 - ( mean(annoOperLens) / mean(predOperLens) );
fprintf( 'relative change in operon lengths: %4.1f%%\n', 100*relOperLenChange );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,lenDistrib.ps'] );

% --- joint genes [nucleotide level]
t = geneFracs0(:,3) ./ ( geneFracs0(:,1) .* geneFracs0(:,2) );
figure;
bar( reshape(t,2,length(chrs))' );
ax = axis;
axis( [ 0 length(chrs)+1 ax(3:4) ] );
grid on;
title( [annot,': Joint Gene Incidence (Annotation and Prediction)'] );
ylabel( 'observed / expected [nucleotide level]' );
legend( { '+ strand', '- strand' }, 2 );
set( gca, 'XTickLabel', chrs );
print( '-dps2c', ['eval_lsl/pics/',annot,',genes,overlap0.ps'] );

% --- joint operons [nucleotide level]
t = operFracs0(:,3) ./ ( operFracs0(:,1) .* operFracs0(:,2) );
u = operCovers0(:,3) .* operCovers0(:,4) ./ ( operCovers0(:,1) .* operCovers0(:,2) );
assert( norm(t-u) <= 1e-12 );
figure;
bar( reshape(t,2,length(chrs))' );
ax = axis;
axis( [ 0 length(chrs)+1 ax(3:4) ] );
grid on;
title( [annot,': Joint Operon Incidence (Annotation and Prediction)'] );
ylabel( 'observed / expected [nucleotide level]' );
legend( { '+ strand', '- strand' }, 2 );
set( gca, 'XTickLabel', chrs );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,overlap0.ps'] );

% --- gene content of operons
operFracs1(nofContigs+1,:) = mean( operFracs1, 1 );
figure;
bar( operFracs1 );
ax = axis;
axis( [ 0 nofContigs+2 ax(3:4) ] );
grid on;
title( [annot,': Average Number of Genes in Operons'] );
ylabel( 'avg. genes / operons' );
legend( { 'annot genes / annot operon', 'annot genes / pred operon', 'pred genes / pred operon' }, 3 );
set( gca, 'XTickLabel', contigNames );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,overlap1.ps'] );

% --- overlap of operons
operFracs2(nofContigs+1,:) = mean( operFracs2, 1 );
figure;
bar( operFracs2 );
ax = axis;
axis( [ 0 nofContigs+2 ax(3:4) ] );
grid on;
title( [annot,': Operon Overlap of mGene with Annotation'] );
ylabel( 'number of annotated operons' );
legend( { 'no overlapping mGene operon', '1 overlapping mGene operon', '2 overlapping mGene operons', '3 overlapping mGene operons' }, 1 );
set( gca, 'XTickLabel', contigNames );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,overlap2.ps'] );



