
clear;
%paths;
evaluate_operons_params;
strands = '+-';



% === load data
fprintf( '\n' );
fn_allData = [ tmp_dir , '/genesAndOperons.mat' ];
fprintf( '=== loading %s ===\n', fn_allData );
load( fn_allData, 'allData', 'counts', 'chrs', 'chrLens', 'contigNames' );
nofContigs = 2 * length(chrs);



% === go thru (chr,strand) pairs
fprintf( '\n' );
k = 0;
predOperConfs = repmat( nan, [ nofContigs, 2, 3 ] );
annoOperAnnoGeneDists = cell( 5, 1 );
predOperAnnoGeneDists = cell( 5, 1 );
predOperPredGeneDists = cell( 5, 1 );
for( iChr = 1:length(chrs) )
  chr = chrs{ iChr };
  for( strand = strands )
    fprintf( '=== chromosome/strand: %s%s ===\n', chr, strand );
    k = k + 1;
    
    % --- find items on (chr,strand) pair
    idx = find_items( allData, chr, strand, [], [] );
    data = allData( idx );
    [ adjMat, edges ] = find_overlaps( data );
    annoGenes = find_items( data, [], [], annot, 'gene' );
    predGenes = find_items( data, [], [], mGene, 'gene' );
    annoOpers = find_items( data, [], [], annot, 'operon' );
    predOpers = find_items( data, [], [], mGene, 'operon' );
    nofAnnoGenes = length( annoGenes );
    nofAnnoOpers = length( annoOpers );
    nofPredGenes = length( predGenes );
    nofPredOpers = length( predOpers );
    
    % --- go thru predicted operons
    predOperStats = repmat( nan, nofPredOpers, 5 );
    for( ii = 1:nofPredOpers )
      i = predOpers( ii );
      predOverlap = find( full( adjMat(i,:) ) );
      % - overlap with annotated operons
      annoOpersOverlap = predOverlap( find_items( data(predOverlap), [], [], annot, 'operon' ) );
      predOperStats(ii,1) = length( annoOpersOverlap );
      % - overlap with annotated genes
      annoGenesOverlap = predOverlap( find_items( data(predOverlap), [], [], annot, 'gene' ) );
      confs = [ data(annoGenesOverlap).conf ];
      predOperStats(ii,2:4) = hist( confs, [-1,0,+1] );
      predOperStats(ii,5) = max( [ confs, -1 ] );
    end;
    predOperConfs( k, 2, 3 ) = sum( predOperStats(:,1)==0 & predOperStats(:,5)==-1 );
    predOperConfs( k, 2, 2 ) = sum( predOperStats(:,1)==0 & predOperStats(:,5)==0 );
    predOperConfs( k, 2, 1 ) = sum( predOperStats(:,1)==0 & predOperStats(:,5)==+1 );
    predOperConfs( k, 1, 3 ) = sum( predOperStats(:,1)==1 & predOperStats(:,5)==-1 );
    predOperConfs( k, 1, 2 ) = sum( predOperStats(:,1)==1 & predOperStats(:,5)==0 );
    predOperConfs( k, 1, 1 ) = sum( predOperStats(:,1)==1 & predOperStats(:,5)==+1 );
    
    annoGeneDists = [ data(annoGenes(2:end)).start ] - [ data(annoGenes(1:(end-1))).stop ];
    predGeneDists = [ data(predGenes(2:end)).start ] - [ data(predGenes(1:(end-1))).stop ];

    % --- annotated gene distances wrt annotated operons
    gapsAnnoOpers = zeros( 1, nofAnnoGenes-1 );
    assert_all( size(gapsAnnoOpers) == size(annoGeneDists) );
    for( j = 1:nofAnnoOpers )
      is = find( adjMat( annoGenes, annoOpers(j) ) );
      if( isempty(is) )
	fprintf( 'WARNING: no annotated genes in annotated operon %s\n', data(annoOpers(j)).name );
	continue;
      end;
      assert( length(is) == max(is)-min(is)+1 );
      assert_all( is' == min(is):max(is) );
      gapsAnnoOpers( is(1:(end-1)) ) = -1;
      s = min(is) - 1;
      t = max(is);
      if( s > 0 )
	gapsAnnoOpers( s ) = gapsAnnoOpers( s ) + 1;
      end;
      if( t <= nofAnnoGenes-1 )
	gapsAnnoOpers( t ) = gapsAnnoOpers( t ) + 2;
      end;
    end;
    for( l = -1:3 )
      annoOperAnnoGeneDists{l+2} = [ annoOperAnnoGeneDists{l+2} , annoGeneDists(gapsAnnoOpers==l) ];
    end;

    % --- annotated gene distances wrt predicted operons
    annoGapsPredOpers = zeros( 1, nofAnnoGenes-1 );
    assert_all( size(annoGapsPredOpers) == size(annoGeneDists) );
    for( j = 1:nofPredOpers )
      is = find( adjMat( annoGenes, predOpers(j) ) );
      if( isempty(is) )
	fprintf( 'WARNING: no annotated genes in predicted operon %s\n', data(predOpers(j)).name );
	continue;
      end;
      if( length(is) ~= max(is)-min(is)+1 )
	fprintf( 'WARNING: annotated genes in predicted operon %s not consecutive -- ignoring\n', data(predOpers(j)).name );
	continue;
      end;
      assert( length(is) == max(is)-min(is)+1 );
      assert_all( is' == min(is):max(is) );
      annoGapsPredOpers( is(1:(end-1)) ) = -1;
      s = min(is) - 1;
      t = max(is);
      if( s > 0 )
	annoGapsPredOpers( s ) = annoGapsPredOpers( s ) + 1;
      end;
      if( t <= nofAnnoGenes-1 )
	annoGapsPredOpers( t ) = annoGapsPredOpers( t ) + 2;
      end;
    end;
    for( l = -1:3 )
      predOperAnnoGeneDists{l+2} = [ predOperAnnoGeneDists{l+2} , annoGeneDists(annoGapsPredOpers==l) ];
    end;

    % --- annotated gene distances wrt predicted operons
    predGapsPredOpers = zeros( 1, nofPredGenes-1 );
    assert_all( size(predGapsPredOpers) == size(predGeneDists) );
    for( j = 1:nofPredOpers )
      is = find( adjMat( predGenes, predOpers(j) ) );
      if( isempty(is) )
	fprintf( 'WARNING: no predicted genes in predicted operon %s\n', data(predOpers(j)).name );
	continue;
      end;
      if( length(is) ~= max(is)-min(is)+1 )
	fprintf( 'WARNING: predicted genes in predicted operon %s not consecutive -- ignoring\n', data(predOpers(j)).name );
	continue;
      end;
      assert( length(is) == max(is)-min(is)+1 );
      assert_all( is' == min(is):max(is) );
      predGapsPredOpers( is(1:(end-1)) ) = -1;
      s = min(is) - 1;
      t = max(is);
      if( s > 0 )
	predGapsPredOpers( s ) = predGapsPredOpers( s ) + 1;
      end;
      if( t <= nofPredGenes-1 )
	predGapsPredOpers( t ) = predGapsPredOpers( t ) + 2;
      end;
    end;
    for( l = -1:3 )
      predOperPredGeneDists{l+2} = [ predOperPredGeneDists{l+2} , predGeneDists(predGapsPredOpers==l) ];
    end;

    % --- done with (chr,strand) pair
    fprintf( '\n' );
  end;
end;
% --- clean distances
for( l = -1:3 )
  temp = annoOperAnnoGeneDists{l+2};
  annoOperAnnoGeneDists{l+2} = temp( temp > 0 );
end;
for( l = -1:3 )
  temp = predOperAnnoGeneDists{l+2};
  predOperAnnoGeneDists{l+2} = temp( temp > 0 );
end;
for( l = -1:3 )
  temp = predOperPredGeneDists{l+2};
  predOperPredGeneDists{l+2} = temp( temp > 0 );
end;
annoOperAnnoGeneDists{6} = [ annoOperAnnoGeneDists{3} , annoOperAnnoGeneDists{4} , annoOperAnnoGeneDists{5} ];
predOperAnnoGeneDists{6} = [ predOperAnnoGeneDists{3} , predOperAnnoGeneDists{4} , predOperAnnoGeneDists{5} ];
predOperPredGeneDists{6} = [ predOperPredGeneDists{3} , predOperPredGeneDists{4} , predOperPredGeneDists{5} ];


% === plot

% --- operon covering
figure;
T = squeeze( sum( predOperConfs, 1 ) );
bar( T );
title( ['mGene Operon Analysis [',annot,']'] );
ylabel( 'number of mGene operon predictions' );
legendStrs = {
    'with genes annotated as "confirmed"',
    'with genes annotated as "partially confirmed"',
    'with genes annotated as "predicted"'
};
legend( legendStrs, 1 );
set( gca, 'XTickLabel', { 'annotated operons', 'newly predicted operons' } );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,conf1.ps'] );

% --- gene distance ditributions
log2DistRange = 4:0.5:16;
annoHistWithin  = hist( log2(annoOperAnnoGeneDists{1}), log2DistRange ) / length(annoOperAnnoGeneDists{1});
annoHistNeighb  = hist( log2(annoOperAnnoGeneDists{6}), log2DistRange ) / length(annoOperAnnoGeneDists{6});
annoHistAway    = hist( log2(annoOperAnnoGeneDists{2}), log2DistRange ) / length(annoOperAnnoGeneDists{2});
pred1HistWithin = hist( log2(predOperAnnoGeneDists{1}), log2DistRange ) / length(predOperAnnoGeneDists{1});
pred1HistNeighb = hist( log2(predOperAnnoGeneDists{6}), log2DistRange ) / length(predOperAnnoGeneDists{6});
pred1HistAway   = hist( log2(predOperAnnoGeneDists{2}), log2DistRange ) / length(predOperAnnoGeneDists{2});
pred2HistWithin = hist( log2(predOperPredGeneDists{1}), log2DistRange ) / length(predOperPredGeneDists{1});
pred2HistNeighb = hist( log2(predOperPredGeneDists{6}), log2DistRange ) / length(predOperPredGeneDists{6});
pred2HistAway   = hist( log2(predOperPredGeneDists{2}), log2DistRange ) / length(predOperPredGeneDists{2});

% - wrt annotated genes
figure;
hold on;
plot( 2.^log2DistRange, annoHistWithin,  'Color', 'g', 'LineWidth', 2 );
plot( 2.^log2DistRange, annoHistNeighb,  'Color', 'b', 'LineWidth', 2 );
plot( 2.^log2DistRange, annoHistAway,    'Color', 'r', 'LineWidth', 2 );
plot( 2.^log2DistRange, pred1HistWithin, 'Color', 'g', 'LineWidth', 3, 'LineStyle', '-.' );
plot( 2.^log2DistRange, pred1HistNeighb, 'Color', 'b', 'LineWidth', 3, 'LineStyle', '-.' );
plot( 2.^log2DistRange, pred1HistAway,   'Color', 'r', 'LineWidth', 3, 'LineStyle', '-.' );
set( gca, 'XScale', 'log');
grid on;
title( ['Gene Distances [',annot,']'] );
xlabel( 'genes distance' );
ylabel( 'fraction of annotated genes [%]' );
%ylabel( 'fraction of mGene-predicted genes [%]' );
legendStrs = {
    'within an annotated operon'
    'adjacent to an annotated operon'
    'away from annotated operons'
    'within a predicted operon'
    'adjacent to a predicted operon'
    'away from predicted operons'
};
legend( legendStrs, 1 );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,geneDists1.ps'] );

% - wrt annotated genes
figure;
hold on;
plot( 2.^log2DistRange, annoHistWithin,  'Color', 'g', 'LineWidth', 2 );
plot( 2.^log2DistRange, annoHistNeighb,  'Color', 'b', 'LineWidth', 2 );
plot( 2.^log2DistRange, annoHistAway,    'Color', 'r', 'LineWidth', 2 );
plot( 2.^log2DistRange, pred2HistWithin, 'Color', 'g', 'LineWidth', 3, 'LineStyle', '-.' );
plot( 2.^log2DistRange, pred2HistNeighb, 'Color', 'b', 'LineWidth', 3, 'LineStyle', '-.' );
plot( 2.^log2DistRange, pred2HistAway,   'Color', 'r', 'LineWidth', 3, 'LineStyle', '-.' );
set( gca, 'XScale', 'log');
grid on;
title( ['Gene Distances [',annot,']'] );
xlabel( 'genes distance' );
ylabel( 'fraction of annotated genes [%]' );
%ylabel( 'fraction of mGene-predicted genes [%]' );
legendStrs = {
    'within an annotated operon'
    'adjacent to an annotated operon'
    'away from annotated operons'
    'within a predicted operon'
    'adjacent to a predicted operon'
    'away from predicted operons'
};
legend( legendStrs, 1 );
print( '-dps2c', ['eval_lsl/pics/',annot,',operons,geneDists2.ps'] );



