
genes = {};
segments = [];
for j=1:1022, 
  fn_pred=sprintf('_viterbi_block%i.mat',j);
  if fexist(fn_pred)
    load(fn_pred,'pred')
  else
    warning('file does not exist: %s',fn_pred)
    continue;
  end
  if isempty(pred.genes{1}), continue, end
  genes = {genes{:} pred.genes{:}};
  for k =1:length(pred.genes)
    segments = [segments; pred.genes{k}];
  end
end

idx_exons = find(segments(:,3)==2);
idx_introns = find(segments(:,3)==5);
idx_utr5 = find(segments(:,3)==1);
idx_utr3 = find(segments(:,3)==3);


fprintf(1,'number of predicted genes: %i\n', length(genes))
fprintf(1,'number of predicted exons: %i\n', length(idx_exons))
fprintf(1,'number of predicted introns: %i\n', length(idx_introns))
fprintf(1,'number of predicted 5p-utrs: %i\n', length(idx_utr5))
fprintf(1,'number of predicted 3p-utrs: %i\n', length(idx_utr3))

lengths_exon = segments(idx_exons,2)-segments(idx_exons,1);
lengths_intron = segments(idx_introns,2)-segments(idx_introns,1);

fprintf(1,'mean exon length: %f\n', mean(lengths_exon))
fprintf(1,'min exon length: %i\n', min(lengths_exon))
fprintf(1,'max exon length: %i\n', max(lengths_exon))
fprintf(1,'mean intron length: %f\n', mean(lengths_intron))
fprintf(1,'min intron length: %i\n', min(lengths_intron))
fprintf(1,'max intron length: %i\n', max(lengths_intron))

figure; hist(log(segments(idx_exons,2)-segments(idx_exons,1)),100),title('exon length')
figure; hist(log(segments(idx_introns,2)-segments(idx_introns,1)),100),title('intron lengths')
figure; hist(log(segments(idx_utr5,2)-segments(idx_utr5,1)),100),title('5p-UTR lengths')
figure; hist(log(segments(idx_utr3,2)-segments(idx_utr3,1)),100),title('3p-UTR lengths')



return

