
function [ items ] = import_gff_subfile( fn_gff_sub, id1, id2 );

% === const
TAB = sprintf( '\t' );
nofGffCols = 9;

% === get number of items
[ status, result ] = unix( ['wc -l ',fn_gff_sub] );
assert( status == 0 );
p = find( result == ' ' );
nofItems = str2num( result( 1:(p(1)-1) ) );
fprintf( 'reading %d %ss from annotation\n', nofItems, id1 );

% === init data structure
items = [];
items(nofItems,1).chr = '';
items(nofItems,1).strand = '';
items(nofItems,1).name = '';
items(nofItems,1).start = 0;
items(nofItems,1).stop  = 0;

% === read file
fid = fopen( fn_gff_sub, 'rt' );
line = fgetl( fid );
i = 1;
while( ischar(line) )
  p = find( line == TAB );
  assert( length(p) == nofGffCols-1 );
  
  % --- split line
  p = [ 0 , p , length(line)+1 ];
  for( j = 1:nofGffCols )
    q{j} = line( (p(j)+1) : (p(j+1)-1) );
  end;
  
  % --- collect info
  items(i).chr = q{1};
  assert( strcmp( q{2}, id1 ) );
  assert( strcmp( q{3}, id2 ) );
  items(i).start = str2num( q{4} );
  items(i).stop  = str2num( q{5} );
  assert( 0 < items(i).start );
  assert( items(i).start < items(i).stop );
  items(i).strand = q{7};
  assert_all( items(i).strand == '+' | items(i).strand == '-' );
  name = q{9};
  if( all( name(1:3) == 'ID=' ) )
    assert_all( name(1:3) == 'ID=' );
    p = min( find( name == ';' ) );
    if( ~isempty(p) )
      items(i).name = name( 4:(p-1) );
    else
      items(i).name = name( 4:end );
    end;
  else
    items(i).name = name;
  end;
  assert_all( lower(items(i).name(1:length(id2))) == id2 );
  
  % -- next line
  line = fgetl( fid );
  i = i + 1;
end;
assert( line == - 1 );
assert( i == nofItems+1 );
fclose( fid );

% === eliminate duplicates
s = [ items.start ];
t = [ items.stop ];
idx = find( s(1:(end-1))==s(2:end) & t(1:(end-1))==t(2:end) );
for( k = length(idx):-1:1 )
  i = idx( k );
  assert( strcmp( items(i).chr, items(i+1).chr ) );
  assert( items(i).strand == items(i+1).strand );
  fprintf( 'WARNING (%s%s): items %s and %s are identical\n', items(i).chr, items(i).strand, items(i).name, items(i+1).name );
  items(i).name = [ items(i).name ',' items(i+1).name ];
  items(i+1) = [];
end;
if( ~isempty(idx) )
  fprintf( 'WARNING: merged %d duplicate items\n', length(idx) );
end;

% === sort [necessary for gff3 files]
[ dummy, perm ] = sort( [ items.start] );
n = length( items );
if( any(perm~=(1:n)) )
  items = items( perm );
  fprintf( 'sorting %d %ss\n', n, id2 );
end;

