function [blocks,shift,pos]=map_predictionsToDifferentAnno(fn_in,fn_out,old_genome_info,new_genome_info)
  


%-----------------------------

  
for f=1:length(fn_in),
  fprintf('loading predictions from %s...\n', fn_in{f});
  load(fn_in{f}, 'blocks');
  old_chr_seq = upper(load_genomic(old_genome_info.contig_names{blocks.chr_num}, '+', ...
                                      1, inf,old_genome_info));
  new_chr_seq = upper(load_genomic(new_genome_info.contig_names{blocks.chr_num},'+', ...
                                      1, inf,new_genome_info));
  
  segments = blocks.prediction.segments;
  if blocks.strand=='+'
    genestr = blocks.genestr;
  else
    genestr = reverse_complement(blocks.genestr);
  end
  % try to map to nGASP sequence coordinates
  w = 5;
  max_shift = 1000;
  shift = nan(size(segments,1),1);
  if blocks.strand=='+'
    pos = segments(:,1);
    pos2 = segments(:,2);
  else
    pos = length(new_chr_seq)-segments(:,1)+1;
    pos2 =  length(new_chr_seq)-segments(:,2)+1; 
  end
  good_idx = find(pos > w+max_shift&pos <length(new_chr_seq)-w-max_shift);
  if length(good_idx)<length(pos),
    fprintf('removing %i positions\n',length(pos)-length(good_idx) );
  end
  for i=good_idx',
    new_seq = new_chr_seq(pos(i)-w:pos(i)+w);
    if ~ismember('X',genestr(pos(i)-w:pos(i)+w))
      % assert(isequal(genestr(pos(i)-w:pos(i)+w), new_seq))
    end
    old_seq = old_chr_seq(pos(i)-w-max_shift:pos(i)+w+max_shift);
    possible_shifts = strfind(old_seq, new_seq) - max_shift - 1;
    if ~isempty(possible_shifts),
      [tmp idx] = min(abs(possible_shifts));
      shift(i) = possible_shifts(idx);
      if shift(i)~=0,
        if(~isequal(new_seq, old_seq(max_shift+[1:length(new_seq)]+shift(i))));
          warning('old genestr not ok')
        else
          % fprintf('old genestr for pos %i ok\r',i)
        end
      end
    end
  end
  fprintf('shifts found for %2.1f%% positions\n', ...
          100*mean(~isnan(shift)));
  hist(shift,[-10:10]);drawnow;grid on
  
  % shift pos and segments into place for WS160 genome release
  shift(isnan(shift))=0;      
  new_pos = pos +shift;
  new_pos2 = pos2 +shift;    
  % segments(good_idx(2:end),2)= segments(good_idx(2:end),2) + shift(2:end);
  %%% check
  for i=good_idx',
    new_seq = new_chr_seq(pos(i)-w:pos(i)+w);
    old_seq = old_chr_seq(new_pos(i)-w:new_pos(i)+w);
    if~(isequal(new_seq, old_seq))
      warning('sequence not identical')
    end
    %%check consensus
    if blocks.strand=='+'
      %%ACCEPTOR CONSENSUS
      if segments(i,3)==3&segments(i-1,3)==7         
        if ~(isequal(old_chr_seq(new_pos(i)+[-2:-1]),'AG'))
          warning('invalid acceptor\n')
          %keyboard
        end
      end
      %%DONOR CONSENSUS                                                                      
      if segments(i,3)==3&segments(i+1,3)==7                                                             
        if ~(isequal(old_chr_seq(new_pos2(i)+[0:1]),'GT')|isequal(old_chr_seq(new_pos2(i)+[0:1]),'GC'))    
          warning('invalid donor\n')
        end    
      end
      %%TIS
      if segments(i,3)==3&(segments(i-1,3)==1|segments(i-1,3)==2)
        if ~(isequal(old_chr_seq(new_pos(i)+[0:2]),'ATG'))
          warning('invalid TIS\n')                                                              
          keyboard                                                                       
        end                                                                       
      end                                                       
      %%STOP
      if segments(i,3)==3&(segments(i+1,3)==4|segments(i+1,3)==5)
        if ~(isequal(old_chr_seq(new_pos2(i)+[0:2]),'TAA')| ...
             isequal(old_chr_seq(new_pos2(i)+[0:2]),'TGA')|...
             isequal(old_chr_seq(new_pos2(i)+[0:2]),'TAG'))
          warning('invalid STOP\n')
        end
      end       
    else %%strand=='-'
      if segments(i,3)==3&segments(i-1,3)==7       
        assert(isequal(old_chr_seq(new_pos(i)+[1:2]),'CT'))
      end
      if segments(i,3)==3&segments(i+1,3)==7
        if ~(isequal(old_chr_seq(new_pos2(i)+[-1:0]),'AC')|isequal(old_chr_seq(new_pos2(i)+[-1:0]),'GC'))
          warning('invalid donor\n')
        end
      end
      %%TIS          
      if segments(i,3)==3&(segments(i-1,3)==1|segments(i-1,3)==2)
        if ~(isequal(old_chr_seq(new_pos(i)+[-2:0]),'CAT')) 
          warning('invalid tis\n')                                                                          
          keyboard 
        end                                                                          
      end 
      %%STOP                                                                                                                         
      if segments(i,3)==3&(segments(i+1,3)==4|segments(i+1,3)==5)
        if ~(isequal(old_chr_seq(new_pos2(i)+[-2:0]),'TTA')|... 
             isequal(old_chr_seq(new_pos2(i)+[-2:0]),'TCA')|...
             isequal(old_chr_seq(new_pos2(i)+[-2:0]),'CTA'))
             warning('invalid stop\n')                                                                          
        end 
      end               
    end
  end
  if blocks.strand=='+'                                                                  
    segments(:,1) = new_pos;                                                                     
    segments(:,2) = new_pos2; 
  else
    segments(:,1) = length(old_chr_seq) - new_pos+2;        
    segments(:,2) = length(old_chr_seq) - new_pos2+2;
  end              
  blocks.prediction.segments = segments;
  blocks.genestr = old_chr_seq;
  
  save(fn_out{f}, 'blocks','shift','pos');
  % clear blocks
end
