
function [ mapA2c, mapB2c, info ] = match_items_via_overlap( itemsA, itemsB );


% === prepare itemsA
nofItemsA = length( itemsA );
sA = [ itemsA.start ];
tA = [ itemsA.stop ];
assert_all( sA < tA );
assert_all( sA(1:(end-1)) <= sA(2:end) );
%assert_all( tA(1:(end-1)) <= tA(2:end) );
%%% if( any( sA(1:(end-1)) == sA(2:end) ) )
%%%   fprintf( 'WARNING: several items with same start!\n' );
%%%   idx = find( sA(1:(end-1)) == sA(2:end) );
%%%   for( i = idx )
%%%     itemsA( i )
%%%     itemsA( i+1 )
%%%   end;
%%% end;


% === prepare itemsB
nofItemsB = length( itemsB );
sB = [ itemsB.start ];
tB = [ itemsB.stop ];
assert_all( sB < tB );
assert_all( sB(1:(end-1)) <= sB(2:end) );
%assert_all( tB(1:(end-1)) <= tB(2:end) );


% === match items
nofItems = nofItemsA + nofItemsB;
s = [ sA , sB ];
t = [ tA , tB ];
assert_all( size(s) == [1,nofItems] );
assert_all( size(t) == [1,nofItems] );
adjMatS = sparse( [], [], [], nofItems, nofItems );
adjMatT = sparse( [], [], [], nofItems, nofItems );
for( i = 1:nofItems )
  s_i = s(i);
  t_i = t(i);
  jS = find( ( s_i<=s & s<=t_i ) | ( s<=s_i & s_i<=t ) );
  jT = find( ( s_i<=t & t<=t_i ) | ( s<=t_i & t_i<=t ) );
  adjMatS( i, jS ) = 1;
  adjMatT( i, jT ) = 1;
end;
assert_all( adjMatS == adjMatT );
adjMat = adjMatS;
clear adjMatS;
clear adjMatT;
assert_all( adjMat == adjMat' );
assert_all( diag(adjMat) == 1 );


% === cluster items
nofRedItems = 0;
mapAb2c = repmat( nan, nofItems, 3 );
for( i = 1:nofItems )
  if( ~isnan(mapAb2c(i,1)) )
    continue;
  end;
  js = find( adjMat(i,:) == 1 );
  ns = find( adjMat(i,:) == 0 );
  assert( any( js == i ) );
  jsA = js( js <= nofItemsA );
  jsB = js( js >  nofItemsA );
  % --- single fully connected cluster?
  M = adjMat( js, js );
  N = adjMat( js, ns );
  if( all(M(:)==1) & all(N(:)==0) )
    nofRedItems = nofRedItems + 1;
    mapAb2c(js,1) = nofRedItems;
    mapAb2c(js,2) = length( jsA );
    mapAb2c(js,3) = length( jsB );
    continue;
  end;
  % --- single item set involved, but not fully connected?
  if( isempty(jsA) | isempty(jsB) )
    % in principle, must now look for fully connected subsets...
    % being lazy, now just register item as singleton
    % - first check for consistency
    idx = js( ~isnan( mapAb2c(js,1) ) );
    assert_all( mapAb2c(idx,2:3) == 0 );
    % - then register item as singleton
    nofRedItems = nofRedItems + 1;
    mapAb2c(i,:) = [ nofRedItems , 0 , 0 ];
    continue;
  end;
  % --- both item sets involved; check separately.
  MA = adjMat( jsA, jsA );
  MB = adjMat( jsB, jsB );
  NA = adjMat( jsA, ns(ns<=nofItemsA) );
  NB = adjMat( jsB, ns(ns> nofItemsA) );
  if( all(MA(:)==1) & all(NA(:)==0) )
    nofRedItems = nofRedItems + 1;
    mapAb2c(jsA,1) = nofRedItems;
    mapAb2c(jsA,2) = length(jsA);
    mapAb2c(jsA,3) = 0;
  elseif( i <= nofItemsA )
    idx = jsA( ~isnan( mapAb2c(jsA,1) ) );
    assert_all( mapAb2c(idx,2:3) == 0 );
    nofRedItems = nofRedItems + 1;
    mapAb2c(i,:) = [ nofRedItems , 0 , 0 ];
  end;
  if( all(MB(:)==1) & all(NB(:)==0) )
    nofRedItems = nofRedItems + 1;
    mapAb2c(jsB,1) = nofRedItems;
    mapAb2c(jsB,2) = 0;
    mapAb2c(jsB,3) = length(jsB);
  elseif( i > nofItemsA )
    idx = jsB( ~isnan( mapAb2c(jsB,1) ) );
    assert_all( mapAb2c(idx,2:3) == 0 );
    nofRedItems = nofRedItems + 1;
    mapAb2c(i,:) = [ nofRedItems , 0 , 0 ];
  end;
end;
% --- split joint cluster map
assert_all( ~isnan(mapAb2c) );
mapA2c = mapAb2c( 1:nofItemsA, : );
mapB2c = mapAb2c( nofItemsA+(1:nofItemsB), : );


% === return additional information
info = [];
info.nofItems = nofItems;
info.nofRedItems = nofRedItems;
% --- cluster counts (set-wise)
info.nofRedItemsA = length( unique( mapA2c(:,1) ) );
info.nofRedItemsB = length( unique( mapB2c(:,1) ) );
idxClustA = find( mapA2c(:,2) > 1 );
idxClustB = find( mapB2c(:,3) > 1 );
info.nofClustersA = length( unique( mapA2c(idxClustA,1) ) );
info.nofClustersB = length( unique( mapB2c(idxClustB,1) ) );
% --- count of mappings
info.idxJointA = find( mapA2c(:,3) > 0 );
info.idxJointB = find( mapB2c(:,2) > 0 );
assert_all( mapA2c(info.idxJointA,2) > 0 );
assert_all( mapB2c(info.idxJointB,3) > 0 );
nofJointA = length( unique( mapA2c(info.idxJointA,1) ) );
nofJointB = length( unique( mapB2c(info.idxJointB,1) ) );
assert( nofJointA == nofJointB );
info.nofMapped = nofJointA;
% --- edges
info.adjMat = adjMat;
info.nofEdges = full( sum( adjMat(:) ) ) - nofItems;
idx = find( mapAb2c(:,2) + mapAb2c(:,3) > 1 );
info.clusterIds = unique( mapAb2c(idx,1) );
%fprintf( '[%d clusters; %d in A, %d in B]\n', length(info.clusterIds), info.nofClustersA, info.nofClustersB );
nofEdgesInCliques = 0;
for( clusterId = info.clusterIds' )
  idxClust = find( mapAb2c(:,1) == clusterId );
  clustSize = length( idxClust );
  assert( clustSize > 1 );
  assert_all( adjMat(idxClust,idxClust) == 1 );
  adjMat(idxClust,idxClust) = 0;
  nofEdgesInCliques = nofEdgesInCliques + clustSize^2 - clustSize;
end;
info.nofSingletons = full( sum( diag(adjMat) ) );
adjMat = adjMat - diag(diag(adjMat));
assert_all( diag(adjMat) == 0 );
info.nofEdgesInCliques = nofEdgesInCliques;
info.nofEdgesRemaining = full( sum( adjMat(:) ) );
clear adjMat;

