function blocks = construct_block_predictions(PAR)

% blocks = construct_block_predictions(PAR)
% loads a gene structure from PAR.fn_pred_genes with gene predictions and
% maps them to the appropraite block (blocks loaded from file
% PAR.fn_blocks).
% If blocks has already fields "prediction" or "pred", they will be
% overwritten. Final structure blocks is saved to file PAR.fn_blocks_save
% PAR.model additionally needed. 
% 
% functions called: gene2segments
%  
% see also make_model.m  
%  
% GZ (modified by GS 19/01/2006) 
  
  
addpath ~/svn/projects/nGASP/LSL/
model = PAR.model;

%-- load predictions --%
load(PAR.fn_pred_genes, 'genes');
gene_strands = [genes(:).strand];

%-- load block structure --%
blocks = arrayload2(PAR.fn_blocks, 'blocks');
model = PAR.model;
if isfield(blocks,'predictions')
  blocks = rmfield(blocks,'predictions')
end
if isfield(blocks,'pred')
  blocks = rmfield(blocks,'pred')
end
blocks(1).pred = struct;
blocks(1).prediction = struct;

genes_covered = [];
%-- map predicted genes to blocks
for b=1:length(blocks),
  block = blocks(b);
  if isfield(block,'range')
    b_start = block.range(1);
    b_stop  = block.range(2);
  else
    if (block.strand == '+'),
      b_start = block.block_start;
      b_stop  = block.block_end;
    else
      b_start = block.block_end;
      b_stop  = block.block_start;
    end
  end
  pred = struct;
  pred.ids = [];
  str_idx = find(gene_strands==block.strand);
  for i=1:length(str_idx),
    gene = genes(str_idx(i));
    if equal(gene.chr, block.chr) & b_start<gene.start & gene.stop<b_stop,
      pred.ids(end+1) = str_idx(i);
      genes_covered = [genes_covered, str_idx(i)];
    end
  end
  block.pred = pred;
  blocks(b) = block;
  
  fprintf('  processed %i blocks\r',b);
end
fprintf('\n\n');

% assert(length(unique(genes_covered))==length(genes_covered))
%-- attach gene segmentations to block.pred (local coordinates)
for b=1:length(blocks),
  block     = blocks(b);
  pred      = block.pred;
  strand    = block.strand;
  offset    = block.offset;
  % block_len = abs(offset-block.block_end);
  block_len = length(block.genestr);
  gene_starts = zeros(1,length(pred.ids));
  if length(pred.ids)>0,
    for i=1:length(pred.ids)
      pred.genes{i} = gene2segments(genes(pred.ids(i)),strand,offset,model);
      gene_starts(i) = pred.genes{i}(1,1);
    end
    [gene_starts order] = sort(gene_starts);
    pred.genes = {pred.genes{order}};
    pred.ids   = pred.ids(order);
    % remove duplicate genes (YES, some identical predictions
    % appear more than once under different names...)
    rm_idx = [];
    for i=2:length(pred.genes),
      if all(all(equal(pred.genes{i},pred.genes{i-1}))),
        rm_idx = [rm_idx i];
        warning(sprintf('removed duplicate %i (%s identical to %s)', ...
                        i, genes(pred.ids(i)).name, genes(pred.ids(i-1)).name));
      end
    end
    pred.genes(rm_idx)  = [];
    pred.ids(rm_idx)    = [];
    gene_starts(rm_idx) = [];
    % test for overlaps and remove shorter isoform
    rm_idx = [];
    for i=1:length(pred.genes),
      for j=i+1:length(pred.genes),
        if pred.genes{j}(1,1)<=pred.genes{i}(end,2),
          len_i = pred.genes{i}(end,2)-pred.genes{i}(1,1);
          len_j = pred.genes{j}(end,2)-pred.genes{j}(1,1);
          if len_i < len_j,
            rm_idx = [rm_idx i];
          else
            rm_idx = [rm_idx j];
          end
          warning(sprintf('Overlapping predictions (%s and %s)! Removed shorter isoform %i', ...
                          genes(pred.ids(i)).name, genes(pred.ids(j)).name), rm_idx(end));
        end
      end
    end
    pred.genes(rm_idx)  = [];
    pred.ids(rm_idx)    = [];
    gene_starts(rm_idx) = [];
  else
    pred.genes = {};
  end   
  upstream_gene_end = 1;
  pred.segments = [];
  for i=1:length(gene_starts)
    pred.segments = [pred.segments; ...
                     [upstream_gene_end gene_starts(i) model.segments.intergenic]; ...
                     pred.genes{i}];   
    upstream_gene_end = pred.genes{i}(end,2);
  end
  pred.segments = [pred.segments; ...
                   [upstream_gene_end block_len model.segments.intergenic]];
  % assert proper segmentation
  assert(all(pred.segments(:,1) <= pred.segments(:,2)));
  assert(all(pred.segments(2:end,1) == pred.segments(1:end-1,2)));

  block.pred = pred;
  block.prediction = pred;
  blocks(b) = block;
  
  fprintf('  processed %i blocks\r', b);
end
fprintf('\n\n');


arraysave2(PAR.fn_blocks_save, blocks,'blocks');


