function res = evaluate_block_predictions(PAR);

% res = evaluate_block_predictions(PAR)
% to evaluate predicticion against annotation
% Load blocks from file PAR.fn_blocks:
% Blocks must have a field pred, a struct that resembles truth in
% that it has the fields segments and genes (segments per individual
% gene).
% If pred is a vector of several predictions, pred(end) will be evaluated.
%
% additional fields PAR.model       (see make_model.m for details)
%                   PAR.verb        (output level, default 2)
%                   PAR.mask_on     (could be 'confirmed','unconfirmed','genic','all'; 
%                                    default: 'all')
%
% returns res       res.nucleotides (error  on nucleotide level: 
%                                    SN: sensitivity, SP:specificity,
%                                    number of observed,... )
%                   res.cds_exons   (error  on coding exons level:
%                   res.transcripts (error  on transcript level:
%
% functions called eval_segmentation
%   
% GZ (modified by GS 19/01/2006) 

  
addpath ~/svn/projects/nGASP/LSL/
addpath ~/svn/projects/nGASP/viz/

blocks = arrayload2(PAR.fn_blocks,'blocks')

if ~isfield(PAR,'verb'),
  verb = 2;
else
  verb = PAR.verb;
end

model = PAR.model ;

if ~isfield(PAR,'mask_on')
  mask_on == []
else
  mask_on = PAR.mask_on;
end

% here we count the number of nucleotides in coding exons
nucleotides.num_obsv = 0;
nucleotides.num_pred = 0;
nucleotides.num_corr = 0;

% here we count coding exon entities
cds_exons.num_obsv = 0;
cds_exons.num_pred = 0;
cds_exons.num_corr = 0;
cds_exons.ME = 0; % missing exons (no overlap to pred. cds)
cds_exons.WE = 0; % wrong exons (no overlap to annotated cds)

% here we count whole transcripts (= genes, if not alt. spliced)
transcripts.num_obsv = 0;
transcripts.num_pred = 0;
transcripts.num_corr = 0;
transcripts.MG = 0; % missing gene(no overlap to pred. cds)
transcripts.WG = 0; % wrong gene (no overlap to annotated cds)
% here we count tis 
tis.num_obsv = 0;
tis.num_pred = 0;
tis.num_corr = 0;

% here we count stop 
stop.num_obsv = 0;
stop.num_pred = 0;
stop.num_corr = 0;

% here we count merged 
merged.num_pred = 0;

% here we count cut genes
cut.num_pred = 0;

IDS(length(blocks)).false_trsc = [];
IDS(length(blocks)).false_tis = [];
IDS(length(blocks)).false_stop = [];
IDS(length(blocks)).cut_gene = [];
IDS(length(blocks)).merged_gene = [];
IDS(length(blocks)).wrong_gene = [];
IDS(length(blocks)).missed_gene = [];

% evaluation of predictions
for b = 1:length(blocks),
  block = blocks(b);
  % block_len = abs(block.block_end-block.block_start)+1;
  block_len = length(block.genestr);
  % if isempty(blocks(b).gene_ids)
  %   blocks(b).id
  %   continue
  % end
  % flag confirmed segments for separate evaluation
  if isempty(mask_on),
    mask = [1 block_len];
  else
    if equal(mask_on, 'confirmed'),
      idx = find(block.gene_confirmed==1);
      mask = [block.gene_pos(idx,1)-500, ...
              block.gene_pos(idx,2)+500];
    elseif equal(mask_on, 'genic'),
      mask = [block.gene_pos(:,1)-500, ...
              block.gene_pos(:,2)+500];
    elseif equal(mask_on, 'unconfirmed'),
      idx = find(block.gene_confirmed==1);
      conf = [block.gene_pos(idx,1)-500, ...
              block.gene_pos(idx,2)+500];
      mask = [];
      if isempty(conf),
        mask = [1 block_len];
      else
        start = 1;
        for i=1:size(conf,1),
          mask = [mask; [start conf(i,1)-1]];
          start = conf(i,2)+1;
        end
        mask = [mask; start block_len];
      end
    elseif equal(mask_on, 'all'),
      mask = [1 block_len];
    else
      error('unknown evidence %s!', varagin{1});
    end
  end
  % join overlapping mask intervals
  mask_str = zeros(1,block_len);
  for i=1:size(mask,1),
    mask_str(mask(i,1):mask(i,2)) = 1;
  end

  sta_idx = find(mask_str==1 & [0 mask_str(1:end-1)]~=1);
  stp_idx = find(mask_str==1 & [mask_str(2:end)   0]~=1);
  clear mask 
  mask = [sta_idx' stp_idx'];
  
  num_obs     = [];
  num_pred    = [];
  num_correct = [];
  num_missg   = [];
  num_wrong   = [];
  ID = [];
  [num_obs, num_pred, num_correct, num_missg, num_wrong,ID] = ...
      eval_segmentation(block.prediction, block.truth(1), model, mask);

  % check all truth entries and take most favourable one
  for t=2:length(block.truth),
    [num_obs(t), num_pred(t), num_correct(t), num_missg(t), num_wrong(t),ID(t)] = ...
        eval_segmentation(block.prediction, block.truth(t), model, mask);
  end
  
  best_t = 1;
  % use evaluation on exon level to decide which annotation to choose
  sn_best = num_correct(best_t).segm.cds_exon./(num_obs(best_t).segm.cds_exon+1e-8);
  sp_best = num_correct(best_t).segm.cds_exon./(num_pred(best_t).segm.cds_exon+1e-8);
  for t=2:length(block.truth),
    sn_new = num_correct(t).segm.cds_exon./(num_obs(t).segm.cds_exon+1e-8);
    sp_new = num_correct(t).segm.cds_exon./(num_pred(t).segm.cds_exon+1e-8);
    if sn_new+sp_new > sn_best+sp_best,
      best_t = t;
      sn_best = num_correct(best_t).segm.cds_exon./num_obs(best_t).segm.cds_exon;
      sp_best = num_correct(best_t).segm.cds_exon./num_pred(best_t).segm.cds_exon;
    end
  end
  
  IDS(b) = ID(best_t);
  
  %draw_pred(block.truth,block.prediction,model,mask)
  %keyboard
  
  % counts on nucleotide level
  nucleotides.num_obsv = nucleotides.num_obsv + num_obs(best_t).nucl.cds_exon;
  nucleotides.num_pred = nucleotides.num_pred + num_pred(best_t).nucl.cds_exon;
  nucleotides.num_corr = nucleotides.num_corr + num_correct(best_t).nucl.cds_exon;

  % counts on coding exon level (considers only exons of the
  % single most favourable transcript, not all exons in all transcripts)
  cds_exons.num_obsv = cds_exons.num_obsv + num_obs(best_t).segm.cds_exon;
  cds_exons.num_pred = cds_exons.num_pred + num_pred(best_t).segm.cds_exon;
  cds_exons.num_corr = cds_exons.num_corr + num_correct(best_t).segm.cds_exon;
  % missing exons (annotated exons with <1 bp overlap to predicted exons)
  cds_exons.ME = cds_exons.ME + num_missg(best_t).segm.cds_exon;
  % wrong exons (predicted exons with <1 bp overlap to annotated exons)
  cds_exons.WE = cds_exons.WE + num_wrong(best_t).segm.cds_exon;
  
  % counts on transcript level 
  transcripts.num_obsv = transcripts.num_obsv + num_obs(best_t).trsc;
  transcripts.num_pred = transcripts.num_pred + num_pred(best_t).trsc;
  transcripts.num_corr = transcripts.num_corr + num_correct(best_t).trsc;
  % missing transcript
  transcripts.MG = transcripts.MG + num_missg(best_t).trsc;
  % wrong transcript 
  transcripts.WG = transcripts.WG + num_wrong(best_t).trsc;
  
  % here we count tis 
  tis.num_obsv = tis.num_obsv + num_obs(best_t).tis;
  tis.num_pred = tis.num_pred + num_pred(best_t).tis ;
  tis.num_corr = tis.num_corr + num_correct(best_t).tis ;
  
  % here we count stop 
  stop.num_obsv = stop.num_obsv + num_obs(best_t).stop;
  stop.num_pred = stop.num_pred + num_pred(best_t).stop;
  stop.num_corr = stop.num_corr + num_correct(best_t).stop;
  
  % here we count merged 
  merged.num_pred = merged.num_pred + num_pred(best_t).merged_gene;
  
  % here we count merged 
  cut.num_pred = cut.num_pred + num_pred(best_t).cut_gene;
  
  
  if verb >= 1,
    %cds_exons
    fprintf('  processed %i blocks\r',b);
  end
end
if verb >= 1,
  fprintf('\n');
end
if verb >= 2,
  fprintf('\n');
end

% Compute sensitivity and specificity
nucleotides.SN = nucleotides.num_corr/nucleotides.num_obsv;
nucleotides.SP = nucleotides.num_corr/nucleotides.num_pred;

cds_exons.SN = cds_exons.num_corr/cds_exons.num_obsv;
cds_exons.SP = cds_exons.num_corr/cds_exons.num_pred;
cds_exons.ME = cds_exons.ME./cds_exons.num_obsv;
cds_exons.WE = cds_exons.WE./cds_exons.num_pred;

transcripts.SN = transcripts.num_corr/transcripts.num_obsv;
transcripts.SP = transcripts.num_corr/transcripts.num_pred;
transcripts.MG = transcripts.MG/transcripts.num_obsv;
transcripts.WG = transcripts.WG/transcripts.num_pred;

merged.pc = merged.num_pred/transcripts.num_obsv ;
cut.pc =  cut.num_pred/transcripts.num_obsv ;

tis.SN = tis.num_corr/tis.num_obsv;
tis.SP = tis.num_corr/tis.num_pred;

stop.SN = stop.num_corr/stop.num_obsv;
stop.SP = stop.num_corr/stop.num_pred;

% Print evaluation summary
if verb >= 2,
  fprintf('Evaluation\n');
  fprintf('  on nucleotide level:\n');
  fprintf('    sensitivity: %2.2f%%\n', 100*nucleotides.SN);
  fprintf('    specificity: %2.2f%%\n\n', 100*nucleotides.SP);
  
  fprintf('  on coding exon level:\n');
  fprintf('    sensitivity: %2.2f%% (missing exons: %2.2f%%)\n', ...
	  100*cds_exons.SN, 100*cds_exons.ME);
  fprintf('    specificity: %2.2f%% (wrong exons:   %2.2f%%)\n\n', ...
	  100*cds_exons.SP, 100*cds_exons.WE);
  
  fprintf('  on transcript level:\n');
  fprintf('    sensitivity: %2.2f%% (cut genes: %2.2f)\n', ...
          100*transcripts.SN, 100*cut.pc);
  fprintf('    specificity: %2.2f%% (merged genes: %2.2f)\n\n'...
          , 100*transcripts.SP,100*merged.pc);
  
  fprintf('  TIS:\n');
  fprintf('    sensitivity: %2.2f%%\n', 100*tis.SN);
  fprintf('    specificity: %2.2f%%\n\n', 100*tis.SP);
  
  fprintf('  STOP:\n');
  fprintf('    sensitivity: %2.2f%%\n', 100*stop.SN);
  fprintf('    specificity: %2.2f%%\n\n', 100*stop.SP);
end

% return results

res.nucleotides = nucleotides ;
res.cds_exons = cds_exons ;
res.transcripts = transcripts ;
res.tis = tis;
res.stop = stop;
res.merged = merged;
res.cut = cut;
res.IDS = IDS;
