
clear;
%paths;
evaluate_operons_params;



% === read confirmation status
load( fn_conf, 'genes' );
nofGenes = length( genes );
confMap = [];
confMap.names = cell( nofGenes, 1 );
confMap.confs = repmat( nan, nofGenes, 1 );
for( i = 1:nofGenes )
  t = genes(i).transcript_status;
  assert_all( t==-1 | t==0 | t==+1 );
  if( isempty(t) )
    t = -1;
    i
  end;
  confMap.names{i} = genes(i).name;
  confMap.confs(i) = round( mean(t) );
end;
clear genes;
clear nofGenes;



% === read annotated operons
fn_gff_gn = [ tmp_dir '/extracted_genes.'   suffix ];
fn_gff_op = [ tmp_dir '/extracted_operons.' suffix ];
fprintf( 'reading annotation: %s\n', fn_gff );
if( ~ exist(tmp_dir,'dir') )
  success = mkdir( 'data_processing_lsl', 'operons' );
  success = mkdir( 'data_processing_lsl/operons', annot );
end;
if( ~ exist(fn_gff_gn,'file') )
  com = sprintf( 'grep -E Coding_transcript[[:space:]]+gene %s > %s', fn_gff, fn_gff_gn );
  system( com );
end;
if( ~ exist(fn_gff_op,'file') )
  com = sprintf( 'grep operon %s > %s', fn_gff, fn_gff_op );
  system( com );
end;
% --- read annotation
[ allAnnoGenes ] = import_gff_subfile( fn_gff_gn, 'Coding_transcript', 'gene' );
[ allAnnoOpers ] = import_gff_subfile( fn_gff_op, 'operon', 'operon' );
% --- add gene confidences
n = length( allAnnoGenes );
for( i = 1:n )
  name = allAnnoGenes(i).name;
  j = strmatch( name, confMap.names );
  conf = nan;
  if( isempty(j) )
    fprintf( 'WARNING: no confidence for gene %s\n', name );
    conf = nan;
  end;
  if( length(j) == 1 )
    conf = confMap.confs(j);
  end;
  if( length(j) > 1 )
    fprintf( 'WARNING: %d confidences for gene %s\n', length(j), name );
    conf = confMap.confs(j);
    if( length(unique(conf)) == 1 )
      conf = unique( conf );
    else
      conf = nan;
    end;
  end;
  allAnnoGenes(i).conf = conf;
end;
% --- no operon confidences
n = length( allAnnoOpers );
for( i = 1:n )
  allAnnoOpers(i).conf = nan;
end;



% === collect predicted operons
load( fn_triv, 'trivial_regions' );
% --- PAR
load( fn_par, 'PAR' );
if( isfield(PAR,'cleave_offset') )
  if( exist('cleave','var') )
    assert( cleave == PAR.cleave_offset );
  else
    cleave = PAR.cleave_offset;
  end;
end;
model = PAR.model;
segments = PAR.model.segments;
PAR.organism.name = organismName;

% --- go thru contigs, ie chromosome/strand pairs
allData = [];
chrs = {};
chrLens = zeros( nofContigs, 1 );
counts = repmat( nan, nofContigs, 6 );
for( k = 1:nofContigs )
  fprintf( '\n' );
  
  % --- load predictions
  switch( version )
   case '170'
    fn_out1g = sprintf( '%s/genes_contig%i_cleave%1.2f.mat',  out_dir, k, cleave );
    fn_out1b = sprintf( '%s/concat_contig%i_cleave%1.2f.mat', out_dir, k, cleave );
   case '180'
    strands = '+-';
    kName = [ num2str(ceil(k/2)), strands(mod(k-1,2)+1) ];
    fn_out1g = sprintf( '%s/pred_genes%s_cleave=%3.2f.mat',  out_dir, kName, cleave );
    fn_out1b = sprintf( '%s/pred_blocks%s_cleave=%3.2f.mat', out_dir, kName, cleave );
   otherwise
    error( 'unknown genome annotation version' );
  end;
  load( fn_out1g, 'genes' );
  load( fn_out1b, 'blocks' );
  chr = blocks.chr;
  if( exist('chrMapKeys','var') )
    i = strmatch( chr, chrMapKeys );
    assert( length(i) == 1 );
    chr = chrMapVals{ i };
  end;
  strand = blocks.strand;
  assert( trivial_regions(k).strand == strand );
  fprintf( '=== chromosome/strand: %s%s ===\n', chr, strand );
  if( ~strcmp( trivial_regions(k).chr, chr ) )
    fprintf( 'WARNING: chromosom "%s" mapped to "%s".\n', trivial_regions(k).chr, chr );
  end;
  segs = blocks.prediction.segments;
  assert_all( segs(2:end,1) == segs(1:(end-1),2) );
  %%% if( strand == '-' )
  %%%   segs(:,[2,1]) = trivial_regions(k).stop - ( segs(:,[1,2]) - trivial_regions(k).start );
  %%%   %segs(:,[1,2]) = segs( end:-1:1, [1,2] );
  %%% end;
  chrLens( k ) = trivial_regions(k).stop;
  nofPredOpers = sum( segs(:,3) == segments.intergenic ) - 1;
  nofPredGenes = nofPredOpers + sum( segs(:,3) == segments.intercistronic );

  % --- find predicted operons and genes
  predGenes = [];
  predGenes( nofPredGenes, 1 ).chr = '';
  predGenes( nofPredGenes, 1 ).strand = '';
  predGenes( nofPredGenes, 1 ).name = '';
  predGenes( nofPredGenes, 1 ).start = 0;
  predGenes( nofPredGenes, 1 ).stop  = 0;
  predGenes( nofPredGenes, 1 ).segStart = 0;
  predGenes( nofPredGenes, 1 ).segStop  = 0;
  predGenes( nofPredGenes, 1 ).conf = nan;
  predOpers = [];
  predOpers( nofPredOpers, 1 ).chr = '';
  predOpers( nofPredOpers, 1 ).strand = '';
  predOpers( nofPredOpers, 1 ).name = '';
  predOpers( nofPredOpers, 1 ).start = 0;
  predOpers( nofPredOpers, 1 ).stop  = 0;
  predOpers( nofPredOpers, 1 ).segStart = 0;
  predOpers( nofPredOpers, 1 ).segStop  = 0;
  predOpers( nofPredOpers, 1 ).conf = nan;
  operStarts = find( segs(:,3) == segments.intergenic );
  predG2o = repmat( nan, nofPredGenes, 1 );
  predO2g = [];
  predO2g(nofPredOpers).fullGenes = {};
  iGene = 0;
  iOper = 0;
  for( i = 1:(length(operStarts)-1) )
    operSegs = segs( operStarts(i):operStarts(i+1), : );
    t = find( operSegs(:,3) == segments.intercistronic );
    if( isempty(t) )
      % -- singleton gene
      iGene = iGene + 1;
      predGenes(iGene).name = sprintf( '%s%sG%d', chr, strand, predGenes(iGene).start );
      predGenes(iGene).start = operSegs(1,2);
      predGenes(iGene).stop  = operSegs(end,1);
      predGenes(iGene).segStart = operStarts(i);
      predGenes(iGene).segStop  = operStarts(i+1);
      predG2o(iGene) = 0;
    else
      % -- true operon
      iOper = iOper + 1;
      predOpers(iOper).name = sprintf( '%s%sO%d', chr, strand, predOpers(iOper).start );
      predOpers(iOper).start = operSegs(1,2);
      predOpers(iOper).stop  = operSegs(end,1);
      predOpers(iOper).segStart = operStarts(i);
      predOpers(iOper).segStop  = operStarts(i+1);
      n = length(t) + 1;
      predG2o( iGene+(1:n) ) = iOper;
      predO2g(iOper).fullGenes = iGene + (1:n);
      predO2g(iOper).partialGenes = [];
      t = [ 1 ; t ; length(operSegs) ];
      for( j = 1:n )
	iGene = iGene + 1;
	predGenes(iGene).name = sprintf( '%s%sG%d', chr, strand, predGenes(iGene).start );
	predGenes(iGene).start = operSegs( t(j),   2 );
	predGenes(iGene).stop  = operSegs( t(j+1), 1 );
	predGenes(iGene).segStart = operStarts(i) + t(j);
	predGenes(iGene).segStop  = operStarts(i) + t(j+1);
      end;
    end;
  end;
  assert( iGene == nofPredGenes );
  assert( iOper <= nofPredOpers );
  nofPredOpers = iOper;
  predOpers = predOpers( 1:nofPredOpers );
  predO2g = predO2g( 1:nofPredOpers );
  assert_all( ~isnan(predG2o) );
  
  % --- correction for negative strand
  if( strand == '-' )
    t = trivial_regions(k).stop + trivial_regions(k).start;
    predGenes = predGenes( end:-1:1 );
    predOpers = predOpers( end:-1:1 );
    for( iOper = 1:nofPredOpers )
      start = t - predOpers(iOper).stop;
      stop  = t - predOpers(iOper).start;
      assert( start < stop );
      predOpers(iOper).start = start;
      predOpers(iOper).stop  = stop;
    end;
    for( iGene = 1:nofPredGenes )
      start = t - predGenes(iGene).stop;
      stop  = t - predGenes(iGene).start;
      assert( start < stop );
      predGenes(iGene).start = start;
      predGenes(iGene).stop  = stop;
    end;
  end;
  
  % --- fetch annotation
  idxGenes = find( strcmp(chr,{allAnnoGenes.chr}) & [allAnnoGenes.strand]==strand );
  idxOpers = find( strcmp(chr,{allAnnoOpers.chr}) & [allAnnoOpers.strand]==strand );
  annoGenes = allAnnoGenes( idxGenes );
  annoOpers = allAnnoOpers( idxOpers );
  nofAnnoGenes = length( annoGenes );
  nofAnnoOpers = length( annoOpers );
  
  % --- clean annotated operons
  annoOperLens = [annoOpers.stop] - [annoOpers.start];
  idx = find( annoOperLens > maxOperonLength );
  if( ~isempty(idx) )
    for( i = idx )
      oper = annoOpers( i );
      fprintf( 'WARNING: operon %s discarded as it is too long (%d).\n', oper.name, annoOperLens(i) );
    end;
    annoOpers(idx) = [];
    nofAnnoOpers = nofAnnoOpers - length(idx);
    assert( length(annoOpers) == nofAnnoOpers );
  end;
  
  % --- match annotated genes to annotated operons
  annoG2o = repmat( 0, nofAnnoGenes, 1 );
  annoO2g = [];
  annoO2g(nofAnnoOpers).fullGenes = {};
  annoO2g(nofAnnoOpers).partialGenes = {};
  s = [ annoGenes.start ];
  t = [ annoGenes.stop ];
  for( iOper = 1:nofAnnoOpers )
    oper = annoOpers( iOper );
    idx = find( oper.start <= s & t <= oper.stop );
    %assert( length(idx) >= 1 );
    if( isempty(idx) )
      fprintf( 'WARNING: operon %s has no genes.\n', oper.name );
    else
      annoG2o(idx) = iOper;
    end;
    annoO2g(iOper).fullGenes = idx;
    idx1 = find( s < oper.start & oper.start <= t );
    idx2 = find( s <= oper.stop & oper.stop < t );
    idx = [ idx1 , idx2 ];
    annoO2g(iOper).partialGenes = idx;
    if( length(idx) > 0 )
      fprintf( 'WARNING: operon %s [%d-%d] has %d partial overlaps with genes.\n', oper.name, oper.start, oper.stop, length(idx) );
      for( i = idx )
	gene = annoGenes( i );
	fprintf( 'WARNING:   gene %s [%d-%d].\n', gene.name, gene.start, gene.stop );
      end;
    end;
  end;
  fprintf( '\n' );
  
  % --- output info
  annoGenesInOpers = [ annoO2g.fullGenes ];
  predGenesInOpers = [ predO2g.fullGenes ];
  nofAnnoGenesInOpers = length( unique( annoGenesInOpers ) );
  nofPredGenesInOpers = length( unique( predGenesInOpers ) );
  fprintf( 'annotated: %4d genes; %4d genes are in %3d operons\n', nofAnnoGenes, nofAnnoGenesInOpers, nofAnnoOpers );
  fprintf( 'predicted: %4d genes; %4d genes are in %3d operons\n', nofPredGenes, nofPredGenesInOpers, nofPredOpers );
  counts( k, 1:3 ) = [ nofAnnoGenes, nofAnnoGenesInOpers, nofAnnoOpers ];
  counts( k, 4:6 ) = [ nofPredGenes, nofPredGenesInOpers, nofPredOpers ];

  % --- unify and collect data for storage
  for( i = 1:nofPredGenes )
    predGenes(i).chr = chr;
    predGenes(i).strand = strand;
    predGenes(i).source = mGene;
    predGenes(i).type = 'gene';
  end;
  for( i = 1:nofPredOpers )
    predOpers(i).chr = chr;
    predOpers(i).strand = strand;
    predOpers(i).source = mGene;
    predOpers(i).type = 'operon';
  end;
  for( i = 1:nofAnnoGenes )
    annoGenes(i).segStart = 0;
    annoGenes(i).segStop = 0;
    annoGenes(i).source = annot;
    annoGenes(i).type = 'gene';
  end;
  for( i = 1:nofAnnoOpers )
    annoOpers(i).segStart = 0;
    annoOpers(i).segStop = 0;
    annoOpers(i).source = annot;
    annoOpers(i).type = 'operon';
  end;
  allData = [ allData ; predGenes ];
  allData = [ allData ; predOpers ];
  allData = [ allData ; annoGenes ];
  allData = [ allData ; annoOpers ];
  % --- chromosomes
  if( strand == '+' )
    chrs{end+1} = chr;
  end;
  
end;
fprintf( '\n' );



% === finish
chrLens = reshape( chrLens, 2, length(chrs) )';
counts(nofContigs+1,:) = mean( counts, 1 );

% --- prepare figure legend
contigNames = cell( nofContigs+1, 1 );
for( i = 1:length(chrs) )
  contigNames{ 2*i-1 } = [ chrs{i} '+' ];
  contigNames{ 2*i   } = [ chrs{i} '-' ];
end;
contigNames{ nofContigs+1 } = 'mean';

% --- store data
fn_allData = [ tmp_dir , '/genesAndOperons.mat' ];
save( fn_allData, 'allData', 'chrs', 'chrLens', 'counts', 'contigNames' );



