function new_block = reconcatinate_blocks_new(blocks,model,genome_info,block_overlap,test_regions)
% blocks = reconcatinate_blocks_new(blocks,model,genome_info,block_overlap)
%
% assumes that all blocks are (on the positive strand) succeding regions
% on one chromome. Blocks may overlap with 2*block_overlap. A single New_block is
% returned, which is a concatination of all blocks in local
% coordinates. Predictions in the overlapping regions are combined such
% that valid genes without overlap are generated. 





if nargin<4
  block_overlap=0
end

%-----------------------------

intergenic_seg_id = model.segments.intergenic;
if isfield(model.segments,'intergenictrans') 
  intergenic_seg_id = [intergenic_seg_id model.segments.intergenictrans] ;
end
assert(length(unique([blocks.strand]))==1);
assert(length(unique([blocks.chr_num]))==1); 
 
strand = blocks(1).strand;
new_block.chr = blocks(1).chr;
new_block.chr_num = blocks(1).chr_num;
new_block.strand = strand;
new_block.id = blocks(1).id;
new_block.start = blocks(1).start;
new_block.stop = blocks(end).stop;

if nargin<5
  test_regions = [];
else
  test_regions = test_regions([test_regions.chr_num]==blocks(1).chr_num&[test_regions.strand]==strand);
end


if strand== '+'   
  new_block.offset = blocks(1).offset;
  block_sign = 1;
  block_ids = [1:length(blocks)];
else
  new_block.offset = blocks(end).offset; 
  block_sign = -1;
  % start with right most block; will be the beginning of the new block!
  block_ids = [length(blocks):-1:1];
end
% new_block.stop = blocks(1).offset + length(blocks(1).genestr)-1;
len = abs(new_block.stop-new_block.start)+1;

% re-concatenate
segments = [];

genestr  = repmat('Z', 1, len);
if ~isempty(test_regions)
  if strand=='+'
    for i=1:length(test_regions)
      genestr(test_regions(i).start:test_regions(i).stop)='X';
    end
  else
    for i=1:length(test_regions)
      genestr(len+1-[test_regions(i).start:test_regions(i).stop])='X';
    end
  end
end

num_blocks = 0;

for b=block_ids,
  test = [];
  if ~isempty(test_regions)
    test = test_regions([test_regions.start]<=blocks(b).start&[test_regions.stop]>=blocks(b).stop);
    assert(length(test)==1)
  end
  num_blocks = num_blocks + 1;
  if ~isfield(blocks(b).prediction,'segments')
    blocks(b).prediction.segments = [1 length(blocks(b).seq) 0]
    b
    warning('no prediction found')
  end
  seg = blocks(b).prediction.segments;
  %% check seg positions
  [idx1,idx2] = find(seg(:,1:2)==0);
  if ~isempty(idx1) 
    for jj=1:length(idx1)
      seg(idx1(jj),idx2(jj))=1;    
    end  
    warning('position = 0 found')
    assert(all(seg(:,2)-seg(:,1)>=0))
  else   
    assert(all(seg(:,2)-seg(:,1)>=0))   
  end
  assert(issorted(seg(:,1)))
  assert(issorted(seg(:,2)))
  assert(all(seg(2:end,1)-seg(1:end-1,2)==0))
  %% find indices of first segment which has no overlap with preciding block 
  if num_blocks==1
    idx_l = 1;
  elseif (~isempty(test)&test.start==blocks(b).start & strand=='+')|...
        (~isempty(test)&test.stop==blocks(b).stop & strand=='-')
    idx_l = 1;
  else
    idx_l = min(find(seg(:,1)>=2*block_overlap));
    if isempty(idx_l)
      assert(size(seg,1)==1)
      seg(2,:) = [2*block_overlap seg(1,2:3)];
      seg(1,2) = 2*block_overlap;
      idx_l = min(find(seg(:,1)>=2*block_overlap));
    end
  end  
  %% find indices of last segment which has no overlap with succiding block 
  if num_blocks==length(blocks)
    idx_r = size(seg,1);
  elseif (~isempty(test)&test.stop==blocks(b).stop & strand=='+')|...
        (~isempty(test)&test.start==blocks(b).start & strand=='-')
    idx_r = size(seg,1);
  else
    idx_r = max(find(seg(:,2)<=length(blocks(b).seq)-2*block_overlap));
    if isempty(idx_r)
      assert(size(seg,1)==1)
      seg(2,:) = [length(blocks(b).seq)-2*block_overlap seg(1,2:3)];
      seg(1,2) = length(blocks(b).seq)-2*block_overlap;
      idx_r = max(find(seg(:,2)<=length(blocks(b).seq)-2*block_overlap));
    end
  end

  
  %% convert to global coordinates
  seg(:,1:2) = block_sign*seg(:,1:2) + blocks(b).offset;
  %% convert to new local coordinates
  seg(:,1:2) = block_sign*(seg(:,1:2) - new_block.offset); 

  assert(issorted(seg(:,1)))
  assert(issorted(seg(:,2)))
  assert(all(seg(2:end,1)-seg(1:end-1,2)==0))
                
  old_end = seg(end,2);
  if num_blocks==1,
    %% only add segments which are not in right overlap of blocks
    segments = seg(1:idx_r,:);
  else
    if ~isempty(test)&...
          (test.start==blocks(b).start& strand=='+'|...
          test.stop==blocks(b).stop& strand=='-')
      idx_seq = block_sign*(-new_block.offset+blocks(b).offset)+[1:length(blocks(b).seq)];
      assert(all(genestr(idx_seq)=='X'))
    else
      idx_seq = block_sign*(-new_block.offset+blocks(b).offset)+[1:block_overlap*2];
      assert(isequal(genestr(idx_seq),blocks(b).seq(1:block_overlap*2) ))
    end
    

    %% check for segments in left block overlap
    seg_inoverlap = seg(1:idx_l-1,:);
    seg_good = seg(idx_l:idx_r,:);
    
    %% no overlap (distinct test regions)
    if isempty(seg_inoverlap)
      seg_spacer = [segments(end,2) seg_good(1,1) Inf];
      seg_good = [seg_spacer; seg_good];
    else
      assert(ismember(seg_inoverlap(1,3),intergenic_seg_id));
      assert(old_seg_inoverlap(end,3)==model.segments.intergenic)
    
      
      %% just one overlapping segment,has to be intergenic 
      if size(seg_inoverlap,1)==1& size(old_seg_inoverlap,1)==1 
        % assert(seg_inoverlap(1,3) ==old_seg_inoverlap(1,3));
        if  size(seg_good,1)==1 &seg_good(1,3)==0
          seg_good = [old_seg_inoverlap(1,1) seg_good(1,2:3)];
        else
          seg_good = [[old_seg_inoverlap(1,1) seg_inoverlap(1,2:3)];seg_good];
        end
        assert(seg_good(1,1)==segments(end,2));
        fprintf('all good\n')
        
      %% overlapping segments are identical
      elseif size(seg_inoverlap,1)==size(old_seg_inoverlap,1)&...
            all(all(seg_inoverlap(2:end-1,:) ==old_seg_inoverlap(2:end-1,:)))&...
            all(seg_inoverlap(1,2:3) ==old_seg_inoverlap(1,2:3))&...
            all(seg_inoverlap(end,[1 3]) ==old_seg_inoverlap(end,[1 3]))
        warning('genes in overlap; but both blocks agree')       
        seg_inoverlap(1,1) = old_seg_inoverlap(1,1);
        seg_good = [seg_inoverlap; seg_good];
        assert(seg_good(1,1)==segments(end,2));
      
      % leftmost gene of new block starts in overlap
      elseif size(old_seg_inoverlap,1)==1&ismember(old_seg_inoverlap(1,3),intergenic_seg_id)&...
            seg_inoverlap(1,2)-seg_inoverlap(1,1)>=1000 
        
        seg_inoverlap(1,1)=old_seg_inoverlap(1,1);
        seg_good = [seg_inoverlap; seg_good];
        
      % right most gene of old block ends in overlap
      elseif size(seg_inoverlap,1)==1&ismember(seg_inoverlap(1,3),intergenic_seg_id)&...
            old_seg_inoverlap(end,2)-old_seg_inoverlap(end,1)>=1000
        
        old_seg_inoverlap(end,2) = seg_inoverlap(end,2);
        seg_good = [old_seg_inoverlap; seg_good];
        
        % leftmost gene of new block starts in overlap and right most gene of old block ends in overlap
      elseif size(seg_inoverlap,1)~=1&size(old_seg_inoverlap,1)~=1 &...
            seg_inoverlap(1,2)-old_seg_inoverlap(end,1)>0
        
        seg_inoverlap(1,1)=old_seg_inoverlap(end,1);
        seg_good = [old_seg_inoverlap(1:end-1,:); seg_inoverlap; seg_good];   
        % warning('gene goes over both blocks')
        % keyboard
      else
        % keyboard
        warning('disagreement in overlap')
        % view_overlapping_predictions(old_seg_inoverlap,seg_inoverlap,model)
        if all(seg_inoverlap(1,2:3) ==old_seg_inoverlap(1,2:3))
          seg_inoverlap(1,1)=old_seg_inoverlap(1,1);
        end
        if all(seg_inoverlap(end,[1 3]) ==old_seg_inoverlap(end,[1 3]))
          old_seg_inoverlap(end,2)=seg_inoverlap(end,2);
        end
        [tmp,idx_good,idx_good_old] = intersect(seg_inoverlap,old_seg_inoverlap,'rows');
        idx_bad = setdiff([1:size(seg_inoverlap,1)],idx_good);
        idx_bad_old = setdiff([1:size(old_seg_inoverlap,1)],idx_good_old);
        fprintf('%i of %i segments are disagreeing\n',length(idx_bad),size(seg_inoverlap,1))
        % pause
        % keyboard
        if min(idx_bad)>max(idx_good)&old_seg_inoverlap(end,2)-seg_inoverlap(idx_bad(1),2)<2000
          seg_good = [seg_inoverlap; seg_good]; 
        elseif max(idx_bad)<min(idx_good)
          assert(max(idx_bad_old)<min(idx_good_old))
          if old_seg_inoverlap(idx_bad_old(1),1)-seg_inoverlap(1,1)<2000
            seg_good = [old_seg_inoverlap; seg_good]; 
          else
            keyboard
          end
        elseif all(size(seg_inoverlap)==size(old_seg_inoverlap))&...
              all(isequal(seg_inoverlap(idx_good,:),old_seg_inoverlap(idx_good,:)))&...
              all(idx_bad(2:end)-idx_bad(1:end-1)==1)&...
              all(seg_inoverlap(idx_bad,3)== old_seg_inoverlap(idx_bad,3))
          assert(isequal(idx_good,idx_good_old))
          %% inner gene different predicted
          seg_good = [seg_inoverlap; seg_good]; 
        elseif ~isempty(idx_good_old) 
          old_seg_inoverlap_ = old_seg_inoverlap(1:idx_good_old(end),:);
          seg_inoverlap_ = seg_inoverlap(idx_good(end)+1:end,:);
          seg_good = [old_seg_inoverlap_;seg_inoverlap_; seg_good]; 
        else
          if ~ismember(old_seg_inoverlap(1,3),intergenic_seg_id) & ...
                ismember(seg_inoverlap(end,3),intergenic_seg_id)
             warning('disagreement!! taking old prediction')
             old_seg_inoverlap(end,2) = seg_inoverlap(end,2) ;
             seg_good = [old_seg_inoverlap; seg_good]; 
          elseif ismember(old_seg_inoverlap(1,3),intergenic_seg_id) & ...
                ~ismember(seg_inoverlap(end,3),intergenic_seg_id)
            warning('disagreement!! taking new prediction')
            seg_inoverlap(1,1)=old_seg_inoverlap(1,1);
            seg_good = [seg_inoverlap; seg_good]; 
          elseif  ismember(old_seg_inoverlap(1,3),intergenic_seg_id) & ...
                ismember(seg_inoverlap(end,3),intergenic_seg_id)
            warning('disagreement!! taking new prediction')
            seg_inoverlap(1,1)=old_seg_inoverlap(1,1);
            seg_good = [seg_inoverlap; seg_good]; 
          else
            keyboard
          end
        end
      end
    end %% if overlap is empty 
    assert(issorted(seg_good(:,1)));
    assert(issorted(seg_good(:,2)));
    if  size(seg_good,1)==1 &ismember(seg_good(1,3),intergenic_seg_id)&ismember(segments(end,3),intergenic_seg_id)
      assert(segments(end,2)==seg_good(1,1))
      segments(end,2) = seg_good(1,2);
    else
      segments = [segments;seg_good];
    end
    assert(issorted(segments(:,1)));
    assert(issorted(segments(:,2)));
    assert(all(segments(2:end,1)==segments(1:end-1,2)))
  end %% if first block
  old_seg_inoverlap = seg(idx_r+1:end,:);
  idx_seq = sort(block_sign*(-new_block.offset+[blocks(b).start:blocks(b).stop]));
  assert(all(genestr(idx_seq)~='Z'))
  genestr(idx_seq) = blocks(b).seq;
end %loop over blocks

%% check consensus of acceptor
idx = find(segments(2:end,3)==3&segments(1:end-1,3)==model.segments.intron)+1;                                                                    
if ~(all(genestr(segments(idx,1)-2)=='A'))                                                           
  warning('invalid acceptor')
  keyboard
end
if ~(all(genestr(segments(idx,1)-1)=='G'))
  warning('invalid acceptor')                                                                     
  keyboard                                                                 
end                                               
%% check consensus of donor
idx = find(segments(1:end-1,3)==3&segments(2:end,3)==model.segments.intron);
if ~(all(genestr(segments(idx,2))=='G'))
  warning('invalid donor') 
end 
if ~(all(genestr(segments(idx,2)+1)=='C'| ... 
         genestr(segments(idx,2)+1)=='T'))
  warning('invalid donor')                                                              
end  

assert(issorted(segments(:,1)));
assert(issorted(segments(:,2)));
assert(all(segments(2:end,1)==segments(1:end-1,2)))

new_block.prediction.segments = segments;
new_block.seq = genestr;


% keyboard