function [ res, param ] = findRelevantKmers( pos0, Contrib, param );
% [ res, param ] = findRelevantKmers( pos0, Contrib, param );

% === init
alphabet = 'ACGT';
alphabetSize = length( alphabet );
% --- prepare
[ nofKmers, seqLen ] = size( Contrib );
k_ = log(nofKmers) / log(alphabetSize);
k = round( k_ );
assert( abs(k-k_) < 1e-6 );
% --- default params
if( ~exist('param','var') )
  param = [];
end;
param = setDefault( param, 'quant', round( k*seqLen/10 ) );
param = setDefault( param, 'balanced', 0 );
param = setDefault( param, 'output', 1 ); 
param = setDefault( param, 'normalize', 0 );
param = setDefault( param, 'range', [1:size(Contrib,2)] );
param = setDefault( param, 'merge', 1 );
param = setDefault( param, 'per_kmer', 5);
param = setDefault( param, 'sort_by_position', 0);
param = setDefault( param, 'with_sign', 0); %0 all, -1 negative, +1 positive


% === find k-mers with high absolute contribution
res = [];

offs=min(param.range)-1;
Contrib=Contrib(:,param.range);

if param.normalize,
	Contrib=Contrib/max(Contrib(:));
end

% --- find relevant k-mers
[i,j] = find( abs(Contrib) );
contrib = Contrib( i + (j-1)*nofKmers );
position = j - pos0+offs;

% --- build k-mer strings
n = length( i );
kmers = repmat( '.', n, k );
i = i - 1;
for( l = k:-1:1 )
kmers(:,l) = alphabet( mod(i,alphabetSize) + 1 );
i = floor( i / alphabetSize );
end;
assert(all(i == 0));
% --- sort by absolute contibution
if param.with_sign~=0,
	contrib=contrib(sign(contrib)~=param.with_sign);
end

[ dummy, perm ] = sort( abs(contrib), 'descend' );

position = position( perm );
contrib  = contrib( perm );
kmers = kmers( perm, : );

if param.merge
  [kmers, dummy,j]=unique(kmers,'rows');
  positions=cell(1,size(kmers,1));
  contribs=cell(1,size(kmers,1));

  for i=1:length(j)
	  positions{j(i)}=[positions{j(i)} position(i)];
	  contribs{j(i)}=[contribs{j(i)} contrib(i)];
  end

  res.position = positions;
  res.contrib  = contribs;
  res.kmers = kmers;

  out=zeros(1,length(res.contrib));
  for i=1:length(res.contrib),

	  if param.sort_by_position
		  [dummy, idx]=sort(res.position{i});
	  else
		  [dummy, idx]=sort(abs(res.contrib{i}), 'descend');
	  end
	  res.contrib{i}=res.contrib{i}(idx);
	  res.position{i}=res.position{i}(idx);
	  if (length(res.contrib{i})>0)
		  [a,b]=max(abs(res.contrib{i}));
		  out(i)=sign(res.contrib{i}(b)) * a;
	  else
		  out(i)=0;
	  end
  end
  [dummy,idx]=sort(abs(out),'descend');
  res.kmers=res.kmers(idx,:);
  res.contrib=res.contrib(idx);
  res.position=res.position(idx);
else
  res.position = cell(length(position));
  res.contrib = cell(length(position));
  for i=1:length(position),
	  res.position{i}=position(i);
	  res.contrib{i}= contrib(i);
  end
  res.kmers = kmers;
end

ii=1:min(param.quant, length(res.position));
res.position = res.position(ii);
res.contrib  = res.contrib(ii);
res.kmers = res.kmers(ii,:);

% === output
if( param.output )
  if isfield(param, 'filename')
  	fid = fopen(param.filename,'w');
	fprintf(fid, 'File: %s\n\n', param.filename);
	fprintf(fid, 'Format description:\n');
	fprintf(fid, '  motif1: location1/score1 location2/score2 ...\n');
	fprintf(fid, '  motif2: location1/score1 location2/score2 ...\n');
	fprintf(fid, '  ...\n\n');
	fprintf(fid, 'Positive scores: Enhancer or Characteristic Motifs\nNegative Scores: Silencer or Atypical Motif\n');
	fprintf(fid, '=================================================\n\n');
  else
	fid = 1; %stdout
  end
  for( i = ii )
    fprintf(fid, '%s: ', res.kmers(i,:));

	for j=1:min(param.per_kmer,length(res.position{i})),
		if res.position{i}(j) < 0,
			sig1='-';
		else
			sig1='+';
		end
		if res.contrib{i}(j) < 0,
			sig2='-';
		else
			sig2='+';
		end
		fprintf(fid,  '%s%02d/%s%02.2f ', sig1, abs(res.position{i}(j)), sig2, abs(res.contrib{i}(j)) );
	end
	if (length(res.position{i})>param.per_kmer)
		fprintf(fid,  '...\n');
	else
		fprintf(fid,  '\n');
	end
    %fprintf( '%+4d  %s :  %8.2f\n', res.position{i}, res.kmers(i,:), res.contrib{i} );
    %fprintf( '%+4d  %s :  %8.2f\n', res.position(i), res.kmers(i,:), res.contrib(i) );
  end;
  if isfield(param, 'filename')
  	fclose(fid);
  end
end;
