function blocks = construct_block_predictions(source)

addpath ../LSL/

% get the model used for LSL
PAR.dir = '/fml/ag-raetsch/share/projects/nGASP/';
PAR.bins     = 20;   % number of plif bins
PAR.use_gc   = 1; 
PAR.fn_polya_signals = [PAR.dir 'polya/RESULTS_FOR_2nd_LAYER/' ...
                    'signal_sixmers.mat'];
load(PAR.fn_polya_signals,'signal_sixmers');
PAR.polya_sixmers = {signal_sixmers.sixmer};
model = make_model(PAR);

% load gene predictions
pred_file = ['/fml/ag-raetsch/share/projects/nGASP/gene_finders/' ...
            source '_pred_complete.mat'];
load(pred_file, 'valid_genes');
genes = valid_genes;
clear valid_genes
gene_strands = [genes(:).strand];

% load blocks used for LSL
block_file = ['/fml/ag-raetsch/share/projects/nGASP/LSL/data/' ...
              'blocks_gs_1.mat'];
load(block_file, 'blocks');
blocks(1).pred = struct;


% map predicted genes to blocks
for b=1:length(blocks),
  block = blocks(b);
  if (block.strand == '+'),
    b_start = block.block_start;
    b_stop  = block.block_end;
  else
    b_start = block.block_end;
    b_stop  = block.block_start;
  end

  pred = struct;
  pred.ids = [];
  pred.source = source;
  str_idx = find(gene_strands==block.strand);
  for i=1:length(str_idx),
    gene = genes(str_idx(i));
    if equal(gene.chr, block.chr) & b_start<gene.start & gene.stop<b_stop,
      pred.ids(end+1) = str_idx(i);
    end
  end
  block.pred = pred;
  blocks(b) = block;
  
  fprintf('  processed %i blocks\r',b);
end
fprintf('\n\n');


% attach gene segmentations to block.pred (local coordinates)
for b=1:length(blocks),
  block     = blocks(b);
  pred      = block.pred;
  strand    = block.strand;
  offset    = block.offset;
  block_len = abs(offset-block.block_end);
  
  gene_starts = zeros(1,length(pred.ids));
  if length(pred.ids)>0,
    for i=1:length(pred.ids)
      pred.genes{i} = gene2segments(genes(pred.ids(i)),strand,offset,model);
      gene_starts(i) = pred.genes{i}(1,1);
    end
    [gene_starts order] = sort(gene_starts);
    pred.genes = {pred.genes{order}};
    pred.ids   = pred.ids(order);
    % remove duplicate genes (YES, some identical predictions
    % appear more than once under different names...)
    rm_idx = [];
    for i=2:length(pred.genes),
      if all(all(equal(pred.genes{i},pred.genes{i-1}))),
        rm_idx = [rm_idx i];
        warning(sprintf('removed duplicate %i (%s identical to %s)', ...
                        i, genes(pred.ids(i)).name, genes(pred.ids(i-1)).name));
      end
    end
    pred.genes(rm_idx)  = [];
    pred.ids(rm_idx)    = [];
    gene_starts(rm_idx) = [];
    % test for overlaps and remove shorter isoform
    rm_idx = [];
    for i=1:length(pred.genes),
      for j=i+1:length(pred.genes),
        if pred.genes{j}(1,1)<=pred.genes{i}(end,2),
          len_i = pred.genes{i}(end,2)-pred.genes{i}(1,1);
          len_j = pred.genes{j}(end,2)-pred.genes{j}(1,1);
          if len_i < len_j,
            rm_idx = [rm_idx i];
          else
            rm_idx = [rm_idx j];
          end
          warning(sprintf('Overlapping predictions (%s and %s)! Removed shorter isoform %i', ...
                          genes(pred.ids(i)).name, genes(pred.ids(j)).name), rm_idx(end));
        end
      end
    end
    pred.genes(rm_idx)  = [];
    pred.ids(rm_idx)    = [];
    gene_starts(rm_idx) = [];
  else
    pred.genes = {};
  end   
  upstream_gene_end = 1;
  pred.segments = [];
  for i=1:length(gene_starts)
    pred.segments = [pred.segments; ...
                     [upstream_gene_end gene_starts(i) model.segments.intergenic]; ...
                     pred.genes{i}];   
    upstream_gene_end = pred.genes{i}(end,2);
  end
  pred.segments = [pred.segments; ...
                   [upstream_gene_end block_len model.segments.intergenic]];
  % assert proper segmentation
  assert(all(pred.segments(:,1) <= pred.segments(:,2)));
  assert(all(pred.segments(2:end,1) == pred.segments(1:end-1,2)));

  block.pred = pred;
  blocks(b) = block;
  
  fprintf('  processed %i blocks\r', b);
end
fprintf('\n\n');

pred_file = ['/fml/ag-raetsch/share/projects/nGASP/gene_finders/' ...
            source '_pred_blocks.mat'];
save(pred_file, 'blocks', 'genes');

