function [small_blocks new_block_id] = split_blocks(big_block, new_block_id, mask)
% splits a block into smaller ones at unmasked sites
% where there are no (predicted) genes.

BLOCK_SIZE = 10000;

fprintf('splitting block in smaller ones of size ~%i\n', BLOCK_SIZE);

switch big_block.strand,
 case '+',
  mask = mask(big_block.block_start:big_block.block_end);
 case '-',
  mask = mask(big_block.block_end:big_block.block_start);
end

b = 1;
len = abs(big_block.block_end - big_block.block_start) + 1;
while b<=len,
  e = min(len, b + BLOCK_SIZE);
  if mask(e)>0,
    ige_start = min(find(mask(e+1:end)==0));
    ige_end   = ige_start + min(find(mask(e+ige_start+1:end)>0));
    if isempty(ige_end),
      e = len;
    else
      e = e + floor((ige_start+ige_end)/2);
    end
  end
  assert(mask(e)==0);
    
  new_block.chr = big_block.chr;
  new_block.chr_num = big_block.chr_num;
  new_block.strand = big_block.strand;
  new_block.id = new_block_id;
  
  switch big_block.strand,
   case '+',
    new_block.block_start = big_block.block_start + b - 1;
    new_block.block_end   = big_block.block_start + e - 1;
    new_block.offset      = new_block.block_start - 1;
   case '-',
    new_block.block_start = big_block.block_end + e - 1;
    new_block.block_end   = big_block.block_end + b - 1;
    new_block.offset      = new_block.block_start + 1;
  end
  new_block.genestr = big_block.genestr(b:e);
  
  if isfield(big_block, 'prediction'),
    segm = big_block.prediction.segments;
    idx = find(segm(:,2) > b & segm(:,1) <  e);
    segm = segm(idx);
    segm(1,1) = b;
    segm(end,2) = e+1; % segm has half-open intervals
    new_block_segm = segm;
  end
  
  if isfield(big_block, 'details'),
    fn = fieldnames(big_block.details.pos);
    new_block.details.pos = [];
    new_block.details.score = [];
    for f=1:length(fn),
      pos = getfield(big_block.details.pos, fn{f});
      score = getfield(big_block.details.score, fn{f});
      idx = find(pos>=b & pos<=e);
      
      switch big_block.strand,
       case '+',
        pos = pos(idx) - new_block.block_start - 1;
       case '-',
        pos = pos(idx) - new_block.block_end + 1;
      end
      new_block.details.pos ...
          = setfield(new_block.details.pos, fn{f}, pos);
      
      new_block.details.score ...
          = setfield(new_block.details.score, fn{f}, score(idx));
    end
  end
  
  small_blocks(new_block_id) = new_block;
  new_block_id = new_block_id + 1;

  b = e + 1;
  fprintf('  %2.1f%%\r', 100*b/len);
end
fprintf('  %2.1f%%\n\n', 100*b/len);
