function [total,predict,correct,dist] = eval_path(pred,truth,model,start,stop)
 


true_segments = truth.segments ;
pred_segments = pred.segments ;

true_path = truth.path ;
true_pos = truth.pos ;%%%???? truth.pos
true_pos(true_path==getfield(model.state_ids,'intergenic_long'))=[];
true_path(true_path==getfield(model.state_ids,'intergenic_long'))=[];

pred_path = pred.path ;
pred_pos = pred.pos ;
pred_pos(pred_path==getfield(model.state_ids,'intergenic_long'))=[];
pred_path(pred_path==getfield(model.state_ids,'intergenic_long'))=[];


% determining start position for evaluation
if ~exist('start') | isempty(start)
  start = min(true_segments(1,1),pred_segments(1,1)) ;
end ;
if ~isempty(true_segments) & ~isempty(pred_segments)
  assert(start>=min(true_segments(1,1),pred_segments(1,1))) ;
  assert(any(start==[true_segments(1,1),pred_segments(1,1)])) ;
end ;

% determining stop position for evaluation
if ~exist('stop') | isempty(stop)
  stop = max(true_segments(end,2),pred_segments(end,2)) ;
end ;
if ~isempty(true_segments) & ~isempty(pred_segments)
  assert(stop<=max(true_segments(end,2),pred_segments(end,2))) ;
  assert(any(stop==[true_segments(end,2),pred_segments(end,2)])) ;
end ;


% transforming for efficiency (smaller arrays)
true_segments    = true_segments-start+1 ;
pred = pred_segments-start+1 ;
stop  = stop-start+1 ;
start = 1 ;

%%% evaluate by segments & nucleotides
segment_names = fieldnames(model.segments)  ;
total.num_seg = []; predict.num_seg = []; correct.num_seg = [];
total.num_nuc = []; predict.num_nuc= []; correct.num_nuc = [];
for i = 1:length(segment_names)
  map_true = zeros(stop,1) ;map_pred = zeros(stop,1) ;
  true_idx = find(true_segments(:,3)==getfield(model.segments,segment_names{i}));
  true_segs = true_segments(true_idx,:);
  total.num_seg = setfield(total.num_seg,segment_names{i}, size(true_segs,1));
  total.num_nuc = setfield(total.num_nuc,segment_names{i}, sum(true_segs(1:end,2)-true_segs(1:end,1)));
  map_true(true_segs(:,1):true_segs(:,2)) = 1 ;
  pred_idx = find(pred_segments(:,3)==getfield(model.segments,segment_names{i}));
  pred_segs = pred_segments(pred_idx,:);
  predict.num_seg = setfield(predict.num_seg,segment_names{i}, size(pred_segs,1));
  predict.num_nuc = setfield(predict.num_nuc,segment_names{i}, sum(pred_segs(1:end,2)-pred_segs(1:end,1)));
  map_pred(pred_segs(:,1):pred_segs(:,2)) = 1 ;
  [tmp,idx1,idx2] = intersect(pred_segs, true_segs,'rows');
  correct.num_seg = setfield(correct.num_seg,segment_names{i},size(tmp,1));
  correct.num_nuc = setfield(correct.num_nuc,segment_names{i},sum(map_pred&map_true));
end  

%%% evaluate by distance to true signals
signal_names = fieldnames(model.signals)  ;
dist = [];
true_states = unique(true_path) ;
for i = 1:length(signal_names)
  state_ids = getfield(model.state_ids,signal_names{i}) ;
  
  true_idx = find(ismember(true_path,state_ids));
  pred_idx = find(ismember(pred_path,state_ids));
  if ~isempty(pred_idx)&~isempty(pred_idx)
    D = [];
    for t = 1:length(true_idx)
      D(t) = min(abs(pred_pos(pred_idx) - true_pos(true_idx(t))));
    end
  else
    D=[];
  end
  dist = setfield(dist,signal_names{i},D);
end

