function [penalty_array,dict_weights,state_signals,a_trans] = gen_penalty_array(model,cont_subset_id, parameters, max_len) ;
% [penalty_array,dict_weights,state_signals,a_trans] = gen_penalty_array(model,cont_subset_id, parameters, max_len) ;
  
if nargin<3 || isempty(parameters)
  use_param = 0 ;
else
  use_param = 1 ;
end ;

content_names = fieldnames(model.contents) ;
length_names = fieldnames(model.lengths) ;
signal_names = fieldnames(model.signals) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generate penalty_array. set plif penalties to zero 

if nargin<4 || isempty(max_len)
  max_len = 0 ;
  for s = 1:model.cnt_lengths,
    len = getfield(model.lengths_range, length_names{s}) ;
    if len(2)>max_len,
      max_len = len(2) ;
    end ;
  end ;
end ;

penalty_array = {} ;
dict_weights = [] ;
num_svms = 0 ;
for s = 1:model.cnt_contents,
  pen.id        = getfield(model.contents, content_names{s}) ;
  assert(pen.id == length(penalty_array)+1) ;
  pen.name      = ['contents_' content_names{s}] ;
  pen.limits    = getfield(model.boundaries.contents, content_names{s}) ;
  pen.limits    = pen.limits(1:end-1) ;
  if ~use_param
    pen.penalties = zeros(1,length(pen.limits)) ;
  else
    pen.penalties = getfield(parameters, pen.name) ;
    pen.penalties = pen.penalties(:)' ;
  end ;
  % keyboard
  pen.transform = '' ;
  pen.min_value   = -1e6 ;
  pen.max_value   = 1e6 ;
  pen.use_cache = 0 ;
  num_svms = num_svms + 1 ;
  pen.use_svm   = num_svms ;
  dict_weights(num_svms, :) = getfield(model.dict_weights(cont_subset_id), content_names{s}) ;
  penalty_array{end+1} = pen ;
end ;

if size(dict_weights,1)<8
  % current shogun requirement
  dict_weights(8,1)=0 ;
end ;

for s = 1:model.cnt_lengths,
  pen.id        = getfield(model.lengths, length_names{s}) ;
  assert(pen.id == length(penalty_array)+1) ;
  pen.name      = ['lengths_' length_names{s}] ;
  pen.limits    = getfield(model.boundaries.lengths, length_names{s}) ;
  pen.limits    = pen.limits(1:end-1) ;
  if ~use_param
    pen.penalties = zeros(1,length(pen.limits)) ;
  else
    pen.penalties = getfield(parameters, pen.name) ;
    pen.penalties = pen.penalties(:)' ;
  end ;
  pen.transform = 'log(+1)' ;
  len=getfield(model.lengths_range, length_names{s}) ;
  pen.min_value   = len(1) ;
  pen.max_value   = min(len(2), max_len) ;
  pen.use_cache = 1 ;
  pen.use_svm   = 0 ;
  penalty_array{end+1} = pen ;
end ;

for s = 1:model.cnt_signals,
  pen.id        = getfield(model.signals, signal_names{s}) ;
  assert(pen.id == length(penalty_array)+1) ;
  pen.name      = ['signals_' signal_names{s}] ;
  pen.limits    = getfield(model.boundaries.signals, signal_names{s}) ;
  pen.limits    = pen.limits(1:end-1) ;
  pen.penalties = zeros(1,length(pen.limits)) ;
  if ~use_param
    pen.penalties = zeros(1,length(pen.limits)) ;
  else
    pen.penalties = getfield(parameters, pen.name) ;
    pen.penalties = pen.penalties(:)' ;
  end ;
  pen.transform = '' ;
  pen.min_value   = -1e6 ;
  pen.max_value   = 1e6 ;
  pen.use_cache = 0 ;
  pen.use_svm   = 0 ;
  penalty_array{end+1} = pen ;
end ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

state_signals=zeros(model.cnt_states, 1) ;
for i=1:model.cnt_states,
  if length(model.states(i).signal)==1
    state_signals(i,1)=model.states(i).signal ;
  elseif length(model.states(i).signal)==2,
    state_signals(i,2)=0 ;
    state_signals(i,1:2)=model.states(i).signal ;
  elseif length(model.states(i).signal)==3,
    state_signals(i,3)=0 ;
    state_signals(i,1:3)=model.states(i).signal ;
  end 
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


A = model.A ;
if use_param
  A(~isinf(A)) = parameters.transitions ;
  model.A = A;
end

a_trans = model.a_trans ;
k=0 ;
for i=1:size(A,1)
  idx = find(~isinf(A(i,:))) ;
  val = A(i,idx) ;
  assert(all(a_trans(1,k+1:k+length(idx))==i-1)) ;
  assert(all(a_trans(2,k+1:k+length(idx))==idx-1)) ;  
  a_trans(3,k+1:k+length(idx))=val ;    
  k=k+length(idx) ;
end ;
a_trans=a_trans' ;
[tmp,idx]=sort(a_trans(:,2)) ;
a_trans = a_trans(idx,:) ;

