function [seq,pos]=gen_seq(block, model, generate_seq)
% [seq,pos]=gen_seq(block, model)

if nargin<3,
  generate_seq = 1 ;
end ;

signal_names = fieldnames(model.signals) ;
signal_ids = zeros(1,length(signal_names)) ;
pos=[] ;
for i=1:length(signal_names) 
  new_pos = getfield(block.details.pos, signal_names{i}) ;
  pos = [pos new_pos(:)'] ;
  signal_ids(i) = getfield(model.signals, signal_names{i}) ;
end ;
pos = unique(pos) ;


thirddim=1 ;
for i=1:length(model.states)
  if length(model.states(i).signal)==2 && thirddim<2,
    thirddim=2 ;
  end ;    
  if length(model.states(i).signal)==3,
    thirddim=3 ;
  end ;    
end ;

seq=-inf(model.cnt_states, length(pos), thirddim) ;

if ~generate_seq,
  return ;
end 
% keyboard
genestr=lower(block.genestr) ;
for i=1:length(model.states) 
  %i
  if isempty(model.states(i).signal)
    continue ; 
  end ;
  
  sig_name = signal_names{find(model.states(i).signal(1)==signal_ids)} ;
  sig_pos = getfield(block.details.pos, sig_name) ;
  sig_scores = getfield(block.details.score, sig_name) ;
  [sig_pos,iidx] = unique(sig_pos) ;
  sig_scores = sig_scores(iidx) ;

  if length(model.states(i).signal)>=2,
    % handle the case with two signals. Encode the second signal's
    % score in the first one
    sig_name = signal_names{find(model.states(i).signal(2)==signal_ids)} ;
    sig_pos2 = getfield(block.details.pos, sig_name) ;
    sig_scores2 = getfield(block.details.score, sig_name) ;
    [sig_pos2,iidx] = unique(sig_pos2) ;
    sig_scores2 = sig_scores2(iidx) ;

    [sig_pos, idx1, idx2]=intersect(sig_pos, sig_pos2) ;
    sig_scores = sig_scores(idx1) ;
    sig_scores2 = sig_scores2(idx2) ;
  end ;

  if length(model.states(i).signal)>=3,
    % handle the case with two signals. Encode the second signal's
    % score in the first one
    sig_name = signal_names{find(model.states(i).signal(3)==signal_ids)} ;
    sig_pos3 = getfield(block.details.pos, sig_name) ;
    sig_scores3 = getfield(block.details.score, sig_name) ;
    [sig_pos3,iidx] = unique(sig_pos3) ;
    sig_scores3 = sig_scores3(iidx) ;

    [sig_pos, idx1, idx2]=intersect(sig_pos, sig_pos3) ;
    sig_scores = sig_scores(idx1) ;
    sig_scores2 = sig_scores2(idx1) ;
    sig_scores3 = sig_scores3(idx2) ;
  end ;
  % keyboard
  if ~isempty(model.states(i).consensus) 
    cons_found = zeros(1,length(sig_pos)) ;
    for j=1:length(sig_pos),
      for k=1:length(model.states(i).consensus)
        offset = model.states(i).consensus_pos(k) ;
        motif = lower(model.states(i).consensus{k}) ;
        if isequal(genestr(sig_pos(j)-offset:sig_pos(j)-offset+ length(motif)-1), motif),
          cons_found(j) = 1 ;
        end ;
      end ;
    end ;
    %assert(all(cons_found)) ;
    sig_pos(~cons_found)=[] ;
    sig_scores(~cons_found)=[] ;
    if length(model.states(i).signal)>=2
      sig_scores2(~cons_found)=[] ;
    end ;
    if length(model.states(i).signal)>=3
      sig_scores3(~cons_found)=[] ;
    end ;
  end ;
  
  if ~isempty(model.states(i).non_consensus) 
    ncons_found = zeros(1,length(sig_pos)) ;
    for j=1:length(sig_pos),
      for k=1:length(model.states(i).non_consensus)
        offset = model.states(i).non_consensus_pos(k) ;
        motif = lower(model.states(i).non_consensus{k}) ;
        if isequal(genestr(sig_pos(j)-offset:sig_pos(j)-offset+ length(motif)-1), motif),
          ncons_found(j) = 1 ;
        end ;
      end ;
    end ;
    sig_pos(ncons_found~=0)=[] ;
    sig_scores(ncons_found~=0)=[] ;
    if length(model.states(i).signal)>=2
      sig_scores2(ncons_found~=0)=[] ;
    end ;
    if length(model.states(i).signal)>=3
      sig_scores3(ncons_found~=0)=[] ;
    end ;
  end ;
  
  [tmp,idx1,idx2]=intersect(pos,sig_pos) ;
  assert(length(idx2)==length(unique(sig_pos))) ;
  seq(i, idx1, 1) = sig_scores(idx2)  ;
  
  if length(model.states(i).signal)>=2
    assert(length(sig_scores)==length(sig_scores2)) ;
    seq(i, idx1, 2) = sig_scores2(idx2) ;
  end ;

  if length(model.states(i).signal)>=3
    assert(length(sig_scores)==length(sig_scores3)) ;
    seq(i, idx1, 3) = sig_scores3(idx2) ;
  end ;
  % keyboard
end ;

seq(model.state_ids.seq_start, 1, 1) = 0 ;
seq(model.state_ids.seq_end, end, 1) = 0 ;
seq(model.state_ids.intergenic_long, :, 1) = 0 ;

return ;

intersect(find(~isinf(seq(model.state_ids.don(3),:))),find(~isinf(seq(model.state_ids.don(4),:))))

intersect(find(~isinf(seq(model.state_ids.don(5),:))),find(~isinf(seq(model.state_ids.don(6),:))))
intersect(find(~isinf(seq(model.state_ids.don(5),:))),find(~isinf(seq(model.state_ids.don(7),:))))
intersect(find(~isinf(seq(model.state_ids.don(6),:))),find(~isinf(seq(model.state_ids.don(7),:))))

isequal(union(find(~isinf(seq(model.state_ids.acc(3),:))),find(~isinf(seq(model.state_ids.acc(4),:)))),find(~isinf(seq(model.state_ids.acc(2),:))))

isequal(union(find(~isinf(seq(model.state_ids.acc(5),:))),union(find(~isinf(seq(model.state_ids.acc(6),:))),find(~isinf(seq(model.state_ids.acc(7),:))))),find(~isinf(seq(model.state_ids.acc(2),:))))


seq(seq>20)=-1 ;
imagesc(seq) ;

keyboard
