function blocks = gen_truth(blocks, model)
% blocks = gen_truth(blocks, model)

% length threshold for distance between genes, below:
% intercistronic, otherwise intergenic 
max_intercistronic_length = 5000;
assert(max_intercistronic_length<=model.lengths_range.intercistronic(2));

% option to prefer the longest valid isoform
use_longest = 1;

blocks(1).truth = struct;
% blocks(1).seq = []; 
% blocks(1).pos = [];
blocks(1).has_valid_transcript = [];
blocks(1).overlaps_with_prev = [];

for i=1:length(blocks)
  block = blocks(i);

  isoforms = {};
  upstream_gene_end = 1;
  for g=1:length(block.gene_ids),
    exons  = {};
    tis    = {};
    stop   = {};
    tss    = {};
    trans  = {};
    polya  = {};
    cleave = {};

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % determine valid transcripts %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if ~block.gene_complete(g),
      error(sprintf('incomplete gene (block %i, gene %i)',i,g));
    end
    if block.is_alt(g),
      [transcripts,all_tis,all_stop] = get_isoforms(block,g,model);
      if isempty(transcripts),
        error(sprintf('no isoform with ORF found (block %i, gene %i)',i,g));
      end
      % if splice donors or acceptors are invalid take next isoform
      iso_idx = 1:length(transcripts);
      if ~use_longest,
        iso_idx = randperm(length(transcripts));
      end
      
      invalid_sites = [block.has_invalid_don{g}; ...
                       block.has_invalid_acc{g}];
      valid_idx = ones(size(iso_idx));
      if ~isempty(invalid_sites),
        for j=1:length(transcripts)
          % if any donor or acceptor is invalid take next isoform instead        
          for k=1:length(invalid_sites)
            if any(any(transcripts{iso_idx(j)}(2:end,1)==invalid_sites(k))) ...
                  || any(any(transcripts{iso_idx(j)}(1:end-1,2)==invalid_sites(k))),
              valid_idx(j) = 0;
              break
            end
          end
        end
      end
      if all(valid_idx==0), 
        warning(sprintf('no isoform without invalid don or acc found (block %i, gene %i)',i,g));
        %keyboard
        block.has_valid_transcript(g) = 0;
        exons{1} = transcripts{1};
      else
        block.has_valid_transcript(g) = 1;      
        iso_idx = iso_idx(find(valid_idx));      
        for j=1:length(iso_idx),
          exons{j}  = transcripts{iso_idx(j)};
          tss{j}    = exons{j}(1,1) ;
          tis{j}    = all_tis(iso_idx(j));
          stop{j}   = all_stop(iso_idx(j));
          cleave{j} = exons{j}(end,2) ;
          
          idx = find(block.cleave{g}==cleave{j});
          polya_u = unique(block.polya{g}(idx));
          polya_u(isnan(polya_u)) = [];
          polya{j}  = [];
          if length(polya_u)>1,
            keyboard
            for k=1:length(polya_u),
              if isnan(polya_u(k)), 
                continue
              end
              labels(k) = block.details.label.polya(find(block.details.pos.polya==polya_u(k)));
            end
            [tmp,mx_idx] = max(labels);
            polya{j}  = polya_u(mx_idx);
          elseif isnan(polya_u), 
            polya{j}= polya_u
          end
          % whenever a trans splice site has been found, the first exon
          % start has identical coordinates
          idx = find(block.trans{g}==exons{j}(1,1));
          trans{j}    = unique(block.trans{g}(idx));     
        end
      end
    else %%% gene is not alt
      exons{1}  = block.splicegraph{g}{1}';
      tss{1}    = block.tss{g};
      trans{1}  = block.trans{g};
      if isnan(trans{1}),
        trans{1} = [];
      end
      tis{1}    = block.tis{g};
      stop{1}   = block.stop{g};
      cleave{1} = block.cleave{g};
      polya{1}  = block.polya{g} ;
      if isnan(polya{1}), 
        polya{1} = []; 
      end
      assert(length(polya{1})<=1);
    
      block.has_valid_transcript(g) = 1;
      invalid_sites = [block.has_invalid_don{g}; ...
                       block.has_invalid_acc{g}];
      for k=1:length(invalid_sites),
        if any(any(exons{1}==invalid_sites(k))),
          %keyboard
          block.has_valid_transcript(g) = 0;
          break
        end
      end
    end

    if ~block.has_valid_transcript(g),
      idx1= min(find(block.details.pos.tss>=exons{1}(end,1)));
      idx2= min(find(block.details.pos.cleave>=exons{1}(end,2)));
      isoforms{g}{1} = [block.details.pos.tss(idx1) block.details.pos.cleave(idx2)  nan];
      upstream_gene_end = block.details.pos.cleave(idx2);
      gene_start = block.details.pos.tss(idx1);
      overlaps(j) = upstream_gene_end >= gene_start;
      block.overlaps_with_prev(g) = any(overlaps);
      continue
    end
    
    assert(length(exons)==length(tss));
    assert(length(exons)==length(trans));
    assert(length(exons)==length(tis));
    assert(length(exons)==length(stop));
    assert(length(exons)==length(polya));
    assert(length(exons)==length(cleave));
    
    if 0% ~isempty(setdiff([tss{:}], [block.details.pos.tss; block.details.pos.trans]));
      det=[block.details.pos.tss; block.details.pos.trans];
      for ii=1:length(tss)
        idx = find(~ismember(tss{ii},det));
        for jj=idx
          [tmp,ee]=min(abs(det-tss{ii}));
          tss{ii}(jj) =det(ee);
        end
      end
    end
    % assert(isempty(setdiff([tss{:}], [block.details.pos.tss; block.details.pos.trans])));
    % assert(isempty(setdiff([cleave{:}], block.details.pos.cleave)));
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % generate segmentation for all valid transcripts of a gene %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    overlaps = zeros(1,length(exons));
    gene_end = 1;
    for j=1:length(exons),
      isoform = [];
      [isoform, tss{j}, trans{j},incomplete] = append_utr5p_segment(isoform, exons{j}, tis{j}, ...
                                                        tss{j}, trans{j}, model, block);
      if incomplete
        block.gene_complete(g) =0;
      end
      isoform = append_cds_exon_segment(isoform, exons{j}, tis{j}, stop{j}, model);    
      isoform = append_utr3p_segment(isoform, exons{j}, stop{j}, polya{j}, cleave{j}, model,block);

      % assert a tight segmentation
      assert(all(isoform(2:end, 1) == isoform(1:end-1, 2)));

      % assert that there is an ORF
      assert(sum(isoform(:,3)==model.segments.cds_exon ...
                 | isoform(:,3)==model.segments.utr5exon ...
                 | isoform(:,3)==model.segments.utr3exon)<=size(exons{j},1)+2);
      cds_idx = find(isoform(:,3)==model.segments.cds_exon);
      assert(mod(sum(isoform(cds_idx,2)-isoform(cds_idx)),3)==0)
      str = [];
      for k=1:length(cds_idx),
        str = [str block.genestr(isoform(cds_idx(k),1):isoform(cds_idx(k),2)-1)];
      end
      f = find_open_frames(lower(str),1); 
      assert(f(1)~=0);
      
      gene_start = isoform(1,1);
      gene_end = max(gene_end,isoform(end,2));
      if upstream_gene_end >= gene_start,
        overlaps(j) = 1;      
        warning(sprintf('overlapping genes (block %i, gene %i)',i,g));
      end
      
      has_trans(g,j) = ~isempty(trans{j});
      has_tss(g,j) = ~isempty(tss{j});
      isoforms{g}{j} = isoform;
    end %loop over transcripts (isoforms)
    
    upstream_gene_end = gene_end;
    block.overlaps_with_prev(g) = any(overlaps);
    %keyboard
  end %%% loop over genes

  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % generate segmentation for all valid transcripts of a block %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  
  % %%% get all candidate positions, build seq
  % %%% already done in generate_blocks
  % if ~model.use_gc, 
  %   idx = find(lower(block.genestr(block.details.dpos+1))=='t');
  %   block.details.pos.don = block.details.pos.don(idx);
  %    block.details.score.don = block.details.score.don(idx);
  % end
  % [block.seq, block.pos] = gen_seq(block, model);


  %%% determine all combinations of valid isoforms in current block
  iso_idx = [1:length(isoforms{1})]';
  num_segmentations = length(isoforms{1});
  
  for g=2:length(isoforms),
    tmp = repmat(iso_idx,length(isoforms{g}),1);
    last_col = [];
    for j=1:length(isoforms{g}),
      last_col = [last_col; j*ones(size(iso_idx,1),1)];
    end
    iso_idx = [tmp last_col];
    num_segmentations = num_segmentations * length(isoforms{g});
  end
  assert(size(iso_idx,1)==num_segmentations);
  assert(size(iso_idx,2)==length(isoforms));


  %%% build truth, one entry per valid block segmentation 
  truth = struct;
  for j=1:num_segmentations, 
    upstream_gene_end = 1;
  
    truth(j).segments = [];
    truth(j).genes = {};
    for g=1:length(isoforms),
      truth(j).genes{g} = isoforms{g}{iso_idx(j,g)};
      % has_trans = truth(j).genes{g}(1,3) == model.segments.transexon;
      %if has_trans, keyboard ; end ;

      % decide whether to annotate spacer between genes as intergenic or intercistronic
      gene_start = truth(j).genes{g}(1,1);
      spc_dist = gene_start - upstream_gene_end;
      if ~has_trans(g,iso_idx(j,g))|...
            (has_trans(g,iso_idx(j,g))&has_tss(g,iso_idx(j,g)));
        % if ~has_trans | upstream_gene_end==1,
        truth(j).segments = [truth(j).segments; ...
                            [upstream_gene_end gene_start model.segments.intergenic]; ...
                            truth(j).genes{g}];
      elseif spc_dist<=max_intercistronic_length ...
            & has_trans(g,iso_idx(j,g)) & ~has_tss(g,iso_idx(j,g)) &upstream_gene_end~=1,
        % if ~has_trans | upstream_gene_end==1,
        truth(j).segments = [truth(j).segments; ...
                            [upstream_gene_end gene_start model.segments.intercistronic]; ...
                            truth(j).genes{g}];
      elseif has_trans(g,iso_idx(j,g))&...
            (spc_dist>max_intercistronic_length |has_tss(g,iso_idx(j,g))|upstream_gene_end==1),
        % if ~has_trans | upstream_gene_end==1,
        truth(j).segments = [truth(j).segments; ...
                            [upstream_gene_end gene_start model.segments.intergenictrans]; ...
                            truth(j).genes{g}];
      else
        
        keyboard
        
      end
      upstream_gene_end = truth(j).genes{g}(end,2);
    end
    % append last intergenic segment
    block_len = abs(block.block_end-block.offset);
    truth(j).segments = [truth(j).segments; [upstream_gene_end block_len model.segments.intergenic]];
    % assert tight segmentation
    assert(all(truth(j).segments(2:end,1) == truth(j).segments(1:end-1,2)));
    if ~any(block.overlaps_with_prev) & all(block.gene_complete)&all(block.has_valid_transcript)
      if ~all(truth(j).segments(:,1)<=truth(j).segments(:,2))
        warning('this was an assert ... which did not always hold')
        keyboard
      end ;
    end
    cleave = [];
    polya = [];
    tss = [];
    for k=1:length(truth(j).genes),
      if ~isempty(truth(j).genes{k}),
        cleave = [cleave truth(j).genes{k}(end,2)];
        tss = [tss truth(j).genes{k}(1,1)];
      end
    end
    if all(block.has_valid_transcript)&~any(block.overlaps_with_prev)
      assert(isempty(setdiff(tss, [block.details.pos.tss; block.details.pos.trans])));
      assert(isempty(setdiff(cleave, block.details.pos.cleave)));
    end
    % [block.seq, block.pos] = gen_seq(block, model);
    
    %which segmentation2path
    %keyboard

    [truth(j).path, truth(j).pos_idx, truth(j).pos, ok] = segmentation2path(truth(j), block, model);
  end %%% loop over segmentations

  block.truth = truth;
  blocks(i) = block;
  
%%%%% simple printing
  if 0,
    num_segments = size(block.truth(j).segments,1);
    patch([block.truth(j).segments(:,1)';block.truth(j).segments(:,1)'; ...
           block.truth(j).segments(:,2)';block.truth(j).segments(:,2)'], ...
          [ones(1,num_segments);zeros(1,num_segments); ...
           zeros(1,num_segments);ones(1,num_segments)],...
          block.truth(j).segments(:,3)');
    axis([1 block_len, -0.2, 1.2]);
    keyboard
  end
%%%%%
  

  fprintf('Processed block %i\r', i);
end %%% loop over blocks
fprintf('\n\n');
