function DNA_blocks = generate_blocks(PAR)

% "blocks" is a structure similar to previously used "genes". it
% includes longer stretches with at normally two genes.

% cont_names =  PAR.genome_info.contig_names;

  
% fid = fopen([PAR.fn_log 'ms_gfwd_starter.txt'],'a+');
% fprintf(fid,'\n\n starting ms_gfwd_starter \n');
% fprintf(fid,'date: %s\n',date);

if ~isfield(PAR,'min_dist')
  PAR.min_dist = 1000;
end

if PAR.use_annotation,
  load(PAR.fn_gene_names, 'confirmed_genes_names', 'unconfirmed_genes_names') ;
  load(PAR.fn_anno,'all_genes');
  load(PAR.fn_region ,'training_regions');
  load(PAR.fn_signal ,'training_signals');
  load(PAR.fn_polya_scores ,'predicted_region_signals');
  load(PAR.fn_tss_scores,'preds');
  load(PAR.fn_content,'content_blocks');
  regions = training_regions; clear training_regions ;
  signals = training_signals; clear training_signals ;
else
  load(PAR.fn_region ,'test_regions');
  load(PAR.fn_polya_scores ,'predicted_test_blocks');
  predicted_region_signals = predicted_test_blocks;
  load(PAR.fn_tss_scores,'preds');
  regions = test_regions; clear testing_regions ;
end

nofRegions = length(regions);
blocks = struct;

N_genes_per_block = [];
N_blocks = 1;
collect_gene_ids = [];
strand = {'+','-'} ;

for r = 1:nofRegions
  sprintf('generating blocks on region %i\n',r)
  load(sprintf('%s%i.mat',PAR.fn_ss_scores,r),'splice_cand');
  T = load(sprintf('%s%i.mat',PAR.fn_trans_scores,r),'splice_cand');
  seq = upper(regions(r).seq);
  chr = regions(r).chr;
    
  for str = 1:length(strand)
    
    %%%PREPARE DETECTOR OUTPUTS
    POS.acc = splice_cand.acc_pos{str}' ;
    SCORE.acc = splice_cand.acc_confcum{str}';
    if isfield(splice_cand, 'acc_out')
      OUTPUT.acc = splice_cand.acc_out{str}';
    else
      OUTPUT.acc = nan*ones(size(splice_cand.acc_confcum{str}'));
    end ;
    POS.trans = T.splice_cand.acc_pos{str}' ;
    SCORE.trans = T.splice_cand.acc_confcum{str}';
    POS.don = splice_cand.don_pos{str}' ;
    SCORE.don = splice_cand.don_confcum{str}' ;
    if isfield(splice_cand, 'don_out')
      OUTPUT.don = splice_cand.don_out{str}';
    else
      OUTPUT.don = nan*ones(size(splice_cand.don_confcum{str}'));
    end ;
    POS.tss = preds(r,str).predTss(:,1);
    SCORE.tss = preds(r,str).predTss(:,2);
    SCORE.tis = preds(r,str).predSta(:,2);
    SCORE.stop = preds(r,str).predSto(:,2);
    POS.cleave = preds(r,str).predEnd(:,1) ;
    SCORE.cleave = preds(r,str).predEnd(:,2);
    POS.tis = preds(r,str).predSta(:,1) ;
    POS.stop = preds(r,str).predSto(:,1) ;
    POS.polya = predicted_region_signals(r).svm_positions{str}(:,1) ;
    SCORE.polya = predicted_region_signals(r).svm_conf_cum{str} ;
    SIXMER.polya = predicted_region_signals(r).svm_positions{str}(:,2) ;
    if PAR.use_annotation,
      SCORE.polya = predicted_region_signals(r).svm_conf_cum{str} ;
      LABEL.polya = predicted_region_signals(r).svm_labels{str} ;
    else
      tmp = [];
      for i=1:size(predicted_region_signals(r).svm_conf_cum,1)
        tmp = [tmp predicted_region_signals(r).svm_conf_cum{i,str}];
      end
      SCORE.polya = mean(tmp,2) ;
      assert(all(size(SCORE.polya)==size(POS.polya)))
    end
    %%%% check all POS are in genome coordinates;
    fieldn = fieldnames(POS);
    for f = 1:length(fieldn)
      pp = getfield(POS,fieldn{f}); 
      if min(pp)<regions(r).start | max(pp)>regions(r).stop
        POS = setfield(POS,fieldn{f},pp+regions(r).start-1);
        pp = getfield(POS,fieldn{f}); 
      end
      assert(min(pp)>=regions(r).start | max(pp)<=regions(r).stop)
    end
      
    if strand{str}=='+'
      POS.tis = POS.tis - 1 ;
      POS.stop = POS.stop - 1 ;
    else
      POS.tis = POS.tis + 1 ;
      POS.stop = POS.stop  + 1 ;
    end
    %%%% 
    %%%%%%%%%%%%%%%%%%
    if strand{str}=='+'
      block_start = regions(r).start ;
    else %%% strand{str}=='-'
      block_start = regions(r).stop ;
    end
    if PAR.use_annotation,
      sig = signals(r);
      fprintf('number of genes in region %i on strand %s : %i\n', r, strand{str},sig.nofGenes(str))
      gene_ids = unique(sig.allTss{str}(:,2));
      assert(sig.nofGenes(str)==length(gene_ids))
      starts = []; stops =[];
      if strand{str}=='+'
        for g = 1:sig.nofGenes(str)
          starts = [starts min(sig.allTss{str}(sig.allTss{str}(:,2) == gene_ids(g),1))];
          stops = [stops max(sig.allEnd{str}(sig.allTss{str}(:,2) == gene_ids(g),1))];
        end
        [starts,idx] = sort(starts) ; 
        stops = stops(idx) ;
        gene_ids =  gene_ids(idx) ;
        dist = [starts(2:end) - stops(1:end-1) 500000 500000];
        % keyboard
      else %%% strand{str}=='-'
        for g = 1:sig.nofGenes(str)
          starts = [starts max(sig.allTss{str}(sig.allTss{str}(:,2) == gene_ids(g),1))];
          stops = [stops min(sig.allEnd{str}(sig.allTss{str}(:,2) == gene_ids(g),1))];
        end
        [starts,idx] = sort(starts) ; 
        starts = starts(end:-1:1);
        stops = stops(idx) ;
        stops = stops(end:-1:1);
        gene_ids =  gene_ids(idx) ;
        gene_ids =  gene_ids(end:-1:1) ;
        dist = [-starts(2:end) + stops(1:end-1) 500000 500000];
        % keyboard
      end 
      gene_names = {all_genes(gene_ids).name} ;
      gene_confirmed = ismember(gene_names, confirmed_genes_names) ;
      gene_unconfirmed = ismember(gene_names, unconfirmed_genes_names) ;
      assert(all(gene_confirmed+gene_unconfirmed)) ;
    else %%%  if ~PAR.use_annotation
      gene_ids = [1;2];
      if strand{str}=='+'
        stops = [block_start+100 regions(r).stop] ;
      else
        stops = [block_start-100 regions(r).start] ;
      end
      dist = [100 500000 500000];
    end %%%  if PAR.use_annotation
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%% build DNA_block
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    id = 1;
    while id<=length(gene_ids) 
      N_genes = 2;
      cont = 1;
      while cont
        if dist(id+N_genes-1)>PAR.min_dist & all(dist(id:id+N_genes-2)<500000)
          DNA_block = struct;
          DNA_block.id = N_blocks;
          DNA_block.reg = r ;
          DNA_block.chr = chr ;
          DNA_block.strand = strand{str};
          if PAR.use_annotation,
            DNA_block.gene_ids = gene_ids(id:id+N_genes-1);
            DNA_block.gene_names = gene_names(id:id+N_genes-1);
            DNA_block.gene_confirmed = gene_confirmed(id:id+N_genes-1);
          end
          if strand{str}=='+'
            block_end = stops(id+N_genes-1)+floor(dist(id+N_genes-1)/2);
            if block_end < max(stops(id:id+N_genes-1)),
              block_end = max(stops(id:id+N_genes-1))+500;
            end
            if block_end > regions(r).stop 
              block_end = regions(r).stop ;
            end
            assert(block_end>block_start)
            DNA_block.block_end = block_end;
            DNA_block.offset = block_start - 1;
            DNA_block.block_start = block_start;
            DNA_block.genestr = seq([block_start:block_end]-regions(r).start+1);
            if PAR.use_annotation,
              DNA_block.gene_pos(:,1) = starts(id:id+N_genes-1)-DNA_block.offset;
              DNA_block.gene_pos(:,2) = stops(id:id+N_genes-1)-DNA_block.offset;
            end
            if isfield(PAR, 'use_conservation') & PAR.use_conservation,
              DNA_block.conservation = regions(r).conservationScores(...
                  [block_start:block_end]-regions(r).start+1);
            end
            block_start = block_end + 1;
          else %%% strand{str}=='-'
            block_end = stops(id+N_genes-1) - ceil(dist(id+N_genes-1)/2) +1;
            if block_end > min(stops(id:id+N_genes-1)),
              block_end = min(stops(id:id+N_genes-1)) - 499;
            end
            if block_end < regions(r).start 
              block_end = regions(r).start ; 
            end
            
            DNA_block.block_end = block_end;
            DNA_block.offset = block_start + 1 ;
            DNA_block.block_start = block_start ;
            DNA_block.genestr = seq([block_end:block_start]-regions(r).start+1) ;
            DNA_block.genestr = reverse_complement(DNA_block.genestr) ;
            if PAR.use_annotation,
              DNA_block.gene_pos(:,1) = -(starts(id:id+N_genes-1)-DNA_block.offset);
              DNA_block.gene_pos(:,2) = -(stops(id:id+N_genes-1)-DNA_block.offset);
            end
            if isfield(PAR, 'use_conservation') & PAR.use_conservation,
              DNA_block.conservation = regions(r).conservationScores(...
                  [block_end:block_start]-regions(r).start+1);
              DNA_block.conservation = DNA_block.conservation(end:-1:1);
            end
            block_start = block_end - 1; 
          end %%% strand{str}=='-'
          if PAR.use_annotation,
            content_block = content_blocks(DNA_block.id);
            DNA_block = fill_block_2(DNA_block,POS,SCORE,SIXMER,OUTPUT,PAR.model,content_block,LABEL,all_genes) ;
          else
            DNA_block = fill_block_2(DNA_block,POS,SCORE,SIXMER,OUTPUT,PAR.model) ;
          end

          % DNA_block = orderfields(DNA_block) ;
          %%% get all candidate positions, build seq and pos
          if ~PAR.model.use_gc, 
            idx = find(lower(DNA_block.genestr(DNA_block.details.pos.don+1))=='t');
            DNA_block.details.pos.don = DNA_block.details.pos.don(idx);
            DNA_block.details.score.don = DNA_block.details.score.don(idx);
            DNA_block.details.output.don = DNA_block.details.output.don(idx);
          end
          if ~PAR.use_annotation,
            [DNA_block.seq, DNA_block.pos] = gen_seq(DNA_block, PAR.model);
          end
          %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         
          if ~exist('DNA_blocks')
            DNA_blocks(1) = DNA_block;
          else
            DNA_blocks(end+1) = DNA_block;
          end
          
          collect_gene_ids = [collect_gene_ids;gene_ids(id:id+N_genes-1)];
          
          cont = 0 ;
          N_genes_per_block = [N_genes_per_block N_genes];
        elseif any(dist(id:id+N_genes-1) >100000)==1
          fprintf('end of fragment\n');
          % keyboard
          if N_genes>2
            error('some error')
          end
          N_genes = N_genes-1;
        else
          N_genes = N_genes+1;
        end
      end %% while cont
      N_blocks = N_blocks+1;
      id = id + N_genes;
    end %% while id<=length(gene_ids) 
    % keyboard
    assert(length(intersect(collect_gene_ids,gene_ids))==length(gene_ids))
    % fprintf(['chromosome %i strand %s: \t %i genes in %i blocks\n ' ...
    %          '%i\n'],chr,str{1},length(orig_id),N_blocks);
  end  %%% loop over strands
end ; %%% loop over regions
      % assert(isempty(setdiff([1:length(PAR.genes)],collect_gene_ids)))


%save(PAR.fn_blocks,'DNA_blocks')  
