function [good_isoforms_,tis,stop] = get_isoforms(block,gene_id,model) ;
% isoforms = get_isoforms(block,gene_id,model) ;
%
% Generates all possible isoforms from the splicegraph,
% and locate all the start and stop codons.
% The reason for genesfilename is so that we can choose between the
% EST confirmed genes and the predicted genes.
%
% Written by: Cheng Soon Ong, 4 October 2006
% Modified on: 12 November 2006

splicegraph = block.splicegraph{gene_id} ;

% find all the leftmost and rightmost exons (transcription starts and stops)
tstart = [];
vertices = splicegraph{1};
edges = splicegraph{2};

%if block.strand=='-'
%  edges= edges(end:-1:1,end:-1:1) ; % hack
%end ;

for ixv = 1:size(vertices,2)
  if sum(edges(ixv,1:ixv-1)) == 0
    tstart(end+1) = ixv;
  end
end

% find all transcription ends
tstop = [];
for ixv = 1:size(vertices,2)
  if sum(edges(ixv+1:end,ixv)) == 0
    tstop(end+1) = ixv;
  end
end

% do a depth first search through the graph, generating a list of isoforms
% - for each transcription start, do depth first search.
global isoforms
isoforms = {};

for ixt = tstart
  dfs_visit(triu(edges),ixt,[]);
end

% for each path, find all the start and stop codons.
% and get the longest in frame coding sequence

tis=[]; stop=[]; len=[] ; utr_len=[] ; good_isoforms=[] ;
for ixf = 1:length(isoforms)
  for i = 1:length(block.tis{gene_id})
    for j = 1:length(block.stop{gene_id})
      exons = vertices(:,isoforms{ixf})';
      idx = find(exons(:,1)<=block.tis{gene_id}(i)&exons(:,2)>= ...
                 block.tis{gene_id}(i));
      if isempty(idx), continue ; end ;
      utr_len_ = sum(exons(1:idx-1,2)-exons(1:idx-1,1)) + block.tis{gene_id}(i) - exons(idx,1)  ;
      exons(1:idx-1,:) = [];
      exons(1,1) = block.tis{gene_id}(i);
      idx = find(exons(:,1)<=block.stop{gene_id}(j) & exons(:,2)>=block.stop{gene_id}(j));
      if isempty(idx), continue ; end ;
      utr_len_ = utr_len_ + sum(exons(idx+1:end,2)-exons(idx+1:end,1)) + exons(idx,2) - block.stop{gene_id}(j)   ;
      exons(idx+1:end,:) = [];
      exons(end,2) = block.stop{gene_id}(j)+3;

      if ~mod(sum(exons(:,2)-exons(:,1)),3)
        str = [];
        for k=1:size(exons,1)
          str = [str block.genestr(exons(k,1):exons(k,2)-1)];
        end
        f=find_open_frames(lower(str),1) ;
        if f(1)==1,
          tis(end+1) = block.tis{gene_id}(i) ;
          stop(end+1) = block.stop{gene_id}(j) ;
          len(end+1) = length(str) ;
          utr_len(end+1) = utr_len_ ;
          good_isoforms{end+1}=isoforms{ixf} ;
        end ;
      end
    end
  end
end 

[tmp idx] = sort(len+1e-5*utr_len,'descend');
good_isoforms_={} ;
for i=1:length(idx)
  good_isoforms_{i} = vertices(:,good_isoforms{idx(i)})';
end ;
tis=tis(idx);
stop =stop(idx);

