function [blocks,model,QP,parameters]= init_path(PAR)
% RESULT = init_path(PAR)
%

max_neg  = PAR.max_neg ;

% request a cplex license
lpenv = cplex_license(1, 2) ;

if lpenv == 0,
  %pause(100) ;
  rproc_rerun('could not obtain cplex license') ; 
end ;

if (fexist(PAR.fn_init) || fexist([PAR.fn_init '.mat'])) && ~PAR.ignore_init_file
  disp('loading initialization data') ;
  load(PAR.fn_init, 'model', 'QP') ;
  blocks=load_struct(PAR.fn_init, 'blocks') ;
else
  disp('initializing blocks') ;
  
  %%% load data
  %global XX_blocks
  %if ~isempty(XX_blocks),
  %  blocks=XX_blocks ;
  % warning('using blocks from global variable');
  %else
  blocks=load_struct(PAR.fn_blocks, 'blocks') ;
  %end ;
  %XX_blocks = blocks ;
  load(PAR.fn_splits, 'blocks_split') ; 
  
  %%% create training set
  block_idx = [blocks_split{PAR.train_split_idx}] ;
  if isfield(PAR, 'num_train'),
    %keyboard
    block_idx = block_idx(1:PAR.num_train);
  end 
  blocks = blocks(block_idx) ;
  
  fprintf('training with %i blocks\n', length(block_idx)) ;
  
  if isfield(PAR, 'fn_content_conserv')
    model = prepare(blocks, PAR.model, PAR.fn_content, PAR.fn_content_conserv) ; 
  else
    model = prepare(blocks, PAR.model, PAR.fn_content) ;
  end ;

  if isfield(PAR, 'fn_boundary_model'),
    fprintf('using boundary model in %s\n', PAR.fn_boundary_model) ;
    L=load(PAR.fn_boundary_model) ;
    model.boundaries = L.boundaries ;
  end ;

  % initialize matrices for quadratic program
  [QP,blocks] = initialize(blocks, model, PAR) ;

  save_struct(PAR.fn_init, blocks, 'blocks') ;
  save(PAR.fn_init, '-append', 'model', 'QP') ;
end ;

tic;[res,lambda,how] = qp_solve(lpenv, QP.Q, QP.f, QP.A, QP.b, QP.lb, QP.ub, 0, 1, 'bar');toc;

OBJ = sum(res.*QP.f)+0.5*res'*QP.Q*res ;
parameters = init_weights(model) ;
parameters = vector2weights(res(1:model.cnt_parameters)', parameters) ;

how
QP.res = res;

% return license
cplex_close(lpenv) ;
