function [QP, blocks] = initialize(blocks, model, PAR, generate_neg_constraints)
% [ Q, f, A, b, lb, ub, blocks] = initialize(blocks, model, PAR, parameters, , generate_neg_constraints=1)
  
if nargin<4,
  generate_neg_constraints=1 ;
end ;

INF = PAR.INF;
C_regul = PAR.C_regul; 
bins = PAR.bins;
cnt_plif_detector = model.cnt_signals + model.cnt_contents  ;
cnt_plif_lengths = model.cnt_lengths  ;
cnt_transitions = model.cnt_transitions ;
max_neg = PAR.max_neg ;
margin = PAR.margin;

%%%%%%%

if isfield(model, 'use_conservation') && model.use_conservation 
  disp('Converting conservation levels') ;
  for id = 1:length(blocks),
    blocks(id).genestr_conserv = conservation2DNA_alphabet(blocks(id).conservation) ;
  end ;  
end ;

%%%%%%
content_names = fieldnames(model.contents) ;
id_contents = [] ;

for i=1:length(content_names),
  m = getfield(model.contents, content_names{i}) ;
  id_contents(end+1) = m;
  C_regul.id2C(m) = C_regul.contents;
end ;
signal_names = fieldnames(model.signals) ;
id_signals = [] ;
for i=1:length(signal_names),
  m = getfield(model.signals, signal_names{i}) ;
  id_signals(end+1) = m ;
  C_regul.id2C(m) = C_regul.signals;
end ;
length_names = fieldnames(model.lengths) ;
id_length = [] ;
for i=1:length(length_names),
  m = getfield(model.lengths, length_names{i}) ;
  id_length(end+1) = m ;
  C_regul.id2C(m) = C_regul.lengths;
end ;

%%%%%%
monoton_id = [id_contents id_signals] ;
nof_monoton = 0;
for m = monoton_id,
  param_ids = model.plif2param{m};
  nof_monoton = nof_monoton + length(param_ids) - 1;
end ;

smooth_id = [monoton_id id_length] ;
nof_smooth = 0;
for m = smooth_id,
  param_ids = model.plif2param{m};
  nof_smooth = nof_smooth + length(param_ids) - 1;
end ;

nof_trans_smooth = model.cnt_transitions ;
nof_smooth = nof_smooth + nof_trans_smooth ;

% keyboard
%%%%%%

clear A A1 A1_sparse
A1 = zeros(10000, model.cnt_parameters) ;
A1_sparse = [] ;
A2 = sparse([],[],[],length(blocks)*max_neg, length(blocks)+nof_smooth,...
            length(blocks)*max_neg) ; 

q = 0 ; r = 0 ; q2 = 0 ;
num_complete = 0 ;

blocks(1).pred = struct;
for block_id = 1:length(blocks)
  if mod(block_id,50)==0, 
    block_id
  end ;
  block = blocks(block_id);
  block.truth(1).seg_path = [] ;
  for t=1:length(block.truth)
    block.truth(t) = gen_loss_matrix(block.truth(t), block.pos, block.gene_confirmed, model) ;
    [weights,loss] = compute_weights(block.truth(t), block, model);
    %if t==1, assert(loss==0) ; else assert(loss~=0) ; end ;
    if isempty(weights)
      warning('empty weights') ;
      block.truth(t).weights = blocks(1).truth(1).weights ; % hack
      keyboard
    else
      block.truth(t).weights = weights ;
    end
  end
  % generate wrong paths
  block.pred = gen_decoys(block, model, max_neg) ;
  % draw_pred(block.truth,block.pred,model)
  max_path = length(block.pred) ;
  for i = 1:max_path,
    [weights,loss] = compute_weights(block.pred(i), block, model);
    if isempty(weights)
      block.pred(i) = [];
      warning('invalid decoy path')
      max_path = max_path-1;
    else
      %assert(loss~=0) ;
      block.pred(i).weights = weights ;
      block.pred(i).loss = loss ;
    end
  end
  blocks(block_id) = block ;
  clear weights ;
  
  if generate_neg_constraints,
    for i=1:max_path,
      q = q+1 ; 
      q2 = q2+1 ; 
      A1(q2,1:model.cnt_parameters) = weights2vector(block.pred(i).weights)-weights2vector(block.truth(1).weights) ;
      
      if all(abs(A1(q2,:))<1e-3)
        error('haeh')
      end ;
      
      if q2==size(A1,1)
        A1_sparse = [A1_sparse; sparse(A1)] ;
        q2 = 0 ;
        A1(:) = 0 ;
      end ;
      A2(q, block_id)= -1 ; 
    end ;
  end ;
end ;

A1 = [A1_sparse; sparse(A1(1:q2,:))] ;
assert(q==size(A1,1));
clear A1_sparse
A2 = A2(1:q,:) ;
A = [A1 A2] ;

clear A1 A2
b = -margin*ones(q,1) ;

% penalization of 1st derivatives
B = spzeros(2*nof_smooth, model.cnt_parameters+length(blocks)+nof_smooth, ...
            6*nof_smooth) ;
QP.f = zeros(model.cnt_parameters+length(blocks)+nof_smooth,1) ;
QP.f( model.cnt_parameters+(1:length(blocks)) ) = 1;
q = 0 ;
for m = smooth_id
  param_ids = model.plif2param{m};
  for i = param_ids(1:end-1) 
    q = q + 1;
    B(q+q-1,i) = 1 ;
    B(q+q-1,i+1)=-1 ;
    B(q+q-1,model.cnt_parameters+length(blocks)+q)=-1 ;
    B(q+q,i)   = -1 ;
    B(q+q,i+1) = +1 ;
    B(q+q,model.cnt_parameters+length(blocks)+q)=-1 ;
    QP.f(model.cnt_parameters+length(blocks)+q) = C_regul.id2C(m) ;
  end;
end ;

% add L1 constraints and penalty for transitions
param_ids = model.transitions2param ;
for i = param_ids
  q = q + 1;
  B(q+q-1,i) = 1 ;
  B(q+q-1,model.cnt_parameters+length(blocks)+q)=-1 ;
  B(q+q,i)   = -1 ;
  B(q+q,model.cnt_parameters+length(blocks)+q)=-1 ;
  QP.f(model.cnt_parameters+length(blocks)+q) = C_regul.transitions ;
end;
% figure; imagesc(B)
assert( q == nof_smooth );
A = [A;B]; 
b = [b; zeros(2*nof_smooth,1)] ;
clear B


% monotonicity for all PLIFS

C = spzeros(nof_monoton, model.cnt_parameters+length(blocks)+nof_smooth, 2*nof_monoton) ;
q = 0 ;
for m = monoton_id,
  param_ids = model.plif2param{m};
  for i = param_ids(1:end-1)
    q = q+1 ;
    C(q,i) = 1 ;
    C(q,i+1) = -1 ;
  end ;
end ;
% figure;imagesc(C)
QP.A = [A;C]; 
QP.b = [b; zeros(nof_monoton,1)] ;

QP.lb = [-INF*ones(model.cnt_parameters,1) ; zeros(length(blocks)+nof_smooth,1)] ;
QP.ub = [+INF*ones(model.cnt_parameters+length(blocks)+nof_smooth,1)]  ;
% QP.ub = [];

Q = spzeros(model.cnt_parameters+length(blocks)+nof_smooth) ;
for i=1:model.cnt_parameters,
  if( i < model.cnt_parameters-model.cnt_transitions )
    Q(i,i) = C_regul.plif_ys_sq;
  else
    Q(i,i) = C_regul.transitions_sq ;
  end;
end ;
for i=model.cnt_parameters+length(blocks)+(1:nof_smooth-nof_trans_smooth)
  Q(i,i) = Q(i,i)+C_regul.smoothness_sq ;
end ;
QP.Q = Q;

if isfield(PAR, 'fn_regul_solution') ,
  L=load(PAR.fn_regul_solution) ;
  fprintf('regularizing with C=%1.2f against solution (norm=%1.2f)\nfrom file %s \n', ...
          C_regul.solution, norm(L.QP.res(1:model.cnt_parameters)), ...
          PAR.fn_regul_solution) ;
  assert(L.TRAIN_PAR.model.cnt_parameters==model.cnt_parameters) ;
  for k=1:L.TRAIN_PAR.model.cnt_parameters
    QP.Q(k,k) = QP.Q(k,k) + C_regul.solution ;
    QP.f(k) = QP.f(k) - C_regul.solution*L.QP.res(k) ;
  end ;
end ;

%%   checks
r = zeros(size(QP.f)) ;
r(model.cnt_parameters+(1:length(blocks))) = 1;
assert(all(QP.A*r-QP.b==0)) ;
