function [model] = make_model_conserv(PAR)
% [model] = make_model(bins)

model.use_conservation = 1 ;

model.bins = PAR.bins ;
model.use_gc =  PAR.use_gc;
polya_sixmers = PAR.polya_sixmers;
%polya_shift = 30 ;
%cleave_shift = polya_shift ;

% declare segment types
model.segments.intergenic = 0 ;
model.segments.utr5exon   = 1 ;
model.segments.transexon  = 2 ;
model.segments.cds_exon   = 3 ;
model.segments.utr3exon   = 4 ;
model.segments.polya_tail = 5 ;
model.segments.intercistronic  = 6 ;
model.segments.intron     = 7 ;
model.cnt_segments = length(fieldnames(model.segments));

% declare content types
content_start =  1 ;
model.contents.intergenic = content_start+0 ;
model.contents.utr_exon   = content_start+1 ;
model.contents.cds_exon   = content_start+2 ;
model.contents.intron     = content_start+3 ;
model.contents.intercistronic = content_start+4;
model.contents.cds_frame0   = content_start+5 ;
model.contents.cds_frame1   = content_start+6 ;
model.contents.cds_frame2   = content_start+7 ;
model.contents.intergenic_conserv = content_start+8 ;
model.contents.utr_exon_conserv   = content_start+9 ;
model.contents.cds_exon_conserv   = content_start+10 ;
model.contents.intron_conserv     = content_start+11 ;
model.contents.intercistronic_conserv = content_start+12;
model.contents.cds_frame0_conserv   = content_start+13 ;
model.contents.cds_frame1_conserv   = content_start+14 ;
model.contents.cds_frame2_conserv   = content_start+15 ;
model.cnt_contents = length(fieldnames(model.contents));

% declare content range
model.contents_range = struct;
content_names = fieldnames(model.contents) ;
for s = 1:length(content_names)
  model.contents_range = setfield(model.contents_range, content_names{s},[-10,10]);
end

% the first 5 contents consider all k-mers. the last three
% frame-content sensors only consider codons, with an offset from
% the end of the segment
model.mod_words = [ones(1,5) 3*ones(1,3);
                   zeros(1,5) 0 1 2]' ;
model.mod_words = [model.mod_words;model.mod_words] ;

% declare length types
length_start = model.cnt_contents+1 ;
model.lengths.intergenic = length_start+0;
model.lengths.utr5exon = length_start+1 ;
model.lengths.transexon = length_start+2 ;
model.lengths.single_cds_exon = length_start+3 ;
model.lengths.first_cds_exon = length_start+4 ;
model.lengths.middle_cds_exon = length_start+5 ;
model.lengths.last_cds_exon = length_start+6 ;
model.lengths.utr3exon = length_start+7;
model.lengths.polya_tail = length_start+8;
model.lengths.intron = length_start+9 ;
model.lengths.intercistronic = length_start+10 ;
model.lengths.intergenic_long = length_start+11 ;
model.cnt_lengths = length(fieldnames(model.lengths));

% declare length range
model.lengths_range.intergenic = [0, 999] ;
model.lengths_range.utr5exon   = [0, 2000] ;
model.lengths_range.transexon  = [1, 1000] ;
model.lengths_range.single_cds_exon = [100, 4000] ; %%%100, from don => acc+stop
model.lengths_range.first_cds_exon  = [3, 4000] ;
model.lengths_range.middle_cds_exon   = [3, 4000] ;
model.lengths_range.last_cds_exon   = [1, 4000] ;
model.lengths_range.utr3exon    = [1, 2000] ;
model.lengths_range.polya_tail  = [0, 150] ;
model.lengths_range.intercistronic = [1, 1000] ;
model.lengths_range.intergenic_long = [500-70, 500+70] ; % due to irregular spacing 
model.lengths_range.intron      = [30, 21000] ;%%%30, from don => acc+stop

% declare signal types
signal_start = model.cnt_contents + model.cnt_lengths ;
model.signals.tss = signal_start+1;
model.consensus.tss = {} ;
model.consensus_pos.tss = [];

model.signals.tis = signal_start+2;
model.consensus.tis = {'atg'} ;
model.consensus_pos.tis = [0];

model.signals.don = signal_start+3;
model.consensus.don = {'gt','gc'} ;
model.consensus_pos.don = [0,0];

model.signals.trans = signal_start+4;
model.consensus.trans = {'ag'} ;
model.consensus_pos.trans = [2];

model.signals.transacc = signal_start+5;
model.consensus.transacc = {'ag'} ;
model.consensus_pos.transacc = [2];

model.signals.acc = signal_start+6;
model.consensus.acc = {'ag'} ;
model.consensus_pos.acc = [2];

model.signals.stop = signal_start+7 ;
model.consensus.stop = {'taa','tga','tag'} ;
model.consensus_pos.stop = [0,0,0];

model.signals.polya = signal_start+8 ;
model.consensus.polya = polya_sixmers;
model.consensus_pos.polya = zeros(1,length(polya_sixmers)) ;

model.signals.cleave = signal_start+9 ;
model.consensus.cleave = {} ;
model.consensus_pos.cleave = [];
model.cnt_signals = length(fieldnames(model.signals));

% declare signal ranges for PLiFs
model.signals_range.tss = [-5,5] ;
model.signals_range.tis = [-5,5] ;
model.signals_range.trans = [0,1] ;
model.signals_range.transacc = [0,1] ;
model.signals_range.don = [0,1] ;
model.signals_range.acc = [0,1] ;
model.signals_range.stop = [-5,5] ;
model.signals_range.polya = [0,1] ;
model.signals_range.cleave = [-5,5] ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define states
model.states(1) = struct ;
cnt=0 ;

% sequence start 
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_start' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_start=cnt ;

signal_names = fieldnames(model.signals) ;
for i=1:model.cnt_signals
  if strcmp(signal_names{i}, 'transacc'), continue; end ;
  if ~strcmp(signal_names{i},'acc') && ~strcmp(signal_names{i},'don') ...
        && ~strcmp(signal_names{i}, 'tis') && ~strcmp(signal_names{i}, 'stop')...
        && ~strcmp(signal_names{i}, 'trans')
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},cnt) ;
  elseif strcmp(signal_names{i},'tis') || strcmp(signal_names{i},'stop')
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    if strcmp(signal_names{i}, 'tis')
      model.states(cnt).consensus = {'agatg'} ;
      model.states(cnt).consensus_pos = 2 ;
      model.states(cnt).signal = [model.signals.acc model.signals.tis] ;%%%% !!! 2 Zahlen
    else
      model.states(cnt).consensus = {'agtaa', 'agtag', 'agtga'} ;
      model.states(cnt).consensus_pos = [2 2 2] ;
      model.states(cnt).signal = [model.signals.acc model.signals.stop] ;%%%% !!! 2 Zahlen
    end
  elseif strcmp(signal_names{i},'trans') 
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    model.state_ids=setfield(model.state_ids, signal_names{i}, cnt) ;
  elseif strcmp(signal_names{i},'don')
    cnt_don = cnt ;
    cnt=cnt+8 ;
    for id = cnt_don+1:cnt_don+8 ;
      model.states(id).consensus = getfield(model.consensus,signal_names{i});
      model.states(id).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(id).non_consensus = {} ;
      model.states(id).non_consensus_pos = [] ;
      model.states(id).signal=model.signals.don ;
    end ;
    model.states(cnt_don+1).name='UTR5_don' ;
    model.states(cnt_don+2).name='CDS_don_0' ;
    model.states(cnt_don+3).name='CDS_don_1a' ;
    model.states(cnt_don+3).non_consensus = {'t'} ;
    model.states(cnt_don+3).non_consensus_pos= [1] ;
    model.states(cnt_don+4).name='CDS_don_1b' ;
    model.states(cnt_don+4).consensus = {'tgt','tgc'} ;
    model.states(cnt_don+4).consensus_pos= [1,1] ;
    model.states(cnt_don+5).name='CDS_don_2a' ;
    model.states(cnt_don+5).non_consensus = {'ta', 'tg'} ;
    model.states(cnt_don+5).non_consensus_pos= [2,2] ;
    model.states(cnt_don+6).name='CDS_don_2b' ;
    model.states(cnt_don+6).consensus = {'tagt', 'tagc'} ;
    model.states(cnt_don+6).consensus_pos= [2,2] ;
    model.states(cnt_don+7).name='CDS_don_2c' ;
    model.states(cnt_don+7).consensus = {'tggt', 'tggc'} ;
    model.states(cnt_don+7).consensus_pos= [2,2] ;
    model.states(cnt_don+8).name='UTR3_don' ;
    
    model.state_ids.don=cnt_don+1:cnt_don+8 ;
  elseif strcmp(signal_names{i},'acc')
    cnt_acc = cnt ;
    cnt=cnt+8 ;
    for id = cnt_acc+1:cnt_acc+8 ;
      model.states(id).consensus = {'ag'} ;
      model.states(id).consensus_pos= [2] ;
      model.states(id).non_consensus={} ;
      model.states(id).non_consensus_pos=[] ;
      model.states(id).signal=model.signals.acc ;
    end ;
    model.states(cnt_acc+1).name='UTR5_acc' ;
    model.states(cnt_acc+2).name='CDS_acc_0' ;
    model.states(cnt_acc+3).name='CDS_acc_1a' ;
    model.states(cnt_acc+4).name='CDS_acc_1b' ;
    model.states(cnt_acc+4).non_consensus = {'aa','ag','ga'} ;
    model.states(cnt_acc+4).non_consensus_pos= [0,0,0] ;
    model.states(cnt_acc+5).name='CDS_acc_2a' ;
    model.states(cnt_acc+6).name='CDS_acc_2b' ;
    model.states(cnt_acc+6).consensus = {'agt', 'agc'} ;
    model.states(cnt_acc+6).consensus_pos= [2, 2] ;
    model.states(cnt_acc+7).name='CDS_acc_2c' ;
    model.states(cnt_acc+7).non_consensus = {'a'} ;
    model.states(cnt_acc+7).non_consensus_pos= [0] ;
    model.states(cnt_acc+8).name='UTR3_acc' ;
    
    model.state_ids.acc=cnt_acc+1:cnt_acc+8 ;
%    cnt
  end
end

% intergenic_long
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='intergenic_long' ;
model.states(cnt).signal=[] ;
model.state_ids.intergenic_long=cnt ;

% sequence end
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_end' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_end = cnt ;

model.cnt_states = cnt ; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define mappings between length, contents and segments
 
model.plif_links = [] ;
 
model.plif_links(end+1,:) = [model.lengths.intergenic      model.contents.intergenic     model.contents.intergenic_conserv] ;
model.plif_links(end+1,:) = [model.lengths.first_cds_exon  model.contents.cds_exon       model.contents.cds_exon_conserv] ;
model.plif_links(end+1,:) = [model.lengths.last_cds_exon   model.contents.cds_exon       model.contents.cds_exon_conserv] ;
model.plif_links(end+1,:) = [model.lengths.middle_cds_exon model.contents.cds_exon       model.contents.cds_exon_conserv] ;
model.plif_links(end+1,:) = [model.lengths.single_cds_exon model.contents.cds_exon       model.contents.cds_exon_conserv] ;
model.plif_links(end+1,:) = [model.lengths.intron          model.contents.intron         model.contents.intron_conserv] ;
model.plif_links(end+1,:) = [model.lengths.transexon       model.contents.utr_exon       model.contents.utr_exon_conserv] ;
model.plif_links(end+1,:) = [model.lengths.utr5exon        model.contents.utr_exon       model.contents.utr_exon_conserv] ;
model.plif_links(end+1,:) = [model.lengths.utr3exon        model.contents.utr_exon       model.contents.utr_exon_conserv] ;
model.plif_links(end+1,:) = [model.lengths.polya_tail      model.contents.utr_exon       model.contents.utr_exon_conserv] ;
model.plif_links(end+1,:) = [model.lengths.intercistronic  model.contents.intercistronic model.contents.intercistronic_conserv] ;
model.plif_links(end+1,:) = [model.lengths.intergenic_long model.contents.intergenic     model.contents.intergenic_conserv] ;

model.seg_links = [] ;
model.seg_links(end+1,:) = [model.lengths.intergenic      model.segments.intergenic] ;
model.seg_links(end+1,:) = [model.lengths.first_cds_exon  model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.last_cds_exon   model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.middle_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.single_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.intron          model.segments.intron] ;
model.seg_links(end+1,:) = [model.lengths.transexon       model.segments.transexon] ;
model.seg_links(end+1,:) = [model.lengths.utr5exon        model.segments.utr5exon] ;
model.seg_links(end+1,:) = [model.lengths.utr3exon        model.segments.utr3exon] ;
model.seg_links(end+1,:) = [model.lengths.polya_tail      model.segments.polya_tail] ;
model.seg_links(end+1,:) = [model.lengths.intercistronic  model.segments.intercistronic] ;
model.seg_links(end+1,:) = [model.lengths.intergenic_long model.segments.intergenic] ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define transitions

A = -inf(model.cnt_states, model.cnt_states);
p = [0 -inf(1,model.cnt_states-1)] ;
q = [-inf(1,model.cnt_states-1) 0] ;
penalties = zeros(model.cnt_states, model.cnt_states) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
use_tss           = 1 ;
allow_missing_tss = 0 ;
use_one_gene      = 0 ; % force exactly one gene
use_trans         = 1 ; 
use_polya         = 1 ; % polyA signal optional before cleave
use_cleave        = 1 ; % cleaving site exists and has to follow
                        % after stop
use_multi         = 1 ; % allow more than one gene 
use_operon        = 1 ;

% code does not work yet otherwise
assert(use_cleave) ;

% start constructing transition matrix A
A=-inf*ones(model.cnt_states) ;
orf_from = -1*ones(model.cnt_states,1) ;
orf_to = -1*ones(model.cnt_states,1) ;

orf_from(model.state_ids.acc(2:7))= [ 0 1 1 2 2 2] ;
orf_from(model.state_ids.tis)     = 0 ;
orf_to(model.state_ids.don(2:7))  = [0 1 1 2 2 2] ;
orf_to(model.state_ids.stop(1))   = 0 ;
orf_info=[orf_from, orf_to] ;

gene_start_states = model.state_ids.tss ;
if use_tss,
  if allow_missing_tss,
    A(model.state_ids.seq_start, [model.state_ids.tis model.state_ids.tss final])=model.lengths.intergenic ;
    gene_start_states = [model.state_ids.tss model.state_ids.tis] ;
  else
    A(model.state_ids.seq_start, [model.state_ids.tss model.state_ids.seq_end]) = model.lengths.intergenic ;
    A(model.state_ids.seq_start, model.state_ids.intergenic_long) = model.lengths.intergenic_long ;
    A(model.state_ids.intergenic_long, model.state_ids.intergenic_long)=0 ;
    A(model.state_ids.intergenic_long, [model.state_ids.tss model.state_ids.seq_end])=model.lengths.intergenic_long ;
  end ;
else
  if use_one_gene,
    A(model.state_ids.seq_start, model.state_ids.tis)=model.lengths.intergenic ;
  else
    A(model.state_ids.seq_start, [model.state_ids.tis model.state_ids.seq_end])=model.lengths.intergenic ;
  end ;
end ;

if use_trans,
  A(model.state_ids.intergenic_long, model.state_ids.trans) = model.lengths.intergenic ;
  A(model.state_ids.seq_start, model.state_ids.trans) = model.lengths.intergenic ;
  if use_tss,
    A(model.state_ids.tss, model.state_ids.trans)    = model.lengths.utr5exon ; 
    A(model.state_ids.acc(1), model.state_ids.trans) = model.lengths.utr5exon ; 
    A(model.state_ids.trans, model.state_ids.tis(1)) = model.lengths.transexon ; 
    A(model.state_ids.tss, model.state_ids.tis(2))   = model.lengths.utr5exon ;
  else
    A(model.state_ids.trans, model.state_ids.tis(1)) = model.lengths.transexon ;
  end ;
end ;
A(model.state_ids.tis, model.state_ids.stop(1)) = model.lengths.single_cds_exon ;

if use_cleave,
  if use_multi,
    A(model.state_ids.stop, model.state_ids.cleave)       = model.lengths.utr3exon ;
	A(model.state_ids.cleave, gene_start_states)          = model.lengths.intergenic ; 
	A(model.state_ids.cleave, model.state_ids.intergenic_long) = model.lengths.intergenic_long ; 
	A(model.state_ids.intergenic_long, model.state_ids.intergenic_long) = 0;
	A(model.state_ids.intergenic_long, gene_start_states) = model.lengths.intergenic_long ; 
	A(model.state_ids.cleave, model.state_ids.seq_end)    = model.lengths.intergenic ; 
	A(model.state_ids.cleave, model.state_ids.intergenic_long)  = model.lengths.intergenic_long ; 
	A(model.state_ids.intergenic_long, model.state_ids.seq_end) = model.lengths.intergenic_long ; 
	gene_stop_states = [model.state_ids.cleave model.state_ids.seq_end] ;
  else
    A(model.state_ids.cleave, model.state_ids.seq_end)    = model.lengths.intergenic ;
	gene_stop_states = [model.state_ids.cleave model.state_ids.seq_end] ;
  end ;
else
  if use_multi,
	A(model.state_ids.stop, gene_start_states) = model.lengths.intergenic  ;
	A(model.state_ids.stop, model.state_ids.seq_end) = model.lengths.intergenic ;
	gene_stop_states = [model.state_ids.stop model.state_ids.seq_end] ;
  else
    A(model.state_ids.stop, model.state_ids.seq_end) = model.lengths.intergenic ;
	gene_stop_states = [model.state_ids.stop model.state_ids.seq_end] ;
  end ;
end ;

p=-inf(model.cnt_states,1) ;
p(model.state_ids.seq_start) = 0 ;
q=-inf(model.cnt_states,1) ;
q(gene_stop_states) = 0 ;

% basic stuff
% 5' UTR
A(model.state_ids.tss, model.state_ids.tis)=model.lengths.utr5exon ;
A(model.state_ids.tss, model.state_ids.don(1))=model.lengths.utr5exon ;
% A(model.state_ids.acc(1), model.state_ids.tis)=model.lengths.utr5exon ;
A(model.state_ids.acc(1), model.state_ids.tis(1))=model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.tis(2))=model.lengths.utr5exon ;
%%
A(model.state_ids.acc(1), model.state_ids.don(1)) = model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.acc(1)) = model.lengths.intron ;

% CDS 
A(model.state_ids.tis, model.state_ids.don(2:7))=model.lengths.first_cds_exon ;
for i=2:7,
  A(model.state_ids.don(i), model.state_ids.acc(i)) = model.lengths.intron ;
end ;
A(model.state_ids.acc(2:7), model.state_ids.don(2:7)) = model.lengths.middle_cds_exon ;
A(model.state_ids.acc(2:7), model.state_ids.stop(1))=model.lengths.last_cds_exon ;
% agstop 
A(model.state_ids.don(2), model.state_ids.stop(2)) = model.lengths.intron ;

% 3' UTR
A(model.state_ids.stop, model.state_ids.don(8))=model.lengths.utr3exon ;
A(model.state_ids.don(8), model.state_ids.acc(8)) = model.lengths.intron ;
A(model.state_ids.acc(8), model.state_ids.don(8))=model.lengths.utr3exon ;
A(model.state_ids.stop, model.state_ids.cleave)=model.lengths.utr3exon ;
A(model.state_ids.acc(8), model.state_ids.cleave)=model.lengths.utr3exon ;

if use_polya,
  % polyA has to be in last 3'UTR exon
  A(model.state_ids.stop, model.state_ids.polya) = model.lengths.utr3exon ;
  A(model.state_ids.polya, model.state_ids.cleave) = model.lengths.polya_tail ;
  A(model.state_ids.acc(8), model.state_ids.polya)=model.lengths.utr3exon ;
end ;

if use_operon,
  A(model.state_ids.cleave, model.state_ids.trans) = model.lengths.intercistronic ;
end ;

%%% penalties contain for each allowed transition pointers to lengths plifs
penalties = zeros(size(A)) ;
penalties(~isinf(A))=A(~isinf(A)) ;
penalties = penalties' ;

%%% add pointers to content PLIFs
penalties(:,:,2:3) = zeros(size(penalties,1), size(penalties,2), 2) ;
for i=1:size(model.plif_links,1)
  [idx1,idx2] = find(penalties(:,:,1)==model.plif_links(i,1));
  for j=1:length(idx1)
    penalties(idx1(j),idx2(j),2:3) = model.plif_links(i,2:3);
  end ;
end
%%% add pointers to frame PLIFs
tmp = zeros(size(penalties(:,:,1))) ;
tmp(model.state_ids.tis, model.state_ids.stop(1)) = model.contents.cds_frame0 ;% single cds exon 
tmp(model.state_ids.tis, model.state_ids.don(2)) = model.contents.cds_frame0 ;% first cds exon 
tmp(model.state_ids.tis, model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% first cds exon 
tmp(model.state_ids.tis, model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% first cds exon 

tmp(model.state_ids.acc(2:7), model.state_ids.don(2)) = model.contents.cds_frame0 ;% middle cds exon 
tmp(model.state_ids.acc(2:7), model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% middle cds exon 
tmp(model.state_ids.acc(2:7), model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% middle cds exon 

tmp(model.state_ids.acc(2:7), model.state_ids.stop(1)) = model.contents.cds_frame0 ;% last cds exon 

tmp2 = zeros(size(penalties(:,:,1))) ;
tmp2(model.state_ids.tis, model.state_ids.stop(1)) = model.contents.cds_frame0_conserv ;% single cds exon 
tmp2(model.state_ids.tis, model.state_ids.don(2)) = model.contents.cds_frame0_conserv ;% first cds exon 
tmp2(model.state_ids.tis, model.state_ids.don(3:4)) = model.contents.cds_frame1_conserv ;% first cds exon 
tmp2(model.state_ids.tis, model.state_ids.don(5:7)) = model.contents.cds_frame2_conserv ;% first cds exon 

tmp2(model.state_ids.acc(2:7), model.state_ids.don(2)) = model.contents.cds_frame0_conserv ;% middle cds exon 
tmp2(model.state_ids.acc(2:7), model.state_ids.don(3:4)) = model.contents.cds_frame1_conserv ;% middle cds exon 
tmp2(model.state_ids.acc(2:7), model.state_ids.don(5:7)) = model.contents.cds_frame2_conserv ;% middle cds exon 

tmp2(model.state_ids.acc(2:7), model.state_ids.stop(1)) = model.contents.cds_frame0_conserv ;% last cds exon 

penalties(:,:,4) = tmp' ;
penalties(:,:,5) = tmp2' ;

%keyboard
%for i=1:size(penalties,1)
%  for j=1:size(penalties,2)
%    idx=find(penalties(i,j,:)~=0) ;
%    idx2=find(penalties(i,j,:)==0) ;
%    assert(equal(idx', 1:length(idx))) ;
%    %penalties(i,j,:) = [penalties(i,j,idx) penalties(i,j,idx2)] ;
%  end ;
%end ;
A(~isinf(A))=0 ;

model.intergenic_long.transitions = [...
    model.state_ids.seq_start model.state_ids.tss
    model.state_ids.seq_start model.state_ids.trans
    model.state_ids.cleave model.state_ids.tss
    model.state_ids.cleave model.state_ids.trans
    model.state_ids.cleave model.state_ids.seq_end] ;
model.intergenic_long.max_short_length = 1000 ;
model.intergenic_long.long_segment_parts = 500 ;
model.intergenic_long.step = 30 ; % makes about 30 (step) * 40 (grid)=1200nt per step

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model.A = A ;
model.p = p ;
model.q = q ;
model.penalties = penalties ;
model.orf_info = orf_info ;

model.active_transitions = find(~isinf(A)) ;
model.cnt_transitions = length(model.active_transitions) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model.cnt_plifs = model.cnt_contents+model.cnt_lengths+model.cnt_signals ;
model.cnt_parameters = model.cnt_transitions + model.cnt_plifs*model.bins ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% define loss 

a_trans = zeros(4,sum(~isinf(A(:)))) ;
k = 0 ;
for i=1:size(A,1)
  idx = find(~isinf(A(i,:))) ;
  val = A(i,idx) ;
  pen = penalties(idx,i,1) ;
  a_trans(1,k+1:k+length(idx))=i-1 ;
  a_trans(2,k+1:k+length(idx))=idx-1 ;  
  a_trans(3,k+1:k+length(idx))=val ;    
  for q=1:length(pen) 
    idx_=find(model.seg_links(:,1)==pen(q)) ;
    if isempty(idx_), 
      assert(pen(q)==0) ; 
    else
      assert(length(idx_)==1) ;
      pen(q) = model.seg_links(idx_,2) ;
    end ;
  end ;
  a_trans(4,k+1:k+length(idx))= pen;
  k=k+length(idx) ;
end ;
%a_trans = a_trans' ;
%[tmp,idx] = sort(a_trans(:,2)) ;
%a_trans = a_trans(idx,:) ;
model.a_trans = a_trans ;

% create loss matrices
model.loss.segments = ones(model.cnt_segments) - eye(model.cnt_segments);
equiv = [model.segments.intergenic, model.segments.intercistronic;
         model.segments.utr3exon, model.segments.polya_tail;
         model.segments.intergenic, model.segments.polya_tail;
         model.segments.utr5exon, model.segments.transexon] ;
for i=1:size(equiv,1)
  model.loss.segments(equiv(i,1)+1, equiv(i,2)+1) = 0 ;
  model.loss.segments(equiv(i,2)+1, equiv(i,1)+1) = 0 ;
end ;
% tripple loss for coding
%model.loss.segments(model.segments.cds_exon+1, :) = 3*model.loss.segments(model.segments.cds_exon+1, :) ;
%model.loss.segments(:, model.segments.cds_exon+1) = 3*model.loss.segments(:, model.segments.cds_exon+1) ;

% first part is for segments, the second for nucleotides
model.loss.segments = [model.loss.segments 0.0*model.loss.segments] ;

model.loss.zero_range.tss = 50 ;
model.loss.zero_range.tis = 0;
model.loss.zero_range.don = 0;
model.loss.zero_range.acc = 0;
model.loss.zero_range.trans = 0;
model.loss.zero_range.stop = 0;
model.loss.zero_range.polya = 50 ;
model.loss.zero_range.cleave = 50 ;

model.loss.confirmed = 1 ;
model.loss.confirmed_tol = 1000 ; % region around confirmed gene
model.loss.unconfirmed = 1 ;

% model.loss.mustbe_corr.intergenic = 0 ;
% model.loss.mustbe_corr.utr5exon   = 0;
% model.loss.mustbe_corr.transexon  = 0 ;
% model.loss.mustbe_corr.cds_exon   = 1 ;
% model.loss.mustbe_corr.utr3exon   = 0 ;
% model.loss.mustbe_corr.polya_tail = 0 ;
% model.loss.mustbe_corr.intercistronic  = 0 ;
% model.loss.mustbe_corr.intron     = 1 ;
return ;
