function [model] = make_model_onlyorf(PAR)
% [model] = make_model(bins)

model.bins = PAR.lsl.bins ;
model.use_gc =  PAR.lsl.use_gc;

% declare segment types
model.segments.intergenic = 0 ;
model.segments.cds_exon   = 1 ;
model.segments.intron     = 2 ;
model.cnt_segments = length(fieldnames(model.segments));

% declare content types
content_start =  1 ;
model.contents.intergenic = content_start+0 ;
model.contents.cds_exon   = content_start+1 ;
model.contents.intron     = content_start+2 ;
model.contents.cds_frame0   = content_start+3 ;
model.contents.cds_frame1   = content_start+4 ;
model.contents.cds_frame2   = content_start+5 ;
model.cnt_contents = length(fieldnames(model.contents));

% declare content range
model.contents_range = struct;
content_names = fieldnames(model.contents) ;
for s = 1:length(content_names)
  model.contents_range = setfield(model.contents_range, content_names{s},[-10,10]);
end

% the first 5 contents consider all k-mers. the last three
% frame-content sensors only consider codons, with an offset from
% the end of the segment
model.mod_words = [ones(1,5) 3*ones(1,3);
                   zeros(1,5) 0 1 2]' ;

% declare length types
length_start = model.cnt_contents+1 ;
model.lengths.intergenic = length_start+0;
model.lengths.single_cds_exon = length_start+1 ;
model.lengths.first_cds_exon = length_start+2 ;
model.lengths.middle_cds_exon = length_start+3 ;
model.lengths.last_cds_exon = length_start+4 ;
model.lengths.intron = length_start+5 ;
model.cnt_lengths = length(fieldnames(model.lengths));

% declare length range
model.lengths_range.intergenic = [0, 999] ;
model.lengths_range.single_cds_exon = [100, 4000] ; %%%100, from don => acc+cdsStop
model.lengths_range.first_cds_exon  = [3, 4000] ;
model.lengths_range.middle_cds_exon   = [3, 4000] ;
model.lengths_range.last_cds_exon   = [1, 4000] ;
model.lengths_range.intron      = [30, 21000] ;%%%30, from don => acc+cdsStop

% declare signal types
signal_start = model.cnt_contents + model.cnt_lengths ;

model.signals.tis = signal_start+1;
model.consensus.tis = {'atg'} ;
model.consensus_pos.tis = [0];

model.signals.don = signal_start+2;
model.consensus.don = {'gt','gc'} ;
model.consensus_pos.don = [0,0];

model.signals.acc = signal_start+3;
model.consensus.acc = {'ag'} ;
model.consensus_pos.acc = [2];

model.signals.cdsStop = signal_start+4 ;
model.consensus.cdsStop = {'taa','tga','tag'} ;
model.consensus_pos.cdsStop = [0,0,0];

model.cnt_signals = length(fieldnames(model.signals));

model.signals_range.tis = [0,1] ;
model.signals_range.don = [0,1] ;
model.signals_range.acc = [0,1] ;
model.signals_range.cdsStop = [0,1] ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define states
model.states(1) = struct ;
cnt=0 ;

% sequence start 
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_start' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_start=cnt ;

signal_names = fieldnames(model.signals) ;
for i=1:model.cnt_signals
  if ~strcmp(signal_names{i},'acc') && ~strcmp(signal_names{i},'don') ...
        && ~strcmp(signal_names{i}, 'tis') && ~strcmp(signal_names{i}, 'cdsStop')
    keyboard
    
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},cnt) ;
  elseif strcmp(signal_names{i},'tis') || strcmp(signal_names{i},'cdsStop')
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    if strcmp(signal_names{i}, 'tis')
      model.states(cnt).consensus = {'agatg'} ;
      model.states(cnt).consensus_pos = 2 ;
      model.states(cnt).signal = [model.signals.acc model.signals.tis] ;%%%% !!! 2 Zahlen
    else
      model.states(cnt).consensus = {'agtaa', 'agtag', 'agtga'} ;
      model.states(cnt).consensus_pos = [2 2 2] ;
      model.states(cnt).signal = [model.signals.acc model.signals.cdsStop] ;%%%% !!! 2 Zahlen
    end
  elseif strcmp(signal_names{i},'don')
    cnt_don = cnt ;
    cnt=cnt+6 ;
    for id = cnt_don+1:cnt_don+6 ;
      model.states(id).consensus = getfield(model.consensus,signal_names{i});
      model.states(id).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(id).non_consensus = {} ;
      model.states(id).non_consensus_pos = [] ;
      model.states(id).signal=model.signals.don ;
    end ;
    model.states(cnt_don+1).name='CDS_don_0' ;
    model.states(cnt_don+2).name='CDS_don_1a' ;
    model.states(cnt_don+2).non_consensus = {'t'} ;
    model.states(cnt_don+2).non_consensus_pos= [1] ;
    model.states(cnt_don+3).name='CDS_don_1b' ;
    model.states(cnt_don+3).consensus = {'tgt','tgc'} ;
    model.states(cnt_don+3).consensus_pos= [1,1] ;
    model.states(cnt_don+4).name='CDS_don_2a' ;
    model.states(cnt_don+4).non_consensus = {'ta', 'tg'} ;
    model.states(cnt_don+4).non_consensus_pos= [2,2] ;
    model.states(cnt_don+5).name='CDS_don_2b' ;
    model.states(cnt_don+5).consensus = {'tagt', 'tagc'} ;
    model.states(cnt_don+5).consensus_pos= [2,2] ;
    model.states(cnt_don+6).name='CDS_don_2c' ;
    model.states(cnt_don+6).consensus = {'tggt', 'tggc'} ;
    model.states(cnt_don+6).consensus_pos= [2,2] ;
    
    model.state_ids.don=cnt_don+1:cnt_don+6 ;
  elseif strcmp(signal_names{i},'acc')
    cnt_acc = cnt ;
    cnt=cnt+6 ;
    for id = cnt_acc+1:cnt_acc+6 ;
      model.states(id).consensus = {'ag'} ;
      model.states(id).consensus_pos= [2] ;
      model.states(id).non_consensus={} ;
      model.states(id).non_consensus_pos=[] ;
      model.states(id).signal=model.signals.acc ;
    end ;
    model.states(cnt_acc+1).name='CDS_acc_0' ;
    model.states(cnt_acc+2).name='CDS_acc_1a' ;
    model.states(cnt_acc+3).name='CDS_acc_1b' ;
    model.states(cnt_acc+3).non_consensus = {'aa','ag','ga'} ;
    model.states(cnt_acc+3).non_consensus_pos= [0,0,0] ;
    model.states(cnt_acc+4).name='CDS_acc_2a' ;
    model.states(cnt_acc+5).name='CDS_acc_2b' ;
    model.states(cnt_acc+5).consensus = {'agt', 'agc'} ;
    model.states(cnt_acc+5).consensus_pos= [2, 2] ;
    model.states(cnt_acc+6).name='CDS_acc_2c' ;
    model.states(cnt_acc+6).non_consensus = {'a'} ;
    model.states(cnt_acc+6).non_consensus_pos= [0] ;
    
    model.state_ids.acc=cnt_acc+1:cnt_acc+6 ;
%    cnt
  end
end

% sequence end
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_end' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_end = cnt ;

model.cnt_states = cnt ; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define mappings between length, contents and segments
 
model.plif_links = [] ;
 
model.plif_links(end+1,:) = [model.lengths.intergenic      model.contents.intergenic] ;
model.plif_links(end+1,:) = [model.lengths.first_cds_exon  model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.last_cds_exon   model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.middle_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.single_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.intron          model.contents.intron] ;

model.seg_links = [] ;
model.seg_links(end+1,:) = [model.lengths.intergenic      model.segments.intergenic] ;
model.seg_links(end+1,:) = [model.lengths.first_cds_exon  model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.last_cds_exon   model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.middle_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.single_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.intron          model.segments.intron] ;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

use_one_gene      = 0 ; % force exactly one gene
use_multi         = 1 ; % allow more than one gene 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define transitions

A = -inf(model.cnt_states, model.cnt_states);
p = [0 -inf(1,model.cnt_states-1)] ;
q = [-inf(1,model.cnt_states-1) 0] ;
transition_pointers = zeros(model.cnt_states, model.cnt_states) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% start constructing transition matrix A

orf_from = -1*ones(model.cnt_states,1) ;
orf_to = -1*ones(model.cnt_states,1) ;

orf_from(model.state_ids.acc)= [ 0 1 1 2 2 2] ;
orf_from(model.state_ids.tis)     = 0 ;
orf_to(model.state_ids.don)  = [0 1 1 2 2 2] ;
orf_to(model.state_ids.cdsStop(1))   = 0 ;
orf_info=[orf_from, orf_to] ;

gene_start_states = model.state_ids.tis ;
if use_one_gene,
  A(model.state_ids.seq_start, model.state_ids.tis)=model.lengths.intergenic ;
else
  A(model.state_ids.seq_start, [model.state_ids.tis model.state_ids.seq_end])=model.lengths.intergenic ;
end ;

A(model.state_ids.tis, model.state_ids.cdsStop(1)) = model.lengths.single_cds_exon ;

if use_multi,
  A(model.state_ids.cdsStop, [gene_start_states model.state_ids.seq_end])          = model.lengths.intergenic ; 
  gene_stop_states = [model.state_ids.cdsStop model.state_ids.seq_end] ;
else
  A(model.state_ids.cdsStop, model.state_ids.seq_end) = model.lengths.intergenic ;
  gene_stop_states = [model.state_ids.cdsStop model.state_ids.seq_end] ;
end ;

p=-inf(model.cnt_states,1) ;
p(model.state_ids.seq_start) = 0 ;
q=-inf(model.cnt_states,1) ;
q(gene_stop_states) = 0 ;

% basic stuff
% CDS 
A(model.state_ids.tis, model.state_ids.don)=model.lengths.first_cds_exon ;
for i=1:6,
  A(model.state_ids.don(i), model.state_ids.acc(i)) = model.lengths.intron ;
end ;
A(model.state_ids.acc, model.state_ids.don) = model.lengths.middle_cds_exon ;
A(model.state_ids.acc, model.state_ids.cdsStop(1))=model.lengths.last_cds_exon ;
% agstop 
A(model.state_ids.don(1), model.state_ids.cdsStop(2)) = model.lengths.intron ;


%%% transition_pointers contain for each allowed transition pointers to lengths plifs
transition_pointers = zeros(size(A)) ;
transition_pointers(~isinf(A))=A(~isinf(A)) ;
transition_pointers = transition_pointers' ;

%%% add pointers to content PLIFs
transition_pointers(:,:,2) = zeros(size(transition_pointers,1), size(transition_pointers,2), 1) ;
for i=1:size(model.plif_links,1)
  [idx1,idx2] = find(transition_pointers(:,:,1)==model.plif_links(i,1));
  for j=1:length(idx1)
    transition_pointers(idx1(j),idx2(j),2) = model.plif_links(i,2);
  end ;
end

%%% add pointers to frame PLIFs
tmp = zeros(size(transition_pointers(:,:,1))) ;
tmp(model.state_ids.tis, model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% single cds exon 
tmp(model.state_ids.tis, model.state_ids.don(1)) = model.contents.cds_frame0 ;% first cds exon 
tmp(model.state_ids.tis, model.state_ids.don(2:3)) = model.contents.cds_frame1 ;% first cds exon 
tmp(model.state_ids.tis, model.state_ids.don(4:6)) = model.contents.cds_frame2 ;% first cds exon 

tmp(model.state_ids.acc, model.state_ids.don(1)) = model.contents.cds_frame0 ;% middle cds exon 
tmp(model.state_ids.acc, model.state_ids.don(2:3)) = model.contents.cds_frame1 ;% middle cds exon 
tmp(model.state_ids.acc, model.state_ids.don(4:6)) = model.contents.cds_frame2 ;% middle cds exon 

tmp(model.state_ids.acc, model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% last cds exon 

transition_pointers(:,:,3) = tmp' ;
A(~isinf(A))=0 ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model.A = A ;
model.p = p ;
model.q = q ;
model.transition_pointers = transition_pointers ;
model.orf_info = orf_info ;

model.active_transitions = find(~isinf(A)) ;
model.cnt_transitions = length(model.active_transitions) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model.cnt_plifs = model.cnt_contents+model.cnt_lengths+model.cnt_signals ;
model.cnt_parameters = model.cnt_transitions + model.cnt_plifs*model.bins ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% define loss 

a_trans = zeros(4,sum(~isinf(A(:)))) ;
% [from,to,val,segment]
k = 0 ;
for i=1:size(A,1)
  idx = find(~isinf(A(i,:))) ;
  val = A(i,idx) ;
  pen = transition_pointers(idx,i,1) ;
  a_trans(1,k+1:k+length(idx))=i-1 ;
  a_trans(2,k+1:k+length(idx))=idx-1 ;  
  a_trans(3,k+1:k+length(idx))=val ;    
  for q=1:length(pen) 
    idx_=find(model.seg_links(:,1)==pen(q)) ;
    if isempty(idx_), 
      assert(pen(q)==0) ; 
    else
      assert(length(idx_)==1) ;
      pen(q) = model.seg_links(idx_,2) ;
    end ;
  end ;
  a_trans(4,k+1:k+length(idx))= pen;
  k=k+length(idx) ;
end ;
%a_trans = a_trans' ;
%[tmp,idx] = sort(a_trans(:,2)) ;
%a_trans = a_trans(idx,:) ;
model.a_trans = a_trans ;

% create loss matrices
model.loss.segments = ones(model.cnt_segments) - eye(model.cnt_segments);

if 1,
  different = [model.segments.intergenic, model.segments.cds_exon, 3 ;
               model.segments.intergenic, model.segments.intron, 3;
               model.segments.intron, model.segments.cds_exon, 3];
  for i=1:size(different,1)
    model.loss.segments(different(i,1)+1, different(i,2)+1) = different(i,3) ;
    model.loss.segments(different(i,2)+1, different(i,1)+1) = different(i,3) ;
  end ;
end ;

% tripple loss for coding
%model.loss.segments(model.segments.cds_exon+1, :) = 3*model.loss.segments(model.segments.cds_exon+1, :) ;
%model.loss.segments(:, model.segments.cds_exon+1) = 3*model.loss.segments(:, model.segments.cds_exon+1) ;

% first part is for segments, the second for nucleotides
model.loss.segments = [model.loss.segments 0.0*model.loss.segments] ;

model.loss.zero_range.tis = 0;
model.loss.zero_range.don = 0;
model.loss.zero_range.acc = 0;
model.loss.zero_range.cdsStop = 0;

model.loss.confirmed = 1 ;
model.loss.confirmed_tol = 1000 ; % region around confirmed gene
model.loss.unconfirmed = 1 ;

return ;
