function [blocks,PAR,P] = pred_path(PAR)
% [blocks,PAR,P] = pred_path(PAR)
%
%
check_split = 0 ;
  
fid = fopen( sprintf('%spred.txt',PAR.fn_log),'a+') ;
fprintf(fid,'\n\n pred path \n');
fprintf(fid,'date: %s\n',date);
fprintf(fid,'\n start \n');

if ~isfield(PAR, 'submit')
  PAR.submit=1; 
end ;

%% parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isfield(PAR, 'iteration')
  iteration = 1 ;
  while (1)
    fname = [PAR.fn_train '_iteration' num2str(iteration) '.mat'];
    if ~fexist(fname), iteration=iteration-1; break ; end 
    iteration=iteration+1 ;
  end ;
else
  iteration = PAR.iteration;
end ;
fname = [PAR.fn_train '_iteration' num2str(iteration) '.mat'];
fprintf('load iteration %i\n', iteration) ;
if check_split,
  load(fname, 'QP', 'TRAIN_PAR', 'num_examples');
  fname = [PAR.fn_train '_iteration' num2str(iteration) '_blocks.mat'];
  blocks_train=load_struct(fname);
  idx = [blocks_train.id] ;
  clear blocks_train
else
  load(fname, 'QP', 'TRAIN_PAR', 'num_examples');
end ;

%% load data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

blocks=load_struct(PAR.fn_blocks, 'blocks') ;
if isfield(PAR,'fn_splits')
  load(PAR.fn_splits, 'blocks_split') ; 
  block_idx = [blocks_split{PAR.pred_split_idx}] ;
  if check_split
    if ~isempty(intersect(idx, block_idx))
      disp('test and training are overlapping') ;
      keyboard
      disp('correcting indices') ;
      block_idx = setdiff([blocks_split{:}], idx) ;
    end ;
  end ;
  blocks = blocks(block_idx) ;
end
fprintf('predicting on %i blocks\n', length(blocks)) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isfield(TRAIN_PAR.model, 'use_conservation') && TRAIN_PAR.model.use_conservation 
  disp('Converting conservation levels') ;
  for id = 1:length(blocks),
    blocks(id).genestr_conserv = conservation2DNA_alphabet(blocks(id).conservation) ;
  end ;  
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%if isfield(PAR, 'use_gc') && ~PAR.use_gc
%  disp('Removing GC splice sites') ;
%  for id = 1:length(blocks),
%    idx=find(blocks(id).genestr(blocks(id).details.pos.don+1)=='T') ;
%    blocks(id).details.pos.don=blocks(id).details.pos.don(idx) ;
%    blocks(id).details.score.don=blocks(id).details.score.don(idx) ;
%    blocks(id).details.output.don=blocks(id).details.output.don(idx) ;
%  end ;  
%end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% avoid multiple loading of the big constant parameter file
global GLOBAL_cPAR ;
GLOBAL_cPAR=[] ;

cPAR=[] ;
cPAR.model=TRAIN_PAR.model ;
%cPAR.blocks_fname = ['~/' tempname '.mat'] ;
%cPAR.blocks = blocks ;
[xis, cPAR.parameters] = res2param(QP.res, num_examples, TRAIN_PAR.model, QP);

P=[] ;
P.cPAR_fname = sprintf('~/tmp/%i-%1.10f.mat', round(rand(1)*1e8), now) ;
assert(~fexist(P.cPAR_fname)) ;
save(P.cPAR_fname, 'cPAR') ;

%save_struct(cPAR.blocks_fname, blocks) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

P.viterbi_nbest = 1;

num = 0 ; model = TRAIN_PAR.model ;
jobinfo = rproc_empty(0) ;
fprintf(fid,'\n submitting %i viterbi jobs ...',length(blocks));
for block_id = 1:length(blocks),
  num = num+1 ;

  %idx = find(blocks(block_id).truth(1).segments(:,3)~=model.segments.intergenic) ;
  %max_len = max(500, max(blocks(block_id).truth(1).segments(idx,2)- ...
  %                       blocks(block_id).truth(1).segments(idx,1))) ;
  %max_len = max_len*1.2+1000 
  %P.max_len = max_len ;

  P.block_id = block_id ;
  P.block = blocks(block_id) ;
  P.fn_save = sprintf('%s_viterbi_block%i.mat', PAR.fn_pred, blocks(block_id).id); 
  P.prediction = 1 ;
  MEMREQ = PAR.RPROC.MEMREQ;
  options = PAR.RPROC.options;
  options.identifier = ['G'  num2str(block_id)] ;
  time_req = P.viterbi_nbest*length(blocks(block_id).genestr)*sqrt(22000)*0.03/60/100 ;
  options.addpaths={[pwd '/lsl_paper'], '~/svn/projects/genefinding/shogun'} ;
  if PAR.submit,
    jobinfo(num) = rproc_create('gen_paths', P, MEMREQ, options, time_req) ;
  else
    pred = gen_paths(P) ;
  end ;
end
if PAR.submit,
  [jobinfo, meta_jobinfo] = rproc_submit_batch(jobinfo, PAR.exm_per_batch) ;
  [meta_jobinfo,num_crashed] = rproc_wait(meta_jobinfo, 10);
end ;

unix(['rm ' P.cPAR_fname]) ;

blocks(1).pred = struct;
for block_id = 1:length(blocks),
  block = blocks(block_id) ;
  while ~fexist(sprintf('%s_viterbi_block%i.mat', PAR.fn_pred, block.id ))
    fprintf('.') ;
    pause(10)
  end
  load(sprintf('%s_viterbi_block%i.mat', PAR.fn_pred, block.id ),'pred');   
  blocks(block_id).prediction = pred; 
end

blocks=rmfield(blocks, 'seq') ;
save(PAR.fn_pred, 'blocks', 'PAR', 'P') ;

rproc_cleanup(jobinfo) ;
rproc_cleanup(meta_jobinfo) ;
