function jobinfo = predict_blocks(PAR, blocks, signal_genome_info, signal_masks)
% [blocks,PAR,P] = predict_blocks(PAR, blocks, signal_genome_info, signal_masks)
%

cleave_offset = PAR.method.par_ms.cleave_offset ;
fn_pred = PAR.FN.output_lsl.fn_pred;
RPROC = PAR.RPROC;

%% parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isfield(PAR, 'iteration')
  iteration = 1 ;
  while (1)
    fname = [PAR.fn_train '_iteration' num2str(iteration) '.mat'];
    if ~fexist(fname), iteration=iteration-1; break ; end 
    iteration=iteration+1 ;
  end ;
else
  iteration = PAR.iteration;
end ;
fname = [PAR.fn_train '_iteration' num2str(iteration) '.mat'];
fprintf('load iteration %i\n', iteration) ;
load(fname, 'QP', 'TRAIN_PAR', 'num_examples');
% keyboard
%% load data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin<2 || isempty(blocks)
  blocks=load_struct(PAR.fn_blocks, 'blocks') ;
  if isfield(PAR,'fn_splits')
    load(PAR.fn_splits, 'blocks_split') ; 
    block_idx = [blocks_split{PAR.pred_split_idx}] ;
    blocks = blocks(block_idx) ;
  end
end ;
fprintf('predicting on %i blocks\n', length(blocks)) ;

%% load signal masks %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin>=3 && (nargin<4 || isempty(signal_masks))
  signal_masks=load_struct(sprintf('%s/signal_masks.mat', signal_genome_info.basedir), 'signal_masks') ;
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% avoid multiple loading of the big constant parameter file
global GLOBAL_cPAR ;
GLOBAL_cPAR=[] ;

cPAR=[] ;
cPAR.model=TRAIN_PAR.model ;
[xis, cPAR.parameters] = res2param(QP.res, num_examples, TRAIN_PAR.model, QP);

fprintf('using cleave_offset=%1.2f\n', cleave_offset) ;
% cPAR.parameters.transitions(find(cPAR.model.a_trans(2,:)==cPAR.model.state_ids.cleave-1)) = cPAR.parameters.transitions(find(cPAR.model.a_trans(2,:)==cPAR.model.state_ids.cleave-1))+cleave_offset ;
cPAR.parameters.transitions(find(cPAR.model.a_trans(2,:)==cPAR.model.state_ids.tis(1)-1)) = ...
    cPAR.parameters.transitions(find(cPAR.model.a_trans(2,:)==cPAR.model.state_ids.tis(1)-1))+cleave_offset ;

P=[] ;
P.cPAR_fname = sprintf('~/tmp/sge/%i-%1.10f.mat', round(rand(1)*1e8), now) ;
assert(~fexist(P.cPAR_fname)) ;
save(P.cPAR_fname, 'cPAR') ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

P.viterbi_nbest = 1;

num = 0 ; model = TRAIN_PAR.model ;
jobinfo = rproc_empty(1) ;
for block_id = 1:length(blocks),
  fprintf('block %i\n',block_id) ;
  P.block_id = block_id ;
  P.block = blocks(block_id) ;

  % P.fn_save = sprintf('%s/blocks_pred/pred_split=_viterbi_block%i_cleave=%1.2f.mat', fn_pred, blocks(block_id).id, cleave_offset) ; 
  P.fn_save = sprintf('%s/blocks_pred/pred_block%i_cleave=%1.2f.mat', fn_pred, blocks(block_id).id, cleave_offset) ; 
  if  fexist(P.fn_save)
    continue
  end
  num = num+1 ;
  if nargin>=3 && ~isempty(signal_genome_info),
    block=P.block ;
    chr_num=find(ismember(signal_genome_info.contig_names, block.chr)) ;
    assert(length(chr_num)==1) ;
    block.chr_num=chr_num ;

    block=apply_signal_mask(block, signal_masks); 
    block.genestr(block.genestr=='N')='A'
    P.block=block ;
  end ;
  P.prediction = 1 ;
  options = RPROC.options;
  % options.waitonfull=1;
  options.identifier = ['LSL'  PAR.organism.name(3:5) num2str(blocks(block_id).id) '-'] ;
  MEMREQ = ceil(length(blocks(block_id).genestr)/1000000)*1700;
  % time_req = 10*24*3600 ;
  time_req = length(blocks(block_id).genestr) ;
  if ~PAR.tasks.run_locally
    which sg
    jobinfo(num) = rproc_create('predict_blocks_helper', P, MEMREQ, options, time_req) ;
  else
    addpath ~raetsch/svn/projects/genefinding/shogun
    predict_blocks_helper(P);
  end ;
end
if ~PAR.tasks.run_locally 
  [jobinfo, meta_jobinfo] = rproc_submit_batch(jobinfo, RPROC.exm_per_batch) ;
  % [meta_jobinfo,num_crashed] = rproc_wait(meta_jobinfo, 10);
end ;

% unix(['rm ' P.cPAR_fname]) ;

return

