function [model,lengths] = prepare(blocks, model, fn_content, fn_content_conserv)
% model = prepare(blocks, model, fn_content) 

INF = 1e20 ; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% boundaries for signal and content detectors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% signal detectors
  
boundaries = struct;
boundaries.signals = struct;
boundaries.contents = struct;
boundaries.lengths = struct;

sig_names = fieldnames(model.signals) ;
for s = 1:length(sig_names)
  range = getfield(model.signals_range, sig_names{s}) ;
  boundaries.signals = setfield(boundaries.signals, sig_names{s},[-INF linspace(range(1),range(2),model.bins-1) INF]);
end

%%% content detectors

content_names = fieldnames(model.contents) ;
for s = 1:length(content_names)
  range = getfield(model.contents_range, content_names{s}) ;
  boundaries.contents = setfield(boundaries.contents, content_names{s},[-INF linspace(range(1),range(2),model.bins-1) INF]);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% length distributions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

length_names = fieldnames(model.lengths) ;
lengths = [] ;
for s = 1:model.cnt_lengths
  lengths = setfield(lengths, length_names{s}, []) ;
end

segments_names = fieldnames(model.segments) ;
for id = 1:length(blocks),
  block = blocks(id) ;
  segments = block.truth(1).segments ;
  genes = block.truth(1).genes ;
  for s = 1:length(segments_names)
    s_id = getfield(model.segments, segments_names{s}) ;
    if s_id == model.segments.cds_exon,
      for j=1:length(genes),
        gene_segments = genes{j} ;
        idx = find(gene_segments(:,3)==s_id) ;
        if length(idx)==1,
          lengths.single_cds_exon = [lengths.single_cds_exon ;gene_segments(idx,2)-gene_segments(idx,1)];
        elseif length(idx)>1,
          lengths.first_cds_exon = [lengths.first_cds_exon ;gene_segments(idx(1),2)-gene_segments(idx(1),1)] ;
          lengths.last_cds_exon = [lengths.last_cds_exon ;gene_segments(idx(end),2)-gene_segments(idx(end),1)] ;
          lengths.middle_cds_exon = [lengths.middle_cds_exon ;gene_segments(idx(2:end-1),2)-gene_segments(idx(2:end-1),1)] ;
        else
          assert(0) ;
        end ;
      end ;
    else
      idx = find(segments(:,3)==s_id) ;
      list = getfield(lengths, segments_names{s}) ;
      list = [list; segments(idx,2)-segments(idx,1)] ;
      lengths = setfield(lengths, segments_names{s}, list) ;
    end ;
  end
end

for s = 1:length(length_names)
  range = getfield(model.lengths_range, length_names{s}) ;
  sout = getfield(lengths,length_names{s}) ;
  sout(sout<range(1)|sout>range(2)) = [];
  sout = sort(log(sout+1))' ;
  if length(sout)<100,
    fprintf('too few points: %s %i\n',length_names{s}, length(sout));  
    sout = log(logspace(log10(5),log10(10000),1000)+1) ; 
  end ;
  boundaries.lengths = setfield(boundaries.lengths,length_names{s}, [-INF sout(ceil([1/model.bins:1/model.bins:1-1/model.bins]*length(sout))) INF]) ;
end
model.boundaries=boundaries ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get content weights
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isempty(content_names)
  model.dict_weights = load_dict_weights(fn_content, content_names) ;
  if isfield(model, 'use_conservation') && model.use_conservation,
    model.dict_weights = load_dict_weights(fn_content_conserv, content_names, '_conserv', model.dict_weights) ;
  end ;
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mappings of PLIFS to parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

map_plif2param = init_weights(model);
map_plif2param = rmfield(map_plif2param,'transitions');
weights_name = fieldnames(map_plif2param);
start=0;
for w=1:length(weights_name)
  len=length(getfield(map_plif2param, weights_name{w}));
  model.plif2param{w} = start+(1:len);
  start = start + len;
end
model.transitions2param = start+1:model.cnt_parameters ;

assert(model.cnt_parameters-model.cnt_transitions==start)
