function [result] = regenerate_constraints(PP);
%   [blocks,QP,viterbi_nbest] = solve_qp(blocks,idx_temp,PAR,QP,res);
% 
%

addpath ~/svn/projects/genefinding/utils
addpath /fml/ag-raetsch/share/software/matlab_tools/shogun
% addpath ~raetsch/projects/shogun/src

% avoid multiple loading of the big constant parameter file
global GLOBAL_cPAR ;
if isempty(GLOBAL_cPAR),
  L = load(PP.cPAR_fname) ;
  GLOBAL_cPAR = L ;
else
  L = GLOBAL_cPAR ;
end ;

blocks = PP.blocks ; % L.cPAR.blocks(PP.block_idx) ;
block_idx = PP.block_idx ;
num_examples = L.cPAR.num_examples ;
PAR = L.cPAR.PAR ;
QP = L.cPAR.QP ;
old_model = L.cPAR.old_model ;
old_block = PP.old_block ;

max_neg  = PAR.max_neg ;
opt_step = PAR.opt_step;
model = PAR.model ;
margin_tol = 10 ;

clear A1 A1_sparse
A1 = zeros(opt_step, model.cnt_parameters) ;
A2 = sparse([],[],[], length(blocks)*max_neg, size(QP.A,2)-model.cnt_parameters, length(blocks)*max_neg) ; 
b = zeros(opt_step, 1) ;
q = 0 ; r = 0 ; q2 = 0 ;

[xis,parameters] = res2param(QP.res, num_examples, model, QP);
param_vec = weights2vector(parameters) ;
fprintf('\n') ;
for id = 1:length(blocks),
  block = blocks(id) ;
  fprintf('id=%i\n', id) ;
  pred = block.pred ;
  if isempty(pred), warning('empty'); continue; end ;
  
  loss=[] ;
  for num = 1:length(block.truth)
    [weights, loss(num)] = compute_weights(block.truth(num), block, model, parameters) ;    

    %if norm(weights2vector(block.truth(num).weights)-weights2vector(weights))>1e-6,
    %  fprintf('weights differ\n') ;
    block.truth(num).weights=weights ;
    %end ;
    if loss(num)~=0,
      warning('loss not zero')
      loss(num)
      %keyboard
    end 
  end ;

  num_identical = 0 ; take_idx=[] ; loss=[] ;
  for pred_num = 1:length(pred)
    old_pred = pred(pred_num) ;
    if isempty(old_pred.genes{1}), continue ; end ;

    % decode segments again with old model (fix merged genes)
    [pred(pred_num).segments, pred(pred_num).genes, ok] = path2segmentation(old_pred.path, old_pred.pos_idx, old_block, old_model) ;
    if ~ok, continue ; end ;

    % recode segments with new model
    [pred(pred_num).path, pred(pred_num).pos_idx, pred(pred_num).pos, ok]=segmentation2path(pred(pred_num), block, model);    
    if ~ok, continue ; end ;

    % sanity check: decode again and compare
    [segments, genes, ok] = path2segmentation(pred(pred_num).path, pred(pred_num).pos_idx, block, model) ;
    if ~ok, continue ; end ;
    if ~isequal(pred(pred_num).genes, genes),
      warning('decoded genes are not identical with input') ;
      ok=0 ;
      continue ;
    end ;

    [weights, loss(pred_num), tmp, tmp, tmp, ok] = compute_weights(pred(pred_num), block, model, parameters, false) ;    
    if ~ok, continue ; end ;
      
    %if norm(weights2vector(pred(pred_num).weights)-weights2vector(weights))>1e-6,
    %  fprintf('weights differ\n') ;
    %end ;
    pred(pred_num).weights=weights ;
    %if ~isnan(pred(pred_num).loss) & loss(pred_num) ~= pred(pred_num).loss
    %  fprintf('losses differ: %1.2f != %1.2f\n', loss(pred_num), pred(pred_num).loss) ;
    %  %keyboard
    %end 
    pred(pred_num).loss = loss(pred_num) ;

    identical = 0 ; 
    for t=1:length(block.truth)
      identical = equal(pred(pred_num).path, block.truth(t).path) & equal(pred(pred_num).pos, block.truth(t).pos) ;
      identical = identical | (loss(pred_num)==0) ;
      if identical, 
        if t>1,
          % exchange
          disp('exchange truths') ;
          tmp=block.truth(1);
          block.truth(1)=block.truth(t) ;
          block.truth(t)=tmp ;
        end;
        break ; 
      end ;
    end ;
    if identical, num_identical=num_identical+1 ; end ;

    vec = weights2vector(block.truth(1).weights) - weights2vector(pred(pred_num).weights);
    diff_before = -sum(param_vec.*vec);
    
    if identical,
      %assert(norm(vec)<1e-8) ;
      assert(~loss(pred_num)) ;
    else
      assert(norm(vec)>1e-8) ;
      assert(loss(pred_num)>0) ;
      
      if diff_before+loss(pred_num)-xis(block_idx(id)) < -margin_tol,
        fprintf('No margin violation: %f > %f = %f - %f\n', -diff_before, loss(pred_num) - xis(block_idx(id)), loss(pred_num), xis(block_idx(id))) ;
        continue
      end
      if abs(diff_before+loss(pred_num)-xis(block_idx(id))) < 1e-6,
        fprintf('Adding active constraint %f > %f = %f - %f \n', -diff_before, loss(pred_num) - xis(block_idx(id)), loss(pred_num), xis(block_idx(id))) ;
      elseif diff_before+loss(pred_num)-xis(block_idx(id)) < 1e-6,
        fprintf('Adding constraint %f > %f = %f - %f \n', -diff_before, loss(pred_num) - xis(block_idx(id)), loss(pred_num), xis(block_idx(id))) ;
      else
        fprintf('Adding violated constraint %f > %f = %f - %f \n', -diff_before, loss(pred_num) - xis(block_idx(id)), loss(pred_num), xis(block_idx(id))) ;
      end ;
      take_idx(end+1) = pred_num ;

      q  = q+1 ; 
      q2 = q2+1 ;
      A1(q2,1:model.cnt_parameters) = -vec;
      b(q2)= -loss(pred_num) ;
      A2(q, block_idx(id)) = -1 ; % slack
      
      if q2==size(A1,1)
        assert(0) ;

        keyboard
        fprintf('*\n') ;  
        A1 = sparse(A1(1:q2,:)) ;
        assert(q==size(A1,1));
        A2 = A2(1:q,:) ;
        A = [A1 A2] ;
        QP.A = [QP.A; A] ;
        QP.b = [QP.b; b] ;
        
        %tic;[res,lambda,how] = qp_solve(QP.lpenv, QP.Q, QP.f, QP.A, QP.b, QP.lb, QP.ub, 0, 1, PAR.solver);toc;
        %tic;[res,lambda,how] = qp_solve(QP.lpenv, QP.Q, QP.f, QP.A, QP.b, QP.lb, QP.ub, 0, 1, 'sift');toc;
        
        %OBJ = sum(res.*f) + 0.5*res'*QP.Q*res ;
        %[xis,parameters,hvar,H] = res2param(res,num_examples,model);
        %sum_xis = sum(xis) ;

        q2 = 0; q = 0 ;
        A1 = zeros(opt_step,model.cnt_parameters) ;
        A2 = sparse([],[],[],length(blocks_train)*max_neg, length(blocks_train)+model.cnt_lengths*(bins-1),...
                    length(blocks_train)*max_neg) ; 
        b = zeros(opt_step, 1) ;
      end ;
    end ;
  end ; %%% loop over predictions 
  num_identical
  block.pred = pred(take_idx) ;
  blocks(id) = block ;
end ;%%% loop over blocks

A1     = sparse(A1(1:q2,:)) ;
b      = b(1:q2) ;
A2     = A2(1:q,:) ;
A      = [A1 A2] ;
QP.A   = [QP.A;A] ;
QP.b   = [QP.b;b] ;


result.blocks = block;
result.QP = QP ;
