function [path,pos_idx,pos,ok]=segmentation2path(SEG, block, model,verb)
% [path,pos_idx,pos,ok]=segmentation2path(SEG, block, model)
  

%pos = [0     length(block.pos)-1] ;
%path = [model.state_ids.seq_start    model.state_ids.seq_end]-1 ;
  
if nargin<4
  verb=1;
end
  
genestr=lower(block.genestr) ;

path = [] ;
pos = [] ;
path(end+1) = model.state_ids.seq_start ;
pos(end+1) = block.pos(1) ;
ok=1 ;
any_invalid = 0 ;
gene_has_trans = 0;
gene_in_operon = 0;
intergenic_idx = find(SEG.segments(:,3)==model.segments.intergenic|...
                      SEG.segments(:,3)==model.segments.intergenictrans|...
                      SEG.segments(:,3)==model.segments.intercistronic);
assert(length(intergenic_idx)==length(SEG.genes)+1)
for g_idx=1:length(SEG.genes)
  segments = SEG.genes{g_idx} ; %%% hack for invalid acceptor or donor
  if (size(segments,1)==1 & isnan(segments(1,3)))
    path(end+1:end+2) = [model.state_ids.tss(1) model.state_ids.cdsStop(1)] ;
    pos(end+1:end+2) = segments(1:2)';
    if verb,warning('gene is invalid') ;end
    % keyboard
    any_invalid = 1 ;
    continue
  end
  if g_idx>1&segments(1,1)<SEG.genes{g_idx-1}(end,2)
    ok=0 ;
    if 1,warning('overlapping genes') ;end
  end
  if SEG.segments(intergenic_idx(g_idx),3)==model.segments.intergenic
    gene_has_trans = 0;
    gene_in_operon = 0;
  elseif SEG.segments(intergenic_idx(g_idx),3)==model.segments.intergenictrans
    gene_has_trans = 1;
    gene_in_operon = 0;
  elseif SEG.segments(intergenic_idx(g_idx),3)==model.segments.intercistronic
    gene_has_trans = 1;
    gene_in_operon = 1;
  end
  if block.id==170
    % keyboard
  end
  
  idx = find(segments(:,3)==model.segments.transexon) ;
  if ~isempty(idx) 
    assert(~gene_in_operon)
    gene_has_trans = 1;
    trans_segments = segments(idx,:) ;
    path(end+1) = model.state_ids.tss(1) ;
    pos(end+1) = trans_segments(1,1) ;
    if length(idx)>1
      if verb,warning('more than one transexon found!!');end
      for i=1:size(idx)-1
        path(end+1)= model.state_ids.don(1) ;
        pos(end+1) = trans_segments(i, 2) ;
        path(end+1)= model.state_ids.acc(1) ;
        pos(end+1) = trans_segments(i+1, 1) ;
      end ;
    end
    if segments(idx(end)+1,1)==segments(idx(end)+1,2)& ...
          segments(idx(end)+1,3) == model.segments.utr5exon,
      % this is the trans-splice agatg case
      assert(find(segments(:,3)==model.segments.utr5exon)==idx(end)+1)
      % path(end+1) = model.state_ids.trans(2) ;
      path(end+1) = model.state_ids.SL1(2) ;
      pos(end+1) = trans_segments(end,2) ;
    else
      % path(end+1) = model.state_ids.trans(1) ;
      path(end+1) = model.state_ids.SL1(1); 
      pos(end+1) = trans_segments(end,2) ;
    end
  end ;

  % consider 5' utr
  idx = find(segments(:,3)==model.segments.utr5exon) ;
  if ~isempty(idx),
    utr5_segments = segments(idx,:) ;
    if ~gene_has_trans
      assert(segments(idx(1),1)~= segments(idx(1),2))
      path(end+1) = model.state_ids.tss(1) ;
      pos(end+1) = utr5_segments(1,1) ;
    elseif gene_has_trans& ~gene_in_operon&segments(idx(1),1)~= segments(idx(1),2)&...
          path(end) ~= model.state_ids.SL1(1) ;
      path(end+1) = model.state_ids.SL1(1) ;
      pos(end+1) = utr5_segments(1,1) ;
    elseif gene_has_trans& ~gene_in_operon&segments(idx(1),1)== segments(idx(1),2)&...
          path(end) ~= model.state_ids.SL1(2) ;
      path(end+1) = model.state_ids.SL1(2) ;
      pos(end+1) = utr5_segments(1,1) ;
    elseif gene_has_trans& gene_in_operon&segments(idx(1),1)~= segments(idx(1),2)&...
          path(end) ~= model.state_ids.SL2(1) ;
      path(end+1) = model.state_ids.SL2(1) ;
      pos(end+1) = utr5_segments(1,1) ;
    elseif gene_has_trans& gene_in_operon&segments(idx(1),1)== segments(idx(1),2)
      path(end+1) = model.state_ids.SL2(2) ;
      pos(end+1) = utr5_segments(1,1) ;
    end
    for i=1:size(utr5_segments,1)-1
      path(end+1)= model.state_ids.don(1) ;
      pos(end+1) = utr5_segments(i, 2) ;
      if utr5_segments(i+1, 1)~=utr5_segments(i+1, 2)
        path(end+1)= model.state_ids.acc(1) ;
        pos(end+1) = utr5_segments(i+1, 1) ;
      else
        path(end+1)= model.state_ids.tis(2) ;
        pos(end+1) = utr5_segments(i+1, 1) ;
      end
    end ;
  end ;
 


  % consider coding region
  idx = find(segments(:,3)==model.segments.cds_exon) ;
  coding_segments = segments(idx,:) ;
  if idx(1)>1 && segments(idx(1)-1,3)==model.segments.intron
    % special case agatg
    path(end+1) = model.state_ids.don(1) ;
    pos(end+1) = segments(idx(1)-1,1) ;
    path(end+1) = model.state_ids.tis(2) ;
    pos(end+1) = segments(idx(1)-1,2) ;
    
  elseif path(end)~= model.state_ids.SL1(2)&...
        path(end)~= model.state_ids.SL2(2)&...
        path(end)~= model.state_ids.tis(2), % model.state_ids.trans(2),
    path(end+1) = model.state_ids.tis(1) ;
    pos(end+1) = coding_segments(1,1) ;
  end ;

  %spliced = '' ;
  %for i=1:size(coding_segments,1)
  %  spliced = [spliced genestr(coding_segments(i,1):coding_segments(i,2)-1)] ;
  %end ;
  %assert(mod(length(spliced),3)==0) ;
  %f=find_open_frames(spliced,1)

  frame = mod(coding_segments(1,2)-coding_segments(1,1),3) ;
  for i=1:size(coding_segments,1)-1
    pos(end+1) = coding_segments(i,2) ;
    pos(end+1) = coding_segments(i+1,1) ;
    if frame==0,
      path(end+1)= model.state_ids.don(1+1) ;
      path(end+1)= model.state_ids.acc(1+1) ;
    elseif frame==1,
      if genestr(coding_segments(i,2)-1)~='t'
        path(end+1)=model.state_ids.don(1+2) ;
        path(end+1)=model.state_ids.acc(1+2) ;
      else
        path(end+1)=model.state_ids.don(1+3) ;
        path(end+1)=model.state_ids.acc(1+3) ;
      end ;
    elseif frame==2,
      if genestr(coding_segments(i,2)-2)=='t' & genestr(coding_segments(i,2)-1)=='a'
        path(end+1)=model.state_ids.don(1+5) ;
        path(end+1)=model.state_ids.acc(1+5) ;
      elseif genestr(coding_segments(i,2)-2)=='t' & genestr(coding_segments(i,2)-1)=='g'
        path(end+1)=model.state_ids.don(1+6) ;
        path(end+1)=model.state_ids.acc(1+6) ;
      else
        path(end+1)=model.state_ids.don(1+4) ;
        path(end+1)=model.state_ids.acc(1+4) ;
      end ;
    end ;
    frame = mod(frame+coding_segments(i+1,2)-coding_segments(i+1,1),3) ;
  end ;
  
  % consider 3' utr
  utr3_segments = [] ;

  idx = find(segments(:,3)==model.segments.cds_exon) ;
  if segments(idx(end)+1,3)==model.segments.intron 
    % special case: agstop
    %keyboard
    path(end+1)= model.state_ids.don(2) ;
    pos(end+1) = segments(idx(end)+1,1) ;
    path(end+1)= model.state_ids.cdsStop(2) ;
    pos(end+1) = segments(idx(end)+1,2) ;
  else
    %keyboard
    % normal stop
    pos(end+1)=coding_segments(end,2) ;
    if frame==0,
      path(end+1)=model.state_ids.cdsStop(1) ;
    elseif frame==1,
      path(end+1)=model.state_ids.cdsStop(1) ;
    elseif frame==2,
      path(end+1)=model.state_ids.cdsStop(1) ;
    end ;
  end ;

  idx = find(segments(:,3)==model.segments.utr3exon) ;
  if ~isempty(idx),
    utr3_segments = segments(idx,:) ;
    for i = 1:size(utr3_segments,1)-1
      path(end+1)= model.state_ids.don(end) ;
      pos(end+1) = utr3_segments(i, 2) ;
      path(end+1)= model.state_ids.acc(end) ;
      pos(end+1) = utr3_segments(i+1, 1) ;
    end ;
  end ;

  % consider 3' utr
  idx = find(segments(:,3)==model.segments.polya_tail) ;
  if ~isempty(idx),
    assert(length(idx)==1) ;
    utr3_segments = segments(idx,:) ;
    path(end+1) = model.state_ids.polya(1) ;
    pos(end+1) = utr3_segments(1,1) ;
  end ;

 
  if SEG.segments(intergenic_idx(g_idx+1),3)==model.segments.intergenic
    path(end+1) = model.state_ids.cleave(1) ;
  elseif SEG.segments(intergenic_idx(g_idx+1),3)==model.segments.intergenictrans
    path(end+1) = model.state_ids.cleave(1) ;
  elseif SEG.segments(intergenic_idx(g_idx+1),3)==model.segments.intercistronic
    path(end+1) = model.state_ids.cleave(2) ;
  end
  pos(end+1) = utr3_segments(end,2) ;
  
end ;

path(end+1) = model.state_ids.seq_end ;
pos(end+1)  = block.pos(end) ;

if ~any_invalid,
  for i=1:length(pos)-1
    if ~(model.A(path(i),path(i+1))>=-1000),
      ok=0 ;
    end ;
  end ;
else
  ok=0;
end ;
% 

for i=1:size(model.intergenic_long.transitions, 1)
  idx = find([path(1:end-1)==model.intergenic_long.transitions(i,1) & ...
              path(2:end)==model.intergenic_long.transitions(i,2) & ...
              pos(2:end)-pos(1:end-1) >= model.intergenic_long.max_short_length], 1, 'first') ;
  
  while(~isempty(idx))
    excess = pos(idx+1) - pos(idx) - model.intergenic_long.max_short_length ;
    assert(all(excess>=0)) ;

    % fill in the minimum number of transitions necessary to generate the true path
    idx2 = find(block.pos>pos(idx)+model.intergenic_long.long_segment_parts & block.pos<pos(idx+1)-model.intergenic_long.long_segment_parts) ;
    num_steps = ceil((block.pos(idx2(end))-block.pos(idx2(1)))/model.intergenic_long.step)+1 ;
    assert(length(idx2)>=num_steps) ;
    %if block.pos(idx2(end))-block.pos(idx2(1)) > model.lengths_range.intron(2),
    %  warning(sprintf('intergenic gap (%i) is longer than maximal intron length (%i), true path might not be realizable',...
    %                  block.pos(idx2(end))-block.pos(idx2(1)), model.lengths_range.intron(2))) ;
    %end ;
    path = [path(1:idx) model.state_ids.intergenic_long*ones(1,num_steps) path(idx+1:end)] ;
    pos = [pos(1:idx) block.pos(idx2([round(linspace(1, length(idx2), num_steps))])) pos(idx+1:end)] ;

    idx = find([path(1:end-1)==model.intergenic_long.transitions(i,1) & ...
                path(2:end)==model.intergenic_long.transitions(i,2) & ...
                pos(2:end)-pos(1:end-1) >= model.intergenic_long.max_short_length], 1, 'first') ;
  end ;
  
end ;
assert(length(pos)==length(path)) ;
if ~isempty(setdiff(SEG.segments(2:end-1,1),pos)),
  warning('some position differences')
  setdiff(SEG.segments(2:end-1,1),pos)
  ok=0 ;
end 

[tmp,idx, pos_idx] = intersect(pos,block.pos) ;
if ~isempty(setdiff(pos, block.pos(pos_idx)))
  [tmp,idx1]=setdiff(pos, block.pos(pos_idx)) ;
  ok=0 ;
  if verb, warning('position(s) not found') ;end
  keyboard
end ;

if ~issorted(pos)
  pos_idx=[];
  for i=1:length(pos)
    pos_idx=[pos_idx find(block.pos==pos(i))];
  end
  ok=0 ;
  if verb, warning('positions not sorted!, maybe overlapping genes ');end
else
  assert(length(unique(pos))==length(pos));
  path = path(idx) ;
  pos = block.pos(pos_idx) ;
end
%%%%
