function QP = solve_qp(QP, PAR) ;
%   QP = solve_qp(QP, PAR) ;
%

global last_p_lp last_num_con
QP

if 1%equal(last_p_lp, -1) 

  % try to get a 8 cpu license
  lpenv = cplex_license(1, 2) ;
  %if lpenv==0,
  %  lpenv = cplex_license(0, 2) ;
  %end ;
  %if lpenv==0,
  %  lpenv = cplex_license(0, 1) ;
  %end ;

  for i=1:8,
    how = lp_set_param(lpenv,'CPX_PARAM_THREADS', i,1) ;
    how = lp_set_param(lpenv,'CPX_PARAM_BARTHREADS', i,1) ;
  end ;

  if nnz(QP.Q)==0
    disp('Solving linear program (no increments)') ;      
    how=lp_set_param(lpenv,'CPX_PARAM_BARCROSSALG', -1,1) ; % don't create lp basis after barrier solving
    how=lp_set_param(lpenv,'CPX_PARAM_PREDUAL', 1, 1) ; 
    tic;[res,lambda,how] = lp_solve(lpenv, QP.f, QP.A, QP.b, QP.lb, QP.ub, 0, 1, 'dual');toc;
    how
  else
    disp('Solving quadratic program (no increments)') ;   
    disp('preddual on') ;
    how=lp_set_param(lpenv,'CPX_PARAM_PREDUAL', 1, 1) ; 
    how=lp_set_param(lpenv,'CPX_PARAM_WORKMEM', 10000, 1) ; 
    how=lp_set_param(lpenv,'CPX_PARAM_BAREPCOMP', 1e-4, 1) ; 
    how=lp_set_param(lpenv,'CPX_PARAM_EPRHS', 1e-4, 1) ; 
    %how=lp_set_param(lpenv,'CPX_PARAM_AGGFILL', 100, 1) ; 
    %how=lp_set_param(lpenv,'CPX_PARAM_AGGCUTLIM', 100, 1) ; 

    how=lp_set_param(lpenv,'CPX_PARAM_BARCROSSALG', -1,1) ; % don't create lp basis after barrier solving
    
    idx = 1:size(QP.A,1) ;
    if 1,
      Diff = (QP.A*QP.res-QP.b) ;
      for p=95:-5:10
        ttt=prctile(Diff,p) ;
        if prctile(Diff,p)<-20, % some heuristic to make sure the gap is large enough
          idx = find(Diff>prctile(Diff,p)) ;
          fprintf('removing %2.0f%% of all constraints (d=%1.2f):\nreduced problem has %i constraints (originally %i)\n', p, prctile(Diff,p), length(idx), size(QP.A,1)) ;
          break ;
        end ;
      end ;
    end ;
    %save ~/tmp/QP QP
    QP.con_idx = idx ;
    tic; [res,lambda,how] = qp_solve(lpenv, QP.Q, QP.f, QP.A(idx,:), QP.b(idx), QP.lb, QP.ub, 0, 1, 'bar'); toc;
    %save ~/tmp/QP QP res lambda how
  end ;
  QP.res = res ;
    
  how = lp_set_param(lpenv,'CPX_PARAM_THREADS', 1,1) ;
  how = lp_set_param(lpenv,'CPX_PARAM_BARTHREADS', 1,1) ;
  [lpenv,status]=cplex_init_quit(1, '', lpenv) ;
  %assert(all(QP.A*res-QP.b<=1e-5)) ;

  return 
end ;

if isempty(last_p_lp) 
  how=lp_set_param(lpenv,'CPX_PARAM_BARCROSSALG', 0,1) ; % create lp basis after barrier solving
  if nnz(QP.Q)==0
    disp('Solving linear program') ;      
    how=lp_set_param(lpenv,'CPX_PARAM_PREDUAL', 1, 1) ; 
    tic;[res,lambda,how,last_p_lp] = lp_solve(lpenv, QP.f, QP.A, QP.b, QP.lb, QP.ub, 0, 1, 'bar');toc;
    %tic;[res,lambda,how]=lp_resolve(lpenv, last_p_lp, 1, 'primal');toc;
  else
    disp('Solving quadratic program') ;      
    tic;[res,lambda,how,last_p_lp] = qp_solve(lpenv, QP.Q, QP.f, QP.A, QP.b, QP.lb, QP.ub, 0, 1, 'con');toc;
    %tic;[res,lambda,how]=lp_resolve(lpenv, last_p_lp, 1, 'qp-con');toc;
  end ;
  last_num_con = size(QP.A,1) ;
else
  if (last_num_con<size(QP.A,1))
    fprintf('adding %i constraints\n', size(QP.A,1)-last_num_con) ;
    how = lp_addrows(lpenv, last_p_lp, QP.A(last_num_con+1:end,:)', QP.b(last_num_con+1:end), 0, 1) ;
  end ;
  OBJ = sum(QP.res.*QP.f)+0.5*QP.res'*QP.Q*QP.res ;
  %how=lp_set_param(lpenv,'CPX_PARAM_NETEPRHS', 1e-4, 1)  % allow some deviations
  %how=lp_set_param(lpenv,'CPX_PARAM_NETEPOPT', min(1e-2, OBJ*1e-6), 1)  % scale accuracy with objective 
  if nnz(QP.Q)==0
    disp('Solving linear program (incremental)') ;      
    how=lp_set_param(lpenv,'CPX_PARAM_PREDUAL', 1, 1) ; 
    tic;[res,lambda,how]=lp_resolve(lpenv, last_p_lp, 1, 'bar');toc;
  else
    disp('Solving quadratic program (incremental)') ;      
    %keyboard
    tic;[res,lambda,how]=lp_resolve(lpenv, last_p_lp, 1, 'bar');toc;
  end ;
  last_num_con = size(QP.A,1) ;
end

QP.res = res ;
%% check if solution is feasible
assert(all(QP.A*res-QP.b<=1e-10)) ;
