%addpath /fml/ag-raetsch/share/software/matlab_tools/shogun
lpenv=cplex_license(0,8) ;
  
addpath ~/svn/projects/genefinding/lsl
addpath ~/svn/projects/genefinding/utils
addpath ~/svn/projects/genefinding/eval

tester_config_pacificus

unix(sprintf('mkdir -p %s', PAR.dir)) ;
unix(sprintf('mkdir -p %s/log_files', PAR.dir)) ;
unix(sprintf('mkdir -p %s/data', PAR.dir)) ;
unix(sprintf('mkdir -p %s/train', PAR.dir)) ;
unix(sprintf('mkdir -p %s/predictions', PAR.dir)) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%% generate blocks
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PAR.min_dist = 1000;
PAR.num_splits = 5 ; 

if PAR.new_signals
  L=load(PAR.fn_polya_signals,'consensus') ;
  PAR.polya_sixmers = L.consensus ;
else
  load(PAR.fn_polya_signals,'signal_sixmers') ;
  PAR.polya_sixmers = {signal_sixmers.sixmer} ;
end ;

if isfield(PAR, 'use_conservation') && PAR.use_conservation
  PAR.model = make_model_conserv(PAR);
else
  PAR.model = make_model(PAR);
end ;

% some introns seem to be shorter
%PAR.model.lengths_range.intron      = [30, 21000] ; %%%20, from don => acc+stop

PAR.use_annotation = 1;

if ~fexist(PAR.fn_blocks),
  load(PAR.fn_blocks_pre) ;
  
  blocks_new = [] ;
  for i=1:length(blocks)
    i
    idx=find(blocks(i).truth.segments(:,3)==PAR.model.segments.intron) ;
    if ~all(blocks(i).truth.segments(idx,2)-blocks(i).truth.segments(idx,1)>30),
      fprintf('block with intron of length %i skipped\n', min(blocks(i).truth.segments(idx,2)-blocks(i).truth.segments(idx,1))) ;
      continue ;
    end ;
    blocks(i).gene_confirmed=zeros(1,length(blocks(i).truth.genes)) ;
    blocks(i).content.subset_id=1;
    blocks(i).genestr = upper(blocks(i).genestr) ;
    idx = find(blocks(i).genestr~='A' & blocks(i).genestr~='C' & ...
               blocks(i).genestr~='G' & blocks(i).genestr~='T') ;
    blocks(i).genestr(idx) = 'A' ;
    block = blocks(i) ;
    block.id = i ;
    [block.seq, block.pos] = gen_seq(blocks(i), PAR.model);
    
    [block.truth.path,block.truth.pos_idx,block.truth.pos,ok] = segmentation2path(block.truth, block, PAR.model);
    if ok==0, continue ; end ;
    
    if isempty(blocks_new), blocks_new = block; else blocks_new(end+1) = block ; end ;
  end ;
  keyboard

  blocks=blocks_new ;
  save(PAR.fn_blocks, 'blocks') ;
end ;

if ~fexist(PAR.fn_splits),
  load(PAR.fn_blocks) ;
  rand('seed', 12324) ;
  good_idx = 1:length(blocks) ;
  idx = randperm(length(good_idx));
  split_length = floor(length(good_idx)/PAR.num_splits);
  for i=1:PAR.num_splits
    if i~=PAR.num_splits
      blocks_split{i} = good_idx(idx((i-1)*split_length+1:(i)*split_length)) ; 
    else
      blocks_split{i} = good_idx(idx((i-1)*split_length+1:end)) 
    end
  end
  save(PAR.fn_splits,'blocks_split')
  clear blocks ;
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%% modelselection on path
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

partition.sets = [1,1,1,2,3
                  1,1,1,1,2];
                  
partition.info.train = 1;
partition.info.eval = 2;
partition.info.pred = 3;

%%% train models

unix(sprintf('mkdir -p %s/result', PAR.dir)) ;

PAR.opt_step = 1000 ;
PAR.max_neg = 2 ;
PAR.margin = 1 ;
PAR.RPROC.MEMREQ = 1000;
PAR.RPROC.options = [];
PAR.RPROC.options.arch = 64 ;
PAR.RPROC.options.express = 0 ;

PAR.INF = 1e20;

PAR.max_num_iterations = 100;

PAR.fn_init =  sprintf('%strain/%s-%s/init.mat',PAR.dir,PAR.user,PAR.dt);
PAR.fn_train = sprintf('%strain/%s-%s/train',PAR.dir,PAR.user,PAR.dt);
PAR.fn_pred = sprintf('%spredictions/%s-%s/pred_split=%i',PAR.dir,PAR.user,PAR.dt,PAR.pred_split_idx);
PAR.fn_log = sprintf('%slog_files/%s_%s_log_',PAR.dir,PAR.user,PAR.dt);

unix(sprintf('mkdir -p %strain/%s-%s', PAR.dir,PAR.user,PAR.dt)) ;
unix(sprintf('mkdir -p %spredictions/%s-%s', PAR.dir,PAR.user,PAR.dt)) ;

%train_path(PAR) ;

return

pred_path(PAR) ;

%PAR.pred_split_idx = 1:4 ;
PAR.fn_pred = sprintf('%spredictions/%s_%s_pred%i:%i.mat',PAR.dir,PAR.user,PAR.dt,PAR.pred_split_idx(1),PAR.pred_split_idx(end));
pred_path(PAR) ;

% test error
addpath eval
tmp = load(PAR.fn_pred);
test_blocks = tmp.blocks;
evaluate_block_predictions(test_blocks, 2, 'confirmed');
evaluate_block_predictions(test_blocks, 2);


id=1 ;
draw_pred(test_blocks(id).truth, test_blocks(id).pred, PAR.model) 


PAR.pred_split_idx=5 ;
PAR.fn_pred = sprintf('%spredictions/%s_%s_pred_cheat%i:%i.mat',PAR.dir,PAR.user,PAR.dt,PAR.pred_split_idx(1),PAR.pred_split_idx(end));
pred_path_cheat(PAR) ;


PAR.cleave_offset = 3.5 ;
pred_path_cleave(PAR) ;
L=load([PAR.fn_pred sprintf('_cleave%1.2f.mat', PAR.cleave_offset)], 'blocks', 'PAR', 'P') ;
evaluate_block_predictions(L.blocks);



cleave_offsets=-2.5:0.5:5%[-3:0.5:3] ;
for cleave_offset=cleave_offsets
  PAR.cleave_offset = cleave_offset ;
  pred_path_cleave(PAR) ;
end ;

%cleave_offsets=0:0.5:5%[0:10] ;
for cleave_offset=cleave_offsets
  PAR.cleave_offset = cleave_offset ;
  cleave_offset
  L=load([PAR.fn_pred sprintf('_cleave%1.2f.mat', PAR.cleave_offset)], 'blocks', 'PAR', 'P') ;
  evaluate_block_predictions(L.blocks, 2, 'confirmed');
end ;

