function ok = check_file_consistency(PAR)
%  check_final_files(PAR)

fn_SVMs = PAR.FN.output_sig.(PAR.Signal_name).fn_SVMs;
fn_pred = PAR.FN.output_sig.(PAR.Signal_name).fn_SVMs;

ok = 1;

fprintf('start checking file consistency')

for p = 1:size(PAR.SETs.partitions,1) %% loop over partitions  
  fprintf('\n\n-----------\n');
  fprintf('PARTITION %i\n',p);
  fprintf('-----------\n');
  fprintf('\nSITE: %s\n',PAR.Signal_name);
  
  % load best SVM
  fn_best = sprintf('%sbest_partition=%i.mat', fn_SVMs,p) ;
  fprintf('loading best model from file %s\n',fn_best);
  L1 = load(fn_best) ;
  assert(isfield(L1,'prob'))
  assert(isfield(L1,'prob_cum'))
  assert(isfield(L1,'limits'))
  % load original training file  
  fprintf('loading original training file %s\n',L1.Train.PAR.fn_SVMs);
  L2 = load(L1.Train.PAR.fn_SVMs);
  
  % compare 
  fprintf('compare Train\n');
  
  diff = compare_PAR(L1.Train.PAR.SETs,L2.Train.PAR.SETs,1) ;
  assert(all(ismember(diff,{'which_val','which_pred','which_plif'})))
  diff = compare_PAR(L1.Train.PAR,L2.Train.PAR,1) ;
  assert(all(ismember(diff,{'RPROC','SETs','partition','eval_on','fn_SVM_eval','fn_plifs','fn_pred','tasks'})))
  diff = compare_PAR(L1.Train,L2.Train,1) ;
  assert(all(ismember(diff,{'PAR'})))
  
  %load prediction on test set
  fn_test = sprintf('%spartition=%i.mat', fn_pred,p) ;
  fprintf('loading prediction on test set\n');
  L3 = load(fn_test);
  assert(isfield(L3,'Conf'))
  assert(isfield(L3,'Conf_cum'))

  %load prediction on plifs
  fn_plif = sprintf('%spartition=%i_plifs.mat', fn_pred,p) ;
  fprintf('loading prediction on plif set\n');
  L4 =  load(fn_plif);
    
  assert(~ismember(L3.PAR.SETs.which_val,L3.PAR.SETs.which_train))
  assert(~ismember(L3.PAR.SETs.which_pred,[L3.PAR.SETs.which_val ...
                      L3.PAR.SETs.which_train]))
  fprintf('compare PAR from best test and plif\n');
  diff = compare_PAR(L3.PAR,L4.PAR,1) ; 
  assert(all(ismember(diff,{'plif_on'})))
  assert(isequal(L3.svmFileName,L4.svmFileName))
  assert(isequal(L3.sg_ver,L4.sg_ver))
  
  % load original evaluation file
  fprintf('loading original evaluation file %s\n',L4.PAR.fn_SVM_eval);
  L5 = load(L4.PAR.fn_SVM_eval);
  diff = compare_PAR(L4.PAR.FN,L5.PAR.FN);
  %   assert(isempty(diff))
  assert(all(ismember(diff,{'svn_info'    'output_cont'})))
  diff = compare_PAR(L4.PAR.Signals.(L4.PAR.Signal_name), L5.PAR.Signals.(L4.PAR.Signal_name));
  assert(isempty(diff))
  diff = compare_PAR(L4.PAR.method,L5.PAR.method);
  assert(isempty(diff))
  diff = compare_PAR(L4.PAR.SETs,L5.PAR.SETs,1);
  assert(all(ismember(diff,{'which_plif'})))
  
  if L4.PAR.SETs.which_plif==L4.PAR.SETs.which_val
    diff = compare_PAR(L4,L5,1); 
    assert(all(ismember(diff,{'Conf','Conf_cum','PAR'})))
  end  
  
  fprintf('compare PAR from best SVM and test\n');
  L1.Train.PAR.SETs = rmfield(L1.Train.PAR.SETs,'which_pred');
  assert(isequal(L1.Train.PAR.fn_SVMs,L3.svmFileName))
  diff = compare_PAR(L1.Train.PAR.SETs,L3.PAR.SETs,1) ;
  assert(all(ismember(diff,{'which_plif','which_pred','which_val'})))
  diff = compare_PAR(L1.Train.PAR,L3.PAR,1) ;
  assert(all(ismember(diff,{'SETs','partition','eval_on','fn_plifs','fn_pred','fn_SVM_eval'})))

      
end