function dataset_size_exp(PAR)


%collect model selection parameters
if isfield(PAR.method, 'kernels') & ~isempty(PAR.method.kernels{1})
  PAR.method.par_ms = collect_ms_parameters(PAR, PAR.method.par_ms);
end
[models,par_names] = generate_all_models(PAR.method.par_ms);
%select first model
PAR = assign_model(PAR, models, par_names,1);


sig = PAR.Signal_name;
method = ['_' PAR.method.name];
if isfield(PAR.method, 'kernels')&& isfield(PAR.method.kernels{1}, 'name')
  kern1 = ['_' PAR.method.kernels{1}.name];
else
  kern1 = '';
end

fn_examples = [PAR.FN.input_dir,PAR.organism.name,'/dataset-size_exp/',sig,method,kern1,'/examples'];
fn_SVMs     = [PAR.FN.input_dir,PAR.organism.name,'/dataset-size_exp/',sig,method,kern1,'/SVMs']; 
fn_pred     = [PAR.FN.input_dir,PAR.organism.name,'/dataset-size_exp/',sig,method,kern1,'/pred']; 
fn_res      = [PAR.FN.input_dir,PAR.organism.name,'/dataset-size_exp/',sig,method,kern1,'/results.mat']; 
fn_time     = [PAR.FN.input_dir,PAR.organism.name,'/dataset-size_exp/',sig,method,kern1,'/time.eps']; 
fn_rfv      = [PAR.FN.input_dir,PAR.organism.name,'/dataset-size_exp/',sig,method,kern1,'/rfv.eps']; 
fn_rsv      = [PAR.FN.input_dir,PAR.organism.name,'/dataset-size_exp/',sig,method,kern1,'/rsv.eps']; 
unix(['mkdir ',PAR.FN.input_dir,PAR.organism.name,'/dataset-size_exp/']);
unix(['mkdir ',PAR.FN.input_dir,PAR.organism.name,'/dataset-size_exp/',sig,method,kern1]);


sep = floor(logspace(log10(1000),log10(8500000),10));

fn_ex=sprintf('%s_%i.mat',fn_examples, length(sep));
if ~fexist(fn_ex)
  Data=load_data(PAR,[1 2 3 4],'train', PAR.SETs.train_subsample_neg,PAR.SETs.train_subsample_pos);
  idxp = randperm(size(Data.XT,2)) ;
end


%save examplefiles
for j=1:length(sep)
  j
  fn_ex=sprintf('%s_%i.mat',fn_examples, j);
  if ~fexist(fn_ex)
    XT = Data.XT(:,idxp(1:sep(j)));
    LT = Data.LT(idxp(1:sep(j)));
    Pos.region_id = Data.Pos.region_id(idxp(1:sep(j)));
    Pos.pos = Data.Pos.pos(idxp(1:sep(j)));
    save_matrix(fn_ex, XT, 'XT');
    save(fn_ex,'-append','LT','Pos');
    clear XT LT Pos
  end
end
clear Data

%train SVMs
q=1;
for j=1:0%length(sep)
  j
  %output-dir for tssTrain
  PAR.fn_SVMs = sprintf('%s_%i.mat',fn_SVMs, j);
  if ~fexist(PAR.fn_SVMs)
    fn_ex=sprintf('%s_%i.mat',fn_examples, j);
    D=load(fn_ex,'LT','Pos');
    D.XT = load_matrix(fn_ex,'XT');
    D.PAR = PAR;
    %feval(PAR.method.train_fct,D);
    clear sg
    PAR.RPROC.options.ncpus = 1 ;
    if sep(j)>2000000, 
      %PAR.RPROC.options.ncpus = 12; 
      feval(PAR.method.train_fct,D);
    else
      jobinfo(q) = rproc(PAR.method.train_fct, D , 5000+15000*sep(j)/sep(end), PAR.RPROC.options,PAR.RPROC.time_req);
    end ;
    q=q+1;
    clear D
  end
end
if exist('jobinfo')
  [jobinfo ncrashed]=rproc_wait(jobinfo, 10, 1, 0)
  if ncrashed>0
    return
  end
end


%predict on testset
Data_pred=load_data(PAR,[5],'test', PAR.SETs.test_subsample_neg,PAR.SETs.test_subsample_pos);
q=1;
jobinfo_pred = rproc_empty(0) ;
for j=1:6%length(sep) 
  fn_pred_tmp = sprintf('%s_%i.mat',fn_pred, j);
  if ~fexist(fn_pred_tmp)
    D=[] ;
    D.svmFileName = sprintf('%s_%i.mat', fn_SVMs, j);
    load(D.svmFileName, 'Train');
    D.Train=Train;
    D.outFileName = fn_pred_tmp;
    D.Data_PAR = PAR ;
    D.Data = Data_pred;
    %feval(PAR.method.predict_fct,D)
    jobinfo_pred(q) = rproc(PAR.method.predict_fct, D , PAR.RPROC.MEMREQ, PAR.RPROC.options, PAR.RPROC.time_req);
    q=q+1;
  end ;
end

if exist('jobinfo_pred') 
  rproc_wait(jobinfo_pred, 30, 1, 0)
end

% collect results
RSV=[]; RFV=[] ; TrainTime = [] ;
for j=1:length(sep)
  fn_pred_tmp = sprintf('%s_%i.mat',fn_pred, j);
  L = load(fn_pred_tmp, 'RSV','RFV') ;
  RSV(j)=L.RSV;
  RFV(j)=L.RFV;
  svmFileName = sprintf('%s_%i.mat', fn_SVMs, j);
  load(svmFileName, 'Train');
  TrainTime(j) = Train.time ;
end

save(fn_res, 'RSV','RFV','sep', 'TrainTime')  

close all
loglog(sep, TrainTime, '-o')
xlabel('Number of training examples') ;
ylabel('CPU time in seconds required for training') ;
print('-depsc2', fn_time) ;

close all
semilogx(sep, RFV, '-o')
xlabel('Number of training examples') ;
ylabel('Area under the precision recall curve') ;
print('-depsc2', fn_rfv) ;

close all
semilogx(sep, RSV, '-o')
xlabel('Number of training examples') ;
ylabel('Area under the receiver operator curve') ;
print('-depsc2', fn_rsv) ;
keyboard

return
