function idx = filter_cands( region, PAR, mode,verb );
% idx = filter_cands( region, PAR, mode,verb );

  


if nargin<4
  verb=1;
end
if nargin>2
  assert(strcmp(mode,'train')|strcmp(mode,'eval')|strcmp(mode,'test'))
else
  mode='train';
end    

Signal = getfield(PAR.Signals, PAR.Signal_name);
FILTER = Signal.filter_label.(mode);
off = PAR.regions.offset;
sig_field = getfield(region.Signals, PAR.Signal_name);

if isfield(FILTER,'use_label')
  sig_field.label = region.Signals.(PAR.Signal_name).(FILTER.use_label);
end
sig_field.label2 = region.Signals.(PAR.Signal_name).label;  

if verb
  fprintf('\nnb of positives: %i/%i\n', sum(sig_field.label==1),length(sig_field.label));
  fprintf('nb of negatives: %i/%i\n', sum(sig_field.label==-1),length(sig_field.label));
end


% don't use positions in overlap of regions

idx = find(sig_field.pos>region.reg_no_overlap(1)&sig_field.pos<region.reg_no_overlap(2));
% idx = find(sig_field.pos>off&sig_field.pos<length(region.seq)-off);
if verb
  fprintf('discarding positions in overlab of region\n');
  fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
end

% no intergenic
if FILTER.use_intergenic==0  
  intergenic = find(sig_field.gene_id==0&sig_field.label==-1);
  idx = setdiff(idx, intergenic);
  if verb
    fprintf('discarding negative intergenic positions\n');
    fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
  end
end

%no alternative genes
if FILTER.use_alt==0
  alt = find(sig_field.altgenic==1);
  idx = setdiff(idx, alt);
  if verb
    fprintf('discarding alternative genes/regions(only for splice sites)\n');
    fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
  end
end

%no alternative genes for negative generation
if isfield(FILTER,'use_alt_neg') && FILTER.use_alt_neg==0
  alt = find(sig_field.altgenic==1&sig_field.label==-1);
  idx = setdiff(idx, alt);
  if verb
    fprintf('discarding negatives in alternative genes/regions(only for splice sites)\n');
    fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
  end
end

%don't use decoys
if isfield(FILTER,'use_decoys') && FILTER.use_decoys==0
  decoys = find(sig_field.label2==-1);
  idx = setdiff(idx, decoys);
  if verb
    fprintf('discarding decoys (only for aletrnative splice sites)\n');
    fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
  end
end

% keyboard
% don't use positive positions with low confidence
if isfield(FILTER, 'conf_pos') && ~isempty(FILTER.conf_pos)
  conf_names = fieldnames(FILTER.conf_pos);
else
  conf_names = {};
end
thresholds = struct ;
for i=1:length(conf_names)
  thresholds.([conf_names{i} '_conf']) = FILTER.conf_pos.(conf_names{i});
  conf_names{i} = [conf_names{i} '_conf'];
end

if isfield(FILTER,'any_conf_pos');
  conf_names{end+1} = 'any_conf';
  thresholds.any_conf = FILTER.any_conf_pos;
end
high_confs = filter_confs(sig_field,conf_names,thresholds,verb);

negs = find(sig_field.label==-1);
idx = intersect(idx, [high_confs; negs]);

if verb
  f = fieldnames(thresholds);
  fprintf(1,'thresholds:');
  for j=1:length(f), fprintf(1,' %s ',f{j}); end
  fprintf('\nnb of remaining negatives: %i (%i)\n', sum(sig_field.label(idx)==-1),length(idx));
  fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
end

% don't use positives with low skip confidence
if isfield(FILTER,'conf_pos_skip') && ~isempty(FILTER.conf_pos_skip),
  conf_names = fieldnames(FILTER.conf_pos_skip);
else
  conf_names={};
end
for i=1:length(conf_names)
  thresholds.([conf_names{i} '_conf_skip']) = FILTER.conf_pos_skip.(conf_names{i});
  conf_names{i} = [conf_names{i} '_conf_skip'];
end

if isfield(FILTER,'any_conf_pos_skip');
  conf_names{end+1} = 'any_conf_skip';
  thresshold.any_conf_skip = FILTER.any_conf_pos_skip;
end
high_confs = filter_confs(sig_field,conf_names,thresholds,verb);

negs = find(sig_field.label==-1);
idx = intersect(idx, [high_confs; negs]);
if verb
  f = fieldnames(thresholds);
  fprintf(1,'thresholds:');
  for j=1:length(f), fprintf(1,' %s ',f{j}); end
  fprintf('\nnb of remaining negatives: %i (%i)\n', sum(sig_field.label(idx)==-1),length(idx));
  fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
end

% don't use negative positions (true splicesites) with low confidence
if isfield(FILTER,'conf_neg') && ~isempty(FILTER.conf_neg)
  conf_names = fieldnames(FILTER.conf_neg);
else
  conf_names = {};
end
for i=1:length(conf_names)
  thresholds.([conf_names{i} '_conf']) = FILTER.conf_neg.(conf_names{i});
  conf_names{i} = [conf_names{i} '_conf'];
end
if isfield(FILTER,'any_conf_neg');
  thresholds.any_conf = FILTER.any_conf_neg;
  conf_names{end+1} = 'any_conf';
end
high_confs = filter_confs(sig_field,conf_names,thresholds,verb) ; 

% keyboard
pos = find(sig_field.label==1);
decoys = find(sig_field.label==-1&sig_field.label2==-1);

if verb
  f = fieldnames(thresholds);
  fprintf(1,'thresholds:');
  for j=1:length(f), fprintf(1,' %s ',f{j}); end
  fprintf('\nnb of remaining negatives: %i (%i)\n', sum(sig_field.label(idx)==-1),length(idx));
  fprintf('nb of remaining negative splicesites: %i (%i)\n', sum(sig_field.label(idx)==-1&sig_field.label2(idx)==1),length(idx));
end
idx = intersect(idx, [high_confs; pos; decoys]);
if verb
  fprintf('nb of remaining negative splicesites: %i (%i)\n', sum(sig_field.label(idx)==-1&sig_field.label2(idx)==1),length(idx));
end

% don't use negatives (decoys) with low skip confidence
if isfield(FILTER, 'conf_neg_skip') && ~isempty(FILTER.conf_neg_skip)
  conf_names = fieldnames(FILTER.conf_neg_skip);
else
  conf_names={};
end
for i=1:length(conf_names)
  thresholds.([conf_names{i} '_conf_skip']) = FILTER.conf_neg_skip.(conf_names{i});
  conf_names{i} = [conf_names{i} '_conf_skip'];
end
if isfield(FILTER,'any_conf_neg_skip');
  thresholds.any_conf_skip = FILTER.any_conf_neg_skip;
  conf_names{end+1} = 'any_conf_skip';
end
high_confs = filter_confs(sig_field,conf_names,thresholds,verb);

pos = find(sig_field.label==1);
splice = find(sig_field.label==-1&sig_field.label2==1);
if verb
  f = fieldnames(thresholds);
  fprintf(1,'thresholds:');
  for j=1:length(f), fprintf(1,' %s ',f{j}); end
  % fprintf('\nnb of remaining decoys: %i (%i)\n', sum(sig_field.label(idx)==-1&sig_field.label2(idx)==-1),length(idx));
  fprintf('nb of remaining decoys: %i (%i)\n', sum(sig_field.label(idx)==-1&sig_field.label2(idx)==-1),length(idx));
end
idx = intersect(idx, [high_confs; pos; splice]);
if verb
  fprintf('nb of remaining decoys: %i (%i)\n', sum(sig_field.label(idx)==-1&sig_field.label2(idx)==-1),length(idx));
end



% use negatives only in genes where positives signal found 
if isfield(FILTER,'onlywithpos') & FILTER.onlywithpos==1  
  withpos = find(sig_field.(['has_' PAR.Signal_name])~=0);
  intergenic = find(sig_field.gene_id==0);
  idx = intersect(idx, [withpos intergenic]);
  if verb
    fprintf('negatives generated from genes with at least one true signal\n');
    fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
  end
end

% don't use genes with transsplice site  
if isfield(FILTER,'rm_trans')&  FILTER.rm_trans==1
  trans = find(sig_field.has_trans);
  idx = setdiff(idx, trans);
  if verb
    fprintf('discarding genes with transsplice\n');
    fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
  end
end

% negatives are only true acceptor sites %%%FOR TRANSSPLICE SITES
if  isfield(FILTER,'only_acc') & FILTER.only_acc==1
  nonacc = find(sig_field.isacc~=1&sig_field.label==-1);
  idx = setdiff(idx, nonacc);
  if verb
    fprintf('negatives generated only from true acceptor sites\n');
    fprintf('nb of remaining positives: %i (%i) \n', sum(sig_field.label(idx)==1),length(idx));
  end
end

% don't use positive positions which are far away from gene start
if isfield(FILTER,'pos_dist') 
  far_pos = find(sig_field.dist>FILTER.pos_dist&sig_field.label==1);
  idx = setdiff(idx, far_pos);
  if verb
    fprintf('positives are only used if they are within %i bp from gene start\n',FILTER.pos_dist);
    fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
  end
end

% don't use predicted positions 
if isfield(FILTER,'use_pred') & FILTER.use_pred==0
  only_pred = find(sig_field.anno_SL1_conf==0&sig_field.anno_SL2_conf==0&sig_field.pred_conf~=0);
  idx = setdiff(idx, only_pred);
  if verb
    fprintf('positive exampls derived only from predictions are ignored \n');
    fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
  end
end


if verb
  fprintf('nb of remaining positives: %i (%i)\n', sum(sig_field.label(idx)==1),length(idx));
end
%idx = find((sig_field.conf==3 | sig_field.label==-1)&...
%           sig_field.pos>off&sig_field.pos<length(region.seq)-off);
%find((sig_field.label==1|rand(length(sig_field.pos),1)<PAR.SETs.subsample_neg)

if 0% sum(sig_field.label(idx)==1)==0
  keyboard
end