function gen_training_examples4signal(PAR)
% gen_labels_and_train(PAR)

%paths
jobinfo = [] ;
input_sig = getfield(PAR.FN.input_sig, PAR.Signal_name);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% CHECK FILES (genes, trivial_regions, blocks, splits)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~fexist(PAR.FN.output.fn_genes_signals)
  if 0,
    error(sprintf('Genes File does not exist (%s); create it first with prepare_complete_genes',PAR.FN.output.fn_genes_signals)) ;
  end 
else
  Old = load(PAR.FN.output.fn_genes_signals,'PAR');
  % assert(isequal(PAR.FN.input,Old.PAR.FN.input))
  diff_fields = compare_PAR(PAR.info_genes,Old.PAR.info_genes) ;
  if ~isempty(diff_fields)
    warning('genes file exists, info_genes different')  
  end
  if ~isequal(PAR.label_source.use_anno,Old.PAR.label_source.use_anno)|...
        ~isequal(PAR.label_source.use_confirmed,Old.PAR.label_source.use_confirmed)
    error('genes file exists, label sources are differnt!')  
  end
  if isfield(Old.PAR.label_source,PAR.Signal_name)&&...
        ~isequal(PAR.label_source.(PAR.Signal_name),Old.PAR.label_source.(PAR.Signal_name))
    warning('genes file exists, label sources for specified signal are differnt!')  
  end
  clear diff_fields Old
end

if ~fexist(PAR.FN.output.fn_trivial_regions)
  error('trivial_regions file does not exist; create it first with prepare_regions')
else
  load(PAR.FN.output.fn_trivial_regions,'trivial_regions')
  %assert(isequal(trivial_regions(1).config,PAR.FN.input.fn_genome_config));
end

if ~fexist(PAR.FN.output.fn_training_blocks)
  error('training block file does not exist; create it first with prepare_regions')
else
  Old = load(PAR.FN.output.fn_training_blocks, 'PAR');
  %assert(isequal(Old.PAR.FN.output.fn_trivial_regions,PAR.FN.output.fn_trivial_regions));
  %assert(isequal(Old.PAR.FN.input.fn_genome_config,PAR.FN.input.fn_genome_config));
  diff_fields = compare_PAR(PAR.regions,Old.PAR.regions) ;
  if ~isempty(diff_fields)
    warning('training blocks exist, Parameters are different ')
  end
  load(PAR.FN.output.fn_training_blocks,'blocks','split')
  assert(length(fieldnames(split))==PAR.SETs.num_splits);
  clear Old diff_fields
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% save labels
generate_examples=1;
if fexist([input_sig.fn_candsites 'PAR.mat'])
  fprintf(1,'files with candidate pos already exists\n')
  Old = load([input_sig.fn_candsites 'PAR.mat'],'PAR');
  diff_fields = compare_PAR(PAR.Signals,Old.PAR.Signals) ;
  if ismember('Signal',PAR.Signal_name)
    warning('label file exists, Signal parameters are differnt!')  
    % fprintf('label file exists, Signal parameters are differnt!')  
  end
  [temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',input_sig.fn_candsites));
  if (str2num(num_files)==length(trivial_regions))
    generate_examples=0;
    fprintf(1,'all cand files found; no label generation\n')
  else
    fprintf(1,'%i/%i cand files missing; start label generation\n',length(trivial_regions)-str2num(num_files),length(trivial_regions))
  end
  clear Old
end

if generate_examples
  fprintf(1,'generating labels\n')
  jobinfo = rproc_empty(0) ;
  D.PAR = PAR ;
  for j=1:length(trivial_regions)
    D.t_region = trivial_regions(j);
    %keyboard
    filename = sprintf('%scontig_%i%s',PAR.FN.input_sig.(PAR.Signal_name).fn_candsites,trivial_regions(j).chr_num,trivial_regions(j).strand);
    if fexist(filename)
      continue
    end
    if PAR.tasks.signals.run_locally
      save_label_files_starter(D);
    else
      jobinfo(j) = rproc('save_label_files_starter', D, PAR.RPROC.MEMREQ/2, [], PAR.RPROC.time_req);
      % save_label_files_starter(D)
    end;
    % save_label_files(t_region,PAR,genes)
  end
  [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 60, 1, 0);
  [temp, num_files] = unix(sprintf('ls %scontig_*.pos | wc -l',input_sig.fn_candsites));
  assert(str2num(num_files)==length(trivial_regions))
  if num_crashed>0, 
    num_crashed
    return
  end 
  rproc_cleanup(jobinfo_1) ;
end



modes = {'train','test'};
for mode=modes
  [filename exists]=gen_example_filename(PAR,mode{1},PAR.SETs.num_splits);
  if ~exists
    %regions = retrieve_signals(regions,input_sig.fn_candsites,PAR.Signal_name,{'label',Signal.Conf_names{:}});
    %save_examples(regions,PAR);
    fprintf(1, 'SAVE EXAMPLES ....');
    D.blocks = blocks;
    D.split = split;
    D.PAR = PAR;
    % jobinfo = rproc('save_examples', D, PAR.RPROC.MEMREQ*2, PAR.RPROC.options,PAR.RPROC.time_req);
    % [jobinfo_1, num_crashed]=rproc_wait(jobinfo, 60, 1, 0);
    save_examples(D) ; num_crashed = 0 ;
    if num_crashed>0
      return
    end
  end
end

