function init_svm_and_kernels(PAR)


conservWdsWeighting =0;
%------------------------------------------------------------------------------%
% init svm
%------------------------------------------------------------------------------%

% === cleaning
sg( 'send_command', 'clean_features TRAIN' );
sg( 'send_command', 'clean_preproc' );
sg( 'send_command', 'clean_kernel');
%clear sg;

%%%sg( 'send_command', 'loglevel ALL');   % DEBUG
if ~isfield(PAR.method, 'svm_name')
  warning( 'no svm specified, set to default: LIGHT')
  PAR.method.svm_name='LIGHT'
end

% === SVM
sg( 'send_command', sprintf( 'new_svm %s',PAR.method.svm_name ) );
%sg( 'send_command', 'svm_epsilon 1e-4' );
%%%sg( 'send_command', sprintf( 'svm_epsilon %.1e', PAR.svmEps ) );
sg('send_command', 'svm_epsilon 1e-5');
%%%sg( 'send_command', sprintf( 'use_linadd %d', PAR.useLinadd ) );
sg('send_command', 'use_linadd 1');
%%%sg( 'send_command', sprintf( 'svm_qpsize %d', PAR.qpSize ) );
if ~isfield(PAR.method, 'qpsize')
  warning( 'qpsize not specified, set to default: 50')
  PAR.method.qpsize=50;
end
sg('send_command', sprintf('svm_qpsize %i',PAR.method.qpsize)) ; 
if( conservWdsWeighting > 0 )
  sg( 'send_command', 'use_batch_computation 0' );
else
  sg( 'send_command', 'use_batch_computation 1' );
end

% === parallel training?
if(1)% PAR.parallel )
  %sg( 'send_command', sprintf( 'threads %d', PAR.parallel ) );
  sg('send_command', 'threads 16');
end;


%------------------------------------------------------------------------------%
% init kernels
%------------------------------------------------------------------------------%

nofKernels = length(PAR.method.kernels);
assert( nofKernels >= 1 );
if( nofKernels == 1 )
  addKernel = 'set_kernel';
else
  addKernel = 'add_kernel 1';
  sg( 'send_command', sprintf( 'set_kernel COMBINED 100'));%i', PAR.method.cacheSize ) );
end;

for k = 1:nofKernels
  kernel = PAR.method.kernels{k};
  % for compatibility with older versions
  if ~isfield(kernel,'kernel_name')&isfield(kernel, 'name')
    kernel.kernel_name = kernel.name;
  end
  fprintf(1, '%s: %s\n', addKernel, kernel.kernel_name)
  %if k>1
  %  assert(~isfield(kernel,'C'))%reg. par C is used from kernel one only
  %end
  
  switch kernel.kernel_name
   case 'WD'
    if ~isfield(kernel, 'mismatch')
      kernel.mismatch=0;
    end
    assert(length(kernel.order)==1)
    assert(length(kernel.mismatch)==1)
    sg( 'send_command', sprintf( '%s WEIGHTEDDEGREE CHAR 10 %i %i 1 1', addKernel, kernel.order, kernel.mismatch ) );
    
   case 'WDS'
    if ~isfield(kernel, 'mismatch')
      kernel.mismatch=0;
    end
    if ~isfield(kernel, 'shift')
      kernel.shift=0;
    end
    if ~isfield(kernel, 'shift_const')
      kernel.shift_const=0;
    end
    assert(length(kernel.order)==1)
    assert(length(kernel.mismatch)==1)
    len = kernel.rwin-kernel.lwin+1;
    shifts = ceil(kernel.shift * abs( (kernel.lwin:kernel.rwin)+1 ));
    shifts = shifts + kernel.shift_const;
    shifts = sprintf( '%i ', shifts );
    sg( 'send_command', sprintf( '%s WEIGHTEDDEGREEPOS3 CHAR 10 %i %i %i 1 %s', addKernel, kernel.order, kernel.mismatch, len, shifts ) );
    clear shifts len
    
   case 'SP'
    use_sign=0;
    % assert(~use_sign|~PAR.tasks.pred_from_seq)
    sg( 'send_command', sprintf( '%s COMMSTRING WORD 10 %i', addKernel, use_sign) );
    if PAR.tasks.pred_from_seq
      sg('loglevel','ALL')
      sg( 'use_diagonal_speedup',1)
    end
   case 'RBF'
    if ~isfield(kernel, 'cache')
      kernel.cache = 40;
    end
    assert(length(kernel.sigma)==1)
    sg( 'send_command', sprintf( '%s GAUSSIAN REAL %i %f', addKernel,kernel.cache,kernel.sigma ) );
   
   case 'POLY'
    if ~isfield(kernel, 'cache')
      kernel.cache = 40;
    end
    if ~isfield(kernel, 'type')
      kernel.type = 'REAL';
    end 
    if ~isfield(kernel, 'norm')
      kernel.type = 'R';
    end
    assert(length(kernel.degree)==1)
    sg( 'send_command', sprintf( '%s POLY %s %i %i %i', addKernel,kernel.type,kernel.cache,kernel.degree,kernel.norm ) );
   
   case 'LINEAR'
    if ~isfield(kernel, 'cache')
      kernel.cache = 40;
    end
    if ~isfield(kernel, 'type')
      kernel.type = 'REAL';
    end
    sg( 'send_command', sprintf( '%s LINEAR %s %i', addKernel,kernel.type ,kernel.cache) );
    
   otherwise
    fprintf(1, 'not yet implemented: kernel_name: %s\n', kernel.kernel_name)
  end;
  
end;%loop over kernels

if isfield(PAR.method, 'subkernel_weights') 
  assert(length(PAR.method.subkernel_weights)==nofKernels)
  sg('set_subkernel_weights',PAR.method.subkernel_weights);
end
 
% for compatibility with older versions
if ~isfield(PAR.method.par_ms,'C')& isfield(PAR.method.kernels{1},'C')
  PAR.method.par_ms.C = PAR.method.kernels{1}.C;
end
sg('send_command', sprintf('c %1.2e', PAR.method.par_ms.C)) ;

disp('done with kernel setup');
