function jobinfo=ms_gfwd_starter( PAR_1 )

% ms_gfwd_starter(PAR)
%
% used for modelselection as well as simple training of splice site
% sensors. 
%  
% * If PAR.modelsel is set to 0, only training is performed:
%   SVMs are trained with all possible parameter combination 
%   specified in PAR.SVM (orders,Cs,shifts) for all iments 
%   (acc/don as specified in ents) and all partitions 
%   for which PAR.SETs.partition_train_on is set to 1.
%   Function TRAIN_SVM_SPLICESITES is called. 
%  
% * If PAR.modelsel is set to 1, SVMs are evaluated on the evaluation
%   sets of each partition. 
%   Function PREDICT_SVM_SPLICESITES is called. 
%   FOR each PAR_1.Signal_nameeriment and partition the best model is chosen and
%   the test error is determeined. 
%   Finally for each PAR_1.Signal_nameeriment one single parameter set is chosen, 
%   (which can be used for retraining on 4 subsets.)
% 
%   files loaded:  
%     - PAR.fn_set (examples of each split, with XT, LT, pos, str, reg)
%   
%   files saved:
%     - PAR.FN.output.fn_SVMs
%       (e.g. /train/result/gfwd_acc_ms_set=[123]_order=x_C=y_shift=z.mat',
%       if trained on sets 1-3)
%       includes structure 'Train':
%       Train.XT, Train.LT, Train.PAR, Train.b, Train.alphas ...
%  
%     - PAR.fn_pred 
%       (e.g. /train/result/gfwd_acc_ms_set=[123_4]_order=x_C=y_shift=z.mat'
%       if trained on sets 1-3 and evaluated on set 4)
%       includes:
%       'LV','Output' (for evaluation set 4),'alphas', 'b', 'RSV','RFV','fn_SVMs';
%
%     - PAR_new.fn_pred 
%       (e.g. /train/gfwd_acc_ms_set=[123_4_5].mat'
%       best model as evaluated on set 4
%       includes: 
%       'LV','Output' (for test set 5),'alphas', 'b', 'RSV','RFV','fn_SVMs';  
%    
%     - PAR_1.fn_SVMs
%       (e.g. acc_partition=1.mat ;
%       best model for partition 1, trained on sets 1-3, 
%       evaluated on set 4, tested on set 5.)
%       includes:   
%       'Train','RFV','RSV'
%   
%     - input_sig.fn_modelsel_par
%       best modelparameters over all partitions
%       includes structure SVM
%       SVM.orders,SVM.Cs,SVM.shifts
%
%  
%cd ~/svn/projects/genefinding/sensors/
  
fid = 1;%fopen(PAR_1.FN.fn_log ,'a+');
fprintf(fid,'\n\n starting ms_gfwd_starter \n');
fprintf(fid,'date: %s\n',date);


Signal_name = PAR_1.Signal_name;
tasks = PAR_1.tasks.signals;
SETs = PAR_1.SETs;
RPROC = PAR_1.RPROC;
method = PAR_1.method;
organism = PAR_1.organism.name;


%PAR = setfield(PAR,PAR_1.method,Model_param);
if ~isequal(Signal_name,'gene')
  output_fn = getfield(PAR_1.FN.output_sig, Signal_name);
else
  output_fn = getfield(PAR_1.FN.output_lsl);
  output_fn.fn_SVMs = output.fn_train;
end
fprintf(fid,'\n\n-----------\n');

%------------------------------------------------------------------------%
% write numeric kernelpars into field for model selection
%------------------------------------------------------------------------%

if isfield(method, 'kernels') & ~isempty(method.kernels{1})
  method.par_ms = collect_ms_parameters(method.kernels, method.par_ms);
end

%------------------------------------------------------------------------%
% initiate parameters
%------------------------------------------------------------------------%
Model_param = getfield(method,'par_ms');
Model_param = orderfields(Model_param) ;

[MODELS,param_names] = generate_all_models(orderfields(method.par_ms));
if ~tasks.modelsel
  fprintf('no model selection. taking first model;\n')
  MODELS = MODELS(:,1);
end

fprintf(fid,'number of models to be tested %s: %i\n',Signal_name,size(MODELS,2));
num_models_tested = size(MODELS,2);
RSV_eval = zeros(SETs.num_partitions,size(MODELS,2));
RFV_eval = zeros(SETs.num_partitions,size(MODELS,2));

%------------------------------------------------------------------------%
% check for saved model files and save models if non are found
%------------------------------------------------------------------------%
if ~fexist(output_fn.fn_models) && ~fexist([output_fn.fn_models '.mat']) 
  %dirName = output_fn.fn_models;
  %p = find( dirName == '/' );
  %p = p( end );
  %dirName = dirName( 1:p );
  %nice_mkdir( dirName );%%this is done in create_filenames
  save(output_fn.fn_models,'MODELS','Model_param')
elseif (fexist(output_fn.fn_models) || fexist([output_fn.fn_models '.mat']) ) 
  LL = load(output_fn.fn_models);
  assert(isequal(LL.Model_param,Model_param))
  if ~isequal(LL.MODELS,MODELS)
    MODELS=append_new_models(MODELS, LL.MODELS);
    old=LL;
    save(output_fn.fn_models,'MODELS','old')
    warning('different other MODELS have already been trained, new models are appended')
  end
  clear LL old
end
num_not_eval = SETs.num_partitions*sum(num_models_tested) ;

fprintf(fid,'total number of jobs for training: %i\n',sum(SETs.partition_train_on)*sum(num_models_tested));
fprintf(fid,'total number of jobs for evaluation: %i\n',num_not_eval);

num_crashed = 1;
num_crashed_1 = 0;
rep = 1;

PAR=PAR_1;
while num_crashed + num_crashed_1 + num_not_eval>0
  q = 0 ; nq = 0 ;
  jobinfo = rproc_empty(0) ;
  jobinfo_1 = rproc_empty(0) ;
  num_not_eval = SETs.num_partitions*sum(num_models_tested) ;
  for p = 1:size(SETs.partitions,1) %% loop over partitions  
    fprintf(fid,'\n\n-----------\n');
    fprintf(fid,'PARTITION %i\n',p);
    fprintf(fid,'-----------\n');
    % if method.submit_batch
    %   Train_data=[];
    %   Test_data=[];
    % end
    
    SETs.which_train = find(ismember(SETs.partitions(p,:),SETs.info.train));
    SETs.which_val = find(ismember(SETs.partitions(p,:),SETs.info.eval));
    SETs.which_pred = find(ismember(SETs.partitions(p,:),SETs.info.pred));
    
    fprintf(fid,'\nSITE: %s\n',Signal_name);
           
    fn_save = [sprintf('%sms_set=[',output_fn.fn_SVMs),...
               sprintf('%i',SETs.which_train)];
    fn_save_pred  = [fn_save '_' sprintf('%i',SETs.which_val)];
    
    PAR.SETs = SETs;
    PAR.partition = p;
    PAR.train_error_on = tasks.get_train_error;
    PAR.train_on = SETs.partition_train_on(p);
    for num_models = 1:size(MODELS,2)
      %--------------------------------------------------------------%
      % assign parameters of model number num_models to the current parameter
      % structure
      %--------------------------------------------------------------%
      
      PAR.method = assign_model(method, MODELS, param_names,num_models); 
      
      %--------------------------------------------------------------%
      %% generate filename for SVMs and predictions
      %--------------------------------------------------------------%
      
      PAR.fn_SVMs = sprintf('%s]_model=%i.mat',fn_save,num_models) ;
      
      %--------------------------------------------------------------%
      %%%%% TRAIN
      %--------------------------------------------------------------%
      if ~fexist(PAR.fn_SVMs) & PAR.train_on
        
        q = q+1 ;
        fprintf(fid,'\n start training on set ');
        fprintf(fid,'%i',SETs.which_train);
            
        options.identifier = sprintf('%strain%s_%i',upper(organism),Signal_name,num_models) ;
        fprintf('creating job (training) %i/%i  \r', q,sum(SETs.partition_train_on)*sum(num_models_tested)) ;
        
        if ~isequal(Signal_name,'gene')
          fct_name = 'load_and_train';
          if strcmp(method.train_fct,'load_content_and_train')
            fct_name = method.train_fct;
           end
        else
          fct_name = 'load_blocks_and_train';
        end
        PAR.tasks.pred_from_seq  =0;
        % keyboard
        if ~isfield(tasks, 'run_locally') || tasks.run_locally==0,
          jobinfo(q) = rproc_create(fct_name, PAR, RPROC.MEMREQ*1.3^(rep-1),RPROC.options,RPROC.time_req) ;
        else
          %if ~isfield(Train_data, sprintf('partition_%i',p)) & ~isequal(Signal_name,'gene')
          %  Train_data.(sprintf('partition_%i',p))=load_data(PAR,PAR.SETs.which_train,'train',PAR.SETs.train_subsample_neg,PAR.SETs.train_subsample_pos);
          %end
          feval(fct_name,PAR);%, Train_data.(sprintf('partition_%i',p)))
        end 
      	
      %--------------------------------------------------------------%
      %%%%% EVAL 
      %--------------------------------------------------------------%
      elseif fexist(PAR.fn_SVMs) 
        PAR.tasks.pred_from_seq  =0;
        fprintf(fid,'\nSVM already trained on set ');
        fprintf(fid,'%i',SETs.which_train);
        
        PAR.fn_eval = sprintf('%s]_model=%i.mat',fn_save_pred,num_models) ;
        % PAR.fn_pred = sprintf('%s_%i].mat',fn_save_pred, PAR.SETs.which_pred_svm); 
        % PAR.fn_pred = sprintf('%s_best_partition=%i.mat',PAR_1.FN.output_fn.(Signal_name).fn_pred,p);
        %disp('PAR.fn_eval exist??')
        if ~fexist(PAR.fn_eval) 
          PAR.eval_on = 1;
          fprintf(fid,'\n\nstart evaluation on set ');
          fprintf(fid,'%i\n',SETs.which_val);
          nq = nq+1;
          options.ncpus = 1;
          options.identifier = sprintf('%spred%s_%i',upper(organism),Signal_name,num_models) ;
          
          fprintf('creating job (evaluation) %i/%i  \r', nq,num_not_eval) ;
          if isfield(method,'pred_fct')&& strcmp(method.pred_fct,'load_content_and_predict')
            pred_fct = 'load_content_and_predict';
          else
            pred_fct = 'load_and_predict';
          end
          if ~isfield(tasks, 'run_locally') || tasks.run_locally==0,
            jobinfo_1(nq) = rproc_create(pred_fct, PAR, RPROC.MEMREQ*1.3^(rep-1), RPROC.options,5*60) ;
            % jobinfo_1(nq) = rproc_create('load_and_predict', PAR, PAR.RPROC.MEMREQ*1.3^(rep-1), PAR.RPROC.options,5*60) ;
            % load_and_predict(PAR)
          else
            %if ~isfield(Test_data, sprintf('partition_%i',p))  
            %  Test_data.(sprintf('partition_%i',p))=load_data(PAR,PAR.SETs.which_val,'test',PAR.SETs.test_subsample_neg,PAR.SETs.test_subsample_pos);
            %end
            load_and_predict(PAR);%, Test_data.(sprintf('partition_%i',p)));
          end
          %fprintf(1,'load_and_predict\n')
          %keyboard
          %%load_and_predict(PAR)
        elseif fexist(PAR.fn_eval) 
          fprintf(fid,'\n model already evaluated on set ');
          fprintf(fid,'%i',PAR.SETs.which_val);
          
          L = load(PAR.fn_eval, 'RSV', 'RFV');%,'fn_SVMs') ;
          num_not_eval = num_not_eval-1;
              
          assert(isnumeric(L.RFV) && prod(size(L.RFV))==1) ;
          assert(isnumeric(L.RSV) && prod(size(L.RSV))==1) ;
          RFV_eval(p,num_models)= L.RFV ;
          RSV_eval(p,num_models)= L.RSV ;

        else
          fprintf(fid,'\n only training, no evaluation ');
          num_not_eval = num_not_eval-1;
        end
      end   %% if trained eval
    end   %% loop over num_models      
    
    fprintf(fid,'experiment %s: DONE\n',Signal_name );
    fprintf(fid,'partition %i: DONE\n', PAR.partition);
  end  %% loop over partitions
  
  if (~isempty(jobinfo) || ~isempty(jobinfo_1)) && isfield(RPROC, 'collect_jobs') && RPROC.collect_jobs,
    jobinfo = [jobinfo jobinfo_1] ;
    return ;
  end ;

  if ~isempty(jobinfo),
    for ii=1:length(jobinfo), jobinfo(ii) = rproc_resubmit(jobinfo(ii)); end ;
  else
    num_crashed = 0 ;
  end
  if ~isempty(jobinfo_1),
    [jobinfo_1, meta_jobinfo_1] = rproc_submit_batch(jobinfo_1, RPROC.exm_per_batch) ;
  end ;
  if ~isempty(jobinfo),
    [jobinfo,num_crashed] = rproc_wait(jobinfo,120,1,0);
  end
  if ~isempty(jobinfo_1),
    [meta_jobinfo_1,num_crashed_1] = rproc_wait(meta_jobinfo_1, 60, 1,0);
  end ;
  try
    L=load(output_fn.fn_models, 'RSV_eval', 'RFV_eval') ;
  catch
    L=struct ;
  end ;
  if ~isfield(L, 'RSV_eval') || ~isfield(L, 'RFV_eval') || ~isequal(L.RSV_eval, RSV_eval) || ~isequal(L.RFV_eval, RFV_eval),
    save(output_fn.fn_models,'-append','RSV_eval','RFV_eval')
    % save_append(output_fn.fn_models, 1,'RSV_eval',RSV_eval, 'RFV_eval',RFV_eval)
  end 
  rep = rep+1;
end  %% loop over crashed jobs

%fclose(fid)



