function save_examples(blocks,split,PAR)
%save_examples(regions,PAR)
%INPUT 	regions with field Signals with field `Signal.name` with fields:
%			-pos
%			-anno_cover
%			-est_cover
%			-cdna_cover
%	PAR with fields
%			-organism
%			-Signal.filter_label
%			-Signal.name
%			-Signal.consensus
%			-FN.input_sig.fn_examples
%			-FN.input.fn_split
%			-FN.input.fn_regions


%only for RPROC
if nargin<3
  %addpath ..
  paths
  PAR = blocks.PAR;
  split = blocks.split;
  blocks = blocks.blocks;
end

Signal = getfield(PAR.Signals, PAR.Signal_name);	
lwin = Signal.lwin_big;
rwin = Signal.rwin_big;
fields = fieldnames(Signal.filter_label);
input_sig = getfield(PAR.FN.input_sig, PAR.Signal_name);
Sig = getfield(PAR.Signals, PAR.Signal_name);

fprintf( 1, 'save_examples... (%d fields, %d splits)\n', length(fields), PAR.SETs.num_splits );
for f = 1:length(fields)
   for s=1:PAR.SETs.num_splits
    %---------------------------------------------
    %CREATE OUTPUT FILENAME
    %----------------------------------------------  
    filename = gen_example_filename(PAR, f, s); 
    
    if ~fexist(filename)
      num_examples = 0;
      POS.fn_blocks = PAR.FN.output.fn_blocks;	 
      split_reg_ids = getfield(split,sprintf('split_%i',s));
      blocks_idx = find(ismember([blocks.id],split_reg_ids));
      split_regs = blocks(blocks_idx);
      clear blocks_idx split_reg_ids
      
      %if ~isfield(split_regs,'seq') 
      %  fprintf(1,'loading sequence...\n')
      %  split_regs = load_sequence(split_regs);
      %  fprintf(1,'\n')
      %end
      
      %---------------------------------------------
      % retreive label information
      %---------------------------------------------
      
      % if ~isfield(split_regs, 'Signals')
      %   temp_region = retrieve_signals(split_regs(1),input_sig.fn_candsites,PAR.Signal_name,{'label',Sig.Conf_names{:}});
      % end
      
      
      %---------------------------------------------
      %----heuristic--->estimate size of chararray; 
      % -some positions are filtered out by the filerfunction: Signal.filter_fct
      % -if the size is to small then this function is very time-consuming
      % -otherwise it is very space-consuming
      %---------------------------------------------
      total_num_pos = retrieve_num_pos(split_regs, input_sig.fn_candsites) ;
      num_pos = ceil(total_num_pos/8) ;
      
      %keyboard
      XT=zeros([lwin+rwin,num_pos],'int8');
      LT=zeros(1,num_pos);
      POS.pos=zeros(1,num_pos);
      POS.region_id=zeros(1,num_pos);

      if isfield(PAR.Signals.(PAR.Signal_name).filter_label.(fields{f}), 'USE_ALL')&&...
            PAR.Signals.(PAR.Signal_name).filter_label.(fields{f}).USE_ALL
        Sig.Conf_names = {} ;
      end ;
      
      t = cputime;
      fprintf(1,'starting with split %i\n',s)
      for r=1:length(split_regs)
        if mod(r,100)==0,
          fprintf('region %i/%i\r', r, length(split_regs)) ;
        end ;
        
        if ~isfield(split_regs, 'Signals')
          temp_region = retrieve_signals(split_regs(r),input_sig.fn_candsites,PAR.Signal_name,{'label',Sig.Conf_names{:}});
        end
        
        sig_field = getfield(temp_region.Signals,PAR.Signal_name);
        if isfield(Signal.filter_label.(fields{f}),'use_label')
          sig_field.label = sig_field.(Signal.filter_label.(fields{f}).use_label);
        end
        
        if ~isfield(split_regs,'seq') 		
          if isfield(PAR.Signals.(PAR.Signal_name).filter_label.(fields{f}), 'USE_ALL')&&...
                PAR.Signals.(PAR.Signal_name).filter_label.(fields{f}).USE_ALL
            if temp_region.strand == '+',
              loffset=0 ; % remember the offset for later
              tmp = max(1,temp_region.start-lwin) ;
              loffset = temp_region.start - tmp ;
              temp_region.start = tmp ;
              temp_region.stop = temp_region.stop + rwin ;
            else
              tmp = temp_region.start-rwin ;
              temp_region.start = tmp ;
              temp_region.stop = temp_region.stop + lwin ;
              loffset = lwin ;
            end ;
          end ;
          temp_region = load_sequence(temp_region);
        end
        seq = temp_region.seq;
        
        %---------------------
        %FILTER EXAMPLES
        %---------------------
        if ~isfield(PAR.Signals.(PAR.Signal_name).filter_label.(fields{f}), 'USE_ALL')||...
              ~PAR.Signals.(PAR.Signal_name).filter_label.(fields{f}).USE_ALL
          if ~isfield(PAR.Signals.(PAR.Signal_name).filter_label.(fields{f}),'conf_pos') ...
                && isfield(PAR.Signals.(PAR.Signal_name).filter_label.(fields{f}),'conf')
            error('fix this: filtering will not work')
          end
          idx = feval(Signal.filter_fct, temp_region, PAR, fields{f},1);
        else
          idx = 1:length(sig_field.pos);
          sig_field.pos = sig_field.pos + loffset ;
        end
        if isempty(idx); continue; end
        
        %--------------------
        %CREATE XTs
        %--------------------
        XT_temp = zeros(lwin+rwin,length(idx),'int8');
        rid_temp= zeros(1,length(idx))+temp_region.id;
        for row=1:lwin+rwin
          XT_temp(row,:)= seq(sig_field.pos(idx)-lwin+row-1);
        end%loop over rows
        LT_temp= sig_field.label(idx)';
        pos_temp = sig_field.pos(idx)';
        assert(size(LT,2)==num_examples||LT(num_examples+1)==0)
        assert(num_examples==0||LT(num_examples)~=0)
        
        %[size(XT,2) num_examples+length(idx) ceil(size(XT,2)*r/length(split_regs))]
        if size(XT,2)<num_examples+length(idx), 
          warning('extending XT') ;
          POS.pos(ceil(size(XT,2)*1.2)) = 0 ;
          POS.region_id(ceil(size(XT,2)*1.2)) = 0 ;
          LT(ceil(size(XT,2)*1.2)) = 0 ;
          XT(1,ceil(size(XT,2)*1.2)) = 0 ;
        end ;
        
        XT(:, num_examples+1:num_examples+length(idx))= XT_temp;	
        LT(num_examples+1:num_examples+length(idx))= LT_temp;
        POS.pos(num_examples+1:num_examples+length(idx))= pos_temp;
        POS.region_id(num_examples+1:num_examples+length(idx))=rid_temp;
        num_examples = num_examples+length(idx);
      end%loop over blocks
      fprintf(1, 'time required for one split: %.2f\n',cputime-t)
      fprintf(1,'save file %s\n',filename)
      %=== remove zeros
      assert(all(LT(num_examples+1:end)==0))
      LT = LT(1:num_examples);
      XT = XT(:,1:num_examples);
      POS.pos = POS.pos(1:num_examples);
      POS.region_id = POS.region_id(1:num_examples);
      
      %----------------------------------------------- 
      % remove examples that appear more than once
      %----------------------------------------------- 
      if 0
        [tmp1 idx tmp2] = unique(XT','rows');
        idx = idx(randperm(length(idx)));
        XT = XT(:,idx);
        LT = LT(idx);
        POS.pos = POS.pos(idx);
        POS.region_id = POS.region_id(idx);
      end
      %-----------------------------------------------
      % exclude examples containing non ACGT-Symbols 
      %-----------------------------------------------
      %xtpos=XT~='A'&XT~='C'&XT~='G'&XT~='T';
      %save space
      xtpos=XT~='A';
      for s='CGT'
        xtpos=xtpos&XT~=s;
      end
      cols = find(sum(xtpos));
      clear xtpos
      if length(cols)>0
        warning(sprintf('%i examples containing non ACGT symbols excluded',length(cols)))
        XT(:,cols)=[];
        LT(cols)=[];
        POS.pos(cols)=[];
        POS.region_id(cols)=[];
      end
      %----------------------------------------------- 
      % save matrix in pieces into one file
      %----------------------------------------------- 
      assert(sum(LT==1)>0);
      fprintf( 'trying to save %s\n', filename );
      save_matrix(filename, XT, 'XT');
      save(filename,'-append','LT','POS','PAR');
      % save_append(filename, 1,'LT', LT, 'POS', POS, 'PAR', PAR);
    end%if ~fexist(filename)
   end%loop over splits
   save_example_statistics(PAR);
end%loop over filter_label fields

