function [all_done,jobinfo] = save_predictions(PAR,strands)

%  save_pred_files(PAR)
%
%  Loads predictions corresponding to each contig for a signal
%  defined in PAR.Signal_name.
%  Saves them to binary files which can be read by 
%  interval_query.
%
% INPUT PAR.FN.input_sig.fn_candsites
%       PAR.Signal_name
%       PAR.Signal.(Signal_name).label_fct
%       PAR.Signal.(Signal_name).consensus
%       PAR.Signal.(Signal_name).offset  
%       PAR.Signal.(Signal_name).lwin_big
%       PAR.Signal.(Signal_name).rwin_big
%       PAR.Signal.(Signal_name).type
%       PAR.Signal.(Signal_name).name
%       PAR.Signal.(Signal_name).Conf_names
%
%       PAR.FN.output_sig.(Signal_name)
%
%  see retrieve_signals, interval_query 
if nargin<2
  strands='+-';
end
  
jobinfo = rproc_empty(0) ;
options = PAR.RPROC.options;
options.ncpus = 1;
options.identifier = sprintf('%scheck%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
MEMREQ = PAR.RPROC.MEMREQ;
if ~isfield(PAR.tasks,'run_locally')
  PAR.tasks.run_locally=0;
end
fid=1;q=0;

genome_info = init_genome(PAR.FN.input.fn_genome_config);

PAR.fn_pred  = PAR.FN.output_sig.(PAR.Signal_name).fn_pred;
filename = sprintf('%sall_good.mat', PAR.fn_pred);
all_pred = 1;all_done = 0;
if fexist(filename)
  load(filename)
  all_good
  if all(all_good(:)==1)
    all_done = 1;
    return
  elseif any(all_good(:)==-1)
    disp('something wrong')
    all_good(all_good==-1)=0;
    PAR.tasks.run_locally =1;
  end
else 
  all_good = zeros(2, length(genome_info.contig_names));
  save(filename, 'all_good')
end

for c = 1:length(genome_info.contig_names)
  for s = strands
    filename = sprintf('%scontig_%i%s', PAR.fn_pred, c, s);
    if ~fexist([filename '.pos'])| ~fexist([filename '.output'])| ~fexist([filename '.Conf'])| ~fexist([filename '.Conf_cum'])
      all_done=0;
      all_pred=0;
      break
    elseif ~all_good(find(strcmp({'+','-'},s)),c) 
      PAR.chrom = c;
      PAR.strand = s;
      if ~PAR.tasks.run_locally
        q=q+1;
        if isfield(PAR.RPROC, 'collect_jobs') && PAR.RPROC.collect_jobs 
          jobinfo(q) = rproc_create('check_predictions', PAR, PAR.RPROC.MEMREQ,options,2*60) ;
        else
          jobinfo(q) = rproc('check_predictions', PAR, MEMREQ,options, 2*60);
        end
      else
        fprintf('checking predictions\n')
        check_predictions(PAR)
      end
    end
  end
end

if all_pred
  return
end

options.identifier = sprintf('%ssave%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
for c = 1:length(genome_info.contig_names)
  for s = strands
    PAR.chrom = c;
    PAR.strand = s;
    filename = sprintf('%scontig_%i%s', PAR.fn_pred, c, s);
    if ~fexist([filename '.pos'])| ~fexist([filename '.output'])| ~fexist([filename '.Conf'])| ~fexist([filename '.Conf_cum'])
      if ~PAR.tasks.run_locally
        q=q+1;
        if isfield(PAR.RPROC, 'collect_jobs') && PAR.RPROC.collect_jobs 
          jobinfo(q) = rproc_create('save_predictions_helper', PAR, PAR.RPROC.MEMREQ,options,2*60) ;
        else
          jobinfo(q) = rproc('save_predictions_helper', PAR, MEMREQ,options, 2*60);
        end
      else
        save_predictions_helper(PAR)
      end
    end
  end %%% loop over strand
end %%% loop over contigs 

