function setFeatures( svmMode, PAR,Data);

% (svmMode, PAR,Data)

%------------------------------------------------------------------------------%
% check
%------------------------------------------------------------------------------%

assert( strcmp(svmMode,'TRAIN') || strcmp(svmMode,'TEST') );

%assert( isscalar(Data.pos0) );
%assert( size(XT,2) > 0 );
%assert( ischar( XT ) );

if ~isfield(PAR.tasks,'use_preproc')
  PAR.tasks.use_preproc=1;
end
%------------------------------------------------------------------------------%
% pass data
%------------------------------------------------------------------------------%

conservWdsWeighting=0;
sg('send_command', 'loglevel ALL');
if PAR.tasks.pred_from_seq
  seq = Data.seq;
  idx = find(seq~='A'&seq~='C'&seq~='G'&seq~='T');
  if ~isempty(idx)
    warning('non AGCT-Symbols found; changed to As')
    seq(idx)='A';
  end
  pos = Data.pos;
else
  XT = Data.XT;
  %pos0=Data.pos0;
  if isfield(Data, 'LT')
    LT = Data.LT;
  end
  if isfield(Data, 'W')
    W = Data.W;
  end
end

%==============================================================================%
% set labels
%==============================================================================%
if strcmp(svmMode,'TRAIN')
  sg( 'set_labels', 'TRAIN', LT ); 
end
%==============================================================================%
% set features
%==============================================================================%



nofKernels = length(PAR.method.kernels);
assert( nofKernels >= 1 );
if( nofKernels == 1 )
  addFeats = 'set_features';
else
  addFeats = 'add_features';
end;


for k = 1:nofKernels
  kernel = PAR.method.kernels{k};
  % for compatibility with older versions
  if ~isfield(kernel, 'kernel_name')&isfield(kernel,'name')
    kernel.kernel_name = kernel.name;
  end
  if ~PAR.tasks.pred_from_seq   
    if ~isfield(kernel,'lwin')
      warning('no lwin specified for kernel')
      kernel.lwin = -PAR.method.center;
    end
    if ~isfield(kernel,'rwin')
      warning('no rwin specified for kernel')
      kernel.rwin = -PAR.method.center+size(XT,1)-1;
    end
    select = (kernel.lwin:kernel.rwin)+PAR.method.center+1;
    if ~isequal(select,[1:size(XT,1)])
      XT_temp = XT(select,:);
    else
      XT_temp = XT;
    end
    %keyboard;
  else
    %positiones should be a 0-based index
    pos_temp=pos+kernel.lwin-1;
    winsize_temp=kernel.rwin-kernel.lwin+1;
    assert(length(pos_temp)>0)
    assert(length(seq)>max(pos_temp)+winsize_temp)
    assert(winsize_temp>0)
  end
  %------------------------------------------------------------------------------%
  % kernel: WD or WDS
  %------------------------------------------------------------------------------%

  if strcmp(kernel.kernel_name,'WD') || strcmp(kernel.kernel_name,'WDS')
    if ~PAR.tasks.pred_from_seq
      fprintf( '--> XT[%d:%d], num_examples=%i,  order=%d\n', min(select), max(select),size(XT,2), kernel.order );
      sg( addFeats, svmMode, XT_temp, 'DNA' );
    else
      sg( addFeats, svmMode,{seq}, 'DNA' );
      sg('from_position_list','TEST', winsize_temp, int32(pos_temp));
    end
  end;
  

  %------------------------------------------------------------------------------%
  % kernel: Spectrum
  %------------------------------------------------------------------------------%

  if strcmp(kernel.kernel_name,'SP' )
    if~PAR.tasks.pred_from_seq 
      fprintf( '--> XT[%d:%d], wordlen=%d\n', min(select), max(select), kernel.wordlen );
      sg( addFeats, svmMode, XT_temp, 'DNA');
    
      if( ~isfield(kernel,'gaps') )
        sg( 'send_command', sprintf( 'convert %s STRING CHAR STRING WORD %i %i', svmMode, kernel.wordlen, kernel.wordlen-1 ) );
      else
        fprintf('--> convert\n')
        sg( 'send_command', sprintf( 'convert %s STRING CHAR STRING WORD %i %i %i', svmMode, kernel.wordlen, kernel.wordlen-1, kernel.gaps ) );
      end;
      
      if( strcmp(svmMode,'TRAIN')&& PAR.tasks.use_preproc )
        fprintf('--> clean_preproc\n')
        sg( 'send_command', 'clean_preproc' );
        fprintf('--> add_preproc SORTWORDSTRING\n')
        sg( 'send_command', 'add_preproc SORTWORDSTRING' );
        fprintf('--> attach_preproc %s\n', svmMode)
        sg( 'send_command', sprintf( 'attach_preproc %s', svmMode ) );
      end;
    else
      % from_position_list doesn't work for training for spectrum kernel  
      assert(strcmp(svmMode,'TEST'))
      sg( addFeats, svmMode, {seq}, 'DNA' );
      % window is cut out from sequence if vonvertion is made after calling 'from_position_list
      % if convertion is done at this place the window size must be reduced by -ordnung-1;
      % (not yet tested) this woud save a lot of time and space. 
      % 
      if( ~isfield(kernel,'gaps') )
        sg( 'send_command', sprintf( 'convert %s STRING CHAR STRING WORD %i %i', svmMode, kernel.wordlen, 0) );
      else
        fprintf('--> convert\n')
        sg( 'send_command', sprintf( 'convert %s STRING CHAR STRING WORD %i %i %i', svmMode, kernel.wordlen, 0, kernel.gaps ) );
      end;
      sg('from_position_list','TEST', winsize_temp- kernel.wordlen+1, int32(pos_temp+kernel.wordlen-1));
    end
  end;
 
  %------------------------------------------------------------------------------%
  % kernel:  RBF 
  %------------------------------------------------------------------------------%
 
  if strcmp(kernel.kernel_name,'RBF')
    fprintf( '--> XT[%d:%d], num_examples=%i,  sigma=%d\n', min(select), max(select),size(XT,2), kernel.sigma );
    sg( addFeats, svmMode, XT_temp);    
  end;
  
  %------------------------------------------------------------------------------%
  % kernel:  POLYNOMIAL
  %------------------------------------------------------------------------------%
 
  if strcmp(kernel.kernel_name,'POLY')
    fprintf( '--> XT[%d:%d], num_examples=%i,  degree=%d\n', min(select), max(select),size(XT,2), kernel.degree );
    sg( addFeats, svmMode, XT_temp);    
  end;
  
  %------------------------------------------------------------------------------%
  % kernel:  LINEAR
  %------------------------------------------------------------------------------%
  
  if strcmp(kernel.kernel_name,'LINEAR')
    fprintf( '--> XT[%d:%d], num_examples=%i,\n', min(select), max(select),size(XT,2) );
    sg( addFeats, svmMode, XT_temp);    
  end;
end%loop over kernels
%------------------------------------------------------------------------------%
% done
%------------------------------------------------------------------------------%

if( PAR.tasks.use_preproc & strcmp(svmMode,'TEST')  & ~PAR.tasks.pred_from_seq )
   fprintf('--> attach_preproc %s', svmMode)
  sg( 'send_command', sprintf( 'attach_preproc %s', svmMode ) );
end;

clear XT;
sg( 'send_command', sprintf( 'init_kernel %s', svmMode ) );

% --- set feature weights
if( conservWdsWeighting > 0 )
  W1 = W( select1, : );
  if( conservWdsWeightNorming )
    fprintf( '--> conserv(WDS): using normalized feature weights!\n' );
    meanW = mean( W1, 1 );
    W1 = W1 .* repmat( 1./meanW, size(W1,1), 1 );
  else
    fprintf( '--> conserv(WDS): using raw feature weights!\n' );
    W1 = 3/size(W1,1) * W1;
  end;
  sg( 'set_subkernel_weights_combined', W1, 1 );
  clear W1;
end;


