function [RFV, RSV] = svm_predict(Train, Data, svmFileName,outFileName);
%------------------------------------------------------------------------------%
% deal with parameters 
%------------------------------------------------------------------------------%
% for usage with rproc
if nargin==1
  paths
  Data  = Train.Data;
  svmFileName = Train.svmFileName;
  outFileName = Train.outFileName;
  Train = Train.Train;
end
PAR = Data.PAR;

if( ~isfield(PAR,'conservWeighting') )
  PAR.conservWeighting = 0;
end;
if( PAR.conservWeighting == 0 )
  W1 = [];
  meanCons = [];
else
  W1 = res.W;
  assert( ~isempty(W1) );
  meanCons = mean( W1, 1 );
end;

%------------------------------------------------------------------------------%
% init svm and set kernel parameters
%------------------------------------------------------------------------------%
init_svm_and_kernels(PAR);
sg_ver = sg('get_version');
%------------------------------------------------------------------------------%
% init training features with labels and examples that only contain those of 
% support vectors
%------------------------------------------------------------------------------%
sg('send_command' , 'clean_features TRAIN');
D.pos0 = PAR.method.center;
D.XT = Train.XT;
D.LT = Train.LT;
PAR_tmp = PAR;
PAR_tmp.tasks.pred_from_seq=0;
if PAR.tasks.pred_from_seq
  PAR_tmp.tasks.use_preproc=0;
end
setFeatures('TRAIN',PAR_tmp,D);
clear D PAR_tmp

%------------------------------------------------------------------------------%
% init test features with examples if pred_from_seq==0 or with seq and positions
% in the case of genome wide predictions
%------------------------------------------------------------------------------%
sg( 'send_command', 'clean_features TEST' );
if ~PAR.tasks.pred_from_seq
  D.XT = Data.XT;
  if isfield(PAR.method, 'feature_type') && isequal(PAR.method.feature_type,'double') && PAR.method.normalize_features
    D.XT(Train.idx_rm,:)  = [];
    D.XT = D.XT - repmat(Train.mean,size(D.XT,2),1)';
    D.XT = D.XT./repmat(Train.std,size(D.XT,2),1)';
  end
  D.pos0 = PAR.method.center;
  D.pos = Data.Pos;
else
  D.seq = Data.seq;
  assert(size(Data.pos,1)==1|size(Data.pos,2)==2)
  if size(Data.pos,1)~=1
    D.pos = Data.pos';
  else
    D.pos = Data.pos;
  end
end

if ~isempty(D.pos)
  PAR.tasks.use_preproc=0;
  setFeatures( 'TEST', PAR, D);
  clear D
  %------------------------------------------------------------------------------%
  % setup svm with trainingdata
  %------------------------------------------------------------------------------%
  %Train.XT should contain only support vector examples
  assert(size(Train.alphas, 1)==size(Train.XT,2))
  % alphas are in sparse format col_1=value; col_2=0-based index
  % change index to 0:size(alphas,2)-1 consistent with XTs  
  Train.alphas(:,2)= (0:size(Train.alphas, 1)-1)';
  sg('set_svm',Train.b,Train.alphas);
  
  %------------------------------------------------------------------------------%
  % classify
  %------------------------------------------------------------------------------%
  if( PAR.method.use_mkl )
    sg( 'set_subkernel_weights', betas );
  end;
  output = sg( 'svm_classify' );
else
  output = [];
end

%------------------------------------------------------------------------------%
% save output
%------------------------------------------------------------------------------%

if isfield(Data, 'LT')
  RSV = calcrocscore(output, Data.LT); 
  RFV= calcrfcscore(output, Data.LT);
  LT = Data.LT;
  %   save( outFileName, 'output','LT','PAR', 'svmFileName','RSV','RFV' );
  POS = Data.Pos;
  fprintf( 'save output to %s\n',outFileName );
  save( outFileName, 'output','LT','POS', 'PAR', 'svmFileName','RSV','RFV','sg_ver' );
else
  pos = Data.pos;
  fprintf( 'save output to %s\n',outFileName );
  save( outFileName, 'output', 'pos', 'PAR', 'svmFileName','sg_ver' );
end

