#!/bin/bash

#
# a small script that tries to ensure that the matlab and the python version of
# interval query return the same values
#
#save 'res_pos.matlab' res_pos -ASCII

echo "-------------------------"
echo "Performing sanity checks"
echo "-------------------------"

( cat <<EOF
addpath /fml/ag-raetsch/home/raetsch/svn/projects/genefinding/utils
fname = '/fml/ag-raetsch/share/projects/genefinding/A_thaliana/jonas_new_genes/exp/sensors/acc/output_SVMWD/pred/contig_1+';
fields={'Conf'} ;
[res_pos, res_score] = interval_query(fname, fields, [1000;10000000]); 
save 'res_score_matlab.txt' res_score -ASCII -DOUBLE
fid = fopen('res_score_matlab.txt','wt');
for i=1:length(res_score),
fprintf(fid,'%2.6f\n',res_score(i));
end
EOF
) > test_script.m

matlab -nojvm -r "test_script;exit;" || echo -e "\nmatlab script failed!\n"


( cat <<EOF
#!/usr/bin/env python
from numpy import inf
from Genefinding import doIntervalQuery
sscore_filename = '/fml/ag-raetsch/share/projects/genefinding/A_thaliana/jonas_new_genes/exp/sensors/acc/output_SVMWD/pred/contig_%d%s';
scores = doIntervalQuery(1,'+',1000,10000000,sscore_filename)
out_fh = open('scores_python.txt','w+')
for elem in scores:
   if elem != -inf:
      out_fh.write('%f\n'%elem)
out_fh.close()
EOF
) > test_script.py

chmod +x test_script.py
./test_script.py || echo -e "\npython script failed!\n"

lineNrMatFile=`wc -l res_score_matlab.txt | cut -f1 -d ' '`
lineNrPyFile=`wc -l scores_python.txt | cut -f1 -d ' '`

if [ $lineNrMatFile -ne $lineNrPyFile ]
then
   echo "Error: Result files of matlab and python are not of equal size!!"
fi

paste --delimiters=- res_score_matlab.txt scores_python.txt | bc -l > diff_result.txt

result=`cat diff_result.txt | sort -u`

if [ $result -ne 0 ]; then 
   echo "Error: Difference in results is greater than allowed tolerance!"
fi

# cleanup
rm test_script.m test_script.py res_score_matlab.txt scores_python.txt diff_result.txt

echo "-------------------------"
echo "End of sanity checks."
echo "Everything seems fine!"
echo "-------------------------"
