
%arrayload2(sprintf('%s/confirmed_sequences_transform.mat', genome_info.basedir), 'genes') ;

fd = fopen('mRNAs.fa', 'w+') ;
mRNAs={} ;
for ix=1:length(genes)
  fprintf('%i\n', ix) ;

  % find all the leftmost and rightmost exons (transcription starts and stops)
  tstart = [];
  vertices = genes(ix).splicegraph{1};
  edges = genes(ix).splicegraph{2};
  for ixv = 1:size(vertices,2)
    if sum(edges(ixv,1:ixv-1)) == 0
      tstart(end+1) = ixv;
    end
  end

  % find all transcription ends
  tstop = [];
  for ixv = 1:size(vertices,2)
    if sum(edges(ixv+1:end,ixv)) == 0
      tstop(end+1) = ixv;
    end
  end
  
  % do a depth first search through the graph, generating a list of isoforms
  % - for each transcription start, do depth first search.
  global g_isoforms
  g_isoforms = {};
  %dfs_visit(triu(edges),1,[]);
  if genes(ix).strands(1) == '+'
    for ixt = tstart
      dfs_visit(triu(edges),ixt,[]);
    end
  else
    for ixt = tstop
      dfs_visit(tril(edges),ixt,[]);
    end
    for ixf = 1:length(g_isoforms)
      g_isoforms{ixf} = g_isoforms{ixf}(end:-1:1);
    end
  end  
  
  L=zeros(1,length(g_isoforms)) ;
  for i=1:length(g_isoforms),
    L(i) = sum(vertices(2,g_isoforms{i})-vertices(1,g_isoforms{i})) ;
  end ;
  [tmp, idx]=max(L) ;

  seq='' ;
  if ~isempty(idx),
    for i=1:length(g_isoforms{idx}),
      seq = [seq '' genes(ix).genestr(vertices(1,g_isoforms{idx}(i)):vertices(2,g_isoforms{idx}(i)))] ;
    end ;
  end ;
  mRNAs{i} = seq ;

  write_fasta(fd, sprintf('seq%i', ix), seq) ;
end ;
fclose(fd) ;

!blat -minIdentity=90 mRNAs.fa mRNAs.fa mRNAs.psl -dots=100 -noHead

fd=fopen('mRNAs.psl', 'r') ;
similarity = sparse(0) ; similarity(length(genes), length(genes))=0 ; 
last_id1 = 0 ;
while ~feof(fd),
  line=fgetl(fd) ;
  if ~ischar(line), break; end ;
  elems=separate(line) ;

  % at least 40% of maximal possible identities
  if str2num(elems{1})>= 0.90*min(str2num(elems{11}), str2num(elems{15})),
    id1 = str2num(elems{10}(4:end)) ; % remove "seq"
    id2 = str2num(elems{14}(4:end)) ; % remove "seq"
    similarity(id1,id2) = 1 ; 
    similarity(id2,id1) = 1 ;
    if last_id1~=id1,
      last_id1 = id1 ;
      fprintf('%i\n', id1) ;
    end ;
  end ;
end ;

% cluster
sets = {} ;
for i=1:length(genes),
  sets{i} = i ;
end ;

changed = 1 ;
while(changed)
  changed = 0 ;
  for i=1:length(genes),
    % find homologuos sets
    idx = find(any(similarity(:,sets{i}),2)) ;
    % join sets with current
    %if length(idx)>1, keyboard ; end ;
    for j=1:length(idx)
      if ~any(idx(j)==sets{i}), 
        if ~isempty(sets{idx(j)}),
          changed = changed + 1 ; 
        end ;
        sets{i} = union(sets{idx(j)}, sets{i}) ;
        sets{idx(j)}=[] ; 
      end ;
    end ;
  end ;
  num_elem = 0 ; num_used = 0 ;
  for i=1:length(genes),
    num_elem = num_elem + length(sets{i}) ;
    num_used = num_used + (length(sets{i})>0) ;
  end ;
  assert(num_elem==length(genes)) ;
  [changed num_used]
end ;

single_alt = 0 ; single_num = 0 ;
multi_alt = 0 ; multi_num = 0 ; multi_set_num = 0 ;

for i=1:length(sets),
  if length(sets{i})>1,
    multi_set_num = multi_set_num + 1 ;
    for j=1:length(sets{i})
      multi_num = multi_num + 1 ;
      if genes(sets{i}(j)).is_alt_spliced,
        multi_alt = multi_alt + 1 ;
      end 
    end ;
  elseif length(sets{i})==1,
    single_num = single_num + 1 ;
    if genes(sets{i}).is_alt_spliced,
      single_alt = single_alt + 1 ;
    end 
  end ;
end ;

single_alt / single_num 
multi_alt / multi_num
