addpath /fml/ag-raetsch/share/software/matlab_tools/utils
addpath ../../utils
addpath ../../splicegraphs
addpath ../../splicegraphs/detect_altsplice

fprintf('initializing genome\n') ;

%genome_info = init_genome('/fml/ag-raetsch/share/projects/altsplicedata/pacificus5/genome.config') ;
%genome_info = init_genome('/fml/ag-raetsch/share/projects/splicing/drosophila_dm5/genome.config') ;
%genome_info = init_genome('/fml/ag-raetsch/share/projects/splicing/elegans_WS170/genome.config') ;
%genome_info = init_genome('/fml/ag-raetsch/share/projects/splicing/elegans_WS120_new/genome.config') ;
%genome_info = init_genome('/fml/ag-raetsch/share/projects/splicing/briggsae_new/genome.config') ;
%genome_info = init_genome('/fml/ag-raetsch/share/projects/splicing/elegans_WS147_new/genome.config') ;
%genome_info = init_genome('/fml/ag-raetsch/share/projects/splicing/arabidopsis_new/genome.config') ;
%genome_info = init_genome('/fml/ag-raetsch/share/projects/splicing/drosophila_new/genome.config') ;
%genome_info = init_genome('/fml/ag-raetsch/share/projects/splicing/elegans_WS147_new_cdna/genome.config') ;
%genome_info = init_genome('/fml/ag-raetsch/share/projects/splicing/musmusculus_new/genome.config') ;
%genome_info = init_genome('/fml/ag-raetsch/share/projects/genomes/human_hg18/genome.config') ;
%genome_info = init_genome('/fml/ag-raetsch/share/projects/splicing/arabidopsis_resequencing/genome.config') ;

fprintf('directory: %s\n', genome_info.basedir) ;


fprintf('reading gff files\n')
fnames={} ;
for i=1:length(genome_info.contig_names) 
fname=sprintf('%s/gff/myexons_%s+.gff', genome_info.basedir, ...
                genome_info.contig_names{i}) ;
  if fexist(fname), fnames{end+1}=fname ; end ;
  fname=sprintf('%s/gff/myexons_%s-.gff', genome_info.basedir, ...
                genome_info.contig_names{i}) ;
  if fexist(fname), fnames{end+1}=fname ; end ;
end ;

genes=[] ;
for i=1:length(fnames)
  genes_ = read_gff_files(fnames(i), genome_info);
  if isempty(genes),
    genes = genes_ ;
  else
    genes(end+1:end+length(genes_)) = genes_ ;
  end ;
end ;
fprintf('-> %d transcripts\n',length(genes));


fprintf('Remove ESTs sequences with only one exon\n');
genes=filter_single_exons(genes);
fprintf('-> %d transcripts\n',length(genes));

fprintf('eliminate duplicated gene descriptors\n');
genes = merge_transcripts_by_colocation(genes, genome_info) ;

fprintf('eliminate exons of length less than 10 at the end or beginning of a transcript') ;
genes = detect_short_exons(genes) ;

fprintf('separate transcripts into separate gene descriptors\n') ;
genes_ = genes(1) ; 
num_transcripts=0 ; 
for i=1:length(genes) ; 
  num_transcripts=num_transcripts+length(genes(i).exons) ;
end ;
genes_(num_transcripts)=genes(1) ;
num_transcripts=0;
for i=1:length(genes) ;
  assert(length(genes(i).exons)==size(genes(i).terminal_info,2)) ;
  for j=1:length(genes(i).exons) ;
    assert(all(size(genes(i).exons{j})==size(genes(i).exons_confirmed{j}))) ;
    num_transcripts=num_transcripts+1 ;
    genes_(num_transcripts)=genes(i) ;
    genes_(num_transcripts).exons=genes(i).exons(j) ;
    genes_(num_transcripts).exons_confirmed=genes(i).exons_confirmed(j) ;
    genes_(num_transcripts).transcripts=genes(i).transcripts(j) ;
    genes_(num_transcripts).strands=genes(i).strands(j) ;
    genes_(num_transcripts).terminal_info=genes(i).terminal_info(:,j) ;
    genes_(num_transcripts).name=genes(i).transcripts{j} ;
  end ;
end ;
genes=genes_ ;

fprintf('Remove ESTs sequences with only one exon\n');
genes=filter_single_exons(genes) ;

fprintf('eliminate duplicated gene descriptors\n');
genes = merge_transcripts_by_colocation(genes, genome_info) ;

fprintf('filter only genes with valid introns and other properties\n') ;
genes = check_genes(genes, genome_info) ;

save(sprintf('%s/confirmed_sequences_transcripts.mat', genome_info.basedir), 'genes') ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% We construct the splice graphs and merge the information. We
% detect some simple alternative splicing events, and take note on
% which genes that the occur. Also keep track of which
% corresponding exons that are involved in these events.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('Build splice graph\n');

build_splice_graph;
%save(sprintf('%s/confirmed_sequences_build.mat', genome_info.basedir), 'genes') ;

fprintf('infer splice graph\n') ;
infer_splice_graph;
%save(sprintf('%s/confirmed_sequences_infer.mat', genome_info.basedir), 'genes') ;

fprintf('Sorting exons...\n') ;
for ix=1:length(genes)
  [dummy,exon_order] = sort(genes(ix).splicegraph{1}(1,:),2,'ascend');
  genes(ix).splicegraph{1} = genes(ix).splicegraph{1}(:,exon_order);
  genes(ix).splicegraph{2} = genes(ix).splicegraph{2}(exon_order,exon_order);
end

fprintf('detect alternatively spliced genes\n') ;
genes = alt_const(genes) ;

save(sprintf('%s/confirmed_sequences.mat', genome_info.basedir), 'genes') ;

% transform genes and build splice graphs again
genes = transform(genes) ;

build_splice_graph
infer_splice_graph

fprintf('Sorting exons...\n') ;
for ix=1:length(genes)
  [dummy,exon_order] = sort(genes(ix).splicegraph{1}(1,:),2,'ascend');
  genes(ix).splicegraph{1} = genes(ix).splicegraph{1}(:,exon_order);
  genes(ix).splicegraph{2} = genes(ix).splicegraph{2}(exon_order,exon_order);
end

genes = alt_const(genes) ;

save(sprintf('%s/confirmed_sequences_transform.mat', genome_info.basedir), 'genes') ;

