#!/bin/tcsh 
#
#$ -cwd
#$ -j y
####$ -N arabidopsis
#$ -l arch=lx26-amd64
####$ -l h_vmem=3G
####$ -R y


# $1 is genome.config
# $2 is the name
# $3 gives memory requirement for internal jobs.

sleep 8


setenv JOBNAME    "$2"
setenv ORIGPATH   `pwd`

if ( -d $ORIGPATH/gff/bin )  set path=( $ORIGPATH/gff/bin $path)
if ( -d $ORIGPATH/gff/scripts )  set path=( $ORIGPATH/gff/scripts $path)


setenv BASEDIR    `list_files basedir $1`
setenv LOGNAME    "$BASEDIR/log3.txt"
setenv GENOMEDIR  "$BASEDIR/genome"
setenv CDNADIR    "$BASEDIR/cDNA"
setenv DBESTDIR   "$BASEDIR/dbEST"
setenv GFFDIR     "$BASEDIR/gff"
setenv STATDIR    "$BASEDIR/statistics"
setenv PREDDIR    "$BASEDIR/prediction"
setenv SSDIR      "$BASEDIR/result"


whoami >> $LOGNAME
date >> $LOGNAME


echo "compute_features.sh" >> $LOGNAME
setenv JOBNUM     `qstat -u $USER | grep $JOBNAME | cut -d ' ' -f 1`
setenv IDENT      `echo $JOBNAME | cut -b 1`
setenv DATNAME    "$JOBNAME.o$JOBNUM" 

echo "Logfile is in $DATNAME"

echo
echo "train msplicer"
mkdir -p -m 775 $SSDIR
cd $ORIGPATH/gff/scripts
/fml/ag-raetsch/share/software/matlab/bin/matlab -nojvm -nodisplay -r "train_msplicer('$1','$ORIGPATH');exit;"

echo
echo "creating features"
mkdir -p -m 775 $PREDDIR
cd $ORIGPATH/gff/scripts
foreach i ('alt_3prime' 'alt_5prime' 'alt_exon' 'alt_intron')
#foreach i ('alt_exon' 'alt_intron')
#foreach i ('alt_5prime' 'alt_3prime')
    #echo $i
    setenv p `echo $i | cut -b 5`
    #echo $p
    # --- only use the alternative splice site
    echo '/fml/ag-raetsch/share/software/matlab/bin/matlab -nojvm -nodisplay -r "' "create_features('$1','$ORIGPATH','$i');exit;" '"' | qsub -j y -cwd -l arch=lx26-amd64 -N "feat_$IDENT$p" -l h_vmem="$3" -l matlab=1
end


while (`qstat -u $USER | grep "feat_$IDENT" |wc -l`)
  echo `qstat -u $USER | grep "feat_$IDENT" |wc -l` jobs left
  sleep 1000
end ;


cat feat_$IDENT?.[eo][1-9]* >> $LOGNAME
rm feat_$IDENT?.[eo][1-9]*

echo "Event files:"
ls -lh $PREDDIR/events_alt_*

echo "Event files:" >> $LOGNAME
ls -lh $PREDDIR/events_alt_* >> $LOGNAME

cd $ORIGPATH
cat $DATNAME >> $LOGNAME
echo "Completed. Logfile $DATNAME in $LOGNAME"
