#!/bin/tcsh 
#
#$ -cwd
#$ -j y
####$ -N arabidopsis
#$ -l arch=lx26-amd64
#$ -l h_vmem=1G
####$ -R y


# make sure that GF is compiled on a 64 bit machine

# $1 is genome.config
# $2 is the jobname for internal qsubs
# $3 is the argument to -l h_mem for internal qsubs

sleep 6


setenv JOBNAME    "$2"
setenv ORIGPATH   "/fml/ag-raetsch/home/ong/ag-raetsch/projects/splicing"


if ( -d $ORIGPATH/gff/bin )  set path=( $ORIGPATH/gff/bin $path)
if ( -d $ORIGPATH/gff/scripts )  set path=( $ORIGPATH/gff/scripts $path)
set path=( /fml/ag-raetsch/share/software/blat $path)  # for blat

qstat -u $USER

setenv BASEDIR    `list_files basedir $1`
setenv LOGNAME    "$BASEDIR/logK.txt"
setenv GENOMEDIR  "$BASEDIR/genome"
setenv CDNADIR    "$BASEDIR/cDNA"
setenv DBESTDIR   "$BASEDIR/dbEST"
setenv GFFDIR     "$BASEDIR/gff"
setenv STATDIR    "$BASEDIR/statistics"
setenv PREDDIR    "$BASEDIR/prediction"
setenv MSDIR      "$BASEDIR/modselect"
setenv SSDIR      "$BASEDIR/result"


whoami >> $LOGNAME
date >> $LOGNAME
echo "compute_kerneltest.sh" >> $LOGNAME
setenv JOBNUM     `qstat -u $USER | grep $JOBNAME | cut -d ' ' -f 1`
setenv IDENT      `echo $JOBNAME | cut -b 1`


if ( ! -e $GENOMEDIR) then
    echo $GENOMEDIR not found!
    exit(2)
endif
if ( ! -e $SSDIR) then
    echo $SSDIR not found!
    exit(2)
endif
if ( ! -e $MSDIR) then
    echo $MSDIR not found!
    exit(2)
endif






#####################################################################
# train the SVM with the best parameters
#####################################################################
cd $ORIGPATH/sensors/altsplice
echo
echo "training the SVM"
foreach i ('alt_3prime' 'alt_5prime' 'alt_exon' 'alt_intron')
#foreach i ('alt_5prime' 'alt_3prime')
#foreach i ('alt_5prime')
    foreach j ('1' '2' '3' '4' '5' '6' '7' '8' '9')
	setenv p `echo $i | cut -b 5`
	mkdir -p -m 775 "$BASEDIR/prediction$j"
	echo '/fml/ag-raetsch/share/software/matlab/bin/matlab -nojvm -nodisplay -r "' "training_feat('$1','$ORIGPATH','$i',$j);exit;" '"' | qsub -j y -cwd -l arch=lx26-amd64 -N "svm_$IDENT$p$j" -l h_vmem="$3" -l matlab=1
    end
end

while (`qstat -u $USER | grep "svm_$IDENT.." |wc -l`)
  echo `qstat -u $USER | grep "svm_$IDENT.." |wc -l` jobs left
  sleep 600
end ;

cat svm_$IDENT??.[eo][1-9]* >> $LOGNAME
rm svm_$IDENT??.[eo][1-9]*

foreach j ('1' '2' '3' '4' '5' '6' '7' '8' '9')
    setenv PREDDIR "$BASEDIR/prediction$j"

    echo "Trained SVMs:"
    ls -lh $PREDDIR/alphas_alt_*.mat
    
    echo "Trained SVMs:" >> $LOGNAME
    ls -lh $PREDDIR/alphas_alt_*.mat >> $LOGNAME
end

cd $ORIGPATH
echo "Completed. Logfile in $LOGNAME"
