#!/bin/tcsh 
#
#$ -cwd
#$ -j y
####$ -N arabidopsis
#$ -l arch=lx26-amd64
#$ -l h_vmem=1G
####$ -R y


# make sure that GF is compiled on a 64 bit machine

# $1 is genome.config
# $2 is the jobname for internal qsubs
# $3 is the argument to -l h_mem for internal qsubs

sleep 6


setenv JOBNAME    "$2"
setenv ORIGPATH   "/fml/ag-raetsch/home/ong/ag-raetsch/projects/splicing"


if ( -d $ORIGPATH/gff/bin )  set path=( $ORIGPATH/gff/bin $path)
if ( -d $ORIGPATH/gff/scripts )  set path=( $ORIGPATH/gff/scripts $path)
set path=( /fml/ag-raetsch/share/software/blat $path)  # for blat

qstat -u $USER

setenv BASEDIR    `list_files basedir $1`
setenv LOGNAME    "$BASEDIR/log4.txt"
setenv GENOMEDIR  "$BASEDIR/genome"
setenv CDNADIR    "$BASEDIR/cDNA"
setenv DBESTDIR   "$BASEDIR/dbEST"
setenv GFFDIR     "$BASEDIR/gff"
setenv STATDIR    "$BASEDIR/statistics"
setenv PREDDIR    "$BASEDIR/prediction"
setenv MSDIR      "$BASEDIR/modselect"


whoami >> $LOGNAME
date >> $LOGNAME
echo "compute_pred.sh" >> $LOGNAME
setenv JOBNUM     `qstat -u $USER | grep $JOBNAME | cut -d ' ' -f 1`
setenv IDENT      `echo $JOBNAME | cut -b 1`
setenv DATNAME    "$JOBNAME.o$JOBNUM" 

echo "logfile is at $ORIGPATH/$DATNAME"




#####################################################################
# create the splits into training, validation and test sets
#####################################################################
cd $ORIGPATH/sensors/altsplice
echo
echo "creating data splits"
#foreach i ('alt_5prime' 'alt_3prime')
foreach i ('alt_3prime' 'alt_5prime' 'alt_exon' 'alt_intron')
#foreach i ('alt_5prime')
    setenv p `echo $i | cut -b 5`
    echo '/fml/ag-raetsch/share/software/matlab/bin/matlab -nojvm -nodisplay -r "' "create_split('$1','$ORIGPATH','$i');exit;" '"' | qsub -j y -cwd -l arch=lx26-amd64 -N "sp_$IDENT$p" -l h_vmem="$3" -l matlab=1
end

while (`qstat -u $USER | grep "sp_$IDENT" |wc -l`)
  echo `qstat -u $USER | grep "sp_$IDENT" |wc -l` jobs left
  sleep 600
end ;


cat sp_$IDENT?.[eo][1-9]* >> $LOGNAME
rm sp_$IDENT?.[eo][1-9]*

echo "Split files:"
ls -lh $PREDDIR/split_alt_*.mat
echo "Split files:" >> $LOGNAME
ls -lh $PREDDIR/split_alt_*.mat >> $LOGNAME

#####################################################################
# run model selection to choose best parameters
#####################################################################
#cd $ORIGPATH/sensors/altsplice
#echo
#echo "doing model selection"
#mkdir -p -m 775 $MSDIR
#foreach i ('alt_5prime' 'alt_3prime')
#foreach i ('alt_3prime' 'alt_5prime' 'alt_exon' 'alt_intron')
#foreach i ('alt_5prime')
#    setenv p `echo $i | cut -b 5`
#    echo '/fml/ag-raetsch/share/software/matlab/bin/matlab -nojvm -nodisplay -r "' "modelselection('$1','$ORIGPATH','$i');exit;" '"' | qsub -j y -cwd -l arch=lx26-amd64 -N "ms_$IDENT$p" -l h_vmem=1G -l matlab=1
#end

#while (`qstat -u $USER | grep "ms_$IDENT" |wc -l`)
#  echo `qstat -u $USER | grep "ms_$IDENT" |wc -l` jobs left
#  sleep 600
#end ;


#cat ms_$IDENT?.[eo][1-9]* >> $LOGNAME
#rm ms_$IDENT?.[eo][1-9]*

#echo "Parameter files:"
#ls -lh $MSDIR/PAR_alt_*_best.mat
#echo "Parameter files:" >> $LOGNAME
#ls -lh $MSDIR/PAR_alt_*_best.mat >> $LOGNAME


#####################################################################
# train the SVM with the best parameters
#####################################################################
cd $ORIGPATH/gff/scripts
echo
echo "training the SVM"
foreach i ('alt_3prime' 'alt_5prime' 'alt_exon' 'alt_intron')
#foreach i ('alt_5prime' 'alt_3prime')
#foreach i ('alt_5prime')
    #echo $i
    setenv p `echo $i | cut -b 5`
    #echo $p

    echo '/fml/ag-raetsch/share/software/matlab/bin/matlab -nojvm -nodisplay -r "' "training('$1','$ORIGPATH','$i');exit;" '"' | qsub -j y -cwd -l arch=lx26-amd64 -N "svm_$IDENT$p" -l h_vmem="$3" -l matlab=1
end


while (`qstat -u $USER | grep "svm_$IDENT" |wc -l`)
  echo `qstat -u $USER | grep "svm_$IDENT" |wc -l` jobs left
  sleep 600
end ;


cat svm_$IDENT?.[eo][1-9]* >> $LOGNAME
rm svm_$IDENT?.[eo][1-9]*

echo "Trained SVMs:"
ls -lh $PREDDIR/alphas_alt_*.mat

echo "Trained SVMs:" >> $LOGNAME
ls -lh $PREDDIR/alphas_alt_*.mat >> $LOGNAME


cd $ORIGPATH
cat $DATNAME >> $LOGNAME
echo "Completed. Logfile $DATNAME in $LOGNAME"
