#!/bin/tcsh

if ( -d ~/svn/projects/splicing/gff/bin )  set path=( ~/svn/projects/splicing/gff/bin $path)
if ( -d ~/svn/projects/splicing/gff/scripts )  set path=( ~/svn/projects/splicing/gff/scripts $path)

if ( -d ~/projects/splicing/gff/bin )  set path=( ~/projects/splicing/gff/bin $path)
if ( -d ~/projects/splicing/gff/scripts )  set path=( ~/projects/splicing/gff/scripts $path)

setenv BASEDIR    `list_files basedir $1`
setenv CDNADIR    "$BASEDIR/cDNA"
setenv DBESTDIR   "$BASEDIR/dbEST"
setenv PROTDIR   "$BASEDIR/protein_sequences"
setenv GFFDIR     "$BASEDIR/gff"

# preparation ############################################################################### 

cut -f 14 $CDNADIR/all_cDNAs.psl  | sort -u > $GFFDIR/cdna_hit_contigs
cut -f 14 $DBESTDIR/all_ESTs_unique.psl | sort -u > $GFFDIR/est_hit_contigs
cut -f 14 $PROTDIR/all_prot.psl | sort -u > $GFFDIR/prot_hit_contigs

rm -f $GFFDIR/all_cdna.gff $GFFDIR/all_cdna.out
touch $GFFDIR/all_cdna.gff $GFFDIR/all_cdna.out
foreach i (`cat $GFFDIR/cdna_hit_contigs $GFFDIR/est_hit_contigs $GFFDIR/prot_hit_contigs`)
	echo $i
	rm -f $GFFDIR/myintrons_{$i}+.gff; touch $GFFDIR/myintrons_{$i}+.gff ;
	rm -f $GFFDIR/myexons_{$i}+.gff ; touch $GFFDIR/myexons_{$i}+.gff ;
	rm -f $GFFDIR/myintrons_{$i}-.gff; touch $GFFDIR/myintrons_{$i}-.gff ;
	rm -f $GFFDIR/myexons_{$i}-.gff ; touch $GFFDIR/myexons_{$i}-.gff ;
end ;

# cDNAs ############################################################################### 

echo generate cDNA sequence derived models
foreach i (`cat $GFFDIR/cdna_hit_contigs`)
  echo " "
  echo $i ;
  psl2gff CDNA $CDNADIR/all_cDNAs.psl $i + $GFFDIR/{$i}+.gff $GFFDIR/$i+_microexon.asc $1 > $GFFDIR/{$i}+.out &
  echo " "
  psl2gff CDNA $CDNADIR/all_cDNAs.psl $i - $GFFDIR/{$i}-.gff $GFFDIR/$i-_microexon.asc $1 > $GFFDIR/{$i}-.out 
end 
wait

foreach i (`cat $GFFDIR/cdna_hit_contigs`)
  echo $i ;
  cat $GFFDIR/{$i}+.gff >> $GFFDIR/all_cdna.gff
  cat $GFFDIR/{$i}+.gff | grep intron > $GFFDIR/myintrons_{$i}+.gff
  cat $GFFDIR/{$i}+.gff | grep exon > $GFFDIR/myexons_{$i}+.gff
  cat $GFFDIR/{$i}-.gff >> $GFFDIR/all_cdna.gff
  cat $GFFDIR/{$i}-.gff | grep intron > $GFFDIR/myintrons_{$i}-.gff 
  cat $GFFDIR/{$i}-.gff | grep exon > $GFFDIR/myexons_{$i}-.gff
  cat $GFFDIR/{$i}+.out >> $GFFDIR/all_cdna.out 
  cat $GFFDIR/{$i}-.out >> $GFFDIR/all_cdna.out 
end 

echo number of cDNA intron errors 
grep "### intron" $GFFDIR/all_cdna.out | wc -l 

echo number of cDNA derived sequences 
grep -e 0$ $GFFDIR/all_cdna.gff| cut -f 9 | sort | uniq | sed "s/GR-//g" | wc -l


# ESTS ###############################################################################
echo generate EST sequence derived models
foreach i (`cat $GFFDIR/est_hit_contigs`)
  echo " "
  echo $i ;
  psl2gff EST $DBESTDIR/all_ESTs_unique.psl $i + $GFFDIR/$i+_est.gff $GFFDIR/$i+_microexon_est.asc $1 > $GFFDIR/$i+_est.out 
  echo " "
  psl2gff EST $DBESTDIR/all_ESTs_unique.psl $i - $GFFDIR/$i-_est.gff $GFFDIR/$i-_microexon_est.asc $1 > $GFFDIR/$i-_est.out 
end
wait

rm -f $GFFDIR/all_est.out $GFFDIR/all_est.gff 
foreach i (`cat $GFFDIR/est_hit_contigs`)
  echo $i ;
  cat $GFFDIR/{$i}+_est.gff | grep intron >> $GFFDIR/myintrons_{$i}+.gff
  cat $GFFDIR/{$i}-_est.gff | grep intron >> $GFFDIR/myintrons_{$i}-.gff
  cat $GFFDIR/{$i}+_est.gff | grep exon >> $GFFDIR/myexons_{$i}+.gff
  cat $GFFDIR/{$i}-_est.gff | grep exon >> $GFFDIR/myexons_{$i}-.gff
  cat $GFFDIR/{$i}+_est.gff | grep matching > $GFFDIR/mymatches_{$i}+.gff
  cat $GFFDIR/{$i}-_est.gff | grep matching > $GFFDIR/mymatches_{$i}-.gff
  cat $GFFDIR/$i+_est.out $GFFDIR/$i-_est.out >> $GFFDIR/all_est.out 
  cat $GFFDIR/{$i}+_est.gff $GFFDIR/{$i}-_est.gff >> $GFFDIR/all_est.gff
end 


echo number of potential AT-AC introns
grep "### intron" $GFFDIR/all_est.out | egrep 'at\|[acgt][acgt][acgt][acgt]..[acgt][acgt][acgt][acgt]\|ac' | wc -l

echo number of potential EST intron errors 
grep "### intron" $GFFDIR/all_est.out | egrep -v 'ct\|[acgt][acgt][acgt][acgt]..[acgt][acgt][acgt][acgt]\|ac' | wc -l 

echo number of EST derived sequences
cat $GFFDIR/all_est.gff | cut -f 9 | sort | uniq | sed "s/EST-//g" | wc -l


# PROTEINS ###############################################################################
echo generate protein sequence derived models
foreach i (`cat $GFFDIR/prot_hit_contigs`)
  echo " "
  echo $i ;
  psl2gff PROT $PROTDIR/all_prot.psl $i + $GFFDIR/$i+_prot.gff $GFFDIR/$i+_microexon_prot.asc $1 > $GFFDIR/$i+_prot.out 
  echo " "
  psl2gff PROT $PROTDIR/all_prot.psl $i - $GFFDIR/$i-_prot.gff $GFFDIR/$i-_microexon_prot.asc $1 > $GFFDIR/$i-_prot.out 
end
wait

rm -f $GFFDIR/all_prot.out $GFFDIR/all_prot.gff
foreach i (`cat $GFFDIR/prot_hit_contigs`)
  echo $i ;
  cat $GFFDIR/{$i}+_prot.gff | grep intron >> $GFFDIR/myintrons_{$i}+.gff
  cat $GFFDIR/{$i}-_prot.gff | grep intron >> $GFFDIR/myintrons_{$i}-.gff
  cat $GFFDIR/{$i}+_prot.gff | grep exon >> $GFFDIR/myexons_{$i}+.gff
  cat $GFFDIR/{$i}-_prot.gff | grep exon >> $GFFDIR/myexons_{$i}-.gff
  cat $GFFDIR/{$i}+_prot.gff | grep matching >> $GFFDIR/mymatches_{$i}+.gff
  cat $GFFDIR/{$i}-_prot.gff | grep matching >> $GFFDIR/mymatches_{$i}-.gff
  cat $GFFDIR/$i+_prot.out $GFFDIR/$i-_prot.out >> $GFFDIR/all_prot.out 
  cat $GFFDIR/{$i}+_prot.gff $GFFDIR/{$i}-_prot.gff >> $GFFDIR/all_prot.gff
end 

# finish #################################################################################

echo number of potential AT-AC introns
grep "### protein_intron" $GFFDIR/all_prot.out | egrep 'at[acgt][acgt]..[acgt][acgt]ac' | wc -l

echo number of potential protein intron errors 
grep "### protein_intron" $GFFDIR/all_prot.out | egrep -v 'ct\|[acgt][acgt][acgt][acgt]..[acgt][acgt][acgt][acgt]\|ac' | wc -l 

echo number of protein derived sequences
cat $GFFDIR/all_prot.gff | cut -f 9 | sort | uniq | sed "s/GR-//g" | wc -l


