function splicegraph_s(info, origpath, gffsource) ;
% This file combines the following four commands.
% It chooses between the first two options using gffsource
%
%read_confirmed_sequences 
%read_annotation_sequences
%read_est_segments
%colocate_sequences


if (nargin < 3)
  gffsource = 'confirmed'
  fprintf(1,'Processing confirmed sequences');
else
  fprintf(1,'Processing %s sequences',gffsource);
end

 
addpath(sprintf('%s/utils', origpath)) ;
addpath(sprintf('%s/splicegraphs', origpath)) ;
addpath(sprintf('%s/splicegraphs/detect_altsplice', origpath)) ;
addpath(sprintf('%s/gff', origpath)) ;
addpath(sprintf('%s/gff/read_sequences', origpath)) ;
addpath /fml/ag-raetsch/share/software/matlab_tools/utils
addpath ~/svn/tools/genomes
addpath ~/svn/tools/rproc


%%%%%%%%%%%%%%%%%%%%%
%%% Genome.Config %%%
%%%%%%%%%%%%%%%%%%%%%
genome_info = init_genome(info) ;
fprintf('directory: %s\n', genome_info.basedir) ;
%%%genome_info = init_genome('/fml/ag-raetsch/share/projects/altsplicedata/rerio/genome.config') ;








if equal(gffsource,'confirmed')
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % read_confirmed_sequences %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%
  infos={} ;
  infos.genome_info=genome_info ;
  infos.fname='' ;
  jobinfo=rproc_empty(1) ;
  options={} ;
  %options.identifier='' ;
  %options.arch=64 ;

  fprintf('reading gff files\n')
  
  old_path=sprintf('%s', pwd) ;

  cd(sprintf('%s/gff/read_sequences', origpath)) ;
  pwd

  fprintf('reading gff files\n')
  fnames={} ;
  for i=1:length(genome_info.contig_names) 
    fname=sprintf('%s/gff/myexons_%s+.gff', genome_info.basedir, ...
		  genome_info.contig_names{i}) ;
    if fexist(fname), fnames{end+1}=fname ; end ;
    fname=sprintf('%s/gff/myexons_%s-.gff', genome_info.basedir, ...
		  genome_info.contig_names{i}) ;
    if fexist(fname), fnames{end+1}=fname ; end ;
  end ;
  
  genes = [];
  
  genes = read_gff_files(fnames,genome_info);
  

  fprintf('-> %d transcripts\n',length(genes));
  
  
  fprintf('Remove ESTs sequences with only one exon\n');
  genes = filter_single_exons(genes);
  fprintf('-> %d transcripts\n',length(genes));
  
  fprintf('eliminate duplicated gene descriptors\n');
  genes = merge_transcripts_by_colocation(genes, genome_info) ;
  fprintf('-> %d transcripts\n',length(genes));
  
  fprintf('eliminate exons of length less than 10 at the end or beginning of a transcript\n') ;
  genes = detect_short_exons(genes) ;
  fprintf('-> %d transcripts\n',length(genes));

  fprintf('eliminate unspliced exons within genes with introns\n');
  genes = filter_unspliced_dna(genes) ;
  fprintf('-> %d transcripts\n',length(genes));

  

  fprintf('separate transcripts into separate gene descriptors\n') ;
  genes_ = genes(1) ; 
  num_transcripts=0 ; 
  for i=1:length(genes) ; 
    num_transcripts=num_transcripts+length(genes(i).exons) ;
  end ;
  genes_(num_transcripts)=genes(1) ;
  num_transcripts=0;
  for i=1:length(genes) ;
    assert(length(genes(i).exons)==size(genes(i).terminal_info,2)) ;
    for j=1:length(genes(i).exons) ;
      assert(all(size(genes(i).exons{j})==size(genes(i).exons_confirmed{j}))) ;
      num_transcripts=num_transcripts+1 ;
      genes_(num_transcripts)=genes(i) ;
      genes_(num_transcripts).exons=genes(i).exons(j) ;
      genes_(num_transcripts).exons_confirmed=genes(i).exons_confirmed(j) ;
      genes_(num_transcripts).transcripts=genes(i).transcripts(j) ;
      genes_(num_transcripts).strands=genes(i).strands(j) ;
      genes_(num_transcripts).terminal_info=genes(i).terminal_info(:,j) ;
      genes_(num_transcripts).name=genes(i).transcripts{j} ;
    end ;
  end ;
  genes=genes_ ;


  fprintf('Remove ESTs sequences with only one exon\n');
  genes=filter_single_exons(genes) ;

  fprintf('eliminate duplicated gene descriptors\n');
  genes = merge_transcripts_by_colocation(genes, genome_info) ;
  
  fprintf('filter only genes with valid introns and other properties\n') ;
  genes = check_genes(genes, genome_info) ;

  save(sprintf('%s/confirmed_sequences_transcripts.mat', genome_info.basedir), 'genes') ;

  cd(sprintf('%s/splicegraphs', origpath)) ;
  pwd


  % SpliceGraph %
  fprintf('build splice graph\n') ;
  build_splice_graph;
  save(sprintf('%s/confirmed_sequences_build.mat', genome_info.basedir), 'genes') ;
  

  fprintf('infer splice graph\n') ;
  %parallel_infer;
  infer_splice_graph;

  save(sprintf('%s/confirmed_sequences_infer.mat', genome_info.basedir), 'genes') ;
  
  genes = alt_const(genes) ;
  save(sprintf('%s/confirmed_sequences.mat', genome_info.basedir), 'genes') ;
  
  %%% transform genes and build splicegraphs again %%%
  %fprintf('transform genes\n') ;
  %genes = transform(genes) ;
  %fprintf('-> %d transcripts\n',length(genes));

  %fprintf('build splice graph\n') ;
  %build_splice_graph;
  %fprintf('infer splice graph\n') ;
  %parallel_infer;
  %infer_splice_graph;

  %genes = alt_const(genes) ;

  %save(sprintf('%s/confirmed_sequences_transform.mat', genome_info.basedir), 'genes') ;



elseif equal(gffsource,'annotation')

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % read_annotation_sequences %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  if equal(genome_info.annotation_fnames{1}(end-2:end),'gff')
    genes = read_gff_files_anno(genome_info) ;
  else
    genes = read_xmlparse_files(genome_info) ;
  end ;


  % eliminate duplicated gene descriptors
  genes = merge_transcripts_by_name(genes) ;
  genes = merge_transcripts_by_colocation(genes, genome_info) ;

  % filter only genes with valid introns and other properties
  genes = check_genes(genes, genome_info) ;
  
  save(sprintf('%s/annotation_sequences_transcripts.mat', genome_info.basedir), 'genes') ;


  % build splicegraphs
  build_splice_graph
  save(sprintf('%s/annotation_sequences_build.mat', genome_info.basedir), 'genes') ;

  %merge_ends_graph ;
  %save(sprintf('%s/annotation_sequences_merge.mat', genome_info.basedir), 'genes') ;
  
  infer_splice_graph
  save(sprintf('%s/annotation_sequences_infer.mat', genome_info.basedir), 'genes') ;
  
  genes = alt_const(genes) ;
  save(sprintf('%s/annotation_sequences.mat', genome_info.basedir), 'genes') ;



else
  fprintf(1,'Unknown GFF source: %s',gffsource)
end










%%%%%%%%%%%%%%%%%%%%%%%%%%
% common post processing %
%%%%%%%%%%%%%%%%%%%%%%%%%%
%read_est_segments
%colocate_sequences





