#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "common.h"
#include "genome.h"
#include "psl2gff.h" // configuration options

int num_fixed = 0, num_total=0;
const int verbosity               = 1 ;


void filter_psl(FILE *psl_file, FILE * out_file)
{
  char line_buf[LINE_BUF_SIZE] ;
  char max_line_buf[LINE_BUF_SIZE]="" ;
  int max_match = 0 ;
  int max_match2 = 0 ;
  char *lines[10000] ;
  int line_matches[10000] ;
  int line_count=0 ;
  
  int match, mismatch, repmatch, Ns, Qgapcnt, Qgapbs, Tgapcnt, Tgapbs;
  int Qsize, Qstart, Qend, Tsize, Tstart, Tend, blockcnt ;

  char strand, strands[NAME_BUF_SIZE] ;
  
  char Qname[NAME_BUF_SIZE], Tname[NAME_BUF_SIZE], 
	  exonsizes_str[LIST_BUF_SIZE], Qstarts_str[LIST_BUF_SIZE], Tstarts_str[LIST_BUF_SIZE] ;

  char prevQname[NAME_BUF_SIZE]="" ;

  int num_considered = 0, num_best_only_passed = 0, num_best_only_filtered = 0 ;
  
  while (1)
  {
      read_line(psl_file, line_buf, LINE_BUF_SIZE) ;
	  assert(line_buf[0]!='-') ;

      if (strlen(line_buf)==0)
		  break ;
	  num_considered++ ;
	  if (!BLAT_BEST_HIT_ONLY)
	  {
		  fprintf(out_file, "%s\n", line_buf);
		  continue ;
	  }

      sscanf(line_buf, "%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%s\t%s\t"
			 "%i\t%i\t%i\t%s\t%i\t%i\t%i\t%i\t%s\t%s\t%s", 
			 &match, &mismatch, &repmatch, &Ns, &Qgapcnt, &Qgapbs, &Tgapcnt, &Tgapbs,
			 strands, Qname, &Qsize, &Qstart, &Qend, Tname, &Tsize, &Tstart, &Tend, 
			 &blockcnt,exonsizes_str,Qstarts_str,Tstarts_str) ;
	  strand=strands[0] ;

      /*fprintf(stderr, "%i;%i;%i;%i;%i;%i;%i;%i;%s;%s;"
			 "%i;%i;%i;%s;%i;%i;%i;%i;%s;%s;%s", 
			 match, mismatch, repmatch, Ns, Qgapcnt, Qgapbs, Tgapcnt, Tgapbs,
			 strands, Qname, Qsize, Qstart, Qend, Tname, Tsize, Tstart, Tend, 
			 blockcnt,exonsizes_str,Qstarts_str,Tstarts_str) ;
	  */
	  //fprintf(stderr, "Qstart=%i, Qend=%i, Qsize=%i\n", Qstart, Qend, Qsize) ;

	  int start_missing = Qstart ;
	  int end_missing = Qsize - Qend ;
	  assert(start_missing>=0) ;
	  assert(end_missing>=0) ;
	  
	  //fprintf(stdout, "match=%i, max_match=%i, Qname=%s\n", match, max_match, Qname) ;

	  if (strcmp(Qname,prevQname)==0)
	  {
		  if (match>max_match)
		  {
			  max_match2=max_match ;
			  max_match = match ;
			  strcpy(max_line_buf, line_buf) ;
		
		  }
		  if (BLAT_BEST_HIT_MARGIN<0)
		  {
			  lines[line_count] = strdup(line_buf) ;
			  line_matches[line_count] = match ;
			  line_count++ ;
			  assert(line_count<10000) ;
		  }
	  } 
	  else
	  {
		  if (BLAT_BEST_HIT_MARGIN>=0)
		  {
			  if ((strlen(max_line_buf)>0) && (((max_match-max_match2) >= BLAT_BEST_HIT_MARGIN*max_match) || (max_match2==0)))
			  {
				  fprintf(out_file, "%s\n", max_line_buf) ;
				  num_best_only_passed++ ;
			  }
			  else
				  num_best_only_filtered++ ;
		  }
		  else
		  {
			  int k ;
			  for (k=0; k<line_count; k++)
				  if ((max_match-line_matches[k]) <= -BLAT_BEST_HIT_MARGIN*max_match)
				  {
					  fprintf(out_file, "%s\n", lines[k]) ;
					  num_best_only_passed++ ;
				  }
				  else
					  num_best_only_filtered++ ;
		  }
		  
		  { // free the lines
			  int k ;
			  for (k=0; k<line_count; k++)
				  free(lines[k]) ;
			  line_count = 0 ;
		  }
		  
		  max_match = match ;
		  max_match2 = 0 ;
		  strcpy(max_line_buf, line_buf) ;
		  
		  if (BLAT_BEST_HIT_MARGIN<0)
		  {
			  lines[line_count] = strdup(line_buf) ;
			  line_matches[line_count] = match ;
			  line_count++ ;
		  }
	  } ;
	  strcpy(prevQname, Qname) ;
  } ;

  if (BLAT_BEST_HIT_MARGIN>=0)
  {
	  if ((strlen(max_line_buf)>0) && (((max_match-max_match2) >= BLAT_BEST_HIT_MARGIN*max_match) || (max_match2==0)))
	  {
		  fprintf(out_file, "%s\n", max_line_buf) ;
		  num_best_only_passed++ ;
	  }
	  else
		  num_best_only_filtered++ ;
  }
  else
  {
	  int k ;
	  for (k=0; k<line_count; k++)
		  if ((max_match-line_matches[k]) <= -BLAT_BEST_HIT_MARGIN*max_match)
		  {
			  fprintf(out_file, "%s\n", lines[k]) ;
			  num_best_only_passed++ ;
		  }
		  else
			  num_best_only_filtered++ ;
  }
  
  fprintf(stderr,"%i psl lines considered, out of %i sequences, %i passed and %i filtered\n", num_considered, num_best_only_passed+num_best_only_filtered, num_best_only_passed, num_best_only_filtered) ;
  
  { // free the lines
	  int k ;
	  for (k=0; k<line_count; k++)
		  free(lines[k]) ;
	  line_count = 0 ;
  }
} 


int main(int argc, char**argv)
{
	FILE * psl_file=NULL, *out_file=stdout ;
	char * genome_config_fname=NULL ;

	if (argc<1)
    {
		fprintf(stderr, "usage: %s <psl-file>  [<genome_config_fname]\n", argv[0]) ;
		exit(-1) ;
    } ;

	// genome config
	if (argc<3)
		genome_config_fname = "genome.config" ;
	else
		genome_config_fname = argv[2] ;
	if (init_genome(genome_config_fname)!=0)
	{
		fprintf(stderr, "genome initialization failed (%s)\n", genome_config_fname) ;
		exit(-1) ;
	}

	
	
	// psl file
	psl_file=fopen(argv[1], "r") ;
	if (!psl_file)
    {
		fprintf(stderr,"could not open %s\n", argv[1]) ;
		exit(-1) ;
    } ;

	if (BLAT_BEST_HIT_ONLY)
		fprintf(stderr, "%s v0.1 (filting best alignments by number of matches; margin=%1.1f%%)\n", argv[0], BLAT_BEST_HIT_MARGIN*100) ;
	else
		fprintf(stderr, "%s v0.1 (filtering switched off)\n", argv[0]) ;
	
	filter_psl(psl_file, out_file) ;
	fflush(stdout) ;

	cleanup_genome() ;

	return 0;
} 

