#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "common.h"
#include "genome.h"
#include "psl2gff.h" // configuration options

int num_fixed = 0, num_total=0;
const int verbosity               = 1 ;

int cDNA_mode = 0 ;
int protein_mode = 0 ;

int num_intron_fixable = 0 ;
// GT-AG: 1  GC-AG: 2  AT-AC: 3
int num_intron_type1=0, num_intron_type2=0, num_intron_type3=0 ;


FILE * microexons_file = NULL ;

struct psl_line
{
	int blockcnt ;
	char strand ;
	char *Qname ;
	int contig_num ;    
	int covered ;
	int min_pos_info, max_pos_info ;
	int *Qexonsizes, *Texonsizes, *Qstarts, *Qends, *Tstarts, *Tends, *intronsizes;
	float *ssQuality, *exonQuality ;
	int perfect_3p ;
	int perfect_5p ;
    int min_pos, max_pos ;
	int island ;
} ;

struct psl_merged
{
	int blockcnt ;
	char strand ;
	char *Qname ;
	int contig_num ;    
	int *Tstarts, *Tends ;
	int *num_exon_confirmed ; // stores the number of times each exon has been (partially) confirmed
	int *num_intron_confirmed ; // stores the number of times each intron has been confirmed
	int valid ; 
	int min_pos_info, max_pos_info ;
} ;

int psl_num = 0 ;
struct psl_line psls[PSL_SIZE] ;
int mpsl_num = 0 ;
struct psl_merged mpsls[PSL_SIZE] ;

/* convert a string list to an array */
void read_list(char *str, int *list, int targetlength, char *error_line)
{
	int i=0 ;
	char str2a[LIST_BUF_SIZE]="" ;
	char str2b[LIST_BUF_SIZE]="" ;
	strcpy(str2a,str) ;
	while (strlen(str2a)>0)
    {
		int item ;
		int ret=sscanf(str2a,"%i,%s", &item, str2b) ;
		assert(ret>0) ;
		strcpy(str2a,str2b) ;
		//fprintf(stdout, "%i,", item) ;
		list[i]=item ;
		i=i+1 ;
		if (ret!=2) break ;
		assert(i<LIST_SIZE) ;
    } ;
	if (targetlength!=i)
	{
		fprintf(stderr, "error: %s\n%s", str, error_line) ;
		exit(-1) ;
	}
} ;

/* convert a string list to an array */
void read_float_list(char *str, float *list, int targetlength, char* error_line)
{
	int i=0 ;
	char str2a[LIST_BUF_SIZE]="" ;
	char str2b[LIST_BUF_SIZE]="" ;
	strcpy(str2a,str) ;
	while (strlen(str2a)>0)
    {
		float item ;
		int ret=sscanf(str2a,"%f,%s", &item, str2b) ;
		assert(ret>0) ;
		strcpy(str2a,str2b) ;
		//fprintf(stdout, "%i,", item) ;
		list[i]=item ;
		i=i+1 ;
		if (ret!=2) break ;
		assert(i<LIST_SIZE) ;
    } ;
	if (targetlength-1==i)
	{
		int j;
		fprintf(stderr, "missing quality number ... \n") ;
		for (j=0; j<i; j++)
			list[j]=0.0 ;
		list[i]=0.0 ;
		return ;
	}
	if (targetlength!=i)
	{
		fprintf(stderr, "error: %s (%i,%i)\n%s", str,i, targetlength, error_line) ;
		exit(-1) ;
	}
} ;

int is_3p_est(struct psl_line * psl)
{
	int i ;
	int num_found_5p=0 ;
	int num_found_3p=0 ;
	int to_be_fixed=0 ;
	int Qgap = 0 ;

	for (i=0; i<psl->blockcnt-1; i++)
	{
		
		to_be_fixed=0 ;
		Qgap = psl->Qstarts[i+1] - psl->Qstarts[i] - psl->Qexonsizes[i] ;
		if (Qgap<0)
		{
			fprintf(stdout,"Qgap<0 (%i) -- bug in blat??\n", Qgap) ;
			to_be_fixed = 1 ;
			return -1 ;
		}
		
		
		assert(Qgap>=0) ;
		
		if ((Qgap>0) && (psl->intronsizes[i] == 0)) 
			to_be_fixed = 1 ;
		if ((Qgap==0) && (psl->intronsizes[i] < MIN_INTRON_LEN) && (psl->intronsizes[i]>0)) 
			to_be_fixed = 1 ;
		if (psl->intronsizes[i] > MAX_INTRON_LEN) 
			to_be_fixed = 1 ;
		if ((psl->intronsizes[i]!=0) && (Qgap!=0))
			to_be_fixed = 1 ;

		int start_diff, end_diff, ok ;
		if ((psl->intronsizes[i] >= MIN_INTRON_LEN) & (!to_be_fixed))
		{
			if (psl->strand=='+')
				ok = check_intron(psl->contig_num, psl->Tends[i]+1, 
								  psl->Tstarts[i+1]-1, psl->strand,
								  &start_diff, &end_diff) ;
			else
				ok = check_intron(psl->contig_num, psl->Tstarts[i]+1,
								  psl->Tends[i+1]-1, psl->strand,
								  &start_diff, &end_diff) ;
			if (ok!=1)
			{
				int ok2 ;
				if (psl->strand=='+')
					ok2 = check_intron_revcomp(psl->contig_num, psl->Tends[i]+1, 
											   psl->Tstarts[i+1]-1, psl->strand,
											   &start_diff, &end_diff) ;
				else
					ok2 = check_intron_revcomp(psl->contig_num, psl->Tstarts[i]+1,
											   psl->Tends[i+1]-1, psl->strand,
											   &start_diff, &end_diff) ;
				if (ok2==1)
					num_found_3p++ ;
				else
					if (ok==-1)
						num_found_5p++ ;
					else if (ok2==-1)
						num_found_3p++ ;
			} 
			else num_found_5p++ ;
		} ;
	}
	if (num_found_3p!=0)
	{
		return 1 ;
	}
	if (num_found_5p!=0)
	{
		assert(num_found_3p==0) ;
		return 0 ;
	}
	return 0 ;
}

/* remove to short or too long introns, handle gaps in the query sequence etc */
int check_psl(struct psl_line * psl, char strand, int *bad_exon_idx)
{
	int i;
	(*bad_exon_idx) = -1 ;
	
	fprintf(stdout, "check_psl: %s with %i blocks\n", psl->Qname, psl->blockcnt) ;

	// check for gaps in the alignment which are unplausible (short introns, Qgaps)
	for (i=0; i<psl->blockcnt-1; i++)
	{
		int Qgap = psl->Qstarts[i+1] - psl->Qstarts[i] - psl->Qexonsizes[i] ;
		int merge_exons = 0 ;
		
		if (psl->intronsizes[i]<0) 
		{
			fprintf(stdout,"### intron with negative size found (%i)\n", psl->intronsizes[i]) ;
			return -1 ;
		} ;
		if (Qgap<0) 
		{
			fprintf(stdout,"### negative Qgap -- bug in blat? (%i)\n", Qgap) ;
			return -1 ;
		} ;
		
		if ((Qgap==0) && 
			(psl->intronsizes[i] <= MAX_CDNA_DELETION) && 
			(psl->intronsizes[i]>0)) 
		{
			fprintf(stdout,"### found a too short intron (%i) -> looks like a short cDNA deletion\n", psl->intronsizes[i]) ;
			merge_exons = 1 ;
		} ;

		if ((Qgap>0) && (Qgap<=MAX_DOUBLE_QGAP) && 
			(psl->intronsizes[i] <= MAX_DOUBLE_TGAP) && 
			(psl->intronsizes[i]>0))
		{
			fprintf(stdout,"### found a double gap -- assuming insert in cDNA (Qgap=%i, intron=%i)\n", Qgap, psl->intronsizes[i]) ;
			merge_exons = 1 ;
		} ;
		
		if ((( (Qgap>0) && !protein_mode) || (Qgap>1) ) && 
			(psl->intronsizes[i] >= 2*MIN_INTRON_LEN+Qgap))
		{
			fprintf(stdout,"### found a double gap -- possibly a micro-exon -> don't fix it (Qgap=%i, intron=%i)\n", Qgap, psl->intronsizes[i]) ;
			fprintf(stdout,"### stuff %i, %i, %i\n", psl->Qstarts[i+1], psl->Qstarts[i], psl->Qexonsizes[i]) ;
			
			fprintf(microexons_file, "%s\t%s\t%c\t%i\t%i\t%i\t%i\n", psl->Qname, contig_names[psl->contig_num], psl->strand, 
					psl->Tstarts[i], psl->Tends[i+1], Qgap, psl->intronsizes[i]) ;
			
		} ;
		
		if ((Qgap>0) && 
			(Qgap<=MAX_CDNA_INSERTION) && 
			(psl->intronsizes[i]==0)) 
		{
			fprintf(stdout,"### found a Qgap without Tgap (Qgap=%i) -> looks like a short cDNA insertion \n", Qgap) ;
			merge_exons = 1 ;
		} ;
		
		if (merge_exons)
		{
			fprintf(stdout, "merging the two putative exons (assuming DNA is correct)\n") ;
			
			num_fixed++ ;
			int Qcorrection=psl->intronsizes[i]-Qgap ;
			
			psl->Texonsizes[i] = psl->intronsizes[i] + psl->Texonsizes[i] + psl->Texonsizes[i+1] ;
			psl->Qexonsizes[i] = Qgap + psl->Qexonsizes[i] + psl->Qexonsizes[i+1] + Qcorrection ;
			int j ;
			for (j=i; j<psl->blockcnt-1; j++)
			{
				psl->Qends[j]=psl->Qends[j+1] + Qcorrection ;
				if (strand=='+')
					psl->Tends[j]=psl->Tends[j+1] ;
				else
					psl->Tstarts[j]=psl->Tstarts[j+1] ;
				
				if (j<psl->blockcnt-2)
					psl->intronsizes[j]=psl->intronsizes[j+1] ;
				if (j>i)
				{
					psl->Qexonsizes[j]=psl->Qexonsizes[j+1] ;
					psl->Texonsizes[j]=psl->Texonsizes[j+1] ;
					
					if (strand=='+')
						psl->Tstarts[j]=psl->Tstarts[j+1] ;
					else
						psl->Tends[j]=psl->Tends[j+1] ;
					
					psl->Qstarts[j] = psl->Qstarts[j+1] + Qcorrection ;
				} ;
			} ;
			// the exon size will be correct for the DNA and incorrect for cDNA
			fprintf(stdout, "new: %s %i-%i[%i], %i-%i, intron=%i\n",  psl->Qname,
					psl->Qstarts[i], psl->Qstarts[i]+psl->Qexonsizes[i], psl->Qends[i],
					psl->Qstarts[i+1], psl->Qends[i+1],
					psl->intronsizes[i]) ;
			i-- ;
			psl->blockcnt-- ;
		}
	} 

	if (!(cDNA_mode || protein_mode) && is_3p_est(psl))
	{
		fprintf(stdout, "### %s is a 3' EST -- ignoring sequence\n", psl->Qname) ;
		return -1 ;
	}
	
	for (i=0; i<psl->blockcnt-1; i++)
	{
		int Qgap = psl->Qstarts[i+1] - psl->Qstarts[i] - psl->Qexonsizes[i] ;
		int ssQuality = psl->ssQuality[i] ;
		
		assert(Qgap>=0) ;
		
		fprintf(stdout, "%s %i-%i[%i], %i-%i, intron=%i Qgap=%i\n",  psl->Qname,
				psl->Qstarts[i], psl->Qstarts[i]+psl->Qexonsizes[i], psl->Qends[i],
				psl->Qstarts[i+1], psl->Qends[i+1],
				psl->intronsizes[i], Qgap) ;

		if ( (Qgap>0) && (psl->intronsizes[i] == 0) ) 
		{
			fprintf(stdout,"###+++ found Qgap=%i without Tgap\n", Qgap) ;
			return i+1 ;
		} ;
		if ((Qgap==0) && (psl->intronsizes[i] < MIN_INTRON_LEN) && (psl->intronsizes[i]>0)) 
		{
			fprintf(stdout,"###+++ found a too short intron (%i)\n", psl->intronsizes[i]) ;
			return i+1 ;
		} ;
		if (psl->intronsizes[i] > MAX_INTRON_LEN) 
		{
			fprintf(stdout,"###+++ found a too long intron (%i)\n", psl->intronsizes[i]) ;
			return i+1 ;
		} ;
		if (ssQuality<MIN_SSQUALITY_SCORE)
		{
			fprintf(stdout,"###+++ ssQuality score of intron %i too low: %i\n", i, ssQuality) ;
			return i+1 ;
		} ;
		
		if ((psl->intronsizes[i]!=0) && (((Qgap!=0) && !protein_mode) || (Qgap>1)) )
		{
			fprintf(stdout,"###+++ found double gap: Qgap=%i  Tgap=%i\n", Qgap, psl->intronsizes[i]) ;
			return i+1 ;
		} ;

		int start_diff, end_diff, ok ;
		if (psl->intronsizes[i] >= MIN_INTRON_LEN)
		{
			if (psl->strand=='+')
				if (protein_mode)
					ok = check_intron_protein(psl->contig_num, psl->Tends[i]+1, 
											  psl->Tstarts[i+1]-1, psl->strand,
											  &start_diff, &end_diff, Qgap) ;
				else
					ok = check_intron(psl->contig_num, psl->Tends[i]+1, 
									  psl->Tstarts[i+1]-1, psl->strand,
									  &start_diff, &end_diff) ;
			else
				if (protein_mode)
					ok = check_intron_protein(psl->contig_num, psl->Tstarts[i]+1,
											  psl->Tends[i+1]-1, psl->strand,
											  &start_diff, &end_diff, Qgap) ;
				else
					ok = check_intron(psl->contig_num, psl->Tstarts[i]+1,
									  psl->Tends[i+1]-1, psl->strand,
									  &start_diff, &end_diff) ;
			if (ok!=1)
			{
				fprintf(stdout,"found bad intron in %s[%i,%i,%i]: %i %i [%i]\n", 
						psl->Qname, i, Qgap, psl->intronsizes[i], 
						start_diff, end_diff, ok) ;
				
				if (ok==-1)
					num_intron_fixable++ ;
				const int min_flanking_exon_len = 2 ;
				
				if ((ok==-1) && ((start_diff==end_diff) || protein_mode))
				{
					if (strand=='+')
					{
						if ((psl->Tends[i]+start_diff-psl->Tstarts[i]<min_flanking_exon_len)||
							(psl->Tends[i+1]-psl->Tstarts[i+1]-end_diff<min_flanking_exon_len))
						{
							fprintf(stdout, "### resulting flanking exons too short for merge: %i %i\n",
									psl->Tends[i]+start_diff-psl->Tstarts[i],
									psl->Tends[i+1]-psl->Tstarts[i+1]-end_diff) ;
							return i+1 ;
						}
						
						fprintf(stdout, "do fix it: %s [%i,%i] -> [%i,%i]\n",
								psl->Qname,  psl->Tends[i], psl->Tstarts[i+1],
								psl->Tends[i]+start_diff, psl->Tstarts[i+1]+end_diff) ;
						psl->Tends[i] += start_diff ;
						psl->Tstarts[i+1]+= end_diff ;
						psl->intronsizes[i]=psl->Tstarts[i+1]-psl->Tends[i]-1 ;

						ok = check_intron(psl->contig_num, psl->Tends[i]+1, 
										  psl->Tstarts[i+1]-1, psl->strand,
										  &start_diff, &end_diff) ;
						assert(ok==1) ;
					}
					if (strand=='-') 
					{
						if ((psl->Tstarts[i]-start_diff-psl->Tends[i]<min_flanking_exon_len)||
							(psl->Tstarts[i+1]-psl->Tends[i+1]+start_diff<min_flanking_exon_len))
						{
							fprintf(stdout, "### resulting flanking exons too short for merge: %i %i\n",
									psl->Tstarts[i]-start_diff-psl->Tends[i],
									psl->Tstarts[i+1]-psl->Tends[i+1]+start_diff) ;
							return i+1 ;
						}
						fprintf(stdout, "do fix it: %s [%i,%i] -> [%i,%i]\n",
								psl->Qname,  psl->Tends[i+1], psl->Tstarts[i],
								psl->Tends[i+1]-start_diff, psl->Tstarts[i]-end_diff) ;
						psl->Tends[i+1] -= start_diff ;
						psl->Tstarts[i] -= end_diff ;
						psl->intronsizes[i] = psl->Tends[i+1] - psl->Tstarts[i] - 1 ;

						ok = check_intron(psl->contig_num, psl->Tstarts[i]+1,
										  psl->Tends[i+1]-1, psl->strand,
										  &start_diff, &end_diff) ;
						assert(ok==1) ;
					} ;
				} 
				else
				{
					fprintf(stdout, "intron not fixable -- force a split\n") ;
					return i+1 ;
				} ;
			} ;
		} ;
	} ;

	for (i=0; i<psl->blockcnt-1; i++)
	{
		int exonQuality = psl->exonQuality[i] ;
		if (exonQuality<MIN_EXONQUALITY_SCORE)
		{
			fprintf(stdout,"###+++ exonQuality score of exon %i too low: %i\n", i, exonQuality) ;
			(*bad_exon_idx) = i ;
			return -1 ;
		}
	}
	
	return 0 ;
} ;

/* free a psl structure */
void free_psl(struct psl_line *psl) 
{
  free(psl->Qname) ;
  free(psl->Qexonsizes) ;
  free(psl->Texonsizes) ;
  free(psl->Qstarts) ;
  free(psl->Qends) ;
  free(psl->Tstarts) ;
  free(psl->Tends) ;
  free(psl->intronsizes) ;
  free(psl->exonQuality) ;
  free(psl->ssQuality) ;
} 

void free_all_psl()
{
  int i=0 ;
  for (i=0; i<psl_num; i++)
    free_psl(&psls[i]) ;
} 

void free_mpsl(struct psl_merged *psl) 
{
  free(psl->Qname) ;
  free(psl->Tstarts) ;
  free(psl->Tends) ;
  free(psl->num_intron_confirmed) ;
  free(psl->num_exon_confirmed) ;
} 
 
void free_all_mpsl()
{
  int i=0 ;
  for (i=0; i<mpsl_num; i++)
    free_mpsl(&mpsls[i]) ;
} 

/* copy a psl structure */
void copy_psl(struct psl_line*dst, struct psl_line *src, int start)
{
  assert(start>=0) ;
  dst->blockcnt=src->blockcnt-start ; 
  dst->strand=src->strand ;

  // generate a new name
  dst->Qname=(char*)malloc(strlen(src->Qname)+6) ; 
  char *buf=strdup(src->Qname) ;
  int b=0 ;
  int ret=sscanf(src->Qname, "%[^=]=%i", buf, &b) ;
  if (ret==2)
  {
	  assert(b>0) ;
	  sprintf(dst->Qname, "%s=%i", buf, b+1) ;
  }
  else
  {
	  char l1='A', l2='A' ;
	  
	  ret=sscanf(src->Qname, "%[^-]-%c%c=%i", buf, &l1, &l2, &b) ;
	  if (ret==4)
	  {
		  assert(b>0) ;
		  sprintf(dst->Qname, "%s-%c%c=%i", buf, l1, l2, b+1) ;
	  }
	  else
	  {
		  assert(!(src->Qname[strlen(src->Qname)-1]=='1' && src->Qname[strlen(src->Qname)-2]=='=')) ;
		  sprintf(dst->Qname, "%s=1", src->Qname) ;
	  }
  }
  free(buf) ;

  // copy content
  dst->contig_num=src->contig_num ;
  dst->Qexonsizes=(int *)malloc((src->blockcnt-start)*sizeof(int)) ;
  copy_int(dst->Qexonsizes, &src->Qexonsizes[start], (src->blockcnt-start)) ;
  dst->Texonsizes=(int *)malloc((src->blockcnt-start)*sizeof(int)) ;
  copy_int(dst->Texonsizes, &src->Texonsizes[start], (src->blockcnt-start)) ;
  dst->Qstarts=(int *)malloc((src->blockcnt-start)*sizeof(int)) ;
  copy_int(dst->Qstarts, &src->Qstarts[start], (src->blockcnt-start)) ;
  dst->Qends=(int *)malloc((src->blockcnt-start)*sizeof(int)) ;
  copy_int(dst->Qends, &src->Qends[start], (src->blockcnt-start)) ;
  dst->Tstarts=(int *)calloc((src->blockcnt-start),sizeof(int)) ;
  copy_int(dst->Tstarts, &src->Tstarts[start], (src->blockcnt-start)) ;
  dst->Tends=(int *)calloc((src->blockcnt-start),sizeof(int)) ;
  copy_int(dst->Tends, &src->Tends[start], (src->blockcnt-start)) ;
  dst->intronsizes=(int *)malloc(((src->blockcnt-start)-1)*sizeof(int)) ;
  copy_int(dst->intronsizes, &src->intronsizes[start], ((src->blockcnt-start)-1)) ;

  dst->exonQuality=(float *)malloc(((src->blockcnt-start))*sizeof(float)) ;
  copy_float(dst->exonQuality, &src->exonQuality[start], ((src->blockcnt-start))) ;
  dst->ssQuality=(float *)malloc(((src->blockcnt-start)-1)*sizeof(float)) ;
  copy_float(dst->ssQuality, &src->ssQuality[start], ((src->blockcnt-start)-1)) ;

  dst->island = src->island ;
  dst->min_pos = src->min_pos ;
  dst->max_pos = src->max_pos ;
  dst->min_pos_info = src->min_pos_info ;
  dst->max_pos_info = src->max_pos_info ;
  dst->perfect_5p = src->perfect_5p ;
  dst->perfect_3p = src->perfect_3p ;
} ;

void read_psl(FILE *psl_file, int target_contig_num, char target_strand)
{
  int num_splitted = 0, num_not_taken=0, num_rejected=0 ;
  char line_buf[LINE_BUF_SIZE] ;
  int match, mismatch, repmatch, Ns, Qgapcnt, Qgapbs, Tgapcnt, Tgapbs;
  int Qsize, Qstart, Qend, Tsize, Tstart, Tend, blockcnt ;

  char strand, strands[NAME_BUF_SIZE]="" ;
  char Qname[NAME_BUF_SIZE], Tname[NAME_BUF_SIZE], 
	  exonsizes_str[LIST_BUF_SIZE], Qstarts_str[LIST_BUF_SIZE], Tstarts_str[LIST_BUF_SIZE],exonQuality_str[LIST_BUF_SIZE],ssQuality_str[LIST_BUF_SIZE] ;
  int exonsizes[LIST_SIZE], Qstarts[LIST_SIZE], Tstarts[LIST_SIZE] ;
  float exonQuality[LIST_SIZE], ssQuality[LIST_SIZE] ;
  int line_num = 0 ;
  psl_num=0 ;

  char prevQname[NAME_BUF_SIZE]="" ;
  char iQname[NAME_BUF_SIZE]="" ;
  int iQ=0 ;

  while (1)
  {
      read_line(psl_file, line_buf, LINE_BUF_SIZE) ;
	  assert(line_buf[0]!='-') ;
	  //if (psl_num>10000)
	  //return ;
	  
      if (strlen(line_buf)==0)
		  break ;
	  strcpy(exonQuality_str,"") ;
	  strcpy(ssQuality_str,"") ;
	  
      sscanf(line_buf, "%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%s\t%s\t"
			 "%i\t%i\t%i\t%s\t%i\t%i\t%i\t%i\t%s\t%s\t%s\t%s\t%s", 
			 &match, &mismatch, &repmatch, &Ns, &Qgapcnt, &Qgapbs, &Tgapcnt, &Tgapbs,
			 strands, Qname, &Qsize, &Qstart, &Qend, Tname, &Tsize, &Tstart, &Tend, 
			 &blockcnt,exonsizes_str,Qstarts_str,Tstarts_str,exonQuality_str,ssQuality_str) ;
	  if (protein_mode)
	  {
		  assert(strlen(strands)==2) ;
		  strand = strands[1] ;
	  }
	  else
	  {
		  assert(strlen(strands)==1) ;
		  strand = strands[0] ;
	  }

      if ((target_strand==strand) &&
		  (strcmp(contig_names[target_contig_num],Tname)==0))
	  {
		  int start_missing = Qstart ;
		  int end_missing = Qsize - Qend ;
		  assert(start_missing>=0) ;
		  assert(end_missing>=0) ;

		  double cover_frac = MIN_EST_COVER_FRAC ;
		  if (cDNA_mode)
			  cover_frac = MIN_CDNA_COVER_FRAC ;
		  if (protein_mode)
			  cover_frac = MIN_PROT_COVER_FRAC ;

		  if (match < cover_frac*Qsize)
		  {
			  num_not_taken+=blockcnt ;
		  }
		  else
		  {
			  if (strcmp(Qname,prevQname)==0)
			  {
				  sprintf(iQname, "%s-%c%c", Qname, 65+(iQ/26),65+(iQ%26)) ;
				  iQ++ ;
				  assert(iQ<26*26) ;
			  } else
			  {
				  sprintf(iQname, "%s", Qname) ;
				  iQ=0 ;
			  } ;
			  strcpy(prevQname,Qname) ;

			  num_total+=blockcnt ;
			  read_list(exonsizes_str, exonsizes, blockcnt, line_buf) ;
			  
			  read_list(Qstarts_str, Qstarts, blockcnt, line_buf) ;
			  read_list(Tstarts_str, Tstarts, blockcnt, line_buf) ;
			  //			  fprintf(stderr, "Qname=%s, blockcnt=%i\n", Qname, blockcnt) ;
			  if (strlen(exonQuality_str)>0)
				  read_float_list(exonQuality_str, exonQuality, blockcnt, line_buf) ;
			  else
				  {
					  int i;
					  for (i=0; i<blockcnt; i++)
						  exonQuality[i]=100.0 ;
				  }
			  if (strlen(ssQuality_str)>0)
					  read_float_list(ssQuality_str, ssQuality, blockcnt-1, line_buf) ;
			  else
			  {
				  int i ;
				  for (i=0; i<blockcnt-1; i++)
					  ssQuality[i]=100.0 ;
			  }

			  if (match>Qsize*0.98 && Qstart<=4 && Qend>=Qsize-4 && Qsize>=100)
				  psls[psl_num].perfect_5p=1 ;
			  else
				  psls[psl_num].perfect_5p=0 ;
			  if (match>Qsize*0.98 && Qstart<=25 && Qend>=Qsize-25 && Qsize>=100)
				  psls[psl_num].perfect_3p=1 ;
			  else
				  psls[psl_num].perfect_3p=0 ;

			  psls[psl_num].covered = 0 ;
			  psls[psl_num].min_pos_info = start_missing ;
			  psls[psl_num].max_pos_info = end_missing ;
			  psls[psl_num].blockcnt=blockcnt ; 
			  psls[psl_num].strand=strand ;
			  psls[psl_num].Qname=strdup(iQname) ; 
			  psls[psl_num].contig_num=target_contig_num ;
			  psls[psl_num].Qexonsizes=(int *)malloc(blockcnt*sizeof(int)) ;
			  psls[psl_num].Texonsizes=(int *)malloc(blockcnt*sizeof(int)) ;
			  copy_int(psls[psl_num].Qexonsizes,exonsizes, blockcnt) ;
			  copy_int(psls[psl_num].Texonsizes,exonsizes, blockcnt) ;
			  if (protein_mode)
			  {
				  int q=0 ;
				  for (q=0; q<blockcnt; q++)
					  psls[psl_num].Texonsizes[q]*=3 ;
				  if (strand=='-')
				  {
					  // voodoo ...
					  for (q=0; q<blockcnt; q++)
					  {
						  //fprintf(stdout, "Tstarts[%i]=%i; Tsize=%i -> %i; ", q, Tstarts[q], Tsize, Tsize-Tstarts[q]-psls[psl_num].Texonsizes[q]) ;
						  Tstarts[q]=Tsize-Tstarts[q]-psls[psl_num].Texonsizes[q] ;
					  }
					  reverse_array(Tstarts, blockcnt) ;
					  reverse_array(psls[psl_num].Texonsizes, blockcnt) ;
				  }
			  }

			  psls[psl_num].exonQuality=(float *)malloc(blockcnt*sizeof(float)) ;
			  psls[psl_num].ssQuality=(float *)malloc((blockcnt-1)*sizeof(float)) ;
			  copy_float(psls[psl_num].exonQuality, exonQuality, blockcnt) ;
			  copy_float(psls[psl_num].ssQuality, ssQuality, blockcnt-1) ;

			  psls[psl_num].Qstarts=(int *)malloc(blockcnt*sizeof(int)) ;
			  psls[psl_num].Qends=(int *)malloc(blockcnt*sizeof(int)) ;
			  copy_int(psls[psl_num].Qstarts,Qstarts, blockcnt) ;
			  psls[psl_num].Tstarts=(int *)calloc(blockcnt,sizeof(int)) ;
			  copy_int(psls[psl_num].Tstarts,Tstarts, blockcnt) ;
			  psls[psl_num].Tends=(int *)calloc(blockcnt,sizeof(int)) ;
			  psls[psl_num].intronsizes=(int *)malloc((blockcnt-1)*sizeof(int)) ;
			  psls[psl_num].min_pos = 0 ;
			  psls[psl_num].max_pos = 1024*1024*1024 ;
			  psls[psl_num].island  = -1 ;
			  
			  if (strand=='-')
			  {
				  int * buf = psls[psl_num].Tstarts ;
				  psls[psl_num].Tstarts=psls[psl_num].Tends ;
				  psls[psl_num].Tends=buf ;
			  } ;

			  int i;
			  for (i=0; i<psls[psl_num].blockcnt; i++)
			  {
				  psls[psl_num].Qends[i] = psls[psl_num].Qstarts[i] + psls[psl_num].Qexonsizes[i]-1 ;
				  if (strand=='+')
				  {
					  psls[psl_num].Tstarts[i]++ ;
					  psls[psl_num].Tends[i] = psls[psl_num].Tstarts[i] + psls[psl_num].Texonsizes[i]-1 ;
				  } 
				  else
				  {
					  psls[psl_num].Tends[i]++ ;
					  psls[psl_num].Tstarts[i] = psls[psl_num].Tends[i] + psls[psl_num].Texonsizes[i]-1 ;
				  } ;
				  
				  assert(psls[psl_num].Qexonsizes[i]>=0) ;
				  assert(psls[psl_num].Texonsizes[i]>=0) ;
			  }
			  for (i=0; i<psls[psl_num].blockcnt-1; i++)
			  {
				  if (strand=='+')
					  psls[psl_num].intronsizes[i]=psls[psl_num].Tstarts[i+1]-psls[psl_num].Tends[i]-1 ;
				  else
					  psls[psl_num].intronsizes[i]=psls[psl_num].Tends[i+1]-psls[psl_num].Tstarts[i]-1 ;
				  if (psls[psl_num].intronsizes[i]<0)
				  {
					  if (strand=='+')
					  {
						  fprintf(stdout,"Negative intron length at psl %d block %d (%i,%i,%i)\n", psl_num, i, psls[psl_num].intronsizes[i], psls[psl_num].Tstarts[i+1], psls[psl_num].Tends[i]) ;
					  }
					  else
					  {
						  fprintf(stdout,"Negative intron length at psl %d block %d (%i,%i,%i)\n", psl_num, i, psls[psl_num].intronsizes[i], psls[psl_num].Tends[i+1], psls[psl_num].Tstarts[i]) ;
					  }
					  //assert(psls[psl_num].intronsizes[i]>=0) ; 
					}
			  } ;
			  
			  int not_ok_idx=1 ;
			  while (not_ok_idx>0)
			  {
				  int bad_exon_idx = -1 ;
				  not_ok_idx=check_psl(&(psls[psl_num]),strand, &bad_exon_idx) ;
				  if (psls[psl_num].blockcnt==0)
					  assert(not_ok_idx==0) ; 
				  
				  if ((not_ok_idx==-1) && (bad_exon_idx!=-1))
				  {
					  fprintf(stdout,"exon not_ok_idx=%i\n", bad_exon_idx) ;
					  copy_psl(&psls[psl_num+1], &psls[psl_num], bad_exon_idx) ;
					  psls[psl_num].blockcnt=bad_exon_idx+1 ;
					  psls[psl_num].max_pos_info=-1 ;
					  psls[psl_num+1].min_pos_info=-1 ;
					  psls[psl_num].perfect_5p=0 ;
					  psls[psl_num].perfect_3p=0 ;
					  psls[psl_num+1].perfect_5p=0 ;
					  psls[psl_num+1].perfect_3p=0 ;	

					  // split exon into two smaller exons -- each 5nt length
					  //fprintf(stdout, "before1: %i %i %i\n", psls[psl_num].Tstarts[bad_exon_idx], psls[psl_num].Tends[bad_exon_idx], psls[psl_num].Texonsizes[bad_exon_idx]) ;
					  //fprintf(stdout, "before1a: %i %i %i\n", psls[psl_num].Tstarts[bad_exon_idx+1], psls[psl_num].Tends[bad_exon_idx+1], psls[psl_num].Texonsizes[bad_exon_idx+1]) ;
					  if (strand=='+')
					  {
						  assert(psls[psl_num].Tstarts[bad_exon_idx]<=psls[psl_num].Tends[bad_exon_idx]) ;
						  if (psls[psl_num].Tends[bad_exon_idx] - psls[psl_num].Tstarts[bad_exon_idx] > 5)
						  {
							  psls[psl_num].Tends[bad_exon_idx] = psls[psl_num].Tstarts[bad_exon_idx] + 5 ;
							  psls[psl_num].Texonsizes[bad_exon_idx] = 5 ;
							  //fprintf(stdout, "after1: %i %i %i\n", psls[psl_num].Tstarts[bad_exon_idx], psls[psl_num].Tends[bad_exon_idx], psls[psl_num].Texonsizes[bad_exon_idx]) ;
						  }
						  
						  //fprintf(stdout, "before2: %i %i %i\n", psls[psl_num+1].Tstarts[0], psls[psl_num+1].Tends[0], psls[psl_num+1].Texonsizes[0]) ;
						  assert(psls[psl_num+1].Tstarts[bad_exon_idx]<=psls[psl_num+1].Tends[bad_exon_idx]) ;
						  if (psls[psl_num+1].Tends[0] - psls[psl_num+1].Tstarts[0] > 5)
						  {
							  psls[psl_num+1].Tstarts[0] = psls[psl_num+1].Tends[0] - 5 ;
							  psls[psl_num+1].Texonsizes[0] = 5 ;
							  //fprintf(stdout, "after2: %i %i %i\n", psls[psl_num+1].Tstarts[0], psls[psl_num+1].Tends[0], psls[psl_num+1].Texonsizes[0]) ;
						  }
					  }
					  else
					  {
						  assert(psls[psl_num].Tstarts[bad_exon_idx]>=psls[psl_num].Tends[bad_exon_idx]) ;
						  if (psls[psl_num].Tstarts[bad_exon_idx] - psls[psl_num].Tends[bad_exon_idx] > 5)
						  {
							  psls[psl_num].Tstarts[bad_exon_idx] = psls[psl_num].Tends[bad_exon_idx] + 5 ;
							  psls[psl_num].Texonsizes[bad_exon_idx] = 5 ;
							  //fprintf(stdout, "after1: %i %i %i\n", psls[psl_num].Tstarts[bad_exon_idx], psls[psl_num].Tends[bad_exon_idx], psls[psl_num].Texonsizes[bad_exon_idx]) ;
						  }
						  
						  //fprintf(stdout, "before2: %i %i %i\n", psls[psl_num+1].Tstarts[0], psls[psl_num+1].Tends[0], psls[psl_num+1].Texonsizes[0]) ;
						  assert(psls[psl_num+1].Tstarts[bad_exon_idx]>=psls[psl_num+1].Tends[bad_exon_idx]) ;
						  if (psls[psl_num+1].Tstarts[0] - psls[psl_num+1].Tends[0] > 5)
						  {
							  psls[psl_num+1].Tends[0] = psls[psl_num+1].Tstarts[0] - 5 ;
							  psls[psl_num+1].Texonsizes[0] = 5 ;
							  //fprintf(stdout, "after2: %i %i %i\n", psls[psl_num+1].Tstarts[0], psls[psl_num+1].Tends[0], psls[psl_num+1].Texonsizes[0]) ;
						  }
					  }
					  //fprintf(stdout, "s1: %i, s2: %i\n", psls[psl_num].blockcnt, psls[psl_num+1].blockcnt) ;
					  not_ok_idx = 1 ; 
				  } ;

				  if (not_ok_idx>0)
				  {
					  fprintf(stdout,"not_ok_idx=%i\n", not_ok_idx) ;
					  copy_psl(&psls[psl_num+1], &psls[psl_num], not_ok_idx) ;
					  psls[psl_num].blockcnt=not_ok_idx ;
					  psls[psl_num].max_pos_info=-1 ;
					  psls[psl_num+1].min_pos_info=-1 ;
					  psls[psl_num].perfect_5p=0 ;
					  psls[psl_num].perfect_3p=0 ;
					  psls[psl_num+1].perfect_5p=0 ;
					  psls[psl_num+1].perfect_3p=0 ;	
				  } ;
				  				  
				  if ((not_ok_idx>=0) && (psls[psl_num].blockcnt>0))
				  {
					  if (strand=='-')
					  {
						  reverse_array(psls[psl_num].Tstarts, psls[psl_num].blockcnt) ;
						  reverse_array(psls[psl_num].Tends, psls[psl_num].blockcnt) ;
						  reverse_array(psls[psl_num].Texonsizes, psls[psl_num].blockcnt) ;
						  reverse_array(psls[psl_num].intronsizes, psls[psl_num].blockcnt-1) ;
						  reverse_array_float(psls[psl_num].exonQuality, psls[psl_num].blockcnt-1) ;
						  reverse_array_float(psls[psl_num].ssQuality, psls[psl_num].blockcnt-1) ;
					  } ;
					  if (not_ok_idx>0) 
					  {
						  num_splitted++ ;
						  fprintf(stdout, "dosplit (%i,%i,%i)\n", not_ok_idx, blockcnt, psls[psl_num].blockcnt) ;
					  }
					  else
						  if (blockcnt!=psls[psl_num].blockcnt)
						  {
							  num_fixed++ ;
							  fprintf(stdout, "psl was fixed (%i,%i,%i)\n", not_ok_idx, blockcnt, psls[psl_num].blockcnt) ;
						  }
					  
					  psl_num=psl_num+1 ;
					  assert(psl_num<PSL_SIZE) ;
				  }
				  else
				  {
					  num_rejected++ ;
					  free_psl(&psls[psl_num]) ;
				  };
			  } ;
		  } ;
	  } ;
      line_num++ ;
  } ;
  fprintf(stdout,"accepted %i psl lines, (%i taken, %i fixed, %i splitted, %i intron_fixable, %i rejected, %i not taken)\n", psl_num, num_total, num_fixed, num_splitted, num_intron_fixable, num_rejected, num_not_taken) ;
} ;

int check_mismatch(int num1, struct psl_line * psl) 
{
  int i,j=0 ;
  int pos1, pos2 ;
  int found = 0 ;
  
  for (i=0; i < psls[num1].blockcnt; i++) 
    {
      for (j=0; j < psl->blockcnt; j++)
	{
	  if ((psls[num1].Tstarts[i] == psl->Tstarts[j]) ||
	      (psls[num1].Tends[i] == psl->Tends[j]))
	    {
	      found = 1 ;
	      break ;
	    } ;
	} ;
      if (found)
	break ;
    } ;
  if (!found)
    return -1 ; // no overlap at all
  pos1=i ;
  pos2=j ;
  i=0 ;
  while ((pos1+i<psls[num1].blockcnt) && (pos2+i<psl->blockcnt))
  {
	  if ((pos1+i<psls[num1].blockcnt-1) && (pos2+i<psl->blockcnt-1))
	  {
		  if ((psls[num1].Tstarts[pos1+i]!=psl->Tstarts[pos2+i]) ||
			  (psls[num1].Tends[pos1+i]!=psl->Tends[pos2+i]))
			  return 1 ;
	  }
	  else if (pos1+i<psls[num1].blockcnt-1) // last exon in sequence 2
	  {
		  if (psls[num1].strand=='+') 
		  {
			  if (psls[num1].Tstarts[pos1+i]!=psl->Tstarts[pos2+i])
				  return 1 ;
			  if ((psls[num1].Tends[pos1+i]+TERMINAL_EXON_END_TOL<psl->Tends[pos2+i]))
				  return 1 ;
		  }
		  else
		  {
			  if (psls[num1].Tstarts[pos1+i]!=psl->Tstarts[pos2+i])
				  return 1 ;
			  if ((psls[num1].Tends[pos1+i]-TERMINAL_EXON_END_TOL>psl->Tends[pos2+i]))
				  return 1 ;
		  }
		  //fprintf(stdout, "accepted match [%i,%i,%i/%i] & [%i,%i,%i/%i]\n", psls[num1].Tstarts[pos1+i], psls[num1].Tends[pos1+i], pos1+i+1, psls[num1].blockcnt, psl->Tstarts[pos2+i], psl->Tends[pos2+i], pos2+i+1, psl->blockcnt) ;
	  }
	  else if (pos2+i<psl->blockcnt-1) // last exon in sequence 1
	  {
		  if (psls[num1].strand=='+')
		  {
			  if (psls[num1].Tstarts[pos1+i]!=psl->Tstarts[pos2+i]) 
				  return 1 ;
			  if (psls[num1].Tends[pos1+i]>psl->Tends[pos2+i]+TERMINAL_EXON_END_TOL)
				  return 1 ;
		  }
		  else
		  {
			  if (psls[num1].Tstarts[pos1+i]!=psl->Tstarts[pos2+i]) 
				  return 1 ;
			  if (psls[num1].Tends[pos1+i]<psl->Tends[pos2+i]-TERMINAL_EXON_END_TOL)
				  return 1 ;
		  }
		  //fprintf(stdout, "accepted match [%i,%i,%i/%i] & [%i,%i,%i/%i]\n", psls[num1].Tstarts[pos1+i], psls[num1].Tends[pos1+i], pos1+i+1, psls[num1].blockcnt, psl->Tstarts[pos2+i], psl->Tends[pos2+i], pos2+i+1,psl->blockcnt) ;
	  } 
	  else // dangling ends
	  {
		  if (psls[num1].Tstarts[pos1+i]!=psl->Tstarts[pos2+i]) 
			  return 1 ;
	  }
      i++ ;
  } ;
  
  i=1 ;
  while ((pos1-i>=0) && (pos2-i>=0))
  {
	  if ((pos1-i>0) && (pos2-i>0))
	  {
		  if ((psls[num1].Tstarts[pos1-i]!=psl->Tstarts[pos2-i]) ||
			  (psls[num1].Tends[pos1-i]!=psl->Tends[pos2-i]))
			  return 1 ;
	  }
	  else if (pos1-i>0) // first exon in sequence 2
	  {
		  if (psls[num1].strand=='+')
		  {
			  if (psls[num1].Tends[pos1-i]!=psl->Tends[pos2-i])
				  return 1 ;
			  if (psls[num1].Tstarts[pos1-i]-TERMINAL_EXON_END_TOL>psl->Tstarts[pos2-i])
				  return 1 ;
		  }
		  else
		  {
			  if (psls[num1].Tends[pos1-i]!=psl->Tends[pos2-i])
				  return 1 ;
			  if (psls[num1].Tstarts[pos1-i]+TERMINAL_EXON_END_TOL<psl->Tstarts[pos2-i])
				  return 1 ;
		  }
		  //fprintf(stdout, "accepted match [%i,%i,%i/%i] & [%i,%i,%i/%i]\n", psls[num1].Tstarts[pos1-i], psls[num1].Tends[pos1-i], pos1-i+1, psls[num1].blockcnt, psl->Tstarts[pos2-i], psl->Tends[pos2-i], pos2-i+1,psl->blockcnt) ;
	  }
	  else if (pos2-i>0) // first exon in sequence 1
	  {
		  if (psls[num1].strand=='+')
		  {
			  if (psls[num1].Tends[pos1-i]!=psl->Tends[pos2-i])
				  return 1 ;
			  if (psls[num1].Tstarts[pos1-i]<psl->Tstarts[pos2-i]-TERMINAL_EXON_END_TOL)
				  return 1 ;
		  }
		  else
		  {
			  if (psls[num1].Tends[pos1-i]!=psl->Tends[pos2-i])
				  return 1 ;
			  if (psls[num1].Tstarts[pos1-i]>psl->Tstarts[pos2-i]+TERMINAL_EXON_END_TOL)
				  return 1 ;
		  }
		  //fprintf(stdout, "accepted match [%i,%i,%i/%i] & [%i,%i,%i/%i]\n", psls[num1].Tstarts[pos1-i], psls[num1].Tends[pos1-i], pos1-i+1, psls[num1].blockcnt, psl->Tstarts[pos2-i], psl->Tends[pos2-i], pos2-i+1,psl->blockcnt) ;
	  } 
	  else // dangling start
	  {
		  if (psls[num1].Tends[pos1-i]!=psl->Tends[pos2-i])
			  return 1 ;
	  } ;
      i++ ;
  } ;

  return 0 ;
} 

void update_minmax(struct psl_line * psl)
{
	psl->min_pos=1024*1024*1024 ;
	psl->max_pos=0 ;
	int j ;
	for (j=0; j<psl->blockcnt; j++)
	{
		//fprintf(stderr,"block %i-%i\n", psl->Tstarts[j], psl->Tends[j]) ;
		if (psl->Tstarts[j]>psl->max_pos)
			psl->max_pos=psl->Tstarts[j] ;
		if (psl->Tstarts[j]<psl->min_pos)
			psl->min_pos=psl->Tstarts[j] ;
		if (psl->Tends[j]>psl->max_pos)
			psl->max_pos=psl->Tends[j] ;
		if (psl->Tends[j]<psl->min_pos)
			psl->min_pos=psl->Tends[j] ;
	}
	assert(psl->blockcnt!=0) ;
	//fprintf(stderr, "range %i-%i\n", psl->min_pos, psl->max_pos) ;
}

/* finds a list of psls, that match the current psl (overlap, but no mismatch) */
int find_match(struct psl_line * psl, int * found, int *island_list, int island_list_len) 
{
	int i,j,k ;
	int num_match = 0 ;
	const int min_pos=psl->min_pos, max_pos=psl->max_pos ;

	int ii;
	for (ii=0; ii<island_list_len; ii++)
	{
		i=island_list[ii] ;
		
		int match = 0 ;
		const int min_posi=psls[i].min_pos, max_posi=psls[i].max_pos ;
		
		if ((max_posi<min_pos) || (min_posi>max_pos))
			continue ;

		for (j=0; j<psls[i].blockcnt; j++)
		{
			// exact exon matches 
			for (k=0; k<psl->blockcnt; k++)
			{
				if ((psl->Tstarts[k]==psls[i].Tstarts[j]) && (psl->Tends[k]==psls[i].Tends[j]))
				{
					int ret = check_mismatch(i,psl) ;
					if (ret<=0)
						match=1 ;
				} ;
			} ;
			// exact intron matches 
			if (j<psls[i].blockcnt-1)
				for (k=0; k<psl->blockcnt-1; k++)
				{
					if ((psl->Tstarts[k+1]==psls[i].Tstarts[j+1]) && (psl->Tends[k]==psls[i].Tends[j]))
					{
						int ret = check_mismatch(i,psl) ;
						if (ret<=0)
							match=1 ;
					} ;
				} ;
		} ;
		if (match)
		{
			found[num_match]=i ;
			num_match++ ;
			assert(num_match<FOUND_BUF_SIZE) ;
		} ;
    }

	return num_match ;
} ;


/* merges a list of psls, puts the exon in right order and performs a few checks */
int merge_contigous(int idx, int num, int *found, struct psl_merged *mpsl) 
{
	char buf[1000] ;
	if (num>1)
		sprintf(buf, "%sx%i", psls[idx].Qname, num) ;
	else
		sprintf(buf, "%s", psls[idx].Qname) ;
	mpsl->Qname = strdup(buf) ;
	mpsl->contig_num = psls[idx].contig_num ;
	mpsl->strand = psls[idx].strand ;

	int min_pos_info=-2 ;
	int max_pos_info=-2 ;

	int *Tstarts=malloc(FOUND_BUF_SIZE*sizeof(int)), 
		*Tends=malloc(FOUND_BUF_SIZE*sizeof(int)),
		*num_exon_confirmed=malloc(FOUND_BUF_SIZE*sizeof(int)),
		*num_intron_confirmed=malloc(FOUND_BUF_SIZE*sizeof(int)) ;
	int i, blockcnt=0 ;
	assert(num>0) ;
	memset(num_exon_confirmed, FOUND_BUF_SIZE*sizeof(int),0) ;
	
	for (i=0; i<num; i++) 
    {
		int j,k ;

		psls[found[i]].covered ++ ;
		for (j=0; j<psls[found[i]].blockcnt; j++)
		{
			int already_there = 0 ;
			//fprintf(stderr,"%i. %i - %i\n", j, psls[found[i]].Tstarts[j], psls[found[i]].Tends[j]) ;
			
			for (k=0; k<blockcnt; k++)
				if ((psls[found[i]].Tstarts[j]==Tstarts[k]) || 
					(psls[found[i]].Tends[j]==Tends[k]))
				{
					if (mpsl->strand=='+')
					{
						// dangling end
						if ((psls[found[i]].Tstarts[j]==Tstarts[k]) && 
							(psls[found[i]].Tends[j]!=Tends[k]))
						{
							if ((k==blockcnt-1) && (j==psls[found[i]].blockcnt-1))
							{
								if (psls[found[i]].Tends[j]>Tends[k])
									max_pos_info=psls[found[i]].max_pos_info ;
								Tends[k]=max(Tends[k],psls[found[i]].Tends[j]) ;
							}
							else if (k==blockcnt-1)
							{
								Tends[k]=psls[found[i]].Tends[j] ;
								max_pos_info=psls[found[i]].max_pos_info ;
							}
							else if (j==psls[found[i]].blockcnt-1)
								Tends[k]=Tends[k] ;
							else
								assert(0) ;
						}
						// dangling start
						if ((psls[found[i]].Tstarts[j]!=Tstarts[k]) && 
							(psls[found[i]].Tends[j]==Tends[k]))
						{
							if ((k==0) && (j==0))
							{
								if (psls[found[i]].Tstarts[j]<Tstarts[k])
									min_pos_info=psls[found[i]].min_pos_info ;
								Tstarts[k]=min(Tstarts[k],psls[found[i]].Tstarts[j]) ;
							}
							else if (k==0)
							{
								min_pos_info=psls[found[i]].min_pos_info ;
								Tstarts[k]=psls[found[i]].Tstarts[j] ;
							}
							else if (j==0)
								Tstarts[k]=Tstarts[k] ;
							else
								assert(0) ;
						}
					}
					else
					{
						// dangling end
						if ((psls[found[i]].Tstarts[j]==Tstarts[k]) && 
							(psls[found[i]].Tends[j]!=Tends[k]))
						{
							if ((k==0) && (j==psls[found[i]].blockcnt-1))
							{
								if (psls[found[i]].Tends[j]<Tends[k])
									min_pos_info = psls[found[i]].min_pos_info ;
								Tends[k]=min(Tends[k],psls[found[i]].Tends[j]) ;
							}
							else if (k==0)
							{
								min_pos_info = psls[found[i]].min_pos_info ;
								Tends[k]=psls[found[i]].Tends[j] ;
							}
							else if (j==psls[found[i]].blockcnt-1)
								Tends[k]=Tends[k] ;
							else
								assert(0) ;
						}
						// dangling start
						if ((psls[found[i]].Tstarts[j]!=Tstarts[k]) && 
							(psls[found[i]].Tends[j]==Tends[k]))
						{
							if ((k==blockcnt-1) && (j==0))
							{
								if (psls[found[i]].Tstarts[j]>Tstarts[k])
									max_pos_info=psls[found[i]].max_pos_info ;
								Tstarts[k]=max(Tstarts[k],psls[found[i]].Tstarts[j]) ;
							}
							else if (k==blockcnt-1)
							{
								max_pos_info=psls[found[i]].max_pos_info ;
								Tstarts[k]=psls[found[i]].Tstarts[j] ;
							}
							else if (j==0)
								Tstarts[k]=Tstarts[k] ;
							else
                                assert(0) ;
 						}
					}
					already_there = 1 ;
					break ;
				} ;
			if (already_there) // insert at right position
			{
				num_exon_confirmed[k]++ ;
				//fprintf(stdout, "already there %i\n",k) ;
			}
			else
			{
				int insert_idx = blockcnt ;
				for (k=0; k<blockcnt; k++) 
				{
					if (psls[found[i]].Tstarts[j]<Tstarts[k])
					{
						// sanity checks
						assert(!(psls[found[i]].Tends[j]>Tends[k])) ;
						assert(!((psls[found[i]].Tends[j]<Tends[k]) &&
								 (psls[found[i]].Tends[j]>Tstarts[k]))) ;
						insert_idx = k ;
						break ;
					} ;
					// more sanity checks
					if (psls[found[i]].Tends[j]>Tends[k])
					{
						assert(!(psls[found[i]].Tstarts[j]<Tstarts[k])) ;
						assert(!((psls[found[i]].Tstarts[j]<Tends[k]) &&
								 (psls[found[i]].Tstarts[j]>Tstarts[k]))) ;
					} ;
				} ;
				
				if (insert_idx==0)
					min_pos_info = psls[found[i]].min_pos_info ;

				if (insert_idx == blockcnt)
				{
					max_pos_info = psls[found[i]].max_pos_info ;
					
					Tstarts[blockcnt]=psls[found[i]].Tstarts[j] ;
					Tends[blockcnt]=psls[found[i]].Tends[j] ;
					num_exon_confirmed[blockcnt] = 1 ;
					//fprintf(stdout, "append %i - %i \n", psls[found[i]].Tstarts[j], psls[found[i]].Tends[j]) ;
					blockcnt ++ ;
				}
				else
				{
					/*if (insert_idx!=0)
					{
						int k ;
						for (k=0; k<blockcnt; k++)
							fprintf(stderr,"%i. [%i-%i] {%i}\n", k, Tstarts[k], Tends[k], num_exon_confirmed[k]) ;
						for (k=0; k<psls[found[i]].blockcnt; k++)
							fprintf(stderr,"%i. [%i-%i]\n", k, psls[found[i]].Tstarts[k], psls[found[i]].Tends[k]) ;
							}*/
					
					// move and insert 
					copy_int(&Tstarts[insert_idx+1], &Tstarts[insert_idx], (blockcnt-insert_idx)) ;
					copy_int(&Tends[insert_idx+1], &Tends[insert_idx], (blockcnt-insert_idx)) ;
					copy_int(&num_exon_confirmed[insert_idx+1], &num_exon_confirmed[insert_idx], (blockcnt-insert_idx)) ;
					Tstarts[insert_idx] = psls[found[i]].Tstarts[j] ;
					Tends[insert_idx]   = psls[found[i]].Tends[j] ;
					//fprintf(stdout, "insert at %i: %i - %i \n", insert_idx, psls[found[i]].Tstarts[j], psls[found[i]].Tends[j]) ;
					num_exon_confirmed[insert_idx] = 1 ;

					/*if (insert_idx!=0)
					{
						fprintf(stderr, "insert_idx!=0\n") ;
						exit(-1) ;
						} ;*/
					
					blockcnt++ ;
				} ;
			} ;
		} ;
    } ;

	/* check for overlaps in resulting sequence */
	for (i=0; i<blockcnt-1; i++)
    {
		assert(Tends[i]<Tstarts[i+1]) ;
		if (mpsl->strand=='+')
		{
			if (Tstarts[i]>Tends[i])
			{
				int j ;
				
				for (j=0; j<blockcnt; j++)
				{
					fprintf(stderr, "### %i - %i\n", Tstarts[j], Tends[j]);
				} ;
				fprintf(stderr, "### %s start=%i end=%i ???\n", mpsl->Qname, Tstarts[i], Tends[i]) ;
				exit(-1) ;
			}
		}
		else
		{
			if (Tstarts[i]<Tends[i])
			{
				int j ;
				
				for (j=0; j<blockcnt; j++)
				{
					fprintf(stderr, "### %i - %i\n", Tstarts[j], Tends[j]);
				} ;
				fprintf(stderr, "### %s start=%i end=%i ???\n", mpsl->Qname, Tstarts[i], Tends[i]) ;
				exit(-1) ;
			}
		}
    } ;

	for (i=0; i<blockcnt-1; i++)
		num_intron_confirmed[i]=1 ;
		
	for (i=0; i<num; i++) 
    {
		int j,k ;
		for (j=0; j<psls[found[i]].blockcnt-1; j++)
		{
			if (found[i]!=idx)
				for (k=0; k<blockcnt-1; k++)
					if ((psls[found[i]].Tstarts[j+1]==Tstarts[k+1]) || 
						(psls[found[i]].Tends[j]==Tends[k]))
					{
						//assert(psls[found[i]].Tstarts[j+1]==Tstarts[k+1]) ;
						//assert(psls[found[i]].Tends[j]==Tends[k]) ;
						num_intron_confirmed[k]++ ;
						break ;
					} ;
		}
	}
	
	mpsl->blockcnt = blockcnt ;
	mpsl->Tstarts = (int*)malloc(sizeof(int)*blockcnt) ;
	copy_int(mpsl->Tstarts, Tstarts, blockcnt) ;
	mpsl->Tends = (int*)malloc(sizeof(int)*blockcnt) ;
	copy_int(mpsl->Tends, Tends, blockcnt) ;
	mpsl->num_exon_confirmed = (int*)malloc(sizeof(int)*blockcnt) ;
	copy_int(mpsl->num_exon_confirmed, num_exon_confirmed, blockcnt) ;
	mpsl->num_intron_confirmed = (int*)malloc(sizeof(int)*(blockcnt-1)) ;
	copy_int(mpsl->num_intron_confirmed, num_intron_confirmed, (blockcnt-1)) ;

	assert(min_pos_info!=-2) ;
	assert(max_pos_info!=-2) ;
	mpsl->min_pos_info=min_pos_info ;
	mpsl->max_pos_info=max_pos_info ;
	
	mpsl->valid = 1 ;
	
	free(Tstarts) ;
	free(Tends) ;
	free(num_exon_confirmed) ;
	free(num_intron_confirmed) ;

	return blockcnt ;
} 

int check_mpsl(struct psl_merged * mpsl)
{
	if (!mpsl->valid)
		return 0 ;
	if (mpsl->blockcnt<=1)
		return 0 ;
	return 1 ;
}


void print_psl(struct psl_line * psl, FILE* fd, int psl_idx)
{
	int i ;
	char prefix[10]="" ;
	if (cDNA_mode)
		strcpy(prefix, "GR") ;
	if (protein_mode)
		strcpy(prefix, "PR") ;
	
	/* output all exons */
	for (i=0; i<psl->blockcnt; i++) 
	{
		if (psl->strand=='+')
		{
			fprintf(fd,"%s\tCoding\texon\t%i\t%i\t%c\t%i\t%i\t%s-%s-%i%c.%i\t%i\t%i\t%i\t%i\n", 
					contig_names[psl->contig_num], 
					psl->Tstarts[i], psl->Tends[i], psl->strand, 
					1 /* num_exon_confirmed */, (i<psl->blockcnt-1) /* num_intron_confirmed */,
					prefix, psl->Qname, psl->contig_num, psl->strand, psl_idx, 
					i, psl->blockcnt, psl->min_pos_info, psl->max_pos_info) ;
		}
		else	
		{
			fprintf(fd,"%s\tCoding\texon\t%i\t%i\t%c\t%i\t%i\t%s-%s-%i%c.%i\t%i\t%i\t%i\t%i\n", contig_names[psl->contig_num], 
					psl->Tends[i], psl->Tstarts[i], psl->strand, 
					1 /* num_exon_confirmed */, (i<psl->blockcnt-1) /* num_intron_confirmed */,
					prefix, psl->Qname, psl->contig_num, psl->strand, psl_idx, 
					i, psl->blockcnt, psl->min_pos_info, psl->max_pos_info) ;
		}
	}
	
	/* output all introns */
	for (i=0; i<psl->blockcnt-1; i++) 
	{
		int start_diff=0 ;
		int end_diff = 0 ;
		int ok = 0 ;
		
		if (psl->strand=='+')
			ok = check_intron(psl->contig_num, psl->Tends[i]+1, 
							  psl->Tstarts[i+1]-1, psl->strand,
							  &start_diff, &end_diff) ;
		else
			ok = check_intron(psl->contig_num, psl->Tstarts[i]+1,
							  psl->Tends[i+1]-1, psl->strand,
							  &start_diff, &end_diff) ;

		if (ok!=1)
		{
			fprintf(stderr,"intron not ok: %i [%i,%i] -- should not happen\n", ok, start_diff, end_diff) ;
			//exit(-1) ;
		} ;
		
		int intron_type ;
		
		if (psl->strand=='+')
			intron_type = get_intron_type(psl->contig_num, psl->Tends[i]+1, 
							  psl->Tstarts[i+1]-1, psl->strand) ;
		else
			intron_type = get_intron_type(psl->contig_num, psl->Tstarts[i]+1,
							  psl->Tends[i+1]-1, psl->strand) ;

		if (intron_type==1)
			num_intron_type1++ ;
		if (intron_type==2)
			num_intron_type2++ ;
		if (intron_type==3)
			num_intron_type3++ ;
		
		if (psl->strand=='+')
		{
			fprintf(fd,"%s\t%s\tintron\t%i\t%i\t%c\t%i\t%i\t%s-%s-%i%c.%i\t%i\t%i\t%i\t%i\n", 
					contig_names[psl->contig_num], psl->Qname, 
					psl->Tends[i]+1, psl->Tstarts[i+1]-1, psl->strand, 
					0 /* num_exon_confirmed */, 1 /* num_intron_confirmed */, 
					prefix, psl->Qname, psl->contig_num, psl->strand, psl_idx, 
					i, psl->blockcnt-1, psl->min_pos_info, psl->max_pos_info) ;
		}
		else
		{			
			fprintf(fd,"%s\t%s\tintron\t%i\t%i\t%c\t%i\t%i\t%s-%s-%i%c.%i\t%i\t%i\t%i\t%i\n", 
					contig_names[psl->contig_num], psl->Qname, 
					psl->Tstarts[i]+1, psl->Tends[i+1]-1, psl->strand, 
					0 /* num_exon_confirmed */, 1 /* num_intron_confirmed */,
					prefix, psl->Qname, psl->contig_num, psl->strand, psl_idx, 
					i, psl->blockcnt-1, psl->min_pos_info, psl->max_pos_info) ;
		}
	} ;
	
}

void print_est_matches(struct psl_line * psl, FILE* fd, int psl_idx)
{
	if (psl->strand=='+')
	{
		if (psl->perfect_3p)
		{
			char *region=NULL ;
			if (get_intron(psl->contig_num, psl->Tends[psl->blockcnt-1]-10, psl->Tends[psl->blockcnt-1]+50, psl->strand, &region))
			{
				int i ;
				for (i=0; i<60; i++)
				{
					int j ;
					int num_a=0 ;
					for (j=i; j<60 && j<i+10; j++)
						if (region[j]==alph[0])
						{
							num_a++ ;
							if (j-i==5 && num_a==6)
								psl->perfect_3p = 0 ;
						}
					if (num_a>=8)
						psl->perfect_3p = 0 ;
				}
				free(region) ;
			}
		} else
			psl->perfect_3p = 0 ;
		
		assert(psl->Tends[psl->blockcnt-1]-psl->Tstarts[0]>=0) ;
		assert(psl->Tends[psl->blockcnt-1]-psl->Tstarts[0]<2000000) ;
		
		if (psl->Tends[psl->blockcnt-1]-psl->Tstarts[0]>MAX_PERFECT_EST_MATCH_LEN)
		{
			psl->perfect_3p = 0 ;
			psl->perfect_5p = 0 ;
		}
		fprintf(fd,"%s\tmatching\tEST\t%i\t%i\t%c\t%s\t%i\t%i\t%i\n", 
				contig_names[psl->contig_num], 
				psl->Tstarts[0], psl->Tends[psl->blockcnt-1], psl->strand, 
				psl->Qname, psl->blockcnt, psl->perfect_5p, psl->perfect_3p) ;
			
	}
	else
	{
		if (psl->perfect_3p)
		{
			char *region=NULL ;
			if (get_intron(psl->contig_num, psl->Tends[0]-50, psl->Tends[0]+10, psl->strand, &region))
			{
				int i ;
				for (i=0; i<60; i++)
				{
					int j ;
					int num_a=0 ;
					for (j=i; j<60 && j<i+10; j++)
						if (region[j]==alph[0])
						{
							num_a++ ;
							if (j-i==5 && num_a==6)
								psl->perfect_3p = 0 ;
						}
					if (num_a>=8)
						psl->perfect_3p = 0 ;
				}
				free(region) ;
			} else
				psl->perfect_3p = 0 ;
		}
		
		assert(psl->Tstarts[psl->blockcnt-1]-psl->Tends[0]>=0) ;
		assert(psl->Tstarts[psl->blockcnt-1]-psl->Tends[0]<2000000) ;
		
		if (psl->Tstarts[psl->blockcnt-1]-psl->Tends[0]>MAX_PERFECT_EST_MATCH_LEN)
		{
			psl->perfect_3p = 0 ;
			psl->perfect_5p = 0 ;
		}

		fprintf(fd,"%s\tmatching\tEST\t%i\t%i\t%c\t%s\t%i\t%i\t%i\n", 
				contig_names[psl->contig_num], 
				psl->Tends[0], psl->Tstarts[psl->blockcnt-1], psl->strand, 
				psl->Qname, psl->blockcnt, psl->perfect_5p, psl->perfect_3p) ;
		
	}
}


void print_mpsl(struct psl_merged * mpsl, FILE* fd, int psl_idx)
{
	if (!mpsl->valid)
		return ;
	if (!check_mpsl(mpsl))
		return ;

	int i ;
	/* check all exons */
	for (i=0; i<mpsl->blockcnt; i++) 
	{
		int num_intron_confirmed = 0 ;
		if (i<mpsl->blockcnt-1)
			num_intron_confirmed=mpsl->num_intron_confirmed[i] ;

		if (mpsl->strand=='+')
		{
			fprintf(fd,"%s\tCoding\texon\t%i\t%i\t%c\t%i\t%i\tEST-%s-%i%c.%i\t%i\t%i\t%i\t%i\n", 
					contig_names[mpsl->contig_num], 
					mpsl->Tstarts[i], mpsl->Tends[i], mpsl->strand, 
					mpsl->num_exon_confirmed[i], num_intron_confirmed,
					mpsl->Qname, mpsl->contig_num, mpsl->strand, psl_idx, i, mpsl->blockcnt,
					mpsl->min_pos_info, mpsl->max_pos_info) ;
		}
		else	
		{
			fprintf(fd,"%s\tCoding\texon\t%i\t%i\t%c\t%i\t%i\tEST-%s-%i%c.%i\t%i\t%i\t%i\t%i\n", contig_names[mpsl->contig_num], 
					mpsl->Tends[i], mpsl->Tstarts[i], mpsl->strand, 
					mpsl->num_exon_confirmed[i], num_intron_confirmed, 
					mpsl->Qname, mpsl->contig_num, mpsl->strand, psl_idx, i, mpsl->blockcnt,
					mpsl->min_pos_info, mpsl->max_pos_info) ;
		}
	}

	/* output all introns */
	for (i=0; i<mpsl->blockcnt-1; i++) 
	{
		int start_diff=0 ;
		int end_diff = 0 ;
		int ok = 0 ;
		
		if (mpsl->strand=='+')
			ok = check_intron(mpsl->contig_num, mpsl->Tends[i]+1, 
							  mpsl->Tstarts[i+1]-1, mpsl->strand,
							  &start_diff, &end_diff) ;
		else
			ok = check_intron(mpsl->contig_num, mpsl->Tstarts[i]+1,
							  mpsl->Tends[i+1]-1, mpsl->strand,
							  &start_diff, &end_diff) ;
		if (ok!=1)
		{
			fprintf(stderr,"intron not ok: %s %i [%i,%i]\n", mpsl->Qname, ok, start_diff, end_diff) ;
			exit(-1) ;
		} ;
		
		int intron_type ;
		if (mpsl->strand=='+')
			intron_type = get_intron_type(mpsl->contig_num, mpsl->Tends[i]+1, 
							  mpsl->Tstarts[i+1]-1, mpsl->strand) ;
		else
			intron_type = get_intron_type(mpsl->contig_num, mpsl->Tstarts[i]+1,
							  mpsl->Tends[i+1]-1, mpsl->strand) ;

		if (intron_type==1)
			num_intron_type1++ ;
		if (intron_type==2)
			num_intron_type2++ ;
		if (intron_type==3)
			num_intron_type3++ ;

		if (mpsl->strand=='+')
		{
			fprintf(fd,"%s\t%s\tintron\t%i\t%i\t%c\t%i\t%i\tEST-%s-%i%c.%i\t%i\t%i\t%i\t%i\n", 
					contig_names[mpsl->contig_num], mpsl->Qname, 
					mpsl->Tends[i]+1, mpsl->Tstarts[i+1]-1, mpsl->strand, 
					0 /* num_exon_confirmed */, mpsl->num_intron_confirmed[i],
					mpsl->Qname, mpsl->contig_num, mpsl->strand, psl_idx,
					i, mpsl->blockcnt-1, mpsl->min_pos_info, mpsl->max_pos_info) ;
		}
		else
		{			
			fprintf(fd,"%s\t%s\tintron\t%i\t%i\t%c\t%i\t%i\tEST-%s-%i%c.%i\t%i\t%i\t%i\t%i\n", 
					contig_names[mpsl->contig_num], mpsl->Qname, 
					mpsl->Tstarts[i]+1, mpsl->Tends[i+1]-1, mpsl->strand, 
					0 /* num_exon_confirmed */, mpsl->num_intron_confirmed[i],
					mpsl->Qname, mpsl->contig_num, mpsl->strand, psl_idx, 
					i, mpsl->blockcnt-1, mpsl->min_pos_info, mpsl->max_pos_info) ;
		}
	} ;
}

int find_contigous(int idx, int * island_list, int island_list_len) 
{
	int *found=malloc(FOUND_BUF_SIZE*sizeof(int)) ;
	int *found2=malloc(FOUND_BUF_SIZE*sizeof(int)) ;

	fprintf(stdout, "mpsl: %i  island_size: %i\n", mpsl_num, island_list_len) ;
	
	int num ;
	
	// find overlaps (or not)
	if (MERGE_EST_TRANSCRIPTS)
	{
		num = find_match(&psls[idx], found, island_list, island_list_len) ;
		assert(num>0) ;
		fprintf(stdout, "%i. (%s) num_match = %i -> ", idx, psls[idx].Qname, num) ;
	
		
		int changed = 1 ;
		while (changed)
		{
			changed = 0 ;
			
			int i, num2 = num ;
			copy_int(found2, found, num) ;
			
			num = 1 ;
			found[0] = idx ;
			
			/* remove mismatching items */
			int j ;
			for (j=0; j<num2; j++)
			{
				int include = 1 ;
				int k;
				for (k=0; k<num; k++) 
				{
					//int ret=check_mismatch(found2[j], &psls[found[k]]) ;
					//fprintf(stdout, "check %s %s -> %i\n", psls[found2[j]].Qname, psls[found[k]].Qname,ret) ;
					if (check_mismatch(found2[j], &psls[found[k]])>0)
					{
						//fprintf(stdout, "%s mismatches %s\n", psls[found2[j]].Qname, psls[found[k]].Qname) ;
						include = 0 ;
						break ;
					}
					if (found2[j]==found[k])
					{
						include = -1 ;
						break ;
					}
				} ;
				if (include==1)
				{
					//fprintf(stdout, "including %s\n", psls[found2[j]].Qname) ;
					found[num]=found2[j] ;
					num++ ;
				} 
				else 
					if (include==0)
						changed=1 ;
			} ;
			fprintf(stdout, "%i -> ", num) ;
			
			/* add new items that match to new items */
			//for (i=0; i<num; i++)
			if (0) // slow and does not help much ..
			{
				num2 = find_match(&psls[found[i]], found2, island_list, island_list_len) ;
				int j ;
				for (j=0; j<num2; j++)
				{
					int include = 1 ;
					int k;
					for (k=0; k<num; k++) 
						if (found2[j]==found[k])
							include = 0 ;
					if (include)
						for (k=0; k<num; k++) 
							if (check_mismatch(found2[j], &psls[found[k]])>0)
								include = 0 ;
					if (include)
					{
						found[num]=found2[j] ;
						num++ ;
						changed=1 ;
					} ;
				} ;
			} ;
		} ;
	}
	else
	{
		num = 1; 
		found[0]=idx ;
	}
	
	merge_contigous(idx, num, found, &mpsls[mpsl_num]) ;
	fprintf(stdout, "%i [%i]\n", num, mpsls[mpsl_num].blockcnt) ;
	
	{
		fprintf(stdout, "@@@ %s: ", mpsls[mpsl_num].Qname) ;
		int i;
		for (i=0; i<num; i++)
			fprintf(stdout, "%s ", psls[found[i]].Qname) ;
		fprintf(stdout, "\n") ;
	}
	mpsl_num++ ;
	
	fflush(stdout) ;
	
	free(found) ;
	free(found2) ;
	
	return num ;
} ;

int main(int argc, char**argv)
{
	FILE * psl_file=NULL, *out_file=NULL ;
	int i ;
	char * genome_config_fname=NULL ;

	if (argc<4)
    {
		fprintf(stderr, "usage: %s [EST|CDNA|PROT] <psl-file> <contig_name> <strand> [<output_fname> [<microexons_fname> [<genome_config_fname]]]\n", argv[0]) ;
		exit(-1) ;
    } ;

	// genome config
	if (argc<8)
		genome_config_fname = "genome.config" ;
	else
		genome_config_fname = argv[7] ;
	if (init_genome(genome_config_fname)!=0)
	{
		fprintf(stderr, "genome initialization failed (%s)\n", genome_config_fname) ;
		exit(-1) ;
	}
	
	// microexons file
	if (argc<7)
		microexons_file = stdout ;
	else
		microexons_file = fopen(argv[6], "w+") ;
	if (microexons_file==NULL)
    {
		fprintf(stderr, "could not open %s\n", argv[6]) ;
		exit(-1) ;
    } ;

	// output file
	if (argc<6)
		out_file = stdout ;
	else
		if (strcmp(argv[5],"STDOUT")==0)
			out_file = stdout ;
		else
			out_file = fopen(argv[5], "w+") ;
	if (out_file==NULL)
    {
		fprintf(stderr, "could not open %s\n", argv[5]) ;
		exit(-1) ;
    } ;

	if (strcmp(argv[1], "CDNA")==0)
	{
		cDNA_mode = 1 ;
		protein_mode = 0 ;
	}
	else if (strcmp(argv[1], "EST")==0)
	{
		cDNA_mode = 0 ;
		protein_mode = 0 ;
	}
	else if (strcmp(argv[1], "PROT")==0)
	{
		cDNA_mode = 0 ;
		protein_mode = 1 ;
	}
	else
	{
		fprintf(stderr, "first argument has to be EST, CDNA or PROT\n") ;
		exit(-1) ;
	}

	// psl file
	psl_file=fopen(argv[2], "r") ;
	if (!psl_file)
    {
		fprintf(stderr,"could not open %s\n", argv[2]) ;
		exit(-1) ;
    } ;

	// contig number
	int contig_num = get_contig_num(argv[3]) ;

	// strand 
	assert(strlen(argv[4])==1) ;
	char strand = argv[4][0] ;
	
	assert((strand=='+') || (strand=='-')) ;
	assert((contig_num>=0) && (contig_num<num_contigs)) ;
	
	fprintf(stdout, "Considering %s, strand=%c\n", contig_names[contig_num], strand) ;
	
	contig_files[contig_num]=fopen(contig_fnames[contig_num],"r") ;
	if (!contig_files[contig_num])
	{
		fprintf(stderr,"could not open %s\n", contig_fnames[contig_num]) ;
		exit(-1) ;
	} ;
	
	read_psl(psl_file, contig_num, strand) ;
	fflush(stdout) ;

	fprintf(stderr, "%s v1.0\ncontig:%s contig_num:%i strand:%c ", argv[0], argv[3], contig_num, strand) ;
	if (cDNA_mode || protein_mode)
	{
		if (cDNA_mode)
			fprintf(stderr, "cDNA mode (no merging).\n") ;
		if (protein_mode)
			fprintf(stderr, "Protein mode (no merging).\n") ;

		for (i=0; i<psl_num; i++)
		{
			if (psls[i].strand=='-')
			{
				reverse_array(psls[i].Tstarts, psls[i].blockcnt) ;
				reverse_array(psls[i].Tends, psls[i].blockcnt) ;
			}
			print_psl(&psls[i], out_file, i) ;
		}
	}
	else
	{
		//////////////////////////////
		fprintf(stderr, "EST mode (merging aligments).\nidentifying islands ... \n") ;
		
		for (i=0; i<psl_num; i++)
			update_minmax(&psls[i]) ;
		int max_pos = 0 ;
		for (i=0; i<psl_num; i++)
			if (psls[i].max_pos>max_pos)
				max_pos = psls[i].max_pos ;
		//fprintf(stderr, "max_pos=%i\n",max_pos) ;
		
		short int * map = (short int*)calloc(max_pos+2,sizeof(short int)) ;
		assert(map!=NULL) ;
		for (i=0; i<psl_num; i++)
		{
			int j=0 ;
			for (j=psls[i].min_pos; j<psls[i].max_pos+1; j++)
				map[j]=1 ;
			
		}
		int num_islands=0, last_zero=1 ;
		for (i=0; i<max_pos; i++)
		{
			if (map[i]==0)
			{
				map[i]=-1 ;
				last_zero=1 ;
			}
			else
			{
				if (last_zero)
				{
					num_islands++ ;
					assert(num_islands<32000) ;
				}
				last_zero=0 ;
				map[i]=num_islands ;
			}
		} 
		num_islands++ ;
		
		fprintf(stderr, "number of islands: %i\n", num_islands-1) ;
		int largest_island = 0 ;
		
		int * island_item_num = calloc(num_islands, sizeof(int)) ;
		for (i=0; i<psl_num; i++)
		{
			assert(map[psls[i].min_pos]!=-1) ;
			island_item_num[map[psls[i].min_pos]]++ ;
			psls[i].island=map[psls[i].min_pos] ;
		}
		
		int **island_lists = (int**)malloc((num_islands)*sizeof(int*)) ;
		for (i=0; i<num_islands; i++)
			island_lists[i]=(int*)malloc((island_item_num[i]+1)*sizeof(int)) ;
		
		int * island_items = calloc(num_islands, sizeof(int)) ;
		for (i=0; i<psl_num; i++)
		{
			island_lists[psls[i].island][island_items[psls[i].island]]=i ;
			assert(island_items[psls[i].island]<island_item_num[psls[i].island]) ;
			island_items[psls[i].island]++ ;
		}
		for (i=0; i<num_islands; i++)
		{
			if (island_items[i]>largest_island)
				largest_island=island_items[i] ;
			assert(island_items[i]==island_item_num[i]) ;
		}
		fprintf(stderr, "largest island: %i\n", largest_island) ;
		free(map) ;
		
		fprintf(stderr, "merging matches based on islands\n") ;
		for (i=0; i<psl_num; i++)
			if (psls[i].covered==0)
				find_contigous(i, island_lists[psls[i].island], island_item_num[psls[i].island]) ;
		
		// cleaning up
		for (i=0; i<num_islands; i++)
			free(island_lists[i]) ;
		free(island_lists) ;
		free(island_items) ;
		free(island_item_num) ;
		
		for (i=0; i<mpsl_num; i++)
			if (mpsls[i].valid)
				print_mpsl(&mpsls[i], out_file, i) ;
		free_all_mpsl() ;

		// printing unmerged EST matches
		fprintf(stderr, "printing unmerged EST matches.\n") ;
		for (i=0; i<psl_num; i++)
		{
			if (psls[i].strand=='-')
			{
				reverse_array(psls[i].Tstarts, psls[i].blockcnt) ;
				reverse_array(psls[i].Tends, psls[i].blockcnt) ;
			}
			print_est_matches(&psls[i], out_file, i) ;
		}

	}
	
	free_all_psl() ;
	
	fclose(psl_file) ;
	if (argc>=6)
		fclose(out_file) ;
	if (argc>=7)
		fclose(microexons_file) ;
	
	fprintf(stderr, "num_intron_type1=%i\n", num_intron_type1) ;
	fprintf(stderr, "num_intron_type2=%i\n", num_intron_type2) ;
	fprintf(stderr, "num_intron_type3=%i\n", num_intron_type3) ;

	cleanup_genome() ;

	return 0;
} 

