function [gap_3prime,ss_3prime,gap_5prime,ss_5prime] = count_altprime_gap(organism)
% function [gap_3prime,ss_3prime,gap_5prime,ss_5prime] = count_altprime_gap(organism)
%
% count the number of corresponding consensus sites, and the sequence length
% between the splice sites for alternative 3 and 5 prime, from the genes structure.
%

addpath ~/svn/projects/splicing/splicegraphs/detect_altsplice
addpath /fml/ag-raetsch/share/software/matlab_tools/utils
addpath ~/svn/tools/genomes

genome_info = init_genome(sprintf('/fml/ag-raetsch/share/projects/altsplicedata/%s/genome.config',...
				  organism));
load(sprintf('/fml/ag-raetsch/share/projects/altsplicedata/%s/confirmed_sequences.mat',...
	     organism), 'genes') ;


% --- detect alternative splicing
idx_alt=[] ;
for i=1:length(genes)
  if genes(i).is_alt_spliced
    idx_alt=[idx_alt, i] ;
  end
end

[idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] = detect_altprime_strict(genes, idx_alt);



% --- compute gap for 5prime
num_5prime = 0;
gap_5prime = repmat(nan,15000,1);
ss_5prime = repmat(nan,15000,1);
for ix1 = 1:length(idx_alt_5prime)
  num_5prime = num_5prime + 1;
  sites = [exon_alt_5prime(ix1).fiveprimesites];
  vertices = genes(idx_alt_5prime(ix1)).splicegraph{1};
  for ix2 = 1:length(sites)-1
    for ix3 = ix2+1:length(sites)
      [gap_5prime(num_5prime,1),ss_5prime(num_5prime,1)] = count_donors(genes(idx_alt_5prime(ix1)),...
						  vertices(:,sites(ix2)),vertices(:,sites(ix3)),genome_info);
    end
  end
end

gap_5prime = gap_5prime(1:num_5prime);
ss_5prime = ss_5prime(1:num_5prime);
fd = fopen('frac_gap.txt','a');
fprintf(fd,'%s\n',organism);
fprintf(fd,'Alternative donors\n');
frac_gap(gap_5prime,ss_5prime,fd);
fclose(fd);

% --- compute gap for 3prime
num_3prime = 0;
gap_3prime = repmat(nan,15000,1);
ss_3prime = repmat(nan,15000,1);
for ix1 = 1:length(idx_alt_3prime)
  num_3prime = num_3prime + 1;
  sites = [exon_alt_3prime(ix1).threeprimesites];
  vertices = genes(idx_alt_3prime(ix1)).splicegraph{1};
  for ix2 = 1:length(sites)-1
    for ix3 = ix2+1:length(sites)
      [gap_3prime(num_3prime,1),ss_3prime(num_3prime,1)] = count_acceptors(genes(idx_alt_3prime(ix1)),...
						  vertices(:,sites(ix2)),vertices(:,sites(ix3)),genome_info);
    end
  end
end

gap_3prime = gap_3prime(1:num_3prime);
ss_3prime = ss_3prime(1:num_3prime);
fd = fopen('frac_gap.txt','a');
fprintf(fd,'Alternative acceptors\n');
frac_gap(gap_3prime,ss_3prime,fd);
fclose(fd);



function [strlen, num_donor] = count_donors(gene, v1, v2, genome_info)

if gene.strands(1) == '+'
  if v2(2) < v1(2)
    start = v2(2)+1;
    stop = v1(2)+2;
  else
    start = v1(2)+1;
    stop = v2(2)+2;
  end
else
  if v2(1) < v1(1)
    start = v2(1)-2;
    stop = v1(1)-1;
  else
    start = v1(1)-2;
    stop = v2(1)-1;
  end    
end
genome_str = load_genomic(gene.chr,gene.strands(1),start,stop,genome_info);
assert(all(genome_str(1:2)=='gt' | genome_str(1:2)=='gc'));
assert(all(genome_str(end-1:end)=='gt' | genome_str(end-1:end)=='gc'));
genome_str = genome_str(3:end-2);
num_donor = length(find_patterns( genome_str, {'gt','gc'} ));
strlen = length(genome_str);



function [strlen, num_acceptor] = count_acceptors(gene, v1, v2, genome_info)

if gene.strands(1) == '+'
  if v2(1) < v1(1)
    start = v2(1)-2;
    stop = v1(1)-1;
  else
    start = v1(1)-2;
    stop = v2(1)-1;
  end
else
  if v2(2) < v1(2)
    start = v2(2)+1;
    stop = v1(2)+2;
  else
    start = v1(2)+1;
    stop = v2(2)+2;
  end    
end

genome_str = load_genomic(gene.chr,gene.strands(1),start,stop,genome_info);
assert(all(genome_str(1:2)=='ag'));
assert(all(genome_str(end-1:end)=='ag'));
genome_str = genome_str(3:end-2);
num_acceptor = length(find_patterns( genome_str, {'ag'} ));
strlen = length(genome_str);
