function [gap_3prime,ss_3prime,gap_5prime,ss_5prime] = count_altprime_gap_events(organism)
%function [gap_3prime,ss_3prime,gap_5prime,ss_5prime] = count_altprime_gap_events(organism)
%
% count the number of corresponding consensus sites, and the sequence length
% between the splice sites for alternative 3 and 5 prime, from the events structure.
%

addpath ~/svn/projects/splicing/splicegraphs/detect_altsplice
addpath /fml/ag-raetsch/share/software/matlab_tools/utils
addpath ~/svn/tools/genomes

genome_info = init_genome(sprintf('/fml/ag-raetsch/share/projects/altsplicedata/%s/genome.config',...
				  organism));
fprintf('processing at: %s\n', genome_info.basedir) ;
as_events5 = load_cell(sprintf('%s/prediction/events_alt_5prime', genome_info.basedir)) ;
posidx = find([as_events5.label]==1);
as_events5 = as_events5(posidx);

confirm2idx = find(([as_events5.alt0_confirm_by]>=2) & ([as_events5.alt1_confirm_by]>=2));
confirm3idx = find(([as_events5.alt0_confirm_by]>=3) & ([as_events5.alt1_confirm_by]>=3));

fprintf(1,'Alternative 5 prime: %d, %d, %d events with 1, 2 and 3 confirmations\n',...
	length(as_events5),length(confirm2idx),length(confirm3idx));


num_5prime = 0;
gap_5prime = repmat(nan,length(as_events5),3);
ss_5prime = repmat(nan,length(as_events5),3);
for ix1 = 1:length(as_events5)
  num_5prime = num_5prime + 1;
  [gap_5prime(num_5prime,1),ss_5prime(num_5prime,1)] = count_donors(as_events5(ix1),genome_info);
end

num_5prime = 0;
for ix1 = confirm2idx
  num_5prime = num_5prime + 1;
  [gap_5prime(num_5prime,2),ss_5prime(num_5prime,2)] = count_donors(as_events5(ix1),genome_info);
end

num_5prime = 0;
for ix1 = confirm3idx
  num_5prime = num_5prime + 1;
  [gap_5prime(num_5prime,3),ss_5prime(num_5prime,3)] = count_donors(as_events5(ix1),genome_info);
end

fd = fopen('frac_gap2.txt','a');
fprintf('Alternative donors\n');
frac_gap_latex(organism,gap_5prime,ss_5prime,'5prime',fd);



as_events3 = load_cell(sprintf('%s/prediction/events_alt_3prime', genome_info.basedir)) ;
posidx = find([as_events3.label]==1);
as_events3 = as_events3(posidx);

confirm2idx = find(([as_events3.alt0_confirm_by]>=2) & ([as_events3.alt1_confirm_by]>=2));
confirm3idx = find(([as_events3.alt0_confirm_by]>=3) & ([as_events3.alt1_confirm_by]>=3));

fprintf(1,'Alternative 3 prime: %d, %d, %d events with 1, 2 and 3 confirmations\n',...
	length(as_events3),length(confirm2idx),length(confirm3idx));


num_3prime = 0;
gap_3prime = repmat(nan,length(as_events3),3);
ss_3prime = repmat(nan,length(as_events3),3);
for ix1 = 1:length(as_events3)
  num_3prime = num_3prime + 1;
  [gap_3prime(num_3prime,1),ss_3prime(num_3prime,1)] = count_acceptors(as_events3(ix1),genome_info);
end

num_3prime = 0;
for ix1 = confirm2idx
  num_3prime = num_3prime + 1;
  [gap_3prime(num_3prime,2),ss_3prime(num_3prime,2)] = count_acceptors(as_events3(ix1),genome_info);
end

num_3prime = 0;
for ix1 = confirm3idx
  num_3prime = num_3prime + 1;
  [gap_3prime(num_3prime,3),ss_3prime(num_3prime,3)] = count_acceptors(as_events3(ix1),genome_info);
end

fprintf('Alternative acceptors\n');
frac_gap_latex(organism,gap_3prime,ss_3prime,'3prime',fd);
fclose(fd)

if 0,
  figure;
  bins = [0:100];
  hist(ss_5prime,bins);
  AA = axis;AA(1:2) = [-0.5 10.5];axis(AA);
  legend({'confirm 1','confirm 2','confirm 3'});
  title('Number of consensus sites between donors');
  exportfig(gcf,'hist_gap_alt_5prime.eps','bounds','tight','Color','rgb');
  
  figure;
  hist(ss_3prime,bins);
  AA = axis;AA(1:2) = [-0.5 10.5];axis(AA);
  legend({'confirm 1','confirm 2','confirm 3'});
  title('Number of consensus sites between acceptors');
  exportfig(gcf,'hist_gap_alt_3prime.eps','bounds','tight','Color','rgb');
end



function [strlen, num_donor] = count_donors(event, genome_info)

if event.strand == '+'
  start = event.exon_5prime(2)+1;
  stop = event.exon_alternative(2)+2;
else
  start = event.exon_alternative(1)-2;
  stop = event.exon_5prime(1)-1;
end
genome_str = load_genomic(event.chr,event.strand,start,stop,genome_info);
assert(all(genome_str(1:2)=='gt' | genome_str(1:2)=='gc'));
assert(all(genome_str(end-1:end)=='gt' | genome_str(end-1:end)=='gc'));
genome_str = genome_str(3:end-2);
num_donor = length(find_patterns( genome_str, {'gt','gc'} ));
strlen = length(genome_str);



function [strlen, num_acceptor] = count_acceptors(event, genome_info)

if event.strand == '+'
  start = event.exon_alternative(1)-2;
  stop = event.exon_3prime(1)-1;
else
  start = event.exon_3prime(2)+1;
  stop = event.exon_alternative(2)+2;
end
genome_str = load_genomic(event.chr,event.strand,start,stop,genome_info);
assert(all(genome_str(1:2)=='ag'));
assert(all(genome_str(end-1:end)=='ag'));
genome_str = genome_str(3:end-2);
num_acceptor = length(find_patterns( genome_str, {'ag'} ));
strlen = length(genome_str);
