function count_est_confirm(info,origpath)

% based on genes2events_alt_intron.m

alt_type = 'alt_intron';



addpath(sprintf('%s/utils', origpath)) ;
addpath(sprintf('%s/gff/read_sequences', origpath)) ;
addpath(sprintf('%s/splicegraphs/detect_altsplice', origpath)) ;
addpath /fml/ag-raetsch/share/software/matlab_tools/utils
addpath(sprintf('%s/sensors', origpath)) ;

% --- initialise genome
genome_info = init_genome(info) ;
fprintf('processing at: %s\n', genome_info.basedir) ;
load(sprintf('%s/confirmed_sequences.mat', genome_info.basedir), 'genes') ;
fprintf('loaded confirmed_sequences.mat\n\n') ;





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% export_intron_reten                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create an empty array
as_events = [];
ENTRYSTEP = 10000;

% --- detect alternative splicing
idx_alt=[] ;
for i=1:length(genes)
  if genes(i).is_alt_spliced
    idx_alt=[idx_alt, i] ;
  end
end
fprintf('Alternatively spliced genes: %i\n\n', length(idx_alt)) ;
[idx_intron_reten, intron_intron_reten] = detect_intronreten(genes, idx_alt) ;


% --- generate examples
fprintf(1,'generate examples\n');
idx_events=0 ;
for idx_genes=1:length(genes)
  if (mod(idx_events,1000)==0)
    fprintf(1,'.') ;
  end
  
  if idx_events >= length(as_events)
    fprintf(1,'Increasing length from %d to %d\n',idx_events,idx_events+ENTRYSTEP);
    as_events(idx_events+ENTRYSTEP).genes_idx= nan ;
    as_events(idx_events+ENTRYSTEP).genes_name = 'testing only';
    as_events(idx_events+ENTRYSTEP).chr = 'CHROMOSOME_99';
    as_events(idx_events+ENTRYSTEP).strand = '*';
    as_events(idx_events+ENTRYSTEP).alt_type = alt_type;
    as_events(idx_events+ENTRYSTEP).label = nan;
    as_events(idx_events+ENTRYSTEP).has_cDNA = nan;
    as_events(idx_events+ENTRYSTEP).is_alt = nan;
    as_events(idx_events+ENTRYSTEP).is_alt_spliced = nan;
    as_events(idx_events+ENTRYSTEP).is_valid = nan;
    as_events(idx_events+ENTRYSTEP).exon_idx = nan;
    as_events(idx_events+ENTRYSTEP).exon_5prime = nan;
    as_events(idx_events+ENTRYSTEP).exon_3prime = nan;
    as_events(idx_events+ENTRYSTEP).exon_alternative = nan;
    as_events(idx_events+ENTRYSTEP).alt0_confirm_by = nan;
    as_events(idx_events+ENTRYSTEP).alt1_confirm_by = nan;
  end

  
  num_exons = size(genes(idx_genes).splicegraph{1},2);
  vertices = genes(idx_genes).splicegraph{1};
  edges = genes(idx_genes).splicegraph{2};
  
  for idx_exon_l = 1:(num_exons-1)
    for idx_exon_r = (idx_exon_l+1):num_exons
      if (edges(idx_exon_l,idx_exon_r))
        idx_events = idx_events + 1;
	
	as_events(idx_events).genes_idx = idx_genes;
	as_events(idx_events).genes_name = genes(idx_genes).name;
	as_events(idx_events).chr = genes(idx_genes).chr;
	as_events(idx_events).strand = genes(idx_genes).strands(1);
	as_events(idx_events).alt_type = alt_type;

	as_events(idx_events).label = -1;
	if ismember(idx_genes,idx_intron_reten)
	  alt_introns = intron_intron_reten(:,find(idx_intron_reten==idx_genes));
	  for ix_intron = 1:size(alt_introns,2)
	    if (idx_exon_l==alt_introns(1,ix_intron)) && (idx_exon_r==alt_introns(2,ix_intron))
	      exon_idx_intron = alt_introns(3,ix_intron);
	      as_events(idx_events).label = 1;
	      break;
	    else
	      % because the intron retention detector returns unique introns, we have to ignore the rest.
	      for search = 1:size(alt_introns,2)
		if (vertices(2,idx_exon_l) == vertices(2,alt_introns(1,search))) ...
		      && (vertices(1,idx_exon_r) == vertices(1,alt_introns(2,search)))
		  as_events(idx_events).label = 0;
		  break;
		end
	      end
	    end
	  end
	end
	
	
	as_events(idx_events).has_cDNA = genes(idx_genes).has_cDNA;
	as_events(idx_events).is_alt = genes(idx_genes).is_alt;
	as_events(idx_events).is_alt_spliced = genes(idx_genes).is_alt_spliced;
	as_events(idx_events).is_valid = genes(idx_genes).is_valid;
	if (genes(idx_genes).strands(1) == '+')
	  as_events(idx_events).exon_5prime = [vertices(1,idx_exon_l) vertices(2,idx_exon_l)];
	  as_events(idx_events).exon_3prime = [vertices(1,idx_exon_r) vertices(2,idx_exon_r)];
	else
	  as_events(idx_events).exon_3prime = [vertices(1,idx_exon_l) vertices(2,idx_exon_l)];
	  as_events(idx_events).exon_5prime = [vertices(1,idx_exon_r) vertices(2,idx_exon_r)];
	end
	
	
	
	
	
	if as_events(idx_events).label == 1
	  if (genes(idx_genes).strands(1) == '+')
	    as_events(idx_events).exon_idx = [idx_exon_l, idx_exon_r, exon_idx_intron];
	  else
	    as_events(idx_events).exon_idx = [idx_exon_r, idx_exon_l, exon_idx_intron];
	  end
	  as_events(idx_events).exon_alternative = [vertices(1,exon_idx_intron) vertices(2,exon_idx_intron)];
	  conf_count1 = 0;
	  for idx1 = 1:length(genes(idx_genes).exons)
	    for idx2 = 1:size(genes(idx_genes).exons{idx1},1)
	      if (vertices(2,idx_exon_l) > genes(idx_genes).exons{idx1}(idx2,1))&&...
		    (vertices(1,idx_exon_r) < genes(idx_genes).exons{idx1}(idx2,2))
		conf_count1 = conf_count1 + 1;
	      end
	    end
	  end
	  as_events(idx_events).alt1_confirm_by = conf_count1;
	  if conf_count1==0,
	    % long exon has been inferred,
	    as_events(idx_events).label = 99;
	  end
	  
	  
	else
	  if (genes(idx_genes).strands(1) == '+')
	    as_events(idx_events).exon_idx = [idx_exon_l, idx_exon_r, 0];
	  else
	    as_events(idx_events).exon_idx = [idx_exon_r, idx_exon_l, 0];
	  end
	  as_events(idx_events).exon_alternative = [];
	  as_events(idx_events).alt1_confirm_by = 0;
	  
	end
	  
        conf_count0 = 0;
        for idx1 = 1:length(genes(idx_genes).exons)
          for idx2 = 1:size(genes(idx_genes).exons{idx1},1)-1
            if (vertices(2,idx_exon_l) == genes(idx_genes).exons{idx1}(idx2,2))&&...
                  (vertices(1,idx_exon_r) == genes(idx_genes).exons{idx1}(idx2+1,1))
              conf_count0 = conf_count0 + 1;
            end
          end
        end
        if conf_count0==0, fprintf(1,'conf_count0==0\n') ; keyboard; end ;
	as_events(idx_events).alt0_confirm_by = conf_count0;
	

      end % if edge
    end % for idx_exon_r
  end % for idx_exon_l
end




clear genes;

as_events = as_events(1:idx_events);

all_labels = [as_events(:).label];
fprintf(1,'Number of intron retentions:\t\t\t\t%d\n',sum(all_labels==1));
fprintf(1,'Number of negative examples:\t\t\t\t%d\n',sum(all_labels==-1));

idx_invalid = find([all_labels == 0] | [all_labels == 99] | [all_labels == 1]);
idx_valid = setdiff(1:idx_events,idx_invalid);
fprintf(1,'Number of inferred exons that were removed: \t\t%d\n',sum(all_labels == 99));
fprintf(1,'Number of repeated introns that were removed:\t\t%d\n',sum(all_labels == 0));


as_events = as_events(idx_valid);
MAX_CONFIRM = 50;
count_conf = zeros(1,MAX_CONFIRM+1);
clear all_labels;

fprintf(1,'Counting confirmations\n');
for ix = 1:MAX_CONFIRM
  count_conf(ix) = length(find([as_events(:).alt0_confirm_by]== ix));
end
count_conf(MAX_CONFIRM+1) = length(find([as_events(:).alt0_confirm_by]>MAX_CONFIRM));

bar(count_conf)
exportfig(gcf,'est_confirms.eps','bounds','tight','Color','rgb');
