function [exon_len05, exon_len95, intron_len05, intron_len95] = count_lengths(organism)

% Count the intron and exon lengths.
%

addpath /fml/ag-raetsch/share/software/matlab_tools/utils
addpath ~/svn/tools/genomes

toplot = 0;

genefile = sprintf('/fml/ag-raetsch/share/projects/altsplicedata/%s/confirmed_sequences.mat', organism);
fprintf('loading: %s\n', genefile) ;
load(genefile, 'genes') ;

exon_len = repmat(nan,1e6,1);
exon_len2 = repmat(nan,1e6,1);
intron_len = repmat(nan,1e6,1);

num_exon = 1;
num_exon2 = 1;
num_intron = 1;
for ix = 1:length(genes)
  if num_exon + 1000 > length(exon_len)
    exon_len(2*num_exon) = nan;
  end
  if num_intron +1000 > length(intron_len)
    intron_len(2*num_intron) = nan;
  end
  vertices = genes(ix).splicegraph{1};
  edges = genes(ix).splicegraph{2};
  
  num_verts = size(vertices,2);
  for ixv = 1:num_verts
    exon_len(num_exon) = vertices(2,ixv) - vertices(1,ixv);
    assert(exon_len(num_exon) >= 0);
    num_exon = num_exon + 1;
    t=triu(edges) ;
    if any(t(ixv,:)) && any(t(:,ixv)),
      exon_len2(num_exon2) = vertices(2,ixv) - vertices(1,ixv);
      num_exon2 = num_exon2 + 1;
    end ;
  end ;
  
  for ixi = 1:num_verts-1
    for ixj = ixi+1:num_verts
      if edges(ixi,ixj)
        intron_len(num_intron) = vertices(1,ixj) - vertices(2,ixi);
        assert(intron_len(num_intron) >= 0);
        num_intron = num_intron + 1;
      end
    end
  end
  
end

num_exon=num_exon-1 ;
num_exon2=num_exon2-1 ;
num_intron=num_intron-1 ;

exon_len = exon_len(1:num_exon);
exon_len2 = exon_len(1:num_exon2);
intron_len = intron_len(1:num_intron);

exon_len = sort(exon_len,'ascend');
exon_len95 = exon_len(round(0.95*length(exon_len)));
%exon_len90 = exon_len(round(0.90*length(exon_len)));
%exon_len10 = exon_len(round(0.10*length(exon_len)));
exon_len05 = exon_len(round(0.05*length(exon_len)));
fprintf(1,'Counted %d exons, 5%% and 95%%of which are %d and %d nt long respectively.\n',...
	num_exon,exon_len05,exon_len95);
if toplot,
  figure(1);
  %hist(exon_len,30);
  title('exon lengths');
  step = (log(exon_len95) - log(exon_len05))/30;
  bins = exp(log(exon_len05):step:log(exon_len95));
  bar(histc(exon_len,bins));
  set(gca,'XTick',[1:5:length(bins)]);
  set(gca,'XTickLabel',round(bins(1:5:end)));
  xlabel('exon lengths')
end

intron_len = sort(intron_len,'ascend');
intron_len95 = intron_len(round(0.95*length(intron_len)));
%intron_len90 = intron_len(round(0.90*length(intron_len)));
%intron_len10 = intron_len(round(0.10*length(intron_len)));
intron_len05 = intron_len(round(0.05*length(intron_len)));
fprintf(1,'Counted %d introns, 5%% and 95%% of which are %d and %d nt long respectively.\n',...
	num_intron,intron_len05,intron_len95);
if toplot,
  figure(2);
  %hist(intron_len,30);
  title('intron lengths');
  step = (log(intron_len95) - log(intron_len05))/30;
  bins = exp(log(intron_len05):step:log(intron_len95));
  bar(histc(intron_len,bins));
  set(gca,'XTick',[1:5:length(bins)]);
  set(gca,'XTickLabel',round(bins(1:5:end)));
  xlabel('intron lengths')
end



return

% Note that taking subsets of exons and introns is not the same as taking subsets of genes
fprintf(1,'Performing several permutations to estimate sensitivity to subsets\n');

rand('state', 8)
num_reps = 20;
frac_train = 0.4;
num_train = floor(frac_train*num_exon);
exon_lengths = repmat(nan,num_reps,2);
for ix = 1:num_reps
  perm = randperm(num_exon);
  tlen = exon_len(perm(1:num_train));
  tlen = sort(tlen,'ascend');
  exon_lengths(ix,1) = tlen(round(0.05*num_train));
  exon_lengths(ix,2) = tlen(round(0.95*num_train));
end
fprintf(1,'5%% and 95%% lengths are %3.1f@%3.1f and %3.1f@%3.1f nt respectively.\n',...
	mean(exon_lengths(:,1)),std(exon_lengths(:,1)),mean(exon_lengths(:,2)),std(exon_lengths(:,2)));

num_train = floor(frac_train*num_intron);
intron_lengths = repmat(nan,num_reps,2);
for ix = 1:num_reps
  perm = randperm(num_intron);
  tlen = intron_len(perm(1:num_train));
  tlen = sort(tlen,'ascend');
  intron_lengths(ix,1) = tlen(round(0.05*num_train));
  intron_lengths(ix,2) = tlen(round(0.95*num_train));
end
fprintf(1,'5%% and 95%% lengths are %3.1f@%3.1f and %3.1f@%3.1f nt respectively.\n',...
	mean(intron_lengths(:,1)),std(intron_lengths(:,1)),mean(intron_lengths(:,2)),std(intron_lengths(:,2)));




