function [counts,altcounts] = count_stop_intron(organism)
% function [counts,altcounts] = count_stop_intron(organism)
% For each intron, count the number of stop codons.
% Take the minimimum for the three frames, and normalize by intron length.

addpath /fml/ag-raetsch/share/software/matlab_tools/utils
addpath ~/svn/tools/genomes/

genome_info = init_genome(sprintf('/fml/ag-raetsch/share/projects/altsplicedata/%s/genome.config',organism));
fprintf('loading %s/confirmed_sequences.mat', genome_info.basedir);
load(sprintf('%s/confirmed_sequences.mat', genome_info.basedir), 'genes') ;
fprintf('... done\n');

STEP = 10000;
counts = repmat(nan,STEP,1);
num_intron = 1;
for ix = 1:length(genes)
  if num_intron + 100 > length(counts)
    fprintf('.');
    counts(length(counts)+STEP) = nan;
  end
  vertices = genes(ix).splicegraph{1};
  edges = genes(ix).splicegraph{2};
  num_verts = size(vertices,2);
  for ixi = 1:num_verts-1
    for ixj = ixi+1:num_verts
      if edges(ixi,ixj)
	start = vertices(2,ixi);
	stop = vertices(1,ixj);
	counts(num_intron) = count_stop_codons(genes(ix).chr,genes(ix).strands(1),start,stop,genome_info);
        num_intron = num_intron + 1;
      end
    end
  end
end
fprintf('\n');
num_intron=num_intron-1 ;
counts = counts(1:num_intron);
assert(all(isfinite(counts)));

fprintf('loading %s/prediction/events_alt_intron.mat',genome_info.basedir);
as_events = load_cell(sprintf('%s/prediction/events_alt_intron.mat',genome_info.basedir));
fprintf('... done\n');

alt_events = as_events(find([as_events.label]==1));
%confirm2idx = find(([alt_events.alt0_confirm_by]>=2) & ([alt_events.alt1_confirm_by]>=2));
%confirm3idx = find(([alt_events.alt0_confirm_by]>=3) & ([alt_events.alt1_confirm_by]>=3));
confirm2idx = find([alt_events.alt1_confirm_by]>=2);
confirm3idx = find([alt_events.alt1_confirm_by]>=3);

altcounts = repmat(nan,length(alt_events),3);
for ix=1:length(alt_events)
  if alt_events(ix).strand == '+'
    start = alt_events(ix).exon_5prime(2);
    stop = alt_events(ix).exon_3prime(1);
  else
    start = alt_events(ix).exon_3prime(1);
    stop = alt_events(ix).exon_5prime(2);
  end
  if stop <= start, keyboard; end;
  altcounts(ix,1) = count_stop_codons(alt_events(ix).chr,alt_events(ix).strand,start,stop,genome_info);
  if ~isempty(find(confirm2idx==ix))
    altcounts(ix,2) = altcounts(ix,1);
  end
  if ~isempty(find(confirm3idx==ix))
    altcounts(ix,3) = altcounts(ix,1);
  end
end

fprintf('Fraction with no stop codons:\n');
fprintf('All introns: %2.1f%%, Alternative (1,2,3 confirms): %2.1f%%, %2.1f%%, %2.1f%%\n',...
	100*sum(counts==0)./sum(isfinite(counts)),...
	100*sum(altcounts==0)./sum(isfinite(altcounts)));

if 1,
  figure(1);
  hist(counts,100);
  set(gca,'FontSize',16);
  AA = axis;
  AA(1:2) = [0 0.05];
  axis(AA);
  text(3/64,1000,'3/64','FontSize',16)
  text(0.01,15000,sprintf('Fraction with no stop codons:\n %2.1f%%',...
			   100*sum(counts==0)./sum(isfinite(counts))),'FontSize',16);
  figure(2);
  hist(altcounts,30);
  set(gca,'FontSize',16);
  AA = axis;
  AA(1:2) = [0 0.05];
  axis(AA);
  text(3/64,10,'3/64','FontSize',16)
  text(0.01,150,sprintf('Fraction with no stop codons:\n %2.1f%%, %2.1f%%, %2.1f%%',...
			  100*sum(altcounts==0)./sum(isfinite(altcounts))),'FontSize',16);
end
