function [edges,introncount,fracs] = count_stop_intron_length(organism)
% function [edges,introncount,fracs] = count_stop_intron_length(organism)
%
% Count the number of stop codons in retained introns.
% Sort into different bins according to intron length.

addpath /fml/ag-raetsch/share/software/matlab_tools/utils
addpath ~/svn/tools/genomes/

genome_info = init_genome(sprintf('/fml/ag-raetsch/share/projects/altsplicedata/%s/genome.config',organism));
fprintf('loading %s/prediction/events_alt_intron.mat',genome_info.basedir);
as_events = load_cell(sprintf('%s/prediction/events_alt_intron.mat',genome_info.basedir));
fprintf('... done\n');

alt_events = as_events(find([as_events.label]==1));

counts = repmat(nan,length(alt_events),1);
intronlens = repmat(nan,length(alt_events),1);

for ix=1:length(alt_events)
  if alt_events(ix).strand == '+'
    start = alt_events(ix).exon_5prime(2);
    stop = alt_events(ix).exon_3prime(1);
  else
    start = alt_events(ix).exon_3prime(1);
    stop = alt_events(ix).exon_5prime(2);
  end
  if stop <= start, keyboard; end;
  counts(ix) = count_stop_codons(alt_events(ix).chr,alt_events(ix).strand,start,stop,genome_info);
  intronlens(ix) = stop-start + 1;
end

numbins = 10;
edges = logspace(log10(min(intronlens)),log10(max(intronlens)),numbins+1);
edges = edges(2:end-1);
fracs = repmat(nan,1,numbins);
introncount = repmat(nan,1,numbins);
countsinbin = counts(find(intronlens<edges(1)));
introncount(1) = length(countsinbin);
fracs(1) = sum(countsinbin==0)./length(countsinbin);
for ix = 2:numbins-1
  countsinbin = counts(find(intronlens>=edges(ix-1)&intronlens<edges(ix)));
  introncount(ix) = length(countsinbin);
  fracs(ix) = sum(countsinbin==0)./length(countsinbin);
end
countsinbin = counts(find(intronlens>=edges(numbins-1)));
introncount(numbins) = length(countsinbin);
fracs(numbins) = sum(countsinbin==0)./length(countsinbin);


if 0,
  figure(1);
  [edges,introncount,fracs] = count_stop_intron_length('test');
  introncount
  p(1) = semilogx(edges,fracs(1:end-1),'ro');
  hold on;
  semilogx(edges,fracs(1:end-1),'r');
  [edges,introncount,fracs] = count_stop_intron_length('elegans');
  introncount
  p(2) = semilogx(edges,fracs(1:end-1),'b+');
  semilogx(edges,fracs(1:end-1),'b');
  [edges,introncount,fracs] = count_stop_intron_length('arabidopsis');
  introncount
  p(3) = semilogx(edges,fracs(1:end-1),'ks');
  semilogx(edges,fracs(1:end-1),'k');
  set(gca,'FontSize',16);
  xlabel('intron length');
  ylabel('fraction with no stop codons');
  legend(p,'human (cDNA)','elegans','arabidopsis');
end
