function counts = count_stop_intron_pos(organism)
% function counts = count_stop_intron_pos(organism)
%
% Count the number of stop codons in retained introns.
% Count the introns on the 5' end of the gene separately from
%  those of the 3' end of the gene.


addpath /fml/ag-raetsch/share/software/matlab_tools/utils
addpath ~/svn/tools/genomes/

genome_info = init_genome(sprintf('/fml/ag-raetsch/share/projects/altsplicedata/%s/genome.config',organism));
fprintf('loading %s/confirmed_sequences.mat', genome_info.basedir);
load(sprintf('%s/confirmed_sequences.mat', genome_info.basedir), 'genes') ;
fprintf('... done\n');

fprintf('loading %s/prediction/events_alt_intron.mat',genome_info.basedir);
as_events = load_cell(sprintf('%s/prediction/events_alt_intron.mat',genome_info.basedir));
fprintf('... done\n');

alt_events = as_events(find([as_events.label]==1));

counts = repmat(nan,length(alt_events),2);

for ix=1:length(alt_events)
  if alt_events(ix).strand == '+'
    start = alt_events(ix).exon_5prime(2);
    stop = alt_events(ix).exon_3prime(1);
  else
    start = alt_events(ix).exon_3prime(1);
    stop = alt_events(ix).exon_5prime(2);
  end
  if stop <= start, keyboard; end;
  altcounts = count_stop_codons(alt_events(ix).chr,alt_events(ix).strand,start,stop,genome_info);
  
  num_exons = size(genes(alt_events(ix).genes_idx).splicegraph{2},1);
  if min(alt_events(ix).exon_idx) < ceil(num_exons/2)
    % 5' end
    counts(ix,1) = altcounts;
  else
    % 3' end
    counts(ix,2) = altcounts;
  end    
end


if 1,
  figure(1);
  hist(counts(:,1),30);
  set(gca,'FontSize',16);
  AA = axis;
  AA(1:2) = [0 0.05];
  axis(AA);
  text(3/64,10,'3/64','FontSize',16)
  text(0.01,80,sprintf('Fraction with no stop codons:\n %2.1f%%',...
			   100*sum(counts(:,1)==0)./sum(isfinite(counts(:,1)))),'FontSize',16);
  figure(2);
  hist(counts(:,2),30);
  set(gca,'FontSize',16);
  AA = axis;
  AA(1:2) = [0 0.05];
  axis(AA);
  text(3/64,10,'3/64','FontSize',16)
  text(0.01,80,sprintf('Fraction with no stop codons:\n %2.1f%%',...
			  100*sum(counts(:,2)==0)./sum(isfinite(counts(:,2)))),'FontSize',16);
end
