function fname = genes2fasta(genes,min_len,max_exons)
% function filename = genes2fasta(genes,min_len,max_exons)
%
% For all genes,
% find the longest isoform of the gene and write the corresponding mRNA sequence to fasta.
% (to be used with psl2similarity.m)
% Ignores isoforms with more than max_exons exons (because of MATLAB recursivestack size of 100).
% Ignores isoforms with fewer than min_len nucleotides.
%
% Written by: Cheng Soon Ong, 23 May 2007

fprintf('Writing Fasta file\n');
fname = tempname ;
assert(~fexist([fname '.fa']) & ~fexist([fname '.psl'])) ;
fd = fopen([fname '.fa'],'w+');

mRNAs = {};
for ix = 1:length(genes)
  if ~mod(ix,1000), fprintf('.');end;
  vertices = genes(ix).splicegraph{1};
  edges = genes(ix).splicegraph{2};
  if size(vertices,2) > max_exons
    fprintf('too many exons (%d > %d) in gene %d -> ignoring in comparison\n', size(vertices,2), max_exons, ix);
    continue;
  end
  isoforms = find_all_isoforms(vertices,edges,genes(ix).strands(1));

  L=zeros(1,length(isoforms)) ;
  for i=1:length(isoforms),
    L(i) = sum(vertices(2,isoforms{i})-vertices(1,isoforms{i})) ;
  end ;
  [longest_len, longest_idx]=max(L) ;

  if longest_len<min_len,
    fprintf('transcript with %i exons has len=%i<%i -> ignoring in comparison\n', ...
	    size(vertices,2), longest_len, min_len) ;
    continue ;
  end ;

  seq = load_mrna(genes(ix).chr,genes(ix).strands(1),...
			  vertices(1,isoforms{longest_idx})',vertices(2,isoforms{longest_idx})');
  write_fasta_simple(fd, sprintf('seq%i', ix), seq) ;
end ;
fclose(fd) ;
fprintf('\n');
