function [isostartpos,isostoppos,isoORFlength] = get_longest_orf(str,strand)
%function [isostartpos,isostoppos,isoORFlength] = get_longest_orf(str,strand)
% Get the longest open reading frame
%
% Written by: Cheng Soon Ong, 4 October 2006
% Modified on: 12 November 2006
% Modified on: 18 April 2007
% - also consider reading frames where either the start and stop codon is not there.

STARTCODON = 'atg';
STOPCODON = {'taa','tag','tga'};


startpos = strfind(str,STARTCODON)';
stoppos = strfind(str,STOPCODON{1});
stoppos = [stoppos,strfind(str,STOPCODON{2})];
stoppos = [stoppos,strfind(str,STOPCODON{3})];
stoppos = sort(stoppos,'ascend')';





rf_start = mod(startpos,3);
rf_stop = mod(stoppos,3);
rfORFlength = zeros(3,1);
rfORFpos1 = zeros(3,1);
rfORFpos2 = zeros(3,1);
for rf = 0:2
  % get start and stop codons for this reading frame.
  pos1 = startpos(rf_start==rf);
  pos2 = stoppos(rf_stop==rf);
  
  % didn't find any start nor stop codons
  if isempty(pos1) && isempty(pos2)
    rfORFlength(rf+1) = length(str) - mod(length(str),3) + 9;
    rfORFpos1(rf+1) = rf-6;
    rfORFpos2(rf+1) = length(str) - mod(length(str),3) + rf + 3;
  else
    pos1 = [rf-6;pos1];
    pos2 = [pos2;length(str)-mod(length(str),3)+rf+3];
    % for each start codon and find the closest stop codon
    diffpos2pos1 = repmat(nan,size(pos1));
    pos2idx = repmat(nan,size(pos1));
    for ix = 1:length(pos1),
      ORFlength = pos2-pos1(ix);
      ORFlength(ORFlength<=0) = nan;
      assert(all((mod(ORFlength,3)==0) | (isnan(ORFlength))));
      [diffpos2pos1(ix),pos2idx(ix)] = min(ORFlength);
    end
    % find the pair of start and stop codons with longest ORF
    [rfORFlength(rf+1),idx] = max(diffpos2pos1);
    rfORFpos1(rf+1) = pos1(idx);
    rfORFpos2(rf+1) = pos2(pos2idx(idx));
  end
end
% compare the three reading frames and take the longest ORF
[maxORFlength, rf_max] = max(rfORFlength);
rf_max = rf_max - 1;
%startpos = startpos(rf_start == rf_max)
%stoppos = stoppos(rf_stop == rf_max)


%if isempty(startpos) || isempty(stoppos) || isnan(maxORFlength) || (maxORFlength <= 0)
%  isostartpos = nan;
%  isostoppos = nan;
%  isoORFlength = nan;
%  return;
%end

isostartpos = rfORFpos1(rf_max+1);
isostoppos = rfORFpos2(rf_max+1)+2;
isoORFlength = maxORFlength+3;
% some sanity checks
assert(isoORFlength>0);
assert(mod(isoORFlength,3)==0);
assert(isostoppos>isostartpos);
assert(isoORFlength==(isostoppos-isostartpos+1));

% trim ORF length if it goes off the ends.
if isostartpos < 1,
  isoORFlength = isoORFlength + isostartpos;
  isostartpos = 0;
else %isostartpos >= 1,
  assert(all(str(isostartpos:isostartpos+2)==STARTCODON));
end
if isostoppos > length(str)
  isoORFlength = isoORFlength - (isostoppos - length(str));
  isostoppos = length(str) + 1;
else % isostoppos <= length(str)
  assert((all(str(isostoppos-2:isostoppos)==STOPCODON{1}))||...
	 (all(str(isostoppos-2:isostoppos)==STOPCODON{2}))||(all(str(isostoppos-2:isostoppos)==STOPCODON{3})));
end

if strand == '-'
  tstart = length(str) - isostoppos;
  tstop = length(str) - isostartpos;
  isostartpos = tstart;
  isostoppos = tstop;
end
