function [false_discovery, true_positive, false_positive] = tpr_vs_fdr(prediction, label)
% function tpr_vs_fdr(prediction, label)
%
% Compute the true positive rate for each false discovery rate
%
% TPR = TP/(TP+FN)
% FDR = FP/(FP+TP)

assert(all(size(prediction)==size(label)));
assert(all(label==1 | label==-1));

numpred = length(prediction);

[sorted_pred,idx_pred] = sort(prediction,'ascend'); 
sorted_label = label(idx_pred);

true_positive = repmat(nan,1,numpred);
false_positive = repmat(nan,1,numpred);
false_discovery = repmat(nan,1,numpred);

for ix = 1:length(sorted_pred)
  TP = sum(sorted_label(ix:end)>0);
  FN = sum(sorted_label(1:ix)>0);
  TN = sum(sorted_label(1:ix)<0);
  FP = sum(sorted_label(ix:end)<0);
  true_positive(ix) = TP/(TP+FN);
  false_positive(ix) = FP/(FP+TN);
  false_discovery(ix) = FP/(TP+FP);
end


