function [jobinfo, num_crashed] = rproc_wait(jobinfo, pausetime,frac_finished,resub_on) 
% jobinfo=rproc_wait(jobinfo, pausetime,frac_finished,resub_on) 

global rproc_wait_jobinfo

rproc_wait_jobinfo=jobinfo ;

if nargin < 2
  pausetime = 120;
end
if nargin < 3
  frac_finished = 1 ;
end
if nargin < 4
  resub_on = 1 ;
end
if resub_on
  fprintf('\n\ncrashed jobs will be resubmitted by rproc_wait\n') ;
else
  fprintf('\n\ncrashed jobs will not be resubmitted by rproc_wait\n') ;
end


num_jobs = 0 ;
num_crashed = 0 ;
for i=1:length(jobinfo),
  if jobinfo(i).created==1 ;
    if isempty(jobinfo(i).time)
      warning('job created but not submitted yet. ignoring') ;
      jobinfo(i).created=0 ;
    else
      num_jobs=num_jobs+1 ;
    end ;
  end ;
end ;

num_finished=0 ;
while (num_finished<num_jobs*frac_finished) || (num_crashed>0)
  if exist('fflush', 'builtin'), 
    fflush(1) ;
  end ;
  pause(pausetime) ;
  num_finished=0 ;
  num_crashed=0 ;
  crashed_files='log files of crashed jobs:' ;
  for id=1:length(jobinfo)
    cur_finished = rproc_finished(jobinfo(id)) ;
    [still_running, qstat_line]=rproc_still_running(jobinfo(id));
    if cur_finished
      num_finished = num_finished +1 ;
    elseif ~still_running 
      num_finished = num_finished +1 ;
      num_crashed = num_crashed +1 ;
      crashed_files = sprintf('%s\n%s', crashed_files, jobinfo(id).log_fname) ;
      if isempty(jobinfo(id).crashed_time)
        jobinfo(id).crashed_time = now ;
      elseif 24*60*(now-jobinfo(id).crashed_time) > max(3*pausetime/60, 1)  & resub_on,
        [reachedlimit, jobwalltime]=rproc_reached_timelimit(jobinfo(id)) ;
        if reachedlimit, % check whether the job has been killed because it reached the time limit
          fprintf('job has been canceled because it used %1.0fs, but time limit was %1.0fs walltime.\nhence, we increase the time limit to %1.0fs.\n', ...
                  jobwalltime, jobinfo(id).time*60, max(jobinfo(id).time, jobwalltime)*2);
          jobinfo(id).time = max(jobinfo(id).time, jobwalltime/60)*2 ;
        end ;

        fprintf('resubmitting job %s\n', jobinfo(id).log_fname) ;
        jobinfo(id) = rproc_resubmit(jobinfo(id)) ;
        jobinfo(id).crashed_time = [] ;
      end ;
    else
      fprintf('%s', qstat_line) ;
    end ;
  end ;
  fprintf('\n%i of %i jobs finished (%i of them crashed) \n', ...
          num_finished, num_jobs, num_crashed) ;
  if sum(crashed_files==sprintf('\n'))>0,
    fprintf('%s\n', crashed_files) ;
  end ;
  if ~resub_on & num_finished==num_jobs*frac_finished
    break
  end
end ;

pause(2);

