function create_whole_genome_regions(region_idx)


load(['/fml/ag-raetsch/share/projects/genefinding/C_elegans/' ...
      'genome_data/trivial_regions.mat'], 'trivial_regions');
regions = trivial_regions;
clear trivial_regions

fn_out = '/fml/ag-raetsch/share/projects/genefinding/C_elegans/lsl/cat-1/data/test_region_';

addpath ~/svn/projects/genefinding/sensors
paths 

fn_genome_config = regions(1).config;
genome_info = init_genome(fn_genome_config);

if ~exist('region_idx', 'var') || isempty(region_idx),
  region_idx = 1:length(regions);
end

fn_polya_signals = '/fml/ag-raetsch/share/projects/genefinding/C_elegans/sensors/polya/truesites_and_consensus.mat';
tmp = load(fn_polya_signals, 'consensus');
polya_sixmers = tmp.consensus;

% for downsampling of tss and cleave positions
resolution.tss    = 20;
resolution.cleave = 40;

% paramter objects, one for each signal
PARAMS = {};
% field names of signals in nGASP data structures
nGASP_names = {};
% hand-coded offsets to shift signals in place for nGASP
signal_offsets = zeros(2,0);

PARAMS{end+1} = C_elegans_acc_SVM_WDS; % should be C_elegans_acc_SVM_WDS
nGASP_names{end+1} = 'acc';
signal_offsets(:,end+1) = [0; 0];
PARAMS{end+1} = C_elegans_don_SVM_WDS; % should be C_elegans_don_SVM_WDS
nGASP_names{end+1} = 'don';
signal_offsets(:,end+1) = [1; 1];
PARAMS{end+1} = C_elegans_tss_SVM_SPWDSP; % should be C_elegans_tss_SVM_COMBINED
nGASP_names{end+1} = 'tss';
signal_offsets(:,end+1) = [0; 0];
PARAMS{end+1} = C_elegans_tis_SVM_COMBINED;
nGASP_names{end+1} = 'tis';
signal_offsets(:,end+1) = [0; 0];
PARAMS{end+1} = C_elegans_cdsStop_SVM_COMBINED;
nGASP_names{end+1} = 'stop';
signal_offsets(:,end+1) = [-2; -2];
PARAMS{end+1} = C_elegans_polya_SVM_COMBINED;
nGASP_names{end+1} = 'polya';
signal_offsets(:,end+1) = [0; -5];
PARAMS{end+1} = C_elegans_cleave_SVM_SPWDSP; % C_elegans_cleave_SVM_COMBINED
nGASP_names{end+1} = 'cleave';
signal_offsets(:,end+1) = [0; 0];
PARAMS{end+1} = C_elegans_transacc_SVM_WDS; 
nGASP_names{end+1} = 'trans';
signal_offsets(:,end+1) = [0; 0];

assert(length(PARAMS) == length(nGASP_names));
assert(length(PARAMS) == size(signal_offsets,2));

% assert that later acc will be processed BEFORE transacc
assert(strmatch('acc',nGASP_names,'exact') < strmatch('trans',nGASP_names,'exact'));

signal_names = {};
base_dirs    = {};

for i=1:length(PARAMS),
  signal_names{i} = PARAMS{i}.Signal_name;
  tmp = getfield(PARAMS{i}.FN.output_sig, signal_names{i});
  base_dirs{i} = tmp.fn_pred;
  
  assert(isequal(fn_genome_config, ...
                 PARAMS{i}.FN.input.fn_genome_config));
end
assert(length(signal_names) == length(nGASP_names));
clear trivial_regions PARAMS SAVE_BINARY tmp i 

tester_config;
PAR.polya_sixmers = polya_sixmers;
model = make_model(PAR)
clear PAR C

fields = {'output', 'Conf', 'Conf_cum'};

test_blocks = [];
for r=region_idx
  tmp_region = regions(r);
  fprintf('Processing %s%s...\n', tmp_region.chr, tmp_region.strand);
  % fill the field genestr
  tmp_region = load_sequence(regions(r));
  switch tmp_region.strand,
   case '+',
    tmp_region.block_start = regions(r).start;
    tmp_region.block_end   = regions(r).stop;
    tmp_region.offset      = regions(r).start - 1;
   case '-',
    tmp_region.block_start = regions(r).stop;
    tmp_region.block_end   = regions(r).start;
    tmp_region.offset      = regions(r).stop + 1;
  end
  tmp_region.genestr = tmp_region.seq;

  tmp_region.details.pos    = [];
  tmp_region.details.score  = [];
  tmp_region.details.output = [];
  
  fprintf('Filling details...\n');
  tic
  for s=1:length(signal_names),
    fprintf('  Processing signal %s...\n', signal_names{s});
    tmp_region = retrieve_signals(tmp_region, base_dirs{s}, ...
                                  signal_names{s}, fields);

    tmp = getfield(tmp_region.Signals, signal_names{s});
    switch tmp_region.strand,
     case '+',
      pos_region = tmp.pos + signal_offsets(1,s);
     case '-',
      pos_region = tmp.pos + signal_offsets(2,s);
    end
    tmp_region.details.pos = setfield(tmp_region.details.pos, ...
                                      nGASP_names{s}, pos_region);
    tmp_region.details.score = setfield(tmp_region.details.score, ...
                                      nGASP_names{s}, tmp.Conf_cum);
    %%% TODO add details.sixmer.polya vector
    switch signal_names{s}, 
     case {'tss', 'cleave'},
      side = floor(getfield(resolution,signal_names{s})/2);
      fprintf('Reducing resolution to one prediction per %i nucleotides\n', ...
              getfield(resolution,signal_names{s}));
      idx = 1:getfield(resolution,signal_names{s}):length(pos_region);
      cc = zeros(1,length(idx));
      pp = zeros(1,length(idx));
      for j=1:length(idx),
        if mod(j,1000)==0, fprintf(' %i  \r', j), end
        win = max(1,idx(j)-side):min(idx(j)+side,length(pos_region));
        neighbour = find((pos_region(win)>=pos_region(idx(j))-side) ...
                         & (pos_region(win)<pos_region(idx(j))+side)) ;

        [mv, mi] = max(tmp.Conf_cum(win(neighbour)));
        assert(tmp.Conf_cum(win(neighbour(mi)))==mv);
  
        pp(j) = pos_region(win(neighbour(mi)));
        cc(j) = tmp.Conf_cum(win(neighbour(mi)));
      end
      tmp_region.details.pos = setfield(tmp_region.details.pos, ...
                                        nGASP_names{s}, pp');
      tmp_region.details.score = setfield(tmp_region.details.score, ...
                                          nGASP_names{s}, cc');
      clear pp cc mi mv win neighbor idx side
     case 'acc',
      % transacc scores are set to acc scores, nGASP field trans
      % has the specific trans-splice site scores
      tmp_region.details.pos.transacc   = tmp_region.details.pos.acc;
      tmp_region.details.score.transacc = tmp_region.details.score.acc;

      % also provide raw SVM outputs
      tmp_region.details.output = setfield(tmp_region.details.output, ...
                                          nGASP_names{s}, tmp.output);
     case 'don',
      % also provide raw SVM outputs
      tmp_region.details.output = setfield(tmp_region.details.output, ...
                                       nGASP_names{s}, tmp.output);
     % needed or not ???
     case 'transacc',
      tmp_region.details.score.trans = tmp_region.details.score.acc ...
          .*tmp.output;
    end
    tmp_region = rmfield(tmp_region, 'Signals');
    clear tmp
  end
  toc
  
  fprintf('Generating seq and pos...\n');
  tic
  [tmp_region.seq, tmp_region.pos] = gen_seq(tmp_region, model);
  toc
  
  tmp_region = rmfield(tmp_region, 'start');
  tmp_region = rmfield(tmp_region, 'stop');
  test_block = tmp_region;

  fprintf('Saving data...\n');
  save('-v7.3', sprintf('%s%s%s_%s.mat', fn_out, test_block.chr, ...
                        test_block.strand, datestr(now,'yyyy-mm-dd')), ...
       'test_block');
  
  clear tmp_region
end

% load /fml/ag-raetsch/share/projects/nGASP/LSL/test_data/cat_1/blocks_2006-12-19.mat

% eof