function genes = parse_genes_from_annotation(genome_info,query,query_not)
% genes = parse_genes_from_annotation(genome_info,genes,query,query_not)
 
if nargin<2
  query = 'Coding_transcript' ;
end


genes = [];

if isempty(genome_info.annotation_fnames)
  warning('no anootation available')
  genes=init_genes;
  return
end

for fi=1:length(genome_info.annotation_fnames)
  fname = genome_info.annotation_fnames{fi};
  tmpfname = tempname ;
  if nargin<3 
    unix(sprintf('grep %s %s | sort -k 9 > %s', query, fname, tmpfname)) ...
    % check locale; LC_ALL=C to sort (setenv LC_ALL C)
  else
    unix(sprintf('grep %s %s | grep -v %s | sort -k 9 > %s', query, fname, query_not,tmpfname)) ;
  end
  
  cds = [] ;
  fd = fopen(tmpfname, 'r+') ;
  gene_names = {};
  
  cnt = 0;
  while ~feof(fd),
    line = fgetl(fd) ; 
    if ~ischar(line),  break; end ;
    if length(line)>10000, keyboard; end ;
    elems = separate(line) ;
    switch lower(elems{3}),
     case 'transcript'
      type=5 ;
     case 'coding_exon'
      type=3 ;
     case 'five_prime_utr'
      type=2 ;
     case 'three_prime_utr'
      type=4 ;
     case 'exon'
      type=0 ;
     case 'intron'
      type=-1 ;
     otherwise
      continue ;
    end ;
    if ~any(type==[2 3 4 5]), continue ; end ;
    if ~any(ismember(genome_info.contig_names, elems{1})), 
      fprintf('not found: %s\n',elems{1});
      continue ; 
    end ;
    if equal(elems{9}(1:12),'Transcript "')
      ii = strfind(elems{9},'Prediction_status');
      if ~isempty(ii)
        pred_stat = elems{9}(ii:end) ;
        if strfind(pred_stat,'Confirmed')
          conf = 1 ;
        elseif strfind(pred_stat,'Partially_confirmed')
          conf = 0   ;
        elseif strfind(pred_stat,'Predicted')
          conf = -1 ;
        else
          keyboard
        end
      else
        conf = -1 ;
      end
      %%% find transcript name
      ii = find(elems{9}=='"') ; 
      ti1 = elems{9}(ii(1)+1:ii(2)-1) ;
      ti = ti1;
      if ti(end)>='a' & ti(end)<='z', 
        ti(end)=[] ;
      end ;
      %%% find gene name
      ii = strfind(elems{9},'Gene');
      if ~isempty(ii)
        ii1 = strfind(elems{9}(ii(1):end),'"');
        gi = elems{9}(ii(1)+ii1(1):ii(1)+ii1(2)-2);
      else
        gi = [];
      end
      %%% find cds name
      ii = strfind(elems{9},'CDS');
      if ~isempty(ii)
        ii1 = strfind(elems{9}(ii(1):end),'"');
        ci = elems{9}(ii(1)+ii1(1):ii(1)+ii1(2)-2);
      else
        ci = [];
      end
    else 
      error(sprintf('do not understand "%s"', elems{9})) ;
    end ;
  
    if length(genes)>0 & equal(ti,genes(end).name),
      ti_idx = length(genes) ;
    else
      ti_idx = [] ;
    end ;
    
    if isempty(ti_idx)
      genes(end+1).id =  length(genes)+1;
      genes(end).name = ti ;
      genes(end).gene_name = gi ;
      genes(end).gene_name2 = {};
      genes(end).transcripts = {} ;
      genes(end).transcript_confirmed = [];  
      genes(end).terminal_info = [] ;
      genes(end).exons = {} ;
      genes(end).exons_confirmed = {} ;
      assert(elems{7}=='+' || elems{7}=='-') ;
      genes(end).strand = char(elems{7}) ;
      genes(end).chr = elems{1} ;
      genes(end).chr_num = strmatch(elems{1},genome_info.contig_names,'exact') ;
      assert(~isempty(genes(end).chr)) ;
      gene_names{end+1} = ti;
      ti_idx = length(genes);
    end ;
    if isempty(genes(ti_idx).gene_name)
      genes(end).gene_name = gi ;
    end
    if ~isempty(gi) &~isempty(genes(ti_idx).gene_name)
      if ~isequal(genes(ti_idx).gene_name,gi)
        %keyboard
        genes(ti_idx).gene_name2{end+1}=gi;
        cnt = cnt+1 ;
      end
    end
    if ~isempty(ci)
      assert(~isempty(strmatch(ti,ci))|~isempty(strmatch(ci,ti))) ;
    end
    idx = find(ismember(genes(ti_idx).transcripts,ti1)) ;
    if isempty(idx) ,
      genes(ti_idx).transcripts{end+1} = ti1; 
      genes(ti_idx).transcript_confirmed(end+1) = -1; 
      genes(ti_idx).exons{end+1} = [] ;
      genes(ti_idx).exons_confirmed{end+1} = [] ;
      genes(ti_idx).terminal_info(:,end+1) = [0;0] ;
      idx = length(genes(ti_idx).transcripts) ;
    end ;
    assert(genes(ti_idx).strand == elems{7} ) 
    if type==5
      genes(ti_idx).transcript_confirmed(idx) = conf;
    else
      genes(ti_idx).exons{idx}(end+1,:) = [str2num(elems{4}), str2num(elems{5}), type];
    end
    if (mod(length(genes),10)==0) fprintf(1,'%i\r',length(genes)); end;
    fprintf(1,'%i\r',length(genes))
  end ;
  fclose(fd) ;
  disp(length(genes));
  cnt
end
for i=1:length(genes)
  genes(i).info_type = [];
  for j=1:length(genes(i).transcripts)
    genes(i).exons{j} = unique(genes(i).exons{j},'rows');
    genes(i).info_type = [genes(i).info_type, -1];
    genes(i).exons_confirmed{j} = ones(size(genes(i).exons{j},1),2).*genes(i).transcript_confirmed(j);
  end
  if all(genes(i).transcript_confirmed==1)
    genes(i).confirmed = 1 ;
  elseif all(genes(i).transcript_confirmed==-1)
    genes(i).confirmed = -1 ;
  elseif all(genes(i).transcript_confirmed==0)
    genes(i).confirmed = 0 ; % partially confirmed
  elseif any(genes(i).transcript_confirmed)
    genes(i).confirmed = 0 ; % partially confirmed
  else
    keyboard
  end
  % genes(i).confirmed = any(genes(i).transcript_confirmed);
end
