function [genes_signals,genes_merged] = prepare_complete_genes(PAR)

  
  
%paths
if  ~isfield(PAR,'add_signals')
  PAR.add_signals.tss = 1; 
  PAR.add_signals.transacc = 0;
  PAR.add_signals.cleave = 1 ; 
  PAR.add_signals.polya = 1;
  PAR.add_signals.tis = 1;
  PAR.add_signals.Stop = 1;
end

input_sig = getfield(PAR.FN.input_sig, PAR.Signal_name);
Signal = getfield(PAR.Signals, PAR.Signal_name);

%% PREPARE TRANSACC (transsplice) from annotation
if PAR.add_signals.transacc
  if (PAR.label_source.transacc.anno_SL1 &~fexist(sprintf('%s.mat',PAR.FN.input_sig.transacc.fn_anno_SL1))) |...
        (PAR.label_source.transacc.anno_SL2 &~fexist(sprintf('%s.mat',PAR.FN.input_sig.transacc.fn_anno_SL2)))  
    fprintf(1,'Preparing transacceptor sites \n');
    parse_transacc_from_anno_wormbase(PAR)
  end
end


%% PREPARE CLEAVE AND TSS FROM EST MATCHES
if PAR.add_signals.cleave |  PAR.add_signals.tss
  if (PAR.label_source.tss.est_clusters &~fexist(PAR.FN.input_sig.tss.fn_est_cluster))  |...
        (PAR.label_source.cleave.est_clusters &~fexist(PAR.FN.input_sig.cleave.fn_est_cluster))  
    fprintf(1,'Preparing cleavage/tss sites from est cluster\n');
    cluster_EST_matches(PAR)
  end
  %% PREPARE CLEAVE AND TSS  FROM FULLENGTH
  if PAR.label_source.tss.fulllength  &~fexist(PAR.FN.input_sig.tss.fn_fulllength )  |...
        PAR.label_source.cleave.fulllength &~fexist(PAR.FN.input_sig.cleave.fn_fulllength )  
    fprintf(1,'Preparing cleavage/tss sites from fulllength \n');
    parse_fulllength_cDNA(PAR)
  end
end

%% PREPARE TIS AND STOP FROM ANNOTATION
if PAR.add_signals.tis |  PAR.add_signals.cdsStop
  if (PAR.label_source.tis.anno &~fexist(PAR.FN.input_sig.tis.fn_anno))  |...
        (PAR.label_source.cdsStop.anno &~fexist(PAR.FN.input_sig.cdsStop.fn_anno))  &&...
        PAR.label_source.use_confirmed
    fprintf(1,'Preparing tis/cdsStop sites \n');      
    mk_cds(PAR);
  end
end 

if PAR.add_signals.polya&PAR.label_source.polya.from_cleave
  %% PREPARE POLYA 
  fprintf(1,'Preparing polya sites \n');
  if ~fexist(PAR.FN.input_sig.polya.fn_polya) 
    assert(isempty(PAR.Signals.polya.consensus{:}))
    find_all_polyas(PAR);
  elseif ~ismember('polya',who('-file',PAR.FN.input_sig.polya.fn_polya))
    load(PAR.FN.input_sig.polya.fn_polya,'consensus')
    assert(isequal(consensus, PAR.Signals.polya.consensus))
    clear consensus
    find_all_polyas(PAR);
  else
    fprintf(1,'\t File already exists \n');
    Old = load(PAR.FN.input_sig.polya.fn_polya,'PAR');
    diff_fields = compare_PAR(PAR.label_source.polya,Old.PAR.label_source.polya);
    if ~isempty(diff_fields)
      %error(['polya file exists, generated from differnt input files / ' ...
      %       'with different parameters'])  
    end
    clear Old diff_fields
  end
end

% build genes_signals according to specifications in PAR.label_source
% (e.i. from annotation, confirmed_sequences, tss,cleave,polya)
if  fexist(PAR.FN.output.fn_genes_signals)
  Old = load(PAR.FN.output.fn_genes_signals,'PAR'); 
  diff_fields = compare_PAR(PAR.FN.input,Old.PAR.FN.input) ;
  if ~isempty(diff_fields)
    warning('genes file exists, generated from differnt input files\n')  
  end
  diff_fields = compare_PAR(PAR.info_genes,Old.PAR.info_genes) ;
  if ~isempty(diff_fields)
    warning('genes file exists, info_genes different\n')  
  end
  diff_fields = compare_PAR(PAR.label_source,Old.PAR.label_source) ;
  if ~isempty(diff_fields)
    warning('genes file exists, label sources are differnt!\n')  
  end
  diff_fields = compare_PAR(PAR.add_signals,Old.PAR.add_signals,1) ;
  fprintf(1,'loading genes from file\n')
  genes_signals = load_struct(PAR.FN.output.fn_genes_signals,'genes');
  if ~isempty(diff_fields)
    fprintf('other signals already included\n')
    do_process=0;
    for i=1:length(diff_fields)
      if PAR.add_signals.(diff_fields{i})
        do_process=1;
      end
    end
    if do_process
      fprintf('adding signals\n')
      PAR.add_signals
      genes_signals = process_genes(PAR,genes_signals,Old.PAR.add_signals);
    end
  end
  clear diff_fields Old
else
  fprintf(1,'generating genes\n')
  genes_signals = process_genes(PAR);
end



% merge all available information
% (e.i. from annotation, confirmed_sequences, tss,cleave,polya)
if (PAR.label_source.use_anno==1&PAR.label_source.use_confirmed==1)
  genes_merged = genes_signals;
elseif fexist(PAR.FN.output.fn_genes_merged)
  genes_merged = load_struct(PAR.FN.output.fn_genes_merged,'genes');
elseif fexist(PAR.FN.output.fn_genes_anno)& fexist(PAR.FN.output.fn_genes_conf)
  fprintf(1,'generating merged genes\n')
  PAR.label_source.use_anno=1;
  PAR.label_source.use_confirmed=1;
  genes_merged = process_genes(PAR);
else
  genes_merged = genes_signals;
end

