function genes = process_gff3_genes(genes,genome_info,model) 
% genes = process_gff3_genes(genes,genome_info,model)  

  
if nargin<3
  warning('no model given!')
  m.exon = 0;
  m.transexon = 1;
  m.utr5exon = 2;
  m.cds_exon = 3;
  m.utr3exon = 4; 
else
  m.exon  = 0;
  if isfield(model.segments,'transexon')
    m.transexon = model.segments.transexon;   
  else
    m.transexon=-1;
  end
  m.utr5exon = model.segments.utr5exon;
  m.cds_exon = model.segments.cds_exon;
  m.utr3exon = model.segments.utr3exon ; 
  clear model
end
m.exon=0;

fprintf('processing genes\n')
for i=1:length(genes)
  if (mod(i,100)==0), 
    fprintf('%i/%i\r',i,length(genes))
  end
  genes(i).id = i;
  if exist('genome_info')&(~isfield(genes,'chr_num')|isempty(genes(i).chr_num))
    genes(i).chr_num= strmatch(upper(genes(i).chr),upper(genome_info.contig_names),'exact');
    assert(~isempty(genes(i).chr_num))
  end
  if isempty(genes(i).exons)
    keyboard
%    genes(i).transcripts = {'dummy'};
%    genes(i).exons={[genes(i).start genes(i).stop 3]};
  end
  genes(i).tis = [] ;
  genes(i).cdsStop = [] ;
  genes(i).cds_exons = {} ;
  genes(i).utr5_exons = {} ;
  genes(i).utr3_exons = {} ;
  gene = genes(i) ;
  for j=1:length(gene.transcripts)
    gene.exons_confirmed{j} =[]; 
    exons = gene.exons{j} ;
    if size(exons,2)<3, 
      %warning('no exon meta info, identifying as cds_exon') ;
      exons(:,3)=m.cds_exon ; 
    end ;
    [exons(:,1),idx] = sort(exons(:,1)) ;
    exons(:,2:end) = exons(idx,2:end) ;
    %assert(all((exons(:,2)-exons(:,1))>=0)) ;
    if ~all((exons(:,2)-exons(:,1))>=0),
      warning('gene invalid') ;
      genes(i).is_valid(j)=0 ;
      continue ;
    end ;
    if ~isempty(exons),
      if gene.strand=='+',
        gene.tss(j) = exons(1,1);
        gene.cleave(j) = exons(end,2);
      else
        gene.tss(j) = exons(end,2);
        gene.cleave(j) = exons(1,1);
      end ;
    end
    idx_trans = find(exons(:,3)==m.transexon); 
    if ~isempty(idx_trans)
      exons(idx_trans,:) = [];
      warning('transexon removed')
    end
    %remove redundant information 
    if all(ismember([m.exon,m.cds_exon],unique(exons(:,3))))
      idx_cds = find(exons(:,3)==m.cds_exon);
      idx_exon = find(exons(:,3)==m.exon);
      [tmp,tmp,idx2] = intersect(exons(idx_cds,1:2),exons(idx_exon,1:2),'rows')
      exons(idx_exon(idx2),:)=[];
    end
    %% overlapping exons are merged
    if ~all((exons(2:end,1)-exons(1:end-1,2))>=0)
      keyboard
      warning('overlapping exons found within one transcript\n')
      idx = find(exons(2:end,1)-exons(1:end-1,2)<0) ;
      if (exons(idx,3)==exons(idx+1,3))
        exons(idx,2) =  exons(idx+1,2);
        exons(idx+1,:) =[];
        assert(all((exons(:,2)-exons(:,1))>=0));
        assert(all((exons(2:end,1)-exons(1:end-1,2))>=0));
      end
    end
    gene.cds_exons{j} = exons(find(exons(:,3)==m.cds_exon),:) ;
    gene.utr5_exons{j} = exons(find(exons(:,3)==m.utr5exon),:) ;
    gene.utr3_exons{j} = exons(find(exons(:,3)==m.utr3exon),:) ;
 
    if gene.strand=='+',
      if size(gene.utr3_exons{j},1)>1& gene.utr3_exons{j}(1,2)-gene.utr3_exons{j}(1,1)<3
        gene.utr3_exons{j} = [gene.utr3_exons{j}(1,1) gene.utr3_exons{j}(end,2) gene.utr3_exons{j}(1,3)];
        idx = find(exons(:,3)==m.utr3exon);
        exons(idx(1),2) = exons(idx(end),2);
        exons(idx(2:end),:) = [];
        warning('TAGT case: invalid donor!')
      end
      %determine phase of cds exon
      gene.cds_exons{j}(1,3) = 0;
      cds_length = 0;
      for e=2:size(gene.cds_exons{j},1)
        cds_length = cds_length + gene.cds_exons{j}(e-1,2)-gene.cds_exons{j}(e-1,1);
        phase = mod(cds_length,3);
        switch phase
         case 0
          gene.cds_exons{j}(e,3) = phase;
         case 1
          gene.cds_exons{j}(e,3) = 2;
         case 2
          gene.cds_exons{j}(e,3) = 1;
        end
      end
    
      % assert(any(exons(1,3)==[2 3])) ;
      % assert(any(exons(end,3)==[3 4])) ;
      if ~isempty(exons) && exons(1,3)==m.utr5exon, %% 'five_prime_utr' 
        idx2 = find(exons(:,3)==m.utr5exon,1,'last') ;
        idx3 = find(exons(:,3)==m.cds_exon,1,'first') ;
        tis = exons(idx3,1) ;
        if exons(idx2,2)==exons(idx3,1)
          exons_ = exons(1:idx2,:) ;
          exons_(idx2,2:3)=[exons(idx3,2) m.cds_exon];
          exons_=[exons_; exons(idx3+1:end,:)] ;
          exons=exons_ ;
        else 
          %keyboard
        end ;
      else
        if ~isempty(exons),
          tis=exons(1,1) ;
        end ;
      end ;
      if ~isempty(exons) && exons(end,3)==m.utr3exon, %% three_prime_utr 
        idx3=find(exons(:,3)==m.cds_exon,1,'last') ; %% 'coding_exon' type=3 ;
        idx4=find(exons(:,3)==m.utr3exon,1,'first') ; 
        cdsStop = exons(idx3,2) ;
        if exons(idx3,2)==exons(idx4,1),
          exons_ = exons(1:idx3,:) ;
          exons_(idx3,2)=exons(idx4,2) ;
          exons_=[exons_; exons(idx4+1:end,:)] ;
          exons=exons_ ;
        else
          %keyboard
        end ;
      else
        if ~isempty(exons),
          cdsStop=exons(end,2) ;
        end ;
      end ;
      cdsStop=cdsStop-3 ;
    else % gene.strand=='-',
      if size(gene.utr3_exons{j},1)>1& gene.utr3_exons{j}(end,2)-gene.utr3_exons{j}(end,1)<3
        gene.utr3_exons{j} = [gene.utr3_exons{j}(1,1) gene.utr3_exons{j}(end,2) gene.utr3_exons{j}(1,3)];
        idx = find(exons(:,3)==m.utr3exon);
        exons(idx(end),1) = exons(idx(1),1);
        exons(idx(1:end-1),:) = [];
        warning('TAGT case: invalid donor!')
      end
      
      if ~isempty(gene.cds_exons{j})
        gene.cds_exons{j}(end,3) = 0;
        cds_length = 0;
        for e=size(gene.cds_exons{j},1)-1:-1:1
          cds_length = cds_length + gene.cds_exons{j}(e+1,2)-gene.cds_exons{j}(e+1,1);
          phase = mod(cds_length,3);
          switch phase
           case 0
            gene.cds_exons{j}(e,3) = phase;
           case 1
            gene.cds_exons{j}(e,3) = 2;
           case 2
            gene.cds_exons{j}(e,3) = 1;
          end
        end
      end
     
      % assert(any(exons(end,3)==[2 3])) ;
      % assert(any(exons(1,3)==[3 4])) ;
      if ~isempty(exons) && exons(1,3)==m.utr3exon,%% three_prime_utr 
        idx4=find(exons(:,3)==m.utr3exon,1,'last') ;
        idx3=find(exons(:,3)==m.cds_exon,1,'first') ;
        cdsStop = exons(idx3,1) ;
        if exons(idx4,2)==exons(idx3,1),
          exons_ = exons(1:idx4,:) ;
          exons_(idx4,2:3)=[exons(idx3,2) m.cds_exon] ;
          exons_=[exons_; exons(idx3+1:end,:)] ;
          exons=exons_ ;
        else
          %keyboard
        end ;
      else
        if ~isempty(exons),
          cdsStop=exons(1,1) ;
        end ;
      end ;
      cdsStop=cdsStop+3 ;
      if ~isempty(exons) && exons(end,3)== m.utr5exon,%% five_prime_utr 
        idx3=find(exons(:,3)== m.cds_exon,1,'last') ;
        idx2=find(exons(:,3)== m.utr5exon,1,'first') ;
        tis = exons(idx3,2) ;
        if exons(idx3,2)==exons(idx2,1),
          exons_ = exons(1:idx3,:) ;
          exons_(idx3,2)=exons(idx2,2) ;
          exons_=[exons_; exons(idx2+1:end,:)] ;
          exons=exons_ ;
        end ;
      else
        if ~isempty(exons),
          tis=exons(end,2) ;
        end ;
      end ;
      exons(2:end,1) = exons(2:end,1);
    end ; %strand==-'
    exons(:,3)=[] ;
    gene.exons{j}=exons ;
    
    if ~isempty(tis)
      gene.tis(end+1)=tis ;
    end
    if ~isempty(cdsStop)
      gene.cdsStop(end+1)=cdsStop ;
    end
  end ; %%loop over transcripts
  genes(i)=gene ;
end ;%%loop over genes
