function genes = read_genes_from_gff3(gff_fname,save_fname,genome_info,source,by_contig,run_locally,is_gff2)
% genes = read_genes_from_gff3(gff_fname,save_fname,genome_info,source,by_contig,run_locally)
% 
% genes = parse_gff3(gff_fname, save_fname, [], source);

if nargin<6
  run_locally=1;
end
if nargin<7
  is_gff2=0;
end

if is_gff2
  TYPE.short_names = {'CDS','exon','coding_exon'};
  TYPE.IDs = {'1' '2' '3'};
  TYPE.hierachies = [2 3 3];
else
  TYPE=[];
end

[ret nofgenes] = unix(['grep gene ' gff_fname '| wc -l']);
assert(ret==0)
nofgenes=str2num(nofgenes);

genes = init_genes(max(nofgenes,1));
len = 0;
% keyboard
if by_contig
  for contig=1:length(genome_info.contig_names)
    cname = genome_info.contig_names{contig};
    if ~run_locally
      Data.cname = cname;
      Data.source=source;
      tmp_files{contig}=['~' tempname];
      Data.save_fname=tmp_files{contig};
      Data.gff_fname=gff_fname;
      Data.TYPE = TYPE;
      Data.genome_info=genome_info;
      options.addpaths={'~/svn/projects/genefinding/data_processing_sensors'} ;
      jobinfo(contig)=rproc('parse_gff3_rproc',Data,2000,options,10*60);
    else
      new_genes =  parse_gff3(gff_fname,save_fname,source,cname,TYPE);
      genes(len+1:len+length(new_genes))=new_genes;
      len = len + length(new_genes);
      clear new_genes
    end
  end
  if ~run_locally
    jobinfo = rproc_wait(jobinfo,30,1,0);
    for contig=1:length(genome_info.contig_names)
      l = load(tmp_files{contig},'genes'); 
      genes(len+1:len+length(l.genes))=l.genes;
      len = len + length(l.genes);
      clear l
    end
  end
  genes(len+1:end)=[]; 
  %for j=1:len, assert(~isempty(genes(j).id)), end 
  for j=1:len, genes(j).id=j; end 
  
  save(save_fname, 'genes')                                                                                                                                   
else
  %  keyboard
  genes =  parse_gff3(gff_fname,save_fname,source,[],TYPE);
end
% keyboard
if run_locally
  genes = process_gff3_genes(genes,genome_info) ;
  if is_gff2
    genes = convert_gff2gff3_genes(genes);  
  end
  
  which_checks.exons_sorted = 1;
  which_checks.intron_length = 1;
  which_checks.splicesites = 1;
  which_checks.orf = 1;
  which_checks.gene_length = 1;
  which_checks.graph = 0;
  which_checks.transacc = 0;
  
  genes = check_genes(genes, genome_info,which_checks);
  
  
  % build splicegraphs
  build_splice_graph
  genes = infer_splice_graph_caller(genes)
  fprintf('Sorting exons...\n');
  for i=1:length(genes)
    [dummy,exon_order] = sort(genes(i).splicegraph{1}(1,:),2,'ascend');
    genes(i).splicegraph{1} = genes(i).splicegraph{1}(:,exon_order);
    genes(i).splicegraph{2} = genes(i).splicegraph{2}(exon_order,exon_order);
  end
  genes = alt_const(genes);
  
  save(save_fname,'genes')
end