function gunion = genes_union(g1,g2)
% function gunion = genes_union(g1,g2)
%
% Perform a "set union" operation on genes.
% Assume that each gene has the following fields:
%  chr, strand, start, stop
% These are the only fields returned.
%
% Two genes are merged if they have the same chr and strand,
% and their start and stop points overlap.
%
% Problem: NOT SYMMETRIC

for ix2 = 1:length(g2)
  g2(ix2).merged = 0;
end

for ix1 = 1:length(g1)
  gunion(ix1).chr = g1(ix1).chr;
  gunion(ix1).strand = g1(ix1).strand;
  for ix2 = 1:length(g2)
    if strcmp(g1(ix1).chr,g2(ix2).chr) && (g1(ix1).strand==g2(ix2).strand)...
	  && segment_overlap(g1(ix1).start,g1(ix1).stop,g2(ix2).start,g2(ix2).stop)
      gunion(ix1).start = min(g1(ix1).start,g2(ix2).start);
      gunion(ix1).stop = max(g1(ix1).stop,g2(ix2).stop);
      g2(ix2).merged = 1;
    else
      gunion(ix1).start = g1(ix1).start;
      gunion(ix1).stop = g1(ix1).stop;
    end
  end
end

for ix2 = 1:length(g2)
  if ~g2(ix2).merged
    gunion(end+1).chr = g2(ix2).chr;
    gunion(end).strand = g2(ix2).strand;
    gunion(end).start = g2(ix2).start;
    gunion(end).stop = g2(ix2).stop;
  end
end

