function genes = parse_gene_pred_from_gff_file(annotation_fname, source)
% genes = parse_gene_pred_from_gff_files(annotation_fnames, source)
  
genes = [];
gene_cnt = 0;
padded_id_len = 20;
source = lower(source);

disp(annotation_fname);

num_cds = 21681;
res_file = ['/fml/ag-raetsch/share/projects/nGASP/gene_finders/' ...
            source '_' num2str(num_cds) 'cds.mat'];

if fexist(res_file)
  load(res_file);
  fprintf('\nloaded gene data from %s\n', res_file);
else
  % first parse in which only predictions of genes (CDS) are
  % extracted (no introns or exons)
  [fd msg] = fopen(annotation_fname, 'r');
  disp(msg);
  while ~feof(fd),
    line = fgetl(fd); 
    if line(1)=='#',
      %disp(line) 
      continue
    end
    if ~ischar(line),
      break
    end
    
    elems = separate(line);
    
    tmp = separate(elems{9}, ';');
    if ~equal(lower(elems{2}),source)
      %disp(line)
      continue
    end
    
    identifier = '';    
    for i=1:length(tmp),
      if isequal(tmp{i}(1:3), 'CDS'),
        idx = find(tmp{i}=='"');
        assert(length(idx)==2);
        identifier = tmp{i}(idx(1)+1:idx(2)-1);
      end 
    end
    
    if ~equal(elems{3}, 'CDS'),
      continue
    end
    
    genes(end+1).name = identifier;
    assert(~isempty(genes(end).name)) ;
    
    genes(end).chr = elems{1};
    assert(~isempty(genes(end).chr));
    
    genes(end).start = str2num(elems{4});
    genes(end).stop = str2num(elems{5});
    assert(elems{7}=='+' || elems{7}=='-') ;
    genes(end).strand = elems{7} ;
    genes(end).exons = {} ;
    
    gene_cnt = gene_cnt+1;
    fprintf('  parsed %i gene predictions\r', gene_cnt);
  end % end first parse
  fclose(fd);
  fprintf('\n\n');
  
  % create a more accessible representations of gene_identifiers
  % for quick lookups
  gene_identifiers = char(zeros(1,19+padded_id_len*gene_cnt));
  for g=1:length(genes),
    gene_identifiers((g)*20:(g+1)*20-1) = [genes(g).name ...
                        char(zeros(1,padded_id_len-length(genes(g).name)))];
  end

  save(res_file, 'genes', 'gene_identifiers');
end

num_segments = 350539;
res_file = ['/fml/ag-raetsch/share/projects/nGASP/gene_finders/' ...
            source '_' num2str(num_segments) 'exons.mat'];

if fexist(res_file)
  load(res_file);
  fprintf('\nloaded segment data from %s\n', res_file);
else
  % parse file again this time looking for exons and introns
  [fd msg] = fopen(annotation_fname, 'r');
  disp(msg);
  segment_cnt = 0;
  while ~feof(fd),
    line = fgetl(fd); 
    if line(1)=='#',
      %disp(line) 
      continue
    end
    if ~ischar(line),
      break
    end
    
    elems = separate(line);
    
    tmp = separate(elems{9}, ';');
    if ~equal(lower(elems{2}),source)
      %disp(line)
      continue
    end  
    identifier = '';    
    for i=1:length(tmp),
      if isequal(tmp{i}(1:3), 'CDS'),
        idx = find(tmp{i}=='"');
        assert(length(idx)==2);
        identifier = tmp{i}(idx(1)+1:idx(2)-1);
      end 
    end
    assert(~isempty(identifier));
    parent = [identifier char(zeros(1,20-length(identifier)))];
    parent_idx = strfind(gene_identifiers, parent)./20;
    
    assert(~isempty(parent_idx) & length(parent_idx)==1);
    assert(equal(identifier,genes(parent_idx).name));
    
    switch lower(elems{3}),
     case 'cds'
      continue % already processed in first parse
     case 'exon'
      type = 0;
     case 'intron'
      type = -1;
     case 'coding_exon'
      type = 3;
     otherwise
      disp(line);
      continue
    end
    
    if isempty(genes(parent_idx).exons),
      genes(parent_idx).exons = cell(1,1);
    end
    genes(parent_idx).exons{1}(end+1,:) ...
        = [str2num(elems{4}), str2num(elems{5}), type];
    
    segment_cnt = segment_cnt+1;
    fprintf('  parsed %i predicted gene segments\r', segment_cnt);
  end % end second parse
  fclose(fd);
  fprintf('\n\n');
  
  save(res_file, 'genes', 'gene_identifiers');
end


% postprocess segments
for g=1:length(genes),
  [dummy perm_idx] = sort(genes(g).exons{1}(:,1));
  exons = genes(g).exons{1}(perm_idx,:);

  if ~(exons(1,1) >= genes(g).start) keyboard; end
  if ~(exons(end,2) <= genes(g).stop) keyboard; end
  
  % determine reading frame and remove CDS exons
  cds_idx = find(exons(:,3)==3);
  if genes(g).strand == '+',
    tis  = min(exons(cds_idx,1));
    stop = max(exons(cds_idx,2));
  else
    tis  = max(exons(cds_idx,2));
    stop = min(exons(cds_idx,1));
  end
  cds_exons = exons(cds_idx,:);
  exons(cds_idx,:) = [];
  
  % remove introns
  intr_idx = find(exons(:,3)==-1);
  if ~isempty(intr_idx),
    if ~(intr_idx(1)>1 & intr_idx(end)<size(exons,1)),
      % "curate" crappy predictions
      if exons(intr_idx(end),1)==exons(intr_idx(end),2) ...
            & any(exons(:,2)==genes(g).stop),
        exons(intr_idx(end),:) = [];
        intr_idx(end) = [] ;
      else
        keyboard
      end
    end
  end
  for i=1:length(intr_idx),
    assert(exons(intr_idx(i),1) == exons(intr_idx(i)-1,2)+1);
    assert(exons(intr_idx(i),2) == exons(intr_idx(i)+1,1)-1);
  end
  exons(intr_idx,:) = [];
  
  genes(g).transcripts{1} = genes(g).name;
  genes(g).strands{1}     = genes(g).strand;
  % trim off CHROMOSOME_ prefix
  genes(g).chr            = genes(g).chr(length('CHROMOSOME_')+1:end);
  genes(g).exons{1}       = exons(:,1:2);
  genes(g).cds_exons{1}   = cds_exons(:,1:2);
  genes(g).tis_pos        = tis;
  genes(g).stop_pos       = stop;
end % end postprocessing


