
addpath ../utils/

%%%%% CHANGE THIS
% load gene struct

%graphfilename = include_preds(info, origpath, fn_genes_test)
%genes = arrayload2(graphfilename, 'genes') ;
genes = alt_const(genes) ;

% initialize output file
dt = datestr(now, 'mmm_dd_yyyy');
%gff_fname = ['/fml/ag-raetsch/share/projects/nGASP/LSL/cat-1/test_predictions/gr_final_linear-30-12-2006/G3A_gene_predictions_category1_V3_' dt '.gff'];
gff_fname = ['/fml/ag-raetsch/share/projects/nGASP/LSL/cat-2/test_predictions/gr_con_final-30-12-2006/G3A_gene_predictions_category2_V2_final_' dt '.gff'];
%gff_fname = ['/fml/ag-raetsch/share/projects/nGASP/LSL/cat-3/test_predictions/gr_final_linear-30-12-2006/G3A_gene_predictions_category3_V3_' dt '.gff'];

tmp = load ('/fml/ag-raetsch/share/projects/nGASP/test_regions.mat');
regions = tmp.test_regions;
clear tmp

% just to compare to blocks
%tmp = load('/fml/ag-raetsch/share/projects/nGASP/LSL/test_predictions/gs-21-12-2006/pred_split=_cleave3.00_genes_corrected.mat');
%orig_blocks = tmp.blocks;
%PAR = tmp.PAR;
%clear tmp

% convert gene struct to blocks
%blocks = struct;
clear blocks
cnt_alt = 0;
for g=1:length(genes),
  gene = genes(g);
  
  reg_seq = regions(gene.reg).seq;
  % first, transform gene into local coordinates
  offset = gene.offset; % this is actually the block offset
  if gene.strand == '+',
    splicegraph = gene.splicegraph;
	splicegraph{1} = splicegraph{1} - offset;
	all_TIS  = gene.cds_exons{1}(1,1)       - offset;
	all_STOP = gene.cds_exons{1}(end,2) + 1 - offset;
	seq      = reg_seq(gene.block_start-offset:gene.block_end-offset);
  else 
    assert(gene.strand == '-');
    splicegraph = gene.splicegraph;
    splicegraph{1} = offset - splicegraph{1};
    % bring exon start and end into right order
    splicegraph{1} = splicegraph{1}(end:-1:1,:); 
    % bring exons into right order
    splicegraph{1} = splicegraph{1}(:,end:-1:1);
    % rearrange edges in splicegraph accordingly
    splicegraph{2} = splicegraph{2}(end:-1:1,end:-1:1); 
    all_TIS  = offset - gene.cds_exons{1}(end,2);
    all_STOP = offset - gene.cds_exons{1}(1,1) + 1;
    seq      = reg_seq(offset-gene.block_start:offset-gene.block_end);
    seq      = reverse_complement(seq);
  end
  assert(length(seq) == abs(gene.block_end-gene.block_start)+1);
  assert(equal(upper(seq(all_TIS:all_TIS+2)), 'ATG'));
  assert(equal(upper(seq(all_STOP:all_STOP+2)), 'TAA') ...
		 || equal(upper(seq(all_STOP:all_STOP+2)), 'TAG') ...
		 || equal(upper(seq(all_STOP:all_STOP+2)), 'TGA'));

  % second, find valid isoforms
  [isoform_segments] = get_isoforms_eval(splicegraph,all_TIS,all_STOP,seq,PAR.model);
  if length(isoform_segments)>1,
	cnt_alt = cnt_alt + 1;
  end

  % third, reconstruct block structures
  block_id = gene.id;
  ginb_idx = str2num(gene.name(strfind(gene.name,'gene=')+5:end));

  block = [];
  block.id = block_id;
  block.reg = gene.reg;
  block.chr = gene.chr;
  block.strand = gene.strand;
  block.block_start = gene.block_start;
  block.block_end   = gene.block_end;
  block.offset      = offset;
  block.genestr     = seq;
  block.prediction  = struct;

  if ~exist('blocks','var') || length(blocks)<block_id || isempty(blocks(block_id)),
    for i=1:length(isoform_segments),
      block.prediction.genes{i,ginb_idx} = isoform_segments{i};
    end
    blocks(block_id) = block;
  else
    assert(blocks(block_id).chr == block.chr);
    assert(blocks(block_id).strand == block.strand);
    assert(blocks(block_id).block_start == block.block_start);
    %assert(blocks(block_id_.block_end == block.block_end));

    for i=1:length(isoform_segments),
      blocks(block_id).prediction.genes{i,ginb_idx} = isoform_segments{i};
    end
  end


  fprintf('  parsed %i genes\r', g);
end
fprintf('\n\nCreated %i blocks\r',length(blocks));
fprintf('\n\nFound  %i alternatively spliced genes\r',cnt_alt);


% initialize ID_cnt
ID_cnt                 = struct;
ID_cnt.id_len          = 7;
ID_cnt.current_gene_id = 0;
ID_cnt.current_mrna_id = 0;
ID_cnt.current_exon_id = 0;
ID_cnt.current_cds_id  = 0;


% initialize outfile
if fexist(gff_fname)
  fprintf('replacing file %s...\n', gff_fname);
  [fd msg] = fopen(gff_fname, 'w+');
  disp(msg);
else
  fprintf('creating file %s...\n', gff_fname);
  [fd msg] = fopen(gff_fname, 'w+');
  disp(msg);
end

fprintf(fd,'##gff-version 3\n');

% name of our genefinder
source = 'G3A';

for b=1:length(blocks),
  fprintf('writing block %i...\n', b);  
  block = blocks(b);
  region_id = block.chr;
  ID_cnt = append_block_pred_to_file(...
      block,gff_fname,source,region_id,...
      ID_cnt,PAR.model,regions);
end

fclose(fd);

ID_cnt
