function [blocks,PAR,P] = pred_path_cleave_genome(PAR)
% [blocks,PAR,P] = pred_path(PAR)
%
%
check_split = 0 ;

cleave_offset = PAR.cleave_offset
fid = fopen( sprintf('%spred.txt',PAR.fn_log),'a+') ;
fprintf(fid,'\n\n pred path \n');
fprintf(fid,'date: %s\n',date);
fprintf(fid,'\n start \n');

%% parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isfield(PAR, 'iteration')
  iteration = 1 ;
  while (1)
    fname = [PAR.fn_train '_iteration' num2str(iteration) '.mat'];
    if ~fexist(fname), iteration=iteration-1; break ; end 
    iteration=iteration+1 ;
  end ;
else
  iteration = PAR.iteration;
end ;
fname = [PAR.fn_train '_iteration' num2str(iteration) '.mat'];
%PAR.fn_train = [PAR.fn_train '_iteration' num2str(iteration) '.mat'];
fprintf('load iteration %i\n', iteration) ;
if check_split,
  load(fname, 'QP', 'TRAIN_PAR', 'num_examples');
  fname = [PAR.fn_train '_iteration' num2str(iteration) '_blocks.mat'];
  blocks_train=load_struct(fname);
  idx = [blocks_train.id] ;
  clear blocks_train
else
  load(fname, 'QP', 'TRAIN_PAR', 'num_examples');
end ;

%% load data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

L=load(PAR.fn_blocks) ;
if isfield(L,'blocks'),
  blocks=L.blocks ;
  clear L;
end ;
if isfield(L,'test_block'),
  if ~isfield(PAR, 'regenerate_seq')
    PAR.regenerate_seq=0 ;
  end ;

  [seq,pos]=gen_seq(L.test_block,PAR.model, PAR.regenerate_seq) ;
  if PAR.regenerate_seq
    L.test_block.seq = seq ;
  else
    L.test_block = rmfield(L.test_block, 'seq') ;
  end ;
  L.test_block.pos = pos ;
  clear seq pos 

  blocks=L.test_block ;
  assert(~isfield(blocks, 'content')) ;

  blocks.content.subset_id=1; 
  blocks.id = PAR.id ;

  % exclude genic regions
  L=load(PAR.fn_mask) ;
  L.mask = any(L.mask) ;
  gene_pos = find(L.mask==1) ;
  good_pos = setdiff(blocks.pos, gene_pos) ;

  num_chunks = 30 ;
  [tmp,idx_good]=intersect(blocks.pos, good_pos) ;
  idx=round(linspace(1,length(idx_good),num_chunks+1)) ;
  idx=idx_good(idx) ;

  blocks.offset=[] ;
  blocks.range=[] ;
  newblocks = blocks([]) ;
  for i=1:length(idx)-1,
    range = max(idx(i)-2,1):min(idx(i+1)+2,length(blocks.pos)) ;
    block = blocks ;
    block.range=range ;
    if PAR.regenerate_seq
      block.seq=blocks.seq(:,range,:) ;
      block.seq(PAR.model.state_ids.seq_start, 1, 1) = 0 ;
      block.seq(PAR.model.state_ids.seq_end, end, 1) = 0 ;
      block.seq(PAR.model.state_ids.intergenic_long, :, 1) = 0 ;
    end ;
    block.offset = max(1,blocks.pos(range(1)) - 200);
    block.pos = blocks.pos(range) - block.offset + 1 ; % +1, otherwise there is a shift of genestr vs. pos
    block.genestr = blocks.genestr(block.offset:min(length(blocks.genestr), ...
                                                    blocks.pos(range(end))+200));

    newblocks(i) = block ;
  end ;
  blocks = newblocks ;
end ;

if isfield(PAR,'fn_splits')
  load(PAR.fn_splits, 'blocks_split') ; 
  block_idx = [blocks_split{PAR.pred_split_idx}] ;
  if check_split
    if ~isempty(intersect(idx, block_idx))
      disp('test and training are overlapping') ;
      keyboard
      block_idx = setdiff([blocks_split{:}], idx) ;
    end ;
  end ;
  blocks = blocks(block_idx) ;
end
fprintf('predicting on %i blocks\n', length(blocks)) ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isfield(TRAIN_PAR.model, 'use_conservation') && TRAIN_PAR.model.use_conservation 
  disp('Converting conservation levels') ;
  for id = 1:length(blocks),
    blocks(id).genestr_conserv = conservation2DNA_alphabet(blocks(id).conservation) ;
  end ;  
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% avoid multiple loading of the big constant parameter file
global GLOBAL_cPAR ;
GLOBAL_cPAR=[] ;

cPAR=[] ;
cPAR.model=TRAIN_PAR.model ;
%cPAR.blocks_fname = ['~/' tempname '.mat'] ;
%cPAR.blocks = blocks ;
[xis, cPAR.parameters] = res2param(QP.res, num_examples, TRAIN_PAR.model, QP);
cPAR.parameters.transitions(find(cPAR.model.a_trans(2,:)==cPAR.model.state_ids.cleave-1)) = cPAR.parameters.transitions(find(cPAR.model.a_trans(2,:)==cPAR.model.state_ids.cleave-1))+cleave_offset ;

P=[] ;
P.cPAR_fname = sprintf('~/tmp/%i-%1.10f.mat', round(rand(1)*1e8), now) ;
assert(~fexist(P.cPAR_fname)) ;
save(P.cPAR_fname, 'cPAR') ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

P.viterbi_nbest = 1;

num = 0 ; model = TRAIN_PAR.model ;
jobinfo = rproc_empty(0) ;
fprintf(fid,'\n submitting %i viterbi jobs ...',length(blocks));
for block_id = 1:length(blocks),
  num = num+1 ;

  %idx = find(blocks(block_id).truth(1).segments(:,3)~=model.segments.intergenic) ;
  %max_len = max(500, max(blocks(block_id).truth(1).segments(idx,2)- ...
  %                       blocks(block_id).truth(1).segments(idx,1))) ;
  %max_len = max_len*1.2+1000 
  P.max_len = 22000 ;

  P.block_id = block_id ;
  P.block = blocks(block_id) ;
  P.fn_save = sprintf('%s_viterbi_block%i_cleave%1.2f.mat', PAR.fn_pred, ...
                      block_id, cleave_offset); 
  P.prediction = 1 ;
  MEMREQ = PAR.RPROC.MEMREQ;
  options = PAR.RPROC.options;
  options.identifier = ['PPCG'  num2str(block_id) '_'] ;
  time_req = P.viterbi_nbest*length(blocks(block_id).genestr)*sqrt(22000)*0.03/60/100 ;
  if ~fexist(P.fn_save)
    jobinfo(end+1) = rproc('gen_paths', P, MEMREQ, options, time_req) ;
    %tic;pred = gen_paths(P) ;toc;keyboard
  else
    fprintf('%s exists\n', P.fn_save) ;
  end ;
end
%return

%[jobinfo, meta_jobinfo] = rproc_submit_batch(jobinfo, PAR.exm_per_batch) ;
%[meta_jobinfo,num_crashed] = rproc_wait(meta_jobinfo, 10);
[jobinfo,num_crashed] = rproc_wait(jobinfo, 10);

unix(['rm ' P.cPAR_fname]) ;

blocks(1).prediction = [] ;
for block_id = 1:length(blocks),
  block = blocks(block_id) ;
  while ~fexist(sprintf('%s_viterbi_block%i_cleave%1.2f.mat', ...
                        PAR.fn_pred, block_id, cleave_offset ))
    fprintf('.') ;
    continue
    pause(10)
  end
  load(sprintf('%s_viterbi_block%i_cleave%1.2f.mat', PAR.fn_pred, ...
               block_id, cleave_offset ),'pred');   
  blocks(block_id).prediction = pred; 
end
if PAR.regenerate_seq
  blocks=rmfield(blocks,'seq') ;
end ;
%blocks=rmfield(blocks,'pos') ;
blocks=rmfield(blocks,'details') ;

save_struct([PAR.fn_pred sprintf('_cleave%1.2f.mat', cleave_offset)], blocks, 'blocks') ;
save([PAR.fn_pred sprintf('_cleave%1.2f.mat', cleave_offset)], '-append', 'PAR', 'P') ;
