function write_genes_cds2fasta(genes,genome_info,cds_fn,protein_fn)

% write_fasta(genes,genome_info,cds_fn,protein_fn)
%   
% loads mRNA sequence for all genes and writes it to a fasta file (cds_fn).
% If protein_fn is specified, the sequences will also be translated and
% written to the fasta file protein_fn.
% 
% INPUT - genes required fields:  genes.name
%                                 genes.chr 
%                                 genes.strand
%                                 genes.cds_exons
%   
%        - genome_info
%        - cds_fn 
%        - protein_fn (optional)
% 
% SEE ALSO  write_fasta, load_genomic_2
  
[fd1 msg] = fopen(cds_fn, 'w+');
if nargin>4
  [fd2 msg] = fopen(protein_fn, 'w+');
end
for i=1:length(genes)
  for j=1:length(genes(i).cds_exons)
    cds_exons = genes(i).cds_exons{j};
    if genes(i).strand=='+'
      cds_exons(:,2) = cds_exons(:,2)-1;
    else
      cds_exons(:,1) = cds_exons(:,1)+1;
    end
    cds = load_genomic_2(genes(i).chr, char(genes(i).strand),cds_exons(:,1),cds_exons(:,2),genome_info,1);
    f = find_open_frames(cds,1) ;
    assert(mod(length(cds),3)==0 && f(1)==1 && any(cds(1:3)=='atg')&&...
           any(ismember({'taa', 'tag', 'tga'}, cds(end-2:end))))
    write_fasta(fd1, genes(i).name, cds) ;
    if exist('fd2')
      aa = translate(cds);
      write_fasta(fd2, genes(i).name, aa) ;
    end
  end
end
fclose(fd1);
fclose(fd2);