function [valid_genes,genes] = check_genes(genes, genome_info, coding, ...
                                           check_spliceconsensus, noGC) ;
%  [valid_genes, all_genes] = check_transcripts(genes, genome_info, coding) ;
  
if nargin<3, coding=0; end ;
if nargin<4, check_spliceconsensus=1 ; end ;
if nargin<5, noGC = 0 ; end ;

invalid_transcripts = {} ;
num_invalid_genes = 0 ;
num_introns_not_ok=0 ;
num_orf_not_ok=0 ;
%acceptor={} ;
%donor={} ;
for i=1:length(genes), 

  if coding
    fprintf('gene=%i  invalid_transcripts=%i  num_introns_not_ok=%i  num_orf_not_ok=%i\r', i, length(invalid_transcripts), num_introns_not_ok,num_orf_not_ok);
  else
    fprintf('gene=%i  invalid_transcripts=%i  num_introns_not_ok=%i  \r', i, length(invalid_transcripts), num_introns_not_ok);
  end ;
  
  is_cDNA = [] ;
  for j=1:length(genes(i).transcripts)
    is_cDNA(j) = equal(genes(i).transcripts{j}(1:3), 'GR-') ;
  end 
  genes(i).is_cDNA = is_cDNA ;
  genes(i).has_cDNA = any(is_cDNA) ;

  starts = [] ;
  ends = [] ; 
  transcript_valid = [] ;
  is_valid = 1 ;
  for j=1:length(genes(i).exons),
    [tmp,idx] = sort(genes(i).exons{j}(:,1)) ;
    %if ~equal(idx',1:length(idx)) && ~equal(idx',length(idx):-1:1), keyboard ; end ;
    
    genes(i).exons{j} = genes(i).exons{j}(idx,:) ;
    genes(i).exons_confirmed{j} = genes(i).exons_confirmed{j}(idx,:) ;
    
    [tmp,idx] = sort(genes(i).exons{j}(:,2)) ;
    if ~equal(idx',1:length(idx)), 
      fprintf('something wrong with exons\n') ;
      is_valid = 0 ; 
    end ;

    exons_not_ok = 0 ;
    %for k=1:size(genes(i).exons{j},1)
    %  if genes(i).exons{j}(k,2)-genes(i).exons{j}(k,1)<5,
    %    fprintf('\nfound exon of length %i\n', genes(i).exons{j}(k, 2)-genes(i).exons{j}(k,1)) ;
    %    exons_not_ok = exons_not_ok + 1 ;
    %  end 
    %end ;
    
    introns_not_ok = 0 ;
    for k=1:size(genes(i).exons{j},1)-1
      if genes(i).exons{j}(k+1,1)-genes(i).exons{j}(k,2)> ...
            genome_info.max_intron_len, 
        introns_not_ok=introns_not_ok+1; 
        %reason=1 ;
        break ; 
      end ; 
      if genes(i).exons{j}(k+1,1)-genes(i).exons{j}(k,2)<10, 
        introns_not_ok=introns_not_ok+1; 
        %reason=2 ;
        break ; 
      end ; 
      if check_spliceconsensus,
        intron = lower(load_mrna(genes(i).chr, char(genes(i).strands(1)), genes(i).exons{j}(k,2)+1, genes(i).exons{j}(k+1,1)-1)) ;
        intron2 = lower(load_mrna(genes(i).chr, char(genes(i).strands(1)), genes(i).exons{j}(k,2)-1, genes(i).exons{j}(k+1,1)+1)) ;
        if ~equal(intron(end-1:end),'ag'), 
          %acceptor{end+1}=intron(end-1:end) ;
          introns_not_ok=introns_not_ok+1 ; 
          %reason=3 ;
          break ;
        end ;
        if ~equal(intron(1:2),'gt') & (~equal(intron(1:2),'gc') | noGC), 
          %donor{end+1}=intron(1:2) ;
          introns_not_ok=introns_not_ok+1 ; 
          %reason=4 ;
          break ;
        end ;
      end
    end ;
    %if introns_not_ok, genes(i).strands, keyboard ; end ;
    if introns_not_ok, num_introns_not_ok=num_introns_not_ok+introns_not_ok; end ;

    orf_not_ok = 0 ;
    if coding,
      transcript='' ;
      for k=1:size(genes(i).exons{j},1)
        exon = lower(load_mrna(genes(i).chr, char(genes(i).strands(1)), genes(i).exons{j}(k,1), genes(i).exons{j}(k,2))) ;
        if char(genes(i).strands(1))=='+',
          transcript=[transcript exon] ;
        else
          transcript=[exon transcript] ;
        end         
      end
      f=find_open_frames(transcript,1) ;
      if mod(length(transcript),3)~=0 || f(1)~=1 || ...
            any(transcript(1:3)~='atg') || ...
            ~any(ismember({'taa', 'tag', 'tga'}, transcript(end-2:end)))
        %transcript
        %mod(length(transcript),3)
        %find_open_frames(transcript,1)
        %keyboard
        num_orf_not_ok=num_orf_not_ok+1 ;
        orf_not_ok = 1 ;
      end ;
    end ;

    starts(end+1)= genes(i).exons{j}(1,1) ;
    ends(end+1)  = genes(i).exons{j}(end,2) ;
    if any(genes(i).exons{j}(2:end,1)-genes(i).exons{j}(1:end-1,2)>genome_info.max_intron_len)
      fprintf('intron len: %i\n', genes(i).exons{j}(2:end,1)-genes(i).exons{j}(1:end-1,2)) ;
      transcript_valid(end+1) = 0 ;
    else
      transcript_valid(end+1) = (introns_not_ok==0) && (orf_not_ok==0) && (exons_not_ok==0);
    end ;
    if ~transcript_valid(end), invalid_transcripts{end+1}=genes(i).transcripts{j} ; end ;
  end ;
  genes(i).transcript_valid = transcript_valid ;

  % only check the region of the valid transcripts
  starts=[] ;
  ends=[] ;
  for j=1:length(genes(i).exons),
    if genes(i).transcript_valid(j),
      starts(end+1)= genes(i).exons{j}(1,1) ;
      ends(end+1)  = genes(i).exons{j}(end,2) ;
    end ;
  end ;
  if max(ends)-min(starts)>genome_info.max_intron_len*10,
    fprintf('gene length: %i\n', max(ends)-min(starts)) ;
    genes(i).is_valid = 0 ;
  else
    genes(i).is_valid = is_valid & ~isempty(genes(i).exons) ;
  end ;
  if ~genes(i).is_valid, num_invalid_genes = num_invalid_genes + 1; end ;
end ;

%keyboard

idx = find([genes(:).is_valid]) ;
valid_genes = genes(idx) ;

has_transcripts = zeros(1,length(valid_genes)) ;
for i=1:length(valid_genes), 
  valid_genes(i).has_cDNA = any(valid_genes(i).is_cDNA & valid_genes(i).transcript_valid) ;
  idx = find(valid_genes(i).transcript_valid) ;
  has_transcripts(i) = ~isempty(idx) ;
  valid_genes(i).transcript_valid = valid_genes(i).transcript_valid(idx) ;
  valid_genes(i).is_cDNA = valid_genes(i).is_cDNA(idx) ;
  valid_genes(i).exons = valid_genes(i).exons(idx) ;
  valid_genes(i).exons_confirmed = valid_genes(i).exons_confirmed(idx) ;
  valid_genes(i).transcripts = valid_genes(i).transcripts(idx) ;
  valid_genes(i).terminal_info = valid_genes(i).terminal_info(:,idx) ;
end ;
idx = find(has_transcripts) ;
valid_genes = valid_genes(idx) ;
length(valid_genes)

