#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>

#include "common.h"
#include "genome.h"


int copyfastaline(FILE *contig_file, FILE *out_file)
{
    char buf[100] ;
    int cnt = fread(buf, sizeof(char), 50, contig_file) ;
    buf[cnt]=0 ;
    {
	int q ;
	for (q=0; q<cnt; q++)
	    buf[q] = toupper(buf[q]) ;
    }
    fprintf(out_file, "%s\n", buf) ;
    return 0 ;
}

/* Read all the fasta files, and concatenate them into one file,
   with each fasta entry about 10Mb long, with 10kb overlap */
int main(int argc, char**argv)
{
	FILE *out_file=stdout ;
	char * genome_config_fname=NULL ;
	char * organism ; 
	int i ;

	const int CHUNKSIZE = 10000000 ;
	const int OVERLAP = 10000 ;
	long filesize ;
	long startidx, endidx ;
	long curpos ;

	if (argc<2)
    {
		fprintf(stderr, "usage: %s organism  [<genome_config_fname]\n", argv[0]) ;
		exit(-1) ;
    } ;
	organism = argv[1] ;
	fprintf(stderr, "%s v0.1 (%s)\n", argv[0], organism) ;

	// genome config
	if (argc<3)
		genome_config_fname = "genome.config" ;
	else
		genome_config_fname = argv[2] ;
	if (init_genome(genome_config_fname)!=0)
	{
		fprintf(stderr, "genome initialization failed (%s)\n", genome_config_fname) ;
		exit(-1) ;
	}

	for (i=0; i<num_contigs; i++)
	    {
		contig_files[i]=fopen(contig_fnames[i],"r") ;
		fseek(contig_files[i], 0, SEEK_END) ;
		filesize = ftell(contig_files[i]) ;
		//fprintf(stderr, "File size %li bytes\n", filesize);
		
		char * cn = strdup(contig_names[i]) ;
		int j ;
		for (j=0; j<strlen(cn); j++)
		    if (cn[j]==':')
			cn[j]='-' ;
		
		if (filesize < 2*CHUNKSIZE)
		    {
			fprintf(out_file, ">%s:%s:%i:+:%li\n", organism, cn, 1, filesize) ;
			fseek(contig_files[i], 0, SEEK_SET) ;
			
			while (!feof(contig_files[i]))
			    copyfastaline(contig_files[i], out_file);
		    }
		else
		    {
			for (startidx=1; startidx<filesize; startidx+=CHUNKSIZE)
			    {
				endidx = startidx+CHUNKSIZE+OVERLAP ;
				if (endidx > filesize)
				    endidx = filesize ;
				
				//fprintf(stderr, "start:%li\t end:%li\t filesize:%li\n",startidx,endidx,filesize);
				//fprintf(stderr, ">%s:%s:%li:+:%li\n", organism, cn, startidx, endidx) ;

				fprintf(out_file, ">%s:%s:%li:+:%li\n", organism, cn, startidx, filesize) ;
				fseek(contig_files[i], startidx-1, SEEK_SET) ;
				curpos = ftell(contig_files[i]) ;
				
				while (!feof(contig_files[i]) && curpos < endidx)
				    {
					copyfastaline(contig_files[i], out_file);
					curpos = ftell(contig_files[i]) ;
				    }
			    }
		    }
		fclose(contig_files[i]) ;
		contig_files[i] = NULL ;
		free(cn) ;
	    }

	cleanup_genome() ;

	return 0;
} 

