function genes = parse_gff3(gff_fname, save_fname,source,contig,TYPE)
% genes = parse_gff3(gff_fname, save_fname,source)
% can also be used for gff2 format:
% TYPE.short_names = {'CDS','exon','coding_exon'};
% TYPE.IDs = {'1' '2' '3'};
% TYPE.hierachies = [2 3 3];
  
if nargin<3
  source = [];
end
  
  
if nargin<4
  contig = [];
end

if nargin<5|isempty(TYPE)
  %%% SOFA mapping
  SOFA_short_names = {'gene','mRNA','exon','CDS','intron','polyA_sequence','polyA_site','five_prime_UTR','three_prime_UTR'};
  SOFA_IDs = {'SO:0000704','SO:0000234','SO:0000147','SO:0000316','SO:0000188','SO:0000610','SO:0000553','SO:0000204','SO:0000205'};
  hierachies = [1 2 3 3 3 3 3 3 3];
else 
  SOFA_short_names = TYPE.short_names;
  SOFA_IDs = TYPE.IDs;
  hierachies = TYPE.hierachies;
end

status={'Predicted','Partially_confirmed','Confirmed'};

%%% GFF file summary
% keyboard
tmp_fname = gff3_statistics(gff_fname,source,contig);

[tmp,num_genes] = unix(sprintf('cut -f 3 %s | grep gene |wc -l',tmp_fname));
num_genes = str2num(num_genes);

if fexist(sprintf('%s.mat',save_fname))
  fprintf('file %s already exists\n',save_fname)
  fprintf('\nwill be overridden %s\n', save_fname);
  %load(save_fname,'genes');
  %assert(length(genes)==num_genes) 
end

% first parse: genes only are extracted 

types = SOFA_short_names(hierachies==1);
types_IDs = SOFA_IDs(hierachies==1);
fprintf('\n------starting first parse: only look for types\n');
for i=1:length(types)
  [tmp,num] = unix(sprintf('cut -f 3 %s | grep %s |wc -l',tmp_fname,types{i}));
  num_genes(i) = str2num(num);
  fprintf('%s\t%i\n',types{i},num_genes(i));
end
num_genes = sum(num_genes);
if num_genes ==0
  warning('no genes found; looking for hierachy 2 (mRNAs instead)\n')
  types = SOFA_short_names(hierachies==2);
  types_IDs = SOFA_IDs(hierachies==2);
  fprintf('\n------starting first parse: only look for types\n');
  for i=1:length(types)
    [tmp,num] = unix(sprintf('cut -f 3 %s | grep %s |wc -l',tmp_fname,types{i}));
    num_genes(i) = str2num(num);
    fprintf('%s\t%i\n',types{i},num_genes(i));
  end
  num_genes = sum(num_genes);
end
if num_genes ==0
   warning('no genes found')
  return
end
genes = init_genes(num_genes);
[fd msg] = fopen(tmp_fname, 'r');
genes_cnt = 0;
while ~feof(fd),
  parse_starter
  identifier = '';    alias = '';
  for i=1:length(attributes),
    id=strfind(attributes{i},'ID=');
    if ~isempty(id),
      identifier = attributes{i}(id+3:end);
      break;
    elseif length(attributes)==1
      warning('no ID found 1')
      identifier = attributes{1};
    else
      warning('no ID found')
      identifier = attributes{1};
      % keyboard
    end
  end
  for i=1:length(attributes) 
    id=strfind(attributes{i},'Alias=');
    if ~isempty(id),
      alias = attributes{i}(id+3:end);
      break;
    end 
  end
  assert(~isequal(identifier(1:3),'ID=')) ;
  assert(~isempty(identifier));

  other_idx=strmatch(identifier,{genes(1:genes_cnt).name},'exact');
  if ~isempty(other_idx)
    warning('gene appears more than once\n')
    assert(isequal(genes(other_idx).alias,alias));
    assert(isequal(genes(other_idx).chr,elems{1}));
    assert(isequal(genes(other_idx).strand,elems{7})) ;
    if ~isequal(genes(other_idx).start,str2num(elems{4}))
      genes(other_idx).start = min(genes(other_idx).start,str2num(elems{4}));
    end
    if ~isequal(genes(other_idx).stop,str2num(elems{5}))
      genes(other_idx).stop = max(genes(other_idx).stop,str2num(elems{5}));
    end 
    % assert(isequal(genes(other_idx).start,str2num(elems{4})));
    % assert(isequal(genes(other_idx).stop,str2num(elems{5})));
    continue
  end
  genes_cnt =  genes_cnt +1;
  genes(genes_cnt).name = identifier;
  genes(genes_cnt).alias = alias;
  genes(genes_cnt).chr = elems{1};
  genes(genes_cnt).strand = elems{7} ;
  genes(genes_cnt).start(end+1) = str2num(elems{4});
  genes(genes_cnt).stop(end+1) = str2num(elems{5});
  fprintf('  parsed %i genes\r', genes_cnt);  
end % end first parse
% assert(genes_cnt==num_genes)
genes=genes(1:genes_cnt);
fclose(fd);
fprintf('\n\n');
  
  
% second parse: mRNAs only are extracted 

types = SOFA_short_names(hierachies==2);
types_IDs = SOFA_IDs(hierachies==2);
fprintf('\n------starting second parse: only look for types\n');
for i=1:length(types)
  [tmp,num] = unix(sprintf('cut -f 3 %s | grep %s |wc -l',tmp_fname,types{i}));
  num_transcripts(i) = str2num(num);
  fprintf('%s\t%i\n',types{i},num_transcripts(i));
end

num_transcripts = sum(num_transcripts);
transcript_gene_map = zeros(1,num_transcripts);
transcript_names = cell(1,num_transcripts);

gene_names = {genes.name} ;

[fd msg] = fopen(tmp_fname, 'r');
transcript_cnt = 0;
while ~feof(fd),
  parse_starter
  identifier = ''; parent = ''; pred_status = '';
  for i=1:length(attributes),
    id=strfind(attributes{i},'ID=');
    if ~isempty(id),
      identifier = attributes{i}(id+3:end);
    end 
    id=strfind(attributes{i},'Parent=');
    if ~isempty(id),
      parent = attributes{i}(id+7:end);
    end
    id=strfind(attributes{i},'prediction_status=');
    if ~isempty(id),
      pred_status = attributes{i}(id+18:end);
      pred_status = strmatch(pred_status,status)-2;
    end
  end
  if isempty(identifier)&isempty(parent)% &length(attributes)==1
    identifier = attributes{1};
    parent = attributes{1};
  end    
  parent_idx=strmatch(parent,gene_names,'exact');
  if isempty(parent_idx)
    parent_idx=strmatch(identifier,gene_names,'exact');
  end
  if isempty(parent_idx),
    if isempty(parent),
      warning(sprintf('Empty parent: using transcript name as parent (%s)', identifier)) ;
      parent = identifier ;
    end 
    keyboard
    warning(sprintf('Could not find parent gene ("%s"): Created a new gene descriptor', parent)) ;
    genes(end+1).name = parent ;
    genes(end).alias = parent ;
    genes(end).chr = elems{1};
    genes(end).strand = elems{7} ;
    genes(end).start(end+1) = 0 ;
    genes(end).stop(end+1) = inf ;
    gene_names = {genes.name} ;
    parent_idx=length(genes) ;
  end ;
  assert(isequal(elems{1}, genes(parent_idx).chr)) ;
  if~(isequal(elems{7}, genes(parent_idx).strand)) ;
    warning('strand of parent gene different \n')
    elems{7} = genes(parent_idx).strand ;
  end
  assert(~isempty(parent_idx))
  if ~isempty(genes(parent_idx).transcripts)
    other_idx = strmatch(identifier,genes(parent_idx).transcripts,'exact');
    if ~isempty(other_idx);
      warning('transcript appears more than once! \n')
      if ~isequal(genes(parent_idx).start(other_idx),str2num(elems{4})) 
        genes(parent_idx).start = min(genes(parent_idx).start(other_idx),str2num(elems{4}));             
      end                                                                                                                               
      if ~isequal(genes(parent_idx).stop(other_idx),str2num(elems{5}))            
        genes(parent_idx).stop(other_idx) = max(genes(parent_idx).stop(other_idx),str2num(elems{5}));             
      end
      continue
    end
  end
  genes(parent_idx).transcripts{end+1} = identifier;
  if ~isempty(pred_status)
    genes(parent_idx).transcript_status(end+1) = pred_status;
  end
  if isempty(genes(parent_idx).exons) 
    genes(parent_idx).exons = cell(1,1) ;
  else
    genes(parent_idx).exons{end+1} = [] ;
  end
  %transcript must lie within gene boundaries
  if ~(str2num(elems{4})>=genes(parent_idx).start(1)&str2num(elems{4})<=genes(parent_idx).stop(1))
    warning(sprintf('transcript outside gene boundaries (%s; %s)', identifier, parent)) ;
  end
  if ~(str2num(elems{5})>=genes(parent_idx).start(1)&str2num(elems{5})<=genes(parent_idx).stop(1))
    warning(sprintf('transcript outside gene boundaries (%s; %s)', identifier, parent)) ;
  end
  genes(parent_idx).start(end+1) = str2num(elems{4});
  genes(parent_idx).stop(end+1) = str2num(elems{5}); 
  transcript_cnt = transcript_cnt +1;
  transcript_gene_map(transcript_cnt) = parent_idx;
  transcript_names{transcript_cnt} = identifier;
  fprintf('  parsed %i transcripts\r', transcript_cnt);  
end %end second parse
transcript_names = transcript_names(1:transcript_cnt); 
% assert(transcript_cnt==num_transcripts)
fclose(fd);
fprintf('\n\n');

% correcting gene boundaries for orphan transcripts
for i=genes_cnt+1:length(genes),
  genes(i).start(1) = min(genes(i).start(2:end)) ;
  genes(i).stop(1) = max(genes(i).stop(2:end)) ;
end ;

% third parse: genes are filled with exons

types = SOFA_short_names(hierachies==3);
types_IDs = SOFA_IDs(hierachies==3);
fprintf('\n------starting third parse: only look for type\n');
for i=1:length(types)
  [tmp,num] = unix(sprintf('cut -f 3 %s | grep %s |wc -l',tmp_fname,types{i}));
  num_segments(i) = str2num(num);
  fprintf('%s\t%i\n',types{i},num_segments(i));
end
num_segments = sum(num_segments);

fprintf('skipping introns (redundant with exons)\n');

[fd msg] = fopen(tmp_fname, 'r');
disp(msg);
segment_cnt = 0;segment_skipped=0;

while ~feof(fd),

  parse_starter
  switch type_name ,
   case {'CDS','coding_exon'}
    type = 3;
   case 'three_prime_UTR'
    type = 4;
   case 'five_prime_UTR'
    type = 2; 
   case 'exon'
    type = 0;
   otherwise
    segment_skipped = segment_skipped+1;
    % disp(Line);
    continue
  end
  
  parent = '';
  for i=1:length(attributes), 
    id=strfind(attributes{i},'Parent=');
    if ~isempty(id),
      parent = attributes{i}(id+7:end);
    end
  end
  if isempty(parent) %&  length(attributes)==1
    parent = attributes{1};
  end
  assert(~isempty(parent));
  parents = separate(parent, ',') ;
  for parent=parents,
    parent_idx=strmatch(parent,transcript_names,'exact');
    assert(~isempty(parent_idx) & length(parent_idx)==1);
    gene_idx = transcript_gene_map(parent_idx);

    transcript_idx = strmatch(parent,genes(gene_idx).transcripts,'exact');
    assert(~isempty(transcript_idx) & length(transcript_idx)==1);

    assert(isequal(elems{1}, genes(gene_idx).chr)) ;
    assert(isequal(elems{7}, genes(gene_idx).strand)) ;

    if elems{7}=='+'
      genes(gene_idx).exons{transcript_idx}(end+1,:) = [str2num(elems{4}), str2num(elems{5})+1, type];
    else
      genes(gene_idx).exons{transcript_idx}(end+1,:) = [str2num(elems{4})-1, str2num(elems{5}), type];
    end
%    assert(str2num(elems{4})>=genes(gene_idx).start(1+transcript_idx)&str2num(elems{4})<=genes(gene_idx).stop(1+transcript_idx))
%    assert(str2num(elems{5})>=genes(gene_idx).start(1+transcript_idx)&str2num(elems{5})<=genes(gene_idx).stop(1+transcript_idx))
  end ;
  segment_cnt = segment_cnt+1;
  if mod(segment_cnt,100)==0
    fprintf('  parsed %i gene segments\r', segment_cnt);
  end
end % end third parse
fclose(fd);


%assert(segment_cnt+segment_skipped==num_segments)
fprintf('\n\n');
for i=1:length(genes)
  genes(i).start = min(genes(i).start);
  genes(i).stop = max(genes(i).stop);
end
unix(['rm ' tmp_fname]);
save(save_fname, 'genes');


